@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,9 +19,17 @@
 
 #include "stdafx.h"
 
+#include "base/file_utilities.h"
+#include "base/file_functions.h"
+#include "base/string_utilities.h"
+#include "base/log.h"
+
 #include "wb_context_model.h"
+
+#include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "workbench/wb_command_ui.h"
+
 #include "wb_component.h"
 #include "wb_component_basic.h"
 #include "wb_component_physical.h"
@@ -29,37 +37,45 @@
 #include "model/wb_model_diagram_form.h"
 #include "wb_overview_physical.h"
 #include "wb_overview_physical_schema.h"
+#include "objimpl/ui/mforms_ObjectReference_impl.h"
 #include "workbench/wb_model_file.h"
 #include "model/wb_overview_physical.h"
+#include "model/wb_user_datatypes.h"
+#include "model/wb_history_tree.h"
 #include "wbcanvas/model_diagram_impl.h"
 #include "wbcanvas/workbench_physical_model_impl.h"
 #include "workbench/wb_model_file.h"
-
 #include "grtdb/db_object_helpers.h"
 #include "grt/clipboard.h"
 #include "grt/common.h"
-#include "base/file_utilities.h"
-#include "base/file_functions.h"
-#include "base/string_utilities.h"
 
+#include "grtpp_notifications.h"
+
+#include "user_defined_type_editor.h"
+#include "wb_template_list.h"
+
+#include "mforms/tabview.h"
+#include "mforms/tabview_dock.h"
 #include "mforms/utilities.h"
 #include "mforms/menubar.h"
-
+#include "mforms/dockingpoint.h"
 
 using namespace base;
 using namespace bec;
 using namespace wb;
 using namespace base;
 
+DEFAULT_LOG_DOMAIN("ModelContext");
+
 static std::map<std::string, std::string> auto_save_files;
 
 
 WBContextModel::WBContextModel(WBContextUI *wbui)
-: _wbui(wbui), _file(0), _locked_view_for_plugin_exec(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
+: _wbui(wbui), _file(0), _current_user_type_editor(0), _locked_view_for_plugin_exec(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
 
 {  
   _overview= new PhysicalOverviewBE(_wbui->get_wb());
-  
+
   scoped_connect(_wbui->get_wb()->get_grt_manager()->get_clipboard()->signal_changed(),boost::bind(&WBContextModel::selection_changed, this));
   scoped_connect(_overview->signal_selection_changed(),boost::bind(&WBContextModel::selection_changed, this)); // make edit menu captions to update
 
@@ -86,19 +102,43 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
                              boost::bind(&WBContextModel::has_selected_figures, this));
   
   base::NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
-  
+
   // setup auto-save for model
   int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
   if (interval > 0)
     _auto_save_timer = _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), interval);
   _auto_save_interval = interval;
-  
+
+  // DON'T set up any UI here. This is running on a background thread!
+  _secondary_sidebar = NULL;
+  _sidebar_dockpoint = NULL;
+  _template_panel = NULL;
+
+  // Default table templates list
+  grt::DictRef options(_wbui->get_wb()->get_root()->options()->options());
+  if (!options.has_key("TableTemplates"))
+  {
+    grt::ListRef<db_Table> templates = grt::ListRef<db_Table>::cast_from(_wbui->get_wb()->get_grt_manager()->get_grt()->unserialize(make_path(_wbui->get_wb()->get_datadir(), "data/table_templates.xml")));
+    options.set("TableTemplates", templates);
+  }
+
   scoped_connect(_wbui->get_wb()->get_root()->options()->signal_dict_changed(),boost::bind(&WBContextModel::option_changed, this, _1, _2, _3));
+
+  setup_secondary_sidebar();
 }
 
 
 WBContextModel::~WBContextModel()
 {
+  _grtmodel_panel.clear();
+
+  if (_secondary_sidebar != NULL)
+    _secondary_sidebar->release();
+  if (_sidebar_dockpoint != NULL)
+    _sidebar_dockpoint->release();
+
+  delete _template_panel;
+
   base::NotificationCenter::get()->remove_observer(this);
 
   if (_doc.is_valid() && _doc->physicalModels().is_valid() && _doc->physicalModels().count() > 0)
@@ -118,6 +158,37 @@ WBContextModel::~WBContextModel()
 }
 
 
+void WBContextModel::setup_secondary_sidebar()
+{
+  // Setup the secondary model sidebar, which should be shared between all model tabs
+  _secondary_sidebar = mforms::manage(new mforms::TabView(mforms::TabViewSelectorSecondary));
+  _template_panel = new TableTemplatePanel(_wbui->get_wb()->get_grt(), this);
+  _secondary_sidebar->add_page(_template_panel, _("Templates"));
+}
+
+
+mforms::TreeNodeView *WBContextModel::create_user_type_list()
+{
+  UserDatatypeList *type_list;
+
+  type_list = new UserDatatypeList(_wbui->get_wb());
+  type_list->set_catalog(_wbui->get_wb()->get_document()->physicalModels()[0]->catalog());
+  type_list->refresh();
+
+  type_list->scoped_connect(&_udt_list_changed, boost::bind(&UserDatatypeList::refresh, type_list));
+
+  return type_list;
+}
+
+
+mforms::TreeNodeView *WBContextModel::create_history_tree()
+{
+  HistoryTree *history_tree= new HistoryTree(_wbui->get_wb()->get_grt_manager(), _wbui->get_wb()->get_grt()->get_undo_manager());
+  history_tree->refresh();
+  return history_tree;
+}
+
+
 grt::GRT *WBContextModel::get_grt()
 {
   return _wbui->get_wb()->get_grt();
@@ -212,7 +283,7 @@ void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
       }
       else
       {
-        std::string fname = g_basename(d->c_str());
+        std::string fname = base::basename(*d);
         fname = fname.substr(0, fname.rfind('.')).append(".mwb");
         // if no real_path file in the autosave, this could be an autosave from an older version
         files[fname] = *d;
@@ -220,7 +291,7 @@ void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
     }
     else
     {
-      g_message("Found model auto-save %s, but it is empty. Deleting it...", d->c_str());
+      log_info("Found model auto-save %s, but it is empty. Deleting it...\n", d->c_str());
       base_rmdir_recursively(d->c_str());
     }
   }
@@ -289,8 +360,19 @@ void WBContextModel::model_created(ModelFile *file, workbench_DocumentRef doc)
   _doc->physicalModels().get(0)->get_data()->set_delegate(this);
   
   _doc->physicalModels()[0]->get_data()->realize();
-  
+
   _wbui->get_wb()->request_refresh(RefreshNewModel, "", 0);
+
+  // setup GRT proxy object
+  _grtmodel_panel = ui_ModelPanelRef(_doc.get_grt());
+  _grtmodel_panel->model(_doc->physicalModels()[0]);
+
+  if (_sidebar_dockpoint == NULL)
+    _sidebar_dockpoint = mforms::manage(new mforms::DockingPoint(new mforms::TabViewDockingPoint(_secondary_sidebar, MODEL_DOCKING_POINT), true));
+  _grtmodel_panel->commonSidebar(mforms_to_grt(_doc.get_grt(), _sidebar_dockpoint));
+
+  grt::DictRef info(_doc.get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelCreated", _grtmodel_panel, info);
 }
 
 
@@ -308,16 +390,34 @@ void WBContextModel::model_loaded(ModelFile *file, workbench_DocumentRef doc)
   _wbui->get_wb()->request_refresh(RefreshNewModel, "", 0);
   
   std::string temp_dir = _file->get_tempdir_path();
-  for (std::map<std::string, std::string>::iterator iter= ::auto_save_files.begin();
+  for (std::map<std::string, std::string>::iterator iter = ::auto_save_files.begin();
        iter != ::auto_save_files.end(); ++iter)
   {
     if (iter->second == temp_dir)
     {
       ::auto_save_files.erase(iter);
-      _wbui->refresh_home_models();
+      _wbui->refresh_home_documents();
       break;
     }
   }
+
+  // setup GRT proxy object
+  _grtmodel_panel = ui_ModelPanelRef(get_grt());
+  _grtmodel_panel->model(_doc->physicalModels()[0]);
+
+  if (_sidebar_dockpoint == NULL)
+    _sidebar_dockpoint = mforms::manage(new mforms::DockingPoint(new mforms::TabViewDockingPoint(_secondary_sidebar, MODEL_DOCKING_POINT), true));
+  _grtmodel_panel->commonSidebar(mforms_to_grt(_doc.get_grt(), _sidebar_dockpoint));
+
+  grt::DictRef info(get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelOpened", _grtmodel_panel, info);
+}
+
+
+void WBContextModel::model_closed()
+{
+  grt::DictRef info(get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelClosed", _grtmodel_panel, info);
 }
 
 
@@ -851,36 +951,40 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
       if (can_copy && !model->is_copyable(*iterator))
         can_copy= false;
     }
-  
-  item.checked= false;
-  item.enabled= can_copy && can_delete;
-  item.caption= label.empty() ? _("Cut") : strfmt(_("Cut %s"), label.c_str());
-  item.name= "builtin:cut";
-  item.type= MenuAction;
-  items.push_back(item);
-  
-  item.enabled= can_copy;
-  item.caption= label.empty() ? _("Copy") : strfmt(_("Copy %s"), label.c_str());
-  item.name= "builtin:copy";
-  item.type= MenuAction;
-  items.push_back(item);
-  
-  item.enabled= form->can_paste();
-  item.caption= strfmt(_("Paste %s"), wb->get_clipboard()->get_content_description().c_str());
-  item.name= "builtin:paste";
-  item.type= MenuAction;
-  items.push_back(item);
 
-// this is already implemented, but not yet exposed.. uncomment once the whole thing is working
-//  item.enabled= form->can_paste();
-//  item.caption= strfmt(_("Duplicate %s"), wb->get_clipboard()->get_content_description().c_str());
-//  item.name= "builtin:duplicate";
-//  item.type= MenuAction;
-//  items.push_back(item);
+  // can't copy/paste diagrams
+  if (nodes.empty() || nodes[0][0] != 0)
+  {
+    item.checked= false;
+    item.enabled= can_copy && can_delete;
+    item.caption= label.empty() ? _("Cut") : strfmt(_("Cut %s"), label.c_str());
+    item.name= "builtin:cut";
+    item.type= MenuAction;
+    items.push_back(item);
+    
+    item.enabled= can_copy;
+    item.caption= label.empty() ? _("Copy") : strfmt(_("Copy %s"), label.c_str());
+    item.name= "builtin:copy";
+    item.type= MenuAction;
+    items.push_back(item);
+    
+    item.enabled= form->can_paste();
+    item.caption= strfmt(_("Paste %s"), wb->get_clipboard()->get_content_description().c_str());
+    item.name= "builtin:paste";
+    item.type= MenuAction;
+    items.push_back(item);
+
+  // this is already implemented, but not yet exposed.. uncomment once the whole thing is working
+  //  item.enabled= form->can_paste();
+  //  item.caption= strfmt(_("Duplicate %s"), wb->get_clipboard()->get_content_description().c_str());
+  //  item.name= "builtin:duplicate";
+  //  item.type= MenuAction;
+  //  items.push_back(item);
+
+    item.type= MenuSeparator;
+    items.push_back(item);
+  }
 
-  item.type= MenuSeparator;
-  items.push_back(item);
-  
   // Plugin dependent menu entries.
   size_t old_count= items.size();
   
@@ -933,12 +1037,15 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   item.name= "builtin:delete";
   item.type= MenuAction;
   items.push_back(item);
-  
-  item.caption= label.empty() ? _("Remove Figure") : strfmt(_("Remove Figure %s"), label.c_str());
-  item.name= "builtin:removeFigure";
-  item.type= MenuAction;
-  item.enabled = can_remove;
-  items.push_back(item);
+
+  if (nodes.empty() || nodes[0][0] != 0)
+  {
+    item.caption= label.empty() ? _("Remove Figure") : strfmt(_("Remove Figure %s"), label.c_str());
+    item.name= "builtin:removeFigure";
+    item.type= MenuAction;
+    item.enabled = can_remove;
+    items.push_back(item);
+  }
   return int(items.size() - initial_count);
 }
 
@@ -979,12 +1086,12 @@ int WBContextModel::add_object_plugins_to_popup_menu(const grt::ListRef<GrtObjec
       for (std::list<std::string>::const_iterator ctx= groups.begin();
            ctx != groups.end(); ++ctx)
       {
-        if ((*ctx)[ctx->size()-1] == '*' && g_strncasecmp(ctx->c_str(), str.c_str(), (guint) ctx->size() - 1) == 0)
+        if ((*ctx)[ctx->size()-1] == '*' && g_ascii_strncasecmp(ctx->c_str(), str.c_str(), (guint) ctx->size() - 1) == 0)
         {
           match= true;
           break;
         }
-        else if ((*ctx)[ctx->size() - 1] != '*' && g_strcasecmp(ctx->c_str(), str.c_str()) == 0)
+        else if ((*ctx)[ctx->size() - 1] != '*' && g_ascii_strcasecmp(ctx->c_str(), str.c_str()) == 0)
         {
           match= true;
           break;
@@ -1363,3 +1470,54 @@ void WBContextModel::end_plugin_exec()
     _locked_view_for_plugin_exec->unlock_redraw();
   _locked_view_for_plugin_exec = 0;
 }
+
+
+/**
+ * Called by the front end when the user data type editor has been closed. We can then remove our reference to it.
+ */
+static void userTypeEditorClosed(UserDefinedTypeEditor **editor_ptr)
+{
+  *editor_ptr = NULL;
+}
+
+void WBContextModel::show_user_type_editor(workbench_physical_ModelRef model)
+{
+  if (_current_user_type_editor == NULL)
+  {
+    _current_user_type_editor = new UserDefinedTypeEditor(get_wbui(), model);
+    scoped_connect(_current_user_type_editor->signal_closed(), boost::bind(userTypeEditorClosed, &_current_user_type_editor));
+  }
+  _current_user_type_editor->show_modal(NULL, NULL);
+}
+
+mforms::View *WBContextModel::shared_secondary_sidebar()
+{
+  return _secondary_sidebar;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterNotifDocs
+{
+  RegisterNotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GRNModelOpened",
+                                                           "modeling",
+                                                           "Sent when a model document finishes loading.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNModelCreated",
+                                                           "modeling",
+                                                           "Sent when a new model document is created.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNModelClosed",
+                                                           "modeling",
+                                                           "Sent when a model document is closed.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+  }
+} initdocs;
+
