@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,7 +31,6 @@
 #include "wb_helpers.h"
 #include "cppconn/driver.h"
 #include "cppconn/sqlstring.h"
-#include "driver/mysql_driver.h"
 
 #include "sqlide/wb_sql_editor_form.h"
 
@@ -39,17 +38,16 @@
 using namespace grt;
 using namespace wb;
 using namespace sql;
-using namespace sql::mysql;
 
-/* This class is a friend of the Db_sql_editor on DEBUG mode, 
+/* This class is a friend of the SqlEditorForm on DEBUG mode, 
    this way everything is available for testing */
-class Db_sql_editor_tester
+class EditorFormTester
 {
 private:
   wb::LiveSchemaTree::NodeChildrenUpdaterSlot updater_slot;
   wb::LiveSchemaTree::NewSchemaContentArrivedSlot schema_content_arrived_slot;
 public:
-  Db_sql_editor_tester():
+  EditorFormTester():
       _expect_update_node_children(false),
       _expect_schema_content_arrived(false),
       _mock_validate_schema_content(false),
@@ -57,97 +55,97 @@ public:
       _mock_propagate_update_node_children(false)
 
   {
-    updater_slot = boost::bind(&Db_sql_editor_tester::mock_update_node_children, this, _1, _2, _3, _4);
-    schema_content_arrived_slot = boost::bind(&Db_sql_editor_tester::mock_schema_content_arrived, this, _1, _2, _3, _4, _5, _6);
+    updater_slot = boost::bind(&EditorFormTester::mock_update_node_children, this, _1, _2, _3, _4, _5);
+    schema_content_arrived_slot = boost::bind(&EditorFormTester::mock_schema_content_arrived, this, _1, _2, _3, _4, _5, _6);
   }
-  void set_target(const Db_sql_editor::Ref& sql_editor)
+  void set_target(const SqlEditorForm::Ref& sql_editor)
   {
-    db_sql_editor = sql_editor;
+    _form = sql_editor;
   }
 
   void fetch_column_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::ObjectType object_type)
   {
-    db_sql_editor->get_live_tree()->fetch_column_data(schema_name, obj_name, object_type, updater_slot);
+    _form->get_live_tree()->fetch_column_data(schema_name, obj_name, object_type, updater_slot);
   }
 
   void fetch_index_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::ObjectType object_type)
   {
-    db_sql_editor->get_live_tree()->fetch_index_data(schema_name, obj_name, object_type, updater_slot);
+    _form->get_live_tree()->fetch_index_data(schema_name, obj_name, object_type, updater_slot);
   }
 
   void fetch_trigger_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::ObjectType object_type)
   {
-    db_sql_editor->get_live_tree()->fetch_trigger_data(schema_name, obj_name, object_type, updater_slot);
+    _form->get_live_tree()->fetch_trigger_data(schema_name, obj_name, object_type, updater_slot);
   }
 
   void fetch_foreign_key_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::ObjectType object_type)
   {
-    db_sql_editor->get_live_tree()->fetch_foreign_key_data(schema_name, obj_name, object_type, updater_slot);
+    _form->get_live_tree()->fetch_foreign_key_data(schema_name, obj_name, object_type, updater_slot);
   }
 
   void fetch_schema_contents(const std::string &schema_name)
   {
-    db_sql_editor->get_live_tree()->fetch_schema_contents(schema_name, schema_content_arrived_slot);
+    _form->get_live_tree()->fetch_schema_contents(schema_name, schema_content_arrived_slot);
 
     sleep(1);
 
-    db_sql_editor->grt_manager()->perform_idle_tasks();
+    _form->grt_manager()->perform_idle_tasks();
   }
 
   std::list<std::string> fetch_schema_list()
   {
-    return db_sql_editor->get_live_tree()->fetch_schema_list();
+    return _form->get_live_tree()->fetch_schema_list();
   }
 
   void load_schema_list()
   {
-    return db_sql_editor->get_live_tree()->tree_refresh();
+    return _form->get_live_tree()->tree_refresh();
   }
 
   void load_schema_data(const std::string& schema )
   {
-    mforms::TreeNodeRef schema_node = db_sql_editor->get_live_tree()->_schema_tree->get_node_for_object(schema, wb::LiveSchemaTree::Schema, "");
-    db_sql_editor->get_live_tree()->_schema_tree->expand_toggled(schema_node, true);
+    mforms::TreeNodeRef schema_node = _form->get_live_tree()->_schema_tree->get_node_for_object(schema, wb::LiveSchemaTree::Schema, "");
+    _form->get_live_tree()->_schema_tree->expand_toggled(schema_node, true);
   }
 
   bool is_editable_select(const std::string &sql, std::string& schema_name, std::string& table_name, std::string& error)
   {
-    return db_sql_editor->get_live_tree()->is_editable_select(sql, schema_name, table_name, error);
+    return _form->get_live_tree()->is_editable_select(sql, schema_name, table_name, error);
   }
 
   void exec_sql(std::string &sql)
   {
-    db_sql_editor->exec_sql_returning_results(sql, false);
+    _form->exec_sql_returning_results(sql, false);
   }
 
   /* mock function that will simulate the schema list loading using this thread */
   void tree_refresh()
   {
-    std::list<std::string> schema_list = db_sql_editor->get_live_tree()->fetch_schema_list();
+    std::list<std::string> schema_list = _form->get_live_tree()->fetch_schema_list();
 
-    db_sql_editor->get_live_tree()->_schema_tree->update_schemata(schema_list);
+    _form->get_live_tree()->_schema_tree->update_schemata(schema_list);
   }
 
   void set_lst_model_view(mforms::TreeNodeView * pmodel_view)
   {
-    db_sql_editor->get_live_tree()->_schema_tree->set_model_view(pmodel_view);
-    db_sql_editor->get_live_tree()->_schema_tree->enable_events(true);
+    _form->get_live_tree()->_schema_tree->set_model_view(pmodel_view);
+    _form->get_live_tree()->_schema_tree->enable_events(true);
   }
 
-  bool mock_update_node_children(mforms::TreeNodeRef& parent, std::list<std::string>& children, wb::LiveSchemaTree::ObjectType type, bool just_append = false)
+  bool mock_update_node_children(mforms::TreeNodeRef& parent, std::list<std::string>& children, wb::LiveSchemaTree::ObjectType type, bool sorted = false, bool just_append = false)
   {
     tut::ensure(_check_id + " : Unexpected call to update_node_children", _expect_update_node_children);
     _expect_update_node_children = false;
 
     if (_mock_propagate_update_node_children)
     {
-      db_sql_editor->get_live_tree()->_schema_tree->update_node_children(parent, children, type, just_append);
+      _form->get_live_tree()->_schema_tree->update_node_children(parent, children, type, sorted, just_append);
     }
 
     return false;
   }
 
-  void mock_schema_content_arrived(const std::string &schema_name, const std::list<std::string>& tables, const std::list<std::string>& views, const std::list<std::string>& procedures, const std::list<std::string>& functions, bool just_append)
+  void mock_schema_content_arrived(const std::string &schema_name, std::list<std::string>* tables, std::list<std::string>* views, std::list<std::string>* procedures, std::list<std::string>* functions, bool just_append)
   {
     tut::ensure(_check_id + " : Unexpected call to schema_content_arrived", _expect_schema_content_arrived);
     _expect_schema_content_arrived = false;
@@ -157,35 +155,35 @@ public:
       _mock_validate_schema_content = false;
 
 
-      tut::ensure_equals(_check_id + " : Unexpected number of tables received", tables.size(), _expected_tables.size());
+      tut::ensure_equals(_check_id + " : Unexpected number of tables received", tables->size(), _expected_tables.size());
 
-      std::list<std::string>::const_iterator index, end = tables.end();
-      for(index = tables.begin(); index != end; index++)
+      std::list<std::string>::const_iterator index, end = tables->end();
+      for(index = tables->begin(); index != end; index++)
         tut::ensure(_check_id + " : Unexpected table retrieved", std::find( _expected_tables.begin(), _expected_tables.end(), *index) != _expected_tables.end());
 
       _expected_tables.clear();
 
 
-      tut::ensure_equals(_check_id + " : Unexpected number of views received", views.size(), _expected_views.size());
+      tut::ensure_equals(_check_id + " : Unexpected number of views received", views->size(), _expected_views.size());
 
-      end = views.end();
-      for(index = views.begin(); index != end; index++)
+      end = views->end();
+      for(index = views->begin(); index != end; index++)
         tut::ensure(_check_id + " : Unexpected view retrieved", std::find( _expected_views.begin(), _expected_views.end(), *index) != _expected_views.end());
 
       _expected_views.clear();
 
-      tut::ensure_equals(_check_id + " : Unexpected number of procedures received", procedures.size(), _expected_procedures.size());
+      tut::ensure_equals(_check_id + " : Unexpected number of procedures received", procedures->size(), _expected_procedures.size());
 
-      end = procedures.end();
-      for(index = procedures.begin(); index != end; index++)
+      end = procedures->end();
+      for(index = procedures->begin(); index != end; index++)
         tut::ensure(_check_id + " : Unexpected procedure retrieved", std::find( _expected_procedures.begin(), _expected_procedures.end(), *index) != _expected_procedures.end());
 
       _expected_procedures.clear();
 
-      tut::ensure_equals(_check_id + " : Unexpected number of functions received", functions.size(), _expected_functions.size());
+      tut::ensure_equals(_check_id + " : Unexpected number of functions received", functions->size(), _expected_functions.size());
 
-      end = functions.end();
-      for(index = functions.begin(); index != end; index++)
+      end = functions->end();
+      for(index = functions->begin(); index != end; index++)
         tut::ensure(_check_id + " : Unexpected function retrieved", std::find( _expected_functions.begin(), _expected_functions.end(), *index) != _expected_functions.end());
 
       _expected_functions.clear();
@@ -193,7 +191,7 @@ public:
 
     if (_mock_propagate_schema_content)
     {
-      //db_sql_editor->get_live_tree()->_schema_tree->schema_contents_arrived(schema_name, tables, views, procedures, functions, just_append);
+      //_form->get_live_tree()->_schema_tree->schema_contents_arrived(schema_name, tables, views, procedures, functions, just_append);
     }
   }
 
@@ -224,7 +222,7 @@ public:
   std::string _check_id;
 
 private:
-  Db_sql_editor::Ref db_sql_editor;
+  SqlEditorForm::Ref _form;
 
 
 };
@@ -234,8 +232,8 @@ public:
   WBTester tester;
   WBContextSQLIDE wb_context_sqlide;
   sql::ConnectionWrapper connection;
-  Db_sql_editor::Ref db_sql_editor;
-  Db_sql_editor_tester db_sql_editor_tester;
+  SqlEditorForm::Ref form;
+  EditorFormTester form_tester;
   mforms::TreeNodeView *pmodel_view;
   GRT grt;
 
@@ -273,14 +271,13 @@ TEST_DATA_CONSTRUCTOR(wb_sql_editor_form_test):wb_context_sqlide(tester.wbui)
 
   db_mgmt_ConnectionRef my_connection(tester.grt);
   set_connection_properties(tester.grt, my_connection);
-  db_sql_editor = Db_sql_editor::create(&wb_context_sqlide, my_connection);
-  db_sql_editor->connect();
+  form = SqlEditorForm::create(&wb_context_sqlide, my_connection);
+  form->connect(boost::shared_ptr<sql::TunnelConnection>());
 
-  mforms::stub::init();
   pmodel_view = new mforms::TreeNodeView(mforms::TreeNoColumns | mforms::TreeNoBorder | mforms::TreeSidebar | mforms::TreeNoHeader);
 
-  db_sql_editor_tester.set_target(db_sql_editor);
-  db_sql_editor_tester.set_lst_model_view(pmodel_view);
+  form_tester.set_target(form);
+  form_tester.set_lst_model_view(pmodel_view);
 
     //"USE `wb_sql_editor_form_test`;"
   static const char *sql1 =//
@@ -413,10 +410,10 @@ TEST_DATA_CONSTRUCTOR(wb_sql_editor_form_test):wb_context_sqlide(tester.wbui)
     " DELIMITER ;\n";
 
   std::string sql(sql1);
-  db_sql_editor_tester.exec_sql(sql);
+  form_tester.exec_sql(sql);
 
-  db_sql_editor_tester.tree_refresh();
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester.tree_refresh();
+  form_tester.load_schema_data("wb_sql_editor_form_test");
  
 }
 
@@ -428,7 +425,7 @@ TEST_MODULE(wb_sql_editor_form_test, "sql editor form test");
 /* Testing fetch_schema_list */
 TEST_FUNCTION(1)
 {
-  std::list<std::string> schema_list = db_sql_editor_tester.fetch_schema_list();
+  std::list<std::string> schema_list = form_tester.fetch_schema_list();
 
   ensure ("TF001CHK001: Unexpected number of schemas retrieved", schema_list.size() > 0);
 
@@ -446,38 +443,38 @@ TEST_FUNCTION(1)
 TEST_FUNCTION(2)
 {
   // Loads the schema list
-  db_sql_editor_tester.tree_refresh();
+  form_tester.tree_refresh();
 
   // Sets the expectations..
-  db_sql_editor_tester._expect_schema_content_arrived = true;
-  db_sql_editor_tester._mock_validate_schema_content = true;
+  form_tester._expect_schema_content_arrived = true;
+  form_tester._mock_validate_schema_content = true;
 
-  db_sql_editor_tester._expected_tables.push_back("language");
-  db_sql_editor_tester._expected_tables.push_back("film");
-  db_sql_editor_tester._expected_tables.push_back("film_text");
-  db_sql_editor_tester._expected_tables.push_back("dummy_table");
-  db_sql_editor_tester._expected_tables.push_back("complex_pk_table");
-  db_sql_editor_tester._expected_tables.push_back("no_pk_table");
-  db_sql_editor_tester._expected_tables.push_back("pk_table_unique_not_null");
-  db_sql_editor_tester._expected_tables.push_back("no_pk_table_unique_not_null");
+  form_tester._expected_tables.push_back("language");
+  form_tester._expected_tables.push_back("film");
+  form_tester._expected_tables.push_back("film_text");
+  form_tester._expected_tables.push_back("dummy_table");
+  form_tester._expected_tables.push_back("complex_pk_table");
+  form_tester._expected_tables.push_back("no_pk_table");
+  form_tester._expected_tables.push_back("pk_table_unique_not_null");
+  form_tester._expected_tables.push_back("no_pk_table_unique_not_null");
 
-  db_sql_editor_tester._expected_views.push_back("dummy_film_view");
+  form_tester._expected_views.push_back("dummy_film_view");
 
-  db_sql_editor_tester._expected_procedures.push_back("get_films");
+  form_tester._expected_procedures.push_back("get_films");
 
-  db_sql_editor_tester._expected_functions.push_back("dummy_function");
-  db_sql_editor_tester._expected_functions.push_back("other_function");
+  form_tester._expected_functions.push_back("dummy_function");
+  form_tester._expected_functions.push_back("other_function");
 
-  db_sql_editor_tester._check_id = "TF002CHK001";
+  form_tester._check_id = "TF002CHK001";
 
   // Loads the specific schema...
-  db_sql_editor_tester.fetch_schema_contents("wb_sql_editor_form_test");
+  form_tester.fetch_schema_contents("wb_sql_editor_form_test");
 
-  db_sql_editor_tester.clean_and_reset();
+  form_tester.clean_and_reset();
 
 }
 
-/* Testing for Db_sql_editor::fetch_column_data */
+/* Testing for SqlEditorForm::fetch_column_data */
 TEST_FUNCTION(3)
 {
   mforms::TreeNodeRef table_node;
@@ -487,18 +484,18 @@ TEST_FUNCTION(3)
   wb::LiveSchemaTree::ColumnData *pchild_data;
 
   // Loads the schema list into the tree...
-  db_sql_editor_tester.tree_refresh();
+  form_tester.tree_refresh();
 
   // Loads a specific schema contents...
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester.load_schema_data("wb_sql_editor_form_test");
 
   /* Loads the column data from the language table */
-  db_sql_editor_tester._expect_update_node_children = true;
-  db_sql_editor_tester._mock_propagate_update_node_children = true;
-  db_sql_editor_tester._check_id = "TF003CHK001";
-  db_sql_editor_tester.fetch_column_data("wb_sql_editor_form_test", "language", wb::LiveSchemaTree::Table);
+  form_tester._expect_update_node_children = true;
+  form_tester._mock_propagate_update_node_children = true;
+  form_tester._check_id = "TF003CHK001";
+  form_tester.fetch_column_data("wb_sql_editor_form_test", "language", wb::LiveSchemaTree::Table);
 
-  table_node = db_sql_editor->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "language");
+  table_node = form->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "language");
   pdata = dynamic_cast<wb::LiveSchemaTree::TableData*>(table_node->get_data());
 
   ensure("TF003CHK002 : Columns were not loaded", pdata->is_data_loaded(wb::LiveSchemaTree::COLUMN_DATA));
@@ -529,7 +526,7 @@ TEST_FUNCTION(3)
   ensure("TF003CHK004 : Unexpected foreign key flag", !pchild_data->is_fk);
 }
 
-/* Testing for Db_sql_editor::fetch_index_data */
+/* Testing for SqlEditorForm::fetch_index_data */
 TEST_FUNCTION(4)
 {
   mforms::TreeNodeRef table_node;
@@ -539,19 +536,19 @@ TEST_FUNCTION(4)
   wb::LiveSchemaTree::IndexData *pchild_data;
 
   // Loads the schema list into the tree...
-  db_sql_editor_tester.tree_refresh();
+  form_tester.tree_refresh();
 
   // Loads a specific schema contents...
-  db_sql_editor_tester._check_id = "TF004CHK001";
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester._check_id = "TF004CHK001";
+  form_tester.load_schema_data("wb_sql_editor_form_test");
 
   /* Loads the index data from the film table */
-  db_sql_editor_tester._expect_update_node_children = true;
-  db_sql_editor_tester._mock_propagate_update_node_children = true;
-  db_sql_editor_tester._check_id = "TF004CHK002";
-  db_sql_editor_tester.fetch_index_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
+  form_tester._expect_update_node_children = true;
+  form_tester._mock_propagate_update_node_children = true;
+  form_tester._check_id = "TF004CHK002";
+  form_tester.fetch_index_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
 
-  table_node = db_sql_editor->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
+  table_node = form->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
   pdata = dynamic_cast<wb::LiveSchemaTree::TableData*>(table_node->get_data());
 
   ensure("TF004CHK003 : Indexes were not loaded", pdata->is_data_loaded(wb::LiveSchemaTree::INDEX_DATA));
@@ -594,7 +591,7 @@ TEST_FUNCTION(4)
 }
 
 
-/* Testing for Db_sql_editor::fetch_trigger_data */
+/* Testing for SqlEditorForm::fetch_trigger_data */
 TEST_FUNCTION(5)
 {
   mforms::TreeNodeRef table_node;
@@ -604,19 +601,19 @@ TEST_FUNCTION(5)
   wb::LiveSchemaTree::TriggerData *pchild_data;
 
   // Loads the schema list into the tree...
-  db_sql_editor_tester.tree_refresh();
+  form_tester.tree_refresh();
 
   // Loads a specific schema contents...
-  db_sql_editor_tester._check_id = "TF005CHK001";
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester._check_id = "TF005CHK001";
+  form_tester.load_schema_data("wb_sql_editor_form_test");
 
   /* Loads the trigger data from the film table */
-  db_sql_editor_tester._expect_update_node_children = true;
-  db_sql_editor_tester._mock_propagate_update_node_children = true;
-  db_sql_editor_tester._check_id = "TF005CHK002";
-  db_sql_editor_tester.fetch_trigger_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
+  form_tester._expect_update_node_children = true;
+  form_tester._mock_propagate_update_node_children = true;
+  form_tester._check_id = "TF005CHK002";
+  form_tester.fetch_trigger_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
 
-  table_node = db_sql_editor->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
+  table_node = form->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
   pdata = dynamic_cast<wb::LiveSchemaTree::TableData*>(table_node->get_data());
 
   ensure("TF005CHK003 : Triggers were not loaded", pdata->is_data_loaded(wb::LiveSchemaTree::TRIGGER_DATA));
@@ -650,7 +647,7 @@ TEST_FUNCTION(5)
   ensure_equals("TF005CHK006 : Unexpected trigger timing", pchild_data->timing, 14); // 14 is AFTER
 }
 
-/* Testing for Db_sql_editor::fetch_foreign_key_data */
+/* Testing for SqlEditorForm::fetch_foreign_key_data */
 TEST_FUNCTION(6)
 {
   mforms::TreeNodeRef table_node;
@@ -660,19 +657,19 @@ TEST_FUNCTION(6)
   wb::LiveSchemaTree::FKData *pchild_data;
 
   // Loads the schema list into the tree...
-  db_sql_editor_tester.tree_refresh();
+  form_tester.tree_refresh();
 
   // Loads a specific schema contents...
-  db_sql_editor_tester._check_id = "TF006CHK001";
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester._check_id = "TF006CHK001";
+  form_tester.load_schema_data("wb_sql_editor_form_test");
 
   /* Loads the foreign key data from the film table */
-  db_sql_editor_tester._expect_update_node_children = true;
-  db_sql_editor_tester._mock_propagate_update_node_children = true;
-  db_sql_editor_tester._check_id = "TF006CHK002";
-  db_sql_editor_tester.fetch_foreign_key_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
+  form_tester._expect_update_node_children = true;
+  form_tester._mock_propagate_update_node_children = true;
+  form_tester._check_id = "TF006CHK002";
+  form_tester.fetch_foreign_key_data("wb_sql_editor_form_test", "film", wb::LiveSchemaTree::Table);
 
-  table_node = db_sql_editor->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
+  table_node = form->get_live_tree()->get_schema_tree()->get_node_for_object("wb_sql_editor_form_test", wb::LiveSchemaTree::Table, "film");
   pdata = dynamic_cast<wb::LiveSchemaTree::TableData*>(table_node->get_data());
 
   ensure("TF006CHK003 : Foreign keys were not loaded", pdata->is_data_loaded(wb::LiveSchemaTree::FK_DATA));
@@ -699,11 +696,11 @@ TEST_FUNCTION(6)
   ensure_equals("TF006CHK005 : Unexpected foreign key delete rule", pchild_data->referenced_table, "language");
 }
 
-// Testing : Db_sql_editor::is_editable_select
+// Testing : SqlEditorForm::is_editable_select
 TEST_FUNCTION(7)
 {
-  db_sql_editor_tester.load_schema_list();
-  db_sql_editor_tester.load_schema_data("wb_sql_editor_form_test");
+  form_tester.load_schema_list();
+  form_tester.load_schema_data("wb_sql_editor_form_test");
 
   std::string error;
   std::string schema_name;
@@ -711,85 +708,85 @@ TEST_FUNCTION(7)
   bool ret;
 
   /* Failure case: using functions as columns */
-  ensure("TF007CHK001 Unexpected editable select success", !db_sql_editor_tester.is_editable_select("select count(*) from wb_sql_editor_form_test.film", schema_name, table_name, error));
+  ensure("TF007CHK001 Unexpected editable select success", !form_tester.is_editable_select("select count(*) from wb_sql_editor_form_test.film", schema_name, table_name, error));
   ensure_equals("TF007CHK002 Unexpected error on editable select", error, "The statement is not editable.");
 
   /* Success case : using a non loaded schema, the required table will be loaded right away */
-  ret = db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.dummy_table", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.dummy_table", schema_name, table_name, error);
   ensure_equals("TF007CHK004 Unexpected error on editable select", error, "");
   ensure("TF007CHK003 Unexpected editable select failure", ret);
 
   /* Success case : loading all columns from a table having primary key */
-  ret = db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
   ensure_equals("TF007CHK006 Unexpected error on editable select", error, "");
   ensure("TF007CHK005 Unexpected editable select failure", ret);
 
   /* Success case : loading columns include primary key */
-  ret = db_sql_editor_tester.is_editable_select("select film_id, title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select film_id, title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
   ensure_equals("TF007CHK008 Unexpected error on editable select", error, "");
   ensure("TF007CHK007 Unexpected editable select failure", ret);
 
   /* Failure case : loading columns not include primary key */
-  ensure("TF007CHK009 Unexpected editable select sucess", !db_sql_editor_tester.is_editable_select("select title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error));
+  ensure("TF007CHK009 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error));
   ensure_equals("TF007CHK010 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table film_text.");
 
 
   /* Success case : loading all columns from a table having complex primary key */
-  ret = db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
   ensure_equals("TF007CHK012 Unexpected error on editable select", error, "");
   ensure("TF007CHK011 Unexpected editable select failure", ret);
 
   /* Success case : loading columns include primary key but not all */
-  ensure("TF007CHK013 Unexpected editable select success", !db_sql_editor_tester.is_editable_select("select dummy_first_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error));
+  ensure("TF007CHK013 Unexpected editable select success", !form_tester.is_editable_select("select dummy_first_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error));
   ensure_equals("TF007CHK014 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table complex_pk_table.");
 
 
   /* Success case : loading columns include all the primary key columns */
-  ret = db_sql_editor_tester.is_editable_select("select dummy_first_id, dummy_second_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select dummy_first_id, dummy_second_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
   ensure_equals("TF007CHK016 Unexpected error on editable select", error, "");
   ensure("TF007CHK015 Unexpected editable select failure", ret);
 
 
   /* failure case : loading all columns from a table not having primary key */
-  ensure("TF007CHK017 Unexpected editable select sucess", !db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table", schema_name, table_name, error));
+  ensure("TF007CHK017 Unexpected editable select sucess", !form_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table", schema_name, table_name, error));
   ensure_equals("TF007CHK018 Unexpected error on editable select", error, "Table no_pk_table has no primary keys or unique non nullable columns defined. Only tables with primary keys or unique non nullable columns can be edited.");
 
 
   /* Success case : loading all columns from a table having primary key and unique non nullable column */
-  ret = db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
   ensure_equals("TF007CHK021 Unexpected error on editable select", error, "");
   ensure("TF007CHK020 Unexpected editable select failure", ret);
 
   /* Success case : loading columns include primary key */
-  ret = db_sql_editor_tester.is_editable_select("select dummy_pk, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select dummy_pk, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
   ensure_equals("TF007CHK023 Unexpected error on editable select", error, "");
   ensure("TF007CHK022 Unexpected editable select failure", ret);
 
   /* Success case : loading columns include unique non nullable column but not PK */
-  ret = db_sql_editor_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
   ensure_equals("TF007CHK025 Unexpected error on editable select", error, "");
   ensure("TF007CHK024 Unexpected editable select failure", ret);
 
   /* Failure case : loading columns not include primary key nor unique non nullable column */
-  ensure("TF007CHK026 Unexpected editable select sucess", !db_sql_editor_tester.is_editable_select("select title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error));
+  ensure("TF007CHK026 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error));
   ensure_equals("TF007CHK027 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table pk_table_unique_not_null.");
   
   /* Success case : loading all columns from a table NOT having primary key BUT unique non nullable column */
-  ret = db_sql_editor_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
   ensure_equals("TF007CHK031 Unexpected error on editable select", error, "");
   ensure("TF007CHK030 Unexpected editable select failure", ret);
 
   /* Success case : loading columns include unique non nullable column */
-  ret = db_sql_editor_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
+  ret = form_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
   ensure_equals("TF007CHK033 Unexpected error on editable select", error, "");
   ensure("TF007CHK032 Unexpected editable select failure", ret);
 
   /* Failure case : loading columns not unique non nullable column */
-  ensure("TF007CHK034 Unexpected editable select sucess", !db_sql_editor_tester.is_editable_select("select title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error));
+  ensure("TF007CHK034 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error));
   ensure_equals("TF007CHK035 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table no_pk_table_unique_not_null.");
   
   /* Failure case : allowed to do scalar operations over results (bug #64435) */
-  ensure("TF007CHK040 Unexpected editable select sucess", !db_sql_editor_tester.is_editable_select("select ifnull(max(language_id),0)+1 from language", schema_name, table_name, error));
+  ensure("TF007CHK040 Unexpected editable select sucess", !form_tester.is_editable_select("select ifnull(max(language_id),0)+1 from language", schema_name, table_name, error));
   ensure_equals("TF007CHK041 Unexpected error on editable select", error, "The statement is not editable.");
   
 }
@@ -798,9 +795,9 @@ TEST_FUNCTION(100)
 {
   // cleanup
   std::string sql = "DROP DATABASE wb_sql_editor_form_test";
-  db_sql_editor_tester.exec_sql(sql);
-  db_sql_editor->close();
-  db_sql_editor.reset();
+  form_tester.exec_sql(sql);
+  form->close();
+  form.reset();
 }
 
 END_TESTS
\ No newline at end of file
