@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,25 +20,39 @@
 
 #include "stdafx.h"
 
-#include "workbench/wb_context_ui.h"
-#include "sqlide/wb_context_sqlide.h"
-#include "sqlide/wb_sql_editor_form.h"
-#include "sqlide/wb_sql_editor_snippets.h"
-#include "sqlide/wb_sql_editor_tree_controller.h"
-#include "grt/common.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
 #include "base/sqlstring.h"
+#include "base/trackable.h"
+#include "base/log.h"
+#include "base/notifications.h"
+#include "base/ui_form.h"
+
+#include "grtpp.h"
+
+#include "grts/structs.h"
+#include "grts/structs.app.h"
+
+#include "grt/editor_base.h"
+
+#include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+#include "workbench/wb_command_ui.h"
+
+#include "sqlide/wb_context_sqlide.h"
+#include "sqlide/wb_sql_editor_form.h"
+#include "sqlide/wb_sql_editor_snippets.h"
+#include "sqlide/wb_sql_editor_help.h"
+#include "sqlide/wb_sql_editor_tree_controller.h"
+#include "grt/common.h"
 
 #include "objimpl/db.query/db_query_Editor.h"
 #include "objimpl/db.query/db_query_Resultset.h"
 #include "objimpl/db.query/db_query_EditableResultset.h"
 #include "objimpl/db.query/db_query_QueryBuffer.h"
 #include "grts/structs.db.query.h"
-#include "base/trackable.h"
 
 #include "grtdb/db_object_helpers.h"
-#include "base/log.h"
 
 #include "mforms/code_editor.h"
 
@@ -105,7 +119,7 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
         {
           db_query_ResultsetRef e(editor->resultsets()[i]);
           
-          if (e->get_data()->recordset == rset)
+          if (dynamic_cast<WBRecordsetResultset*>(e->get_data())->recordset == rset)
           {
             editor->resultsets().remove(i);
             e->reset_references();
@@ -162,7 +176,15 @@ public:
       return _editor->connection_descriptor();
     return db_mgmt_ConnectionRef();
   }
-  
+
+  virtual grt::IntegerRef isConnected() const
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      return grt::IntegerRef(_editor->connected() ? 1 : 0);
+    return grt::IntegerRef(0);
+  }
+
   virtual db_query_QueryEditorRef addQueryEditor()
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
@@ -210,7 +232,24 @@ public:
 
     return grt::IntegerRef(0);
   }
-  
+
+  virtual db_query_ResultsetRef executeManagementQuery(const std::string &sql, bool log)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      return ref->exec_management_query(sql, log);
+    }
+    return db_query_ResultsetRef();
+  }
+
+  virtual void executeManagementCommand(const std::string &sql, bool log)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      ref->exec_management_sql(sql, log);
+  }
+
   virtual db_query_EditableResultsetRef createTableEditResultset(const std::string &schema, const std::string &table, const std::string &where, bool showGrid)
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
@@ -264,6 +303,45 @@ public:
     return db_query_QueryEditorRef();
   }
 
+  virtual void editLiveObject(const db_DatabaseObjectRef &object, const db_CatalogRef &catalog)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      ref->get_live_tree()->open_alter_object_editor(object, catalog);
+    }
+  }
+
+  virtual void alterLiveObject(const std::string &type, const std::string &schemaName, const std::string &objectName)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      wb::LiveSchemaTree::ObjectType otype;
+      if (type == "db.Schema")
+        otype = wb::LiveSchemaTree::Schema;
+      else if (type == "db.Table")
+        otype = wb::LiveSchemaTree::Table;
+      else if (type == "db.View")
+        otype = wb::LiveSchemaTree::View;
+      else if (type == "db.StoredProcedure")
+        otype = wb::LiveSchemaTree::Procedure;
+      else if (type == "db.Function")
+        otype = wb::LiveSchemaTree::Function;
+      else
+        return;
+      ref->get_live_tree()->do_alter_live_object(otype, schemaName, objectName);
+    }
+  }
+
+  virtual grt::ListRef<db_query_LiveDBObject> schemaTreeSelection() const
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      return grt::ListRef<db_query_LiveDBObject>::cast_from(ref->get_live_tree()->get_schema_tree()->get_selected_objects());
+    return grt::ListRef<db_query_LiveDBObject>();
+  }
+
   void detach()
   {
     _editor.reset();
@@ -279,42 +357,42 @@ protected:
 
 void WBContextSQLIDE::call_in_editor(void (SqlEditorForm::*method)())
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)();
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)();
 }
 
 
 void WBContextSQLIDE::call_in_editor_str(void (SqlEditorForm::*method)(const std::string &arg), const std::string &arg)
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)(arg);
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)(arg);
 }
 
 
 void WBContextSQLIDE::call_in_editor_bool(void (SqlEditorForm::*method)(bool arg), bool arg)
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)(arg);
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)(arg);
 }
 
 static void call_export(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
   {
-    int editor = db_sql_editor->active_sql_editor_index();
+    int editor = form->active_sql_editor_index();
     if (editor >= 0)
-      db_sql_editor->show_export_recordset(editor, db_sql_editor->active_recordset(editor));
+      form->show_export_recordset(editor, form->active_recordset(editor));
   }
 }
 
 static bool validate_export(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  return db_sql_editor && db_sql_editor->active_sql_editor_index() >= 0 && db_sql_editor->active_recordset(db_sql_editor->active_sql_editor_index());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  return form && form->active_sql_editor_index() >= 0 && form->active_recordset(form->active_sql_editor_index());
 }
 
 static void call_run_file(wb::WBContextSQLIDE *sqlide)
@@ -376,11 +454,11 @@ static bool validate_revert(wb::WBContextSQLIDE *sqlide)
 
 static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    std::string text = db_sql_editor->active_sql_editor()->selected_text();
-    if (db_sql_editor->save_snippet(text))
+    std::string text = form->active_sql_editor()->selected_text();
+    if (form->save_snippet(text))
       sqlide->get_grt_manager()->replace_status_text("SQL saved to snippets list.");
   }
 }
@@ -388,30 +466,26 @@ static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 
 static void call_continue_on_error(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
-    db_sql_editor->continue_on_error(!db_sql_editor->continue_on_error());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
+    form->continue_on_error(!form->continue_on_error());
 }
 
 
 static void call_reconnect(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor && !db_sql_editor->is_running_query())
-  {
-    db_sql_editor->grt_manager()->replace_status_text("Reconnecting...");
-    db_sql_editor->connect();
-    db_sql_editor->grt_manager()->replace_status_text("Connection reopened.");
-  }
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+
+  sqlide->reconnect_editor(form);
 }
 
 
 static void call_new_connection(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    db_mgmt_ConnectionRef conn(db_sql_editor->connection_descriptor());
+    db_mgmt_ConnectionRef conn(form->connection_descriptor());
     sqlide->get_wbui()->get_wb()->add_new_query_window(conn);
   }
 }
@@ -419,30 +493,30 @@ static void call_new_connection(wb::WBContextSQLIDE *sqlide)
 
 static void call_exec_sql(wb::WBContextSQLIDE *sqlide, bool current_statement_only)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
-    db_sql_editor->run_editor_contents(current_statement_only);
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
+    form->run_editor_contents(current_statement_only);
 }  
 
 static bool validate_exec_sql(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  return (form && !form->is_running_query() && form->connected());
 }
 
 
 static void call_save_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    int i = db_sql_editor->active_sql_editor_index();
+    int i = form->active_sql_editor_index();
     if (i >= 0)
     {
-      Recordset::Ref rs= db_sql_editor->active_recordset(i);
+      Recordset::Ref rs= form->active_recordset(i);
       if (rs)
       {
-        db_sql_editor->do_partial_ui_refresh(SqlEditorForm::SaveRecordsetChanges);
+        form->do_partial_ui_refresh(SqlEditorForm::SaveRecordsetChanges);
         rs->apply_changes();
       }
     }
@@ -451,16 +525,16 @@ static void call_save_edits(wb::WBContextSQLIDE *sqlide)
 
 static void call_discard_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    int i = db_sql_editor->active_sql_editor_index();
+    int i = form->active_sql_editor_index();
     if (i >= 0)
     {
-      Recordset::Ref rs= db_sql_editor->active_recordset(i);
+      Recordset::Ref rs= form->active_recordset(i);
       if (rs)
       {
-        db_sql_editor->do_partial_ui_refresh(SqlEditorForm::DiscardRecordsetChanges);
+        form->do_partial_ui_refresh(SqlEditorForm::DiscardRecordsetChanges);
         rs->rollback();
       }
     }
@@ -471,13 +545,13 @@ static void call_discard_edits(wb::WBContextSQLIDE *sqlide)
 
 static bool validate_save_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (!db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (!form)
     return false;
-  int i = db_sql_editor->active_sql_editor_index();
+  int i = form->active_sql_editor_index();
   if (i < 0)
     return false;
-  Recordset::Ref rs= db_sql_editor->active_recordset(i);
+  Recordset::Ref rs= form->active_recordset(i);
   return (rs && rs->has_pending_changes());
 }
 
@@ -493,13 +567,13 @@ static bool validate_list_members(wb::WBContextSQLIDE *sqlide)
 
 static void new_script_tab(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
   {
     if (sqlide->get_grt_manager()->get_app_option_int("DbSqlEditor:DiscardUnsavedQueryTabs", 0))
-      db_sql_editor->new_sql_scratch_area();
+      form->new_sql_scratch_area();
     else
-      db_sql_editor->new_sql_script_file();
+      form->new_sql_script_file();
   }
 }
 
@@ -645,6 +719,7 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.new_table", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_table")); 
   cmdui->add_builtin_command("query.new_view", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_view"));
   cmdui->add_builtin_command("query.new_routine", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_routine"));
+  cmdui->add_builtin_command("query.new_function", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_function"));
   
   cmdui->add_builtin_command("query.savesnippet", boost::bind(call_save_snippet, this));
   cmdui->add_builtin_command("query.new_connection", boost::bind(call_new_connection, this));
@@ -692,12 +767,37 @@ void WBContextSQLIDE::finalize()
 }
 
 
-static void *connect_editor(SqlEditorForm::Ref editor)
+void WBContextSQLIDE::reconnect_editor(SqlEditorForm *editor)
+{
+  boost::shared_ptr<sql::TunnelConnection> tunnel;
+
+  // open tunnel, if needed
+  try
+  {
+    tunnel = sql::DriverManager::getDriverManager()->getTunnel(editor->connection_descriptor());
+  }
+  catch (grt::user_cancelled)
+  {
+    editor->grt_manager()->replace_status_text("Tunnel connection cancelled.");
+    return;
+  }
+
+  if (editor && !editor->is_running_query())
+  {
+    editor->grt_manager()->replace_status_text("Reconnecting...");
+    if (editor->connect(tunnel))
+      editor->grt_manager()->replace_status_text("Connection reopened.");
+    else
+      editor->grt_manager()->replace_status_text("Could not reconnect.");
+  }
+}
+
+static void *connect_editor(SqlEditorForm::Ref editor, boost::shared_ptr<sql::TunnelConnection> tunnel)
 {
   try
   {
     log_debug3("Connecting SQL editor...\n");
-    editor->connect();
+    editor->connect(tunnel);
   }
   catch (sql::AuthenticationError &exc)
   {
@@ -721,13 +821,16 @@ static bool cancel_connect_editor()
 
 SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_ConnectionRef &conn)
 {
+  // start by opening the tunnel, if needed
+  boost::shared_ptr<sql::TunnelConnection> tunnel = sql::DriverManager::getDriverManager()->getTunnel(conn);
+
   SqlEditorForm::Ref editor(SqlEditorForm::create(this, conn));
 
   void *result_ptr = 0;
   if (!mforms::Utilities::run_cancelable_task(_("Opening SQL Editor"), 
                                               strfmt(_("An SQL editor instance for '%s' is opening and should be available in a "
                                                        "moment.\n\nPlease stand by..."), conn->name().c_str()),
-                                              boost::bind(connect_editor, editor),
+                                              boost::bind(connect_editor, editor, tunnel),
                                               boost::bind(cancel_connect_editor),
                                               result_ptr))
     throw grt::user_cancelled("canceled");
@@ -754,11 +857,9 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
     
     throw std::runtime_error(tmp);
   }
-  
-  editor->finish_startup();
 
   {
-    // create entry for grt tree
+    // Create entry for grt tree and update volatile data in the connection.
     db_query_EditorRef object(_wbui->get_wb()->get_grt());
     object->owner(_wbui->get_wb()->get_root());
     object->name(conn->name());
@@ -767,6 +868,11 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
 
     {
       std::map<std::string, std::string> details(editor->connection_details());
+      grt::DictRef parameter_values = conn->parameterValues();
+      parameter_values.gset("serverVersion", details["dbmsProductVersion"]);
+      std::time_t time = std::time(0);
+      parameter_values.gset("lastConnected", (long)time);
+
       GrtVersionRef version = CatalogHelper::parse_version(object.get_grt(), details["dbmsProductVersion"]);
       version->owner(object);
       version->name(grt::StringRef(details["dbmsProductName"]));
@@ -775,9 +881,11 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
 
     _wbui->get_wb()->get_root()->sqlEditors().insert(object);
   }
-    
+
   _open_editors.push_back(editor);
 
+  editor->finish_startup();
+
   // setup auto-save for model
   if (!_auto_save_active)
   {
@@ -820,24 +928,6 @@ bool WBContextSQLIDE::activate_live_object(GrtObjectRef object)
   return editor->get_live_tree()->activate_live_object(object);
 }
 
-
-bool WBContextSQLIDE::create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name)
-{
-  SqlEditorForm *editor= get_active_sql_editor();
-  if (!editor)
-    return false;
-  return editor->get_live_tree()->create_live_object(object_type, owner_name, obj_name);
-}
-
-
-bool WBContextSQLIDE::drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name)
-{
-  SqlEditorForm *editor= get_active_sql_editor();
-  if (!editor)
-    return false;
-  return editor->get_live_tree()->drop_live_object(object_type, owner_name, obj_name);
-}
-
 //--------------------------------------------------------------------------------------------------
 
 void WBContextSQLIDE::open_document(const std::string &path)
@@ -933,6 +1023,8 @@ void WBContextSQLIDE::update_plugin_arguments_pool(bec::ArgumentPool &args)
         if (rset.is_valid())
           args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
       }
+      else
+        args.add_entries_for_object("activeSQLEditor", editor);
     }
   }
 }
@@ -953,3 +1045,56 @@ db_query_EditorRef WBContextSQLIDE::get_grt_editor_object(SqlEditorForm *editor)
 }
 
 
+
+
+
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterNotifDocs
+{
+  RegisterNotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GRNSQLEditorOpened",
+                                                           "sqlide",
+                                                           "Sent when a connection tab finishes initializing and is about to be shown on screen.",
+                                                           "db.query.Editor instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNSQLEditorReconnected",
+                                                           "sqlide",
+                                                           "Sent when the connection state of a SQL editor changes (reconnect, disconnect) it's DB connection. Resent whenever the user clicks Reconnect.",
+                                                           "db.query.Editor instance",
+                                                           "connected - whether the connection is open");
+
+    base::NotificationCenter::get()->register_notification("GRNServerStateChanged",
+                                                           "sqlide",
+                                                           "Sent by the Admin module when it is detected that the server state (running or stopped) changed.",
+                                                           "db.query.Editor instance",
+                                                           "state - running or stopped\n"
+                                                           "connection - the connection object for the server");
+
+    base::NotificationCenter::get()->register_notification("GRNLiveDBObjectMenuWillShow",
+                                                           "sqlide",
+                                                           "Sent when the context menu is about to be shown for a live DB object (eg. from the live schema tree in the SQL IDE sidebar).",
+                                                           "db.query.Editor instance",
+                                                           "menu - mforms.ObjectReference of a mforms.Menu object which is being shown\n"
+                                                           "selection - a list of db.query.LiveDBObject for the selected objects");
+
+    base::NotificationCenter::get()->register_notification("GRNLiveDBObjectSelectionDidChange",
+                                                           "sqlide",
+                                                           "Sent when the selection in the schema tree changes. Avoid hooking slow actions to this as it will make object selection less smooth.",
+                                                           "db.query.Editor instance",
+                                                           "selection-size - number of selected items in the tree");
+
+    base::NotificationCenter::get()->register_notification("GRNSQLResultSetMenuWillShow",
+                                                         "sqlide",
+                                                         "Sent when the context menu is about to be shown for a resultset in the SQL IDE.",
+                                                         "db.query.Resultset instance",
+                                                         "menu - mforms.ObjectReference of a mforms.Menu object which is being shown\n"
+                                                         "selected_rows - a list of int values of the selected rows\n"
+                                                         "selected_column - if this value is set, the selection is a single cell at selected_rows[0], selected_column");
+  }
+} initdocs;
+
+
