@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,7 +43,8 @@ namespace wb
     static const std::string SCHEMA_TAG;
     static const std::string TABLES_TAG;
     static const std::string VIEWS_TAG;
-    static const std::string ROUTINES_TAG;
+    static const std::string PROCEDURES_TAG;
+    static const std::string FUNCTIONS_TAG;
     static const std::string TABLE_TAG;
     static const std::string VIEW_TAG;
     static const std::string ROUTINE_TAG;
@@ -58,19 +59,23 @@ namespace wb
 
 
     static const std::string FETCHING_CAPTION;
+    static const std::string ERROR_FETCHING_CAPTION;
     static const std::string TABLES_CAPTION;
     static const std::string VIEWS_CAPTION;
-    static const std::string ROUTINES_CAPTION;
+    static const std::string PROCEDURES_CAPTION;
+    static const std::string FUNCTIONS_CAPTION;
 
     static const std::string COLUMNS_CAPTION;
     static const std::string INDEXES_CAPTION;
     static const std::string TRIGGERS_CAPTION;
     static const std::string FOREIGN_KEYS_CAPTION;
 
+    static const std::string LST_INFO_BOX_DETAIL_ROW;
 
     static const int TABLES_NODE_INDEX;
     static const int VIEWS_NODE_INDEX;
-    static const int ROUTINES_NODE_INDEX;
+    static const int PROCEDURES_NODE_INDEX;
+    static const int FUNCTIONS_NODE_INDEX;
     static const int TABLE_COLUMNS_NODE_INDEX;
     static const int TABLE_INDEXES_NODE_INDEX;
     static const int TABLE_FOREIGN_KEYS_NODE_INDEX;
@@ -94,7 +99,8 @@ namespace wb
 
       TableCollection,
       ViewCollection,
-      RoutineCollection,
+      ProcedureCollection,
+      FunctionCollection,
 
       ColumnCollection,
       IndexCollection,
@@ -117,7 +123,9 @@ namespace wb
     {
       DatabaseObject, // Schema, Table, View, Procedure, Function
       SchemaObject,   // Table, View, Procedure, Function
-      TableOrView     // Table, View
+      TableOrView,     // Table, View
+      ColumnObject,     // TableColumn, ViewVolumn
+      RoutineObject     // Procedure, Function
     };
 
     // One entry in a list that describes a single object.
@@ -146,12 +154,18 @@ namespace wb
       ObjectType _type;
     public:
       ColumnData(ObjectType type = TableColumn) : 
-          LSTData(), is_pk(false), is_fk(false), is_id(false){_type = type;}
+          LSTData(), is_pk(false), is_fk(false), is_id(false) { _type = type; }
+      // NOTE than name will contain the column name duplicating this info as it
+      //      is already on the TreeNode. Duplicating it is better than having
+      //      full HTML for all the columns on all the tables in the db.
+      std::string name;
       std::string type;
       std::string default_value;
+      std::string charset_collation;
       bool is_pk;
       bool is_fk;
       bool is_id;
+      bool is_idx;
       
       virtual void copy(LSTData* other);
       virtual ObjectType get_type() {return _type; }
@@ -166,6 +180,8 @@ namespace wb
       unsigned char update_rule;
       unsigned char delete_rule;
       std::string referenced_table;
+      std::string from_cols;
+      std::string to_cols;
 
       virtual void copy(LSTData* other);
       virtual ObjectType get_type() {return LiveSchemaTree::ForeignKey; }
@@ -179,9 +195,11 @@ namespace wb
       IndexData():LSTData(), unique(false), type(0){}
       bool unique;
       unsigned char type;
+      std::vector<std::string>columns;
 
       virtual void copy(LSTData* other);
       virtual ObjectType get_type() {return LiveSchemaTree::Index; }
+      virtual std::string get_details(bool full, const mforms::TreeNodeRef& node);
       virtual std::string get_object_name() { return _("Index"); }
     };
 
@@ -194,6 +212,7 @@ namespace wb
 
       virtual void copy(LSTData* other);
       virtual ObjectType get_type() {return LiveSchemaTree::Trigger; }
+      virtual std::string get_details(bool full, const mforms::TreeNodeRef& node);
       virtual std::string get_object_name() { return _("Trigger"); }
     };
 
@@ -215,6 +234,7 @@ namespace wb
       ProcedureData():ObjectData(){}
       virtual ObjectType get_type() {return LiveSchemaTree::Procedure; }
       virtual std::string get_object_name() { return _("Procedure"); }
+      virtual std::string get_details(bool full, const mforms::TreeNodeRef& node);
     };
 
     class MYSQLWBBACKEND_PUBLIC_FUNC FunctionData: public ObjectData
@@ -223,6 +243,7 @@ namespace wb
       FunctionData():ObjectData(){}
       virtual ObjectType get_type() {return LiveSchemaTree::Function; }
       virtual std::string get_object_name() { return _("Function"); }
+      virtual std::string get_details(bool full, const mforms::TreeNodeRef& node);
     };
 
     class MYSQLWBBACKEND_PUBLIC_FUNC ViewData: public ObjectData
@@ -280,25 +301,20 @@ namespace wb
       virtual bool fetch_data_for_filter(const std::string &, const std::string &, const NewSchemaContentArrivedSlot&) = 0;
       virtual bool fetch_schema_contents(const std::string &, const NewSchemaContentArrivedSlot&) = 0;
       virtual bool fetch_object_details(const std::string& schema_name, const std::string& object_name, wb::LiveSchemaTree::ObjectType type, short, const NodeChildrenUpdaterSlot&) = 0;
+      virtual bool fetch_routine_details(const std::string& schema_name, const std::string& object_name, wb::LiveSchemaTree::ObjectType type) = 0;
     };
     
     struct Delegate
     {
       virtual void tree_refresh() = 0;
+      virtual bool tree_special_action(const std::string&) = 0;
       virtual void tree_activate_objects(const std::string&, const std::vector<ChangeRecord>&) = 0;
-      virtual void tree_alter_objects(const std::vector<ChangeRecord>&) = 0;
-      virtual void tree_create_object(ObjectType, const std::string&, const std::string&) = 0;
-      virtual void tree_drop_objects(const std::vector<ChangeRecord>&) = 0;
     };
     
     
     typedef boost::signals2::signal<int (const std::string&)> SqlEditorTextInsertSignal;
-    typedef boost::function<bec::MenuItemList (std::string, std::string, std::string)> ObjectPluginMenuItemsSlot;
-    typedef boost::function<bool (std::string, std::string, std::string, std::string)> CallObjectPluginMenuItemsSlot;
 
     SqlEditorTextInsertSignal sql_editor_text_insert_signal;
-    ObjectPluginMenuItemsSlot object_plugin_items_slot;
-    CallObjectPluginMenuItemsSlot call_object_plugin_items_slot;
 
   protected:
     boost::weak_ptr<FetchDelegate> _fetch_delegate;
@@ -309,10 +325,10 @@ namespace wb
 
     bool _case_sensitive_identifiers;
 
-    boost::function<bool (const std::string& first, const std::string& second)> compare_func;
     void schema_contents_arrived(const std::string &schema_name, std::list<std::string> *tables, std::list<std::string> *views, std::list<std::string> *procedures, std::list<std::string> *functions, bool just_append);
     void load_table_details(mforms::TreeNodeRef& node, int fetch_mask);
     void fetch_table_details(ObjectType object_type, const std::string schema_name, const std::string object_name, int fetch_mask);
+    void load_routine_details(mforms::TreeNodeRef& node);
     void load_schema_content(mforms::TreeNodeRef& schema_node);
     void reload_object_data(mforms::TreeNodeRef& node);
     void discard_object_data(mforms::TreeNodeRef& node, int data_mask);
@@ -344,13 +360,18 @@ namespace wb
 
     void set_active_schema(const std::string &schema);
 
+    void set_no_connection();
+
     void update_live_object_state(ObjectType type, const std::string &schema_name, const std::string &old_obj_name, const std::string &new_obj_name);
 
     virtual std::string get_field_description(const mforms::TreeNodeRef& node);
     
     mforms::TreeNodeRef get_node_for_object(const std::string &schema_name, ObjectType type, const std::string& name);
     mforms::TreeNodeRef create_node_for_object(const std::string &schema_name, ObjectType type, const std::string& name);
-    
+
+    // returns a list of db.query.LiveDBObject
+    grt::BaseListRef get_selected_objects();
+
     virtual bool activate_popup_item_for_nodes(const std::string &name, const std::list<mforms::TreeNodeRef> &orig_nodes);
     virtual bec::MenuItemList get_popup_items_for_nodes(const std::list<mforms::TreeNodeRef> &nodes);
 
@@ -366,12 +387,9 @@ namespace wb
     void update_node_icon(mforms::TreeNodeRef node);
 
     void update_schemata(std::list<std::string> &schema_list);
-    bool compare_identifiers(const std::string &first, const std::string &second);
-    int string_compare(const std::string &first, const std::string &second);
     mforms::TreeNodeRef binary_search_node(const mforms::TreeNodeRef& parent, int first, int last, const std::string &name, ObjectType type, int &position);
     mforms::TreeNodeRef get_child_node(const mforms::TreeNodeRef& parent, const std::string& name, ObjectType type = Any, bool binary_search = true);
     bool find_child_position(const mforms::TreeNodeRef& parent, const std::string& name, ObjectType type, int &position);
-    void find_routine_ranges(const mforms::TreeNodeRef& parent, int& pstart, int& pend, int& fstart, int& fend);
     void expand_toggled(mforms::TreeNodeRef node, bool value);
     void node_activated(mforms::TreeNodeRef node, int column);
     void set_case_sensitive_identifiers(bool flag);
