@@ -37,6 +37,7 @@
 #include "base/notifications.h"
 #include "base/sqlstring.h"
 #include "base/file_functions.h"
+#include "base/file_utilities.h"
 #include "base/log.h"
 #include "base/boost_smart_ptr_helpers.h"
 
@@ -45,11 +46,7 @@
 
 #include "objimpl/db.query/db_query_Resultset.h"
 
-// TODO: check if this inclusion is still necessary.
-// this doesn't belong here, but there's no other workaround for having access to mysql_info
-// because of that we also need to link wbprivate directly to the connector, making their
-// dynamic loading pretty much useless
-#include <cppconn/../driver/mysql_connection.h>
+#include <mysql_connection.h>
 
 #include <boost/foreach.hpp>
 #include <boost/scoped_ptr.hpp>
@@ -59,7 +56,6 @@
 #include "query_side_palette.h"
 
 #include "mforms/menubar.h"
-#include "mforms/grttreeview.h"
 #include "mforms/hypertext.h" // needed for d-tor
 #include "mforms/tabview.h" // needed for d-tor
 #include "mforms/splitter.h" // needed for d-tor
@@ -191,9 +187,12 @@ SqlEditorForm::SqlEditorForm(wb::WBContextSQLIDE *wbsql, const db_mgmt_Connectio
   _history(DbSqlEditorHistory::create(_grtm))
 {
   _log = DbSqlEditorLog::create(this, _grtm, 500);
-  
+
+  NotificationCenter::get()->add_observer(this, "GNApplicationActivated");
   NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
   NotificationCenter::get()->add_observer(this, "GNFormTitleDidChange");
+  NotificationCenter::get()->add_observer(this, "GNColorsChanged");
+  GRTNotificationCenter::get()->add_grt_observer(this, "GRNServerStateChanged");
   
   _sql_editors_mutex= g_mutex_new();
   _aux_dbc_conn_mutex= g_mutex_new();
@@ -216,13 +215,22 @@ SqlEditorForm::SqlEditorForm(wb::WBContextSQLIDE *wbsql, const db_mgmt_Connectio
     _keep_alive_thread->on_exit.connect(boost::bind(&SqlEditorForm::reset_keep_alive_thread, this));
   }
 
+  _lower_case_table_names = 0;
+  if (_grtm->get_app_option_int("SqlEditor:LimitRows"))
+    _limit_rows = _grtm->get_app_option_int("SqlEditor:LimitRowsCount", 10);
+  else
+    _limit_rows = 0;
+
   _continue_on_error= (_grtm->get_app_option_int("DbSqlEditor:ContinueOnError", 0) != 0);
+
+  // set initial autocommit mode value
+  _usr_dbc_conn->autocommit_mode= (_grtm->get_app_option_int("DbSqlEditor:AutocommitMode", 1) != 0);
 }
 
 
 void SqlEditorForm::finish_startup()
 {
-  setup_sidebars();
+  setup_side_palette();
 
   _live_tree->finish_init();
 
@@ -230,6 +238,8 @@ void SqlEditorForm::finish_startup()
     _live_tree->on_active_schema_change(_usr_dbc_conn->active_schema);
 
   _grtm->run_once_when_idle(this, boost::bind(&SqlEditorForm::update_menu_and_toolbar, this));
+
+  GRTNotificationCenter::get()->send_grt("GRNSQLEditorOpened", wbsql()->get_grt_editor_object(this), grt::DictRef());
 }
 
 //-------------------------------------------------------------------------------------------------- 
@@ -286,6 +296,7 @@ SqlEditorForm::~SqlEditorForm()
   }
 
   NotificationCenter::get()->remove_observer(this);
+  GRTNotificationCenter::get()->remove_grt_observer(this);
 
   delete _auto_completion_cache;
   delete _autosave_lock;
@@ -311,6 +322,18 @@ SqlEditorForm::~SqlEditorForm()
 }
 
 
+void SqlEditorForm::handle_grt_notification(const std::string &name, grt::ObjectRef sender, grt::DictRef info)
+{
+  if (name == "GRNServerStateChanged")
+  {
+    db_mgmt_ConnectionRef conn(db_mgmt_ConnectionRef::cast_from(info.get("connection")));
+
+    // reconnect when idle, to avoid any deadlocks
+    if (conn.is_valid() && conn == connection_descriptor())
+      _grtm->run_once_when_idle(this, boost::bind(&WBContextSQLIDE::reconnect_editor, wbsql(), this));
+  }
+}
+
 void SqlEditorForm::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
   if (name == "GNMainFormChanged")
@@ -336,7 +359,16 @@ void SqlEditorForm::handle_notification(const std::string &name, void *sender, b
         title_changed();
       }
     }
-  }  
+  }
+  else if (name == "GNColorsChanged")
+  {
+    // Single colors or the entire color scheme changed.
+    update_toolbar_icons();
+  }
+  else if (name == "GNApplicationActivated")
+  {
+    check_external_file_changes();
+  }
 }
 
 
@@ -726,8 +758,9 @@ struct ConnectionErrorInfo
 {
   sql::AuthenticationError *auth_error;
   bool password_expired;
+  bool server_probably_down;
 
-  ConnectionErrorInfo() : auth_error(0), password_expired(0) {}
+  ConnectionErrorInfo() : auth_error(0), password_expired(false), server_probably_down(false) {}
   ~ConnectionErrorInfo()
   {
     delete auth_error;
@@ -735,18 +768,16 @@ struct ConnectionErrorInfo
 };
 
 
-void SqlEditorForm::connect()
+bool SqlEditorForm::connect(boost::shared_ptr<sql::TunnelConnection> tunnel)
 {
   sql::Authentication::Ref auth = _dbc_auth;//sql::Authentication::create(_connection, "");
-  boost::shared_ptr<sql::TunnelConnection> tunnel;
   enum PasswordMethod {
     NoPassword,
     KeychainPassword,
     InteractivePassword
   } current_method = NoPassword;
-  
-  // open tunnel, if needed
-  tunnel = sql::DriverManager::getDriverManager()->getTunnel(_connection);
+
+  reset();
 
   // In the 1st connection attempt, no password is supplied
   // If it fails, keychain is checked and used if it exists
@@ -770,6 +801,8 @@ void SqlEditorForm::connect()
 
       if (!error_ptr.auth_error)
         throw;
+      else if (error_ptr.server_probably_down)
+        return false;
 
       if (current_method == NoPassword)
       {
@@ -800,6 +833,26 @@ void SqlEditorForm::connect()
     }
     break;
   }
+
+  // we should only send this after the initial connection
+  // assumes setup_side_palette() is called in finish_init(), signalizing that the editor was already initialized once
+  if (_side_palette) // we're in a thread here, so make sure the notification is sent from the main thread
+  {
+    _grtm->run_once_when_idle(this, boost::bind(&SqlEditorForm::update_connected_state, this));
+  }
+
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void SqlEditorForm::update_connected_state()
+{
+  grt::DictRef args(_grtm->get_grt());
+  args.gset("connected", connected());
+  GRTNotificationCenter::get()->send_grt("GRNSQLEditorReconnected", wbsql()->get_grt_editor_object(this), args);
+
+  update_menu_and_toolbar();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -822,49 +875,50 @@ grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::T
     GMutexLock aux_dbc_conn_mutex(_aux_dbc_conn_mutex);
     GMutexLock usr_dbc_conn_mutex(_usr_dbc_conn_mutex);
 
-    reset();
-
     _aux_dbc_conn->ref.reset();
     _usr_dbc_conn->ref.reset();
+
+    // connection info
+    _connection_details["name"] = _connection->name();
+    _connection_details["hostName"] = _connection->parameterValues().get_string("hostName");
+    _connection_details["port"] = strfmt("%li\n", _connection->parameterValues().get_int("port"));
+    _connection_details["socket"] = _connection->parameterValues().get_string("socket");
+    _connection_details["driverName"] = _connection->driver()->name();
+    _connection_details["userName"] = _connection->parameterValues().get_string("userName");
+      
+    // Connection:
+    _connection_info= "<div style=\"color=#3b3b3b; font-weight:bold\">Connection:</div>";
+    _connection_info.append(create_html_line("Name: ", _connection->name()));
+    // Host:
+    if (_connection->driver()->name() == "MysqlNativeSocket")
+    {
+#ifdef _WIN32
+      std::string name = _connection->parameterValues().get_string("socket", "");
+      if (name.empty())
+        name = "pipe";
+#else
+      std::string name = _connection->parameterValues().get_string("socket", "");
+      if (name.empty())
+        name = "UNIX socket";
+#endif
+      _connection_info.append(create_html_line("Host:", "localhost (" + name + ")"));
+    }
+    else
+    {
+      _connection_info.append(create_html_line("Host:", _connection->parameterValues().get_string("hostName")));
+      _connection_info.append(create_html_line("Port:", strfmt("%i", (int)_connection->parameterValues().get_int("port"))));
+    }
+
+    // open connections
     create_connection(_aux_dbc_conn, _connection, tunnel, auth, _aux_dbc_conn->autocommit_mode);
     create_connection(_usr_dbc_conn, _connection, tunnel, auth, _usr_dbc_conn->autocommit_mode);
 
     cache_sql_mode();
 
-    // connection info
     try
     {
-      _connection_details["name"] = _connection->name();
-      _connection_details["hostName"] = _connection->parameterValues().get_string("hostName");
-      _connection_details["port"] = strfmt("%li\n", _connection->parameterValues().get_int("port"));
-      _connection_details["socket"] = _connection->parameterValues().get_string("socket");
-      _connection_details["driverName"] = _connection->driver()->name();
       _connection_details["dbmsProductName"] = _usr_dbc_conn->ref->getMetaData()->getDatabaseProductName();
       _connection_details["dbmsProductVersion"] = _usr_dbc_conn->ref->getMetaData()->getDatabaseProductVersion();
-      _connection_details["userName"] = _connection->parameterValues().get_string("userName");
-        
-      // Connection:
-      _connection_info= "<div style=\"color=#3b3b3b; font-weight:bold\">Connection:</div>";
-      _connection_info.append(create_html_line("Name: ", _connection->name()));
-      // Host:
-      if (_connection->driver()->name() == "MysqlNativeSocket")
-      {
-#ifdef _WIN32
-        std::string name = _connection->parameterValues().get_string("socket", "");
-        if (name.empty())
-          name = "pipe";
-#else
-        std::string name = _connection->parameterValues().get_string("socket", "");
-        if (name.empty())
-          name = "UNIX socket";
-#endif
-        _connection_info.append(create_html_line("Host:", "localhost (" + name + ")"));
-      }
-      else
-      {
-        _connection_info.append(create_html_line("Host:", _connection->parameterValues().get_string("hostName")));
-        _connection_info.append(create_html_line("Port:", strfmt("%i", (int)_connection->parameterValues().get_int("port"))));
-      }
 
       // Server:
       _connection_info.append(create_html_line("Server:", _connection_details["dbmsProductName"]));
@@ -880,6 +934,11 @@ grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::T
         if (rs->next())
           _connection_info.append(create_html_line("Current User:", rs->getString(1)));
       }
+
+      // get lower_case_table_names value
+      std::string value;
+      if (get_session_variable(_usr_dbc_conn, "lower_case_table_names", value))
+        _lower_case_table_names = atoi(value.c_str());
     }
     CATCH_ANY_EXCEPTION_AND_DISPATCH(_("Get connection information"));
   }
@@ -890,11 +949,29 @@ grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::T
   }
   catch (sql::SQLException &exc)
   {
-    if (exc.getErrorCode() == 1820)
+    if (exc.getErrorCode() == 1820) // ER_MUST_CHANGE_PASSWORD_LOGIN
       err_ptr->password_expired = true;
+    else if (exc.getErrorCode() == 2003 || exc.getErrorCode() == 2002) // ERROR 2003 (HY000): Can't connect to MySQL server on X.Y.Z.W (or via socket)
+    {
+      _connection_info.append(create_html_line("", "<b><span style='color: red'>NO CONNECTION</span></b>"));
+      add_log_message(WarningMsg, exc.what(), "Could not connect, server may not be running.", "");
+
+      err_ptr->server_probably_down = true;
+
+      // if there's no connection, then we continue anyway if this is a local connection or
+      // a remote connection with remote admin enabled.. 
+      grt::Module *m = _grtm->get_grt()->get_module("WbAdmin");
+      grt::BaseListRef args(_grtm->get_grt());
+      args.ginsert(_connection);
+      if (!m || *grt::IntegerRef::cast_from(m->call_function("checkConnectionForRemoteAdmin", args)) == 0)
+        throw exc;
+
+      log_info("Error 2003 connecting to server, assuming server is down and opening editor with no connection\n");
+      return grt::StringRef();
+    }
     throw;
   }
-  if (_grtm->get_app_option_int("DbSqlEditor:CodeCompletionEnabled") == 1)
+  if (_grtm->get_app_option_int("DbSqlEditor:CodeCompletionEnabled") == 1 && connected())
   {
     std::string cache_dir = _grtm->get_user_datadir() + "/cache/";
     try
@@ -942,7 +1019,9 @@ void SqlEditorForm::on_cache_action(bool active)
 
 bool SqlEditorForm::connected() const
 {
-  return (_usr_dbc_conn && _usr_dbc_conn->ref.get_ptr() && !_usr_dbc_conn->ref->isClosed());
+  if (_usr_dbc_conn_mutex && _usr_dbc_conn && _usr_dbc_conn->ref.get_ptr())
+    return true; // we don't need to PING the server every time we want to check if the editor is connected
+  return false;
 }
 
 
@@ -968,6 +1047,7 @@ GMutexLock SqlEditorForm::ensure_valid_usr_connection()
 
 void SqlEditorForm::close_connection(sql::Dbc_connection_handler::Ref &dbc_conn)
 {
+  sql::Dbc_connection_handler::Ref myref(dbc_conn);
   if (dbc_conn && dbc_conn->ref.get_ptr())
   {
     try
@@ -986,6 +1066,8 @@ GMutexLock SqlEditorForm::ensure_valid_dbc_connection(sql::Dbc_connection_handle
 {
   GMutexLock mutex_lock(dbc_conn_mutex);
   bool valid= false;
+
+  sql::Dbc_connection_handler::Ref myref(dbc_conn);
   if (dbc_conn && dbc_conn->ref.get_ptr())
   {
     if (dbc_conn->ref->isClosed())
@@ -1004,7 +1086,7 @@ GMutexLock SqlEditorForm::ensure_valid_dbc_connection(sql::Dbc_connection_handle
       valid= true;
   }
   if (!valid)
-    throw std::runtime_error("DBMS connection is not available");
+    throw grt::db_not_connected("DBMS connection is not available");
 
   return mutex_lock;
 }
@@ -1154,6 +1236,9 @@ void SqlEditorForm::do_explain_sql(const std::string &sql)
 void SqlEditorForm::exec_sql_retaining_editor_contents(const std::string &sql_script, Sql_editor::Ref editor, bool sync, bool dont_add_limit_clause)
 {
   auto_save();
+
+  if (!connected())
+    throw grt::db_not_connected("Not connected");
   
   RecordsetsRef recordsets;
   int i = sql_editor_index(editor);
@@ -1184,6 +1269,9 @@ void SqlEditorForm::run_editor_contents(bool current_statement_only)
 
 RecordsetsRef SqlEditorForm::exec_sql_returning_results(const std::string &sql_script, bool dont_add_limit_clause)
 {
+  if (!connected())
+    throw grt::db_not_connected("Not connected");
+
   RecordsetsRef rsets(new Recordsets());
   
   do_exec_sql(_grtm->get_grt(), weak_ptr_from(this), boost::shared_ptr<std::string>(new std::string(sql_script)),
@@ -1445,12 +1533,11 @@ grt::StringRef SqlEditorForm::do_exec_sql(grt::GRT *grt, Ptr self_ptr, boost::sh
 
           data_storage->additional_clauses(editable_query_tail);
           {
-            bool limit_rows= !dont_add_limit_clause && (0 != _grtm->get_app_option_int("SqlEditor:LimitRows", 0));
+            bool limit_rows= !dont_add_limit_clause && _limit_rows > 0;
             data_storage->limit_rows(limit_rows);
 
-            int limit_rows_count= _grtm->get_app_option_int("SqlEditor:LimitRowsCount");
-            if (0 != limit_rows_count)
-              data_storage->limit_rows_count(limit_rows_count);
+            if (0 != _limit_rows)
+              data_storage->limit_rows_count(_limit_rows);
           }
           statement= data_storage->decorated_sql_query();
         }
@@ -1595,6 +1682,8 @@ grt::StringRef SqlEditorForm::do_exec_sql(grt::GRT *grt, Ptr self_ptr, boost::sh
                                        (table_name.empty() ? _("Result") : table_name.c_str()),
                                        ++*rs_sequence));
                     rs->generator_query(statement);
+                    scoped_connect(rs->get_context_menu()->signal_will_show(),
+                          boost::bind(&SqlEditorForm::on_recordset_context_menu_show, this, Recordset::Ptr(rs), Sql_editor::Ptr(editor)));
                     rs->action_list().register_action("recall_query",
                       boost::bind(&SqlEditorForm::recall_recordset_query, this, Recordset::Ptr(rs)));
                     rs->action_list().register_action("record_export",
@@ -1607,8 +1696,6 @@ grt::StringRef SqlEditorForm::do_exec_sql(grt::GRT *grt, Ptr self_ptr, boost::sh
                       boost::bind(&bec::RefreshUI::do_partial_ui_refresh, this, (int)RefreshRecordsetTitle));
 
                     rs->data_storage(data_storage);
-                    rs->popup_menu_items_slot= boost::bind(&SqlEditorForm::recordset_popup_menu_items, this, _1, _2, Recordset::Ptr(rs));
-                    rs->call_popup_menu_item_slot= boost::bind(&SqlEditorForm::call_recordset_popup_menu_item, this, _1, _2, _3, Recordset::Ptr(rs));
                     rs->reset(true);
                     if (data_storage->valid()) // query statement
                     {
@@ -1674,6 +1761,124 @@ stop_processing_sql_script:
 }
 
 
+void SqlEditorForm::exec_management_sql(const std::string &sql, bool log)
+{
+  sql::Dbc_connection_handler::Ref conn;
+  base::GMutexLock lock(ensure_valid_aux_connection(conn));
+  if (conn)
+  {
+    RowId rid = log ? add_log_message(DbSqlEditorLog::BusyMsg, _("Executing "), sql, "- / ?") : 0;
+    boost::scoped_ptr<sql::Statement> stmt(conn->ref->createStatement());
+    Timer statement_exec_timer(false);
+    try
+    {
+      stmt->execute(sql);
+    }
+    catch (sql::SQLException &e)
+    {
+      if (log)
+        set_log_message(rid, DbSqlEditorLog::ErrorMsg, strfmt(SQL_EXCEPTION_MSG_FORMAT, e.getErrorCode(), e.what()), sql, "");
+      throw;
+    }
+    CATCH_EXCEPTION_AND_DISPATCH(sql);
+
+    if (log)
+      set_log_message(rid, DbSqlEditorLog::OKMsg, _("OK"), sql, statement_exec_timer.duration_formatted());
+
+    handle_command_side_effects(sql);
+  }
+}
+
+
+void SqlEditorForm::handle_command_side_effects(const std::string &sql)
+{
+  SqlFacade::Ref sql_facade= SqlFacade::instance_for_rdbms(rdbms());
+
+  std::string object_type;
+  std::string schema_name = active_schema();
+  std::vector<std::pair<std::string, std::string> > object_names;
+
+  // special hack, check for some special commands and update UI accordingly
+  if (sql_facade->parseDropStatement(sql, object_type, object_names) && !object_names.empty())
+  {
+    if (object_type == "db.Table")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Table, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.View")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::View, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.StoredProcedure")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Procedure, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.Function")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Function, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.Index")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Index, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.Trigger")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Trigger, object_names[i].first.empty() ? schema_name : object_names[i].first, object_names[i].second, "");
+    }
+    else if (object_type == "db.Schema")
+    {
+      for (int i = object_names.size()-1; i >= 0; --i)
+      {
+        _live_tree->refresh_live_object_in_overview(LiveSchemaTree::Schema, object_names[i].first, object_names[i].first, "");
+        schema_name = object_names[i].first;
+      }
+      if ((schema_name.size() > 0) && (active_schema() == schema_name) && connection_descriptor().is_valid())
+      {
+        std::string default_schema= connection_descriptor()->parameterValues().get_string("schema", "");
+        if (schema_name == default_schema)
+          default_schema = "";
+        _grtm->run_once_when_idle(this, boost::bind(&set_active_schema, shared_from_this(), default_schema));
+      }
+    }
+  }
+}
+
+db_query_ResultsetRef SqlEditorForm::exec_management_query(const std::string &sql, bool log)
+{
+  sql::Dbc_connection_handler::Ref conn;
+  base::GMutexLock lock(ensure_valid_aux_connection(conn));
+  if (conn)
+  {
+    RowId rid = log ? add_log_message(DbSqlEditorLog::BusyMsg, _("Executing "), sql, "- / ?") : 0;
+    boost::scoped_ptr<sql::Statement> stmt(conn->ref->createStatement());
+    Timer statement_exec_timer(false);
+    try
+    {
+      boost::shared_ptr<sql::ResultSet> results(stmt->executeQuery(sql));
+
+      if (log)
+        set_log_message(rid, DbSqlEditorLog::OKMsg, _("OK"), sql, statement_exec_timer.duration_formatted());
+
+      return grtwrap_recordset(wbsql()->get_grt_editor_object(this), results);
+    }
+    catch (sql::SQLException &e)
+    {
+      if (log)
+        set_log_message(rid, DbSqlEditorLog::ErrorMsg, strfmt(SQL_EXCEPTION_MSG_FORMAT, e.getErrorCode(), e.what()), sql, "");
+      throw;
+    }
+    CATCH_EXCEPTION_AND_DISPATCH(sql);
+  }
+  return db_query_ResultsetRef();
+}
+
+
 bool SqlEditorForm::is_running_query()
 {
   return _is_running_query;
@@ -1728,7 +1933,7 @@ void SqlEditorForm::active_recordset(int editor, Recordset::Ref rset)
     db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(_sql_editors[editor]->editor->grtobj()));
     for (size_t c = qeditor->resultsets().count(), i = 0; i < c; i++)
     {
-      if (qeditor->resultsets()[i]->get_data()->recordset == rset)
+      if (dynamic_cast<WBRecordsetResultset*>(qeditor->resultsets()[i]->get_data())->recordset == rset)
       {
         found = true;
         qeditor->activeResultset(qeditor->resultsets()[i]);
@@ -1917,7 +2122,7 @@ void SqlEditorForm::apply_object_alter_script(std::string &alter_script, bec::DB
     // in case of alter script failure:
     // try to restore object since it could had been successfully dropped before the alter script failed
     db_DatabaseObjectRef db_object= obj_editor->get_dbobject();
-    std::string original_object_ddl_script= grt::StringRef::cast_from(db_object->customData().get("originalObjectDDL"));
+    std::string original_object_ddl_script= db_object->customData().get_string("originalObjectDDL", "");
     if (!original_object_ddl_script.empty())
     {
       // reuse the setting schema statement which is the first statement of the alter script
@@ -1979,7 +2184,7 @@ void SqlEditorForm::apply_object_alter_script(std::string &alter_script, bec::DB
       if (!schema_name.empty())
         schema= db_SchemaRef::cast_from(db_object->owner());
       
-      wb::LiveSchemaTree::ObjectType db_object_type;
+      wb::LiveSchemaTree::ObjectType db_object_type = wb::LiveSchemaTree::Any;
       if (db_SchemaRef::can_wrap(db_object))
         db_object_type= wb::LiveSchemaTree::Schema;
       else if (db_TableRef::can_wrap(db_object))
@@ -2070,9 +2275,6 @@ void SqlEditorForm::cache_active_schema_name()
   if(_auto_completion_cache)
     _auto_completion_cache->refresh_schema_cache_if_needed(schema);
   
-  // should also happen in Main thread
-  //_live_tree->on_active_schema_change(schema);
-
   exec_sql_task->execute_in_main_thread(
       boost::bind(&SqlEditorForm::update_editor_title_schema, this, schema),
       false,
@@ -2084,9 +2286,6 @@ void SqlEditorForm::active_schema(const std::string &value)
 {
   try
   {
-    if (value.empty())
-      return;
-
     if (value == active_schema())
       return;
 
@@ -2095,13 +2294,15 @@ void SqlEditorForm::active_schema(const std::string &value)
   
     {
       GMutexLock aux_dbc_conn_mutex= ensure_valid_aux_connection();
-      _aux_dbc_conn->ref->setSchema(value);
+      if (!value.empty())
+        _aux_dbc_conn->ref->setSchema(value);
       _aux_dbc_conn->active_schema= value;
     }
 
     {
       GMutexLock usr_dbc_conn_mutex= ensure_valid_usr_connection();
-      _usr_dbc_conn->ref->setSchema(value);
+      if (!value.empty())
+        _usr_dbc_conn->ref->setSchema(value);
       _usr_dbc_conn->active_schema= value;
     }
 
@@ -2117,7 +2318,10 @@ void SqlEditorForm::active_schema(const std::string &value)
 
     update_editor_title_schema(value);
     
-    grt_manager()->replace_status_text(strfmt(_("Active schema changed to %s"), value.c_str()));
+    if (value.empty())
+      grt_manager()->replace_status_text(_("Active schema was cleared"));
+    else
+      grt_manager()->replace_status_text(strfmt(_("Active schema changed to %s"), value.c_str()));
     
     _grtm->get_grt()->call_module_function("Workbench", "saveConnections", grt::BaseListRef());
   }
@@ -2157,37 +2361,73 @@ int SqlEditorForm::count_connection_editors(const std::string &conn_name)
   return count;  
 }
 
- 
+//--------------------------------------------------------------------------------------------------
+
 std::string SqlEditorForm::update_title()
 {
-  std::string caption= _("SQL Editor");
+  std::string caption;
   std::string editor_connection = get_session_name();
   
   if (!editor_connection.empty())
-    caption+= strfmt(" (%s", editor_connection.c_str());
+    caption+= strfmt("%s", editor_connection.c_str());
   else
   {
     if (_connection->driver()->name() == "MysqlNativeSocket")
-      caption+= " (localhost";
+      caption+= "localhost";
     else
-      caption+= strfmt(" (%s", truncate_text(editor_connection,21).c_str());
+      caption+= strfmt("%s", truncate_text(editor_connection,21).c_str());
   }
 
-  // This function is called in three different scenarios, in all of them the active 
-  // schema name should be included on the editor title if there are 2 editor to 
-  // the same connection
-  //             
-  // - New Editor : this is a new editor and need to check if active schema is needed
-  // - New Editor : other is the new editor, so need to add the active schema to my title
-  // - Activating a new schema : to reflect the new schema on the title
-  if(count_connection_editors(editor_connection) > 1 && !_usr_dbc_conn->active_schema.empty())
-    caption+= strfmt(" - %s", truncate_text(_usr_dbc_conn->active_schema, 20).c_str());
-
-  caption+= ")";
+  // only show schema name if there's more than 1 tab to the same connection, to save space
+  if (!_usr_dbc_conn->active_schema.empty() && count_connection_editors(editor_connection) > 1)
+    caption+= strfmt(" (%s)", truncate_text(_usr_dbc_conn->active_schema, 20).c_str());
 
   return caption;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns the current server version (or a reasonable default if not connected) in compact form
+ * as needed for parsing on various occasions (context help, auto completion, error parsing).
+ */
+int SqlEditorForm::server_version()
+{
+  GrtVersionRef version = rdbms()->version();
+
+  // Create a server version of the form "Mmmrr" as long int for quick comparisons.
+  if (version.is_valid())
+    return version->majorNumber() * 10000 + version->minorNumber() * 100 + version->releaseNumber();
+  else
+    return 50503;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns a list of valid charsets for this connection as needed for parsing.
+ */
+std::set<std::string> SqlEditorForm::valid_charsets()
+{
+  if (_charsets.empty())
+  {
+    grt::ListRef<db_CharacterSet> list = rdbms()->characterSets();
+    for (size_t i = 0; i < list->count(); i++)
+      _charsets.insert(base::tolower(*list[i]->name()));
+
+    // 3 character sets were added in version 5.5.3. Remove them from the list if the current version
+    // is lower than that.
+    if (server_version() < 50503)
+    {
+      _charsets.erase("utf8mb4");
+      _charsets.erase("utf16");
+      _charsets.erase("utf32");
+    }
+  }
+  return _charsets;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool SqlEditorForm::save_snippet(const std::string &text)
 {
@@ -2217,6 +2457,7 @@ bool SqlEditorForm::can_close_(bool interactive)
     return false;
 
   _live_tree->prepare_close();
+  _grtm->set_app_option("DbSqlEditor:ActiveSidePaletteTab", grt::IntegerRef(_side_palette->get_active_tab()));
   
   bool check_scratch_editors = true;
   bool save_workspace_on_close = false;
@@ -2245,7 +2486,7 @@ bool SqlEditorForm::can_close_(bool interactive)
 
         std::string n= sql_editor_path(i);
         if (!n.empty())
-          n= std::string(g_basename(n.c_str())).append(" - ").append(n);
+          n= base::basename(n).append(" - ").append(n);
         else
           n= "Unsaved SQL Query";
         dialog.add_item("Script Buffers", n);
@@ -2284,7 +2525,7 @@ bool SqlEditorForm::can_close_(bool interactive)
     {
       for (int i = 0; i < sql_editor_count(); i++)
       {
-        // sql_editor_wilL_close() checks for unsaved recordsets
+        // sql_editor_will_close() checks for unsaved recordsets
         if (!sql_editor_will_close(i))
           return false;
       }
@@ -2316,65 +2557,41 @@ bool SqlEditorForm::can_close_(bool interactive)
 }
 
 
-
-bec::MenuItemList SqlEditorForm::recordset_popup_menu_items(const std::vector<int> &rows, int column, Recordset::Ptr rs_ptr)
+void SqlEditorForm::check_external_file_changes()
 {
-  RETVAL_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, rs_ptr, rs, bec::MenuItemList())
-  if (rs)
+  for (int i = 0; i < sql_editor_count(); i++)
   {
-    bec::ArgumentPool argpool;
-    
-    if (!rows.empty())
+    if (!_sql_editors[i]->filename.empty())
     {
-      grt::IntegerListRef row_list(grt_manager()->get_grt());
-      for (std::vector<int>::const_iterator iter= rows.begin(); iter != rows.end(); ++iter)
-        row_list.insert(grt::IntegerRef(*iter));
-      argpool.add_simple_value("selectedRows", row_list);
-      argpool.add_simple_value("clickedRow", grt::IntegerRef(rows[0]));
+      time_t ts;
+      if (base::file_mtime(_sql_editors[i]->filename, ts))
+      {
+        if (ts > _sql_editors[i]->file_timestamp)
+        {
+          // File was changed externally. For now we ignore local changes if the user chooses to reload.
+          std::string connection_description = connection_descriptor().is_valid() ?
+            base::strfmt("(from connection to %s) ", connection_descriptor()->name().c_str()) : "";
+          if (mforms::Utilities::show_warning("File Changed",
+            base::strfmt(_("File %s %swas changed from outside MySQL Workbench.\nWould you like to discard your changes and reload it?"),
+              _sql_editors[i]->filename.c_str(), connection_description.c_str()),
+              "Reload File", "Ignore", "") == mforms::ResultOk
+            )
+          {
+            _sql_editors[i]->editor->sql("");
+            sql_editor_open_file(i, _sql_editors[i]->filename, _sql_editors[i]->orig_encoding);
+          }
+          else
+          {
+            _sql_editors[i]->editor->get_editor_control()->set_text(_sql_editors[i]->editor->get_editor_control()->get_text(false).data());
+            _sql_editors[i]->file_timestamp = ts;
+          }
+        }
+      }
     }
-    argpool.add_simple_value("clickedColumn", grt::IntegerRef(column));
-
-    _grtm->update_plugin_arguments_pool(argpool);
-    _wbsql->update_plugin_arguments_pool(argpool);
-
-    std::list<std::string> groups;
-    groups.push_back("Menu/SQL/Resultset");
-
-    return grt_manager()->get_plugin_context_menu_items(groups, argpool);    
   }
-  
-  return bec::MenuItemList();
 }
 
 
-bool SqlEditorForm::call_recordset_popup_menu_item(const std::string &item, const std::vector<int> &rows,
-                                                   int column, Recordset::Ptr rs_ptr)
-{
-  RETVAL_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, rs_ptr, rs, false)
-  
-  if (rs)
-  {
-    bec::ArgumentPool argpool;
-    
-    if (!rows.empty())
-    {
-      grt::IntegerListRef row_list(grt_manager()->get_grt());
-      for (std::vector<int>::const_iterator iter= rows.begin(); iter != rows.end(); ++iter)
-        row_list.insert(grt::IntegerRef(*iter));
-      argpool["app.PluginInputDefinition:selectedRows"]= row_list;
-      argpool["app.PluginInputDefinition:clickedRow"]= grt::IntegerRef(rows[0]);
-    }
-    argpool["app.PluginInputDefinition:clickedColumn"]= grt::IntegerRef(column);
-    
-    _grtm->update_plugin_arguments_pool(argpool);
-    _wbsql->update_plugin_arguments_pool(argpool);
-    
-    return _wbsql->get_cmdui()->activate_command(item, argpool);
-  }
-  
-  return false;
-}
-
 void SqlEditorForm::update_editor_title_schema(const std::string& schema)
 {
   _live_tree->on_active_schema_change(schema);
