@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,8 +19,6 @@
 #ifndef _WB_SQL_EDITOR_FORM_H_
 #define _WB_SQL_EDITOR_FORM_H_
 
-// renamed from db_sql_editor_be.h
-
 #include "workbench/wb_backend_public_interface.h"
 #include "sqlide/recordset_be.h"
 #include "sqlide/sql_editor_be.h"
@@ -31,6 +29,7 @@
 #include "grts/structs.db.mgmt.h"
 #include "base/file_utilities.h"
 #include "base/ui_form.h"
+#include "grtpp_notifications.h"
 #include "grt/refresh_ui.h"
 #include "sqlide/wb_context_sqlide.h"
 #include <boost/enable_shared_from_this.hpp>
@@ -39,6 +38,7 @@
 namespace mforms {
   class ToolBar;
   class View;
+  class MenuItem;
 };
 
 namespace bec
@@ -47,6 +47,7 @@ namespace bec
 }
 
 
+#define MAIN_DOCKING_POINT "db.query.Editor:main"
 #define RESULT_DOCKING_POINT "db.Query.QueryEditor:result"
 
 class QuerySidePalette;
@@ -56,13 +57,12 @@ class AutoCompleteCache;
 typedef std::vector<Recordset::Ref> Recordsets;
 typedef boost::shared_ptr<std::vector<Recordset::Ref> > RecordsetsRef;
 
-#define Db_sql_editor SqlEditorForm
-class MYSQLWBBACKEND_PUBLIC_FUNC SqlEditorForm : public bec::UIForm, public bec::RefreshUI, base::Observer,
+class MYSQLWBBACKEND_PUBLIC_FUNC SqlEditorForm : public bec::UIForm, public bec::RefreshUI, grt::GRTObserver,
                                                 public boost::enable_shared_from_this<SqlEditorForm>
 {
 public:
 #if defined(ENABLE_TESTING)
-  friend class Db_sql_editor_tester;
+  friend class EditorFormTester;
 #endif
 
   enum
@@ -81,10 +81,12 @@ public:
   
 protected:
   SqlEditorForm(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
+
+  void update_menu_and_toolbar();
+  void update_toolbar_icons();
 public:
   virtual ~SqlEditorForm();
 
-public:
   virtual void close();
   virtual bool is_main_form() { return true; }
   virtual std::string get_form_context_name() const;
@@ -102,7 +104,6 @@ public:
   bec::GRTManager * grt_manager() const { return _grtm; }
   wb::WBContextSQLIDE *wbsql() const { return _wbsql; }
 
-  void update_menu_and_toolbar();
   void validate_menubar();
 private:
   wb::WBContextSQLIDE *_wbsql;
@@ -153,12 +154,13 @@ private:
     RecordsetsRef recordsets;
     Recordset::Ref active_recordset;
     GMutex *recordset_mutex;
+    time_t file_timestamp;
     int rs_sequence;
     bool is_scratch;
     bool start_collapsed;
     bool busy;
     
-    Sql_editor_info() : rs_sequence(0), is_scratch(false), start_collapsed(false), busy(false) {}
+    Sql_editor_info() : rs_sequence(0), is_scratch(false), start_collapsed(false), busy(false) { memset(&file_timestamp, 0, sizeof(file_timestamp)); }
   };
   typedef std::vector<Sql_editor_info::Ref> Sql_editors;
   Sql_editors _sql_editors;
@@ -166,7 +168,7 @@ private:
   int _scratch_editors_serial;
   GMutex *_sql_editors_mutex;
   
-  boost::shared_ptr<mforms::ToolBar> setup_editor_toolbar();
+  boost::shared_ptr<mforms::ToolBar> setup_editor_toolbar(Sql_editor::Ref editor);
   void set_editor_tool_items_enbled(const std::string &name, bool flag);
   void set_editor_tool_items_checked(const std::string &name, bool flag);
 public:
@@ -194,14 +196,19 @@ public:
   std::string update_title();
 
   std::map<std::string, std::string> connection_details() { return _connection_details; }
+  int server_version();
+  std::set<std::string> valid_charsets();
 private:
   std::map<std::string, std::string> _connection_details;
+  std::set<std::string> _charsets;
 
   grt::StringRef do_connect(grt::GRT *grt, boost::shared_ptr<sql::TunnelConnection> tunnel, sql::Authentication::Ref &auth,
-    sql::AuthenticationError *&autherr_ptr);
+    struct ConnectionErrorInfo *autherr_ptr);
   grt::StringRef do_disconnect(grt::GRT *grt);
+
+  void update_connected_state();
 public:
-  void connect();
+  bool connect(boost::shared_ptr<sql::TunnelConnection> tunnel);
   bool connected() const;
   void finish_startup();
   void title_changed();
@@ -216,6 +223,11 @@ public:
   void run_editor_contents(bool current_statement_only);
 
   void list_members();
+
+  void limit_rows(mforms::MenuItem *menu, const char *limit);
+
+  std::string sql_mode() const { return _sql_mode; };
+  int lower_case_table_names() const { return _lower_case_table_names; }
 private:
   void do_commit();
 public:  
@@ -227,6 +239,8 @@ private:
   void cache_sql_mode();
 private:
   std::string _sql_mode;
+  int _lower_case_table_names;
+  int _limit_rows;
 
 private:
   void create_connection(sql::Dbc_connection_handler::Ref &dbc_conn, db_mgmt_ConnectionRef db_mgmt_conn, boost::shared_ptr<sql::TunnelConnection> tunnel, sql::Authentication::Ref auth, bool autocommit_mode);
@@ -266,7 +280,10 @@ public:
   void exec_sql_retaining_editor_contents(const std::string &sql_script, Sql_editor::Ref editor, bool sync, bool dont_add_limit_clause= false);
 
   RecordsetsRef exec_sql_returning_results(const std::string &sql_script, bool dont_add_limit_clause);
-  
+
+  void exec_management_sql(const std::string &sql, bool log);
+  db_query_ResultsetRef exec_management_query(const std::string &sql, bool log);
+
   void explain_sql();
   void explain_current_statement();
   bool is_running_query();
@@ -280,6 +297,8 @@ private:
   grt::StringRef do_exec_sql(grt::GRT *grt, Ptr self_ptr, boost::shared_ptr<std::string> sql,
     Sql_editor::Ref editor, ExecFlags flags, RecordsetsRef result_list);
   void do_explain_sql(const std::string &sql);
+
+  void handle_command_side_effects(const std::string &sql);
 public:
   GrtThreadedTask::Ref exec_sql_task;
 private:
@@ -328,12 +347,16 @@ private:
   void apply_data_changes_commit(std::string &sql_script_text, Recordset::Ptr rs_ptr);
   void recall_recordset_query(Recordset::Ptr rs_ptr);
   void update_editor_title_schema(const std::string& schema);
+
+  void on_recordset_context_menu_show(Recordset::Ptr rs_ptr, Sql_editor::Ptr editor_ptr);
 public:
   void show_export_recordset(int editor_index, Recordset::Ptr rs_ptr);
+  void show_import_recordset(int editor_index, Recordset::Ptr rs_ptr);
   
   bool can_close();
   bool can_close_(bool interactive);
 
+  void check_external_file_changes();
 public:
   void new_sql_script_file();
   void new_sql_scratch_area(bool start_collapsed = false);
@@ -394,12 +417,9 @@ protected:
   std::string _title;
 
 private:
-  bec::MenuItemList recordset_popup_menu_items(const std::vector<int> &rows, int column, Recordset::Ptr rs_ptr);
-  bool call_recordset_popup_menu_item(const std::string &item, const std::vector<int> &rows,
-                                      int column, Recordset::Ptr rs_ptr);
-
+  virtual void handle_grt_notification(const std::string &name, grt::ObjectRef sender, grt::DictRef info);
   virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
-  void setup_sidebars();
+  void setup_side_palette();
   
   void schema_row_selected();
   void side_bar_filter_changed(const std::string& filter);
@@ -409,6 +429,8 @@ public:
 
   bool save_snippet(const std::string &text);
 
+  void show_output_area();
+
   mforms::View *get_sidebar();
   mforms::View *get_side_palette();
 };
