@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,14 +21,17 @@
 #include "wb_sql_editor_form.h"
 #include "wb_sql_editor_form_ui.h"
 #include "query_side_palette.h"
-#include "wb_context_names.h"
+#include "workbench/wb_context_names.h"
 #include "objimpl/db.query/db_query_Resultset.h"
+#include "objimpl/ui/mforms_ObjectReference_impl.h"
 #include "grtui/inserts_export_form.h"
+#include "grtdb/editor_dbobject.h"
 
 #include "sqlide/wb_sql_editor_tree_controller.h"
 #include "sqlide/sql_script_run_wizard.h"
 
 #include "workbench/wb_command_ui.h"
+#include "workbench/wb_context_ui.h"
 #include "base/boost_smart_ptr_helpers.h"
 
 #include <boost/signals2/connection.hpp>
@@ -58,18 +61,74 @@ mforms::ToolBar *SqlEditorForm::get_toolbar()
     _toolbar = _wbsql->get_cmdui()->create_toolbar("data/dbquery_toolbar.xml", boost::bind(&SqlEditorForm::activate_command, this, _1));
     
     update_menu_and_toolbar();
+    update_toolbar_icons();
   }
   return _toolbar;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void SqlEditorForm::limit_rows(mforms::MenuItem *menu, const char *limit)
+{
+  int c = menu->item_count();
+  bool found = false;
+  for (int i = 0; i < c; i++)
+  {
+    mforms::MenuItem *item = menu->get_item(i);
+    if (item->get_type() != mforms::SeparatorMenuItem)
+    {
+      if (item->get_name().compare(limit) == 0)
+      {
+        item->set_checked(true);
+        found = true;
+      }
+      else
+        item->set_checked(false);
+    }
+  }
+
+  _limit_rows = atoi(limit);
+
+  mforms::MenuItem *citem = menu->find_item("custom");
+  if (!found)
+  {
+    if (!citem)
+      citem = menu->add_item_with_title(base::strfmt("Custom (%s)", limit), boost::bind(&SqlEditorForm::limit_rows, this, menu, limit), "custom");
+    else
+      citem->set_title(base::strfmt("Custom (%s)", limit));
+    citem->set_checked(true);
+  }
+  else
+  {
+    if (citem)
+      menu->remove_item(citem);
+  }
+}
+
+
 mforms::MenuBar *SqlEditorForm::get_menubar()
 {
   if (!_menu)
   {
     _menu = _wbsql->get_cmdui()->create_menubar_for_context(WB_CONTEXT_QUERY);
-    
+
+    mforms::MenuItem *limit_item = _menu->find_item("limit_rows");
+    if (limit_item)
+    {
+      static const char*items[] = {
+        "10", "50", "100", "200", "300", "400", "500", "1000", "2000", "5000", "10000", "50000", NULL
+      };
+
+      limit_item->add_item_with_title("Don't Limit", boost::bind(&SqlEditorForm::limit_rows, this, limit_item, "0"), "0");
+      limit_item->add_separator();
+      for (int i = 0; items[i]; i++)
+        limit_item->add_item_with_title(items[i], boost::bind(&SqlEditorForm::limit_rows, this, limit_item, items[i]), items[i]);
+
+      if (_limit_rows <= 0)
+        limit_rows(limit_item, "0");
+      else
+        limit_rows(limit_item, base::strfmt("%i", _limit_rows).c_str());
+    }
     update_menu_and_toolbar();
     
     _menu->set_item_enabled("query.save_edits", false);
@@ -92,89 +151,133 @@ void SqlEditorForm::update_menu_and_toolbar()
   }
   
   bool running = is_running_query();
-  
+  bool connected = this->connected();
+
   if (_menu)
   {
-    _menu->set_item_enabled("query.cancel", running);
-    _menu->set_item_enabled("query.execute", !running);
+    _menu->set_item_enabled("query.cancel", running && connected);
+    _menu->set_item_enabled("query.execute", !running && connected);
     _menu->set_item_enabled("query.reconnect", !running);
-    _menu->set_item_enabled("wb.sqlide.executeToTextOutput", !running);
-    _menu->set_item_enabled("query.execute_current_statement", !running);
-    _menu->set_item_enabled("query.explain", !running);
-    _menu->set_item_enabled("query.explain_current_statement", !running);
+    _menu->set_item_enabled("wb.sqlide.executeToTextOutput", !running && connected);
+    _menu->set_item_enabled("query.execute_current_statement", !running && connected);
+    _menu->set_item_enabled("query.explain", !running && connected);
+    _menu->set_item_enabled("query.explain_current_statement", !running && connected);
     
-    _menu->set_item_enabled("query.commit", !running && !auto_commit());
-    _menu->set_item_enabled("query.rollback", !running && !auto_commit());    
+    _menu->set_item_enabled("query.commit", !running && !auto_commit() && connected);
+    _menu->set_item_enabled("query.rollback", !running && !auto_commit() && connected);
+    _menu->set_item_enabled("query.stopOnError", connected);
     mforms::MenuItem *item = _menu->find_item("query.autocommit");
     if (item)
     {
-      item->set_enabled(!running);
+      item->set_enabled(!running && connected);
       item->set_checked(auto_commit());    
     }
   }
-  
+
   if (_toolbar)
   {
-    /* deprecated
-     _toolbar->set_item_enabled("query.cancel", running);
-     _toolbar->set_item_enabled("query.execute", !running);
-     _toolbar->set_item_enabled("query.reconnect", !running);
-     _toolbar->set_item_enabled("wb.sqlide.executeToTextOutput", !running);
-     _toolbar->set_item_enabled("query.execute_current_statement", !running);
-     _toolbar->set_item_enabled("query.explain", !running);
-     _toolbar->set_item_enabled("query.explain_current_statement", !running);
-     
-     _toolbar->set_item_enabled("query.commit", !running && !auto_commit());
-     _toolbar->set_item_enabled("query.rollback", !running && !auto_commit());
-     mforms::ToolBarItem *item = _toolbar->find_item("query.autocommit");
-     if (item)
-     {
-     item->set_enabled(!running);
-     item->set_checked(auto_commit());
-     }
-     }*/
+    _toolbar->set_item_enabled("query.new_schema", connected);
+    _toolbar->set_item_enabled("query.new_table", connected);
+    _toolbar->set_item_enabled("query.new_view", connected);
+    _toolbar->set_item_enabled("query.new_routine", connected);
+    _toolbar->set_item_enabled("query.new_function", connected);
+    _toolbar->set_item_enabled("wb.dbsearch", connected);
   }
   
-  /*
-   if (running)
-   {
-   set_editor_tool_items_enbled("query.cancel", false);
-   for (Sql_editors::iterator iter = _sql_editors.begin(); iter != _sql_editors.end(); ++iter)
-   if (iter->busy)
-   iter->toolbar->set_item_checked("query.cancel", true);
-   }
-   else*/
-  set_editor_tool_items_enbled("query.cancel", running);
+  set_editor_tool_items_enbled("query.cancel", running && connected);
   
-  set_editor_tool_items_enbled("query.execute", !running);
+  set_editor_tool_items_enbled("query.execute", !running && connected);
   set_editor_tool_items_enbled("query.reconnect", !running);
-  set_editor_tool_items_enbled("wb.sqlide.executeToTextOutput", !running);
-  set_editor_tool_items_enbled("query.execute_current_statement", !running);
-  set_editor_tool_items_enbled("query.explain", !running);
-  set_editor_tool_items_enbled("query.explain_current_statement", !running);
+  set_editor_tool_items_enbled("wb.sqlide.executeToTextOutput", !running && connected);
+  set_editor_tool_items_enbled("query.execute_current_statement", !running && connected);
+  set_editor_tool_items_enbled("query.explain", !running && connected);
+  set_editor_tool_items_enbled("query.explain_current_statement", !running && connected);
   
-  set_editor_tool_items_enbled("query.commit", !running && !auto_commit());
-  set_editor_tool_items_enbled("query.rollback", !running && !auto_commit());    
-  set_editor_tool_items_enbled("query.autocommit", !running);
+  set_editor_tool_items_enbled("query.commit", !running && !auto_commit() && connected);
+  set_editor_tool_items_enbled("query.rollback", !running && !auto_commit() && connected);
+  set_editor_tool_items_enbled("query.autocommit", !running && connected);
+  set_editor_tool_items_enbled("query.stopOnError", connected);
   set_editor_tool_items_checked("query.autocommit", auto_commit());
   set_editor_tool_items_checked("query.stopOnError", !_continue_on_error);
 }
 
+//--------------------------------------------------------------------------------------------------
+
+std::string find_icon_name(std::string icon_name, bool use_win8)
+{
+  std::string::size_type dot_position = icon_name.rfind(".");
+  if (dot_position != std::string::npos)
+  {
+    std::string extension = icon_name.substr(dot_position);
+    std::string name = icon_name.substr(0, dot_position);
+    bool using_win8 = name.rfind("_win8") == name.size() - 5;
+    if (use_win8 != using_win8)
+    {
+      if (use_win8)
+        icon_name = name + "_win8" + extension;
+      else
+        icon_name = name.substr(0, name.size() - 5) + extension;
+    }
+  }
+
+  return icon_name;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void SqlEditorForm::update_toolbar_icons()
+{
+  bool use_win8;
+  
+  switch (base::Color::get_active_scheme())
+  {
+  case base::ColorSchemeStandardWin8:
+  case base::ColorSchemeStandardWin8Alternate:
+    use_win8 = true;
+    break;
+
+  default:
+    use_win8 = false;
+  }
+
+  mforms::ToolBarItem *item = _toolbar->find_item("wb.toggleSidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+    
+  item = _toolbar->find_item("wb.toggleOutputArea");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+    
+  item = _toolbar->find_item("wb.toggleSecondarySidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void SqlEditorForm::validate_menubar()
 {
   if (get_menubar())
-    get_menubar()->validate();
+    _wbsql->get_wbui()->get_command_ui()->revalidate_menu_bar(get_menubar());
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Setup of schema browser/object info/quick help sidebars (puzzling: sometimes called side palette).
+ * Setup of schema browser/object info/quick help side palette.
  */
-void SqlEditorForm::setup_sidebars()
+void SqlEditorForm::setup_side_palette()
 {  
-  // Right hand side bar (quick help, variables).
+  // Right hand side (quick help, snippets).
   _side_palette = mforms::manage(new QuerySidePalette(shared_from_this()));
   
 #ifdef _WIN32
@@ -182,11 +285,12 @@ void SqlEditorForm::setup_sidebars()
   panel = mforms::manage(new mforms::Panel(mforms::StyledHeaderPanel));
   panel->set_title(_("SQL Additions"));
   panel->add(_side_palette);
-  _side_palette_host = panel; // Once we have the variables view this becomes a splitter too.
-
+  _side_palette_host = panel;
 #else
   _side_palette_host = _side_palette;
 #endif
+
+  _side_palette->set_active_tab(_grtm->get_app_option_int("DbSqlEditor:ActiveSidePaletteTab", 0));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -200,12 +304,23 @@ void SqlEditorForm::toolbar_command(const std::string& command)
     _live_tree->tree_create_object(LiveSchemaTree::Table, "", "");
   else if (command == "query.new_view")
     _live_tree->tree_create_object(LiveSchemaTree::View, "", "");
-
-  // TODO: Update this to new procedure and new function once the tool bar is updated
   else if (command == "query.new_routine")
     _live_tree->tree_create_object(LiveSchemaTree::Procedure, "", "");
+  else if (command == "query.new_function")
+    _live_tree->tree_create_object(LiveSchemaTree::Function, "", "");
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void SqlEditorForm::show_output_area()
+{
+  mforms::ToolBarItem *item = _toolbar->find_item("wb.toggleOutputArea");
+  if (item && !item->get_checked())
+  {
+    item->set_checked(true);
+    item->callback();
+  }
+}
 
 //--------------------------------------------------------------------------------------------------
 
@@ -232,32 +347,12 @@ static void toggle_continue_on_error(SqlEditorForm *sql_editor_form)
 
 //--------------------------------------------------------------------------------------------------
 
-static void toggle_show_special_chars(mforms::ToolBarItem *item, SqlEditorForm *sql_editor_form)
-{
-  sql_editor_form->active_sql_editor()->show_special_chars(item->get_checked());
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static void toggle_word_wrap(mforms::ToolBarItem *item, SqlEditorForm *sql_editor_form)
-{
-  sql_editor_form->active_sql_editor()->enable_word_wrap(item->get_checked());
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static void show_find_panel_for_active_editor(SqlEditorForm *sql_editor_form)
-{
-  Sql_editor::Ref editor(sql_editor_form->active_sql_editor());
-  if (editor)
-    editor->get_editor_control()->show_find_panel(false);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar()
+boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar(Sql_editor::Ref editor)
 {
   boost::shared_ptr<mforms::ToolBar> tbar(new mforms::ToolBar(mforms::SecondaryToolBar));
+#ifdef _WIN32
+  tbar->set_size(-1, 27);
+#endif
   mforms::ToolBarItem *item;
   
   item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
@@ -352,48 +447,9 @@ boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar()
   tbar->add_item(item);  
   
   tbar->add_item(mforms::manage(new mforms::ToolBarItem(mforms::SeparatorItem)));
-  
-  item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
-  item->set_name("query.beautify");
-  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_beautifier.png"));
-  item->set_tooltip(_("Beautify/reformat the SQL script"));
-  _wbsql->get_cmdui()->scoped_connect(item->signal_activated(),boost::bind((void (wb::CommandUI::*)(const std::string&))&wb::CommandUI::activate_command, 
-                                                                           _wbsql->get_cmdui(), "plugin:wb.sqlide.enbeautificate"));
-  tbar->add_item(item);  
-  
-  /*
-   item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
-   item->set_name("query.beautify_current_statement");
-   item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_beautifier_current.png"));
-   item->set_tooltip(_("Beautify/reformat the current SQL statement"));
-   item->signal_activated().connect(sigc::bind(sigc::mem_fun(this, &bec::RefreshUI::do_partial_ui_refresh), (int)ShowFindPanel));
-   tbar->add_item(item);  
-   */
-  
-  tbar->add_item(mforms::manage(new mforms::ToolBarItem(mforms::SeparatorItem)));
-  
-  item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
-  item->set_name("query.search");
-  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_find.png"));
-  item->set_tooltip(_("Show the Find panel for the editor"));
-  scoped_connect(item->signal_activated(), boost::bind(show_find_panel_for_active_editor, this));
-  tbar->add_item(item);  
-  
-  item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
-  item->set_name("query.toggleInvisible");
-  item->set_alt_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_special-chars-on.png"));
-  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_special-chars-off.png"));
-  item->set_tooltip(_("Toggle display of invisible characters (spaces, tabs, newlines)"));
-  scoped_connect(item->signal_activated(),boost::bind(toggle_show_special_chars, item, this));
-  tbar->add_item(item);
-  
-  item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
-  item->set_name("query.toggleWordWrap");
-  item->set_alt_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_word-wrap-on.png"));
-  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_word-wrap-off.png"));
-  item->set_tooltip(_("Toggle wrapping of long lines (keep this off for large files)"));
-  scoped_connect(item->signal_activated(),boost::bind(toggle_word_wrap, item, this));
-  tbar->add_item(item);
+
+  // adds generic SQL editor toolbar buttons
+  editor->set_base_toolbar(tbar.get());
 
   return tbar;
 }
@@ -467,7 +523,7 @@ void SqlEditorForm::show_import_recordset(int editor_index, Recordset::Ptr rs_pt
       db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(_sql_editors[editor_index]->editor->grtobj()));
       for (size_t c = qeditor->resultsets().count(), i = 0; i < c; i++)
       {
-        if (qeditor->resultsets()[i]->get_data()->recordset.get() == rs)
+        if (dynamic_cast<WBRecordsetResultset*>(qeditor->resultsets()[i]->get_data())->recordset.get() == rs)
         {
           rset = qeditor->resultsets()[i];
           break;
@@ -497,7 +553,16 @@ bool SqlEditorForm::run_live_object_alteration_wizard(const std::string &alter_s
   on_sql_script_run_progress.disconnect_all_slots();
   on_sql_script_run_statistics.disconnect_all_slots();
   
-  SqlScriptRunWizard wizard(_grtm);
+  // Determine the current online DDL settings, so the wizard can initialize its local settings.
+  std::string algorithm;
+  wbsql()->get_wbui()->get_wb_options_value("", "DbSqlEditor:OnlineDDLAlgorithm", algorithm);
+  std::string lock;
+  wbsql()->get_wbui()->get_wb_options_value("", "DbSqlEditor:OnlineDDLLock", lock);
+  
+  SqlScriptRunWizard wizard(_grtm, rdbms()->version(), algorithm, lock);
+  if (obj_editor)
+    wizard.regenerate_script = boost::bind(&SqlEditorTreeController::generate_alter_script, get_live_tree(),
+                                           obj_editor->get_dbobject(), _1, _2);
   scoped_connection c1(on_sql_script_run_error.connect(boost::bind(&SqlScriptApplyPage::on_error, wizard.apply_page, _1, _2, _3)));
   scoped_connection c2(on_sql_script_run_progress.connect(boost::bind(&SqlScriptApplyPage::on_exec_progress, wizard.apply_page, _1)));
   scoped_connection c3(on_sql_script_run_statistics.connect(boost::bind(&SqlScriptApplyPage::on_exec_stat, wizard.apply_page, _1, _2)));
@@ -543,3 +608,36 @@ int SqlEditorForm::sql_script_stats(long, long)
   return 0;
 }
 
+
+void SqlEditorForm::on_recordset_context_menu_show(Recordset::Ptr rs_ptr, Sql_editor::Ptr editor_ptr)
+{
+  Recordset::Ref rs(rs_ptr.lock());
+  if (rs)
+  {
+    grt::DictRef info(_grtm->get_grt());
+
+    std::vector<int> selection(rs->selected_rows());
+    grt::IntegerListRef rows(_grtm->get_grt());
+    for (std::vector<int>::const_iterator i = selection.begin(); i != selection.end(); ++i)
+      rows.insert(*i);
+
+    info.set("selected-rows", rows);
+    info.gset("selected-column", rs->selected_column());
+    info.set("menu", mforms_to_grt(info.get_grt(), rs->get_context_menu()));
+
+    db_query_QueryBufferRef qbuffer(editor_ptr.lock()->grtobj());
+    if (qbuffer.is_valid() && db_query_QueryEditorRef::can_wrap(qbuffer))
+    {
+      db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(qbuffer));
+      for (size_t c = qeditor->resultsets().count(), i = 0; i < c; i++)
+      {
+        if (dynamic_cast<WBRecordsetResultset*>(qeditor->resultsets()[i]->get_data())->recordset == rs)
+        {
+          grt::GRTNotificationCenter::get()->send_grt("GRNSQLResultsetMenuWillShow", qeditor->resultsets()[i], info);
+          break;
+        }
+      }
+    }
+  }
+}
+
