@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,8 +20,17 @@
 #include "stdafx.h"
 
 #include <string>
-#include "wb_context_ui.h"
+
+#include "base/file_utilities.h"
+#include "base/string_utilities.h"
+#include "base/trackable.h"
+#include "base/notifications.h"
+#include "base/log.h"
+
+#include "grts/structs.db.mgmt.h"
 #include "wb_command_ui.h"
+#include "wb_context_ui.h"
+
 #include "wb_module.h"
 #include "grt/common.h"
 #include "grt/clipboard.h"
@@ -30,18 +39,18 @@
 #include "wb_overview.h"
 
 #include "model/wb_component.h"
-#include "base/string_utilities.h"
 
 #include <mforms/toolbar.h>
 #include <mforms/menubar.h>
 #include <boost/lambda/bind.hpp>
 
-#include "base/log.h"
-
 using namespace bec;
 using namespace wb;
 using namespace base;
 
+using std::max;
+using std::min;
+
 DEFAULT_LOG_DOMAIN(DOMAIN_COMMAND_HANDLING)
 
 struct wb::ParsedCommand
@@ -83,14 +92,16 @@ struct wb::ParsedCommand
 
 
 CommandUI::CommandUI(WBContext *wb)
-  : _wb(wb)
+  : _wb(wb), _include_se(false)
 {
 }
 
 void CommandUI::load_data()
 {
   grt::GRT *grt= _wb->get_grt();
-   
+
+  _include_se = _wb->is_commercial();
+
   _shortcuts= grt::ListRef<app_ShortcutItem>::cast_from(
                    grt->unserialize(make_path(_wb->get_datadir(), "data/shortcuts.xml")));
 }
@@ -217,20 +228,32 @@ bool CommandUI::validate_command_item(const app_CommandItemRef &item, const wb::
       return _wb->get_grt_manager()->check_plugin_runnable(plugin, argpool);
     }
   }
+  else if (cmd.type == "call")
+  {
+    std::string module, function;
+    if (base::partition(cmd.name, ".", module, function))
+    {
+      grt::Module *m = _wb->get_grt()->get_module(module);
+      if (m && m->has_function(function))
+        return true;
+      log_info("Invalid function %s.%s\n", module.c_str(), function.c_str());
+      return false;
+    }
+  }
 
   return true;
 }
 
 
-bool CommandUI::validate_plugin_command(const std::string &name)
+bool CommandUI::validate_plugin_command(app_PluginRef plugin)
 {
-  app_PluginRef plugin(_wb->get_plugin_manager()->get_plugin(name));
+//  app_PluginRef plugin(_wb->get_plugin_manager()->get_plugin(name));
   if (plugin.is_valid())
   {
-    bec::ArgumentPool argpool;
-    _wb->update_plugin_arguments_pool(argpool);
-    argpool["app.PluginInputDefinition:string"]= grt::StringRef("");
-    if (!_wb->get_grt_manager()->check_plugin_runnable(plugin, argpool))
+    //bec::ArgumentPool argpool;
+    //_wb->update_plugin_arguments_pool(argpool);
+    //argpool["app.PluginInputDefinition:string"]= grt::StringRef("");
+    if (!_wb->get_grt_manager()->check_plugin_runnable(plugin, _argpool))
     {
       //g_message("plugin %s is not runnable", name.c_str());
       return false;
@@ -446,7 +469,7 @@ void CommandUI::add_recent_menu(mforms::MenuItem *parent)
   grt::StringListRef strlist(_wb->get_root()->options()->recentFiles());
   
   mforms::MenuItem *item;
-  for (size_t c= std::min(strlist.count(), (size_t)10), i= 0; i < c; i++)
+  for (size_t c= min(strlist.count(), (size_t)10), i= 0; i < c; i++)
   {
     std::string caption;
     if (i < 9)
@@ -487,7 +510,7 @@ void CommandUI::add_plugins_menu_items(mforms::MenuItem *parent, const std::stri
   {
     mforms::MenuItem *item = mforms::manage(new mforms::MenuItem((*iter)->caption()));
     item->set_name(std::string("plugin:").append(*(*iter)->name()));
-    item->set_validator(boost::bind(&CommandUI::validate_plugin_command, this, *(*iter)->name()));
+    item->set_validator(boost::bind(&CommandUI::validate_plugin_command, this, *iter));
     item->validate();
     scoped_connect(item->signal_clicked(),boost::bind((void(CommandUI::*)(const std::string&))&CommandUI::activate_command, this, item->get_name()));
     parent->add_item(item);
@@ -536,13 +559,34 @@ void CommandUI::add_plugins_menu(mforms::MenuItem *parent, const std::string &co
 }
 
 
+void CommandUI::add_scripts_menu(mforms::MenuItem *parent)
+{
+  try
+  {
+    std::list<std::string> pyfiles = base::scan_for_files_matching(bec::make_path(_wb->get_grt_manager()->get_user_script_path(), "*.py"));
+    std::list<std::string> luafiles = base::scan_for_files_matching(bec::make_path(_wb->get_grt_manager()->get_user_script_path(), "*.lua"));
+    std::vector<std::string> files;
+
+    std::copy(pyfiles.begin(), pyfiles.end(), std::back_inserter(files));
+    std::copy(luafiles.begin(), luafiles.end(), std::back_inserter(files));
+    std::sort(files.begin(), files.end());
+
+    for (std::vector<std::string>::const_iterator f = files.begin(); f != files.end(); ++f)
+      parent->add_item_with_title(base::basename(*f), boost::bind(&WBContext::run_script_file, _wb, *f));
+  }
+  catch (...)
+  {
+  }
+}
+
+
 void CommandUI::add_menu_items_for_context(const std::string &context, mforms::MenuItem *parent, const app_MenuItemRef &menu)
 {
   // special menu handling
   if (menu->name() == "open_recent")
     add_recent_menu(parent);
-  else if (menu->name() == "plugins") 
-    add_plugins_menu(parent, context);
+  else if (menu->name() == "run_script_list")
+    add_scripts_menu(parent);
   else
   {
     std::set<std::string> added_menu_items;
@@ -554,12 +598,7 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
     {
       app_MenuItemRef mitem(plist[i]);
       std::string item_context;
-      
-#ifdef EDITION_OSS
-      if (bec::has_suffix(mitem.id(), "/SE"))
-        continue;
-#endif
-      
+
       if (mitem->itemType() != "separator" && added_menu_items.find(mitem->name()) != added_menu_items.end())
         continue;
 
@@ -639,14 +678,41 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
           // skip if the plugin is invalid
           if (!plugin.is_valid())
           {
+            if (!bec::has_suffix(mitem.id(), "/SE") || _include_se)
+              log_warning("Plugin item %s was not found\n", cmd.name.c_str());
+            // if plugin is missing, check if it's supposed to be there
+            if (!_include_se || !bec::has_suffix(mitem.id(), "/SE"))
+              continue;
             enabled= false;
-            log_warning("Plugin item %s was not found\n", cmd.name.c_str());
 #ifdef ENABLE_DEBUG
             caption.append(" (invalid)");
 #endif
           }
           else
-            validator = boost::bind(&CommandUI::validate_plugin_command, this, cmd.name);
+            validator = boost::bind(&CommandUI::validate_plugin_command, this, plugin);
+        }
+        else if (cmd.type == "call")
+        {
+          std::string module, function;
+          if (base::partition(cmd.name, ".", module, function))
+          {
+            grt::Module *m = _wb->get_grt()->get_module(module);
+            if (m && m->has_function(function))
+              enabled = true;
+            else
+            {
+              if (!bec::has_suffix(mitem.id(), "/SE") || _include_se)
+                log_warning("Module function %s was not found\n", cmd.name.c_str());
+              // if plugin is missing, check if it's supposed to be there
+              if (!_include_se || !bec::has_suffix(mitem.id(), "/SE"))
+                continue;
+
+              enabled = false;
+#ifdef ENABLE_DEBUG
+              caption.append(" (invalid)");
+#endif
+            }
+          }
         }
         else if (cmd.type == "builtin")
         {
@@ -688,7 +754,11 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
       
       parent->add_item(item);
     }
-  }    
+  }
+
+  // Add plugin items after other items defined in the XML
+  if (menu->name() == "plugins")
+    add_plugins_menu(parent, context);
 }
 
 
@@ -709,6 +779,9 @@ void CommandUI::menu_will_show(mforms::MenuItem *item)
     item->remove_all();
     add_recent_menu(item);
   }
+  else
+    if (item->get_name() == "edit")
+      revalidate_edit_menu_items();
 }
 
 
@@ -730,12 +803,7 @@ mforms::MenuBar *CommandUI::create_menubar_for_context(const std::string &contex
     
     if (!match_context(item_context, context))
       continue;
-    
-#ifdef EDITION_OSS
-    if (bec::has_suffix(mitem.id(), "/SE"))
-      continue;
-#endif
-    
+
     std::string caption(mitem->caption());
 #ifdef _WIN32
     // turn mnemonic indicator from _ into & for windows
@@ -837,6 +905,21 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
       tbar->add_item(tbitem);
       continue;
     }
+    else if (titem->itemType() == "label")
+    {
+      tbitem = mforms::manage(new mforms::ToolBarItem(mforms::LabelItem));
+      tbitem->set_name(titem->name());
+      tbitem->set_text(titem->tooltip());
+      tbar->add_item(tbitem);
+      continue;
+    }
+    else if (titem->itemType() == "selector")
+    {
+      tbitem = mforms::manage(new mforms::ToolBarItem(mforms::SelectorItem));
+      tbitem->set_name(titem->name());
+
+      scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
+    }
     else if (titem->itemType() == "imagebox")
     {
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::ImageBoxItem));
@@ -849,7 +932,7 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::SearchFieldItem));
 
       scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
-    }    
+    }
     else if (titem->itemType() == "expander")
     {
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::ExpanderItem));
@@ -864,7 +947,7 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
     }
     else
     {
-      g_warning("invalid toolbar item type %s", titem->itemType().c_str());
+      log_warning("invalid toolbar item type %s", titem->itemType().c_str());
       continue;
     }
     /*
@@ -906,6 +989,9 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
           tbitem->set_tooltip(plugin->description());
       }
     }
+    else if (cmd.type == "call")
+    {
+    }
     else if (cmd.type == "builtin")
     {
       //tbitem->set_enabled(validate_command_item(titem, cmd) != ItemDisabled);
@@ -1025,6 +1111,15 @@ bool CommandUI::activate_command(const std::string &command, bec::ArgumentPool a
       _wb->execute_plugin(cmdparts.name, argpool);
       return true;
     }
+    else if (cmdparts.type == "call")
+    {
+      std::string module, function;
+      if (base::partition(cmdparts.name, ".", module, function))
+      {
+        _wb->get_grt()->call_module_function(module, function, grt::BaseListRef(_wb->get_grt()));
+        return true;
+      }
+    }
     else if (cmdparts.type == "builtin")
     {
       execute_builtin_command(cmdparts.name);
@@ -1054,7 +1149,7 @@ void CommandUI::activate_command(const std::string &command)
   
   if (!cmdparts.valid())
     return;
-  
+
   try
   {
 //    bec::UIForm *active_form= get_active_form();
@@ -1079,10 +1174,20 @@ void CommandUI::activate_command(const std::string &command)
         _wb->execute_plugin(cmdparts.name, args);
       }
     }
+    else if (cmdparts.type == "call")
+    {
+      std::string module, function;
+      if (base::partition(cmdparts.name, ".", module, function))
+        _wb->get_grt()->call_module_function(module, function, grt::BaseListRef(_wb->get_grt()));
+    }
   }
   catch (grt::grt_runtime_error &error)
   {
-    _wb->show_exception(command, error);
+    std::string caption(command);
+    app_PluginRef plugin = _wb->get_plugin_manager()->get_plugin(cmdparts.name);
+    if (plugin.is_valid())
+      caption = plugin->caption();
+    _wb->show_exception(caption, error);
   }
   catch (const std::exception& e)
   {
@@ -1091,10 +1196,22 @@ void CommandUI::activate_command(const std::string &command)
 }
 
 
+void CommandUI::revalidate_menu_bar(mforms::MenuBar *menu)
+{
+  _argpool.clear();
+  _wb->update_plugin_arguments_pool(_argpool);
+  _argpool["app.PluginInputDefinition:string"]= grt::StringRef("");
+  menu->validate();
+}
+
 void CommandUI::revalidate_edit_menu_items()
 {
+  _argpool.clear();
+  _wb->update_plugin_arguments_pool(_argpool);
+  _argpool["app.PluginInputDefinition:string"]= grt::StringRef("");
+
   if (mforms::Utilities::in_main_thread())
-    (*signal_validate_edit_menu_items())();
+    _validate_edit_menu_items();
   else
       _wb->get_grt_manager()->run_once_when_idle(boost::bind(&CommandUI::revalidate_edit_menu_items, this));
   //mforms::Utilities::perform_from_main_thread((boost::bind(&CommandUI::revalidate_edit_menu_items, this), (void*)0));
