@@ -17,13 +17,9 @@
  * 02110-1301  USA
  */
 
-
-
 #ifndef _WB_CONTEXT_H_
 #define _WB_CONTEXT_H_
 
-#include "wb_config.h"
-
 #ifndef _WIN32
 #include <vector>
 #endif
@@ -43,12 +39,7 @@
 #include "base/trackable.h"
 
 
-#define APP_MAJOR_NUMBER 5
-#define APP_MINOR_NUMBER 2
-#define APP_RELEASE_NUMBER 46
-#define APP_REVISION_NUMBER 10385
-#define APP_RELEASE_TYPE ""
-
+#include "wb_version.h"
 
 #define WBContext_VERSION 5
 
@@ -85,7 +76,6 @@ namespace wb {
     RefreshNewDiagram,
     RefreshSelection,
     RefreshCloseEditor, // argument: object-id (close all if "")
-    RefreshFindOutput,
     
     RefreshNewModel,
 
@@ -201,7 +191,6 @@ namespace wb {
     bool force_opengl_rendering;
     bool verbose;
     bool quit_when_done;
-    bool no_logo;
     bool testing;     // True if we are currently running unit tests.
     bool init_python; // True by default. Can be switched off for testing.
 
@@ -229,6 +218,7 @@ namespace wb {
     void init_finish_(WBOptions *options);
     void finalize();
     
+    bool is_commercial();
     WBContextUI *get_ui() { return _uicontext; }
 
     bec::UIForm *get_active_form();
@@ -316,14 +306,6 @@ namespace wb {
 
     std::string get_datadir() const { return _datadir; }
     
-    template<class C> grt::Ref<C> get_parent_for_object(const GrtObjectRef &object)
-    {
-      GrtObjectRef obj= object;
-      while (obj.is_valid() && !obj.is_instance(C::static_class_name()))
-        obj= obj->owner();
-      return grt::Ref<C>::cast_from(obj);
-    }
-
     bool cancel_idle_tasks();
     void flush_idle_tasks();
 
@@ -348,8 +330,6 @@ namespace wb {
                                      const boost::function<grt::ValueRef (grt::GRT*)> &function) THROW (grt::grt_runtime_error);
 
     bool activate_live_object(const GrtObjectRef &object);
-    bool create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
-    bool drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
 
     std::string create_attached_file(const std::string &group, const std::string &tmpl);
     void save_attached_file_contents(const std::string &name, const char *data, size_t size);
@@ -423,12 +403,10 @@ namespace wb {
     bool _force_sw_rendering;     // Command line switch.
     bool _force_opengl_rendering; // Command line switch.
 
-    //boost::signals2::signal<void ()> _signal_app_closing;
-    
-    grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt);
+    grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt, boost::shared_ptr<grt::internal::Unserializer> unserializer);
 
     // setup
-    void init_grt_tree(grt::GRT *grt, WBOptions *options);
+    void init_grt_tree(grt::GRT *grt, WBOptions *options, boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void run_init_scripts_grt(grt::GRT *grt, WBOptions *options);
     void init_plugins_grt(grt::GRT *grt, WBOptions *options);
     void init_plugin_groups_grt(grt::GRT *grt, WBOptions *options);
@@ -453,23 +431,21 @@ namespace wb {
     void save_connections();
     void save_instances();
 
-//    boost::signals2::signal<void ()>* signal_app_closing() { return &_signal_app_closing; }
-    
   protected:
     void add_recent_file(const std::string &file);
 
-    void load_app_state();
+    void load_app_state(boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void save_app_state();
-    void load_starters();
+    void load_starters(boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void save_starters();
 
-    grt::ValueRef new_document_grt(grt::GRT *grt);
     grt::ValueRef save_grt(grt::GRT *grt);
     
     grt::ValueRef execute_plugin_grt(grt::GRT *grt, const app_PluginRef &plugin, const grt::BaseListRef &args);
     void plugin_finished(const grt::ValueRef &result, const app_PluginRef &plugin);
 
     bool handle_message(const grt::Message &msg);
+    void handle_grt_message(bec::MessageListStorage::MessageEntryRef message);
 
     void reset_document();
     void reset_listeners();
@@ -485,6 +461,7 @@ namespace wb {
 
     bool install_module_file(const std::string &path);
     bool uninstall_module(grt::Module *module);
+    void run_script_file(const std::string &path);
     
   private:
     bool find_connection_password(const db_mgmt_ConnectionRef &conn, std::string &password);
