@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,18 @@
 
 #include <string>
 
+#include "base/ui_form.h"
+#include "base/string_utilities.h"
+#include "base/log.h"
+#include "base/notifications.h"
+
+#include "grtpp.h"
+
+#include "grts/structs.h"
+#include "grts/structs.app.h"
+
+#include "grt/editor_base.h"
+
 #include "wb_context_ui.h"
 #include "wb_context.h"
 #include "model/wb_component.h"
@@ -31,12 +43,8 @@
 
 #include "wb_overview.h"
 #include "model/wb_overview_physical.h"
-#include "workbench/wb_history_tree.h"
-#include "wb_find_dialog.h"
-#include "model/wb_user_datatypes.h"
 #include "model/wb_diagram_options.h"
 
-#include "grt/editor_base.h"
 #include "grt/clipboard.h"
 #include "grtui/gui_plugin_base.h"
 
@@ -44,13 +52,11 @@
 #include "output_view.h"
 #include "webbrowser_view.h"
 
-#include "mforms/mforms.h"
+#include "mforms/appview.h"
 
-#include "base/string_utilities.h"
-#include "base/log.h"
-#include "base/notifications.h"
+#include "home_screen.h"
 
-#include "model/wb_user_datatypes.h"
+#include "wb_command_ui.h"
 
 #include "plugin_install_window.h"
 
@@ -61,26 +67,17 @@ using namespace base;
 DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
 
 WBContextUI::WBContextUI(bool verbose)
-  : _wb(new WBContext(this, verbose)), _usertypes_tree(new UserDatatypeListBE(_wb)), _command_ui(_wb)
+  : _wb(new WBContext(this, verbose)), _command_ui(new CommandUI(_wb))
 {
   NodeId n; // workaround a bug causing crash when output tab is shown with GLib-ERROR **: The thread system is not yet initialized.
 
   base::NotificationCenter::get()->add_observer(this, "GNAppClosing");
-  base::NotificationCenter::get()->add_observer(this, "GNTextSelectionChanged");
-  base::NotificationCenter::get()->add_observer(this, "GNClipboardChanged");
   
   _catalog_tree= 0;
 
-  _history_tree= 0;
-
   _shell_window= 0;
   
-  _find_dialog= new FindDialogBE(_wb);
-  _find_dialog->set_activate_slot(boost::bind(&WBContextUI::activate_find_result, this, _1, _2, _3));
-
-  _output_view= new OutputView(_wb);
-  scoped_connect(_output_view->get_be()->signal_show(),boost::bind(&WBContextUI::show_output, this));
-    
+  _output_view = NULL;
   _active_form= 0;
   _active_main_form= 0;
   
@@ -89,8 +86,9 @@ WBContextUI::WBContextUI(bool verbose)
   
   _last_unsaved_changes_state= false;
   _quitting= false;
+  _processing_action_open_connection = false;
   
-  _home_screen= NULL;
+  _home_screen = NULL;
 
   // to notify that the save status of the doc has changed
   //_wb->get_grt()->get_undo_manager()->signal_changed().connect(boost::bind(&WBContextUI::history_changed, this));
@@ -98,6 +96,9 @@ WBContextUI::WBContextUI(bool verbose)
   
   // stuff to do when the active form is switched in the UI (through set_active_form)
   _form_change_signal.connect(boost::bind(&WBContextUI::form_changed, this));
+
+  _output_view = new OutputView(_wb);
+  scoped_connect(_output_view->get_be()->signal_show(),boost::bind(&WBContextUI::show_output, this));
 }
 
 
@@ -107,31 +108,25 @@ WBContextUI::~WBContextUI()
   
   _wb->do_close_document(true);
 
-  // is it really necessary to set all member variables to NULL in a dtor?. 
-  // -- Yes, some of these d-tors may trigger things that try to access the destroyed members
+  // TODO: is it really necessary to set all member variables to NULL in a dtor?
   delete _addon_download_window;
   _addon_download_window = 0;
   delete _plugin_install_window;
   _plugin_install_window = 0;
-  delete _find_dialog;
-  _find_dialog = 0;
-  delete _history_tree;
-  _history_tree = 0;
-  //XXX workaround for crash in linux
-#if defined(_WIN32) || defined(__APPLE__)
-  delete _usertypes_tree;
-#endif
-  _usertypes_tree = 0;
   if (_home_screen != NULL)
   {
     _home_screen->release();
     _home_screen = NULL;
   }
+
   delete _shell_window;
-  _shell_window = 0;
+  _shell_window = NULL;
 
   delete _wb;
-  _wb = 0;
+  _wb = NULL;
+
+  delete _command_ui;
+  _command_ui = NULL;
 }
 
 
@@ -139,8 +134,6 @@ void WBContextUI::handle_notification(const std::string &name, void *sender, bas
 {
   if (name == "GNAppClosing")
     home_screen_closing();
-  else if (name == "GNTextSelectionChanged" || name == "GNClipboardChanged")
-    get_command_ui()->revalidate_edit_menu_items();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -160,7 +153,6 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
     options->force_opengl_rendering ? "Yes" : "No",
     options->quit_when_done ? "Yes" : "No");
 
-  _initializing = true;
   bool flag = false;
   try
   {
@@ -180,16 +172,13 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
       WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
     }
 
-    _initializing = false;
   }
   catch (const std::exception& e)
   {
-    _initializing = false;
     log_error("WBContextUI::init, exception '%s'\n", e.what()); // log_error logs to stderr too in debug mode.
   }
   catch (...)
   {
-    _initializing = false;
     log_error("Some exception has happened. It was caught at WBContextUI::init.\n");
   }
 
@@ -208,9 +197,12 @@ GRTShellWindow* WBContextUI::get_shell_window()
 //--------------------------------------------------------------------------------------------------
 
 void WBContextUI::init_finish(WBOptions *options)
-{  
+{
+  g_assert(_wb->get_root().is_valid());
   show_home_screen();
   _wb->init_finish_(options);
+
+  NotificationCenter::get()->send("GNAppStarted", NULL);
 }
 
 
@@ -227,6 +219,16 @@ bool WBContextUI::request_quit()
   if (!_wb->get_grt_manager()->in_main_thread())
     g_warning("request_quit() called in worker thread");
 
+  {
+    NotificationInfo info;
+    info["cancel"] = "0";
+    
+    NotificationCenter::get()->send("GNAppShouldClose", NULL, info);
+    
+    if (info["cancel"] != "0")
+      return false;
+  }
+ 
   if (!_wb->can_close_document())
     return false;
 
@@ -242,11 +244,9 @@ bool WBContextUI::request_quit()
 
 void WBContextUI::perform_quit()
 {
-  _quitting= true;
+  _quitting = true;
   _wb->do_close_document(true);
   _wb->quit_application();
-
-  mforms::ControlFactory::get_instance()->shutdown();
 }
 
 
@@ -266,9 +266,6 @@ void WBContextUI::reset()
   get_physical_overview()->send_refresh_children(bec::NodeId());
   
   _wb->get_component<WBComponentPhysical>()->update_catalog_tree();
-
-  _usertypes_tree->set_catalog(_wb->get_document()->physicalModels()[0]->catalog());
-  _usertypes_tree->set_rdbms(_wb->get_document()->physicalModels()[0]->rdbms());
 }
 
 
@@ -305,27 +302,28 @@ void WBContextUI::overview_selection_changed()
 void WBContextUI::load_app_options(bool update)
 {
   if (!update)
-    _command_ui.load_data();
+    _command_ui->load_data();
 }
 
 
-std::string WBContextUI::get_diagram_path(const model_DiagramRef &diagram)
-{
-  std::string view_id= diagram.id();
-
-  // look in the physicalModels list for this diagram
-  for (size_t c= _wb->get_document()->physicalModels().count(), i= 0; i < c; i++)
-  {
-    workbench_physical_ModelRef model= _wb->get_document()->physicalModels()[i];
-    
-    for (size_t d= model->diagrams().count(), j= 0; j < d; j++)
-    {
-      if (model->diagrams().get(j).id() == view_id)
-        return strfmt("/wb/doc/physicalModels/%i/diagrams/%i", (int) i, (int) j);
-    }
-  }
-  return "";
-}
+// deprecated
+//std::string WBContextUI::get_diagram_path(const model_DiagramRef &diagram)
+//{
+//  std::string view_id= diagram.id();
+//
+//  // look in the physicalModels list for this diagram
+//  for (size_t c= _wb->get_document()->physicalModels().count(), i= 0; i < c; i++)
+//  {
+//    workbench_physical_ModelRef model= _wb->get_document()->physicalModels()[i];
+//    
+//    for (size_t d= model->diagrams().count(), j= 0; j < d; j++)
+//    {
+//      if (model->diagrams().get(j).id() == view_id)
+//        return strfmt("/wb/doc/physicalModels/%i/diagrams/%i", (int) i, (int) j);
+//    }
+//  }
+//  return "";
+//}
 
 
 static void add_script_file(WBContextUI *wbui)
@@ -360,48 +358,45 @@ static void add_note_file(WBContextUI *wbui)
   */
 void WBContextUI::add_backend_builtin_commands()
 {
-  _command_ui.add_builtin_command("overview.home", 
+  _command_ui->add_builtin_command("show_about",
+                                   boost::bind(&WBContextUI::show_about, this));
+  _command_ui->add_builtin_command("overview.home", // TODO: home screen should not be closable so we don't need this.
                                   boost::bind(&WBContextUI::show_home_screen, this));
-  _command_ui.add_builtin_command("show_output_form", 
+  _command_ui->add_builtin_command("show_output_form", 
                                   boost::bind(&WBContextUI::show_output, this));
 
-  _command_ui.add_builtin_command("find_next", 
-    boost::bind(&FindDialogBE::find_next, _find_dialog));
-  _command_ui.add_builtin_command("find_previous", 
-    boost::bind(&FindDialogBE::find_previous, _find_dialog));
-
-  _command_ui.add_builtin_command("add_script_file", 
+  _command_ui->add_builtin_command("add_script_file", 
     boost::bind(add_script_file, this));
-  _command_ui.add_builtin_command("add_note_file", 
+  _command_ui->add_builtin_command("add_note_file", 
     boost::bind(add_note_file, this));
 /*
   // handle some url opening stuff ourselves through mforms
-  _command_ui.add_builtin_command("community_blog",
+  _command_ui->add_builtin_command("community_blog",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/", true));
-  _command_ui.add_builtin_command("community_faq",
+  _command_ui->add_builtin_command("community_faq",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=7", true));
-  _command_ui.add_builtin_command("community_learn",
+  _command_ui->add_builtin_command("community_learn",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=10", true));
-  _command_ui.add_builtin_command("community_discuss",
+  _command_ui->add_builtin_command("community_discuss",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=16", true));
-  _command_ui.add_builtin_command("community_contribute",
+  _command_ui->add_builtin_command("community_contribute",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=17", true));
  */
-  _command_ui.add_builtin_command("web_mysql_home",
+  _command_ui->add_builtin_command("web_mysql_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/", true));
-  _command_ui.add_builtin_command("web_home",
+  _command_ui->add_builtin_command("web_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/products/tools/workbench/", true));
-  _command_ui.add_builtin_command("list_bugs",
+  _command_ui->add_builtin_command("list_bugs",
     boost::bind(&WBContextUI::show_web_page, this, "http://bugs.mysql.com/saved/WB%20bugs", true));
- // _command_ui.add_builtin_command("plugin_site",
+ // _command_ui->add_builtin_command("plugin_site",
  //   boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/addons/", true));
-//  _command_ui.add_builtin_command("check_update",
+//  _command_ui->add_builtin_command("check_update",
 //    boost::bind(&WBContextUI::show_web_page, this, 
 //               base::strfmt("http://wb.mysql.com/workbench/version-check.php?version=%i.%i.%i&revision=%i", 
 //                            APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER), false));
-  _command_ui.add_builtin_command("help_index", boost::bind(&WBContextUI::show_help_index, this));
-  _command_ui.add_builtin_command("locate_log_file", boost::bind(&WBContextUI::locate_log_file, this));
-  _command_ui.add_builtin_command("show_log_file", boost::bind(&WBContextUI::show_log_file, this));
+  _command_ui->add_builtin_command("help_index", boost::bind(&WBContextUI::show_help_index, this));
+  _command_ui->add_builtin_command("locate_log_file", boost::bind(&WBContextUI::locate_log_file, this));
+  _command_ui->add_builtin_command("show_log_file", boost::bind(&WBContextUI::show_log_file, this));
 }
 
 #ifndef ___specialforms
@@ -421,12 +416,6 @@ bec::ValueTreeBE *WBContextUI::get_catalog_tree()
 }
 
 
-UserDatatypeListBE *WBContextUI::get_usertypes_tree()
-{
-  return _usertypes_tree;
-}
-
-
 PhysicalOverviewBE *WBContextUI::get_physical_overview()
 {
   return get_wb()->get_model_context() ? get_wb()->get_model_context()->get_overview() : 0;
@@ -521,8 +510,7 @@ void WBContextUI::form_changed()
   
   if (_active_form != NULL && _active_form == _home_screen)
   {
-    refresh_home_models();
-    refresh_home_instances(0);
+    refresh_home_documents();
     refresh_home_connections();
   }
 
@@ -533,17 +521,6 @@ void WBContextUI::form_changed()
   }
 }
 
-HistoryTreeBE *WBContextUI::get_history_tree()
-{
-  if (!_history_tree)
-  {
-    _history_tree= new HistoryTreeBE(_wb, _wb->get_grt()->get_undo_manager());
-    
-    scoped_connect(_wb->get_grt()->get_undo_manager()->signal_changed(),boost::bind(&HistoryTreeBE::tree_changed, _history_tree, -1, bec::NodeId()));
-  }
-  return _history_tree;
-}
-
 
 bec::ValueInspectorBE *WBContextUI::create_inspector_for_selection(bec::UIForm *form, std::vector<std::string> &items)
 {
@@ -829,34 +806,6 @@ std::string WBContextUI::get_active_diagram_info()
   return "";
 }
 
-
-void WBContextUI::activate_find_result(const GrtObjectRef &location, const GrtObjectRef &object, const std::string &text)
-{
-  _wb->show_status_text(strfmt("Match: %s (%s)", 
-    text.c_str(), object.get_metaclass()->get_attribute("caption").c_str()));
-
-  if (location.is_valid())
-  {
-    model_DiagramRef view;
-
-    if (location.is_instance<model_Diagram>())
-      view= model_DiagramRef::cast_from(location);
-    else
-      view= _wb->get_parent_for_object<model_Diagram>(location);
-
-    if (view.is_valid())
-    {
-      ModelDiagramForm *form= get_wb()->get_model_context()->get_diagram_form_for_diagram_id(view.id());
-
-      if (form)
-      {
-        _wb->switched_view(form->get_view());
-        form->focus_and_make_visible(model_FigureRef::cast_from(location), true);
-      }
-    }
-  }
-}
-
 #endif // ___specialforms
 
 
@@ -881,13 +830,15 @@ void WBContextUI::set_doc_properties(const std::string &caption, const std::stri
 {
   app_DocumentInfoRef info= _wb->get_document()->info();
 
+  grt::AutoUndo undo(_wb->get_grt());
   info->caption(caption);
   info->version(version);
   info->author(author);
   info->project(project);
   info->dateCreated(date_created);
   info->dateChanged(date_changed);
-  info->description(description);  
+  info->description(description);
+  undo.end("Change document properties");
 }
 
 
@@ -1153,6 +1104,7 @@ void WBContextUI::set_active_form(bec::UIForm *form)
 
       base::NotificationInfo info;
       info["form"] = form ? form->form_id() : "";
+      info["context"] = get_active_context(true);
       NotificationCenter::get()->send("GNMainFormChanged", 0, info);
     }
   }
@@ -1227,3 +1179,38 @@ bool WBContextUI::start_plugin_install(const std::string &path)
     _plugin_install_window = new PluginInstallWindow(this);
   return _plugin_install_window->install_plugin(path);
 }
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterWBContextUINotifDocs
+{
+  RegisterWBContextUINotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GNAppStarted",
+                                                           "application",
+                                                           "Sent when Workbench starts up and finishes with various initialization routines.",
+                                                           "",
+                                                           "");
+    
+    base::NotificationCenter::get()->register_notification("GNAppShouldClose",
+                                                           "application",
+                                                           "Sent when the user requests Workbench to close. Close can be cancelled by setting the 'cancel' field in the info dict to 1.",
+                                                           "",
+                                                           "cancel - set to 1 if exit should be cancelled");
+
+    base::NotificationCenter::get()->register_notification("GNMainFormChanged",
+                                                           "application",
+                                                           "Sent when the main tab from the application is switched.",
+                                                           "",
+                                                           "form - the ID of the newly active form\n"
+                                                           "context - the context name of the newly active form");
+
+    base::NotificationCenter::get()->register_notification("GNApplicationActivated",
+                                                           "application",
+                                                           "Sent when the application was activated.",
+                                                           "",
+                                                           "");
+  }
+} initdocs;
+
+
