@@ -21,14 +21,13 @@
 
 #include <stack>
 
-#include "wb_config.h"
-
 #include "wb_overview.h"
 #include "model/wb_overview_physical.h"
 #include "grt/icon_manager.h"
 #include "grt/clipboard.h"
 #include "grt/exceptions.h"
 
+#include "model/wb_context_model.h"
 #include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "grt/validation_manager.h"
@@ -53,7 +52,7 @@ int OverviewBE::Node::get_popup_menu_items(WBContext *wb, bec::MenuItemList &ite
 
 bool OverviewBE::ObjectNode::activate(WBContext *wb)
 {
-  wb->open_object_editor(object);
+  wb->get_grt_manager()->open_object_editor(object, bec::NoFlags);
 
   return true;
 }
@@ -512,7 +511,7 @@ void OverviewBE::begin_selection_marking()
 
 void OverviewBE::end_selection_marking()
 {
-  _selection_change_signal.emit();
+  _selection_change_signal();
 }
 
 
@@ -1071,12 +1070,12 @@ bool OverviewBE::activate_popup_item_for_nodes(const std::string &name, const st
   else if (name == "builtin:delete")
   {
     grt::AutoUndo undo(_wb->get_grt());
-    
+
     for (std::vector<bec::NodeId>::const_iterator end= nodes.end(), iter= nodes.begin();
          iter != end; ++iter)
     {
       Node *n= get_node(*iter);
-      if (n)        
+      if (n)
         n->delete_object(_wb);
     }
     
