@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 #include "grt_manager.h"
 #include <grtpp_util.h>
 #include <grtpp_shell.h>
+#include "base/log.h"
 
 using namespace bec;
 
@@ -383,6 +384,11 @@ void GRTDispatcher::start(boost::shared_ptr<GRTDispatcher> self)
   {
     DPRINT("starting worker thread");
     _thread= g_thread_create(worker_thread, this, FALSE, NULL);
+    if (_thread == 0)
+    {
+      log_error("g_thread_create failed to create the GRT worker thread. Falling back into non-threaded mode.\n");
+      _threading_disabled = true;
+    }
   }
 
   bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_grt);
