@@ -992,6 +992,26 @@ std::string TableHelper::get_document_comment(const std::string &comment, const
   return std::string(find_comment_split(comment, max_len));
 };
 
+
+std::string TableHelper::normalize_table_name_list(const std::string &schema, const std::string &table_name_list)
+{
+  std::vector<std::string> names = base::split(table_name_list, ",");
+
+  for (std::vector<std::string>::iterator it = names.begin(); it != names.end(); ++it)
+  {
+    std::vector<std::string> tokens(base::split_qualified_identifier(base::trim(*it)));
+    if (tokens.size() == 1)
+      tokens.insert(tokens.begin(), schema);
+
+    for (std::vector<std::string>::iterator t = tokens.begin(); t != tokens.end(); ++t)
+      *t = base::quote_identifier(base::unquote_identifier(*t), '`');
+
+    *it = base::join(tokens, ".");
+  }
+
+  return base::join(names, ",");
+}
+
 void ColumnHelper::copy_column(const db_ColumnRef &from, db_ColumnRef &to)
 {
   to->userType(from->userType());
@@ -1157,8 +1177,7 @@ void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
     || (strcmp(column->simpleType()->name().c_str(), "DATE") == 0)
     || (strcmp(column->simpleType()->name().c_str(), "TIME") == 0)
     || (strcmp(column->simpleType()->name().c_str(), "TIMESTAMP") == 0)
-    || (strcmp(column->simpleType()->name().c_str(), "YEAR") == 0)
-    || (strcmp(column->simpleType()->name().c_str(), "BIT") == 0))
+    || (strcmp(column->simpleType()->name().c_str(), "YEAR") == 0))
   {
     column->length(grt::IntegerRef(-1));
     if (default_default_value)
