@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
+
 #include "stdafx.h"
 
 #include "grt/grt_dispatcher.h"
 #include "editor_routine.h"
 #include "base/string_utilities.h"
 
-
 namespace bec {
 
 RoutineEditorBE::RoutineEditorBE(GRTManager *grtm, const db_RoutineRef &routine, const db_mgmt_RdbmsRef &rdbms)
@@ -67,8 +67,19 @@ std::string RoutineEditorBE::get_sql_template(const std::string &template_name,
   size_t pos;
   if (template_name.empty() || template_name == "default")
   {
-    sql= base::strfmt("CREATE PROCEDURE `%s`.`%s` ()\nBEGIN\n\nEND\n",
-      get_schema_name().c_str(), get_name().c_str());
+    std::string routine_type = _routine->routineType();
+
+    if (routine_type == "function")
+    {
+      sql= base::strfmt("CREATE FUNCTION `%s` ()\nRETURNS INTEGER\nBEGIN\n\nRETURN 1;\nEND\n",
+        /*get_schema_name().c_str(), */get_name().c_str());
+      // better leave the schema name out to leave it portable across schemas
+    }
+    else
+    {
+      sql= base::strfmt("CREATE PROCEDURE `%s` ()\nBEGIN\n\nEND\n",
+        /*get_schema_name().c_str(), */get_name().c_str());
+    }
   }
 
   pos= sql.find("BEGIN");
@@ -94,7 +105,7 @@ std::string RoutineEditorBE::get_formatted_sql_for_editing(int &cursor_pos)
   }
   else
   {
-    sql.append(code)/*.append(_non_std_sql_delimiter)*/.append("\n");
+    sql.append(code).append("\n");
   }
 
   return sql;
@@ -105,7 +116,7 @@ void RoutineEditorBE::set_sql(const std::string &sql, bool sync)
 {
   if (get_sql() != sql)
   {
-    set_sql_parser_task_cb(sigc::mem_fun(this, &RoutineEditorBE::parse_sql));
+    set_sql_parser_task_cb(boost::bind(&RoutineEditorBE::parse_sql, this, _1, _2));
     DBObjectEditorBE::set_sql(sql, sync, _routine);
   }
 }
@@ -113,7 +124,6 @@ void RoutineEditorBE::set_sql(const std::string &sql, bool sync)
 
 grt::ValueRef RoutineEditorBE::parse_sql(grt::GRT* grt, grt::StringRef sql)
 {
-  //grt::AutoUndo undo(grt);
   AutoUndoEdit undo(this);
 
   int err_count= _sql_parser->parse_routine(_routine, sql.c_str());
@@ -139,4 +149,10 @@ Sql_editor::Ref RoutineEditorBE::get_sql_editor()
 }
 
 
+std::string RoutineEditorBE::get_title()
+{
+  return base::strfmt("%s - Routine", get_name().c_str()); 
+}
+
+  
 } // namespace bec
