@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -121,13 +121,19 @@ std::vector<std::string> TableColumnsListBE::get_datatype_names()
       group= (*iter)->group();
       types.push_back("-");
     }
+
+    std::string tmp;
+
     if ((*iter)->parameterFormatType() == 1 ||
         (*iter)->parameterFormatType() == 3 ||
         (*iter)->parameterFormatType() == 4 ||
         (*iter)->parameterFormatType() == 10)
-      types.push_back(*(*iter)->name()+"()");
+      tmp = *(*iter)->name()+"()";
     else
-      types.push_back((*iter)->name());
+      tmp = (*iter)->name();
+
+    if (types.empty() || types.back() != tmp)
+      types.push_back(tmp);
   }
   
   // insert UDTs
@@ -257,11 +263,16 @@ void TableColumnsListBE::reorder(const NodeId &node, int nindex)
     update_primary_index_order();
   
     _owner->update_change_date();
-    
+
+    _owner->freeze_refresh_on_object_change();
     (*_owner->get_table()->signal_refreshDisplay())("column");
+    _owner->thaw_refresh_on_object_change(true);
     undo.end(strfmt(_("Reorder Column '%s.%s'"), _owner->get_name().c_str(), _owner->get_table()->columns()[node[0]]->name().c_str()));
 
-    _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnList);
+    if (nindex < node[0])
+      _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnMoveUp);
+    else
+      _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnMoveDown);
   }
 }
 
@@ -1023,7 +1034,7 @@ int TableColumnsListBE::get_column_flag(const ::bec::NodeId &node, const std::st
       for(size_t i= 0; i < flags_count; i++)
       {
         grt::StringRef existing_flag= col_flags.get(i);
-        if(g_strcasecmp(flag_name.c_str(), existing_flag.c_str()) == 0)
+        if(g_ascii_strcasecmp(flag_name.c_str(), existing_flag.c_str()) == 0)
           return 1;
       }
     }else if (col->userType().is_valid())
@@ -1050,10 +1061,12 @@ std::string TableColumnsListBE::quote_value_if_needed(const db_ColumnRef &column
   // Non-text types (like datetime) shouldn't be auto-quoted.
   // Using the "text" and "string" groups here includes more than we'd need (e.g. TEXT blobs), but as long
   // as this function is only used for default values (which blobs cannot have) we are fine.
-  bool isChar = (g_strcasecmp(datatypeGroupName.c_str(), "string") == 0) || 
-                (g_strcasecmp(datatypeGroupName.c_str(), "text") == 0);
+  bool isChar = (g_ascii_strcasecmp(datatypeGroupName.c_str(), "string") == 0) || 
+                (g_ascii_strcasecmp(datatypeGroupName.c_str(), "text") == 0) ||
+                (g_ascii_strcasecmp(datatypeName.c_str(), "ENUM") == 0) ||
+                (g_ascii_strcasecmp(datatypeName.c_str(), "SET") == 0);
 
-  if (isChar && (g_strcasecmp(value.c_str(), "NULL") != 0) && (value != "0") && (value[0] != '\''))
+  if (isChar && (value != "NULL") && (value != "0") && (value[0] != '\''))
     return std::string("'").append(escape_sql_string(value)).append("'");
   return value;
 }
@@ -1214,8 +1227,9 @@ bool TableColumnsListBE::activate_popup_item_for_nodes(const std::string &name,
       
       if (!nodes.empty())
       {
+        int c = 0;
         for (std::list<NodeId>::const_iterator i = added_cols.begin(); i != added_cols.end(); ++i)
-          reorder(*i, nodes[0][0]+1);
+          reorder(*i, nodes[0][0]+c++);
       }
       refresh();
       _owner->update_change_date();
@@ -1420,11 +1434,13 @@ bool IndexColumnsListBE::set_field(const NodeId &node, int column, int value)
   if (node[0] >= count())
     return false;
 
-  if (!_owner->index_editable(_owner->get_selected_index())
-      && column != OrderIndex) // index order can be changed for PKs
-        return false;
-
   icolumn= get_index_column(_owner->get_owner()->get_table()->columns()[node[0]]);
+  if (!_owner->index_editable(_owner->get_selected_index())
+      && column != OrderIndex && !(icolumn->referencedColumn()->simpleType().is_valid() &&  column == Length && icolumn->referencedColumn()->simpleType()->group()->name() == "string")) // index order can be changed for PKs
+        return false;                                                                                                           // index length can be changed for PK if it's text
+
+
+
 
   switch (column)
   {
@@ -1775,7 +1791,7 @@ bool IndexListBE::set_field(const NodeId &node, int column, const std::string &v
 
   index= _owner->get_table()->indices().get(node[0]);
 
-  if (!index_editable(index))
+  if (!index_editable(index) && column != Comment) // Allow editing comments on PKs.
     return false;
 
   switch (column)
@@ -1922,7 +1938,7 @@ static std::set<std::string> get_indexed_column_ids(const db_TableRef &table)
  * @param node of the source column in the FK column list.
  * @return List of possible column names.
  */
-std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const NodeId &node)
+std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const NodeId &node, bool filtered)
 {
   db_ForeignKeyRef fk(_owner->get_selected_fk());
   db_ColumnRef srccolumn(_owner->get_owner()->get_table()->columns()[node[0]]);
@@ -1935,8 +1951,10 @@ std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const N
     std::set<std::string> indexed_column_ids(get_indexed_column_ids(table));
 
     for (size_t c= columns.count(), i= 0; i < c; i++)
-    {      
-      if (_owner->get_owner()->check_column_referenceable_by_fk(columns[i], srccolumn))
+    {     
+      if (!filtered)
+        names.push_back(columns[i]->name());
+      else if (_owner->get_owner()->check_column_referenceable_by_fk(columns[i], srccolumn))
       {
         // column must have index to be in a Fk
         if (table->isPrimaryKeyColumn(columns[i]))
@@ -2130,14 +2148,7 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, const
     if (fk.is_valid() && fk->referencedTable().is_valid())
     {
       int index;
-      
-      // check if column is acceptable
-      std::vector<std::string> allowed_list = get_ref_columns_list(node);
-      if (std::find(allowed_list.begin(), allowed_list.end(), value) == allowed_list.end())
-        return false;
-
       tcolumn= grt::find_named_object_in_list(fk->referencedTable()->columns(), value);
-
       index= get_fk_column_index(node);
 
       // if columns is not enabled yet, enable it
@@ -2149,6 +2160,37 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, const
           return false;
       }
 
+      // check if column is acceptable
+      std::vector<std::string> allowed_list = get_ref_columns_list(node);
+      if (std::find(allowed_list.begin(), allowed_list.end(), value) == allowed_list.end())
+      {
+        std::set<std::string> indexed_column_ids(get_indexed_column_ids(fk->referencedTable()));
+
+        if (indexed_column_ids.find(tcolumn->id()) == indexed_column_ids.end())
+        {
+          mforms::Utilities::show_message("Create Foreign Key",
+                              base::strfmt("Selected column %s must be indexed and be of a compatible type for a Foreign Key to be created.", tcolumn->name().c_str()),
+                              "OK");
+        }
+        else
+        {
+          db_TableRef reftable = fk->referencedTable();
+          std::string hint;
+          hint = base::strfmt("\nHint: source column has type %s%s,\nColumn %s of referenced table is %s%s.",
+                              _owner->get_owner()->get_table()->columns()[node[0]]->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                              _owner->get_owner()->get_table()->columns()[node[0]]->formattedType().c_str(),
+                              value.c_str(),
+                              tcolumn->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                              tcolumn->formattedType().c_str());
+          // show a warning
+          mforms::Utilities::show_message("Create Foreign Key",
+                                          base::strfmt("The selected columns do not have compatible types.\n%s",
+                                                       hint.c_str()),
+                                          "OK");
+        }
+        return false;
+      }
+
       set_fk_column_pair(fk->columns()[index], tcolumn);
     }
     return true;
@@ -2756,14 +2798,15 @@ NodeId TableEditorBE::duplicate_column(const db_ColumnRef &column)
   db_ColumnRef new_column = shallow_copy_object(column);
 
   std::string name = new_column->name();
+  std::string new_name = name;
   for (int i = 1;; i++)
   {
-    if (!find_named_object_in_list(get_table()->columns(), name).is_valid())
+    if (!find_named_object_in_list(get_table()->columns(), new_name).is_valid())
       break;
-    name = strfmt("%s_copy%i", name.c_str(), i);
+    new_name = strfmt("%s_copy%i", name.c_str(), i);
   }
-  if (name != *new_column->name())
-    new_column->name(name);
+  if (new_name != *new_column->name())
+    new_column->name(new_name);
   new_column->owner(get_table());
   get_table()->addColumn(new_column);
 
@@ -3111,7 +3154,20 @@ void TableEditorBE::show_import_wizard()
 
     grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
     if (module)
-      module->call_function("importRecordsetDataFromFile", args);
+    {
+      try
+      {
+        module->call_function("importRecordsetDataFromFile", args);
+      }
+      catch (grt::module_error &exc)
+      {
+        mforms::Utilities::show_error("Import Standard INSERTs", "Error during data import:\n" + exc.inner, "OK");
+      }
+      catch (std::exception &exc)
+      {
+        mforms::Utilities::show_error("Import Standard INSERTs", "Error during data import:\n" + std::string(exc.what()), "OK");
+      }
+    }
     else
       log_error("Can't find module SQLIDEUtils for record importer");
   }
