@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -242,7 +242,7 @@ namespace bec {
     virtual void refresh();
     virtual int count();
 
-    std::vector<std::string> get_ref_columns_list(const NodeId &node);
+    std::vector<std::string> get_ref_columns_list(const NodeId &node, bool filtered=true);
 
     // for editable lists only
     virtual bool set_field(const NodeId &node, int column, const std::string &value);
@@ -328,6 +328,8 @@ namespace bec {
   public:
     enum PartialRefreshes
     {
+      RefreshColumnMoveUp,
+      RefreshColumnMoveDown,
       RefreshColumnList,
       RefreshColumnCollation
     };
