@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -41,7 +41,8 @@ std::string ViewEditorBE::get_query()
   if ( ret.empty() )
   {
     ret = "CREATE VIEW `";
-    ret += get_schema_name() + "`.`" + get_name() + "` AS\n";
+    // make views schema portable by default
+    ret += /*get_schema_name() + "`.`" + */get_name() + "` AS\n";
   }
   
   return ret;
@@ -51,14 +52,14 @@ void ViewEditorBE::set_query(const std::string &sql, bool sync)
 {
   if (get_query() != sql)
   {
-    set_sql_parser_task_cb(sigc::mem_fun(this, &ViewEditorBE::parse_sql));
+    // TODO: is it really necessary to bind it every time we set a new SQL text?
+    set_sql_parser_task_cb(boost::bind(&ViewEditorBE::parse_sql, this, _1, _2));
     set_sql(sql, sync, _view);
   }
 }
 
 grt::ValueRef ViewEditorBE::parse_sql(grt::GRT* grt, grt::StringRef sql)
 {
-  //grt::AutoUndo undo(grt);
   AutoUndoEdit undo(this);
 
   int err_count= _sql_parser->parse_view(_view, sql.c_str());
@@ -81,3 +82,9 @@ Sql_editor::Ref ViewEditorBE::get_sql_editor()
   }
   return sql_editor;
 }
+
+std::string ViewEditorBE::get_title()
+{
+  return base::strfmt("%s - View", get_name().c_str()); 
+}
+
