@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,10 +26,11 @@
 
 #include <grt/common.h>
 #include "base/string_utilities.h"
-
+#include "base/log.h"
 #include "mforms/uistyle.h"
 #include "mforms/utilities.h"
 #include "mforms/checkbox.h"
+#include "mforms/textbox.h"
 
 #define MYSQL_RDBMS_ID "com.mysql.rdbms.mysql"
 
@@ -56,10 +57,12 @@ static bool is_ssh_driver(const std::string &driver_name)
 DbConnectPanel::DbConnectPanel(DbConnectPanelFlags flags)
 : Box(false), _connection(0), 
 _tab(mforms::TabViewSystemStandard),
-_params_panel(mforms::TransparentPanel), _params_table(0), 
+_params_panel(mforms::TransparentPanel), _params_table(0),
+_ssl_panel(mforms::TransparentPanel), _ssl_table(0),
 _advanced_panel(mforms::TransparentPanel), _advanced_table(0),
 _show_connection_combo((flags & DbConnectPanelShowConnectionCombo) != 0), 
-_show_manage_connections((flags & DbConnectPanelShowManageConnections) != 0)
+_show_manage_connections((flags & DbConnectPanelShowManageConnections) != 0),
+_dont_set_default_connection((flags & DbConnectPanelDontSetDefaultConnection) != 0)
 {
   _allow_edit_connections = false;
   _initialized= false;
@@ -120,20 +123,23 @@ _show_manage_connections((flags & DbConnectPanelShowManageConnections) != 0)
     _table.add(mforms::manage(new mforms::Label()), 0, 1, row, row+1, mforms::HFillFlag);
     row++;
   }
-  
-  if (_show_connection_combo)
+
+  if (!(flags & DbConnectPanelHideConnectionName))
   {
-    _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
-    _table.add(&_stored_connection_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
-    _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);
+    if (_show_connection_combo)
+    {
+      _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
+      _table.add(&_stored_connection_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+      _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);
+    }
+    else
+    {
+      _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
+      _table.add(&_name_entry, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+      _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);    
+    }
+    row++;
   }
-  else
-  {
-    _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
-    _table.add(&_name_entry, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
-    _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);    
-  }
-  row++;
   
   _table.add(&_label3, 0, 1, row, row+1, mforms::HFillFlag);
   _table.add(&_driver_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
@@ -141,8 +147,10 @@ _show_manage_connections((flags & DbConnectPanelShowManageConnections) != 0)
 
   _tab.set_name("connect_panel:tab");
   _params_panel.set_name("params_panel");
+  _ssl_panel.set_name("ssl_panel");
   _advanced_panel.set_name("advanced_panel");
   _tab.add_page(&_params_panel, _("Parameters"));
+  _tab.add_page(&_ssl_panel, _("SSL"));
   _tab.add_page(&_advanced_panel, _("Advanced"));
   
   set_name("connect_panel");
@@ -341,10 +349,10 @@ void DbConnectPanel::param_value_changed(mforms::View *sender)
   
   _connection->save_changes();
   
-  std::string error= _connection->validate_driver_params();
+  std::string error = _connection->validate_driver_params();
   if (error != _last_validation)
     _signal_validation_state_changed(error, error.empty());
-  _last_validation= error;
+  _last_validation = error;
 }
 
 
@@ -371,11 +379,11 @@ void DbConnectPanel::enum_param_value_changed(mforms::Selector *sender, std::vec
     param->set_value(grt::StringRef(""));
   
   _connection->save_changes();
-  
+
   std::string error= _connection->validate_driver_params();
   if (error != _last_validation)
     _signal_validation_state_changed(error, error.empty());
-  _last_validation= error;
+  _last_validation = error;
 }
 
 
@@ -424,6 +432,7 @@ void DbConnectPanel::change_active_rdbms()
           _driver_sel.set_selected(default_driver);
         _connection->set_driver_and_update(selected_driver());
       }
+
       _updating = false;      
     }
     else
@@ -537,7 +546,7 @@ void DbConnectPanel::refresh_stored_connections()
     if (!rdbms.is_valid() || ((*iter)->driver().is_valid() && (*iter)->driver()->owner() == rdbms))
     {
       _stored_connection_sel.add_item((*iter)->name());
-      if (*(*iter)->isDefault())
+      if (*(*iter)->isDefault() && !_dont_set_default_connection)
         selected_index = i;
       i++;
     }
@@ -575,7 +584,7 @@ void DbConnectPanel::save_connection_as(const std::string &name)
 
   list = _connection->get_db_mgmt()->otherStoredConns();
   if (list.get_index(conn) != grt::BaseListRef::npos)
-    throw std::invalid_argument("The connection cannot be saved becasue it is already stored");
+    throw std::invalid_argument("The connection cannot be saved because it is already stored");
 
   if ((dup = find_named_object_in_list(list, name, true, "name")).is_valid())
   {
@@ -649,6 +658,7 @@ void DbConnectPanel::set_active_stored_conn(db_mgmt_ConnectionRef connection)
     _driver_sel.set_selected(driver_index);
 
   // mark this connection as the active one for this rdbms type
+  if (!_dont_set_default_connection)
   {
     grt::ListRef<db_mgmt_Connection> conns(connection_list());
     for (size_t c = conns->count(), i = 0; i < c; i++)
@@ -676,7 +686,7 @@ void DbConnectPanel::change_active_stored_conn()
     _updating= true;
 
     if (_show_manage_connections && _stored_connection_sel.get_selected_index() == _stored_connection_sel.get_item_count()-1)
-    {   
+    {
       choosing = true;
       db_mgmt_ConnectionRef connection = open_editor();
       refresh_stored_connections();
@@ -696,7 +706,13 @@ void DbConnectPanel::change_active_stored_conn()
       set_active_stored_conn(name);
       show();
     }
-    _updating= false;
+    _updating = false;
+
+    // Revalidate connection parameters.
+    std::string error = _connection->validate_driver_params();
+    if (error != _last_validation)
+      _signal_validation_state_changed(error, error.empty());
+    _last_validation = error;
   }
 }
 
@@ -715,10 +731,12 @@ void DbConnectPanel::begin_layout()
 {
   if (_params_table)
   {
-    //_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF | _CRTDBG_CHECK_ALWAYS_DF );
-
     _params_panel.remove(_params_table);
-  }  
+  }
+  if (_ssl_table)
+  {
+    _ssl_panel.remove(_ssl_table);
+  }
   if (_advanced_table)
   {
     _advanced_panel.remove(_advanced_table);
@@ -730,6 +748,13 @@ void DbConnectPanel::begin_layout()
   _params_table->set_column_spacing(MF_TABLE_COLUMN_SPACING);
   _params_table->set_padding(MF_PANEL_PADDING);
 
+  _ssl_table = mforms::manage(new mforms::Table());
+  _ssl_table->set_name("ssl_table");
+  _ssl_table->set_column_count(3);
+  _ssl_table->set_row_spacing(MF_TABLE_ROW_SPACING);
+  _ssl_table->set_column_spacing(MF_TABLE_COLUMN_SPACING);
+  _ssl_table->set_padding(MF_PANEL_PADDING);
+
   _advanced_table = mforms::manage(new mforms::Table());
   _advanced_table->set_name("advanced_table");
   _advanced_table->set_column_count(3);
@@ -739,6 +764,7 @@ void DbConnectPanel::begin_layout()
   
   _views.clear();
   _param_rows.clear();
+  _ssl_rows.clear();
   _advanced_rows.clear();  
 }
 
@@ -746,6 +772,7 @@ void DbConnectPanel::begin_layout()
 void DbConnectPanel::end_layout()
 {
   _params_panel.add(_params_table);
+  _ssl_panel.add(_ssl_table);
   _advanced_panel.add(_advanced_table);
 }
 
@@ -816,75 +843,63 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
   bool is_new_line= false;
   Table *table= NULL;
   Box *box= NULL;
+  std::vector<mforms::Box*> *rows = NULL;
 
   switch (driver_param->object()->layoutAdvanced())
   {
   case 0:
-    {
-      if (bounds.top >= (int)_param_rows.size())
-      {
-        is_new_line= true;
-
-        _params_table->set_row_count(_param_rows.size() + 1);
-        _param_rows.push_back(box= new Box(true));
-
-        mforms::TableItemFlags flags;
-#ifdef __APPLE__ // HExpand is messing layout in mac
-        flags = mforms::HFillFlag;
-#else
-        flags = mforms::HExpandFlag | mforms::HFillFlag;
-#endif
-        _params_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, flags);
-        box->set_spacing(4);
-        _views.push_back(box);
-      }
-      else
-        box= _param_rows[bounds.top];
-
-      table= _params_table;
-      break;
-    }
+    rows = &_param_rows;
+    table = _params_table;
+    break;
   case 1:
-    {
-      if (bounds.top >= (int)_advanced_rows.size())
-      {
-        is_new_line= true;
-
-        _advanced_table->set_row_count(_advanced_rows.size() + 1);
-        if (ctrl_type == ::ctCheckBox)
-        {
-          _advanced_rows.push_back(box= new Box(false));
-          box->set_spacing(0);
-        }
-        else
-        {
-          _advanced_rows.push_back(box= new Box(true));
-          box->set_spacing(4);
-        }
-        _views.push_back(box);
-#ifdef __APPLE__ // TODO: HExpand is messing layout in mac, needs fix
-        _advanced_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HFillFlag);
-#else
-        _advanced_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HExpandFlag|mforms::HFillFlag);
-#endif
-      }
-      else
-        box= _advanced_rows[bounds.top];
-
-      table= _advanced_table;
-      break;
-    }
+    rows = &_advanced_rows;
+    table = _advanced_table;
+    break;
+  case 2:
+    rows = &_ssl_rows;
+    table = _ssl_table;
+    break;
   default:
     return;
   }
 
+  if (bounds.top >= (int)rows->size())
+  {
+    is_new_line= true;
+
+    table->set_row_count(rows->size() + 1);
+    if (ctrl_type == ::ctCheckBox && table != _params_table)
+    {
+      rows->push_back(box= new Box(false));
+      box->set_spacing(0);
+    }
+    else
+    {
+      rows->push_back(box= new Box(true));
+      box->set_spacing(4);
+    }
+    _views.push_back(box);
+
+    mforms::TableItemFlags flags;
+#ifdef __APPLE__ // HExpand is messing layout in mac
+    flags = mforms::HFillFlag;
+#else
+    flags = mforms::HExpandFlag | mforms::HFillFlag;
+#endif
+    if (driver_param->get_type() == DbDriverParam::ptText)
+      flags = flags | mforms::VExpandFlag|mforms::VFillFlag;
+    table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, flags);
+  }
+  else
+    box= (*rows)[bounds.top];
+
   switch (ctrl_type)
   {
   case ::ctLabel:
     {
       Label *label= new Label();
       label->set_text(caption);
-      label->set_text_align(mforms::MiddleRight);
+      label->set_text_align(mforms::TopRight);
 
       if (is_new_line)
         table->add(mforms::manage(label), 0, 1, bounds.top, bounds.top + 1, mforms::HFillFlag | mforms::VFillFlag);
@@ -897,6 +912,7 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
     {
       Label *label= new Label();
       label->set_text(caption);
+      label->set_text_align(mforms::TopLeft);
       label->set_style(mforms::SmallHelpTextStyle);
       table->add(mforms::manage(label), 2, 3, bounds.top, bounds.top + 1, mforms::HFillFlag | mforms::VFillFlag);
       _views.push_back(label);
@@ -977,6 +993,28 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
       
       break;
     }
+    case ::ctText:
+    {
+      TextBox *ctrl= new TextBox(mforms::VerticalScrollBar);
+
+      ctrl->set_name(driver_param->get_control_name());
+
+      // value
+      {
+        grt::StringRef value= driver_param->get_value_repr();
+        if (value.is_valid())
+          ctrl->set_value(*value);
+      }
+
+      ctrl->set_size(bounds.width, -1);
+
+      scoped_connect(ctrl->signal_changed(),boost::bind(&DbConnectPanel::param_value_changed, this, ctrl));
+
+      box->add(mforms::manage(ctrl), true, true);
+      _views.push_back(ctrl);
+      
+      break;
+    }
     case ::ctFileSelector:
     {
       FsObjectSelector *ctrl= new FsObjectSelector();
@@ -999,10 +1037,28 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
     {
       mforms::Selector *ctrl = new Selector();
       ctrl->set_name(driver_param->get_control_name());
-      std::vector<std::pair<std::string, std::string> > options = driver_param->get_enum_options();
+      std::vector<std::pair<std::string, std::string> > options;
       std::vector<std::string> option_ids;
       std::string value = driver_param->get_value_repr();
       int idx = -1;
+
+      try
+      {
+        options = driver_param->get_enum_options();
+      }
+      catch (std::exception &e)
+      {
+        log_error("Error calling get_enum_options() for param %s: %s", 
+                driver_param->get_control_name().c_str(),
+                e.what());
+        mforms::Utilities::show_error("Connection Setup",
+                                 base::strfmt("An error occurred while retrieving values for option '%s' from '%s'.\n\n%s",
+                                        driver_param->object()->name().c_str(),
+                                        selected_driver()->name().c_str(),
+                                        e.what()),
+                                        "OK", "", "");
+      }
+
       for (size_t i = 0; i < options.size(); i++)
       {
         ctrl->add_item(options[i].second);
