@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,7 @@
 #include <glib.h>
 
 #include "autocomplete_object_name_cache.h"
+#include "base/string_utilities.h"
 #include "base/log.h"
 #include "base/file_utilities.h"
 #include "base/sqlstring.h"
@@ -171,7 +172,7 @@ std::vector<std::string> AutoCompleteCache::get_matching_table_names(const std::
     base::GMutexLock sd_lock(_shutdown_mutex);
     base::GMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM tables WHERE schema LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
-    q.bind(1, base::escape_sql_string(schema, true) +  "%");
+    q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     q.bind(2, base::escape_sql_string(prefix, true) + "%");
     if (q.emit())
     {
@@ -210,8 +211,8 @@ std::vector<std::string> AutoCompleteCache::get_matching_column_names(const std:
     base::GMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM columns WHERE schema LIKE ? ESCAPE '\\' "
       "AND tabl LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
-    q.bind(1, base::escape_sql_string(schema, true) + "%");
-    q.bind(2, base::escape_sql_string(table, true) + "%");
+    q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
+    q.bind(2, table.size() == 0 ? "%" : base::escape_sql_string(table, true));
     q.bind(3, base::escape_sql_string(prefix, true) + "%");
   
     if (q.emit())
@@ -252,7 +253,7 @@ std::vector<std::string> AutoCompleteCache::get_matching_procedure_names(const s
     base::GMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=0");
-    q.bind(1, base::escape_sql_string(schema, true) + "%");
+    q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     q.bind(2, base::escape_sql_string(prefix, true) + "%");
     if (q.emit())
     {
@@ -292,7 +293,7 @@ std::vector<std::string> AutoCompleteCache::get_matching_function_names(const st
     base::GMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=1");
-    q.bind(1, base::escape_sql_string(schema, true) + "%");
+    q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     q.bind(2, base::escape_sql_string(prefix, true) + "%");
     if (q.emit())
     {
@@ -331,7 +332,7 @@ bool AutoCompleteCache::refresh_schema_cache_if_needed(const std::string &schema
     base::GMutexLock sd_lock(_shutdown_mutex);
     base::GMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT last_refresh FROM schemas WHERE name LIKE ? ESCAPE '\\' ");
-    q.bind(1, base::escape_sql_string(schema, true) + "%");
+    q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     if (q.emit())
     {
       boost::shared_ptr<sqlite::result> matches(q.get_result());
@@ -366,7 +367,7 @@ void AutoCompleteCache::refresh_schema_cache(const std::string &schema)
 
 void AutoCompleteCache::refresh_table_cache(const std::string &schema, const std::string &table)
 {
-  add_pending_refresh(schema+"\n"+table);
+  add_pending_refresh(schema + "\n" + table);
 }
 
 
@@ -474,10 +475,10 @@ void AutoCompleteCache::refresh_tables_w(const std::string &schema)
         {
           std::string type = rs->getString(2);
           std::string table = rs->getString(1);
-          tables.push_back(std::make_pair(table, type=="VIEW"));
+          tables.push_back(std::make_pair(table, type == "VIEW"));
 
           // automatically trigger fetch of columns for this table
-          add_pending_refresh(schema+"\n"+table);
+          add_pending_refresh(schema + "\n" + table);
         }
         log_debug2("updating %li tables...\n", tables.size());
       }
