@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include "cppdbc.h"
 #include "grtui/binary_data_editor.h"
 
+#include "mforms/menubar.h"
 #include "mforms/utilities.h"
 #include "mforms/filechooser.h"
 
@@ -81,6 +82,7 @@ static gint next_id = 0;
 Recordset::Recordset(GRTManager *grtm)
   : VarGridModel(grtm), _show_apply_buttons(false), task(GrtThreadedTask::create(grtm))
 {
+  _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
@@ -94,6 +96,7 @@ Recordset::Recordset(GRTManager *grtm)
 Recordset::Recordset(GrtThreadedTask::Ref parent_task)
   : VarGridModel(parent_task->grtm()), _show_apply_buttons(false), task(GrtThreadedTask::create(parent_task))
 {
+  _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
@@ -105,6 +108,7 @@ Recordset::Recordset(GrtThreadedTask::Ref parent_task)
 
 Recordset::~Recordset()
 {
+  delete _context_menu;
 }
 
 
@@ -303,7 +307,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
 
       // insert new empty data record
       {
-        std::list<sqlite::Variant> bind_vars;
+        std::list<sqlite::variant_t> bind_vars;
         bind_vars.push_back((int)rowid);
         emit_partition_commands(data_swap_db.get(), data_swap_db_partition_count(), "insert into `data%s` (id) values (?)", bind_vars);
       }
@@ -334,7 +338,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
     // init new row fields with null-values
     Cell new_cell= _data.begin() + (_data.size() - _column_count);
     for (ColumnId col= 0; _column_count > col; ++col, ++new_cell)
-      *(new_cell)= sqlite::Null();
+      *(new_cell)= sqlite::null_t();
     _data[_data.size() - _column_count + _rowid_column]= (int)rowid;
   }
 
@@ -342,7 +346,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
 }
 
 
-void Recordset::after_set_field(const NodeId &node, int column, const sqlite::Variant &value)
+void Recordset::after_set_field(const NodeId &node, int column, const sqlite::variant_t &value)
 {
   VarGridModel::after_set_field(node, column, value);
   mark_dirty(node[0], column, value);
@@ -351,7 +355,7 @@ void Recordset::after_set_field(const NodeId &node, int column, const sqlite::Va
 }
 
 
-void Recordset::mark_dirty(RowId row, ColumnId column, const sqlite::Variant &new_value)
+void Recordset::mark_dirty(RowId row, ColumnId column, const sqlite::variant_t &new_value)
 {
   GStaticRecMutexLock data_mutex(_data_mutex);
 
@@ -423,8 +427,8 @@ bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
       node[0]-= processed_node_count;
       RowId row= node[0];
 
-      RowId rowid;
-      if (get_field_(node, _rowid_column, (int&)rowid))
+      int rowid;
+      if (get_field_(node, _rowid_column, rowid))
       {
         boost::shared_ptr<sqlite::connection> data_swap_db= this->data_swap_db();
         sqlide::Sqlite_transaction_guarder transaction_guarder(data_swap_db.get());
@@ -435,28 +439,28 @@ bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
           std::string partition_suffix= data_swap_db_partition_suffix(partition);
           sqlite::command save_deleted_data_record_statement(*data_swap_db,
             strfmt("insert into `deleted_rows%s` select * from `data%s` where id=?", partition_suffix.c_str(), partition_suffix.c_str()));
-          save_deleted_data_record_statement % (int)rowid;
+          save_deleted_data_record_statement % rowid;
           save_deleted_data_record_statement.emit();
         }
 
         // delete data record
         {
-          std::list<sqlite::Variant> bind_vars;
-          bind_vars.push_back((int)rowid);
+          std::list<sqlite::variant_t> bind_vars;
+          bind_vars.push_back(rowid);
           emit_partition_commands(data_swap_db.get(), data_swap_db_partition_count(), "delete from `data%s` where id=?", bind_vars);
         }
 
         // delete data index record
         {
           sqlite::command delete_data_index_record_statement(*data_swap_db, "delete from `data_index` where id=?");
-          delete_data_index_record_statement % (int)rowid;
+          delete_data_index_record_statement % rowid;
           delete_data_index_record_statement.emit();
         }
 
         // log delete action
         {
           sqlite::command add_change_record_statement(*data_swap_db, _add_change_record_statement);
-          add_change_record_statement % (int)rowid;
+          add_change_record_statement % rowid;
           add_change_record_statement % -1;
           static sqlite::null_type null_obj;
           add_change_record_statement % null_obj;
@@ -884,8 +888,8 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
         qv.store_unknown_as_string= true;
         qv.allow_func_escaping= false;
       }
-      sqlite::Variant var_string_type= std::string();
-      sqlite::Variant var_string;
+      sqlite::variant_t var_string_type= std::string();
+      sqlite::variant_t var_string;
       std::string sql_string;
 
       // column filters subclause
@@ -1055,7 +1059,7 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
         {
           if (!token.empty() && token[0] == '\'' && token[token.size()-1] == '\'')
             token = token.substr(1, token.size()-2);
-          set_field(dest_row, i, base::unescape_sql_string(token));
+          set_field(dest_row, i, base::unescape_sql_string(token, '\''));
         }
       }
       dest_row++;
@@ -1064,131 +1068,123 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
 }
 
 
-bec::MenuItemList Recordset::get_popup_menu_items(const std::vector<int> &rows, int clicked_column)
+mforms::ContextMenu *Recordset::get_context_menu()
 {
-  bec::MenuItemList items;
-  bec::MenuItem item;
-  bool ro = is_readonly();
-
-  bec::MenuItemList plugin_items;
-  if (popup_menu_items_slot)
-    plugin_items= popup_menu_items_slot(rows, clicked_column);
-
-  item.name = "edit_cell";
-  if (ro)
-    item.caption = "Open Value in Viewer";
-  else
-    item.caption = "Open Value in Editor";
-  item.enabled = (rows.size() > 0) && (clicked_column >= 0);
-  if (item.enabled)
-  {
-    switch (get_real_column_type(clicked_column))
-    {
-    case StringType:
-    case BlobType:
-      break;
-    default:
-      item.enabled= false;
-      break;
-    }
-  }
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-  
-  item.name = "set_to_null";
-  item.caption = "Set Field(s) to NULL";
-#ifdef _WIN32
-  item.enabled = clicked_column >= 0 && !ro;
-#else
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-#endif
-  items.push_back(item);
-
-  item.name = "set_to_function";
-  item.caption = "Mark Field Value as a Function";
-#ifdef _WIN32
-  item.enabled = clicked_column >= 0 && !ro;
-#else
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-#endif
-  items.push_back(item);
-  
-  item.name = "delete_row";
-  item.caption = "Delete Row(s)";
-  item.enabled = rows.size() > 0 && !ro;
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-  
-  item.name = "load_from_file";
-  item.caption = "Load Value From File";
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-  items.push_back(item);
-
-  item.name = "save_to_file";
-  item.caption = "Save Value To File";
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-
-  item.name = "copy_row";
-  item.caption = "Copy Row Content";
-  item.enabled = rows.size() > 0;
-  items.push_back(item);
-
-  item.name = "copy_row_unquoted";
-  item.caption = "Copy Row Content (unquoted)";
-  item.enabled = rows.size() > 0;
-  items.push_back(item);
-
-  item.name = "copy_row_tabsep";
-  item.caption = "Copy Row Content (tab separated)";
-  item.enabled = rows.size() > 0;
-  items.push_back(item);
-  
-  item.name = "copy_field";
-  item.caption = "Copy Field Content";
-  item.enabled = clicked_column >= 0 && rows.size() == 1;
-  items.push_back(item);
-
-  item.name = "copy_field_unquoted";
-  item.caption = "Copy Field Content (unquoted)";
-  item.enabled = clicked_column >= 0 && rows.size() == 1;
-  items.push_back(item);
-
-  item.name = "paste_row";
-  item.caption = "Paste Row";
-  item.enabled = rows.size() <= 1 && !mforms::Utilities::get_clipboard_text().empty() && !ro;
-  items.push_back(item);
-    
-  if (!plugin_items.empty())
-  {
-    item.name = "";
-    item.type = bec::MenuSeparator;
-    item.caption = "";
-    items.push_back(item);
-    
-    items.insert(items.end(), plugin_items.begin(), plugin_items.end());
-  }
-  return items;
+  if (!_context_menu)
+    _context_menu = new mforms::ContextMenu();
+  return _context_menu;
 }
 
 
-bool Recordset::activate_popup_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
+void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clicked_column)
+{
+  _selected_rows = rows;
+  _selected_column = clicked_column;
+
+  if (_context_menu)
+  {
+    _context_menu->remove_all();
+
+    bool ro = is_readonly();
+
+    mforms::MenuItem *item;
+
+    item = _context_menu->add_item_with_title(ro ? "Open Value in Viewer" : "Open Value in Editor",
+                                              boost::bind(&Recordset::activate_menu_item, this, "edit_cell", rows, clicked_column),
+                                              "edit_cell");
+    item->set_enabled((rows.size() > 0) && (clicked_column >= 0));
+    if (item->get_enabled())
+    {
+      switch (get_real_column_type(clicked_column))
+      {
+      case StringType:
+      case BlobType:
+        break;
+      default:
+        item->set_enabled(false);
+        break;
+      }
+    }
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Set Field(s) to NULL",
+                                       boost::bind(&Recordset::activate_menu_item, this, "set_to_null", rows, clicked_column),
+                                       "set_to_null");
+  #ifdef _WIN32
+    item->set_enabled(clicked_column >= 0 && !ro); //XXX why is this?
+  #else
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+  #endif
+
+    item = _context_menu->add_item_with_title("Mark Field Value as a Function",
+                                      boost::bind(&Recordset::activate_menu_item, this, "set_to_function", rows, clicked_column),
+                                      "set_to_function");
+  #ifdef _WIN32
+    item->set_enabled(clicked_column >= 0 && !ro);
+  #else
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+  #endif
+    
+    item = _context_menu->add_item_with_title("Delete Row(s)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "delete_row", rows, clicked_column),
+                                              "delete_row");
+    item->set_enabled(rows.size() > 0 && !ro);
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Load Value From File...",
+                                              boost::bind(&Recordset::activate_menu_item, this, "load_from_file", rows, clicked_column),
+                                              "load_from_file");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+
+    item = _context_menu->add_item_with_title("Save Value To File...",
+                                              boost::bind(&Recordset::activate_menu_item, this, "save_to_file", rows, clicked_column),
+                                              "save_to_file");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Copy Row",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row", rows, clicked_column),
+                                              "copy_row");
+    item->set_enabled(rows.size() > 0);
+    item = _context_menu->add_item_with_title("Copy Row (with names)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_with_names", rows, clicked_column),
+                                              "copy_row_with_names");
+
+    item = _context_menu->add_item_with_title("Copy Row (unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_unquoted", rows, clicked_column),
+                                              "copy_row_unquoted");
+    item->set_enabled(rows.size() > 0);
+    item = _context_menu->add_item_with_title("Copy Row (with names, unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_unquoted_with_names", rows, clicked_column),
+                                              "copy_row_unquoted_with_names");
+
+    item = _context_menu->add_item_with_title("Copy Row (tab separated)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_tabsep", rows, clicked_column),
+                                              "copy_row_tabsep");
+    item->set_enabled(rows.size() > 0);
+
+    item = _context_menu->add_item_with_title("Copy Field",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_field", rows, clicked_column),
+                                              "copy_field");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1);
+
+    item = _context_menu->add_item_with_title("Copy Field (unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_field_unquoted", rows, clicked_column),
+                                              "copy_field_unquoted");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1);
+
+    item = _context_menu->add_item_with_title("Paste Row",
+                                              boost::bind(&Recordset::activate_menu_item, this, "paste_row", rows, clicked_column),
+                                              "paste_row");
+    item->set_enabled(rows.size() <= 1 && !mforms::Utilities::get_clipboard_text().empty() && !ro);
+  }
+}
+
+
+void Recordset::activate_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
 {
   bool result = false;
   bool need_ui_refresh = false;
@@ -1271,6 +1267,11 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
       result = true;
     }
   }
+  else if (action == "copy_row_with_names")
+  {
+    copy_rows_to_clipboard(rows, ", ", true, true);
+    result = true;
+  }
   else if (action == "copy_row_unquoted")
   {
     if (rows.size() > 0)
@@ -1279,6 +1280,11 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
       result = true;
     }
   }
+  else if (action == "copy_row_unquoted_with_names")
+  {
+    copy_rows_to_clipboard(rows, ", ", false, true);
+    result = true;
+  }
   else if (action == "copy_row_tabsep")
   {
     if (rows.size() > 0)
@@ -1308,18 +1314,13 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
     paste_rows_from_clipboard(rows.empty() ? -1 : rows[0]);
     result = true;
   }
-  else
-    if (call_popup_menu_item_slot(action, rows, clicked_column))
-      result = true;
 
   if (need_ui_refresh)
     refresh_ui();
-
-  return result;
 }
 
 
-void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, std::string sep, bool quoted)
+void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, std::string sep, bool quoted, bool with_header)
 {
   ColumnId editable_col_count= get_column_count();
   if (!editable_col_count)
@@ -1334,6 +1335,19 @@ void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, std::str
 
   Cell cell;
   std::string text;
+
+  if (with_header)
+  {
+    text = "# ";
+    for (ColumnId col= 0; editable_col_count > col; ++col)
+    {
+      if (col > 0)
+        text.append(sep);
+      text.append(get_column_caption(col));
+    }
+    text.append("\n");
+  }
+
   BOOST_FOREACH (RowId row, indeces)
   {
     std::string line;
@@ -1377,13 +1391,6 @@ void Recordset::copy_field_to_clipboard(int row, int column, bool quoted)
   mforms::Utilities::set_clipboard_text(text);
 }
 
-
-void Recordset::invoke_export()
-{
-  export_wizard();
-}
-
-
 static void add_toolbar_action_item(bec::ToolbarItemList &items, bec::IconManager *im, const std::string &item_icon, const std::string &item_name, const std::string &item_tooltip)
 {
   bec::ToolbarItem item;
@@ -1484,9 +1491,11 @@ bec::ToolbarItemList Recordset::get_toolbar_items()
     }
   }
   add_toolbar_separator_item(items);
-  add_toolbar_label_item(items, "Export:");
+  add_toolbar_label_item(items, "File:");
   add_toolbar_action_item(items, im, "record_export", "Export recordset to an external file");
-  
+  if (!is_readonly())
+    add_toolbar_action_item(items, im, "record_import", "Import records from an external file");
+
 #ifndef __APPLE__
   add_toolbar_separator_item(items);
   add_toolbar_label_item(items, "Autosize:");
@@ -1528,9 +1537,6 @@ void Recordset::register_default_actions()
 
   _action_list.register_action("record_refresh",
     boost::bind(&Recordset::refresh, this));
-
-  _action_list.register_action("record_export",
-    boost::bind(&Recordset::invoke_export, this));
 }
 
 
@@ -1546,9 +1552,9 @@ private:
   std::string _encoding;
   bool _read_only;
 public:
-  result_type operator()(const sqlite::Null &v) { return new BinaryDataEditor(_grtm, NULL, 0, _encoding, _read_only); }
+  result_type operator()(const sqlite::null_t &v) { return new BinaryDataEditor(_grtm, NULL, 0, _encoding, _read_only); }
   result_type operator()(const std::string &v) { return new BinaryDataEditor(_grtm, v.c_str(), v.length(), _encoding, _read_only); }
-  result_type operator()(const sqlite::BlobRef &v) { return new BinaryDataEditor(_grtm, ((!v || v->empty()) ? NULL : (const char*)&(*v)[0]), v->size(), _encoding, _read_only); }
+  result_type operator()(const sqlite::blob_ref_t &v) { return new BinaryDataEditor(_grtm, ((!v || v->empty()) ? NULL : (const char*)&(*v)[0]), v->size(), _encoding, _read_only); }
   template<typename V> result_type operator()(const V &v) { return NULL; }
 };
 class DataEditorSelector2 : public boost::static_visitor<BinaryDataEditor*>
@@ -1560,7 +1566,7 @@ private:
   bool _read_only;
 public:
   template<typename V> result_type operator()(const std::string &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "UTF-8")(v); }
-  template<typename V> result_type operator()(const sqlite::BlobRef &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "LATIN1")(v); }
+  template<typename V> result_type operator()(const sqlite::blob_ref_t &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "LATIN1")(v); }
   template<typename T, typename V> result_type operator()(const T &r, const V &v) {
     //return NULL;
     // For unknown types treat them for now as string values. Since we have a binary editor pane there that should work
@@ -1575,8 +1581,8 @@ void Recordset::open_field_data_editor(RowId row, ColumnId column)
   bool res= true;
   try
   {
-    sqlite::Variant blob_value;
-    sqlite::Variant *value;
+    sqlite::variant_t blob_value;
+    sqlite::variant_t *value;
 
     if (sqlide::is_var_blob(_real_column_types[column]))
     {
@@ -1613,7 +1619,7 @@ void Recordset::open_field_data_editor(RowId row, ColumnId column)
 }
 
 
-class DataValueConv : public boost::static_visitor<sqlite::Variant>
+class DataValueConv : public boost::static_visitor<sqlite::variant_t>
 {
 public:
   DataValueConv() : _data(NULL), _length(0) {}
@@ -1623,15 +1629,15 @@ private:
   const char *_data;
   size_t _length;
 public:
-  result_type operator()(const sqlite::BlobRef &t)
+  result_type operator()(const sqlite::blob_ref_t &t)
   {
-    sqlite::BlobRef val= sqlite::BlobRef(new sqlite::Blob());
+    sqlite::blob_ref_t val= sqlite::blob_ref_t(new sqlite::blob_t());
     val->resize(_length);
     memcpy(&(*val)[0], _data, _length);
     return val;
   }
   result_type operator()(const std::string &t) { return std::string(_data, _length); }
-  template<typename T> result_type operator()(const T &t) { return sqlite::Unknown(); }
+  template<typename T> result_type operator()(const T &t) { return sqlite::unknown_t(); }
 };
 
 
@@ -1644,7 +1650,7 @@ void Recordset::set_field_value(RowId row, ColumnId column, BinaryDataEditor *da
 void Recordset::set_field_raw_data(RowId row, ColumnId column, const char *data, size_t data_length)
 {
   DataValueConv data_value_conv(data, data_length);
-  sqlite::Variant value= boost::apply_visitor(data_value_conv, _real_column_types[column]);
+  sqlite::variant_t value= boost::apply_visitor(data_value_conv, _real_column_types[column]);
   if (sqlide::is_var_unknown(value))
     throw std::logic_error("Can't save value of this data type.");
   bec::NodeId node(row);
@@ -1694,7 +1700,7 @@ public:
   DataValueDump(const char *filename) : os(filename, std::ios_base::out|std::ios_base::binary) {}
   std::ofstream os;
 public:
-  result_type operator()(const sqlite::BlobRef &v)
+  result_type operator()(const sqlite::blob_ref_t &v)
   {
     std::copy(v->begin(), v->end(), std::ostreambuf_iterator<char>(os));
   }
@@ -1707,8 +1713,8 @@ void Recordset::save_to_file(const bec::NodeId &node, int column, const std::str
 {
   GStaticRecMutexLock data_mutex(_data_mutex);
       
-  sqlite::Variant blob_value;
-  sqlite::Variant *value;
+  sqlite::variant_t blob_value;
+  sqlite::variant_t *value;
   
   if (sqlide::is_var_blob(_real_column_types[column]))
   {
