@@ -38,7 +38,7 @@ protected:
 
 protected:
   virtual void do_unserialize(Recordset *recordset, sqlite::connection *data_swap_db);
-  virtual void do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::Variant &blob_value);
+  virtual void do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::variant_t &blob_value);
 
 protected:
   virtual void run_sql_script(const Sql_script &sql_script);
