@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,9 +33,13 @@
 
 #include "objimpl/db.query/db_query_QueryBuffer.h"
 
+#include "grtui/file_charset_dialog.h"
+
 #include "mforms/code_editor.h"
 #include "mforms/find_panel.h"
+#include "mforms/toolbar.h"
 #include "mforms/menu.h"
+#include "mforms/filechooser.h"
 
 #include "autocomplete_object_name_cache.h"
 
@@ -60,9 +64,12 @@ public:
 
   long _server_version;
 
+  mforms::Box* _container;
   mforms::Menu* _editor_context_menu;
   mforms::Menu* _editor_text_submenu;
 
+  mforms::ToolBar* _toolbar;
+
   int _last_typed_char;
 
   SqlFacade::Ref _sql_facade;
@@ -103,12 +110,14 @@ public:
   bool _is_refresh_enabled;   // whether FE control is permitted to replace its contents from BE
   bool _is_sql_check_enabled; // Enables automatic syntax checks.
   bool _large_content;        // True if the current content exceeds the limit we set.
+  bool _owns_toolbar;
 
   boost::signals2::signal<void ()> _text_change_signal;
 
   Private(db_mgmt_RdbmsRef rdbms)
     : _grtobj(rdbms->get_grt()), _rdbms(rdbms)
   {
+    _owns_toolbar = false;
   }
 
   /**
@@ -142,7 +151,7 @@ Sql_editor::Ref Sql_editor::create(db_mgmt_RdbmsRef rdbms, GrtVersionRef version
     sql_module= dynamic_cast<Sql::Ref>(rdbms->get_grt()->get_module(def_module_name));
   if (sql_module)
     sql_editor= sql_module->getSqlEditor(rdbms, version);
-  
+
   if (sql_editor)
   {
     // replace the default object with the custom one
@@ -184,7 +193,7 @@ Sql_editor::Sql_editor(db_mgmt_RdbmsRef rdbms, GrtVersionRef version)
   d->_current_timer = NULL;
   d->_is_sql_check_enabled = true;
 
-  _code_editor = new mforms::CodeEditor();
+  _code_editor = new mforms::CodeEditor(this);
   _code_editor->set_font(d->_grtm->get_app_option_string("workbench.general.Editor:Font"));
   _code_editor->set_features(mforms::FeatureUsePopup, false);
   _code_editor->set_features(mforms::FeatureConvertEolOnPaste, true);
@@ -194,8 +203,10 @@ Sql_editor::Sql_editor(db_mgmt_RdbmsRef rdbms, GrtVersionRef version)
 
   setup_auto_completion();
 
+  d->_container = NULL;
   d->_editor_text_submenu = NULL;
   d->_editor_context_menu = NULL;
+  d->_toolbar = NULL;
 
   _editor_config = NULL;
   _auto_completion_cache = NULL;
@@ -243,6 +254,8 @@ Sql_editor::~Sql_editor()
   if (d->_editor_text_submenu != NULL)
     delete d->_editor_text_submenu;
   delete d->_editor_context_menu;
+  if (d->_owns_toolbar)
+    delete d->_toolbar;
   delete _code_editor;
 
   // Descendants which set _editor_config are responsible to free it.
@@ -277,6 +290,215 @@ mforms::CodeEditor* Sql_editor::get_editor_control()
 
 //--------------------------------------------------------------------------------------------------
 
+static void toggle_show_special_chars(mforms::ToolBarItem *item, Sql_editor *sql_editor)
+{
+  sql_editor->show_special_chars(item->get_checked());
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void toggle_word_wrap(mforms::ToolBarItem *item, Sql_editor *sql_editor)
+{
+  sql_editor->enable_word_wrap(item->get_checked());
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void show_find_panel_for_active_editor(Sql_editor *sql_editor)
+{
+  sql_editor->get_editor_control()->show_find_panel(false);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void beautify_script(Sql_editor *sql_editor)
+{
+  grt::GRT *grt = sql_editor->grtobj()->get_grt();
+  grt::BaseListRef args(grt);
+  args.ginsert(sql_editor->grtobj());
+
+  grt->call_module_function("SQLIDEUtils", "enbeautificate", args);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void open_file(Sql_editor *sql_editor)
+{
+  mforms::FileChooser fc(mforms::OpenFile);
+  if (fc.run_modal())
+  {
+    std::string file = fc.get_path();
+
+    gchar *contents;
+    gsize length;
+    GError *error = NULL;
+
+    if (g_file_get_contents(file.c_str(), &contents, &length, &error))
+    {
+      std::string converted;
+
+      mforms::CodeEditor* code_editor = sql_editor->get_editor_control();
+      if (FileCharsetDialog::ensure_filedata_utf8(contents, length, "", file, converted))
+      {
+        g_free(contents);
+        code_editor->set_text_keeping_state(converted.c_str());
+      }
+      else
+      {
+        g_free(contents);
+        code_editor->set_text(_("Data is not UTF8 encoded and cannot be displayed."));
+      }
+    }
+    else if (error)
+    {
+      mforms::Utilities::show_error("Load File", base::strfmt("Could not load file %s:\n%s", file.c_str(), error->message),
+                                    "OK");
+      g_error_free(error);
+    }
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void save_file(Sql_editor *sql_editor)
+{
+  mforms::FileChooser fc(mforms::SaveFile);
+  if (fc.run_modal())
+  {
+    GError *error = NULL;
+    std::string file = fc.get_path();
+    mforms::CodeEditor* code_editor = sql_editor->get_editor_control();
+    std::pair<const char*, size_t> data = code_editor->get_text_ptr();
+
+    if (!g_file_set_contents(file.c_str(), data.first, data.second, &error) && error)
+    {
+      mforms::Utilities::show_error("Save File", base::strfmt("Could not save to file %s:\n%s", file.c_str(), error->message),
+                                    "OK");
+      g_error_free(error);
+    }
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Sql_editor::set_base_toolbar(mforms::ToolBar *toolbar)
+{
+  d->_toolbar = toolbar;
+  d->_owns_toolbar = false;
+
+  mforms::ToolBarItem *item;
+
+  if (d->_is_sql_check_enabled)
+  {
+    item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+    item->set_name("query.beautify");
+    item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_beautifier.png"));
+    item->set_tooltip(_("Beautify/reformat the SQL script"));
+    scoped_connect(item->signal_activated(), boost::bind(beautify_script, this));
+    d->_toolbar->add_item(item);
+  }
+  item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+  item->set_name("query.search");
+  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_find.png"));
+  item->set_tooltip(_("Show the Find panel for the editor"));
+  scoped_connect(item->signal_activated(), boost::bind(show_find_panel_for_active_editor, this));
+  d->_toolbar->add_item(item);
+
+  item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
+  item->set_name("query.toggleInvisible");
+  item->set_alt_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_special-chars-on.png"));
+  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_special-chars-off.png"));
+  item->set_tooltip(_("Toggle display of invisible characters (spaces, tabs, newlines)"));
+  scoped_connect(item->signal_activated(),boost::bind(toggle_show_special_chars, item, this));
+  d->_toolbar->add_item(item);
+
+  item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
+  item->set_name("query.toggleWordWrap");
+  item->set_alt_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_word-wrap-on.png"));
+  item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_word-wrap-off.png"));
+  item->set_tooltip(_("Toggle wrapping of long lines (keep this off for large files)"));
+  scoped_connect(item->signal_activated(),boost::bind(toggle_word_wrap, item, this));
+  d->_toolbar->add_item(item);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void embed_find_panel(mforms::CodeEditor *editor, bool show, mforms::Box *container)
+{
+  mforms::View *panel = editor->get_find_panel();
+  if (show)
+  {
+    if (!panel->get_parent())
+      container->add(panel, false, true);
+  }
+  else
+  {
+    container->remove(panel);
+    editor->focus();
+  }
+}
+
+
+mforms::View* Sql_editor::get_container()
+{
+  if (!d->_container)
+  {
+    d->_container = new mforms::Box(false);
+
+    d->_container->add(get_toolbar(), false, true);
+    get_editor_control()->set_show_find_panel_callback(boost::bind(embed_find_panel, _1, _2, d->_container));
+    d->_container->add_end(get_editor_control(), true, true);
+  }
+  return d->_container;
+};
+
+
+//--------------------------------------------------------------------------------------------------
+
+mforms::ToolBar* Sql_editor::get_toolbar(bool include_file_actions)
+{
+  if (!d->_toolbar)
+  {
+    d->_owns_toolbar = true;
+    d->_toolbar = new mforms::ToolBar(mforms::SecondaryToolBar);
+#ifdef _WIN32
+    d->_toolbar->set_size(-1, 27);
+#endif
+    if (include_file_actions)
+    {
+      mforms::ToolBarItem *item;
+
+      item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+      item->set_name("query.openFile");
+      item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_open.png"));
+      item->set_tooltip(_("Open a script file in this editor"));
+      scoped_connect(item->signal_activated(),boost::bind(open_file, this));
+      d->_toolbar->add_item(item);
+
+      item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+      item->set_name("query.saveFile");
+      item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_save.png"));
+      item->set_tooltip(_("Save the script to a file."));
+      scoped_connect(item->signal_activated(),boost::bind(save_file, this));
+      d->_toolbar->add_item(item);
+
+      d->_toolbar->add_item(mforms::manage(new mforms::ToolBarItem(mforms::SeparatorItem)));
+      
+    }
+    set_base_toolbar(d->_toolbar);
+  }
+  return d->_toolbar;
+};
+
+//--------------------------------------------------------------------------------------------------
+
+mforms::CodeEditorConfig* Sql_editor::get_editor_settings()
+{
+  return _editor_config;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 db_mgmt_RdbmsRef Sql_editor::rdbms()
 {
   return d->_rdbms;
@@ -361,7 +583,7 @@ void Sql_editor::append_text(const std::string &text)
   _code_editor->append_text(text.data(), text.size());
 }
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 /**
  * Used to the set the content of the editor from outside (usually when loading a file).
@@ -372,18 +594,51 @@ void Sql_editor::sql(const char *sql)
   _code_editor->set_eol_mode(mforms::EolLF, true);
 }
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 int Sql_editor::cursor_pos()
 {
   return _code_editor->get_caret_pos();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Returns the caret position as column/row pair. The returned column (char index) is utf-8 save and computes
+ * the actual character index as displayed in the editor, not the byte index in a std::string.
+ * If @local is true then the line position is relative to the statement, otherwise that in the entire editor.
+ */
+std::pair<int, int> Sql_editor::cursor_pos_row_column(bool local)
+{
+  int position = _code_editor->get_caret_pos();
+  int line = _code_editor->line_from_position(position);
+  int line_start, line_end;
+  _code_editor->get_range_of_line(line, line_start, line_end);
+
+  int offset = position - line_start; // This is a byte offset.
+  std::string line_text = _code_editor->get_text_in_range(line_start, line_end);
+  offset = g_utf8_pointer_to_offset(line_text.c_str(), line_text.c_str() + offset);
+
+  if (local)
+  {
+    int min = -1;
+    int max = -1;
+    if (get_current_statement_ranges(min, max))
+      line -= _code_editor->line_from_position(min);
+  }
+  
+  return std::make_pair<int, int>(offset, line);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 void Sql_editor::set_cursor_pos(int position)
 {
   _code_editor->set_caret_pos(position);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 bool Sql_editor::selected_range(int& start, int& end)
 {
   int length;
@@ -1023,3 +1278,9 @@ void Sql_editor::stop_processing()
 
 //--------------------------------------------------------------------------------------------------
 
+void Sql_editor::focus()
+{
+  _code_editor->focus();
+}
+
+//--------------------------------------------------------------------------------------------------
