@@ -43,6 +43,7 @@ namespace mforms {
   class CodeEditor;
   class FindPanel;
   class Menu;
+  class View;
   class CodeEditorConfig;
 };
 
@@ -57,6 +58,7 @@ class AutoCompleteCache;
 #define AC_VIEW_IMAGE     6
 #define AC_COLUMN_IMAGE   7
 #define AC_OPERATOR_IMAGE 8
+#define AC_ENGINE_IMAGE   9
 
 /** Base class for SQL Editor Backend classes.
  Subclassed by specific RDBMS support modules.
@@ -79,22 +81,35 @@ public:
     CompletionWantKeywords           = 1 << 1,
     CompletionWantExprStartKeywords  = 1 << 2, // Keywords allowed to begin an expression. Subset of all normal keywords.
     CompletionWantExprInnerKeywords  = 1 << 3, // Like the exp start keywords plus those between two expressions.
-    CompletionWantFunctions          = 1 << 4,
-    CompletionWantRoutines           = 1 << 5,
-    CompletionWantSchemas            = 1 << 6,
-    CompletionWantTables             = 1 << 7,
-    CompletionWantColumns            = 1 << 8,
-    CompletionWantSelect             = 1 << 9, // For sub queries.
+    CompletionWantRuntimeFunctions   = 1 << 4,
+    CompletionWantFunctions          = 1 << 5,
+    CompletionWantProcedures         = 1 << 6,
+    CompletionWantSchemas            = 1 << 7,
+    CompletionWantTables             = 1 << 8, // Tables + Views actually.
+    CompletionWantColumns            = 1 << 9,
+    CompletionWantEvents             = 1 << 10, // TODO
+    CompletionWantTriggers           = 1 << 11, // TODO
+    CompletionWantIndexes            = 1 << 12, // TODO
+    CompletionWantEngines            = 1 << 13,
+    CompletionWantUsers              = 1 << 14, // TODO
+
+    // Keywords we know that must appear.
+    CompletionWantSelect             = 1 << 16, // For sub queries.
+    CompletionWantBy                 = 1 << 17, // After ORDER and GROUP (not after LOGFILE, though).
   };
 
-#define CompletionWantAKeyword (AutoCompletionWantedParts)(CompletionWantMajorKeywords | CompletionWantKeywords |\
-    CompletionWantExprStartKeywords | CompletionWantExprInnerKeywords | CompletionWantSelect)
+#define CompletionWantAKeyword (AutoCompletionWantedParts) \
+  (\
+    CompletionWantMajorKeywords | CompletionWantKeywords | CompletionWantExprStartKeywords |\
+    CompletionWantExprInnerKeywords | CompletionWantSelect | CompletionWantBy\
+  )
 
   // Context structure for code completion results and token info. Located here as we need it also
   // for unit testing. Otherwise it is completely internal.
   struct AutoCompletionContext
   {
     AutoCompletionWantedParts wanted_parts;
+    long version;               // The server version.
 
     bool check_identifier;     // Sometimes we should not try to parse identifiers.
     bool in_table_reference;   // Don't include aliases if we are in a table reference.
@@ -125,6 +140,7 @@ public:
     AutoCompletionContext()
     {
       wanted_parts = CompletionWantMajorKeywords;
+      version = 50510;
       check_identifier = true;
       in_table_reference = false;
       qualified_identifier = false;
@@ -144,18 +160,25 @@ public:
   typedef boost::weak_ptr<Sql_editor> Ptr;
 
   static Ref create(db_mgmt_RdbmsRef rdbms, GrtVersionRef version, db_query_QueryBufferRef grtobj = db_query_QueryBufferRef());
+
   virtual ~Sql_editor();
 
   db_query_QueryBufferRef grtobj();
-  
+
+  void set_base_toolbar(mforms::ToolBar *toolbar);
+
+  mforms::View* get_container();
+  mforms::ToolBar* get_toolbar(bool include_file_actions = true);
   mforms::CodeEditor* get_editor_control();
   mforms::FindPanel* get_find_panel();
-  
+  mforms::CodeEditorConfig* get_editor_settings();
+
   void show_special_chars(bool flag);
   void enable_word_wrap(bool flag);
 
   db_mgmt_RdbmsRef rdbms();
   bec::GRTManager *grtm();
+  std::string sql_mode() { return _sql_mode; };
 
   int int_option(std::string name);
   std::string string_option(std::string name);
@@ -172,6 +195,7 @@ public:
   std::string get_selected_sql();
 
   int cursor_pos();
+  std::pair<int, int> cursor_pos_row_column(bool local);
   void set_cursor_pos(int position);
 
   bool selected_range(int& start, int& end);
@@ -187,7 +211,7 @@ public:
   void cancel_auto_completion();
   void set_auto_completion_cache(AutoCompleteCache *cache);
 
-  void create_auto_completion_list(AutoCompletionContext &context);
+  bool create_auto_completion_list(AutoCompletionContext &context);
 
   std::string selected_text();
   void set_selected_text(const std::string &new_text);
@@ -203,6 +227,8 @@ public:
   void sql_check_progress_msg_throttle(time_t val);
   void stop_processing();
 
+  void focus();
+
 protected:
   Sql_editor(db_mgmt_RdbmsRef rdbms, GrtVersionRef version);
 
