@@ -21,20 +21,24 @@
 
 #include "sqlide/wb_sql_editor_form.h"
 
+#include "base/string_utilities.h"
+#include "workbench/wb_overview.h"
+#include "ConvUtils.h"
+#include "GrtTemplates.h"
 #include "Overview.h"
-#include "FindDialogBE.h"
 #include "Wb.h"
 
 #include "SQLEditorFormWrapper.h"
 
 #include "mforms/menubar.h"
+#include "mforms/appview.h"
 
+using namespace System::Text;
 using namespace System::Threading;
 
 using namespace base;
 
-namespace MySQL {
-namespace Workbench {
+using namespace MySQL::Workbench;
   
 //--------------------------------------------------------------------------------------------------
 
@@ -118,13 +122,6 @@ void WbContext::activate_command(String^ name)
 
 //--------------------------------------------------------------------------------------------------
 
-FindDialogBE^ WbContext::get_find_dialog()
-{
-  return gcnew FindDialogBE(inner->get_find_dialog());
-}
-
-//--------------------------------------------------------------------------------------------------
-
 Overview^ WbContext::get_physical_overview()
 {
   if (physical_overview == nullptr)
@@ -249,12 +246,12 @@ void WbContext::set_active_form_from_appview(MySQL::Forms::AppViewDockContent^ f
 {
   if (form == nullptr)
   {
-    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 2, "Activating appview 0\n");
+    Logger::LogDebug("WBContext managed", 2, "Activating appview 0\n");
     inner->set_active_form(0);
   }
   else
   {
-    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 2, String::Format("Activating appview {0}\n",
+    Logger::LogDebug("WBContext managed", 2, String::Format("Activating appview {0}\n",
       form->GetAppViewIdentifier()));
     inner->set_active_form(form->get_unmanaged_object());
   }
@@ -377,7 +374,7 @@ String^ WbContext::read_option_value(String^ model, String^ key, String^ default
 
 //--------------------------------------------------------------------------------------------------
 
-void WbContext::set_create_main_form_view(MySQL::Forms::AppImpl^ application, VoidStrUIFormDelegate^ dt)
+void WbContext::set_create_main_form_view(MySQL::Workbench::WbFrontendCallbacks^ cbacks, VoidStrUIFormDelegate^ dt)
 {
   create_main_form_view_delegate= dt;
   create_main_form_view_wrapper_delegate=
@@ -385,7 +382,7 @@ void WbContext::set_create_main_form_view(MySQL::Forms::AppImpl^ application, Vo
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(create_main_form_view_wrapper_delegate);
   VOID_STR_UIFORM_CB cb = static_cast<VOID_STR_UIFORM_CB>(ip.ToPointer());
 
-  ::wb::WBFrontendCallbacks* callbacks = application->get_callbacks();
+  ::wb::WBFrontendCallbacks* callbacks = cbacks->get_callbacks();
   callbacks->create_main_form_view= std::ptr_fun(cb);
 }
 
@@ -394,7 +391,7 @@ void WbContext::set_create_main_form_view(MySQL::Forms::AppImpl^ application, Vo
 void WbContext::create_main_form_view_wrapper(const std::string& view_name, boost::shared_ptr<bec::UIForm> form_be)
 {
   String^ name = CppStringToNativeRaw(view_name);
-  Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 1, String::Format("Creating UI wrapper {0}\n", name));
+  Logger::LogDebug("WBContext managed", 1, String::Format("Creating UI wrapper {0}\n", name));
 
   UIForm ^form = nullptr;
 
@@ -409,7 +406,7 @@ void WbContext::create_main_form_view_wrapper(const std::string& view_name, boos
 
 //--------------------------------------------------------------------------------------------------
 
-MenuStrip^ WbContext::menu_for_form(MySQL::Grt::UIForm^ form)
+MenuStrip^ WbContext::menu_for_form(MySQL::Base::UIForm^ form)
 {
   mforms::MenuBar* menu = NULL;
   
@@ -428,7 +425,30 @@ MenuStrip^ WbContext::menu_for_form(MySQL::Grt::UIForm^ form)
 
 //--------------------------------------------------------------------------------------------------
 
-void WbContext::validate_menu_for_form(MySQL::Grt::UIForm^ form)
+/**
+ * Returns the menu strip for the appview that is docked on the given dock content object.
+ * If there's no menu yet a standard one is created.
+ */
+MenuStrip^ WbContext::menu_for_appview(MySQL::Forms::AppViewDockContent ^content)
+{
+  if (content == nullptr)
+    return nullptr;
+
+  MenuStrip ^menuStrip = content->GetAppView()->GetMenuBar();
+  if (menuStrip != nullptr)
+    return menuStrip;
+
+  mforms::MenuBar* menu = inner->get_command_ui()->create_menubar_for_context("");
+  if (menu == NULL)
+    return nullptr;
+
+  content->get_unmanaged_object()->set_menubar(menu);
+  return content->GetAppView()->GetMenuBar();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void WbContext::validate_menu_for_form(MySQL::Base::UIForm^ form)
 {
   mforms::MenuBar* menu = NULL;
 
@@ -440,7 +460,7 @@ void WbContext::validate_menu_for_form(MySQL::Grt::UIForm^ form)
 
 //--------------------------------------------------------------------------------------------------
 
-ToolStrip^ WbContext::toolbar_for_form(MySQL::Grt::UIForm^ form)
+ToolStrip^ WbContext::toolbar_for_form(MySQL::Base::UIForm^ form)
 {
   mforms::ToolBar* toolbar = NULL;
   
@@ -456,10 +476,22 @@ ToolStrip^ WbContext::toolbar_for_form(MySQL::Grt::UIForm^ form)
 
 //--------------------------------------------------------------------------------------------------
 
+Control ^WbContext::shared_secondary_sidebar()
+{
+  mforms::View *sidebar = inner->get_wb()->get_model_context()->shared_secondary_sidebar();
+  if (sidebar == NULL)
+    return nullptr;
+
+  ViewImpl ^sidebar_impl = (ViewImpl^)ViewImpl::FromUnmanaged(sidebar);
+  return sidebar_impl->get_control();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Searches the toolbar of the given form for a search box and makes it active.
  */
-void WbContext::focus_search_box(MySQL::Grt::UIForm^ form)
+void WbContext::focus_search_box(MySQL::Base::UIForm^ form)
 {
   mforms::ToolBar* toolbar = NULL;
 
@@ -468,7 +500,7 @@ void WbContext::focus_search_box(MySQL::Grt::UIForm^ form)
   if (toolbar == NULL)
     return;
 
-  mforms::ToolBarItem* searchbox = toolbar->find_item("searchbox");
+  mforms::ToolBarItem* searchbox = toolbar->find_item("find");
   if (searchbox != NULL)
   {
     ToolBarItemImpl^ item_impl = (ToolBarItemImpl^) ObjectImpl::FromUnmanaged(searchbox);
@@ -483,7 +515,7 @@ void WbContext::focus_search_box(MySQL::Grt::UIForm^ form)
  * Usually used in response of a search action.
  * XXX: once the command ui can pass on search strings this function can go.
  */
-String^ WbContext::get_search_string(MySQL::Grt::UIForm^ form)
+String^ WbContext::get_search_string(MySQL::Base::UIForm^ form)
 {
   mforms::ToolBar* toolbar = NULL;
 
@@ -492,7 +524,7 @@ String^ WbContext::get_search_string(MySQL::Grt::UIForm^ form)
   if (toolbar == NULL)
     return "";
 
-  mforms::ToolBarItem* searchBox = toolbar->find_item("searchbox");
+  mforms::ToolBarItem* searchBox = toolbar->find_item("find");
   if (searchBox != NULL)
     return CppStringToNativeRaw(searchBox->get_text());
 
@@ -546,7 +578,7 @@ void WbContext::flush_idle_tasks()
   catch(...)
   {
     throw; // Don't wrap the exception into something completely useless. The original exception
-    // often is converted automatically into a proper managed exception.
+           // often is converted automatically into a proper managed exception.
   }
 }
 
@@ -565,6 +597,3 @@ void WbContext::flush_timers()
 }
 
 //--------------------------------------------------------------------------------------------------
-
-} // namespace Workbench
-} // namespace MySQL
