@@ -23,19 +23,18 @@
 #include "grt/editor_base.h"
 #include "GrtTemplates.h"
 #include "DelegateWrapper.h"
-#include "UIForm.h"
 #include "GrtManager.h"
 
-using namespace MySQL::Grt;
 using namespace System;
 using namespace System::Collections::Generic;
 
-#pragma make_public(::bec::BaseEditor)
+using namespace MySQL::Base;
+
+#pragma make_public(bec::BaseEditor)
 
 namespace MySQL {
 namespace Grt {
 
-
 public ref class BaseEditor : public UIForm
 {
 protected:
