@@ -4,7 +4,7 @@
  *
  * Created by Mike Lischke.
  *
- * Copyright 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2009, 2011 Sun Microsystems, Inc. All rights reserved.
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
@@ -21,8 +21,6 @@ static NSCursor* waitCursor;
 #define INPUT_INDICATOR INDIC_MAX - 1
 
 NSString *SCIUpdateUINotification = @"SCIUpdateUI";
-NSString *SCIMarginClickNotification = @"SCIMarginClick";
-NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 
 @implementation InnerView
 
@@ -218,8 +216,10 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 
 - (void) doCommandBySelector: (SEL) selector
 {
- // if ([self respondsToSelector: selector])
-//    [self performSelector: selector withObject: nil];
+  // Dispatch calls to selectors like the super class does, but don't play a sound if a specific
+  // selector is not implemented.
+  if ([self respondsToSelector: @selector(selector)])
+    [self performSelector: selector withObject: nil];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -304,6 +304,7 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
   }
 
   // Keep Scintilla from collecting undo actions for the composition task.
+  undoCollectionWasActive = [mOwner getGeneralProperty: SCI_GETUNDOCOLLECTION] != 0;
   [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 0];
   
   // Note: Scintilla internally works almost always with bytes instead chars, so we need to take
@@ -323,9 +324,12 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
                          value: mMarkedTextRange.length];
   }
   else
+  {
     // Re-enable undo action collection if composition ended (indicated by an empty mark string).
-    [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: range.length == 0];
-  
+    if (undoCollectionWasActive)
+      [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: range.length == 0];
+  }
+
   // Select the part which is indicated in the given range. It does not scroll the caret into view.
   if (range.length > 0)
   {
@@ -349,7 +353,8 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
     mMarkedTextRange = NSMakeRange(NSNotFound, 0);
 
     // Reenable undo action collection, after we are done with text composition.    
-    [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 1];
+    if (undoCollectionWasActive)
+      [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 1];
   }
 }
 
@@ -371,7 +376,8 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
     mMarkedTextRange = NSMakeRange(NSNotFound, 0);
 
     // Reenable undo action collection, after we are done with text composition.    
-    [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 1];
+    if (undoCollectionWasActive)
+      [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 1];
   }
 }
 
@@ -782,20 +788,12 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
       {
         case SCN_MARGINCLICK:
         {
-          long line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
           if (scn->margin == 2)
           {
             // Click on the folder margin. Toggle the current line if possible.
+            long line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
             [editor setGeneralProperty: SCI_TOGGLEFOLD value: line];
           }
-          
-          // Notification to handle special events in editors
-          // e.g. like setting breakpoints or do other actions when the gutter is clicked.
-          NSMutableDictionary* info = [NSMutableDictionary dictionary];
-          [info setValue: [NSNumber numberWithInt: line] forKey: @"line"];
-          [info setValue: [NSNumber numberWithInt: scn->modifiers] forKey: @"modifiers"];
-          [info setValue: [NSNumber numberWithInt: scn->margin] forKey: @"margin"];
-          [editor sendNotification: SCIMarginClickNotification withUserInfo: info];
           break;
         };
         case SCN_MODIFIED:
@@ -846,9 +844,9 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
         case SCEN_KILLFOCUS:
           [editor sendNotification: NSTextDidEndEditingNotification];
           break;
-        case SCEN_SETFOCUS: // Nothing to do for now.
-          break;
       }
+      if (editor.delegate != nil)
+        [editor.delegate command: wParam >> 16];
       break;
     }
   };
@@ -1595,6 +1593,25 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * For backwards compatibility.
+ */
+- (BOOL) findAndHighlightText: (NSString*) searchText
+                    matchCase: (BOOL) matchCase
+                    wholeWord: (BOOL) wholeWord
+                     scrollTo: (BOOL) scrollTo
+                         wrap: (BOOL) wrap
+{
+  return [self findAndHighlightText: searchText
+                          matchCase: matchCase
+                          wholeWord: wholeWord
+                           scrollTo: scrollTo
+                               wrap: wrap
+                          backwards: NO];
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Searches and marks the first occurance of the given text and optionally scrolls it into view.
  *
@@ -1671,7 +1688,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   }
   else
   {
-    // Restore the former selection if we did not found anything.
+    // Restore the former selection if we did not find anything.
     [self setGeneralProperty: SCI_SETSELECTIONSTART value: selectionStart];
     [self setGeneralProperty: SCI_SETSELECTIONEND value: selectionEnd];
   }
