@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,14 +23,17 @@
 #include "base/string_utilities.h"
 #include "base/wb_iterators.h"
 #include "base/notifications.h"
+#include "base/log.h"
 #include <errno.h>
 
+DEFAULT_LOG_DOMAIN("grtshell")
+
 #include "grt/common.h"
 #include "grt_code_editor.h"
 #include "grt_plugin_wizard.h"
 #include "grt_python_debugger.h"
 
-#include "wb_context.h"
+#include "workbench/wb_context.h"
 #include "mforms/app.h"
 #include "mforms/imagebox.h"
 
@@ -64,7 +67,7 @@ _lower_header_panel(mforms::FilledHeaderPanel),
 _lower_tab(mforms::TabViewDocument),
 #endif
   _output_text(mforms::VerticalScrollBar),
-  _snippet_splitter(false), _snippet_page(false),
+  _snippet_splitter(false),
   _snippet_text()
 {
   set_title(("Workbench Scripting Shell"));
@@ -175,12 +178,10 @@ _lower_tab(mforms::TabViewDocument),
   _files_tree = mforms::manage(new mforms::TreeNodeView(mforms::TreeDefault|mforms::TreeNoHeader));
 #endif
 
-  _files_menu.add_item("Add New File", "file-from-template");
-  _files_menu.add_item("Open Script File", "open-script");
+  _files_menu.add_item_with_title("Add New File", boost::bind(&GRTShellWindow::file_menu_activate, this, "file-from-template"));
+  _files_menu.add_item_with_title("Open Script File", boost::bind(&GRTShellWindow::file_menu_activate, this, "open-script"));
   _files_menu.add_separator();
-  _files_menu.add_item("Delete Script", "delete-script");
-
-  _files_menu.set_handler(boost::bind(&GRTShellWindow::file_menu_activate, this, _1));
+  _files_menu.add_item_with_title("Delete Script", boost::bind(&GRTShellWindow::file_menu_activate, this, "delete-script"));
 
   _files_tree->set_context_menu(&_files_menu);
 
@@ -228,11 +229,10 @@ _lower_tab(mforms::TabViewDocument),
   scoped_connect(_global_combo.signal_changed(),boost::bind(&GRTShellWindow::refresh_globals_tree, this));
   _inspector= 0;
 
-  _global_menu.add_item(_("Copy Value"), "copy_value");
-  _global_menu.add_item(_("Copy Path"), "copy_path");
-  _global_menu.add_item(_("Copy Path for Python"), "copy_path_py");
-  _global_menu.set_handler(boost::bind(&GRTShellWindow::handle_global_menu, this, _1));
-  
+  _global_menu.add_item_with_title(_("Copy Value"), boost::bind(&GRTShellWindow::handle_global_menu, this, "copy_value"));
+  _global_menu.add_item_with_title(_("Copy Path"), boost::bind(&GRTShellWindow::handle_global_menu, this, "copy_path"));
+  _global_menu.add_item_with_title(_("Copy Path for Python"), boost::bind(&GRTShellWindow::handle_global_menu, this, "copy_path_py"));
+
   _global_tree.set_context_menu(&_global_menu);
   // 2) Classes tree
 #ifdef _WIN32
@@ -279,7 +279,7 @@ _lower_tab(mforms::TabViewDocument),
   _notifs_splitter.set_back_color("#FFFFFF");
   //_modules_splitter.set_padding(0, 0, 0, 2); TODO: might require workaround.
 #endif
-//  _side_tab.add_page(&_notifs_splitter, "Notifications");
+  _side_tab.add_page(&_notifs_splitter, "Notifications");
   _notifs_splitter.add(&_notifs_tree, 0);
   _notifs_splitter.add(&_notifs_text, 0);
   _notifs_text.set_read_only(true);
@@ -320,27 +320,21 @@ _lower_tab(mforms::TabViewDocument),
 
   // snippets tab
 #ifdef _WIN32
-//  ((View*)&_snippet_page)->set_padding(Padding(0, 0, 0, 21));
-  _snippet_page.set_back_color("#283752");
   _snippet_splitter.set_back_color("#283752");
 #else
-  _snippet_page.set_spacing(8);
-  _snippet_page.set_padding(8);
+  _snippet_splitter.set_padding(8);
 #endif
-  _snippet_page.add(&_snippet_splitter, true, true);
 
   //_snippet_box.set_spacing(8);
-  _snippet_splitter.add(_snippet_list);
-  _snippet_splitter.add(&_snippet_text);
+  _snippet_splitter.add(_snippet_list, 50, true);
+  _snippet_splitter.add(&_snippet_text, 50, false);
 
-  _snippet_menu.add_item("Execute Snippet", "execute");
-  _snippet_menu.add_item("Send to Script Editor", "new_with_snippet");
+  _snippet_menu.add_item_with_title("Execute Snippet", boost::bind(&GRTShellWindow::snippet_menu_activate, this, "execute"));
+  _snippet_menu.add_item_with_title("Send to Script Editor", boost::bind(&GRTShellWindow::snippet_menu_activate, this, "new_with_snippet"));
   _snippet_menu.add_separator();
-  _snippet_menu.add_item("Copy to Clipboard", "copy_clipboard");
+  _snippet_menu.add_item_with_title("Copy to Clipboard", boost::bind(&GRTShellWindow::snippet_menu_activate, this, "copy_clipboard"));
   _snippet_menu.add_separator();
-  _snippet_menu.add_item("Delete Snippet", "delete");
-
-  _snippet_menu.set_handler(boost::bind(&GRTShellWindow::snippet_menu_activate, this, _1));
+  _snippet_menu.add_item_with_title("Delete Snippet", boost::bind(&GRTShellWindow::snippet_menu_activate, this, "delete"));
 
   _snippet_list->set_context_menu(&_snippet_menu);
 
@@ -351,7 +345,7 @@ _lower_tab(mforms::TabViewDocument),
 
   _snippet_list->add_column(mforms::StringColumnType, "Snippet", 500, false);
   _snippet_list->end_columns();
-  _main_tab.add_page(&_snippet_page, "Snippets");
+  _main_tab.add_page(&_snippet_splitter, "Snippets");
 
   scoped_connect(_main_tab.signal_tab_closing(),boost::bind(&GRTShellWindow::on_tab_closing, this, _1));
   scoped_connect(_main_tab.signal_tab_closed(),boost::bind(&GRTShellWindow::on_tab_closed, this, _1));
@@ -378,13 +372,9 @@ _lower_tab(mforms::TabViewDocument),
   
   // Minimum size for the entire window.
   set_size(800, 600);
-  _snippet_splitter.set_position(400);
-  _hsplitter.set_position(250);
-  _global_splitter.set_position(400);
-  _modules_splitter.set_position(400);
-  _classes_splitter.set_position(400);
-  _notifs_splitter.set_position(400);
-  
+
+  grtm()->run_once_when_idle(boost::bind(&GRTShellWindow::set_splitter_positions, this));
+
   grtm()->get_shell()->set_ready_handler(boost::bind(&GRTShellWindow::handle_prompt, this, _1));
   grtm()->get_shell()->set_output_handler(boost::bind(&GRTShellWindow::handle_output, this, _1));
 
@@ -393,6 +383,17 @@ _lower_tab(mforms::TabViewDocument),
   side_tab_changed();  
 }
 
+
+void GRTShellWindow::set_splitter_positions()
+{
+  _hsplitter.set_position(300);
+  _global_splitter.set_position(400);
+  _modules_splitter.set_position(400);
+  _classes_splitter.set_position(400);
+  _notifs_splitter.set_position(400);
+  _snippet_splitter.set_position(200);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 GRTManager *GRTShellWindow::grtm()
@@ -681,21 +682,28 @@ void GRTShellWindow::global_selected()
   }
   
   mforms::TreeNodeRef selected;
-  
-  if ((selected = _global_tree.get_selected_node()))
+
+  try
   {
-    grt::ValueRef value(get_global_at_node(selected));
-    
-    if (value.is_valid())
+    if ((selected = _global_tree.get_selected_node()))
     {
-      _inspector= grtm()->get_new_value_inspector(value, false);
-      refresh_global_list();
+      grt::ValueRef value(get_global_at_node(selected));
+      
+      if (value.is_valid())
+      {
+        _inspector= ValueInspectorBE::create(grtm()->get_grt(), value, false, false);
+        refresh_global_list();
+      }
+      
+      _global_entry.set_value(get_global_path_at_node(selected));
     }
-    
-    _global_entry.set_value(get_global_path_at_node(selected));
+    else
+      _global_entry.set_value("");
+  }
+  catch (std::exception &exc)
+  {
+    log_error("Exception when selecting item in globals tree: %s\n", exc.what());
   }
-  else
-    _global_entry.set_value("");
 }
 
 
@@ -998,7 +1006,7 @@ void GRTShellWindow::snippet_selected()
     read_only = true;
     
     for (int i= 0; i < 6; i++)
-      _snippet_menu.set_item_enabled(i, false);    
+      _snippet_menu.get_item(i)->set_enabled(false);
   }
   else
   {
@@ -1010,9 +1018,9 @@ void GRTShellWindow::snippet_selected()
       for (int i= 0; i < 6; i++)
       {
         if (i != 5)
-          _snippet_menu.set_item_enabled(i, true);
+          _snippet_menu.get_item(i)->set_enabled(true);
         else
-          _snippet_menu.set_item_enabled(i, false); // 5 is delete
+          _snippet_menu.get_item(i)->set_enabled(false); // 5 is delete
       }
     }
     else
@@ -1020,7 +1028,7 @@ void GRTShellWindow::snippet_selected()
       _snippet_delete_button->set_enabled(true);
       
       for (int i= 0; i < 6; i++)
-        _snippet_menu.set_item_enabled(i, true);
+        _snippet_menu.get_item(i)->set_enabled(true);
     }
     mforms::TreeNodeRef node(_snippet_list->get_selected_node());
     if (node)
