@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,7 @@
 #include "stdafx.h"
 #include "plugin_manager_window.h"
 #include "grt/icon_manager.h"
+#include "base/file_utilities.h"
 #include "base/string_utilities.h"
 #include "workbench/wb_context.h"
 
@@ -26,7 +27,7 @@ using namespace mforms;
 using namespace grt;
 
 PluginManagerWindow::PluginManagerWindow(wb::WBContext *wb)
-: Form(0), _wb(wb), _grtm(wb->get_grt_manager()), plugin_list(TreeDefault), 
+: Form(0), _wb(wb), _grtm(wb->get_grt_manager()), plugin_list(TreeDefault|TreeFlatList), 
 plugin_info_panel(LineBorderPanel), plugin_info_box(false), plugin_details_box(false)
 {
   Box *top = manage(new Box(false));
@@ -44,7 +45,7 @@ plugin_info_panel(LineBorderPanel), plugin_info_box(false), plugin_details_box(f
   middle->add(&plugin_info_panel, true, true);
   
   
-  plugin_list.signal_changed().connect(sigc::mem_fun(this, &PluginManagerWindow::list_selection_changed));
+  scoped_connect(plugin_list.signal_changed(),boost::bind(&PluginManagerWindow::list_selection_changed, this));
 
   plugin_info_panel.set_back_color("#ffffff");
   plugin_info_panel.add(&plugin_info_box);
@@ -98,9 +99,9 @@ plugin_info_panel(LineBorderPanel), plugin_info_box(false), plugin_details_box(f
     vbox->add_end(hbox, false, true);
     
     plugin_enabled.set_text("Enable Plugin");
-    plugin_enabled.signal_clicked().connect(sigc::mem_fun(this, &PluginManagerWindow::toggle_enable));
+    scoped_connect(plugin_enabled.signal_clicked(),boost::bind(&PluginManagerWindow::toggle_enable, this));
     plugin_uninstall.set_text("Uninstall");
-    plugin_uninstall.signal_clicked().connect(sigc::mem_fun(this, &PluginManagerWindow::uninstall));
+    scoped_connect(plugin_uninstall.signal_clicked(),boost::bind(&PluginManagerWindow::uninstall, this));
     
     hbox->add(&plugin_enabled, true, true);
     hbox->add(&plugin_uninstall, false, true);
@@ -120,7 +121,7 @@ plugin_info_panel(LineBorderPanel), plugin_info_box(false), plugin_details_box(f
   
   plugin_list.set_row_height(24);
   
-  plugin_show_details.signal_clicked().connect(sigc::mem_fun(this, &PluginManagerWindow::toggle_show_details));
+  scoped_connect(plugin_show_details.signal_clicked(),boost::bind(&PluginManagerWindow::toggle_show_details, this));
   toggle_show_details();
 
   list_selection_changed();
@@ -154,7 +155,7 @@ void PluginManagerWindow::refresh_plugin_list()
   std::string user_plugin_dir= _grtm->get_user_module_path();
   ListRef<app_Plugin> plugins(ListRef<app_Plugin>::cast_from(grt->get("/wb/registry/plugins")));
 
-  plugin_list.clear_rows();
+  plugin_list.clear();
   for (ListRef<app_Plugin>::const_iterator p= plugins.begin(); p != plugins.end(); ++p)
   {
     _module_plugins[(*p)->moduleName()].push_back((*p)->name());
@@ -165,7 +166,7 @@ void PluginManagerWindow::refresh_plugin_list()
       std::string path= module->path();
       if (path.compare(0, user_plugin_dir.size(), user_plugin_dir) == 0)
       {
-        int row= plugin_list.add_row();
+        TreeNodeRef node= plugin_list.add_node();
         
         std::string icon_path;
         if (module->is_bundle())
@@ -174,9 +175,9 @@ void PluginManagerWindow::refresh_plugin_list()
         if (icon_path.empty() || !g_file_test(icon_path.c_str(), G_FILE_TEST_IS_REGULAR))
           icon_path = bec::IconManager::get_instance()->get_icon_path("MySQLPlugin-48.png");
         
-        plugin_list.set(row, 0, icon_path);
-        plugin_list.set(row, 1, (*p)->caption());
-        plugin_list.set_row_tag(row, (*p)->name());
+        node->set_icon_path(0, icon_path);
+        node->set_string(1, (*p)->caption());
+        node->set_tag((*p)->name());
       }
     }
   }
@@ -185,12 +186,12 @@ void PluginManagerWindow::refresh_plugin_list()
 
 void PluginManagerWindow::list_selection_changed()
 {
-  int row= plugin_list.get_selected();
+  TreeNodeRef node= plugin_list.get_selected_node();
   app_PluginRef plugin;
-  if (row >= 0)
+  if (node)
   {
     std::string plugin_name;
-    plugin_name= plugin_list.get_row_tag(row);
+    plugin_name= node->get_tag();
     plugin= _grtm->get_plugin_manager()->get_plugin(plugin_name);
   }
   
@@ -204,7 +205,7 @@ void PluginManagerWindow::list_selection_changed()
     if (module)
     {
       std::string path= module->is_bundle() ? module->bundle_path() : module->path();
-      plugin_path.set_text(std::string("File Name: ").append(g_basename(path.c_str())));
+      plugin_path.set_text(std::string("File Name: ").append(base::basename(path)));
       plugin_path.set_tooltip(path);
       plugin_function.set_text(std::string("Implemented by: ").append(plugin->moduleName()).append(".").append(plugin->moduleFunctionName()));
 
@@ -250,12 +251,12 @@ void PluginManagerWindow::toggle_show_details()
 
 void PluginManagerWindow::uninstall()
 {
-  int row= plugin_list.get_selected();
+  TreeNodeRef node= plugin_list.get_selected_node();
   app_PluginRef plugin;
-  if (row >= 0)
+  if (node)
   {
     std::string plugin_name;
-    plugin_name= plugin_list.get_row_tag(row);
+    plugin_name= node->get_tag();
     plugin= _grtm->get_plugin_manager()->get_plugin(plugin_name);
     if (plugin.is_valid())
     {
@@ -310,11 +311,11 @@ void PluginManagerWindow::uninstall()
 
 void PluginManagerWindow::toggle_enable()
 {
-  int row= plugin_list.get_selected();
-  if (row >= 0)
+  TreeNodeRef node= plugin_list.get_selected_node();
+  if (node)
   {
     std::string plugin_name;
-    plugin_name= plugin_list.get_row_tag(row);
+    plugin_name= node->get_tag();
     app_PluginRef plugin(_grtm->get_plugin_manager()->get_plugin(plugin_name));
     if (plugin.is_valid() && _grtm->get_plugin_manager()->plugin_enabled(plugin_name) != plugin_enabled.get_active())
     {
