@@ -19,16 +19,33 @@
 
 #include "stdafx.h"
 
-#include "preferences_form.h"
-#include "mforms/widgets.h"
-#include "mforms/sectionbox.h"
-
+#include "base/ui_form.h"
 #include "base/string_utilities.h"
 #include "base/util_functions.h"
 
+#include "mforms/widgets.h"
+#include "mforms/sectionbox.h"
+#include "mforms/textbox.h"
+#include "mforms/panel.h"
+#include "mforms/radiobutton.h"
+
+#include "grtpp.h"
+
+#include "grts/structs.h"
+#include "grts/structs.app.h"
+#include "grts/structs.workbench.physical.h"
+
+#include "grt/editor_base.h"
+
+#include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+
+#include "preferences_form.h"
+
 #include "grtdb/db_object_helpers.h"
 #include "snippet_popover.h"
-#include "mforms/textbox.h"
+
+#include "grtpp_notifications.h"
 
 #if defined(_WIN32) || defined(__APPLE__)
 #define HAVE_BUNDLED_MYSQLDUMP
@@ -37,6 +54,117 @@
 using namespace base;
 using namespace mforms;
 
+#include "base/drawing.h"
+
+struct LangFontSet {
+  const char *name;
+
+  // TODO: there are a few fonts missing here.
+  const char *object_title_font;
+  const char *object_section_font;
+  const char *object_item_font;
+  const char *layer_title_font;
+  const char *note_font;
+};
+
+static LangFontSet font_sets[] = {
+  {"Default (Western)",
+    DEFAULT_FONT_FAMILY" Bold 12",
+    DEFAULT_FONT_FAMILY" Bold 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11",
+  },
+#ifdef _WIN32
+  {"Japanese",
+    "Arial Unicode MS Bold 12",
+    "Arial Unicode MS Bold 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11"
+  },
+  {"Korean",
+    "Arial Unicode MS Bold 12",
+    "Arial Unicode MS Bold 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11"
+  },
+  {"Simplified Chinese",
+    "Arial Unicode MS Bold 12",
+    "Arial Unicode MS Bold 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11",
+    "Arial Unicode MS 11"
+  },
+  {"Cyrillic",
+    DEFAULT_FONT_FAMILY" Bold 12",
+    DEFAULT_FONT_FAMILY" Bold 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11"
+  },
+#elif defined(__APPLE__)
+  {"Japanese",
+    "Osaka Bold 12",
+    "Osaka Bold 11",
+    "Osaka 11",
+    "Osaka 11",
+    "Osaka 11"
+  },
+  {"Korean",
+    "AppleGothic Bold 12",
+    "AppleGothic Bold 11",
+    "AppleGothic 11",
+    "AppleGothic 11",
+    "AppleGothic 11"
+  },
+  {"Simplified Chinese",
+    "SimHei Bold 12",
+    "SimHei Bold 11",
+    "SimHei 11",
+    "SimHei 11",
+    "SimHei 11"
+  },
+  {"Cyrillic",
+    DEFAULT_FONT_FAMILY" Bold 12",
+    DEFAULT_FONT_FAMILY" Bold 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11"
+  },
+#else
+  {"Japanese",
+    "VL Gothic Bold 12",
+    "VL Gothic Bold 11",
+    "VL Gothic 11",
+    "VL Gothic 11",
+    "VL Gothic 11"
+  },
+  {"Korean",
+    "WenQuanYi Zen Hei Bold 12",
+    "WenQuanYi Zen Hei Bold 11",
+    "WenQuanYi Zen Hei 11",
+    "WenQuanYi Zen Hei 11",
+    "WenQuanYi Zen Hei 11"
+  },
+  {"Simplified Chinese",
+    "WenQuanYi Zen Hei Bold 12",
+    "WenQuanYi Zen Hei Bold 11",
+    "WenQuanYi Zen Hei 11",
+    "WenQuanYi Zen Hei 11",
+    "WenQuanYi Zen Hei 11"
+  },
+  {"Cyrillic",
+    DEFAULT_FONT_FAMILY" Bold 12",
+    DEFAULT_FONT_FAMILY" Bold 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11",
+    DEFAULT_FONT_FAMILY" 11"
+  },
+#endif
+  {NULL}
+};
 
 static mforms::Label *new_label(const std::string &text, bool right_align=false, bool help=false)
 {
@@ -117,7 +245,7 @@ public:
     label->set_size(170, -1);
     _table.add(entry, 1, 2, _rows-1, _rows, mforms::HFillFlag);
   }
-  
+
   mforms::CheckBox *add_checkbox_option(const std::string &option, const std::string &caption, const std::string &tooltip)
   {
     _table.set_row_count(++_rows);
@@ -137,14 +265,14 @@ public:
   }
 };
 
-
+//----------------- PreferencesForm ----------------------------------------------------------------
 
 PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical_ModelRef &model)
   : Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), 
   _tabview(mforms::TabViewDocument), _button_box(true), _font_list(mforms::TreeFlatList)
 {
-  _wbui= wbui;
-  _model= model;
+  _wbui = wbui;
+  _model = model;
 
   if (!model.is_valid())
     set_title(_("Workbench Preferences"));
@@ -153,7 +281,7 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
 
   set_content(&_top_box);
 #ifdef _WIN32
-  set_back_color("#283752");
+  set_back_color(base::Color::get_application_color_as_string(base::AppColorMainBackground, false));
 #endif
 
   _top_box.set_padding(4);
@@ -183,7 +311,10 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
   {
     _use_global.set_text(_("Use defaults from global settings"));
 #ifdef _WIN32
-    _use_global.set_front_color("#FFFFFF");
+    if (base::Color::get_active_scheme() == ColorSchemeStandardWin7)
+      _use_global.set_front_color("#FFFFFF");
+    else
+      _use_global.set_front_color("#000000");
 #endif
     _bottom_box.add(&_use_global, true, true);
     scoped_connect(_use_global.signal_clicked(),boost::bind(&PreferencesForm::toggle_use_global, this));
@@ -200,11 +331,22 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
   create_mysql_page();
   create_diagram_page();
   if (!_model.is_valid())
+  {
     create_appearance_page();
-
-#if defined(_DEBUG) || defined(ENABLE_DEBUG)
-  create_test_page();
+#ifdef _WIN32
+    create_color_scheme_page();
 #endif
+  }
+
+  grt::DictRef info(wbui->get_wb()->get_grt());
+  if (!_model.is_valid())
+    info.set("options", _wbui->get_wb()->get_wb_options());
+  else
+  {
+    info.set("model-options", _wbui->get_model_options(_model.id()));
+    info.set("model", model);
+  }
+  grt::GRTNotificationCenter::get()->send_grt("GRNPreferencesDidCreate", grt::ObjectRef(), info);
 
   set_size(700, 680);
   center();
@@ -212,10 +354,34 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
   show_values();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+PreferencesForm::~PreferencesForm()
+{
+  for (std::list<Option*>::iterator iter= _options.begin(); iter != _options.end(); ++iter)
+    delete *iter;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::show()
 {
-  run_modal(&_ok_button, &_cancel_button);
+  grt::DictRef info(_wbui->get_wb()->get_grt());
+  if (!_model.is_valid())
+    info.set("options", _wbui->get_wb()->get_wb_options());
+  else
+  {
+    info.set("model-options", _wbui->get_model_options(_model.id()));
+    info.set("model", _model);
+  }
+  grt::GRTNotificationCenter::get()->send_grt("GRNPreferencesWillOpen", grt::ObjectRef(), info);
+  
+  if (run_modal(&_ok_button, &_cancel_button))
+    info.set("saved", grt::IntegerRef(1));
+  else
+    info.set("saved", grt::IntegerRef(0));
+  
+  grt::GRTNotificationCenter::get()->send_grt("GRNPreferencesDidClose", grt::ObjectRef(), info);
 }
 
 
@@ -258,6 +424,7 @@ void PreferencesForm::update_values()
       (*iter)->update_value();
     }
   }
+
   if (!_model.is_valid())
     update_colors_and_fonts();
 
@@ -368,6 +535,12 @@ void PreferencesForm::update_selector_option(const std::string &option_name, mfo
     else
       _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, choices[selector->get_selected_index()]);
   }
+
+  if (option_name == "ColorScheme")
+  {
+    base::Color::set_active_scheme((base::ColorScheme)selector->get_selected_index());
+    NotificationCenter::get()->send("GNColorsChanged", NULL);
+  }
 }
 
 
@@ -463,15 +636,6 @@ mforms::Selector *PreferencesForm::new_selector_option(const std::string &option
   return selector;
 }
 
-
-
-PreferencesForm::~PreferencesForm()
-{
-  for (std::list<Option*>::iterator iter= _options.begin(); iter != _options.end(); ++iter)
-    delete *iter;
-}
-
-
 //--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::ok_clicked()
@@ -565,21 +729,30 @@ void PreferencesForm::create_sqlide_page()
     frame->add(vbox);
     
     {
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowSchemaTreeSchemaContents");
+      mforms::CheckBox *check = new_checkbox_option("DbSqlEditor:ShowSchemaTreeSchemaContents");
       check->set_text(_("Show Schema Contents in Schema Tree"));
-      check->set_tooltip(_(
-                           "Whether to show schema contents (tables, views and routine names) in schema tree."));
+      check->set_tooltip(_("Whether to show schema contents (tables, views and routine names) in "
+        "schema tree."));
       vbox->add(check, false);
     }
     
     {
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowMetadataSchemata");
-      check->set_text(_("Show Metadata Schemata"));
-      check->set_tooltip(_(
-                           "Whether to show metadata/internal schemata in schema tree (eg INFORMATION_SCHEMA, mysql and schemas starting with '.')."));
+      mforms::CheckBox *check = new_checkbox_option("DbSqlEditor:ShowMetadataSchemata");
+      check->set_text(_("Show Data Dictionaries and Internal Schemas"));
+      check->set_tooltip(_("Whether to show data dictionaries/internal schemas in the schema tree "
+        "(eg INFORMATION_SCHEMA, mysql and schemas starting with '.')."));
       vbox->add(check, false);
     }
     
+    {
+      mforms::CheckBox *check = new_checkbox_option("DbSqlEditor:SidebarModeCombined");
+      check->set_text(_("Show Management Tools and Schema Tree in a single tab"));
+      check->set_tooltip(_("Check this if you want to display the management tools and the "
+        "schema list in the same tab page in the sidebar. Uncheck it to have them "
+	"in separate tab pages."));
+      vbox->add(check, false);
+    }
+
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -587,7 +760,6 @@ void PreferencesForm::create_sqlide_page()
       
       tbox->add(new_label(_("DBMS connection keep-alive interval (in seconds):"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("DbSqlEditor:KeepAliveInterval", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
                            "Time interval between sending keep-alive messages to DBMS.\n"
@@ -602,10 +774,8 @@ void PreferencesForm::create_sqlide_page()
       
       tbox->add(new_label(_("DBMS connection read time out (in seconds):"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("DbSqlEditor:ReadTimeOut", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
-      entry->set_tooltip(_(
-                           "Max time the a query can take to return data from the DBMS"));
+      entry->set_tooltip(_("Max time the a query can take to return data from the DBMS"));
       tbox->add(entry, false, false);
     }
   }
@@ -664,7 +834,6 @@ void PreferencesForm::create_sqlide_page()
       
       tbox->add(new_label(_("Max syntax error count:"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("SqlEditor::SyntaxCheck::MaxErrCount", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_("Maximum number of errors for syntax checking.\n"
                            "Syntax errors aren't highlighted beyond this threshold.\n"
@@ -722,6 +891,7 @@ void PreferencesForm::create_sqlide_page()
   }  
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::create_query_page()
 {
@@ -780,6 +950,16 @@ void PreferencesForm::create_query_page()
       vbox->add(check, false);
     }
     
+    {
+      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:AutocommitMode");
+      check->set_text(_("Leave autocommit mode enabled by default"));
+      check->set_tooltip(_(
+                           "Toggles the default autocommit mode for connections.\nWhen enabled, each statement will be commited immediately."
+                           "\nNOTE: all query tabs in the same connection share the same transaction. "
+                           "To have independent transactions, you must open a new connection."));
+      vbox->add(check, false);
+    }
+
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -787,7 +967,6 @@ void PreferencesForm::create_query_page()
       
       tbox->add(new_label(_("Progress status update interval (in milliseconds):"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("DbSqlEditor:ProgressStatusUpdateInterval", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
                            "Time interval between UI updates when running SQL script."));
@@ -865,7 +1044,6 @@ void PreferencesForm::create_query_page()
       
       tbox->add(new_label(_("Limit Rows Count:"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("SqlEditor:LimitRowsCount", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
                            "Every select query to be implicitly adjusted to limit result set to specified number of rows."));
@@ -879,7 +1057,6 @@ void PreferencesForm::create_query_page()
       
       tbox->add(new_label(_("Max. Field Value Length to Display (in bytes):"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("Recordset:FieldValueTruncationThreshold", false);
-      entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
                            "Symbols beyond specified threashold will be truncated when showing in the grid. Doesn't affect editing field values.\n"
@@ -1010,10 +1187,6 @@ void PreferencesForm::create_general_page()
   table = mforms::manage(new OptionTable(this, _("Others"), true));
   top_box->add(table, false, true);
   {
-    table->add_checkbox_option("Sidebar:RightAligned", _("Place sidebar on the right side"), 
-                               _("Whether to place sidebar on the left or right side in module windows.\n"
-                                 "Clear this checbox to place sidebar on the left side."));
-    
 #ifdef _WIN32
     table->add_checkbox_option("DisableSingleInstance", _("Allow more than one instance of MySQL Workbench to run"), 
       _("By default only one instance of MySQL Workbench can run at the same time.\nThis is more resource friendly "
@@ -1411,6 +1584,37 @@ static void update_text_option(grt::DictRef options, const std::string &option_n
 }
 
 
+
+void PreferencesForm::change_font_option(const std::string &option, const std::string &value)
+{
+  std::vector<std::string>::const_iterator it;
+  if ((it = std::find(_font_options.begin(), _font_options.end(), option)) != _font_options.end())
+  {
+    int i = it - _font_options.begin();
+    _font_list.node_at_row(i)->set_string(1, value);
+  }
+}
+
+
+void PreferencesForm::font_preset_changed()
+{
+  int i = _font_preset.get_selected_index();
+
+  if (i >= 0)
+  {
+    change_font_option("workbench.physical.TableFigure:TitleFont", font_sets[i].object_title_font);
+    change_font_option("workbench.physical.TableFigure:SectionFont", font_sets[i].object_section_font);
+    change_font_option("workbench.physical.TableFigure:ItemsFont", font_sets[i].object_item_font);
+    change_font_option("workbench.physical.ViewFigure:TitleFont", font_sets[i].object_title_font);
+    change_font_option("workbench.physical.RoutineGroupFigure:TitleFont", font_sets[i].object_title_font);
+    change_font_option("workbench.physical.RoutineGroupFigure:ItemsFont", font_sets[i].object_item_font);
+    change_font_option("workbench.physical.Connection:CaptionFont", font_sets[i].object_item_font);
+    change_font_option("workbench.physical.Layer:TitleFont", font_sets[i].object_item_font);
+    change_font_option("workbench.model.NoteFigure:TextFont", font_sets[i].object_item_font);
+  }
+}
+
+
 void PreferencesForm::create_appearance_page()
 {
   mforms::Box *box= mforms::manage(new mforms::Box(false));
@@ -1464,22 +1668,27 @@ void PreferencesForm::create_appearance_page()
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("Fonts"));
 
-    mforms::Table *table= mforms::manage(new mforms::Table());
+    mforms::Box *content = mforms::manage(new mforms::Box(false));
+    content->set_padding(5);
+    frame->add(content);
 
-    table->set_padding(8);
-    table->set_row_spacing(4);
-    table->set_column_spacing(4);    
-    table->set_row_count(2);
-    table->set_column_count(2);
+    mforms::Box *hbox = mforms::manage(new mforms::Box(true));
+    content->add(hbox, false, true);
+    hbox->set_spacing(12);
+    hbox->set_padding(12);
 
-    frame->add(table);
+    _font_preset.signal_changed()->connect(boost::bind(&PreferencesForm::font_preset_changed, this));
+    for (size_t i = 0; font_sets[i].name; i++)
+      _font_preset.add_item(font_sets[i].name);
+
+    hbox->add(mforms::manage(new mforms::Label("Configure Fonts For:")), false, true);
+    hbox->add(&_font_preset, true, true);
 
     _font_list.add_column(mforms::StringColumnType, _("Location"), 150, false);
     _font_list.add_column(mforms::StringColumnType, _("Font"), 150, true);
     _font_list.end_columns();
 
-    table->add(&_font_list, 0, 1, 0, 1, mforms::FillAndExpand);
-
+    content->add(&_font_list, true, true);
     box->add(frame, true, true);
   }
 
@@ -1487,45 +1696,28 @@ void PreferencesForm::create_appearance_page()
   _tabview.add_page(box, _("Appearance"));
 }
 
-#if defined(_DEBUG) || defined(ENABLE_DEBUG) 
+#ifdef _WIN32
 /**
- * Used to create a forms page and attach it to the options form as a general playground
- * e.g. for testing mforms, creating other pages etc.
+ * Windows specific theming and colors page.
  */
-void PreferencesForm::create_test_page()
+void PreferencesForm::create_color_scheme_page()
 {
-  Box* content= manage(new Box(false));
+  Box* content = manage(new Box(false));
+  content->set_padding(12);
+  content->set_spacing(8);
 
-  _test_button = manage(new Button());
-  content->add(_test_button, false, true);
+  mforms::Panel *frame = mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
+  frame->set_title(_("Color Scheme"));
+  mforms::Box *hbox = mforms::manage(new mforms::Box(true));
+  hbox->add(new_label(_("Select your scheme:")), false, false);
+  mforms::Selector *selector = new_selector_option("ColorScheme", "", true);
+  selector->set_size(250, -1);
+  hbox->add(selector, true, false);
+  hbox->add(new_label(_("The scheme that determines the core colors."), false, true), true, false);
 
-  _test_button->set_text("popover");
-  scoped_connect(_test_button->signal_clicked(), boost::bind(&PreferencesForm::test_clicked, this));
-  
-  _tabview.add_page(content, _("Test page"));
-}
-
-void PreferencesForm::test_clicked()
-{
-  static wb::SnippetPopover* popover = NULL;
-
-  if (popover == NULL)
-  {
-    popover = new wb::SnippetPopover();
-    popover->set_size(376, 257);
-    popover->set_read_only(true);
-    int x = _test_button->get_x();
-    int y = _test_button->get_y();
-    _test_button->client_to_screen(x, y);
-    popover->show(x + _test_button->get_width() / 2, y + _test_button->get_height() / 2, mforms::Below);
-  }
-  else
-  {
-    popover->close();
-    delete popover;
-    popover = NULL;
-  }
-  
+  frame->add(hbox);
+  content->add(frame, false, true);
+  _tabview.add_page(content, _("Theming"));
 }
 
 #endif
@@ -1590,6 +1782,8 @@ void PreferencesForm::show_colors_and_fonts()
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void PreferencesForm::update_colors_and_fonts()
 {
   for (int c= _font_list.count(), i= 0; i < c; i++)
@@ -1600,7 +1794,47 @@ void PreferencesForm::update_colors_and_fonts()
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void PreferencesForm::toggle_use_global()
 {
   _tabview.set_enabled(!_use_global.get_active());
 }
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterPrefsNotifDocs
+{
+  RegisterPrefsNotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GRNPreferencesDidCreate",
+                                                           "preferences",
+                                                           "Sent when the Preferences window is created.",
+                                                           "",
+                                                           "options - the options dictionary being edited\n"
+                                                           "or\n"
+                                                           "model-options - the model specific options dictionary being changed\n"
+                                                           "model-id - the object id of the model for which the options are being changed");
+    
+    base::NotificationCenter::get()->register_notification("GRNPreferencesWillOpen",
+                                                           "preferences",
+                                                           "Sent when Preferences window is about to be shown on screen.",
+                                                           "",
+                                                           "options - the options dictionary being edited\n"
+                                                           "or\n"
+                                                           "model-options - the model specific options dictionary being changed\n"
+                                                           "model-id - the object id of the model for which the options are being changed");    
+    
+    base::NotificationCenter::get()->register_notification("GRNPreferencesDidClose",
+                                                           "preferences",
+                                                           "Sent after Preferences window was closed.",
+                                                           "",
+                                                           "saved - 1 if the user chose to save the options changed or 0 if changes were cancelled\n"
+                                                           "options - the options dictionary being edited\n"
+                                                           "or\n"
+                                                           "model-options - the model specific options dictionary being changed\n"
+                                                           "model-id - the object id of the model for which the options are being changed\n");
+  }
+} initdocs;
+
+
