@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +21,6 @@
 #ifndef _PREFERENCES_FORM_H_
 #define _PREFERENCES_FORM_H_
 
-#include <mforms/fs_object_selector.h>
-
-#include "workbench/wb_context_ui.h"
 #include "workbench/wb_backend_public_interface.h"
 
 #include "mforms/form.h"
@@ -33,6 +30,11 @@
 #include "mforms/checkbox.h"
 #include "mforms/treenodeview.h"
 #include "mforms/selector.h"
+#include "mforms/fs_object_selector.h"
+
+namespace mforms {
+  class RadioButton;
+}
 
 class MYSQLWBBACKEND_PUBLIC_FUNC PreferencesForm : public mforms::Form 
 {
@@ -58,13 +60,17 @@ private:
   
   mforms::CheckBox _use_global;
 
+  mforms::Selector _font_preset;
   mforms::TreeNodeView _font_list;
   std::vector<std::string> _font_options;
-  
-  std::string _model_id; // "" unless we're showing model specific options
+
+  workbench_physical_ModelRef _model; // nil unless we're showing model specific options
   wb::WBContextUI *_wbui;
 
   boost::function<std::string (std::string,std::string)> _edit_font;
+
+  void change_font_option(const std::string &option, const std::string &value);
+  void font_preset_changed();
   
   mforms::TextEntry *new_entry_option(const std::string &option, bool numeric);
   mforms::FsObjectSelector *new_path_option(const std::string &option, bool file);  
@@ -75,9 +81,6 @@ private:
   void ok_clicked();
   void cancel_clicked();
 
-  mforms::Button* _test_button;
-  void test_clicked();
-  
   void code_completion_changed(mforms::CheckBox *cc_box, mforms::CheckBox *auto_cc_box);
 
   void show_values();
@@ -85,7 +88,7 @@ private:
 
   void show_colors_and_fonts();
   void update_colors_and_fonts();
-  
+
   void create_admin_page();
   void create_sqlide_page();
   void create_query_page();
@@ -94,8 +97,8 @@ private:
   void create_mysql_page();
   void create_diagram_page();
   void create_appearance_page();
-#if defined(_DEBUG) || defined(ENABLE_DEBUG)
-  void create_test_page();
+#if _WIN32
+  void create_color_scheme_page();
 #endif
 
   grt::DictRef get_options(bool global= false);
@@ -117,7 +120,7 @@ private:
                               const std::vector<std::string> &choices, const std::string &default_value, bool as_number);
 
 public:
-  PreferencesForm(wb::WBContextUI *wbui, const std::string &model_id= "");
+  PreferencesForm(wb::WBContextUI *wbui, const workbench_physical_ModelRef &model = workbench_physical_ModelRef());
   virtual ~PreferencesForm();
   
   // returned font, title, current font
