@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,13 +18,22 @@
  */
 
 #include "stdafx.h"
-#include "workbench/wb_context_ui.h"
-#include "user_defined_type_editor.h"
-#include "grtui/string_list_editor.h"
+
 #include "base/util_functions.h"
 #include "base/string_utilities.h"
+#include "base/ui_form.h"
+
+#include "mforms/utilities.h"
 #include "mforms/checkbox.h"
-#include "model/wb_user_datatypes.h"
+
+#include "grt/editor_base.h"
+
+#include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+
+#include "user_defined_type_editor.h"
+#include "grtui/string_list_editor.h"
+#include "model/wb_context_model.h"
 
 using namespace base;
 
@@ -77,25 +86,25 @@ _vbox(false), _type_list(mforms::TreeAllowReorderRows|mforms::TreeFlatList), _ar
   _table.add(&_flags_box, 1, 2, 3, 4, mforms::HFillFlag|mforms::VFillFlag|mforms::HExpandFlag);
   _add_button.enable_internal_padding(true);
   _delete_button.enable_internal_padding(true);
-  _apply_button.enable_internal_padding(true);
-  _close_button.enable_internal_padding(true);
+  _ok_button.enable_internal_padding(true);
+  _cancel_button.enable_internal_padding(true);
 
   _button_box.set_padding(12);
-  _button_box.set_spacing(12);
+  _button_box.set_spacing(8);
   
   _add_button.set_text(_("Add"));
-  _add_button.set_size(75, -1);
+  _add_button.set_size(80, -1);
   _button_box.add(&_add_button, false, false);
   _delete_button.set_text(_("Delete"));
-  _delete_button.set_size(75, -1);
+  _delete_button.set_size(80, -1);
   _button_box.add(&_delete_button, false, false);
   
-  _apply_button.set_text(_("Apply"));
-  _apply_button.set_size(75, -1);
-  _button_box.add_end(&_apply_button, false, false);
-  _close_button.set_text(_("Close"));
-  _close_button.set_size(75, -1);
-  _button_box.add_end(&_close_button, false, false);
+  _ok_button.set_text(_("OK"));
+  _ok_button.set_size(80, -1);
+  _button_box.add_end(&_ok_button, false, false);
+  _cancel_button.set_text(_("Cancel"));
+  _cancel_button.set_size(80, -1);
+  _button_box.add_end(&_cancel_button, false, false);
   
   set_size(480, 480);
   center();
@@ -107,8 +116,8 @@ _vbox(false), _type_list(mforms::TreeAllowReorderRows|mforms::TreeFlatList), _ar
   scoped_connect(_add_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::add_clicked, this));
   scoped_connect(_delete_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::delete_clicked, this));
 
-  scoped_connect(_apply_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::apply_clicked, this));
-  scoped_connect(_close_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::close_clicked, this));
+  scoped_connect(_ok_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::ok_clicked, this));
+  scoped_connect(_cancel_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::cancel_clicked, this));
 
   scoped_connect(_name.signal_changed(),boost::bind(&UserDefinedTypeEditor::name_changed, this));
   scoped_connect(_args.signal_changed(),boost::bind(&UserDefinedTypeEditor::args_changed, this));
@@ -218,7 +227,7 @@ void UserDefinedTypeEditor::selected_row()
     {
       for (std::vector<std::string>::const_iterator fl= flags.begin(); fl != flags.end(); ++fl)
       {
-        if (g_strcasecmp(fl->c_str(), (*iter).c_str()) == 0)
+        if (g_ascii_strcasecmp(fl->c_str(), (*iter).c_str()) == 0)
         {
           _flags[i]->set_active(true);
           break;
@@ -373,7 +382,7 @@ static bool is_missing(const grt::ObjectRef &item, const std::vector<db_UserData
 }
 
 
-void UserDefinedTypeEditor::apply_clicked()
+void UserDefinedTypeEditor::ok_clicked()
 {
   // check if everything is ok and then commit
   
@@ -381,7 +390,6 @@ void UserDefinedTypeEditor::apply_clicked()
   std::set<std::string> systemNames;
   
   grt::ListRef<db_UserDatatype> udts(_model->catalog()->userDatatypes());
-  int old_count = udts->count();
   for (grt::ListRef<db_UserDatatype>::const_iterator udt= udts.begin(); udt != udts.end(); ++udt)
   {
     if (g_str_has_prefix((*udt).id().c_str(), "com.mysql.rdbms.mysql.userdatatype"))
@@ -484,11 +492,14 @@ void UserDefinedTypeEditor::apply_clicked()
   
   undo.end_or_cancel_if_empty(_("Edit User Defined Types"));
 
-  _wbui->get_usertypes_tree()->tree_changed(old_count);
+  if (_wbui->get_wb()->get_model_context())
+    _wbui->get_wb()->get_model_context()->_udt_list_changed();
+
+  close();
 }
 
 
-void UserDefinedTypeEditor::close_clicked()
+void UserDefinedTypeEditor::cancel_clicked()
 {
   close();
 }
