@@ -1,7 +1,23 @@
-#include "config.h"
-
+/*
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 #include "gtk/lf_mforms.h"
-
+#include "sqlide/grid_view.h"
 #include <gtkmm/statusbar.h>
 #include <gtkmm/eventbox.h>
 #include <gtkmm/fixed.h>
@@ -36,6 +52,7 @@
 #include "linux_utilities/toolbar_manager.h"
 
 #include "base/string_utilities.h"
+#include "base/geometry.h"
 #include "mforms/../gtk/lf_menubar.h"
 #include "mforms/../gtk/lf_toolbar.h"
 #include "mforms/../gtk/lf_form.h"
@@ -46,22 +63,13 @@ DEFAULT_LOG_DOMAIN("linux.ui")
 using base::strfmt;
 
 
-#ifdef EDITION_OSS
-# define SPLASH_IMAGE "wb_splashscreen_oss.png"
-#else
-# define SPLASH_IMAGE "wb_splashscreen_se.png"
-#endif
-
 
 static void set_window_icons(Gtk::Window *window)
 {
   std::vector<Glib::RefPtr<Gdk::Pixbuf> > icons;
 
-  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-24.png", false));
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-16.png", false));
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-32.png", false));
-  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-48.png", false));
-  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-64.png", false));
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-128.png", false));
 
   window->set_default_icon_list(icons);
@@ -116,8 +124,6 @@ MainForm::MainForm(wb::WBContextUI* ctx)
 
   _model_panel= 0;
 
-  _find_panel= 0;
-
   get_upper_note()->signal_switch_page().connect(sigc::mem_fun(*this, &MainForm::switch_page));
 
   Gtk::Statusbar *status = 0;
@@ -134,6 +140,7 @@ MainForm::MainForm(wb::WBContextUI* ctx)
   mforms::gtk::FormImpl::init_main_form(this->get_mainwindow());
 
   base::NotificationCenter::get()->add_observer(this, "GNFormTitleDidChange");
+  base::NotificationCenter::get()->add_observer(this, "GNFocusChanged");
 }
 
 //------------------------------------------------------------------------------
@@ -146,6 +153,14 @@ MainForm::~MainForm()
     _slots[i].disconnect();
   _slots.clear();
 
+
+  for (std::map<Gtk::Widget*,mforms::Menu*>::iterator it = _menu_list.begin(); it != _menu_list.end(); ++it)
+  {
+    if (it->second)
+      delete it->second;
+  }
+  _menu_list.clear();
+
   notify_callbacks(); // disconnect all signals
 
   delete _model_panel;
@@ -489,15 +504,18 @@ void MainForm::call_find()
       call_find_replace();
   }
 
-  bec::UIForm *form = _wbui_context->get_active_main_form();
+  FormViewBase *form = reinterpret_cast<FormViewBase*>(get_upper_note()->get_nth_page(get_upper_note()->get_current_page())->get_data("FormViewBase"));
   if (form)
   {
-    mforms::ToolBar *toolbar = form->get_toolbar();
+    mforms::ToolBar *toolbar = form->get_form()->get_toolbar();
     if (toolbar)
     {
-      Gtk::Entry *entry = dynamic_cast<Gtk::Entry*>(mforms::gtk::widget_for_toolbar_item_named(toolbar, "searchbox"));
+      Gtk::Entry *entry = dynamic_cast<Gtk::Entry*>(mforms::gtk::widget_for_toolbar_item_named(toolbar, "find"));
       if (entry)
+      {
         entry->grab_focus();
+        form->find_text(entry->get_text());
+      }
     }
   }
 }
@@ -561,10 +579,17 @@ void MainForm::call_copy()
     return;
   }
 
+
+
+  GridView *gv = NULL;
   if (dynamic_cast<Gtk::Editable*>(focused))
       dynamic_cast<Gtk::Editable*>(focused)->copy_clipboard();
   else if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_copy())
     _wbui_context->get_active_form()->copy();
+  else if ((gv = dynamic_cast<GridView*>(focused)))
+    gv->copy();
+
+
 }
 
 
@@ -635,22 +660,6 @@ void MainForm::call_select_all()
     _wbui_context->get_active_form()->select_all();
 }
 
-void MainForm::call_search()
-{
-  FormViewBase *form = reinterpret_cast<FormViewBase*>(get_upper_note()->get_nth_page(get_upper_note()->get_current_page())->get_data("FormViewBase"));
-  if (form)
-  {
-    mforms::ToolBar *toolbar = form->get_form()->get_toolbar();
-    if (toolbar)
-    {
-      Gtk::Entry *entry = dynamic_cast<Gtk::Entry*>(mforms::gtk::widget_for_toolbar_item_named(toolbar, "searchbox"));
-      if (entry)
-        form->find_text(entry->get_text());
-    }
-  }
-}
-
-
 bool MainForm::validate_find()
 {
   std::string context = _wbui_context->get_active_context();
@@ -701,6 +710,12 @@ bool MainForm::validate_copy()
     int s, e;
     if (edit && edit->get_selection_bounds(s, e))
       ret = true;
+    else
+    {
+      GridView *gv = dynamic_cast<GridView*>(get_mainwindow()->get_focus());
+      if (gv)
+        ret = true;
+    }
   }
 
   return ret;
@@ -788,12 +803,6 @@ bool MainForm::validate_select_all()
   return ret;
 }
 
-bool MainForm::validate_search()
-{
-  mforms::CodeEditor *editor = get_focused_code_editor(get_mainwindow());
-  return editor != NULL;
-}
-
 bool MainForm::validate_find_replace()
 {
   mforms::CodeEditor *editor = get_focused_code_editor(get_mainwindow());
@@ -809,8 +818,6 @@ void MainForm::register_commands()
 {
   std::list<std::string> commands;
 
-  commands.push_back("overview.home");
-  commands.push_back("show_about");
   commands.push_back("diagram_size");
   //  commands.push_back("view_model_navigator");
   //  commands.push_back("view_catalog");
@@ -820,7 +827,6 @@ void MainForm::register_commands()
   //  commands.push_back("view_object_description");
   //  commands.push_back("view_undo_history");
   commands.push_back("reset_layout");
-  commands.push_back("show_find_dialog");
   commands.push_back("wb.page_setup");
   commands.push_back("closetab");
   commands.push_back("close_tab");
@@ -832,14 +838,14 @@ void MainForm::register_commands()
   commands.push_back("wb.sidebarHide");
 
   // SQLIDE
-  commands.push_back("wb.toggleSideBar");
-  commands.push_back("wb.toggleSchemataBar");
+  commands.push_back("wb.toggleSidebar");
+  commands.push_back("wb.toggleSecondarySidebar");
   commands.push_back("wb.toggleOutputArea");
 
 
   _wbui_context->get_command_ui()->add_frontend_commands(commands);
 
-  _wbui_context->get_command_ui()->add_builtin_command("search_replace",
+  _wbui_context->get_command_ui()->add_builtin_command("find_replace",
                                                make_slot(&MainForm::call_find_replace),
                                                make_slot(&MainForm::validate_find_replace));
   _wbui_context->get_command_ui()->add_builtin_command("find",
@@ -866,9 +872,6 @@ void MainForm::register_commands()
   _wbui_context->get_command_ui()->add_builtin_command("selectAll",
                                                make_slot(&MainForm::call_select_all),
                                                make_slot(&MainForm::validate_select_all));
-  _wbui_context->get_command_ui()->add_builtin_command("searchbox",
-                                               make_slot(&MainForm::call_search),
-                                               make_slot(&MainForm::validate_search));
 
 }
 
@@ -877,12 +880,8 @@ void MainForm::perform_command_becb(const std::string& command)
 {
   if (command == "reset_layout")
     reset_layout();
-  else if (command == "show_about")
-    show_about();
   else if (command == "diagram_size")
     show_diagram_options();
-  else if (command == "show_find_dialog")
-    show_find();
   else if (command == "wb.page_setup")
     show_page_setup();
   else if (command == "closetab")
@@ -895,8 +894,6 @@ void MainForm::perform_command_becb(const std::string& command)
     get_upper_note()->next_page();
   else if (command == "wb.back_tab")
     get_upper_note()->prev_page();
-  else if (command == "overview.home")
-    get_upper_note()->set_current_page(0);
   /*
    // Help
    else if (command == "help_index")
@@ -904,11 +901,11 @@ void MainForm::perform_command_becb(const std::string& command)
    else if (command == "help_version_check")
    Program.CheckForNewVersion();
    */
-  else if (command == "wb.sidebarHide")
+  else if (command == "wb.toggleSidebar" || command == "wb.toggleSecondarySidebar")
   {
     FormViewBase *form = get_active_pane();
     if (form)
-      form->toggle_sidebar();
+      form->perform_command(command);
   }
   else
   {
@@ -1015,6 +1012,7 @@ void MainForm::tool_changed_becb(mdc::CanvasView* view)
 //------------------------------------------------------------------------------
 void MainForm::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
+
   if (name == "GNFormTitleDidChange")
   {
     std::string form_id = info["form"];
@@ -1030,6 +1028,10 @@ void MainForm::handle_notification(const std::string &name, void *sender, base::
       }
     }
   }
+  else if (name == "GNFocusChanged")
+  {
+    _wbui_context->get_command_ui()->revalidate_edit_menu_items();
+  }
 }
 
 
@@ -1181,8 +1183,6 @@ void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id,
   case wb::RefreshTimer:
     update_timer();
     break;
-  case wb::RefreshFindOutput:
-    break;
 
   case wb::RefreshFinishEdits:
     {
@@ -1263,6 +1263,10 @@ void MainForm::switch_page(GtkNotebookPage*, guint pagenum)
 
     bec::UIForm* uiform = static_cast<bec::UIForm*>(page->get_data("uiform"));
     _wbui_context->set_active_form(uiform);
+
+    FormViewBase *form = static_cast<FormViewBase*>(page->get_data("FormViewBase"));
+    if (form)
+      form->on_activate();
   }
   _wbui_context->get_command_ui()->revalidate_edit_menu_items();
 }
@@ -1471,6 +1475,13 @@ bool MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page)
       {
         if (form)
           form->dispose();
+        std::map<Gtk::Widget*,mforms::Menu*>::iterator it;
+        it = _menu_list.find(page);
+        if (it != _menu_list.end())
+        {
+          delete it->second;
+          _menu_list.erase(it);
+        }
         note->remove_page(*page);
         page->unreference();
 
@@ -1525,8 +1536,8 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
     if (title_label_ret)
       *title_label_ret = label;
 
-    mforms::Menu* menu = label->get_menu();
-    init_tab_menu(menu);
+    mforms::Menu* menu = init_tab_menu(widget);
+    label->set_menu(menu);
     menu->set_handler(sigc::bind(sigc::mem_fun(this, &MainForm::tab_menu_handler), label, widget, note));
 
     if (g_object_is_floating(G_OBJECT(widget->gobj())))
@@ -1544,11 +1555,20 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
 }
 
 //------------------------------------------------------------------------------
-void MainForm::init_tab_menu(mforms::Menu* m)
+mforms::Menu* MainForm::init_tab_menu(Gtk::Widget *widget)
 {
-  m->add_item("Close Tab", "close tab");
-  m->add_item("Close Other Tabs", "close other tabs");
-  m->add_item("Close Other Tabs of This Type", "close similar");
+  std::map<Gtk::Widget*,mforms::Menu*>::iterator it;
+  it = _menu_list.find(widget);
+  if (it == _menu_list.end())
+  {
+    mforms::Menu *m = new mforms::Menu();
+    m->add_item("Close Tab", "close tab");
+    m->add_item("Close Other Tabs", "close other tabs");
+    m->add_item("Close Other Tabs of This Type", "close similar");
+    _menu_list.insert(std::pair<Gtk::Widget*,mforms::Menu*>(widget,m));
+    return m;
+  }
+  return it->second;
 }
 
 //------------------------------------------------------------------------------
@@ -1670,39 +1690,6 @@ void MainForm::reset_layout()
 ///
 }
 
-
-
-void MainForm::show_about()
-{
-  Gtk::Window dialog(Gtk::WINDOW_TOPLEVEL);
-
-  dialog.set_default_size(560, 322);
-  dialog.modify_bg_pixmap(Gtk::STATE_NORMAL, bec::IconManager::get_instance()->get_icon_path(SPLASH_IMAGE));
-  dialog.set_position(Gtk::WIN_POS_CENTER);
-  dialog.set_decorated(false);
-
-  Gtk::Fixed fixed;
-  dialog.add(fixed);
-
-  Gtk::Label label(base::strfmt("Version %s\nrev %i", PACKAGE_VERSION, APP_REVISION_NUMBER));
-  label.set_alignment(1.0, 0.0);
-  label.set_justify(Gtk::JUSTIFY_RIGHT);
-  fixed.put(label, 10, 140);
-  label.set_size_request(540, 50);
-  label.modify_fg(Gtk::STATE_NORMAL, label.get_style()->get_black());
-
-  dialog.set_modal(true);
-  dialog.show_all();
-
-  dialog.add_events(Gdk::BUTTON_PRESS_MASK);
-  dialog.signal_button_press_event().connect_notify(sigc::hide(sigc::ptr_fun(&Gtk::Main::quit)));
-
-  Gtk::Main::instance()->run();
-
-  dialog.hide();
-}
-
-
 void MainForm::show_diagram_options()
 {
   DiagramSizeForm *form= DiagramSizeForm::create(_wbui_context);
@@ -1714,26 +1701,6 @@ void MainForm::show_diagram_options()
 }
 
 
-void MainForm::show_find()
-{
-  if (!_find_panel)
-  {
-    _find_panel= FindPanel::create(_wbui_context);
-
-    Gtk::VBox *vbox;
-    _ui->get_widget("inner_vbox", vbox);
-
-    vbox->pack_start(*_find_panel, false, false);
-    vbox->reorder_child(*_find_panel, 0);
-    _find_panel->unreference();
-
-  }
-
-  _find_panel->show();
-}
-
-
-
 void MainForm::show_page_setup()
 {
   _wbui_context->get_wb()->execute_plugin("wb.print.setup");
@@ -1789,14 +1756,14 @@ static std::string get_executable_path(mforms::App* app, const std::string& file
 }
 
 
-static std::string get_main_window_bounds(mforms::App* app)
+static base::Rect get_main_window_bounds(mforms::App* app)
 {
   Gtk::Window *w = get_mainwindow();
   int x, y;
   
   w->get_window()->get_position(x, y);
   
-  return base::strfmt("%i %i %i %i", x, y, w->get_width(), w->get_height());
+  return base::Rect(x, y, w->get_width(), w->get_height());
 }
 
 
@@ -1968,7 +1935,7 @@ void MainForm::setup_mforms_app()
   cf->_app_impl.get_resource_path = &get_resource_path;
   cf->_app_impl.get_executable_path = &get_executable_path;
   cf->_app_impl.set_status_text= &set_status_text;
-  cf->_app_impl.get_bounds= &get_main_window_bounds;
+  cf->_app_impl.get_application_bounds= &get_main_window_bounds;
   cf->_app_impl.begin_thread_loop= &begin_thread_loop;
   cf->_app_impl.end_thread_loop= &end_thread_loop;
   cf->_app_impl.enter_event_loop= &begin_event_loop;
