@@ -39,8 +39,6 @@ class OverviewPanel;
 class ModelDiagramPanel;
 class OutputBox;
 
-class FindPanel;
-
 class FormViewBase;
 class PluginEditorBase;
 
@@ -77,13 +75,15 @@ public:
   void lock_gui_becb(bool lock);
   void create_main_form_view_becb(const std::string &name, boost::shared_ptr<bec::UIForm> form);
   void destroy_main_form_view_becb(bec::UIForm* form);
-  void quit_app_becb();
+  bool quit_app_becb();
 
   void exiting() { _exiting = true; }
   
   wb::WBContextUI* get_wbui_context() const { return _wbui_context; }
 
 private:
+  std::map<Gtk::Widget*,mforms::Menu*> _menu_list;
+
   virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
 
   void register_commands();
@@ -105,16 +105,15 @@ private:
   void close_main_tab();
   void close_inner_tab();
   
-  void close_tab(Gtk::Notebook *note, Gtk::Widget *widget);
+  bool close_tab(Gtk::Notebook *note, Gtk::Widget *widget);
   void append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const std::string &title,
                        bool auto_switch, 
                        ActiveLabel **title_label_ret= 0);
 
-  void init_tab_menu(mforms::Menu* m);
+//  void init_tab_menu(mforms::Menu* m);
+  mforms::Menu* init_tab_menu(Gtk::Widget *widget);
   void tab_menu_handler(const std::string& action, ActiveLabel* label, Gtk::Widget* widget, Gtk::Notebook* note);
 
-  void show_about();
-  void show_find();
   void show_output();
   void show_diagram_options();
   void show_page_setup();
@@ -192,8 +191,6 @@ private:
   Glib::RefPtr<Gtk::Builder>   _ui;            //!< Glade model wrapper of the main window
   const char                  *_db_glade_file; //!< File name of the glade model of the model overview part
   
-  FindPanel                   *_find_panel;
-
   Glib::RefPtr<Gtk::AccelGroup> _accel_group;
   
   bool _gui_locked;
