@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,40 +24,60 @@
 
 static NSImage *CloseButtonImagePressed = nil;
 static NSImage *CloseButtonImageUnpressed = nil;
+static NSDictionary *MainTabImages = nil;
 static NSDictionary *EditorTabImages = nil;
 
+#define MAIN_TAB_PADDING 34
+
 + (void)initialize
 {
   [self exposeBinding: @"tabStyle"];
   
   NSBundle *bundle = [NSBundle bundleForClass: [MTabSwitcher class]];
-  
-  CloseButtonImagePressed = [[NSImage alloc] initWithContentsOfFile: 
-                             [bundle pathForImageResource: @"TabClose_Pressed.png"]];
-  
-  CloseButtonImageUnpressed = [[NSImage alloc] initWithContentsOfFile: 
-                               [bundle pathForImageResource: @"TabClose_Unpressed.png"]];
-  
-  
+
+  CloseButtonImagePressed = [[NSImage alloc] initWithContentsOfFile:
+                                             [bundle pathForImageResource: @"wb_tab-close_down"]];
+  CloseButtonImageUnpressed = [[NSImage alloc] initWithContentsOfFile:
+                                            [bundle pathForImageResource: @"wb_tab-close"]];
+
+  MainTabImages = [[NSDictionary dictionaryWithObjectsAndKeys:
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_background"]], @"bg",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_left"]], @"left",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_center"]], @"middle",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_right"]], @"right",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_home"]], @"home",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_home_home"]], @"home_home",
+                    [[NSImage alloc] initWithContentsOfFile:
+                      [bundle pathForImageResource: @"maintab_home_background"]], @"home_bg",
+
+                    [[NSImage alloc] initWithContentsOfFile:
+                     [bundle pathForImageResource: @"wb_tab-close_high"]], @"close_high",
+                    [[NSImage alloc] initWithContentsOfFile:
+                     [bundle pathForImageResource: @"wb_tab-close_down"]], @"close_down",
+                    [[NSImage alloc] initWithContentsOfFile:
+                     [bundle pathForImageResource: @"wb_tab-close"]], @"close",
+                      nil] retain];
   EditorTabImages = [[NSDictionary dictionaryWithObjectsAndKeys:
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_00.png"]] autorelease], @"bg",
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_01.png"]] autorelease], @"EA", // Empty - Active
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_02.png"]] autorelease], @"A", // Active
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_03.png"]] autorelease], @"AE", // Active - Empty
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_05.png"]] autorelease], @"I", // Inactive
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_06.png"]] autorelease], @"IE", // Inactive Empty
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_07.png"]] autorelease], @"EI", // Empty - Inactive
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_10.png"]] autorelease], @"AC", // Active close
-                      [[[NSImage alloc] initWithContentsOfFile: 
-                        [bundle pathForImageResource: @"tab_11.png"]] autorelease], @"ACH", // Active close hover
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"tab_background"]], @"bg",
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"tab_left"]], @"left",
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"tab_middle"]], @"middle",
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"tab_right"]], @"right",
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"wb_tab-close_high"]], @"close_high",
+                      [[NSImage alloc] initWithContentsOfFile:
+                       [bundle pathForImageResource: @"wb_tab-close_down"]], @"close_down",
+                      [[NSImage alloc] initWithContentsOfFile:
+                        [bundle pathForImageResource: @"wb_tab-close"]], @"close",
                       nil] retain];
 }
 
@@ -77,6 +97,8 @@ static NSDictionary *EditorTabImages = nil;
                                             owner: self
                                          userInfo: nil];
     [self addTrackingArea: mTrack];
+
+    mCloseButtonRects = [[NSMutableDictionary alloc] init];
     
     [[NSNotificationCenter defaultCenter] addObserver: self
                                              selector: @selector(frameDidChange:)
@@ -88,6 +110,7 @@ static NSDictionary *EditorTabImages = nil;
 
 - (void)dealloc
 {
+  [mCloseButtonRects release];
   for (id tag in mToolTipTags)
     [self removeToolTip: [tag intValue]];
   [mToolTipTags release];
@@ -156,7 +179,7 @@ static NSDictionary *EditorTabImages = nil;
         break;
       case MEditorTabSwitcher:
         size.height = 24;
-        size.width = MAX(ceil([[item label] sizeWithAttributes: mLabelAttributes].width) + 25*2, mMinTabWidth);
+        size.width = MAX(ceil([[item label] sizeWithAttributes: mLabelAttributes].width) + 23*2, mMinTabWidth);
         if (mMaxTabWidth > 0)
           size.width = MIN(mMaxTabWidth, size.width);
         break;
@@ -171,11 +194,10 @@ static NSDictionary *EditorTabImages = nil;
         break;
       case MMainTabSwitcher:
         size.height = 23;
-        if ([mDelegate respondsToSelector: @selector(tabView:iconForItem:)] && 
-            (icon = [mDelegate tabView: mTabView iconForItem: item]))
-          size.width = [icon size].width + 30;
+        if (![item label])
+          size.width = [[MainTabImages objectForKey: @"home"] size].width;
         else
-          size.width = MAX(ceil([[item label] sizeWithAttributes: mLabelAttributes].width) + 30, mMinTabWidth);
+          size.width = MAX(ceil([[item label] sizeWithAttributes: mLabelAttributes].width) + MAIN_TAB_PADDING, mMinTabWidth);
         if (mMaxTabWidth > 0)
           size.width = MIN(mMaxTabWidth, size.width);
         break;
@@ -183,6 +205,7 @@ static NSDictionary *EditorTabImages = nil;
   }
   else
   {
+    // return padding here
     switch (mStyle)
     {
       case MPaletteTabSwitcher:
@@ -204,8 +227,8 @@ static NSDictionary *EditorTabImages = nil;
         break;
       case MEditorTabSwitcher:
         size.height = 24;
-        size.width = 0;
-        break;        
+        size.width = 10;
+        break;
       case MEditorBottomTabSwitcher:
         size.height = 30;
         size.width = 10;
@@ -377,149 +400,100 @@ static NSDictionary *EditorTabImages = nil;
   return YES;
 }
 
-                          
+
+static void tile_image(NSImage *tile, int y, int minX, int maxX)
+{
+  if (tile == nil)
+    [NSException raise: @"Unexpected NULL value" format: @"tile_image() called with nil image"];
+
+  NSSize isize = [tile size];
+
+  int x = minX;
+  for (; x + isize.width < maxX; x += isize.width)
+    [tile drawInRect: NSMakeRect(x, y, isize.width, isize.height)
+            fromRect: NSMakeRect(0, 0, isize.width, isize.height)
+           operation: NSCompositeSourceOver fraction: 1.0];
+  if (x < maxX)
+    [tile drawInRect: NSMakeRect(x, y, (maxX - x), isize.height)
+            fromRect: NSMakeRect(0, 0, (maxX - x), isize.height)
+           operation: NSCompositeSourceOver fraction: 1.0];
+}
+
+static void draw_tab_images(NSImage *left, NSImage *middle, NSImage *right,
+                            int minX, int maxX)
+{
+  NSSize lsize, rsize;
+
+  lsize = [left size];
+  rsize = [right size];
+  [left drawInRect: NSMakeRect(minX - lsize.width, 0, lsize.width, lsize.height)
+         fromRect: NSMakeRect(0, 0, lsize.width, lsize.height)
+        operation: NSCompositeSourceOver fraction: 1.0];
+
+  tile_image(middle, 0, minX, maxX - rsize.width);
+
+  [right drawInRect: NSMakeRect(maxX - rsize.width, 0, rsize.width, rsize.height)
+          fromRect: NSMakeRect(0, 0, rsize.width, rsize.height)
+         operation: NSCompositeSourceOver fraction: 1.0];
+}
+
+
 - (float)drawMainTabStyleInFrame:(NSRect)rect forItem:(NSTabViewItem*)item
 {
   float x = NSMinX(rect);
   float width = [self _sizeOfTabViewItem: item].width;
   float height = NSHeight(rect);
-  if ([item tabState] != NSBackgroundTab)
-  {
-    BOOL isMain = [[self window] isMainWindow];
-    float selectedItemMinX = x+0.5;
-    float selectedItemMaxX = x+width+0.5;
-    
-    [[NSColor colorWithDeviceWhite: 246/255.0 alpha: 1.0] set];
-    
-    NSBezierPath *path = [NSBezierPath bezierPath];
-    
-    [path moveToPoint: NSMakePoint(selectedItemMinX-5, 0)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX-5, 5)
-                                     radius: 5
-                                 startAngle: 270
-                                   endAngle: 360];
-    [path lineToPoint: NSMakePoint(selectedItemMinX, height-5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+5, height-5.5)
-                                     radius: 5
-                                 startAngle: 180
-                                   endAngle: 90
-                                  clockwise: YES];
-    [path lineToPoint: NSMakePoint(selectedItemMaxX - 5, height-0.5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5, height-5.5)
-                                     radius: 5
-                                 startAngle: 90
-                                   endAngle: 0
-                                  clockwise: YES];
-    [path lineToPoint: NSMakePoint(selectedItemMaxX, 5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX+5, 5)
-                                     radius: 5
-                                 startAngle: 180
-                                   endAngle: 270];
-    
-    if (isMain)
-    {
-      [[[[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 206/255.0 alpha:1.0]
-                                      endingColor: [NSColor colorWithDeviceWhite: 230/255.0 alpha:1.0]] autorelease]
-       drawInBezierPath: path angle: 90];
-    }
-    else
-    {
-      [[[[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 233/255.0 alpha:1.0]
-                                      endingColor: [NSColor colorWithDeviceWhite: 240/255.0 alpha:1.0]] autorelease]
-       drawInBezierPath: path angle: 90];
-    }
 
-    [path setLineWidth: 1.0];
-    [path stroke];
-
-    // top rounded corners
-    if (isMain)
-      [[NSColor colorWithDeviceWhite: 161/255.0 alpha: 0.6] set];
-    else
-      [[NSColor colorWithDeviceWhite: 230/255.0 alpha: 0.6] set];
-    path = [NSBezierPath bezierPath];
-    [path moveToPoint: NSMakePoint(selectedItemMinX-0.5, height-4.5)];
-    
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+4.5, height-4.5)
-                                     radius: 5
-                                 startAngle: 180
-                                   endAngle: 90
-                                  clockwise: YES];
-    [path lineToPoint: NSMakePoint(selectedItemMaxX-4.5, height+0.5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5, height-4.5)
-                                     radius: 5
-                                 startAngle: 90
-                                   endAngle: 0
-                                  clockwise: YES];
-    [path setLineWidth: 1.0];
-    [path stroke];
-    
-    // sides
-    if (isMain)
-      [[NSColor colorWithDeviceWhite: 141/255.0 alpha: 0.8] set];
-    else
-      [[NSColor colorWithDeviceWhite: 210/255.0 alpha: 0.8] set];
-    
-    path = [NSBezierPath bezierPath];
-    [path moveToPoint: NSMakePoint(selectedItemMinX-5.5, 1.5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX-5.5, 6)
-                                     radius: 5
-                                 startAngle: 270
-                                   endAngle: 360];
-    [path lineToPoint: NSMakePoint(selectedItemMinX-0.5, height-4)];
-    /*
-     [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+5, height-4.5)
-     radius: 5
-     startAngle: 180
-     endAngle: 90
-     clockwise: YES];
-     [path lineToPoint: NSMakePoint(selectedItemMaxX - 5.0, height+0.5)];
-     [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5.0, height-4.5)
-     radius: 5
-     startAngle: 90
-     endAngle: 0
-     clockwise: YES];
-     */
-    [path moveToPoint: NSMakePoint(selectedItemMaxX, height-4.5)];
-    [path lineToPoint: NSMakePoint(selectedItemMaxX, 6)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX+5.0, 6)
-                                     radius: 5
-                                 startAngle: 180
-                                   endAngle: 270];
-    
-    [path setLineWidth: 1.0];
-    [path stroke];
-  }
-  
-  NSImage *icon = nil;
-  if ([mDelegate respondsToSelector: @selector(tabView:iconForItem:)])
-      icon = [mDelegate tabView: mTabView iconForItem: item];
-    
-  if (icon)
+  if (![item label])
   {
-    [icon drawInRect: NSMakeRect(x + 15, (int)((height-[icon size].height)/2), [icon size].width, [icon size].height)
+    NSImage *icon = [MainTabImages objectForKey: [item tabState] == NSBackgroundTab ? @"home" : @"home_home"];
+    [icon drawInRect: NSMakeRect(x, 0, [icon size].width, [icon size].height)
             fromRect: NSMakeRect(0, 0, [icon size].width, [icon size].height)
            operation: NSCompositeSourceOver fraction: 1.0];
   }
   else
   {
-    if ([mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] && [mDelegate tabView: mTabView itemHasCloseButton: item])
-      x -= [CloseButtonImagePressed size].width / 2;
+    if ([item tabState] != NSBackgroundTab)
+    {
+      //    BOOL isMain = [[self window] isMainWindow];
+      float selectedItemMinX = x+0.5;
+      float selectedItemMaxX = x+width+0.5;
+
+      draw_tab_images([MainTabImages objectForKey: @"left"],
+                      [MainTabImages objectForKey: @"middle"],
+                      [MainTabImages objectForKey: @"right"],
+                      selectedItemMinX, selectedItemMaxX);
+    }
+
+    float textoffs = [[MainTabImages objectForKey: @"left"] size].width + 2;
 
     if ([item tabState] == NSSelectedTab)
       [mLabelShadowAttributes setObject: [NSColor colorWithDeviceWhite: 245/255.0 alpha: 1.0] forKey: NSForegroundColorAttributeName];
     else
       [mLabelShadowAttributes setObject: [NSColor colorWithDeviceWhite: 200/255.0 alpha: 1.0] forKey: NSForegroundColorAttributeName];
     
-    [[item label] drawInRect: NSMakeRect(x + 15, 0, width, height-6) withAttributes: mLabelShadowAttributes];
-    [[item label] drawInRect: NSMakeRect(x + 14, 1, width, height-6) withAttributes: mLabelAttributes];
+    [[item label] drawInRect: NSMakeRect(x + textoffs, 0, width, height-6) withAttributes: mLabelShadowAttributes];
+    [[item label] drawInRect: NSMakeRect(x + textoffs, 1, width, height-6) withAttributes: mLabelAttributes];
     
-    if (mHoverItem == item)
     {
-      NSImage *image = mClosePressed ? CloseButtonImagePressed : CloseButtonImageUnpressed;
-      mCloseButtonRect = NSMakeRect(x+width-[image size].width, 4, [image size].width, [image size].height);
-      [image drawInRect: mCloseButtonRect fromRect: NSMakeRect(0, 0, [image size].width, [image size].height)
-              operation: NSCompositeSourceOver fraction: 1.0];    
+      NSImage *image;
+      NSRect closeRect;
+      if (mCloseHighlighted && mHoverItem == item)
+      {
+        if (mClosePressed)
+          image = [MainTabImages objectForKey: @"close_down"];
+        else
+          image = [MainTabImages objectForKey: @"close_high"];
+      }
+      else
+        image = [MainTabImages objectForKey: @"close"];
+      closeRect = NSMakeRect(x + width - [image size].width - [[MainTabImages objectForKey: @"right"] size].width - 4,
+                             (height - [image size].height) / 2,
+                             [image size].width, [image size].height);
+      [image drawInRect: closeRect fromRect: NSMakeRect(0, 0, [image size].width, [image size].height)
+              operation: NSCompositeSourceOver fraction: 1.0];
+      [mCloseButtonRects setObject: [NSValue valueWithRect: NSInsetRect(closeRect, -4, -4)]
+                            forKey: [item identifier]];
     }
   }
   return width;
@@ -634,78 +608,47 @@ static NSDictionary *EditorTabImages = nil;
   tabRect.origin = rect.origin;
   tabRect.size = [self _sizeOfTabViewItem: item];
   NSImage *icon = [mDelegate respondsToSelector: @selector(tabView:iconForItem:)] ? [mDelegate tabView: mTabView iconForItem: item] : nil;
-  NSImage *leftPart, *middlePart, *rightPart, *closePart;
-  float leftSize, rightSize;
-  float leftPadding, rightPadding;
+  NSImage *closePart = nil;
   NSRect r;
   
   if (NSMaxX(tabRect) > NSMaxX(rect))
     tabRect.size.width = NSMaxX(rect) - NSMinX(tabRect);
   
   if ([item tabState] == NSSelectedTab)
-  {
-    middlePart = [EditorTabImages objectForKey: @"A"];
-    leftPart = [EditorTabImages objectForKey: @"EA"];
-    rightPart = [EditorTabImages objectForKey: @"AE"];
-  }
-  else
-  {
-    middlePart = [EditorTabImages objectForKey: @"I"];
-    leftPart = [EditorTabImages objectForKey: @"EI"];
-    rightPart = [EditorTabImages objectForKey: @"IE"];
-  }
+    draw_tab_images([EditorTabImages objectForKey: @"left"], [EditorTabImages objectForKey: @"middle"], [EditorTabImages objectForKey: @"right"],
+                    NSMinX(tabRect), NSMaxX(tabRect));
+
   if (mHoverItem == item && mCloseHighlighted)
-    closePart = [EditorTabImages objectForKey: @"ACH"];
-  else
-    closePart = [EditorTabImages objectForKey: @"AC"];
-  
-  rightPadding = [closePart size].width + 2;
-  leftPadding = [icon size].width;
-  
-  r.origin = rect.origin;
-  r.size = [leftPart size];
-  leftSize = r.size.width;
-  [leftPart drawInRect: r
-              fromRect: NSZeroRect
-             operation: NSCompositeSourceOver
-              fraction: 1.0];
-  r.origin.x += leftSize;
-  rightSize = [rightPart size].width;
-  [middlePart drawInRect: NSMakeRect(r.origin.x, r.origin.y, NSWidth(tabRect) - leftSize - 2, [middlePart size].height)
-                fromRect: NSZeroRect
-               operation: NSCompositeSourceOver
-                fraction: 1.0];
-  r.size = [rightPart size];
-  r.origin.x += NSWidth(tabRect) - leftSize - 2;
-  [rightPart drawInRect: r
-               fromRect: NSZeroRect
-              operation: NSCompositeSourceOver
-               fraction: 1.0];
-  
-  if (item != mBusyTab)
   {
-    r.size = [closePart size];
-    r.origin.x = NSMaxX(tabRect) - NSWidth(r) - 3;
-    r.origin.y = 3;
+    if (mClosePressed)
+      closePart = [MainTabImages objectForKey: @"close_down"];
+    else
+      closePart = [MainTabImages objectForKey: @"close_high"];
+  }
+  else
+    closePart = [MainTabImages objectForKey: @"close"];
+
+  r.size = [closePart size];
+  r.origin.x = NSMaxX(tabRect) - NSWidth(r) - [[EditorTabImages objectForKey: @"right"] size].width - 4;
+  r.origin.y = (NSHeight(tabRect) - NSHeight(r)) / 2 - 2;
+  if (item != mBusyTab)
     [closePart drawAtPoint: r.origin
                   fromRect: NSZeroRect
                  operation: NSCompositeSourceOver
                   fraction: 1.0];
-    if (mHoverItem == item)
-      mCloseButtonRect = NSMakeRect(NSMinX(r), 3, NSWidth(r), NSWidth(r));
-  }
+  [mCloseButtonRects setObject: [NSValue valueWithRect: NSInsetRect(r, -4, -4)] forKey: [item identifier]];
   
-  [icon drawAtPoint: NSMakePoint(NSMinX(tabRect) + [leftPart size].width + 2, NSMinY(tabRect) + (NSHeight(tabRect) - 4 - [icon size].height)/2)
+  [icon drawAtPoint: NSMakePoint(NSMinX(tabRect) + 5, NSMinY(tabRect) + (NSHeight(tabRect) - 4 - [icon size].height)/2)
            fromRect: NSZeroRect 
           operation: NSCompositeSourceOver
            fraction: [item tabState] == NSSelectedTab ? 1.0 : 0.5];
 
   if ([item tabState] == NSSelectedTab)
-    [label drawInRect: NSMakeRect(NSMinX(tabRect) + leftSize + leftPadding, NSMinY(tabRect) + 2, 
-                                  NSWidth(tabRect) - leftSize - leftPadding - rightPadding, labelSize.height) 
+    [label drawInRect: NSMakeRect(NSMinX(tabRect) - 2, NSMinY(tabRect) + 2,
+                                  NSWidth(tabRect), labelSize.height)
         withAttributes: mLabelShadowAttributes];
-  [label drawInRect: NSMakeRect(NSMinX(tabRect) + leftSize + leftPadding, NSMinY(tabRect) + 3, 
-                                NSWidth(tabRect) - leftSize - leftPadding - rightPadding, labelSize.height)
+  [label drawInRect: NSMakeRect(NSMinX(tabRect) - 2, NSMinY(tabRect) + 3,
+                                NSWidth(tabRect), labelSize.height)
       withAttributes: [item tabState] == NSSelectedTab ? mLabelAttributes : mLabelDisabledAttributes];
   
   return NSWidth(tabRect);
@@ -779,12 +722,14 @@ static NSDictionary *EditorTabImages = nil;
   [label drawAtPoint: NSMakePoint(NSMinX(tabRect) + (NSWidth(tabRect) - labelSize.width) / 2, NSMinY(tabRect) + 9 + (NSHeight(tabRect)-8-labelSize.height)/2)
       withAttributes: mLabelAttributes];
 
-  if (mHoverItem == item)
   {
     NSImage *image = mClosePressed ? CloseButtonImagePressed : CloseButtonImageUnpressed;
-    mCloseButtonRect = NSMakeRect(NSMaxX(tabRect)-16, 9+4, [image size].width, [image size].height);
-    [image drawInRect: mCloseButtonRect fromRect: NSMakeRect(0, 0, [image size].width, [image size].height)
-            operation: NSCompositeSourceOver fraction: 1.0];
+    NSRect closeRect;
+    closeRect = NSMakeRect(NSMaxX(tabRect)-14, (NSHeight(tabRect) - [image size].height) / 2 + 5, [image size].width, [image size].height);
+    if (mHoverItem == item)
+      [image drawInRect: closeRect fromRect: NSMakeRect(0, 0, [image size].width, [image size].height)
+              operation: NSCompositeSourceOver fraction: 1.0];
+    [mCloseButtonRects setObject: [NSValue valueWithRect: NSInsetRect(closeRect, -4, -4)] forKey: [item identifier]];
   }
   
   return NSWidth(tabRect);
@@ -956,6 +901,12 @@ static NSDictionary *EditorTabImages = nil;
       NSRect tabArea = rect;
       tabArea.origin.x = floor(NSMinX(rect)) + padding;
       tabArea.size.width -= 2 * padding - mReservedSpace;
+
+      if (![[mTabView selectedTabViewItem] label])
+        tile_image([MainTabImages objectForKey: @"home_bg"], 0, NSMinX(rect), NSMaxX(rect));
+      else
+        tile_image([MainTabImages objectForKey: @"bg"], 0, NSMinX(rect), NSMaxX(rect));
+
       for (NSTabViewItem *item in [mTabView tabViewItems])
       {
         float w;
@@ -983,18 +934,7 @@ static NSDictionary *EditorTabImages = nil;
         tabArea.origin.x = mTabDragPosition.x - mClickTabOffset.x;
         tabArea.size.width = [self _sizeOfTabViewItem: mHoverItem].width;
         [self drawMainTabStyleInFrame: tabArea forItem: mHoverItem];
-      } 
-      
-      // stroke the baseline
-      [[NSColor colorWithDeviceWhite: 227/255.0 alpha: 1.0] set];
-      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 0.5) toPoint: NSMakePoint(NSMinX(selectedTabRect)-3, 0.5)];  
-      [NSBezierPath strokeLineFromPoint: NSMakePoint(NSMaxX(selectedTabRect)+4, 0.5) toPoint: NSMakePoint(NSWidth(rect), 0.5)];
-      if ([[self window] isMainWindow])
-        [[NSColor colorWithDeviceWhite: 135/255.0 alpha: 1.0] set];
-      else
-        [[NSColor colorWithDeviceWhite: 210/255.0 alpha: 1.0] set];
-      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 1.5) toPoint: NSMakePoint(NSMinX(selectedTabRect)-5, 1.5)];  
-      [NSBezierPath strokeLineFromPoint: NSMakePoint(NSMaxX(selectedTabRect)+3, 1.5) toPoint: NSMakePoint(NSWidth(rect), 1.5)];      
+      }
       break;
     }
   }
@@ -1165,7 +1105,9 @@ static NSDictionary *EditorTabImages = nil;
   mClickedItem = item;
   if (item)
   {
-    if (mHoverItem && item != mBusyTab && NSPointInRect(clickPos, mCloseButtonRect) &&
+    NSRect closeRect = [[mCloseButtonRects objectForKey: [item identifier]] rectValue];
+
+    if (mHoverItem && item != mBusyTab && NSPointInRect(clickPos, closeRect) &&
         ([mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] &&
          [mDelegate tabView: mTabView itemHasCloseButton: item]))
       mClosePressed = YES;
@@ -1176,7 +1118,7 @@ static NSDictionary *EditorTabImages = nil;
     mClickTabOffset = clickPos;
     mClickTabOffset.x -= tabRect.origin.x;
     
-    [self setNeedsDisplay: YES];
+    [self display];
   }
 }
 
@@ -1185,8 +1127,9 @@ static NSDictionary *EditorTabImages = nil;
 {
   if (!mInside)
     mHoverItem = nil;
-  
-  if (mClosePressed && mHoverItem && NSPointInRect([self convertPoint: [theEvent locationInWindow] fromView: nil], mCloseButtonRect))
+
+  if (mClosePressed && mHoverItem && NSPointInRect([self convertPoint: [theEvent locationInWindow] fromView: nil],
+                                                   [[mCloseButtonRects objectForKey: [mHoverItem identifier]] rectValue]))
   {
     NSTabViewItem *item = mHoverItem;
     [self setNeedsDisplay: YES];
@@ -1287,11 +1230,12 @@ static NSDictionary *EditorTabImages = nil;
   {
     NSPoint pos = [self convertPoint: [theEvent locationInWindow] fromView: nil];
     NSTabViewItem *item = [self tabViewItemAtPoint: pos];
+
     mCloseHighlighted = NO;
     if (![mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] ||
         ![mDelegate tabView: mTabView itemHasCloseButton: item])
       item = nil;
-    else if (NSPointInRect(pos, mCloseButtonRect))
+    else if (NSPointInRect(pos, [[mCloseButtonRects objectForKey: [item identifier]] rectValue]))
       mCloseHighlighted = YES;
     mHoverItem = item;
     [self setNeedsDisplay: YES];
@@ -1407,10 +1351,10 @@ static NSDictionary *EditorTabImages = nil;
     {
       case MEditorTabSwitcher:
         if (mBusyTabIndicator)
-          [mBusyTabIndicator setFrameOrigin: NSMakePoint(NSMaxX(rect) - 18, 2)];
+          [mBusyTabIndicator setFrameOrigin: NSMakePoint(NSMaxX(rect) - 23, 2)];
         else
         {
-          mBusyTabIndicator = [[[NSProgressIndicator alloc] initWithFrame: NSMakeRect(NSMaxX(rect) - 18, 2, 16, 16)] autorelease];
+          mBusyTabIndicator = [[[NSProgressIndicator alloc] initWithFrame: NSMakeRect(NSMaxX(rect) - 23, 2, 16, 16)] autorelease];
           [mBusyTabIndicator setControlSize: NSSmallControlSize];
           [mBusyTabIndicator setStyle: NSProgressIndicatorSpinningStyle];
           [mBusyTabIndicator setIndeterminate: YES];
@@ -1440,6 +1384,15 @@ static NSDictionary *EditorTabImages = nil;
   if ([mDelegate respondsToSelector: @selector(tabViewDidChangeNumberOfTabViewItems:)])
     [mDelegate tabViewDidChangeNumberOfTabViewItems: aTabView];
 
+  if ([aTabView numberOfTabViewItems] < [mCloseButtonRects count])
+  {
+    for (id item in [mCloseButtonRects allKeys])
+    {
+      if ([aTabView indexOfTabViewItemWithIdentifier: item] != NSNotFound)
+        [mCloseButtonRects removeObjectForKey: item];
+    }
+  }
+
   if (!mReorderingTab)
   {
     if ([mTabView indexOfTabViewItem: mClickedItem] == NSNotFound)
