@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -514,7 +514,7 @@ static int onRefresh(MResultsetViewer *self)
   {
     int columnIndex = [[aTableColumn identifier] intValue];    
     
-    if (columnIndex > 0)
+    if (columnIndex >= 0)
     {
       [aCell setIsNull: (*mData)->is_field_null(rowIndex, columnIndex)];
       [aCell setIsBlob: (*mData)->get_column_type(columnIndex) == bec::GridModel::BlobType];
