@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,9 +26,11 @@
 #import "MResultsetViewer.h"
 #import "WBSplitViewUnbrokenizerDelegate.h"
 #import "MVerticalLayoutView.h"
+#import "TabViewDockingDelegate.h"
+
+#import <mforms/../cocoa/MFView.h> // for nsviewForView
 
 #include <mforms/toolbar.h>
-#include <mforms/../cocoa/MFView.h> // for nsviewForView
 #include <mforms/code_editor.h>
 #include <mforms/find_panel.h>
 #include <mforms/appview.h>
@@ -39,85 +41,6 @@
 #include "sqlide/wb_sql_editor_form.h"
 
 
-class TabViewDockingPointDelegate : public mforms::DockingPointDelegate
-{
-  NSTabView *_tabView;
-  std::string _type;
-  std::map<NSView*, mforms::AppView*> _views;
-public:
-  TabViewDockingPointDelegate(NSTabView *tabView, const std::string &type) : _tabView(tabView), _type(type) {}
-
-  virtual std::string get_type() { return _type; }
-  
-  mforms::AppView *appview_for_view(NSView *view)
-  {
-    if (_views.find(view) != _views.end())
-      return _views[view];
-    return NULL;
-  }
-  
-  virtual void dock_view(mforms::AppView *view, const std::string &arg1, int arg2)
-  {
-    id v = view->get_data();
-    NSTabViewItem *tabItem = [[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
-    [tabItem setView: v];
-    [_tabView addTabViewItem: tabItem];
-    [_tabView selectTabViewItem: tabItem];
-    [tabItem release];
-    
-    _views[v] = view;
-  }
-  
-  virtual bool select_view(mforms::AppView *view)
-  {
-    NSInteger i = [_tabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
-    if (i >= 0)
-    {
-      NSTabViewItem *item = [_tabView tabViewItemAtIndex: i];
-      if (item)
-      {
-        [_tabView selectTabViewItem: item];
-        return true;
-      }
-    }    
-    return false;
-  }
-  
-  virtual void undock_view(mforms::AppView *view)
-  {
-    NSInteger i = [_tabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
-    if (i >= 0)
-    {
-      NSTabViewItem *item = [_tabView tabViewItemAtIndex: i];
-      if (item)
-      {
-        [_tabView removeTabViewItem: item];
-        _views.erase(view->get_data());
-      }
-    }
-  }
-    
-  virtual void set_view_title(mforms::AppView *view, const std::string &title)
-  {
-    NSInteger i = [_tabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
-    if (i >= 0)
-    {
-      NSTabViewItem *item = [_tabView tabViewItemAtIndex: i];
-      if (item)
-        [item setLabel: [NSString stringWithCPPString: title]];
-    }
-  }
-
-  virtual void get_size(int *w, int *h)
-  {
-    NSRect frame = [_tabView contentRect];
-    *w = (int)NSWidth(frame);
-    *h = (int)NSHeight(frame);
-  }
-};
-
-
-
 @interface ResultTabViewItem : NSTabViewItem
 {
   MResultsetViewer *viewer;
@@ -164,29 +87,15 @@ public:
   return [NSString stringWithCPPString: be->sql_editor_caption(be->sql_editor_index(mBackend))];
 }
 
+- (id)identifier
+{
+  return [self description];
+}
+
 - (NSImage*)tabIcon
 {
-  return [NSImage imageNamed: @"tab_icon_db.query.QueryBuffer.png"];
+  return [NSImage imageNamed: @"tab_icon_db.query.QueryBuffer"];
 }
-/*
-- (MScintillaView*)editorView
-{
-  return mEditor;
-}
-
-- (NSString*)string
-{
-  return [mEditorController string];
-}
-
-- (void)setString:(NSString*)string
-{
-}
-
-- (void)appendString:(NSString*)string
-{
-}
-*/
 
 - (Sql_editor::Ref)editorController
 {
@@ -326,7 +235,7 @@ public:
 - (void)setQueryCollapsed: (BOOL)flag
 {
   mSplitterPosition = NSHeight([[mResultsTabView superview] frame]) + [mSplitView dividerThickness];
-  [mSplitView setPosition: 0
+  [mSplitView setPosition: 100
          ofDividerAtIndex: 0];
   mQueryCollapsed = YES;
 }
@@ -380,7 +289,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     TabViewDockingPointDelegate *deleg = dynamic_cast<TabViewDockingPointDelegate*>(mBottomDockingPoint->get_delegate());
     mforms::AppView *av = deleg->appview_for_view([tabViewItem view]);
     if (av)
-      return av->on_close();
+      deleg->handle_close_clicked(av);
   }
   return YES;
 }
@@ -498,7 +407,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
       mBottomDockingPoint = mforms::manage(new mforms::DockingPoint(new TabViewDockingPointDelegate(mResultsTabView, RESULT_DOCKING_POINT), true));
       db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(mBackend->grtobj()));
       
-      qeditor->resultDockingPoint(mforms_to_grt(qeditor.get_grt(), mBottomDockingPoint, "DockingPoint"));
+      qeditor->resultDockingPoint(mforms_to_grt(qeditor.get_grt(), mBottomDockingPoint));
     }
     
     boost::shared_ptr<mforms::ToolBar> toolbar(editorBE->sql_editor_toolbar(editorBE->sql_editor_index(backend)));
@@ -541,7 +450,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 - (void)dealloc
 {
   mBottomDockingPoint->release();
-  delete mBottomDockingPointDelegate;
+
   [[mResultsTabSwitcher menu] release];
   [mSplitViewDelegate release];
   [mPanel release];
