@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,19 +32,25 @@
 #import "WBMiniToolbar.h"
 #import "GRTListDataSource.h"
 #import "WBQueryTab.h"
+#import "TabViewDockingDelegate.h"
+
+#include "objimpl/ui/mforms_ObjectReference_impl.h"
 
 #import "WBPluginPanel.h"
 #import "WBPluginEditorBase.h"
 #import "MSpinProgressCell.h"
 #include "sqlide/query_side_palette.h"
 #include "mforms/toolbar.h"
+#include "mforms/appview.h"
+
+#import "mforms/../cocoa/MFView.h"
+#import "mforms/../cocoa/MFMenu.h"
 
-#include "mforms/../cocoa/MFView.h"
-#include "mforms/../cocoa/MFMenu.h"
 #include "mforms/toolbar.h"
 
 #define MIN_SIDEBAR_WIDTH 100
 #define MIN_OUTPUT_AREA_HEIGHT 80
+#define MIN_INFO_BOX_HEIGHT 30
 
 @interface ColorBallTextCell : NSTextFieldCell
 {
@@ -292,6 +298,7 @@ static int processTaskFinish(WBSQLQueryPanel *self)
   {
     [self->mOutputTabView selectTabViewItemWithIdentifier: @"actions"];
     [self->mOutputSelector selectItemAtIndex: 0];
+    self->mBackEnd->show_output_area();
   }
   else
   {
@@ -318,8 +325,6 @@ static int processTaskProgress(float progress, const std::string &message, WBSQL
                                                                               object: table]];
   if ([table selectedRow] >= 0)
     [table scrollRowToVisible: [table selectedRow]];
-  else if ([table numberOfRows] > 0)
-    [table scrollRowToVisible: [table numberOfRows]-1];
 }
 
 
@@ -384,34 +389,17 @@ static void addTextToOutput(const std::string &text, bool bring_to_front, WBSQLQ
 #define QUERY_AREA_EXPANDED_MIN_HEIGHT (100)
 #define QUERY_AREA_COLLAPSED_MIN_HEIGHT (0)
 
-/*
-- (void)executeQuery:(id)sender currentStatementOnly: (bool) currentStatementOnly
-{
-  std::string text;
-  WBQueryTab *qtab = [self activeQueryTab];
-  
-  if (qtab)
-  {
-    Sql_editor::Ref editor = [qtab editorController].backend;
-    if (mBackEnd->exec_editor_sql(editor, false, currentStatementOnly, currentStatementOnly))
-      [mUpperTabSwitcher setBusyTab: [mUpperTabView selectedTabViewItem]];
-  }
-}
-*/
 - (void)addEditor: (WBBasePanel*)editor
 {
-  id identifier;
+  id identifier = [editor identifier];
   if ([editor isKindOfClass: [WBPluginPanel class]])
   {
     WBPluginEditorBase *peditor = [(WBPluginPanel*)editor pluginEditor];
     [peditor enableLiveChangeButtons];
     [peditor setCompactMode: NO];
-    identifier = [editor identifier];
   }
-  else
-    identifier = editor;
-  
-  [mEditors addObject: editor];
+
+  [mEditors setObject: editor forKey: identifier];
   NSTabViewItem *item = [[[NSTabViewItem alloc] initWithIdentifier: identifier] autorelease];
   [item setView: [editor topView]];
   [item setLabel: [editor title]];
@@ -448,7 +436,7 @@ static void addTextToOutput(const std::string &text, bool bring_to_front, WBSQLQ
   boost::shared_ptr<Sql_editor> be(mBackEnd->active_sql_editor());
   if (!be) return nil;
   
-  for (id qtab in mEditors)
+  for (id qtab in [mEditors allValues])
   {
     if ([qtab isKindOfClass: [WBQueryTab class]] && ([qtab editorController] == be))
       return qtab;
@@ -466,7 +454,7 @@ static void addTextToOutput(const std::string &text, bool bring_to_front, WBSQLQ
   if (index >= 0)
   {
     Sql_editor::Ref editor(mBackEnd->sql_editor(index));
-    for (id tab in mEditors)
+    for (id tab in [mEditors allValues])
     {
       if ([tab isKindOfClass: [WBQueryTab class]] && ([tab editorController] == editor))
         return tab;
@@ -642,32 +630,10 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
 
 - (void)splitViewDidResizeSubviews:(NSNotification *)notification
 {
-  if ([notification object] == mView)
-  {
-    if (!mBackEnd->grt_manager()->get_app_option_int("DbSqlEditor:SchemaSidebarHidden", 0))
-      mBackEnd->grt_manager()->set_app_option("DbSqlEditor:SchemaSidebarWidth", 
-                                              grt::IntegerRef(mBackEnd->get_sidebar()->get_width()));
-  }
-  else if ([notification object] == mSidebarSplit)
-  {
-    BOOL newCollapseState = [mSidebarSplit isSubviewCollapsed: [[mSidebarSplit subviews] lastObject]];
-    BOOL hidden = !mBackEnd->get_toolbar()->get_item_checked("wb.toggleSideBar");
-    
-    if (newCollapseState != hidden)
-    {
-      mBackEnd->grt_manager()->set_app_option("DbSqlEditor:SidebarHidden", grt::IntegerRef(newCollapseState));
-      mBackEnd->get_toolbar()->set_item_checked("wb.toggleSideBar", !newCollapseState);
-    }
-    if (!newCollapseState)
-    {
-      int width = mBackEnd->get_side_palette()->get_width();
-      if (width <= 0)
-        width = MIN_SIDEBAR_WIDTH;
-      mBackEnd->grt_manager()->set_app_option("DbSqlEditor:SidebarWidth",
-                                            grt::IntegerRef(width));
-    }
-  }
-  else if ([notification object] == mWorkView)
+  if (mHidingSidebar)
+    return;
+
+  if ([notification object] == mWorkView)
   {
     BOOL newCollapseState = [mWorkView isSubviewCollapsed: [[mWorkView subviews] lastObject]];
     BOOL hidden = !mBackEnd->get_toolbar()->get_item_checked("wb.toggleOutputArea");
@@ -686,27 +652,19 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
                                             grt::IntegerRef(height));
     }
   }
+  else
+    [super splitViewDidResizeSubviews: notification];
 }
 
 
 - (BOOL)splitView:(NSSplitView *)splitView shouldAdjustSizeOfSubview:(NSView *)subview
 {
-  if (splitView == mView)
-  {
-    if (subview != mWorkView) // the sidebar
-      return NO;
-  }
-  else if (splitView == mWorkView)
+  if (splitView == mWorkView)
   {
     if (subview == [mOutputTabView superview])
       return NO;
   }
-  else if (splitView == mSidebarSplit)
-  {
-    if (subview == nsviewForView(mBackEnd->get_side_palette()))
-      return NO;
-  }
-  return YES;
+  return [super splitView: splitView shouldAdjustSizeOfSubview: subview];
 }
 
 
@@ -717,33 +675,41 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     if (proposedMin < QUERY_AREA_COLLAPSED_MIN_HEIGHT)
       proposedMin = QUERY_AREA_COLLAPSED_MIN_HEIGHT;
   }
-  return proposedMin;
+  return [super splitView: splitView constrainMinCoordinate: proposedMin ofSubviewAt: dividerIndex];
 }
 
 - (CGFloat)splitView:(NSSplitView *)splitView constrainMaxCoordinate:(CGFloat)proposedMax ofSubviewAt:(NSInteger)dividerIndex
 {
-  if (splitView == mSidebarSplit)
-  {
-    return proposedMax > NSWidth([splitView frame]) - MIN_SIDEBAR_WIDTH ? NSWidth([splitView frame]) - MIN_SIDEBAR_WIDTH : proposedMax;
-  }
-  else if (splitView == mWorkView)
+  if (splitView == mWorkView)
   {
     return proposedMax > NSHeight([splitView frame]) - MIN_OUTPUT_AREA_HEIGHT ? NSHeight([splitView frame]) - MIN_OUTPUT_AREA_HEIGHT : proposedMax;
   }
-  return proposedMax;
+  return [super splitView: splitView constrainMaxCoordinate: proposedMax ofSubviewAt: dividerIndex];
 }
 
 - (BOOL)splitView:(NSSplitView *)splitView canCollapseSubview:(NSView *)subview
 {
-  if (splitView == mSidebarSplit && [[splitView subviews] lastObject] == subview)
-    return YES;
   if (splitView == mWorkView && [[splitView subviews] lastObject] == subview)
     return YES;
-  return NO;
+  return [super splitView: splitView canCollapseSubview: subview];
 }
 
 
-- (void)tabViewDraggerClicked: (NSTabView*) tabView 
+- (void)hideOutputArea:(BOOL)hidden
+{
+  mHidingSidebar = YES;
+  if (!hidden)
+    [mWorkView setPosition: NSHeight([mWorkView frame])-lastOutputAreaHeight ofDividerAtIndex: 0];
+  else
+  {
+    lastOutputAreaHeight = NSHeight([[mOutputTabView superview] frame]);
+    [mWorkView setPosition: NSHeight([mWorkView frame]) ofDividerAtIndex: 0];
+  }
+  mHidingSidebar = NO;
+}
+
+
+- (void)tabViewDraggerClicked: (NSTabView*) tabView
 {
   if (mLastClick > 0 && [NSDate timeIntervalSinceReferenceDate] - mLastClick < 0.3)
   {  
@@ -774,18 +740,51 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     mLastClick= [NSDate timeIntervalSinceReferenceDate];
 }
 
+
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
 {
   if (tabView == mUpperTabView)
   {
-    if ([[tabViewItem identifier] isKindOfClass: [WBQueryTab class]])
+    WBBasePanel *editor = [mEditors objectForKey: [tabViewItem identifier]];
+
+    if ([editor isKindOfClass: [WBQueryTab class]])
     {
-      Sql_editor::Ref ed([[tabViewItem identifier] editorController]);
+      Sql_editor::Ref ed([(WBQueryTab*)editor editorController]);
       
       mBackEnd->active_sql_editor_index(mBackEnd->sql_editor_index(ed));
     }
     else
       mBackEnd->active_sql_editor_index(-1);
+
+    // hide auxiliary sidebars if the administrator tab is active
+    BOOL expanded_mode = NO;
+
+    TabViewDockingPointDelegate* deleg = dynamic_cast<TabViewDockingPointDelegate*>(mDockingPoint->get_delegate());
+    if (deleg)
+    {
+      mforms::AppView* av = deleg->appview_for_view([tabViewItem view]);
+      if (av && av->get_form_context_name() == "administrator")
+        expanded_mode = YES;
+    }
+
+    if (expanded_mode != mExpandedMode)
+    {
+      if (expanded_mode)
+      {
+        mRightPaneWasExpanded = mBackEnd->get_toolbar()->get_item_checked("wb.toggleSecondarySidebar");
+        mBottomPaneWasExpanded = mBackEnd->get_toolbar()->get_item_checked("wb.toggleOutputArea");
+
+        [self hideSideBar: YES secondary: YES];
+        [self hideOutputArea: YES];
+      }
+      else
+      {
+        [self hideSideBar: !mRightPaneWasExpanded secondary: YES];
+        [self hideOutputArea: !mBottomPaneWasExpanded];
+      }
+
+      mExpandedMode = expanded_mode;
+    }
   }
 }
 
@@ -811,10 +810,10 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
 {
   if (tabView == mUpperTabView)
   {
-    if ([[item identifier] isKindOfClass: [WBQueryTab class]])
+    WBBasePanel *editor = [mEditors objectForKey: [item identifier]];
+    if ([editor isKindOfClass: [WBQueryTab class]])
     {
-      WBQueryTab *qtab = [item identifier];
-      if (!mBackEnd->sql_editor_reorder([qtab editorController], index))
+      if (!mBackEnd->sql_editor_reorder([(WBQueryTab*)editor editorController], index))
         return NO;
     }
   }
@@ -827,9 +826,10 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 {
   if (tabView == mUpperTabView)
   {
-    if ([[tabViewItem identifier] isKindOfClass: [WBQueryTab class]])
+    WBBasePanel *editor = [mEditors objectForKey: [tabViewItem identifier]];
+    if ([editor isKindOfClass: [WBQueryTab class]])
     {
-      Sql_editor::Ref ed = [[tabViewItem identifier] editorController];
+      Sql_editor::Ref ed = [(WBQueryTab*)editor editorController];
       
       int idx = mBackEnd->sql_editor_index(ed);
       if (idx < 0)
@@ -839,12 +839,26 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
         return NO;
       
       mBackEnd->remove_sql_editor(idx);
-      
-      [mEditors removeObject: [tabViewItem identifier]];
+
+      [mEditors removeObjectForKey: [tabViewItem identifier]];
     }
-    else 
+    else
     {
-      [mEditors removeObject: [tabViewItem identifier]];
+      TabViewDockingPointDelegate* deleg = dynamic_cast<TabViewDockingPointDelegate*>(mDockingPoint->get_delegate());
+      mforms::AppView *appView = deleg->appview_for_view([tabViewItem view]);
+
+      if (appView)
+      {
+        deleg->handle_close_clicked(appView);
+      }
+      else
+      {
+        id editor = [mEditors objectForKey: [tabViewItem identifier]];
+        if ([editor respondsToSelector: @selector(willClose)])
+          if (![editor willClose])
+            return NO;
+        [mEditors removeObjectForKey: [tabViewItem identifier]];
+      }
     }
     return YES;
   }
@@ -863,11 +877,25 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 {
   if (tabView == mUpperTabView)
   {
-    id tab = [tabViewItem identifier];
+    id tab = [mEditors objectForKey: [tabViewItem identifier]];
     if ([tab respondsToSelector: @selector(tabIcon)])
       return [tab tabIcon];
+    else
+    {
+      TabViewDockingPointDelegate* deleg = dynamic_cast<TabViewDockingPointDelegate*>(mDockingPoint->get_delegate());
+      if (deleg)
+      {
+        mforms::AppView* av = deleg->appview_for_view([tabViewItem view]);
+        if (av)
+        {
+          NSImage *icon = [NSImage imageNamed: [NSString stringWithFormat: @"tab_icon_%s", av->get_form_context_name().c_str()]];
+          if (icon)
+            return icon;
+        }
+      }
+    }
   }
-  return nil;
+  return [NSImage imageNamed: @"tab_icon_plugin"];
 }
 
 
@@ -875,9 +903,10 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 {
   if (tabView == mUpperTabView)
   {
-    if ([[item identifier] isKindOfClass: [WBQueryTab class]])
+    id tab = [mEditors objectForKey: [item identifier]];
+    if ([tab isKindOfClass: [WBQueryTab class]])
     {
-      Sql_editor::Ref editor = [[item identifier] editorController];
+      Sql_editor::Ref editor = [tab editorController];
       
       int idx = mBackEnd->sql_editor_index(editor);
       if (idx >= 0)
@@ -892,9 +921,10 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 {
   if (tabView == mUpperTabView)
   {
-    if ([[item identifier] isKindOfClass: [WBQueryTab class]])
+    id tab = [mEditors objectForKey: [item identifier]];
+    if ([tab isKindOfClass: [WBQueryTab class]])
     {
-      Sql_editor::Ref editor = [[item identifier] editorController];
+      Sql_editor::Ref editor = [tab editorController];
       if (editor)
       {
         int i = mBackEnd->sql_editor_index(editor);
@@ -920,9 +950,10 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     case 51: // save tab
     {
       NSTabViewItem *item = [mUpperTabSwitcher clickedItem];
-      if ([[item identifier] isKindOfClass: [WBQueryTab class]])
+      id tab = [mEditors objectForKey: [item identifier]];
+      if ([tab isKindOfClass: [WBQueryTab class]])
       {
-        Sql_editor::Ref editor = [[item identifier] editorController];
+        Sql_editor::Ref editor = [tab editorController];
         if (editor)
         {
           int i = mBackEnd->sql_editor_index(editor);
@@ -935,9 +966,10 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     case 60: // copy path to clipboard
     {
       NSTabViewItem *item = [mUpperTabSwitcher clickedItem];
-      if ([[item identifier] isKindOfClass: [WBQueryTab class]])
+      id tab = [mEditors objectForKey: [item identifier]];
+      if ([tab isKindOfClass: [WBQueryTab class]])
       {
-        Sql_editor::Ref editor = [[item identifier] editorController];
+        Sql_editor::Ref editor = [tab editorController];
         if (editor)
         {        
           int i = mBackEnd->sql_editor_index(editor);
@@ -963,24 +995,19 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
   self= [super init];
   if (self)
   {
-    BOOL schemaSidebarHidden;
-    BOOL sidebarHidden;
     BOOL outputAreaHidden;
     
     // restore state of toolbar
     {
       mforms::ToolBar *toolbar = be->get_toolbar();      
-      toolbar->set_item_checked("wb.toggleSchemataBar", !(schemaSidebarHidden = be->grt_manager()->get_app_option_int("DbSqlEditor:SchemaSidebarHidden", 0))); 
-      toolbar->set_item_checked("wb.toggleSideBar", !(sidebarHidden = be->grt_manager()->get_app_option_int("DbSqlEditor:SidebarHidden", 0))); 
-      toolbar->set_item_checked("wb.toggleOutputArea", !(outputAreaHidden = be->grt_manager()->get_app_option_int("DbSqlEditor:OutputAreaHidden", 0))); 
+      toolbar->set_item_checked("wb.toggleOutputArea", !(outputAreaHidden = be->grt_manager()->get_app_option_int("DbSqlEditor:OutputAreaHidden", 0)));
     }
-    lastSchematabarWidth = MAX(be->grt_manager()->get_app_option_int("DbSqlEditor:SchemaSidebarWidth", 220), 100);
-    lastSidebarWidth = MAX(be->grt_manager()->get_app_option_int("DbSqlEditor:SidebarWidth", 220), MIN_SIDEBAR_WIDTH);
     lastOutputAreaHeight = MAX(be->grt_manager()->get_app_option_int("DbSqlEditor:OutputAreaHeight", 135), MIN_OUTPUT_AREA_HEIGHT);
 
     [NSBundle loadNibNamed: @"WBSQLQueryPanel"
                      owner: self];
-    
+
+    grtm = be->grt_manager();
     mBackEnd= be;
     mBackEnd->log()->refresh_ui_cb= boost::bind(reloadTable, mMessagesTable, self);
     mBackEnd->history()->entries_model()->refresh_ui_cb= boost::bind(reloadTable, mHistoryTable, self);
@@ -1015,7 +1042,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     if (font)
       [mTextOutput setFont: font];
     
-    [mView setBackgroundColor: [NSColor colorWithDeviceWhite:232/255.0 alpha:1.0]];
+    [mView setBackgroundColor: [NSColor colorWithDeviceWhite:128/255.0 alpha:1.0]];
     
     [mWorkView setDividerThickness: 0];
     [mView setDividerThickness: 1];
@@ -1026,72 +1053,36 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     [mHistoryDetailsTable setDoubleAction: @selector(activateHistoryDetailEntry:)];
     if ([mHistoryTable numberOfRows] > 0)
     {
-      [mHistoryTable selectRowIndexes: [NSIndexSet indexSetWithIndex: [mHistoryTable numberOfRows]-1]
+      [mHistoryTable selectRowIndexes: [NSIndexSet indexSetWithIndex: 0]
                  byExtendingSelection: NO];
     }
     
     // dock the backend provided schema sidebar and restore its width
     {
-      mforms::View *sidebar = mBackEnd->get_sidebar();
-      mSchemataBarView = nsviewForView(sidebar);
-      if (schematabarAtRight)
-        [mView addSubview: mSchemataBarView];
+      mforms::View *sidebar_ = mBackEnd->get_sidebar();
+      sidebar = nsviewForView(sidebar_);
+      if (mSidebarAtRight)
+        [mView addSubview: sidebar];
       else
-        [mView addSubview: mSchemataBarView positioned: NSWindowBelow relativeTo: [[mView subviews] lastObject]];
+        [mView addSubview: sidebar positioned: NSWindowBelow relativeTo: [[mView subviews] lastObject]];
       [mView adjustSubviews];
-
-      if (schemaSidebarHidden)
-      {
-        if (schematabarAtRight)
-          [mView setPosition: NSWidth([mView frame]) ofDividerAtIndex: 0];
-        else
-          [mView setPosition: 0 ofDividerAtIndex: 0];        
-      }
-      else
-      {
-        if (schematabarAtRight)
-          [mView setPosition: NSWidth([mView frame]) - lastSchematabarWidth ofDividerAtIndex: 0];
-        else
-          [mView setPosition: lastSchematabarWidth ofDividerAtIndex: 0];
-        
-        // ugly hack to force the splitter position where we want it.. somehow, without this "adjustment"
-        // the splitter would make the sidebar sized 10px narrower
-        if (mBackEnd->get_sidebar()->get_width() < lastSchematabarWidth)
-        {
-          if (schematabarAtRight)
-            [mView setPosition: NSWidth([mView frame]) - (lastSchematabarWidth - mBackEnd->get_sidebar()->get_width()) ofDividerAtIndex: 0];
-          else
-            [mView setPosition: lastSchematabarWidth + (lastSchematabarWidth - mBackEnd->get_sidebar()->get_width()) ofDividerAtIndex: 0];        
-        }
-      }
     }
     
     // dock the other sidebar
     {    
       mforms::View *view = mBackEnd->get_side_palette();
+      secondarySidebar = nsviewForView(view);
       if (view)
       {
-        if (schematabarAtRight)
-          [mSidebarSplit addSubview: nsviewForView(view) positioned: NSWindowBelow relativeTo: [[mSidebarSplit subviews] lastObject]];      
+        if (mSidebarAtRight)
+          [topView addSubview: secondarySidebar positioned: NSWindowBelow relativeTo: [[topView subviews] lastObject]];
         else
-          [mSidebarSplit addSubview: nsviewForView(view)];
-      }
-      [mSidebarSplit adjustSubviews];
-      if (sidebarHidden)
-      {
-        if (schematabarAtRight)
-          [mSidebarSplit setPosition: 0 ofDividerAtIndex: 0];        
-        else
-          [mSidebarSplit setPosition: NSWidth([mSidebarSplit frame]) ofDividerAtIndex: 0];
-      }
-      else
-      {
-        if (schematabarAtRight)
-          [mSidebarSplit setPosition: lastSidebarWidth ofDividerAtIndex: 0];        
-        else
-          [mSidebarSplit setPosition: NSWidth([mSidebarSplit frame]) - lastSidebarWidth ofDividerAtIndex: 0];
+          [topView addSubview: secondarySidebar];
       }
+      [topView adjustSubviews];
     }
+
+    [self restoreSidebarsFor: "DbSqlEditor" toolbar: be->get_toolbar()];
     
     // restore height of the output area
     if (outputAreaHidden)
@@ -1099,10 +1090,18 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     else
       [mWorkView setPosition: NSHeight([mWorkView frame]) - lastOutputAreaHeight ofDividerAtIndex: 0];
 
-    mEditors = [[NSMutableArray alloc] init];
+    mEditors = [[NSMutableDictionary alloc] init];
     
     mQueryAreaOpen = YES;
     mResultsAreaOpen = YES;
+
+    // setup docking point for mUpperTabView
+    {
+      mDockingPoint = mforms::manage(new mforms::DockingPoint(new TabViewDockingPointDelegate(mUpperTabView, MAIN_DOCKING_POINT), true));
+      db_query_EditorRef qeditor(be->wbsql()->get_grt_editor_object(be.get()));
+
+      qeditor->dockingPoint(mforms_to_grt(qeditor.get_grt(), mDockingPoint));
+    }
     
     // realize pre-existing editors
     for (int i = 0; i < mBackEnd->sql_editor_count(); i++)
@@ -1128,14 +1127,14 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 
 - (void)setRightSidebar:(BOOL)flag
 {
-  schematabarAtRight = flag;
+  mSidebarAtRight = flag;
 
   id view1 = [[mView subviews] objectAtIndex: 0];
   id view2 = [[mView subviews] objectAtIndex: 1];
   
-  if (schematabarAtRight)
+  if (mSidebarAtRight)
   {
-    if (view2 != mSchemataBarView)
+    if (view2 != sidebar)
     {
       [[view1 retain] autorelease];
       [view1 removeFromSuperview];
@@ -1144,7 +1143,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
   }
   else
   {
-    if (view1 != mSchemataBarView)
+    if (view1 != sidebar)
     {
       [[view1 retain] autorelease];
       [view1 removeFromSuperview];
@@ -1155,13 +1154,26 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 
 - (BOOL)willClose
 {
+  // Check for edited tabs in frontend for tab types that the backend doesn't handle
+  for (id qtab in [mEditors allValues])
+  {
+    if (![qtab isKindOfClass: [WBQueryTab class]])
+      if ([qtab respondsToSelector: @selector(willClose)] && ![qtab willClose])
+        return NO;
+  }
+
+  if (!dynamic_cast<TabViewDockingPointDelegate*>(mDockingPoint->get_delegate())->close_all())
+    return NO;
+
   return mBackEnd->can_close();
 }
 
 - (void) dealloc
-{ 
+{
   mBackEnd->close();
-  
+
+  mDockingPoint->release();
+
   [mTextOutputLock release];
   
   [mEditors release];
@@ -1171,63 +1183,20 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
   [super dealloc]; 
 }
 
+
 /**
  * Executes commands sent by the main form that should be handled here.
  */
 - (void) performCommand: (const std::string) command
 {
-  if (command == "wb.toggleSchemataBar")
-  {
-    BOOL hidden = !mBackEnd->get_toolbar()->get_item_checked(command);
-    mBackEnd->grt_manager()->set_app_option("DbSqlEditor:SchemaSidebarHidden", grt::IntegerRef(hidden));
-    if (!hidden)
-    {
-      if (schematabarAtRight) 
-        [mView setPosition: NSWidth([mView frame])-lastSchematabarWidth ofDividerAtIndex: 0];
-      else
-        [mView setPosition: lastSchematabarWidth ofDividerAtIndex: 0];
-    }
-    else
-    {
-      lastSchematabarWidth = NSWidth([mSchemataBarView frame]);
-      if (schematabarAtRight)
-        [mView setPosition: NSWidth([mView frame]) ofDividerAtIndex: 0];
-      else
-        [mView setPosition: 0 ofDividerAtIndex: 0];
-    }
-  }
-  else if (command == "wb.toggleSideBar")
-  {
-    BOOL hidden = !mBackEnd->get_toolbar()->get_item_checked(command);
-    mBackEnd->grt_manager()->set_app_option("DbSqlEditor:SidebarHidden", grt::IntegerRef(hidden));
-    if (!hidden)
-    {
-      if (schematabarAtRight)
-        [mSidebarSplit setPosition: lastSidebarWidth ofDividerAtIndex: 0];
-      else
-        [mSidebarSplit setPosition: NSWidth([mSidebarSplit frame])-lastSidebarWidth ofDividerAtIndex: 0];
-    }
-    else
-    {
-      lastSidebarWidth = mBackEnd->get_side_palette()->get_width();
-      if (schematabarAtRight)
-        [mSidebarSplit setPosition: 0 ofDividerAtIndex: 0];
-      else
-        [mSidebarSplit setPosition: NSWidth([mSidebarSplit frame]) ofDividerAtIndex: 0];
-    }
-  }
-  else if (command == "wb.toggleOutputArea")
+  if (command == "wb.toggleOutputArea")
   {
     BOOL hidden = !mBackEnd->get_toolbar()->get_item_checked(command);
     mBackEnd->grt_manager()->set_app_option("DbSqlEditor:OutputAreaHidden", grt::IntegerRef(hidden));
-    if (!hidden)
-      [mWorkView setPosition: NSHeight([mWorkView frame])-lastOutputAreaHeight ofDividerAtIndex: 0];
-    else
-    {
-      lastOutputAreaHeight = NSHeight([[mOutputTabView superview] frame]);
-      [mWorkView setPosition: NSHeight([mWorkView frame]) ofDividerAtIndex: 0];
-    }
+    [self hideOutputArea: hidden];
   }
+  else
+    [super performCommand: command];
 }
 
 - (void)activateBusyAnimationForActiveEditor
