@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,9 @@
  */
 
 #include "base/string_utilities.h"
+
+#include "workbench/wb_context.h"
+
 #import "WBMainController.h"
 #import "WBMainWindow.h"
 #import "MCppUtilities.h"
@@ -50,6 +53,8 @@
 #include "workbench/wb_module.h"
 #include "base/log.h"
 
+#include "wb_command_ui.h"
+
 static GThread *mainthread= 0;
 
 DEFAULT_LOG_DOMAIN("Workbench")
@@ -316,8 +321,6 @@ static void windowPerformCommand(const std::string &command, WBMainWindow *windo
     [window resetWindowLayout];
   else if (command == "overview.mysql_model")
     [window showMySQLOverview:nil];
-  else if (command == "show_about")
-    [[window owner] showAbout:nil];
   else if (command == "diagram_size")
     [main showDiagramProperties:nil];
   else if (command == "wb.page_setup")
@@ -604,12 +607,6 @@ static bool validate_close_editor(WBMainWindow *mainWindow)
   
   return activePanel != nil && [[mainWindow window] isKeyWindow] && [activePanel respondsToSelector: @selector(closeActiveEditorTab)];
 }
-//--------------------------------------------------------------------------------------------------
-
-static void call_search(WBMainWindow *mainWindow)
-{
-  [mainWindow performSearchObject:nil];
-}
 
 //--------------------------------------------------------------------------------------------------
 
@@ -644,16 +641,20 @@ static void call_find_replace(bool do_replace)
 
 static void call_find(WBMainWindow *mainWindow)
 {
-  if (validate_find_replace())
-    call_find_replace(false);
+  id firstResponder = [[NSApp keyWindow] firstResponder];
+  if ([firstResponder isKindOfClass: [InnerView class]])
+    firstResponder = [firstResponder superview];
+
+  if ([firstResponder isKindOfClass: [MFCodeEditor class]])
+    [firstResponder showFindPanel: NO];
   else
-    [mainWindow focusSearchField: nil];
+    [mainWindow performSearchObject:nil];
 }
 
 static bool validate_find(WBMainWindow *mainWindow)
 {
   bec::UIForm *form = [mainWindow context]->get_active_main_form();
-  if (form && form->get_toolbar() && form->get_toolbar()->find_item("searchbox"))
+  if (form && form->get_toolbar() && form->get_toolbar()->find_item("find"))
     return true;
   return validate_find_replace();
 }
@@ -663,9 +664,12 @@ static bool validate_find(WBMainWindow *mainWindow)
 static void call_undo(WBMainWindow *mainWindow)
 {
   wb::WBContextUI *wbui = [mainWindow context];
+  id firstResponder = [[NSApp keyWindow] firstResponder];
 
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(undo:)])
-    [[[NSApp keyWindow] firstResponder] tryToPerform: @selector(undo:) with:nil];
+  if ([firstResponder respondsToSelector: @selector(undo:)])
+    [firstResponder tryToPerform: @selector(undo:) with:nil];
+  else if ([firstResponder isKindOfClass: [NSTextView class]])
+    return [[firstResponder undoManager] undo];
   else
     if (wbui->get_active_main_form())
       wbui->get_active_main_form()->undo();
@@ -674,9 +678,14 @@ static void call_undo(WBMainWindow *mainWindow)
 static bool validate_undo(WBMainWindow *mainWindow)
 {
   wb::WBContextUI *wbui = [mainWindow context];
-  
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(canUndo)])
-    return [(id)[[NSApp keyWindow] firstResponder] canUndo];
+  id firstResponder = [[NSApp keyWindow] firstResponder];
+
+  if ([firstResponder respondsToSelector: @selector(canUndo)])
+    return [firstResponder canUndo];
+  else if ([firstResponder isKindOfClass: [NSTextView class]])
+    return [[firstResponder undoManager] canUndo];
+  else if ([firstResponder isKindOfClass: [InnerView class]])
+    return true;
   else
     if (wbui->get_active_main_form())
       return wbui->get_active_main_form()->can_undo();
@@ -686,9 +695,12 @@ static bool validate_undo(WBMainWindow *mainWindow)
 static void call_redo(WBMainWindow *mainWindow)
 {
   wb::WBContextUI *wbui = [mainWindow context];
+  id firstResponder = [[NSApp keyWindow] firstResponder];
   
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(redo:)])
-    [[[NSApp keyWindow] firstResponder] tryToPerform: @selector(redo:) with:nil];
+  if ([firstResponder respondsToSelector: @selector(redo:)])
+    [firstResponder tryToPerform: @selector(redo:) with:nil];
+  else if ([firstResponder isKindOfClass: [NSTextView class]])
+    return [[firstResponder undoManager] redo];
   else
     if (wbui->get_active_main_form())
       wbui->get_active_main_form()->redo();
@@ -697,9 +709,14 @@ static void call_redo(WBMainWindow *mainWindow)
 static bool validate_redo(WBMainWindow *mainWindow)
 {
   wb::WBContextUI *wbui = [mainWindow context];
+  id firstResponder = [[NSApp keyWindow] firstResponder];
   
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(canRedo)])
-    return [(id)[[NSApp keyWindow] firstResponder] canRedo];
+  if ([firstResponder respondsToSelector: @selector(canRedo)])
+    return [firstResponder canRedo];
+  else if ([firstResponder isKindOfClass: [NSTextView class]])
+    return [[firstResponder undoManager] canRedo];
+  else if ([firstResponder isKindOfClass: [InnerView class]])
+    return true;
   else
     if (wbui->get_active_main_form())
       return wbui->get_active_main_form()->can_redo();
@@ -727,7 +744,6 @@ static bool validate_redo(WBMainWindow *mainWindow)
   std::list<std::string> commands;
   
   commands.push_back("overview.mysql_model");
-  commands.push_back("show_about");
   commands.push_back("diagram_size");
   //  commands.push_back("view_model_navigator");
   //  commands.push_back("view_catalog");
@@ -741,10 +757,8 @@ static bool validate_redo(WBMainWindow *mainWindow)
   //  commands.push_back("help_index");
   //  commands.push_back("help_version_check");
 
-  commands.push_back("wb.sidebarHide");
-
-  commands.push_back("wb.toggleSideBar");
-  commands.push_back("wb.toggleSchemataBar");
+  commands.push_back("wb.toggleSidebar");
+  commands.push_back("wb.toggleSecondarySidebar");
   commands.push_back("wb.toggleOutputArea");
   
   commands.push_back("wb.next_tab");
@@ -758,20 +772,17 @@ static bool validate_redo(WBMainWindow *mainWindow)
                                                boost::bind(validate_close_tab, mainWindow));
   _wbui->get_command_ui()->add_builtin_command("close_editor", boost::bind(call_close_editor, mainWindow), 
                                                boost::bind(validate_close_editor, mainWindow));
-  
-  _wbui->get_command_ui()->add_builtin_command("searchbox", boost::bind(call_search, mainWindow));
-                                               //boost::bind(validate_closetab), mainWindow));
- 
-  _wbui->get_command_ui()->add_builtin_command("search_replace", boost::bind(call_find_replace, true),
+
+  _wbui->get_command_ui()->add_builtin_command("find", boost::bind(call_find, mainWindow),
+                                               boost::bind(validate_find, mainWindow));
+  _wbui->get_command_ui()->add_builtin_command("find_replace", boost::bind(call_find_replace, true),
                                                boost::bind(validate_find_replace));
 
   _wbui->get_command_ui()->add_builtin_command("undo", boost::bind(call_undo, mainWindow),
                                                boost::bind(validate_undo, mainWindow));
   _wbui->get_command_ui()->add_builtin_command("redo", boost::bind(call_redo, mainWindow),
                                                boost::bind(validate_redo, mainWindow));
-  _wbui->get_command_ui()->add_builtin_command("find", boost::bind(call_find, mainWindow), 
-                                               boost::bind(validate_find, mainWindow));
-  _wbui->get_command_ui()->add_builtin_command("copy", boost::bind(call_copy, _wbui), 
+  _wbui->get_command_ui()->add_builtin_command("copy", boost::bind(call_copy, _wbui),
                                                boost::bind(validate_copy, _wbui));
   _wbui->get_command_ui()->add_builtin_command("cut", boost::bind(call_cut, _wbui), 
                                                boost::bind(validate_cut, _wbui));
@@ -845,6 +856,11 @@ static void flush_main_thread()
   }
 }
 
+- (void)applicationDidBecomeActive: (NSNotification*)notification
+{
+  base::NotificationInfo info;
+  base::NotificationCenter::get()->send("GNApplicationActivated", NULL, info);
+}
 
 static NSString *applicationSupportFolder()
 {
@@ -901,14 +917,12 @@ static NSString *applicationSupportFolder()
       char *path = getenv("PYTHONPATH");
       if (path)
       {
-        path = g_strdup_printf("PYTHONPATH=%s:%s:%s/mysql_libs.zip", path, _options->library_search_path.c_str(),
-                               _options->library_search_path.c_str());
+        path = g_strdup_printf("PYTHONPATH=%s:%s", path, _options->library_search_path.c_str());
         putenv(path);  // path should not be freed
       }
       else
       {
-        path = g_strdup_printf("PYTHONPATH=%s:%s/mysql_libs.zip", _options->library_search_path.c_str(),
-                               _options->library_search_path.c_str());
+        path = g_strdup_printf("PYTHONPATH=%s", _options->library_search_path.c_str());
         putenv(path); // path should not be freed
       }
       
@@ -970,7 +984,6 @@ static void init_mforms()
 {
   log_debug("Initializing mforms\n");
   
-  extern void cf_grttreeview_init();
   extern void cf_tabview_init();
   static BOOL inited= NO;
   
@@ -980,7 +993,6 @@ static void init_mforms()
   
     mforms_cocoa_init();
     cf_tabview_init();
-    cf_grttreeview_init();
   }
 }
 
@@ -1167,32 +1179,6 @@ static void init_mforms()
   }
 }
 
-
-- (IBAction)showAbout:(id)sender
-{
-  if (!aboutWindow)
-  {
-    [NSBundle loadNibNamed:@"About" owner:self];
-  
-#ifdef EDITION_OSS
-    [[[aboutWindow contentView] viewWithTag: 1] setImage:[NSImage imageNamed:@"wb_splashscreen_oss.png"]];
-#elif defined(EDITION_SE)
-    [[[aboutWindow contentView] viewWithTag: 1] setImage:[NSImage imageNamed:@"wb_splashscreen_se.png"]];
-#endif
-    
-    [[[aboutWindow contentView] viewWithTag: 2] setStringValue: [NSString stringWithFormat:@"%i.%i.%i%s%s",
-                                                                 APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, 
-                                                                 strlen(APP_RELEASE_TYPE)>0 ? " " : "", APP_RELEASE_TYPE]];
-    
-    [[[aboutWindow contentView] viewWithTag: 3] setStringValue: [NSString stringWithFormat:@"Revision %i", 
-                                                                 APP_REVISION_NUMBER]];
-    
-    [aboutWindow center];
-  }
-  [aboutWindow makeKeyAndOrderFront:nil];
-}
-
-
 - (IBAction)showDiagramProperties:(id)sender
 {
   WBDiagramSizeController *controller= [[WBDiagramSizeController alloc] initWithWBContext:_wbui];
