@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+#include "wb_context.h"
+
 #import "WBMainWindow.h"
 #import "WBMainController.h"
 #import "WBOverviewPanel.h"
@@ -178,8 +180,11 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (NSTabViewItem*)addTopPanel:(WBBasePanel*)panel
 {
   id tabItem= [[[NSTabViewItem alloc] initWithIdentifier:[panel identifier]] autorelease];
-    
-  [tabItem setView:[panel decoratedTopView]];
+
+  if ([panel title])
+    [tabItem setView:[panel decoratedTopView]];
+  else
+    [tabItem setView:[panel topView]]; // no toolbar for Home tab
   [tabItem setLabel:[panel title]];
   if ([panel respondsToSelector: @selector(initialFirstResponder)])
     [tabItem setInitialFirstResponder: [(id)panel initialFirstResponder]];
@@ -408,7 +413,7 @@ void setup_mforms_app(WBMainWindow *mwin);
       if (oldResponder != nil && [oldResponder isKindOfClass:[NSTextView class]] && [oldResponder isFieldEditor])
         oldResponder= [[oldResponder delegate] isKindOfClass:[NSResponder class]] ? [oldResponder delegate] : nil;
       
-      if ([oldResponder isKindOfClass: [NSTableView class]])
+      if ([oldResponder isKindOfClass: [NSTableView class]] || [oldResponder isKindOfClass: [NSTextView class]])
       {
         [window makeFirstResponder: nil];
         [window makeFirstResponder: oldResponder];
@@ -586,17 +591,6 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
-- (NSImage*)tabView:(NSTabView *)tabView iconForItem:(NSTabViewItem *)tabViewItem
-{
-  id form= [_panels objectForKey:[tabViewItem identifier]]; 
-  bec::UIForm *uif = [form formBE];
-  if (uif && uif->get_form_context_name() == WB_CONTEXT_HOME_GLOBAL)
-    return _homeIcon;
-  return nil;
-}
-
-//--------------------------------------------------------------------------------------------------
-
 - (IBAction)handleMenuAction:(id)sender
 {
   // Close Other Tabs Like This from the main tab
@@ -836,7 +830,7 @@ void setup_mforms_app(WBMainWindow *mwin);
   bec::UIForm *main_form = _wbui->get_active_main_form();
   if (main_form && main_form->get_toolbar())
   {
-    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("searchbox");
+    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("find");
     if (item)
     {
       id view = item->get_data();
@@ -857,10 +851,11 @@ void setup_mforms_app(WBMainWindow *mwin);
   bec::UIForm *main_form= _wbui->get_active_main_form();
   if (main_form && main_form->get_frontend_data() && main_form->get_toolbar())
   {
-    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("searchbox");
+    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("find");
     if (item)
     {
       id form= reinterpret_cast<id>(main_form->get_frontend_data());
+      [self focusSearchField: nil];
       if ([form respondsToSelector:@selector(searchString:)])
         [form searchString: [NSString stringWithCPPString: item->get_text()]];
     }
@@ -1060,8 +1055,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     _closedBottomTabs= [[NSMutableArray alloc] init];
     
     _defaultMainMenu = [[NSApp mainMenu] retain];
-    
-    _homeIcon = [[NSImage imageNamed: @"WB_Home_mac.png"] retain];
   }
   return self;
 }
@@ -1186,8 +1179,16 @@ public:
   {
     WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
     
-    if (panel) 
-      [self closePanel: panel];
+    if (panel)
+    {
+      // close the panel bypassing the willClose handler (that should've already been called much earlier)
+      id identifier = [[[panel identifier] retain] autorelease];
+
+      if ([self->topTabView indexOfTabViewItemWithIdentifier: identifier] != NSNotFound)
+        [self closeTopPanelWithIdentifier: identifier];
+      else
+        [self closeBottomPanelWithIdentifier: identifier];
+    }
   }
   
   virtual void set_view_title(mforms::AppView *view, const std::string &title)
@@ -1236,10 +1237,15 @@ static std::string get_resource_path(mforms::App *app, const std::string &file)
   
   if (!file.empty() && file[0] == '/')
     return file;
-  
+
+  std::string path;
   if (g_str_has_suffix(file.c_str(), ".png"))
-    return bec::IconManager::get_instance()->get_icon_path(file);
-  else
+  {
+    path = bec::IconManager::get_instance()->get_icon_path(file);
+    if (!path.empty())
+      return path;
+  }
+
   {
     char *fn = g_path_get_basename(file.c_str());
     NSString *filename = [NSString stringWithUTF8String:fn];
@@ -1249,6 +1255,13 @@ static std::string get_resource_path(mforms::App *app, const std::string &file)
                                              ofType: [filename pathExtension]];
     if (str)
       return [str UTF8String];
+
+    // Look for the same image but with tiff extension, in case the actual image is
+    // a combined art file.
+    str = [[NSBundle mainBundle] pathForResource: [filename stringByDeletingPathExtension]
+                                          ofType: @"tiff"];
+    if (str)
+      return [str UTF8String];
   }
   return "";
 }
@@ -1277,12 +1290,12 @@ static std::string get_executable_path(mforms::App *app, const std::string &file
 
 //--------------------------------------------------------------------------------------------------
 
-static std::string get_main_window_bounds(mforms::App *app)
+static base::Rect get_main_window_bounds(mforms::App *app)
 {
   WBMainWindow *mwin = app->get_data();
-    
+
   NSRect r = [[mwin window] frame];
-  return base::strfmt("%i %i %i %i", (int)NSMinX(r), (int)NSMinY(r), (int)NSWidth(r), (int)NSHeight(r));
+  return base::Rect(NSMinX(r), NSMaxY(r), NSWidth(r), NSHeight(r));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1360,10 +1373,24 @@ static base::Color get_system_color(mforms::SystemColor color)
       NSColor *c = [[NSColor controlHighlightColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
       return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
     }
+    case mforms::SystemColorContainer:
+    {
+      NSColor *c = [[NSColor controlBackgroundColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
+      return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
+    }
   }
   return base::Color::Color(1, 0, 0);
 }
 
+static float backing_scale_factor(mforms::App *app)
+{
+  WBMainWindow *controller = app->get_data();
+  if ([controller.window respondsToSelector: @selector(backingScaleFactor)])
+    return [controller.window backingScaleFactor]; // Available since 10.7.
+  return 1.0;
+}
+
+
 void setup_mforms_app(WBMainWindow *mwin)
 {
   mforms::ControlFactory *cf = mforms::ControlFactory::get_instance();
@@ -1372,15 +1399,16 @@ void setup_mforms_app(WBMainWindow *mwin)
   mforms::App::instantiate(new MainWindowDockingPoint(mwin), true);
   mforms::App::get()->set_data(mwin);
   
-  cf->_app_impl.get_resource_path= get_resource_path;
-  cf->_app_impl.get_executable_path= get_executable_path;
-  cf->_app_impl.set_status_text= set_status_text;
-  cf->_app_impl.get_bounds= get_main_window_bounds;
-  cf->_app_impl.begin_thread_loop= alloc_ar_pool;
-  cf->_app_impl.end_thread_loop= free_ar_pool;
-  cf->_app_impl.enter_event_loop= enter_event_loop;
-  cf->_app_impl.exit_event_loop= exit_event_loop;
-  cf->_app_impl.get_system_color= get_system_color;
+  cf->_app_impl.get_resource_path = get_resource_path;
+  cf->_app_impl.get_executable_path = get_executable_path;
+  cf->_app_impl.set_status_text = set_status_text;
+  cf->_app_impl.get_application_bounds = get_main_window_bounds;
+  cf->_app_impl.begin_thread_loop = alloc_ar_pool;
+  cf->_app_impl.end_thread_loop = free_ar_pool;
+  cf->_app_impl.enter_event_loop = enter_event_loop;
+  cf->_app_impl.exit_event_loop = exit_event_loop;
+  cf->_app_impl.get_system_color = get_system_color;
+  cf->_app_impl.backing_scale_factor = backing_scale_factor;
 }
 
 //--------------------------------------------------------------------------------------------------
