@@ -1,5 +1,5 @@
 /* 
- * Copyright © 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,46 +14,62 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+#include "tree_model.h"
+#include "wb_overview_physical.h"
+#include "wb_context_model.h"
+#include "workbench/wb_context.h"
+
 #import "WBModelOverviewPanel.h"
 #import "WBOverviewPanel.h"
 #import "WBModelSidebarController.h"
 #import "WBObjectDescriptionController.h"
 #import "GRTListDataSource.h"
 #import "GRTTreeDataSource.h"
+#import "MTabSwitcher.h"
 #import "WBTabView.h"
-#include "wb_overview_physical.h"
+#import "MFView.h"
 
 @implementation WBModelOverviewPanel
 
-- (void)setupWithWBContextUI:(wb::WBContextUI*)wbui
+- (id)initWithWBContextUI:(wb::WBContextUI*)wbui
 {
-  _wbui= wbui;
-  
-  [NSBundle loadNibNamed: @"WBModelOverview" owner: self];
-  [(id)editorTabView createDragger];  
+  self = [super init];
+  if (self)
+  {
+    _wbui= wbui;
+    
+    [NSBundle loadNibNamed: @"WBModelOverview" owner: self];
+    [(id)editorTabView createDragger];
 
-  [overview setupWithOverviewBE: wbui->get_physical_overview()];
-  [sidebarController setupWithWBContextUI: wbui];
-  
-  [descriptionController setWBContext: wbui];
+    [overview setupWithOverviewBE: wbui->get_physical_overview()];
+    [sidebarController setupWithWBContextUI: wbui];
+    [mSwitcherT setTabStyle: MPaletteTabSwitcherSmallText];
+    [mSwitcherB setTabStyle: MPaletteTabSwitcherSmallText];
+    [descriptionController setWBContext: wbui];
 
-  [topView setDividerThickness: 1];
-  
-  [overview rebuildAll];
+    [topView setDividerThickness: 1];
+    [topView setBackgroundColor: [NSColor colorWithDeviceWhite:128/255.0 alpha:1.0]];
 
-  [sidebarController refresh];
+   // [overview rebuildAll];
+    [overview performSelector:@selector(rebuildAll) withObject:nil afterDelay:0.1];
 
-  sidebarHidden = NO;
-  sidebarAtRight = NO;
-  [self setRightSidebar: wbui->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0)];
-  lastSidebarWidth = [sidebar frame].size.width;
+    [sidebarController refresh];
+
+    grtm = _wbui->get_wb()->get_grt_manager();
+
+//    [self setRightSidebar: wbui->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0)];
+    [topView setAutosaveName: @"modelSplitPosition"];
+
+    [self restoreSidebarsFor: "ModelOverview" toolbar: wbui->get_physical_overview()->get_toolbar()];
+  }
+  return self;
 }
 
 
 - (void)dealloc
 {
-  [[sidebarController userTypeDS] release];
-  [[sidebarController historyDS] release];
+  [NSObject cancelPreviousPerformRequestsWithTarget: overview];
+
   [[sidebarController catalogDS] release];
   [sidebarController invalidate];
   
@@ -97,6 +113,16 @@
 
 - (void)didActivate
 {
+  NSView *view = nsviewForView(_wbui->get_wb()->get_model_context()->shared_secondary_sidebar());
+  if ([view superview])
+  {
+    [view retain];
+    [view removeFromSuperview];
+  }
+  [secondarySidebar addSubview: view];
+  [view setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable|NSViewMinXMargin|NSViewMinYMargin|NSViewMaxXMargin|NSViewMaxYMargin];
+  [view setFrame: [secondarySidebar bounds]];
+
   [sidebarController refresh];
 }
 
@@ -122,18 +148,19 @@
 {
   if (subview == bottomContainer)
     return NO;
-  return YES;
+
+  return [super splitView: splitView shouldAdjustSizeOfSubview: subview];
 }
 
 //--------------------------------------------------------------------------------------------------
-
+/*
 - (void)setRightSidebar:(BOOL)flag
 {
   sidebarAtRight = flag;
   
   id view1 = [[topView subviews] objectAtIndex: 0];
   id view2 = [[topView subviews] objectAtIndex: 1];
-  
+
   if (sidebarAtRight)
   {
     if (view2 != sidebar)
@@ -141,7 +168,7 @@
       [[view1 retain] autorelease];
       [view1 removeFromSuperview];
       [topView addSubview: view1];
-    }    
+    }
   }
   else
   {
@@ -152,36 +179,6 @@
       [topView addSubview: view1];
     }
   }
-}
-
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Executes commands sent by the main form that should be handled here.
- */
-- (void) performCommand: (const std::string) command
-{
-  if (command == "wb.sidebarHide") // Actually means toggle.
-  {
-    sidebarHidden = !sidebarHidden;
-    if (!sidebarHidden)
-    {
-      if (sidebarAtRight)
-        [topView setPosition: NSWidth([topView frame])-lastSidebarWidth ofDividerAtIndex: 0];
-      else
-        [topView setPosition: lastSidebarWidth ofDividerAtIndex: 0];
-    }
-    else
-    {
-      lastSidebarWidth = [sidebar frame].size.width;
-      if (sidebarAtRight)
-        [topView setPosition: NSWidth([topView frame]) ofDividerAtIndex: 0];
-      else
-        [topView setPosition: 0 ofDividerAtIndex: 0];
-    }
-  }
-}
-
-//--------------------------------------------------------------------------------------------------
+}*/
 
 @end
