@@ -76,7 +76,7 @@ namespace MySQL.Controls
 
     protected override void Dispose(bool disposing)
     {
-      if (disposing && (Model != null))
+      if (Model != null)
       {
         Model.Dispose();
         Model = null;
@@ -508,6 +508,32 @@ namespace MySQL.Controls
         EditMode = DataGridViewEditMode.EditOnKeystrokeOrF2;
     }
 
+    protected override void OnCellClick(DataGridViewCellEventArgs e)
+    {
+      // Switch to row selection mode if the user clicked on the marker column, so we can select entire rows.
+      // Otherwise use cell selection.
+      DataGridViewSelectionMode newMode;
+      if (e.ColumnIndex == -1)
+        newMode = DataGridViewSelectionMode.FullRowSelect;
+      else
+        newMode = DataGridViewSelectionMode.CellSelect;
+
+      if (newMode == SelectionMode || e.RowIndex == -1)
+        base.OnCellClick(e);
+      else
+      {
+        // Manually handle cell clicks if we have to switch the selection mode or we need a second click.
+        SelectionMode = newMode;
+        if (newMode == DataGridViewSelectionMode.CellSelect)
+        {
+          CurrentCell = this[e.ColumnIndex, e.RowIndex];
+          CurrentCell.Selected = true;
+        }
+        else
+          Rows[e.RowIndex].Selected = true;
+      }
+    }
+
     protected override bool ProcessDataGridViewKey(KeyEventArgs e)
     {
       switch (e.KeyCode)
@@ -519,6 +545,11 @@ namespace MySQL.Controls
         case Keys.End:
           FirstDisplayedScrollingColumnIndex = Columns.Count - 1;
           break;
+          
+        case Keys.F2: // Explicitly handle F2 as the grid doesn't select the cell context.
+          SelectionMode = DataGridViewSelectionMode.CellSelect;
+          BeginEdit(true);
+          return true;
       }
       return base.ProcessDataGridViewKey(e);
     }
@@ -576,13 +607,12 @@ namespace MySQL.Controls
       List<DataGridViewRow> rows = GetSelectedRows();
       List<int> indexes = new List<int>(rows.Count);
       foreach (DataGridViewRow row in rows)
-      {
         indexes.Add(row.Index);
-      }
-      column = (0 < SelectedColumns.Count) ? SelectedColumns[SelectedColumns.Count - 1].Index : -1;
-      if (-1 == column)
+
+      column = (SelectedColumns.Count > 0) ? SelectedColumns[SelectedColumns.Count - 1].Index : -1;
+      if (column == -1)
       {
-        if (0 < SelectedCells.Count)
+        if (SelectedCells.Count > 0)
         {
           if (!SelectedCells[SelectedCells.Count - 1].OwningRow.Selected)
             column = SelectedCells[SelectedCells.Count - 1].ColumnIndex;
