@@ -27,7 +27,10 @@ namespace MySQL.GUI.Workbench
 
     private void reportBugButton_Click(object sender, EventArgs e)
     {
-      wbContext.report_bug(errorInfo);
+      if (wbContext.is_commercial())
+        System.Diagnostics.Process.Start("http://support.oracle.com");
+      else
+        wbContext.report_bug(errorInfo);
     }
 
     private void copyInfoButton_Click(object sender, EventArgs e)
