@@ -26,6 +26,7 @@ using System.Windows.Forms;
 
 using Aga.Controls.Tree;
 
+using MySQL.Base;
 using MySQL.Controls;
 using MySQL.Forms;
 using MySQL.Grt;
@@ -38,7 +39,7 @@ using MySQL.Workbench;
 
 namespace MySQL.GUI.Workbench
 {
-  public partial class MainForm : Form, IMySQLMainForm
+  public partial class MainForm : Form, IWorkbenchObserver
   {
     #region Member Variables
 
@@ -60,12 +61,10 @@ namespace MySQL.GUI.Workbench
 
     // A timer for helping backend timers.
     private System.Windows.Forms.Timer timer = null;
-    
+
     // Special Forms
     private ModelOverviewForm workbenchPhysicalOverviewForm = null;
 
-    private Form findForm = null;
-
     private bool shuttingDown = false;
     private ImageList tabImageList = null;
 
@@ -97,11 +96,10 @@ namespace MySQL.GUI.Workbench
 
       tabImageList = new ImageList();
       tabImageList.ColorDepth = ColorDepth.Depth32Bit;
-      tabImageList.ImageSize = new Size(16, 16);
+      tabImageList.ImageSize = new Size(18, 16);
       ImageListHelper.Add(ApplicationCommand(AppCommand.AppGetResourcePath, null, "WB_Home.png"), tabImageList);
       contentTabControl.ImageList = tabImageList;
 
-
       // Create a timer to be triggered when the backend needs
       timer = new System.Windows.Forms.Timer();
       timer.Tick += new EventHandler(timer_Tick);
@@ -134,10 +132,15 @@ namespace MySQL.GUI.Workbench
       host = new ToolStripControlHost(statusStripImg);
       host.Alignment = ToolStripItemAlignment.Right;
       mainStatusStrip.Items.Add(host);
+
+      ManagedNotificationCenter.AddObserver(this, "GNColorsChanged");
+      ManagedNotificationCenter.AddObserver(this, "GNFocusChanged");
     }
 
     void Destroy()
     {
+      ManagedNotificationCenter.RemoveObserver(this, "");
+
       // All documents are closed in Form_Closing.
     }
 
@@ -150,7 +153,7 @@ namespace MySQL.GUI.Workbench
     public string StatusBarText
     {
       get { return statusText.Text; }
-      set 
+      set
       {
         if (InvokeRequired)
         {
@@ -168,6 +171,24 @@ namespace MySQL.GUI.Workbench
 
     #endregion
 
+    #region IWorkbenchObserver interface
+
+    public void HandleNotification(string name, IntPtr sender, Dictionary<string, string> info)
+    {
+      switch (name)
+      {
+        case "GNColorsChanged":
+          UpdateColors();
+          break;
+
+        case "GNFocusChanged":
+          wbContext.validate_edit_menu();
+          break;
+      }
+    }
+
+    #endregion
+
     #region Properties
 
     public ModelOverviewForm WorkbenchPhysicalOverviewForm
@@ -246,19 +267,19 @@ namespace MySQL.GUI.Workbench
 
       TextBox input = new TextBox();
       input.Location = new Point(10, 30);
-      input.Size = new Size(dialog.Size.Width - (input.Location.X*2 + 1), 50);
+      input.Size = new Size(dialog.Size.Width - (input.Location.X * 2 + 1), 50);
       if (flags != 0)
         input.PasswordChar = '*';
 
       Button ok = new Button();
       ok.Text = "OK"; // if you rename this also update OnRequestInputButton()
-      ok.Location = new Point(dialog.Size.Width/2 - ok.Size.Width - 10, 
+      ok.Location = new Point(dialog.Size.Width / 2 - ok.Size.Width - 10,
         input.Location.Y + input.Size.Height + 10);
       ok.Click += new EventHandler(OnRequestInputButton);
-      
+
       Button cancel = new Button();
       cancel.Text = "Cancel";
-      cancel.Location = new Point(dialog.Size.Width/2 + 10, 
+      cancel.Location = new Point(dialog.Size.Width / 2 + 10,
         input.Location.Y + input.Size.Height + 10);
       cancel.Click += new EventHandler(OnRequestInputButton);
 
@@ -283,7 +304,7 @@ namespace MySQL.GUI.Workbench
         return "";
 
       String result = "";
-      switch (command) 
+      switch (command)
       {
         case AppCommand.AppQuit: // Not yet used.
           Close();
@@ -314,31 +335,31 @@ namespace MySQL.GUI.Workbench
       switch (refresh)
       {
         case RefreshType.RefreshCloseDocument:
-        {
-          if (workbenchPhysicalOverviewForm != null)
           {
-            workbenchPhysicalOverviewForm.ResetDocument(true);
-            workbenchPhysicalOverviewForm.Close();
-            workbenchPhysicalOverviewForm = null;
+            if (workbenchPhysicalOverviewForm != null)
+            {
+              workbenchPhysicalOverviewForm.ResetDocument(true);
+              workbenchPhysicalOverviewForm.Close();
+              workbenchPhysicalOverviewForm = null;
+            }
+
+            wbContext.flush_idle_tasks();
+            wbContext.close_document_finish();
+
+            break;
           }
-
-          wbContext.flush_idle_tasks();
-          wbContext.close_document_finish();
-
-          break;
-        }
         case RefreshType.RefreshNewDiagram:
-        {
-          BaseWindowsCanvasView canvas = BaseWindowsCanvasView.GetFromFixedId(ptr);
-          if (canvas != null)
           {
-            // Open only diagrams which were open when this model was closed last time.
-            ModelDiagramForm modelDiagramForm = canvas.GetOwnerForm() as ModelDiagramForm;
-            if (modelDiagramForm != null && !modelDiagramForm.DiagramWrapper.is_closed())
-              DockDocument(modelDiagramForm, true, true);
+            BaseWindowsCanvasView canvas = BaseWindowsCanvasView.GetFromFixedId(ptr);
+            if (canvas != null)
+            {
+              // Open only diagrams which were open when this model was closed last time.
+              ModelDiagramForm modelDiagramForm = canvas.GetOwnerForm() as ModelDiagramForm;
+              if (modelDiagramForm != null && !modelDiagramForm.DiagramWrapper.is_closed())
+                DockDocument(modelDiagramForm, true, true);
+            }
+            break;
           }
-          break;
-        }
 
         case RefreshType.RefreshNewModel:
           ShowPhysicalOverviewForm(new NodeId(str)); // Make sure the physical overview exists.
@@ -384,9 +405,6 @@ namespace MySQL.GUI.Workbench
             Program.HandleException(e);
           }
           break;
-
-        default:
-          break;
       }
 
       ResumeLayout();
@@ -590,8 +608,8 @@ namespace MySQL.GUI.Workbench
           editorForm.Hide();
       }
     }
-#endregion
- 
+    #endregion
+
     #region DockingPointDelegate
 
     public bool select_view(AppViewImpl view)
@@ -739,9 +757,9 @@ namespace MySQL.GUI.Workbench
         else
           if (activeControl is ScintillaControl)
             (activeControl as ScintillaControl).Cut();
-            else
-              if (wbContext.edit_can_copy()) // TODO: check really necessary?
-                wbContext.edit_cut();
+          else
+            if (wbContext.edit_can_copy()) // TODO: check really necessary?
+              wbContext.edit_cut();
     }
 
     private bool EditCanCut()
@@ -821,11 +839,11 @@ namespace MySQL.GUI.Workbench
         return wbContext.edit_can_select_all();
 
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase || activeControl is ComboBox|| activeControl is ScintillaControl ||
+      if (activeControl is TextBoxBase || activeControl is ComboBox || activeControl is ScintillaControl ||
         activeControl is DataGridView)
-          return true;
-        else
-          return wbContext.edit_can_select_all();
+        return true;
+      else
+        return wbContext.edit_can_select_all();
     }
 
     private void EditDelete()
@@ -867,14 +885,28 @@ namespace MySQL.GUI.Workbench
         // Not an editor control. So try focusing the search box in the toolbar (if there's any).
         IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
         if (wbDocument != null)
+        {
           wbContext.focus_search_box(wbDocument.BackendForm);
+
+          // and then search
+          string searchString = wbContext.get_search_string(wbDocument.BackendForm);
+          if (searchString != "")
+          {
+            if (!wbContext.try_searching_diagram(searchString))
+            {
+              // if active main tab is Overview, search it
+              if (contentTabControl.ActiveDocument == workbenchPhysicalOverviewForm)
+                workbenchPhysicalOverviewForm.SearchAndFocusNode(searchString);
+            }
+          }
+        }
       }
     }
 
     private bool EditCanFind()
     {
       IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
-      if (wbDocument == null)
+      if (wbDocument == null && !EditCanFindReplace())
         return false;
 
       return true;
@@ -895,30 +927,35 @@ namespace MySQL.GUI.Workbench
       return false;
     }
 
-    private void SearchActiveTab()
+    private void UpdateColors()
     {
-      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
-      if (wbDocument == null)
-        return;
+      contentTabControl.RenderWithGlow = false;
+      contentTabControl.UpdateColors();
+      contentTabControl.BackgroundColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      mainStatusStrip.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      mainStatusStrip.ForeColor = Conversions.GetApplicationColor(ApplicationColor.AppColorStatusbar, true);
+      statusText.ForeColor = Conversions.GetApplicationColor(ApplicationColor.AppColorStatusbar, true);
 
-      string searchString = wbContext.get_search_string(wbDocument.BackendForm);
-      if (searchString != "")
+      foreach (ITabDocument document in contentTabControl.Documents)
       {
-        if (!wbContext.try_searching_diagram(searchString))
-        {
-          // if active main tab is Overview, search it
-          if (contentTabControl.ActiveDocument == workbenchPhysicalOverviewForm)
-            workbenchPhysicalOverviewForm.SearchAndFocusNode(searchString);
-        }
+        if (document is IWorkbenchDocument)
+          (document as IWorkbenchDocument).UpdateColors();
+        else
+          if (document is AppViewDockContent)
+            (document as AppViewDockContent).UpdateColors();
       }
     }
-    
-    public void RegisterCommands()
+
+    /// <summary>
+    /// Some initialization done after everything else is set up and available.
+    /// </summary>
+    public void PostInit()
     {
-      List<String> commands= new List<string>();
+      UpdateColors();
+
+      List<String> commands = new List<string>();
 
       commands.Add("overview.mysql_model");
-      commands.Add("show_about");
       commands.Add("diagram_size");
       commands.Add("view_model_navigator");
       commands.Add("view_catalog");
@@ -929,11 +966,16 @@ namespace MySQL.GUI.Workbench
       commands.Add("view_undo_history");
       commands.Add("reset_layout");
       commands.Add("wb.page_setup");
-      commands.Add("closetab");
       commands.Add("help_version_check");
-      commands.Add("wb.sidebarHide");
+      commands.Add("wb.toggleSidebar");
+      commands.Add("wb.toggleSecondarySidebar");
+      commands.Add("closetab");
       commands.Add("close_tab");
 
+      // SQL IDE specific commands.
+      commands.Add("wb.toggleOutputArea");
+      commands.Add("close_editor");
+
       wbContext.add_frontend_commands(commands);
 
       // special command in Windows implemented in wrapper to perform 
@@ -942,47 +984,45 @@ namespace MySQL.GUI.Workbench
         EditUndo, EditCanUndo,
         EditRedo, EditCanRedo,
         EditCopy, EditCanCopy,
-        EditCut, EditCanCut, EditPaste, EditCanPaste, 
+        EditCut, EditCanCut, EditPaste, EditCanPaste,
         EditSelectAll, EditCanSelectAll, EditDelete, EditCanDelete,
-        SearchActiveTab,
         EditFind, EditCanFind,
         EditFindReplace, EditCanFindReplace);
     }
 
     public void PerformCommand(String command)
     {
-      if (command == "overview.mysql_model")
-        ShowPhysicalOverviewForm();
-      else if (command == "show_about")
+      switch (command)
       {
-        new WorkbenchSplashScreen(true).ShowDialog();
+        case "overview.mysql_model":
+          ShowPhysicalOverviewForm();
+          break;
+        case "diagram_size":
+          ShowDiagramOptionsForm();
+          break;
+        case "reset_layout":
+          ResetWindowLayout();
+          break;
+        case "wb.page_setup":
+          ShowPageSettingsForm();
+          break;
+        case "closetab":
+          CloseActiveTab();
+          break;
+        case "close_tab":
+          contentTabControl.CloseTabPage(contentTabControl.SelectedTab);
+          break;
+        case "help_version_check":
+          Program.CheckForNewVersion();
+          break;
+        case "wb.toggleSidebar":
+        case "wb.toggleSecondarySidebar":
+          ForwardCommandToActivDocument(command);
+          break;
+        default:
+          ForwardCommandToActivDocument(command);
+          break;
       }
-      else if (command == "diagram_size")
-        ShowDiagramOptionsForm();
-      else if (command == "reset_layout")
-        ResetWindowLayout();
-      else if (command == "show_find_dialog")
-      {
-        ShowSimpleFindForm();
-      }
-      else if (command == "wb.page_setup")
-        ShowPageSettingsForm();
-      else if (command == "closetab")
-      {
-        CloseActiveTab();
-      }
-      else if (command == "close_tab")
-      {
-        contentTabControl.CloseTabPage(contentTabControl.SelectedTab);
-      }
-      else if (command.Equals("help_version_check"))
-        Program.CheckForNewVersion();
-      else if (command.Equals("wb.sidebarHide"))
-      {
-        ForwardCommandToAllDocuments(command);
-      }
-      else
-        ForwardCommandToActivDocument(command);
     }
 
     /// <summary>
@@ -1012,27 +1052,18 @@ namespace MySQL.GUI.Workbench
         {
           // Reset host if the tab control is not what we are looking for and start over.
           FlatTabControl.TabStyleType style = (host as FlatTabControl).TabStyle;
-          if (style == FlatTabControl.TabStyleType.TopNormal || style == FlatTabControl.TabStyleType.TopHanging
-            || style == FlatTabControl.TabStyleType.TopTransparent)
+          if (style == FlatTabControl.TabStyleType.TopNormal || style == FlatTabControl.TabStyleType.TopTransparent)
             break;
         }
         control = host;
         host = null;
       }
 
-      if (host != null)
+      TabPage page = control as TabPage;
+      if (host != null && page.Text.Length > 0) // Don't close the home screen.
         (host as FlatTabControl).CloseTabPage(control as TabPage);
     }
 
-    private void ShowSimpleFindForm()
-    {
-      if (findForm == null)
-      {
-        findForm = new ObjectFindForm(wbContext);
-      }
-      findForm.ShowDialog();
-    }
-
     private void ShowPageSettingsForm()
     {
       PageSettingsForm form = new PageSettingsForm(wbContext);
@@ -1040,7 +1071,7 @@ namespace MySQL.GUI.Workbench
       form.ShowDialog();
       form.Dispose();
     }
-        
+
     private void ShowDiagramOptionsForm()
     {
       DiagramOptionsForm form = new DiagramOptionsForm(wbContext);
@@ -1105,6 +1136,15 @@ namespace MySQL.GUI.Workbench
 
     #region Other Application Logic
 
+    protected override void WndProc(ref Message m)
+    {
+      if (m.Msg == (int)WM.ACTIVATEAPP && m.WParam != IntPtr.Zero)
+      {
+        ManagedNotificationCenter.Send("GNApplicationActivated", IntPtr.Zero);
+      }
+      base.WndProc(ref m);
+    }
+
     /// <summary>
     /// Try to find the given file in known locations. If name is empty return the absolute application
     /// path (as common root for all resource subfolders).
@@ -1116,7 +1156,7 @@ namespace MySQL.GUI.Workbench
         return Environment.CurrentDirectory;
 
       string result = "";
-      if (File.Exists(name))
+      if (File.Exists(name) || Directory.Exists(name))
         return name;
       foreach (String path in resourcePaths)
         if (File.Exists(path + "/" + name))
@@ -1197,7 +1237,13 @@ namespace MySQL.GUI.Workbench
         int topAreaHeight = contentTabControl.ItemSize.Height + contentTabControl.Margin.Top +
           toolbarHeight + Margin.Top;
 
-        Win32.MARGINS margins = new Win32.MARGINS(2, topAreaHeight, 2, 0);
+        // For Win8 glass area extension is producing weird results (white frames).
+        // So we only extend the top area.
+        Win32.MARGINS margins;
+        if (ControlUtilities.IsWin8OrAbove())
+          margins = new Win32.MARGINS(0, topAreaHeight - 3, 0, 0);
+        else
+          margins = new Win32.MARGINS(3, topAreaHeight, 3, 2);
         Win32.DwmExtendFrameIntoClientArea(Handle, margins);
       }
     }
@@ -1208,7 +1254,8 @@ namespace MySQL.GUI.Workbench
     /// </summary>
     /// <param name="document">The document to adjust, which comes with the actual content.</param>
     /// <param name="backend">The backend UI form used to get toolbar and menu.</param>
-    private void SetupDockLayout(ITabDocument document, UIForm backend)
+    /// <param name="withPadding">Indicates if we need padding around the content or not.</param>
+    private void SetupDockLayout(ITabDocument document, UIForm backend, bool withPadding)
     {
       // Setting up the layout requires some additional nesting of controls.
       // A tab document should have only one container as top level control, which we re-host.
@@ -1224,10 +1271,10 @@ namespace MySQL.GUI.Workbench
 
       // Content area.
       DrawablePanel contentPanel = new DrawablePanel();
-      contentPanel.BackColor = Color.FromArgb(255, 40, 55, 82);
+      contentPanel.BackColor = MySQL.Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
       contentPanel.CustomBackground = false;
       contentPanel.Dock = DockStyle.Fill;
-      contentPanel.Padding = new Padding(7);
+      contentPanel.Padding = withPadding ? new Padding(6) : new Padding(0);
       root.Controls.Add(contentPanel);
       if (content != null)
       {
@@ -1255,34 +1302,35 @@ namespace MySQL.GUI.Workbench
         AppViewDockContent dockContent = document as AppViewDockContent;
         toolbar = dockContent.GetAppView().GetToolBar();
         if (toolbar == null)
-          toolbar = wbContext.toolbar_for_form(null);
-        menu = dockContent.GetAppView().GetMenuBar();
-        if (menu == null)
-          menu = wbContext.menu_for_form(null);
+          toolbar = wbContext.toolbar_for_form(null); // Currently never returns a toolbar.
+        menu = wbContext.menu_for_appview(dockContent);
       }
       DrawablePanel menuPanel = new DrawablePanel();
       if (toolbar != null || menu != null)
       {
-        // For menu + toolbar we have a special gradient background, which we implement by
-        // placing them on another panel.
         menuPanel.BackColor = Color.Transparent;
         menuPanel.CustomBackground = true;
         menuPanel.Dock = DockStyle.Top;
-        menuPanel.Padding = new Padding(5, 0, 5, 0);
         menuPanel.AutoSize = true;
-        menuPanel.PaintBackground += new EventHandler<PaintEventArgs>(menuPanel_PaintBackground);
-
         root.Controls.Add(menuPanel);
         if (toolbar != null)
         {
+          toolbar.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainTab, false);
+
           menuPanel.Controls.Add(toolbar);
           toolbar.Dock = DockStyle.Top;
-          toolbar.AutoSize = true;
-          toolbar.Padding = new Padding(0, 0, 0, 3);
         }
 
         if (menu != null)
         {
+          if (menu.Renderer is Win8MenuStripRenderer)
+            (menu.Renderer as Win8MenuStripRenderer).Logo = Resources.oracle_logo_white;
+          else
+            if (menu.Renderer is TransparentMenuStripRenderer)
+              (menu.Renderer as TransparentMenuStripRenderer).Logo = Resources.oracle_logo_white;
+
+          menu.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainTab, false);
+
           menuPanel.Controls.Add(menu);
           menu.Dock = DockStyle.Top;
           menu.AutoSize = true;
@@ -1333,8 +1381,14 @@ namespace MySQL.GUI.Workbench
     private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
       // Reset status bar to clean up any previously done action message.
+      // Hide the statusbar on the home page, though.
       StatusBarText = "Ready";
 
+      // The home screen tab has no text since we only use an icon there.
+      // So we can use this as quick check here.
+      mainStatusStrip.Visible = contentTabControl.SelectedIndex > -1 &&
+        (contentTabControl.SelectedTab.Text.Length > 0);
+
       // Disable embedded menus for all inactive pages.
       ITabDocument[] documents = contentTabControl.DocumentsToArray();
       for (int i = 0; i < documents.Length; i++)
@@ -1365,7 +1419,7 @@ namespace MySQL.GUI.Workbench
           }
         }
       }
-      
+
       // For the glass effect behind the tabs (and the proper content drawing) we need the
       // height of the tabs as well as menu and toolbar of the active tab.
       int topAreaHeight = 0;
@@ -1379,6 +1433,7 @@ namespace MySQL.GUI.Workbench
         if (tabControl.ActiveDocument != null)
         {
           ITabDocument activeDocument = tabControl.ActiveDocument;
+          activeDocument.Activate();
           topAreaHeight += activeDocument.ToolbarHeight;
 
           if (activeDocument is IWorkbenchDocument)
@@ -1396,7 +1451,7 @@ namespace MySQL.GUI.Workbench
             }
             else
             {
-              wbContext.set_active_form(wbDoc.BackendForm);  
+              wbContext.set_active_form(wbDoc.BackendForm);
             }
           }
           else if (activeDocument is Plugins.DockablePlugin)
@@ -1466,16 +1521,20 @@ namespace MySQL.GUI.Workbench
     /// dock it to the currently active main document.</param>
     public void DockDocument(ITabDocument document, bool main, bool activate)
     {
+      if (document is IWorkbenchDocument)
+        (document as IWorkbenchDocument).UpdateColors();
+      else
+        if (document is AppViewDockContent)
+          (document as AppViewDockContent).UpdateColors();
+
       if (main)
       {
         int index = -1;
         if (!contentTabControl.HasDocument(document))
         {
           IWorkbenchDocument wbDocument = document as IWorkbenchDocument;
-          SetupDockLayout(document, (wbDocument != null) ? wbDocument.BackendForm : null);
+          SetupDockLayout(document, (wbDocument != null) ? wbDocument.BackendForm : null, contentTabControl.TabCount > 0);
           index = contentTabControl.AddDocument(document);
-
-          InstallFocusChangeHandlers(document as Control);
         }
 
         if (index == 0) // Replace the title by an icon.
@@ -1483,7 +1542,7 @@ namespace MySQL.GUI.Workbench
           contentTabControl.TabPages[index].ImageIndex = 0;
           contentTabControl.SetCloseButtonVisibility(0, FlatTabControl.CloseButtonVisiblity.HideButton);
         }
-        
+
         if (activate)
         {
           document.Activate();
@@ -1560,7 +1619,7 @@ namespace MySQL.GUI.Workbench
         if (document is ModelDiagramForm)
         {
           e.Cancel = !(document as IWorkbenchDocument).CanCloseDocument();
-          
+
           // Make the page active that refused to close.
           if (e.Cancel)
           {
@@ -1607,8 +1666,6 @@ namespace MySQL.GUI.Workbench
       }
 
       // Restore previous clipboard chain.
-      Win32.ChangeClipboardChain(this.Handle, nextClipboardViewer);
-      RemoveFocusChangeHandlers(this);
       SaveFormState();
 
       Hide();
@@ -1638,7 +1695,7 @@ namespace MySQL.GUI.Workbench
       {
         contentTabControl.ResumeLayout();
       }
-      
+
       wbContext.perform_quit();
     }
 
@@ -1654,7 +1711,7 @@ namespace MySQL.GUI.Workbench
         x = 100 - w;
       if (y + h < 100)
         y = 100 - h;
-      Rectangle workingArea= Screen.GetWorkingArea(new Point(x, y));
+      Rectangle workingArea = Screen.GetWorkingArea(new Point(x, y));
       if (x > workingArea.Right - 100)
         x = workingArea.Right - 100;
       if (y > workingArea.Bottom - 100)
@@ -1704,12 +1761,6 @@ namespace MySQL.GUI.Workbench
     private void MainForm_Load(object sender, EventArgs e)
     {
       LoadFormState();
-      InstallFocusChangeHandlers(this);
-
-      // Register the form as clipboard listener so we get notified about changes there.
-      nextClipboardViewer = Win32.SetClipboardViewer(this.Handle);
-
-      SplashScreen.CloseSplash();
 
       BringToFront();
       Activate();
@@ -1733,58 +1784,12 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-
     private void timer_Tick(object sender, EventArgs e)
     {
       wbContext.flush_timers();
       UpdateTimer();
     }
 
-
-    private void FocusChanged(object sender, EventArgs e)
-    {
-      // TODO: see InstallFocusChangeHandlers.
-      /*
-      // force a refresh of the menu so that Edit -> Copy/Paste/Cut/Select All can get updated accordingly
-      if (!shuttingDown)
-        wbContext.validate_edit_menu();
-      */
-    }
-
-    //private EventHandler FocusChangeHandler= null;
-
-
-    private void RemoveFocusChangeHandlers(Control control)
-    {
-      /* TODO: see InstallFocusChangeHandlers.
-      if (FocusChangeHandler == null)
-        return;
-
-      // Remove focus change handler we set previously.
-      control.GotFocus -= FocusChangeHandler;
-
-      foreach (Control sub in control.Controls)
-        RemoveFocusChangeHandlers(sub);
-      */
-    }
-
-    private void InstallFocusChangeHandlers(Control control)
-    {
-      // TODO: It shouldn't be necessary to install those handlers anymore. Remove once we are sure.
-      // Validation requests for the edit menu are all triggered by the backend now.
-      /*
-      if (FocusChangeHandler == null)
-        FocusChangeHandler= new EventHandler(FocusChanged);
-
-      // Recursively install event handler for GotFocus.
-      control.GotFocus -= FocusChangeHandler;
-      control.GotFocus += FocusChangeHandler;
-
-      foreach  (Control sub in control.Controls)
-        InstallFocusChangeHandlers(sub);
-    */
-    }
-
     private void tabControl_TabClosing(object sender, TabClosingEventArgs e)
     {
       e.canClose = true;
@@ -1801,9 +1806,6 @@ namespace MySQL.GUI.Workbench
           MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
           e.canClose = content.GetAppView().DocumentClosing();
         }
-
-      if (e.canClose)
-        RemoveFocusChangeHandlers(document as Control);
     }
 
     private void contentTabControl_TabClosed(object sender, TabClosedEventArgs e)
@@ -1820,23 +1822,6 @@ namespace MySQL.GUI.Workbench
         }
     }
 
-    private Bitmap logo = null;
-
-    void menuPanel_PaintBackground(object sender, PaintEventArgs e)
-    {
-      ToolStripHelper.MenuPanelPaint(e.Graphics, (sender as Control).ClientRectangle);
-
-      if (logo == null && File.Exists("images/ui/oracle_logo_white.png"))
-        logo = new Bitmap("images/ui/oracle_logo_white.png");
-
-      if (logo != null)
-      {
-        int x = (sender as Control).Width - logo.Width - 8;
-        int y = 6;
-        e.Graphics.DrawImageUnscaled(logo, new Point(x, y));
-      }
-    }
-
     private void tabsContextMenuItemClick(object sender, EventArgs e)
     {
       TabPage page = contentTabControl.SelectedTab;
@@ -1899,82 +1884,48 @@ namespace MySQL.GUI.Workbench
           break;
       }
     }
-    
+
     #endregion
 
-    #region Windows Message Handling
-
-    private IntPtr nextClipboardViewer = IntPtr.Zero;
-
-    protected override void WndProc(ref Message m)
+    public class MainPageDockDelegate : DockingPointDelegateWrapper
     {
-      switch ((WM)m.Msg)
+      private MainForm _owner = null;
+
+      public MainPageDockDelegate(MainForm owner, Object representedObject)
+        : base(representedObject)
       {
-        case WM.DRAWCLIPBOARD:
-          // Content of the clipboard changed. Notify next viewer in the chain.
-          if (nextClipboardViewer != IntPtr.Zero)
-            Win32.SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
-
-          ManagedNotificationCenter.Send("GNClipboardChanged", this.Handle);
-          break;
-
-        case WM.CHANGECBCHAIN:
-          // The clipboard chain changed. Update our next pointer.
-          if (m.WParam == nextClipboardViewer)
-            nextClipboardViewer = m.LParam;
-          else
-            Win32.SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
-          break;
-
-        default:
-          base.WndProc(ref m);
-          break;
+        _owner = owner;
       }
-    }
 
-    #endregion
+      public override String get_type(Object representedObject)
+      {
+        return "MainWindow";
+      }
 
+      public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+      {
+        _owner.DockDocument(view.GetHost(), true, true);
+      }
+
+      public override void undock_view(Object representedObject, AppViewImpl view)
+      {
+        _owner.UndockDocument(view.GetHost());
+      }
+
+      public override bool select_view(Object representedObject, AppViewImpl view)
+      {
+        return _owner.select_view(view);
+      }
+
+      public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+      {
+        _owner.set_view_title(view, str);
+      }
+
+      public override void get_size(Object representedObject, out int w, out int h)
+      {
+        _owner.get_size(out w, out h);
+      }
+    };
   }
-
-  public class MainPageDockDelegate : DockingPointDelegateWrapper
-  {
-    private MainForm _owner = null;
-
-    public MainPageDockDelegate(MainForm owner, Object representedObject)
-      : base(representedObject)
-    {
-      _owner = owner;
-    }
-
-    public override String get_type(Object representedObject)
-    {
-      return "MainWindow";
-    }
-
-    public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
-    {
-      _owner.DockDocument(view.GetHost(), true, true);
-    }
-
-    public override void undock_view(Object representedObject, AppViewImpl view)
-    {
-      _owner.UndockDocument(view.GetHost());
-    }
-
-    public override bool select_view(Object representedObject, AppViewImpl view)
-    {
-      return _owner.select_view(view);
-    }
-
-    public override void set_view_title(Object representedObject, AppViewImpl view, String str)
-    {
-      _owner.set_view_title(view, str);
-    }
-
-    public override void get_size(Object representedObject, out int w, out int h)
-    {
-      _owner.get_size(out w, out h);
-    }
-  };
-
 }
