@@ -21,17 +21,17 @@ using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.Drawing;
+using System.IO;
 using System.Windows.Forms;
-using MySQL.Controls;
 
+using MySQL.Base;
+using MySQL.Controls;
 using MySQL.Grt;
 using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
-using MySQL.Forms;
 using MySQL.Workbench;
-using System.IO;
 
 namespace MySQL.GUI.Workbench
 {
@@ -118,6 +118,7 @@ namespace MySQL.GUI.Workbench
       Application.AddMessageFilter(wheelMessageFilter);
 
       SetupSideBar();
+      UpdateColors();
     }
 
     private void UpdateTabText()
@@ -243,15 +244,31 @@ namespace MySQL.GUI.Workbench
         case "view_undo_history":
           ShowUndoHistory();
           break;
-        case "wb.sidebarHide":
-          if (mainSplitContainer.Panel1.Controls.Contains(sideSplitContainer))
-            mainSplitContainer.Panel1Collapsed = !mainSplitContainer.Panel1Collapsed;
-          else
-            mainSplitContainer.Panel2Collapsed = !mainSplitContainer.Panel2Collapsed;
+        case "wb.toggleSidebar":
+          mainSplitContainer.Panel1Collapsed = !mainSplitContainer.Panel1Collapsed;
+          break;
+        case "wb.toggleSecondarySidebar":
+          mainContentSplitContainer.Panel2Collapsed = !mainContentSplitContainer.Panel2Collapsed;
           break;
       }
     }
 
+    public void UpdateColors()
+    {
+      ApplyColors(this);
+      modelObjectDescriptionForm.UpdateColors();
+
+      if (Controls.Count > 0 && Controls[0] is DrawablePanel)
+        Controls[0].BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      else
+        BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+
+      mainSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      contentSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      sideSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      scrollPanel.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+    }
+
     public DockablePlugin FindPluginOfType(Type type)
     {
       foreach (ITabDocument content in bottomTabControl.Documents)
@@ -522,7 +539,7 @@ namespace MySQL.GUI.Workbench
 
             overviewPanel.Tag = CreateIdentifier(panelNodeId);
 
-            // Add the collapsing panel to the form. Move every panel explicitely to the top position
+            // Add the collapsing panel to the form. Move every panel explicitly to the top position
             // (moving all others down one step) to make stacking working properly.
             mainPanel.Controls.Add(overviewPanel);
             mainPanel.Controls.SetChildIndex(overviewPanel, 0);
@@ -560,7 +577,7 @@ namespace MySQL.GUI.Workbench
     #region Helper functions
 
     /// <summary>
-    /// Creates a key that can be used for later lookup in a dictonary consisting of
+    /// Creates a key that can be used for later lookup in a dictionary consisting of
     /// a GRT icon id and a view mode (large icon, small icon).
     /// </summary>
     /// <param name="id"></param>
@@ -758,7 +775,7 @@ namespace MySQL.GUI.Workbench
           break;
         case Overview.NodeType.Item:
           {
-            // If the child entry is an item then we don't have any intermediate structure.
+            // If the child entry is an item thean we don't have any intermediate structure.
             ListView sectionListview = GetSectionListview(panelNodeId, overviewPanel, false, displayMode, "", "");
             cacheListview(panelNodeId.repr(), sectionListview);
             PopulateListView(panelNodeId, sectionListview);
@@ -1289,7 +1306,7 @@ namespace MySQL.GUI.Workbench
       {
         if (panel != null && panel.Tag != null)
         {
-          List<MySQL.Grt.MenuItem> items;
+          List<MySQL.Base.MenuItem> items;
           List<NodeId> nodes = new List<NodeId>();
 
           NodeId itemNodeId= new NodeId();
@@ -1349,7 +1366,7 @@ namespace MySQL.GUI.Workbench
 
       if (listView != null && e.Button == MouseButtons.Right)
       {
-        List<MySQL.Grt.MenuItem> items = new List<MySQL.Grt.MenuItem>();
+        List<MySQL.Base.MenuItem> items = new List<MySQL.Base.MenuItem>();
         List<NodeId> item_nodes = new List<NodeId>();
 
         if (listView.SelectedItems.Count == 0)
@@ -1655,10 +1672,9 @@ namespace MySQL.GUI.Workbench
         DockSideDocument(userDatatypesForm, false, true);
         DockSideDocument(historyForm, false, false);
 
-        // Some values constantly crash the designer because code generation is buggy.
-        // So we better set that here.
+        contentSplitContainer.Panel2Collapsed = true;
+        contentSplitContainer.SplitterWidth = 6; 
         contentSplitContainer.Panel2MinSize = 300;
-        sideSplitContainer.SplitterWidth = 6;
       }
       finally
       {
@@ -1788,26 +1804,20 @@ namespace MySQL.GUI.Workbench
     private void LoadFormState()
     {
       int sidebarWidth = wbContext.read_state("sidebar_width", "model_overview", 200);
-      bool left_aligned = wbContext.read_option_value("", "Sidebar:RightAligned", "0") == "0";
+      if (sidebarWidth < 0)
+        sidebarWidth = 200;
+      if (sidebarWidth > mainSplitContainer.Width)
+        sidebarWidth = mainSplitContainer.Width / 2;
 
-      mainSplitContainer.Panel1.Controls.Clear();
-      mainSplitContainer.Panel2.Controls.Clear();
-      if (left_aligned)
-      {
-        mainSplitContainer.FixedPanel = FixedPanel.Panel1;
-        mainSplitContainer.Panel1.Controls.Add(sideSplitContainer);
-        mainSplitContainer.Panel2.Controls.Add(contentSplitContainer);
-        if (mainSplitContainer.Width > sidebarWidth)
-          mainSplitContainer.SplitterDistance = sidebarWidth;
-      }
-      else
-      {
-        mainSplitContainer.FixedPanel = FixedPanel.Panel2;
-        mainSplitContainer.Panel2.Controls.Add(sideSplitContainer);
-        mainSplitContainer.Panel1.Controls.Add(contentSplitContainer);
-        if (mainSplitContainer.Width > sidebarWidth)
-          mainSplitContainer.SplitterDistance = mainSplitContainer.Width - mainSplitContainer.SplitterWidth - sidebarWidth;
-      }
+      mainSplitContainer.SplitterDistance = sidebarWidth;
+
+      sidebarWidth = wbContext.read_state("secondary_sidebar_width", "model_diagram", mainContentSplitContainer.Width - 200);
+      if (mainContentSplitContainer.Width - sidebarWidth < 200)
+        sidebarWidth = mainContentSplitContainer.Width - 200;
+      if (sidebarWidth > mainContentSplitContainer.Width)
+        sidebarWidth = mainContentSplitContainer.Width / 2;
+
+      mainContentSplitContainer.SplitterDistance = sidebarWidth;
 
       NodeId nodeId = wbOverview.get_root();
       String caption;
@@ -1852,18 +1862,20 @@ namespace MySQL.GUI.Workbench
       if (splitterMovePending)
       {
         splitterMovePending = false;
-
-        int sidebarWidth;
-        bool left_aligned = wbContext.read_option_value("", "Sidebar:RightAligned", "0") == "0";
-        if (left_aligned)
-          sidebarWidth = mainSplitContainer.SplitterDistance;
-        else
-          sidebarWidth = mainSplitContainer.Width - mainSplitContainer.SplitterWidth - mainSplitContainer.SplitterDistance;
-        wbContext.save_state("sidebar_width", "model_overview", sidebarWidth);
+        wbContext.save_state("sidebar_width", "model_overview", mainSplitContainer.SplitterDistance);
       }
     }
 
-    private void mainSplitContainer_SplitterMoving(object sender, SplitterCancelEventArgs e)
+    private void mainContentSplitContainer_SplitterMoved(object sender, SplitterEventArgs e)
+    {
+      if (splitterMovePending)
+      {
+        splitterMovePending = false;
+        wbContext.save_state("secondary_sidebar_width", "model_overview", mainSplitContainer.SplitterDistance);
+      }
+    }
+
+    private void SplitterMoving(object sender, SplitterCancelEventArgs e)
     {
       // This event only comes up when the splitter is moved with the mouse (i.e. by the user).
       // We can use it to differentiate between user initiated and programmatic splitter changes.
@@ -1896,6 +1908,60 @@ namespace MySQL.GUI.Workbench
         listsByNode[id] = listview;
     }
 
+    private void ApplyColors(Control parent)
+    {
+      foreach (Control control in parent.Controls)
+      {
+        if (control is FlatTabControl)
+        {
+          FlatTabControl tabView = control as FlatTabControl;
+          tabView.UpdateColors();
+          tabView.BackgroundColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+        }
+        else
+          if (control is HeaderPanel)
+          {
+            HeaderPanel panel = control as HeaderPanel;
+            panel.HeaderColor = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelHeader, false);
+            panel.ForeColor = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelHeader, true);
+            panel.HeaderColorFocused = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelHeaderFocused, false);
+            panel.ForeColorFocused = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelHeaderFocused, true);
+          }
+          else
+            if (control is ToolStrip)
+            {
+              ToolStrip toolStrip = control as ToolStrip;
+              toolStrip.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelToolbar, false);
+              toolStrip.ForeColor = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelToolbar, true);
+            }
+            else
+              if (control is TabPage)
+              {
+                TabPage page = control as TabPage;
+                if (page.Parent is FlatTabControl)
+                {
+                  FlatTabControl view = page.Parent as FlatTabControl;
+                  if (view.TabStyle == FlatTabControl.TabStyleType.BottomNormal)
+                    page.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorPanelContentArea, false);
+                }
+              }
+
+        ApplyColors(control);
+      }
+    }
+
+    public override void Activate()
+    {
+      base.Activate();
+
+      if (secondarySidebarPanel.Controls.Count == 0)
+      {
+        Control secondarySidebar = wbContext.shared_secondary_sidebar();
+        secondarySidebarPanel.Controls.Add(secondarySidebar);
+        secondarySidebar.Dock = DockStyle.Fill;
+      }
+    }
+
     #endregion
 
   }
