@@ -8,16 +8,10 @@ using MySQL.GUI.Workbench;
 // associated with an assembly.
 [assembly: AssemblyTitle("MySQL Workbench")]
 [assembly: AssemblyDescription("MySQL Database Design Tool")]
-#if EDITION_SE
-[assembly: AssemblyConfiguration("SE")]
-#elif EDITION_OSS
-[assembly: AssemblyConfiguration("CE")]
-#else
-[assembly: AssemblyConfiguration("UNKNOWN EDITION")]
-#endif
+[assembly: AssemblyConfiguration("development")]
 [assembly: AssemblyCompany("Oracle Corporation")]
 [assembly: AssemblyProduct("MySQL Workbench")]
-[assembly: AssemblyCopyright("2006-2011")]
+[assembly: AssemblyCopyright("2006-2013")]
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
 
@@ -36,6 +30,6 @@ using MySQL.GUI.Workbench;
 //      Build Number
 //      Revision
 //
-[assembly: AssemblyVersion("5.2.47.10398")]
-[assembly: AssemblyFileVersion("5.2.47.10398")]
+[assembly: AssemblyVersion("6.0.3.11035")]
+[assembly: AssemblyFileVersion("6.0.3.11035")]
 [assembly: AssemblyReleaseType("")]
