@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,23 +20,20 @@
 using System;
 using System.Collections.Generic;
 using System.Drawing;
-using System.Text;
 using System.Windows.Forms;
 
 using MySQL.Controls;
-using MySQL.Grt;
+using MySQL.Forms;
+using MySQL.Base;
 using MySQL.Grt.Db;
+using MySQL.GUI.Workbench.Plugins;
 using MySQL.Utilities;
 using MySQL.Workbench;
-using MySQL.GUI.Workbench.Plugins;
-using MySQL.Forms;
 
 namespace MySQL.GUI.Workbench
 {
   public partial class SqlIdeForm : Plugins.DockablePlugin, IWorkbenchObserver
   {
-    static int InstanceCount = 0;
-
     private List<RecordsetView> pendingRelayouts = new List<RecordsetView>();
 
     private SqlEditorFormWrapper dbSqlEditorBE;
@@ -48,16 +45,22 @@ namespace MySQL.GUI.Workbench
 
     private bool canTrackChanges = false; // True when initialization is done and UI changes can be tracked.
 
+    private bool outputVisibilySetInAdmin = false;
+    private bool outputIsVisible = true;
+    private bool restoreOutputArea = false;
+    private bool secondarySidebarVisibilySetInAdmin = false;
+    private bool secondarySidebarIsVisible = true;
+    private bool restoreSeconarySidebar = false;
+
     #region Initialization
 
     public SqlIdeForm(WbContext wbContext, UIForm uiForm)
       : base((uiForm as SqlEditorFormWrapper).grt_manager())
     {
-      InstanceCount++;
-
       this.wbContext = wbContext;
       dbSqlEditorBE = uiForm as SqlEditorFormWrapper;
       Initialize();
+      UpdateColors();
     }
 
     protected void Initialize()
@@ -66,9 +69,6 @@ namespace MySQL.GUI.Workbench
 
       InitializeComponent();
 
-      if (InstanceCount == 1)
-        RegisterCommands();
-
       outputSelector.SelectedIndex = 0;
 
       TabText = dbSqlEditorBE.get_title();
@@ -97,7 +97,6 @@ namespace MySQL.GUI.Workbench
       logView.Columns[5].AutoSizeMode = DataGridViewAutoSizeColumnMode.DisplayedCells;
       logView.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
       logView.ForeColor = Color.Black;
-      logView.ShowBusyAnimation = true;
       logView.CellContextMenuStripNeeded += new DataGridViewCellContextMenuStripNeededEventHandler(logView_CellContextMenuStripNeeded);
 
       historyEntriesView = new GridView(dbSqlEditorBE.history().entries_model());
@@ -112,7 +111,10 @@ namespace MySQL.GUI.Workbench
       historyEntriesView.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
       historyEntriesView.RowEnter += new DataGridViewCellEventHandler(historyEntriesView_RowEnter);
       historyEntriesView.CellContextMenuStripNeeded += historyEntriesView_CellContextMenuStripNeeded;
+      historyEntriesView.SetRowSelected(0);
       {
+        // TODO: replace by mforms backed menu. Then remove SqlIdeMenuManager.
+        // See if WorkbenchToolbarManager.cs can go entirely when done.
         SqlIdeMenuManager.MenuContext popupMenuContext = new SqlIdeMenuManager.MenuContext();
         popupMenuContext.GetNodesMenuItems = historyEntriesView.Model.get_popup_items_for_nodes;
         popupMenuContext.GetSelectedNodes = historyEntriesView.SelectedNodes;
@@ -151,7 +153,9 @@ namespace MySQL.GUI.Workbench
       palette.Dock = DockStyle.Fill;
       palette.Show();
 
-      outputPageToolStrip.Renderer = new FlatSubToolStripRenderer();
+      UpdateColors();
+
+      dbSqlEditorBE.set_docking_delegate(new MainPaneDockDelegate(this));
 
       ManagedNotificationCenter.AddObserver(this, "GNFormTitleDidChange");
     }
@@ -166,27 +170,43 @@ namespace MySQL.GUI.Workbench
 
       ManagedNotificationCenter.RemoveObserver(this, "GNFormTitleDidChange");
 
-      if (InstanceCount == 1)
-        UnregisterCommands();
-
       // Make a copy of the keys first as the dictionary is modified while closing the recordsets.
       long[] keys = new long[recordset2placeholder.Count];
       recordset2placeholder.Keys.CopyTo(keys, 0);
       foreach (long key in keys)
         CloseRecordset(key);
 
-      // Dispose of all editors explicitly to sync backend release code with our shutdown.
-      // Otherwise essential cleanup is done when the GC cleans up, which might be too late.
-      foreach (SqlEditorWrapper editor in sqlEditors)
-        editor.Dispose();
-      sqlEditors.Clear();
+      // Close all attached editor tabs.
+      foreach (TabPage page in mainContentTabControl.TabPages)
+      {
+        int editorIndex = EditorIndexFromTabPage(page);
+        if (editorIndex >= 0)
+        {
+          // Close also all docked AppViews from the result tab control
+          // (result sets are closed already above).
+          SqlEditorWrapper editor = sqlEditors[editorIndex];
+          FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
+          if (resultTabControl != null)
+          {
+            foreach (TabPage resultTabPage in resultTabControl.TabPages)
+            {
+              if (resultTabPage.Tag is AppViewImpl)
+                RemoveViewFromResultTabview(editor, resultTabPage.Tag as AppViewImpl);
+            }
+          }
+        }
+
+        // AppView still does not support can_close/close duality.
+        if (page.Tag is AppViewImpl)
+          (page.Tag as AppViewImpl).DocumentClosing();
+        else
+          closeEditorTab(page, true);
+      }
 
       historyDetailsView.Dispose();
       historyEntriesView.Dispose();
 
       dbSqlEditorBE.Dispose();
-
-      InstanceCount--;
     }
 
     #endregion
@@ -213,7 +233,7 @@ namespace MySQL.GUI.Workbench
           // TODO: find a better way to report start and stop of editor actions in the mid term.
           runningEditor = ActiveEditor;
           if (runningEditor != null)
-            editorTabControl.SetBusy(TabIndexFromEditor(runningEditor), true);
+            mainContentTabControl.SetBusy(TabIndexFromEditor(runningEditor), true);
           break;
 
         default:
@@ -238,41 +258,32 @@ namespace MySQL.GUI.Workbench
     {
       switch (command)
       {
-        case "wb.toggleSchemataBar":
+        case "wb.toggleSidebar":
           mainSplitContainer.Panel1Collapsed = !mainSplitContainer.Panel1Collapsed;
           wbContext.save_state("sidebar_visible", "query_editor", !mainSplitContainer.Panel1Collapsed);
           break;
         case "wb.toggleOutputArea":
+          outputIsVisible = !outputIsVisible;
+          if (IsAdminActive())
+            outputVisibilySetInAdmin = true;
           contentSplitContainer.Panel2Collapsed = !contentSplitContainer.Panel2Collapsed;
           wbContext.save_state("output_visible", "query_editor", !contentSplitContainer.Panel2Collapsed);
           break;
-        case "wb.toggleSideBar":
+        case "wb.toggleSecondarySidebar":
+          secondarySidebarIsVisible = !secondarySidebarIsVisible;
+          if (IsAdminActive())
+            secondarySidebarVisibilySetInAdmin = true;
           mainContentSplitContainer.Panel2Collapsed = !mainContentSplitContainer.Panel2Collapsed;
           wbContext.save_state("support_sidebar_visible", "query_editor", !mainContentSplitContainer.Panel2Collapsed);
           break;
         case "close_editor":
-          editorTabControl.CloseTabPage(editorTabControl.SelectedTab);
+          mainContentTabControl.CloseTabPage(mainContentTabControl.SelectedTab);
           break;
       }
     }
 
     private List<String> commands = new List<string>();
 
-    private void RegisterCommands()
-    {
-      commands.Add("wb.toggleSchemataBar");
-      commands.Add("wb.toggleOutputArea");
-      commands.Add("wb.toggleSideBar");
-      commands.Add("close_editor");
-
-      wbContext.add_frontend_commands(commands);
-    }
-
-    private void UnregisterCommands()
-    {
-      wbContext.remove_frontend_commands(commands);
-    }
-
     #endregion
 
     #region SQL Script Execution
@@ -299,8 +310,13 @@ namespace MySQL.GUI.Workbench
       int tabIndex = TabIndexFromEditor(editor);
       int editorIndex = EditorIndexFromTabIndex(tabIndex);
 
-      if (dbSqlEditorBE.exec_sql_error_count() == 0)
-        editorTabControl_SelectedIndexChanged(null, null);
+      if (dbSqlEditorBE.exec_sql_error_count() > 0)
+      {
+        outputSelector.SelectedIndex = 0;
+        dbSqlEditorBE.show_output_area();
+      }
+      else // Refocus the code editor again.
+        mainContentTabControl_SelectedIndexChanged(null, null);
 
       logView.ProcessModelRowsChange();
       logView.AutoResizeColumn(1);
@@ -310,9 +326,9 @@ namespace MySQL.GUI.Workbench
       if (logView.Rows.Count > 0)
         logView.SetRowSelected(logView.Rows.Count - 1);
 
-      editorTabControl.SelectedIndex = tabIndex;
-      sqlEditors[editorIndex].get_native_editor().Focus();
-      editorTabControl.SetBusy(tabIndex, false);
+      mainContentTabControl.SelectedIndex = tabIndex;
+      sqlEditors[editorIndex].get_editor_container().Focus();
+      mainContentTabControl.SetBusy(tabIndex, false);
 
       return 0;
     }
@@ -357,7 +373,7 @@ namespace MySQL.GUI.Workbench
       TabPage page = sqlEditor2page[editor];
       if (page == null)
         return -1;
-      return editorTabControl.TabPages.IndexOf(page);
+      return mainContentTabControl.TabPages.IndexOf(page);
     }
 
     /// <summary>
@@ -366,10 +382,10 @@ namespace MySQL.GUI.Workbench
     /// </summary>
     private int EditorIndexFromTabIndex(int tabIndex)
     {
-      if (tabIndex < 0 || tabIndex > editorTabControl.TabCount - 1)
+      if (tabIndex < 0 || tabIndex > mainContentTabControl.TabCount - 1)
         return -1;
 
-      return EditorIndexFromTabPage(editorTabControl.TabPages[tabIndex]);
+      return EditorIndexFromTabPage(mainContentTabControl.TabPages[tabIndex]);
     }
 
     private int EditorIndexFromTabPage(TabPage page)
@@ -406,7 +422,7 @@ namespace MySQL.GUI.Workbench
 
       SqlEditorWrapper sqlEditor = dbSqlEditorBE.sql_editor(index);
       sqlEditor.set_result_docking_delegate(new ResultPaneDockDelegate(this, sqlEditor));
-      Control editorControl = sqlEditor.get_native_editor();
+      Control editorContainer = sqlEditor.get_editor_container();
       String caption = dbSqlEditorBE.sql_editor_caption(index);
 
       TabPage page = new TabPage(caption);
@@ -416,23 +432,27 @@ namespace MySQL.GUI.Workbench
       SplitContainer editorSplitContainer = new SplitContainer();
       editorSplitContainer.Orientation = Orientation.Horizontal;
       editorSplitContainer.Dock = DockStyle.Fill;
+      editorSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      editorSplitContainer.SplitterWidth = 6;
 
       // Upper part (editor + toolbar).
-      editorSplitContainer.Panel1.Controls.Add(editorControl);
-      editorControl.Dock = DockStyle.Fill;
+      editorSplitContainer.Panel1.Controls.Add(editorContainer);
+      editorContainer.Dock = DockStyle.Fill;
       ToolStrip toolStrip = dbSqlEditorBE.get_editor_toolbar(index);
       editorSplitContainer.Panel1.Controls.Add(toolStrip);
       toolStrip.Dock = DockStyle.Top;
 
       // Lower part (record set views).
       FlatTabControl resultTabControl = new FlatTabControl();
-      resultTabControl.BackgroundColor = System.Drawing.Color.FromArgb(40, 55, 82);
       resultTabControl.CanCloseLastTab = true;
-      resultTabControl.ContentPadding = new System.Windows.Forms.Padding(0);
+      resultTabControl.ContentPadding = new Padding(0);
       editorSplitContainer.Panel2.Controls.Add(resultTabControl);
       resultTabControl.Dock = System.Windows.Forms.DockStyle.Fill;
       resultTabControl.HideWhenEmpty = false;
-      resultTabControl.ItemPadding = new System.Windows.Forms.Padding(6, 0, 6, 0);
+      resultTabControl.ItemPadding = new Padding(6, 0, 6, 0);
+      resultTabControl.ItemSize = new Size(75, 19);
+      resultTabControl.Margin = new Padding(0);
+      resultTabControl.Padding = new Point(0, 0);
       resultTabControl.MaxTabSize = 200;
       resultTabControl.Name = "resultTabControl";
       resultTabControl.ShowCloseButton = true;
@@ -446,13 +466,18 @@ namespace MySQL.GUI.Workbench
       resultTabControl.MouseClick += new MouseEventHandler(recordsetTabControlMouseClick);
       resultTabControl.SelectedIndexChanged += new EventHandler(resultTabControl_SelectedIndexChanged);
 
-      page.Controls.Add(editorSplitContainer);
-      editorTabControl.TabPages.Add(page);
+      resultTabControl.UpdateColors();
+      resultTabControl.BackgroundColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
 
+      // Add the new editor first to our internal mappers before adding it to the tab control, as
+      // this triggers further actions that need the mapping.
       sqlEditors.Add(sqlEditor);
       sqlEditor2page[sqlEditor] = page;
       page2SqlEditor[page] = sqlEditor;
 
+      page.Controls.Add(editorSplitContainer);
+      mainContentTabControl.TabPages.Add(page);
+
       // Set an initial height for the result set panel depending on what the user last had.
       int splitterDistance = wbContext.read_state("recordset_height", "query_editor", editorSplitContainer.Height / 2);
       if (editorSplitContainer.Height > splitterDistance + editorSplitContainer.SplitterWidth)
@@ -462,7 +487,7 @@ namespace MySQL.GUI.Workbench
       editorSplitContainer.Panel2Collapsed = true;
 
       // Select tab last so that callbacks will work on attached recordset.
-      editorTabControl.SelectedIndex = editorTabControl.TabCount - 1;
+      mainContentTabControl.SelectedIndex = mainContentTabControl.TabCount - 1;
 
       return sqlEditor;
     }
@@ -490,21 +515,56 @@ namespace MySQL.GUI.Workbench
 
     #region Event handling
 
-    void editorTabControl_SelectedIndexChanged(object sender, EventArgs e)
+    void mainContentTabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
       // This could set -1 as active editor index in the backend if a page was selected
       // that does not hold an SQL editor (e.g. an object editor page).
-      dbSqlEditorBE.active_sql_editor_index(EditorIndexFromTabIndex(editorTabControl.SelectedIndex));
+      dbSqlEditorBE.active_sql_editor_index(EditorIndexFromTabIndex(mainContentTabControl.SelectedIndex));
       UpdateActiveRecordsetInBackend();
 
       SqlEditorWrapper sqlEditor = ActiveEditor;
       if (sqlEditor != null)
-      {
-        Control editorControl = sqlEditor.get_native_editor();
-        if (editorControl != null && editorControl.CanFocus)
-          editorControl.Focus();
-      }
+        sqlEditor.focus();
       UpdateApplyCancel();
+
+      // Switch off output and secondary sidebar if we are switching to an admin page and
+      // those were not explicitly switched on previously. Restore them when switching away.
+      if (IsAdminActive())
+      {
+        if (!outputVisibilySetInAdmin && outputIsVisible)
+        {
+          restoreOutputArea = true;
+          outputIsVisible = false;
+          contentSplitContainer.Panel2Collapsed = true;
+          dbSqlEditorBE.set_tool_item_checked("wb.toggleOutputArea", false);
+
+          // Do not save this temporary state.
+        }
+        if (!secondarySidebarVisibilySetInAdmin && secondarySidebarIsVisible)
+        {
+          restoreSeconarySidebar = true;
+          secondarySidebarIsVisible = false;
+          mainContentSplitContainer.Panel2Collapsed = true;
+          dbSqlEditorBE.set_tool_item_checked("wb.toggleSecondarySidebar", false);
+        }
+      }
+      else
+      {
+        if (restoreOutputArea) // Only set if not manually adjusted in an admin page and it was visible previously.
+        {
+          restoreOutputArea = false;
+          outputIsVisible = true;
+          contentSplitContainer.Panel2Collapsed = false;
+          dbSqlEditorBE.set_tool_item_checked("wb.toggleOutputArea", true);
+        }
+        if (restoreSeconarySidebar) // Ditto.
+        {
+          restoreSeconarySidebar = false;
+          secondarySidebarIsVisible = true;
+          mainContentSplitContainer.Panel2Collapsed = false;
+          dbSqlEditorBE.set_tool_item_checked("wb.toggleSecondarySidebar", true);
+        }
+      }
     }
 
     void editorTabControl_TabClosing(object sender, MySQL.Controls.TabClosingEventArgs e)
@@ -514,43 +574,58 @@ namespace MySQL.GUI.Workbench
         e.canClose = dbSqlEditorBE.sql_editor_will_close(editorIndex);
       else
       {
-        // One of the object editors.
-        ITabDocument tabDocument = editorTabControl.ActiveDocument;
-        if (tabDocument is IWorkbenchDocument)
-          e.canClose = (tabDocument as IWorkbenchDocument).CanCloseDocument();
+        // Any docked app view (plugin)?
+        if (e.page.Tag is AppViewImpl)
+          e.canClose = (e.page.Tag as AppViewImpl).DocumentClosing();
+        else
+        {
+          // One of the object editors.
+          ITabDocument tabDocument = mainContentTabControl.ActiveDocument;
+          if (tabDocument is IWorkbenchDocument)
+            e.canClose = (tabDocument as IWorkbenchDocument).CanCloseDocument();
+        }
       }
     }
 
-    void editorTabControl_TabClosed(object sender, MySQL.Controls.TabClosedEventArgs e)
+    private void closeEditorTab(TabPage page, bool disposing = false)
     {
-      int editorIndex = EditorIndexFromTabPage(e.page);
+      int editorIndex = EditorIndexFromTabPage(page);
       if (editorIndex >= 0)
       {
         RemoveSqlEditor(editorIndex);
 
         // If the last editor was closed create a new, empty one.
-        if (sqlEditors.Count > 0)
+        // But not when we are shutting down.
+        if (!disposing)
         {
-          editorIndex = EditorIndexFromTabIndex(editorTabControl.SelectedIndex);
-          if (editorIndex > -1)
-            dbSqlEditorBE.active_sql_editor_index(editorIndex);
+          if (sqlEditors.Count > 0)
+          {
+            editorIndex = EditorIndexFromTabIndex(mainContentTabControl.SelectedIndex);
+            if (editorIndex > -1)
+              dbSqlEditorBE.active_sql_editor_index(editorIndex);
+          }
+          else
+          {
+            dbSqlEditorBE.new_sql_script_file(); // This will call our callback for setting up the editor.
+            dbSqlEditorBE.active_sql_editor_index(0);
+            SqlEditorWrapper sqlEditor = ActiveEditor;
+            if (sqlEditor != null)
+              ActiveControl = sqlEditor.get_editor_container();
+          }
         }
-        else
-        {
-          dbSqlEditorBE.new_sql_script_file(); // This will call our callback for setting up the editor.
-          dbSqlEditorBE.active_sql_editor_index(0);
-          SqlEditorWrapper sqlEditor = ActiveEditor;
-          if (sqlEditor != null)
-            ActiveControl = sqlEditor.get_native_editor();
-        }
-
       }
       else
       {
-        ITabDocument tabDocument = editorTabControl.DocumentFromPage(e.page);
+        ITabDocument tabDocument = mainContentTabControl.DocumentFromPage(page);
         if (tabDocument is IWorkbenchDocument)
           (tabDocument as IWorkbenchDocument).CloseDocument();
       }
+      // Nothing to do for AppView, except we should finally support OnClosing/OnClose duality there too.
+    }
+
+    void editorTabControl_TabClosed(object sender, MySQL.Controls.TabClosedEventArgs e)
+    {
+      closeEditorTab(e.page);
     }
 
     void resultTabControl_SelectedIndexChanged(object sender, EventArgs e)
@@ -626,7 +701,7 @@ namespace MySQL.GUI.Workbench
     {
       get
       {
-        if (editorTabControl.SelectedTab == null)
+        if (mainContentTabControl.SelectedTab == null)
           return null;
 
         FlatTabControl resultTabControl = GetResultTabControlForEditor(ActiveEditor);
@@ -652,7 +727,7 @@ namespace MySQL.GUI.Workbench
 
     private void UpdateActiveRecordsetInBackend()
     {
-      if (editorTabControl.SelectedTab == null)
+      if (mainContentTabControl.SelectedTab == null)
         return;
 
       int index = dbSqlEditorBE.active_sql_editor_index();
@@ -665,7 +740,7 @@ namespace MySQL.GUI.Workbench
 
     private void OnRecordsetCaptionChange()
     {
-      if (editorTabControl.SelectedTab == null)
+      if (mainContentTabControl.SelectedTab == null)
         return;
 
       FlatTabControl resultTabControl = GetResultTabControlForEditor(ActiveEditor);
@@ -719,11 +794,10 @@ namespace MySQL.GUI.Workbench
       // easily correlate between the messages sent and the editors they are meant for.
       // However, when results arrive we know for sure the sql operation is over and we can safely
       // remove the busy animation for that (here known) editor.
-      editorTabControl.SetBusy(TabIndexFromEditor(sqlEditors[editorIndex]), false);
+      mainContentTabControl.SetBusy(TabIndexFromEditor(sqlEditors[editorIndex]), false);
 
       MySQL.Grt.Db.RecordsetWrapper rs = dbSqlEditorBE.recordset_for_key(editorIndex, key);
       TabPage page = new TabPage(rs.caption());
-      page.BackColor = SystemColors.ButtonFace;
 
       RecordsetView recordsetView = new RecordsetView();
       page.Tag = recordsetView;
@@ -749,7 +823,10 @@ namespace MySQL.GUI.Workbench
       // If there is no result set, however, show the editor.
       editorSplitContainer.Panel2Collapsed = resultTabControl.TabCount == 0;
       if (resultTabControl.TabCount > 0)
-        editorSplitContainer.Panel1Collapsed = dbSqlEditorBE.sql_editor_start_collapsed(editorIndex);
+      {
+        if (dbSqlEditorBE.sql_editor_start_collapsed(editorIndex))
+          editorSplitContainer.SplitterDistance = 100;
+      }
     }
 
     private void CloseRecordset(long key)
@@ -873,7 +950,7 @@ namespace MySQL.GUI.Workbench
       {
         case MouseButtons.Right:
           {
-            int clickedIndex = editorTabControl.TabIndexFromPosition(e.Location);
+            int clickedIndex = mainContentTabControl.TabIndexFromPosition(e.Location);
             if (clickedIndex < 0)
               return;
 
@@ -885,7 +962,7 @@ namespace MySQL.GUI.Workbench
             if (editorIndex > -1)
               path = dbSqlEditorBE.sql_editor_path(editorIndex);
             copyFullPathToClipboardToolStripMenuItem.Enabled = path.Length > 0;
-            Point p = editorTabControl.PointToScreen(e.Location);
+            Point p = mainContentTabControl.PointToScreen(e.Location);
             editorTabsContextMenuStrip.Show(p);
           }
           break;
@@ -922,7 +999,7 @@ namespace MySQL.GUI.Workbench
     {
       ToolStripMenuItem item = sender as ToolStripMenuItem;
       int clickedTab = (int)item.Owner.Tag;
-      TabPage page = editorTabControl.TabPages[clickedTab];
+      TabPage page = mainContentTabControl.TabPages[clickedTab];
       switch (item.Tag as string)
       {
         case "0": // Rename editor and tab. (currently not active)
@@ -934,29 +1011,29 @@ namespace MySQL.GUI.Workbench
           }
           break;
         case "1": // Close editor.
-          editorTabControl.CloseTabPage(page);
+          mainContentTabControl.CloseTabPage(page);
           break;
         case "2": // Close all editors.
           // Close them backwards and only exactly the number of editors that are there when we started.
           // Because on close of the last one a new empty one is created which we do not want to close again.
           // Additionally, some tabs might not be closable due to changed content and the user refusing
           // to close them.
-          for (int i = editorTabControl.TabCount - 1; i >= 0; i--)
-            editorTabControl.CloseTabPage(editorTabControl.TabPages[i]);
+          for (int i = mainContentTabControl.TabCount - 1; i >= 0; i--)
+            mainContentTabControl.CloseTabPage(mainContentTabControl.TabPages[i]);
           break;
         case "3": // Close all editors but the clicked one.
           // Similar to case 2, but we don't need to care for a newly created editor, as we always
           // keep one open.
           List<TabPage> open_tabs = new List<TabPage>();
           
-          for(int index = 0; index < editorTabControl.TabCount; index++)
+          for(int index = 0; index < mainContentTabControl.TabCount; index++)
           {
-            if(editorTabControl.TabPages[index] != page)
-              open_tabs.Add(editorTabControl.TabPages[index]);
+            if(mainContentTabControl.TabPages[index] != page)
+              open_tabs.Add(mainContentTabControl.TabPages[index]);
           }
 
           foreach(TabPage open_page in open_tabs)
-            editorTabControl.CloseTabPage(open_page);
+            mainContentTabControl.CloseTabPage(open_page);
 
           break;
 
@@ -965,12 +1042,12 @@ namespace MySQL.GUI.Workbench
           break;
 
         case "5": // Close all of the same type.
-          ITabDocument document = editorTabControl.DocumentFromIndex(clickedTab);
+          ITabDocument document = mainContentTabControl.DocumentFromIndex(clickedTab);
           bool closeDocument = document != null; // Only two types of pages: documents (table editors) or sql editors.
-          for (int i = editorTabControl.TabCount - 1; i >= 0; i--)
+          for (int i = mainContentTabControl.TabCount - 1; i >= 0; i--)
           {
-            if (closeDocument ^ editorTabControl.DocumentFromIndex(i) == null)
-              editorTabControl.CloseTabPage(editorTabControl.TabPages[i]);
+            if (closeDocument ^ mainContentTabControl.DocumentFromIndex(i) == null)
+              mainContentTabControl.CloseTabPage(mainContentTabControl.TabPages[i]);
           }
           break;
       }
@@ -1118,13 +1195,7 @@ namespace MySQL.GUI.Workbench
       if (!canTrackChanges || IsDisposed || Disposing)
         return;
 
-      int sidebarWidth;
-      bool leftAligned = wbContext.read_option_value("", "Sidebar:RightAligned", "0") == "0";
-      if (leftAligned)
-        sidebarWidth = mainSplitContainer.SplitterDistance;
-      else
-        sidebarWidth = mainSplitContainer.Width - mainSplitContainer.SplitterWidth - mainSplitContainer.SplitterDistance;
-      wbContext.save_state("sidebar_width", "query_editor", sidebarWidth);
+      wbContext.save_state("sidebar_width", "query_editor", mainSplitContainer.SplitterDistance);
     }
 
     private void contentSplitContainer_SplitterMoved(object sender, SplitterEventArgs e)
@@ -1185,21 +1256,12 @@ namespace MySQL.GUI.Workbench
     {
       // Object side bar.
       int splitterDistance = wbContext.read_state("sidebar_width", "query_editor", 200);
-      bool leftAligned = wbContext.read_option_value("", "Sidebar:RightAligned", "0") == "0";
+      if (splitterDistance < 0)
+        splitterDistance = 200;
+      if (splitterDistance > mainSplitContainer.Width)
+        splitterDistance = mainSplitContainer.Width / 2;
 
-      if (leftAligned)
-      {
-        if (mainSplitContainer.Width > splitterDistance)
-          mainSplitContainer.SplitterDistance = splitterDistance;
-      }
-      else
-      {
-        mainSplitContainer.FixedPanel = FixedPanel.Panel2;
-        contentSplitContainer.Parent = mainSplitContainer.Panel1;
-        sideArea.Parent = mainSplitContainer.Panel2;
-        if (mainSplitContainer.Width > splitterDistance + mainSplitContainer.SplitterWidth)
-          mainSplitContainer.SplitterDistance = mainSplitContainer.Width - mainSplitContainer.SplitterWidth - splitterDistance;
-      }
+      mainSplitContainer.SplitterDistance = splitterDistance;
 
       // Output tab. Distance measured from bottom (for easy default value).
       splitterDistance = wbContext.read_state("output_height", "query_editor", 200);
@@ -1213,41 +1275,37 @@ namespace MySQL.GUI.Workbench
 
       // Visibility of the sidebar/output areas.
       bool visible = wbContext.read_state("sidebar_visible", "query_editor", true);
-      dbSqlEditorBE.set_tool_item_checked("wb.toggleSchemataBar", visible);
+      dbSqlEditorBE.set_tool_item_checked("wb.toggleSidebar", visible);
       mainSplitContainer.Panel1Collapsed = !visible;
 
-      visible = wbContext.read_state("output_visible", "query_editor", true);
-      dbSqlEditorBE.set_tool_item_checked("wb.toggleOutputArea", visible);
-      contentSplitContainer.Panel2Collapsed = !visible;
+      outputIsVisible = wbContext.read_state("output_visible", "query_editor", true);
+      dbSqlEditorBE.set_tool_item_checked("wb.toggleOutputArea", outputIsVisible);
+      contentSplitContainer.Panel2Collapsed = !outputIsVisible;
 
-      visible = wbContext.read_state("support_sidebar_visible", "query_editor", true);
-      dbSqlEditorBE.set_tool_item_checked("wb.toggleSideBar", visible);
-      mainContentSplitContainer.Panel2Collapsed = !visible;
+      secondarySidebarIsVisible = wbContext.read_state("support_sidebar_visible", "query_editor", true);
+      dbSqlEditorBE.set_tool_item_checked("wb.toggleSecondarySidebar", secondarySidebarIsVisible);
+      mainContentSplitContainer.Panel2Collapsed = !secondarySidebarIsVisible;
     }
     
     /// <summary>
-    /// Docks the given document to the editor tab control (usually object editors).
+    /// Docks the given document to the editor tab control (object editors and plugins).
     /// </summary>
-    public void DockDocument(ITabDocument document, bool activate)
+    public TabPage DockDocument(ITabDocument document, bool activate)
     {
-      if (!editorTabControl.HasDocument(document))
-        editorTabControl.AddDocument(document);
+      if (document is DockablePlugin)
+        (document as DockablePlugin).UpdateColors();
+
+      if (!mainContentTabControl.HasDocument(document))
+        mainContentTabControl.AddDocument(document);
       if (activate)
         document.Activate();
 
-      if (contentSplitContainer.Panel2Collapsed)
-      {
-        contentSplitContainer.Panel2Collapsed = false;
-
-        // Set a splitter distance or we end up at almost full display. Use a relatively small
-        // value for panel2. The document's min height will kick in and does the right job.
-        contentSplitContainer.SplitterDistance = contentSplitContainer.Height - 100;
-      }
+      return mainContentTabControl.SelectedTab;
     }
 
     public void UndockDocument(ITabDocument document)
     {
-      editorTabControl.RemoveDocument(document);
+      mainContentTabControl.RemoveDocument(document);
     }
 
     #endregion
@@ -1277,10 +1335,10 @@ namespace MySQL.GUI.Workbench
         return null;
 
       int index = TabIndexFromEditor(editor);
-      if (index < 0 || index > editorTabControl.TabCount - 1)
+      if (index < 0 || index > mainContentTabControl.TabCount - 1)
         return null;
 
-      TabPage editorTabPage = editorTabControl.TabPages[index];
+      TabPage editorTabPage = mainContentTabControl.TabPages[index];
       if (editorTabPage.Controls.Count == 0)
         return null;
 
@@ -1325,13 +1383,38 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    public override void UpdateColors()
+    {
+      base.UpdateColors();
+
+      outputPageToolStrip.Renderer = new FlatSubToolStripRenderer();
+
+      readOnlyLabel.ForeColor = Conversions.GetApplicationColor(ApplicationColor.AppColorTabUnselected, true);
+      readOnlyLabel.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorTabUnselected, false);
+      readOnlyPictureBox.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorTabUnselected, false);
+      mainSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      contentSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      mainContentSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      rightSplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+      historySplitContainer.BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+
+      // Adjust all result split containers.
+      foreach (TabPage page in mainContentTabControl.TabPages)
+        if (page.Controls.Count > 0)
+          page.Controls[0].BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+
+      // The side area has the sidebar which is rooted on a split container.
+      if (sideArea.Controls.Count > 0)
+        sideArea.Controls[0].BackColor = Conversions.GetApplicationColor(ApplicationColor.AppColorMainBackground, false);
+    }
+
     public override bool CanCloseDocument()
     {
       // The backend will take care for all SQL editors. We only check object editors here.
       if (!Backend.can_close())
         return false;
 
-      foreach (ITabDocument document in editorTabControl.Documents)
+      foreach (ITabDocument document in mainContentTabControl.Documents)
       {
         if (document is IWorkbenchDocument)
         {
@@ -1339,7 +1422,11 @@ namespace MySQL.GUI.Workbench
           if (!form.can_close())
             return false;
         }
-
+        else if (document is MySQL.Forms.AppViewDockContent)
+        {
+          if (!(document as MySQL.Forms.AppViewDockContent).GetAppView().DocumentClosing())
+            return false;
+        }
       }
       return true;
     }
@@ -1356,19 +1443,8 @@ namespace MySQL.GUI.Workbench
       if (resultTabControl == null)
         return;
 
-      TabPage page = new TabPage(view.GetTitle());
-      page.BackColor = SystemColors.ButtonFace;
-
-      page.Tag = view;
-
-      Control control = view.GetHost();
-      control.Dock = DockStyle.Fill;
-      control.Margin = new Padding(0);
-      control.Padding = new Padding(0);
-      control.Show();
-      page.Controls.Add(control);
-      resultTabControl.TabPages.Add(page);
-      int newIndex = resultTabControl.TabPages.Count - 1;
+      int newIndex = resultTabControl.AddDocument(view.GetHost());
+      resultTabControl.TabPages[newIndex].Tag = view;
       resultTabControl.SetCloseButtonVisibility(newIndex, FlatTabControl.CloseButtonVisiblity.ShowButton);
       resultTabControl.SelectedIndex = newIndex;
 
@@ -1397,22 +1473,6 @@ namespace MySQL.GUI.Workbench
       page.Dispose();
     }
 
-    public bool SelectView(SqlEditorWrapper editor, AppViewImpl view)
-    {
-      FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
-      if (resultTabControl == null)
-        return false;
-      return true;
-    }
-
-    public void SetViewTitle(SqlEditorWrapper editor, AppViewImpl view, String title)
-    {
-      TabPage page = view.GetHost().Parent as TabPage;
-      if (page == null)
-        return;
-      page.Text = title;
-    }
-
     public void GetResultTabViewSize(SqlEditorWrapper editor, out int width, out int height)
     {
       FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
@@ -1427,6 +1487,21 @@ namespace MySQL.GUI.Workbench
       height = resultTabControl.ClientSize.Height;
     }
 
+    /// <summary>
+    /// Returns true if the currently active page in the content tab control is an admin page.
+    /// </summary>
+    /// <returns></returns>
+    private bool IsAdminActive()
+    {
+      ITabDocument document = mainContentTabControl.ActiveDocument;
+      if (document is AppViewDockContent)
+      {
+        if ((document as AppViewDockContent).GetContextName() == "administrator")
+          return true;
+      }
+      return false;
+    }
+
     #endregion
 
   }
@@ -1467,14 +1542,13 @@ namespace MySQL.GUI.Workbench
 
     public override bool select_view(Object representedObject, AppViewImpl view)
     {
-      SqlEditorWrapper wrapper = representedObject as SqlEditorWrapper;
-      return owner.SelectView(wrapper, view);
+      view.GetHost().Activate();
+      return true;
     }
 
     public override void set_view_title(Object representedObject, AppViewImpl view, String str)
     {
-      SqlEditorWrapper wrapper = representedObject as SqlEditorWrapper;
-      owner.SetViewTitle(wrapper, view, str);
+      view.GetHost().TabText = str;
     }
 
     /// <summary>
@@ -1487,6 +1561,60 @@ namespace MySQL.GUI.Workbench
     }
   };
 
+  public class MainPaneDockDelegate : DockingPointDelegateWrapper
+  {
+    private SqlIdeForm owner = null;
+    private TabDocument document;
+
+    public MainPaneDockDelegate(SqlIdeForm owner)
+      : base(null)
+    {
+      this.owner = owner;
+    }
+
+    protected override void Dispose(bool disposing)
+    {
+      base.Dispose(disposing);
+    }
+
+    public override String get_type(Object representedObject)
+    {
+      return "db.query.Editor:main"; // Same as defined in wb_sql_editor_form.h
+    }
+
+    public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+    {
+      document = view.GetHost(); // Only for get_size, since the view is not passed in there.
+      TabPage page = owner.DockDocument(document, true);
+      page.Tag = view;
+    }
+
+    public override void undock_view(Object representedObject, AppViewImpl view)
+    {
+      owner.UndockDocument(view.GetHost());
+    }
+
+    public override bool select_view(Object representedObject, AppViewImpl view)
+    {
+      view.GetHost().Activate();
+      return true;
+    }
+
+    public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+    {
+      view.GetHost().TabText = str;
+    }
+
+    /// <summary>
+    /// Meant to return the size of the host of the view, not the view itself.
+    /// </summary>
+    public override void get_size(Object representedObject, out int w, out int h)
+    {
+      w = document.Width;
+      h = document.Height;
+    }
+  };
+
   #endregion
 
 }
