@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,7 +50,10 @@ namespace base
 
   // turns a UTF8 string into something that can be used as a file name (ie, strips out special chars)
   BASELIBRARY_PUBLIC_FUNC std::string sanitize_file_name(const std::string &s);
-  
+
+  // performs a utf8 casefold + collate between 2 strings
+  BASELIBRARY_PUBLIC_FUNC int utf8_case_collate(const std::string &s1, const std::string &s2);
+
   // Trimming, cleanup etc.
   BASELIBRARY_PUBLIC_FUNC std::string trim_right(const std::string& s, const std::string& t = SPACES);
   BASELIBRARY_PUBLIC_FUNC std::string trim_left(const std::string& s, const std::string& t = SPACES);
@@ -75,13 +78,13 @@ namespace base
   BASELIBRARY_PUBLIC_FUNC std::string make_valid_filename(const std::string &name);
   
   BASELIBRARY_PUBLIC_FUNC std::string escape_sql_string(const std::string &string, bool wildcards = false); // "strings" or 'strings'
-  BASELIBRARY_PUBLIC_FUNC std::string unescape_sql_string(const std::string &string);
+  BASELIBRARY_PUBLIC_FUNC std::string unescape_sql_string(const std::string &string, char escape_char);
   BASELIBRARY_PUBLIC_FUNC std::string escape_backticks(const std::string &string);  // `identifier`
   BASELIBRARY_PUBLIC_FUNC std::string extract_option_from_command_line(const std::string& option,
     const std::string &command_line);
   
   BASELIBRARY_PUBLIC_FUNC bool parse_font_description(const std::string &fontspec, std::string &font,
-                                                      int &size, bool &bold, bool &italic);
+                                                      float &size, bool &bold, bool &italic);
 
   // Searching, splitting etc.
   BASELIBRARY_PUBLIC_FUNC bool starts_with(const std::string& s, const std::string& part);
@@ -90,7 +93,21 @@ namespace base
   BASELIBRARY_PUBLIC_FUNC std::vector<std::string> split_by_set(const std::string &s, const std::string &separator_set, int count = -1);
   BASELIBRARY_PUBLIC_FUNC std::vector<std::string> split_token_list(const std::string &s, int sep);
   BASELIBRARY_PUBLIC_FUNC bool partition(const std::string &s, const std::string &sep, std::string &left, std::string &right);
-  BASELIBRARY_PUBLIC_FUNC int index_of(std::vector<std::string> list, const std::string &s);
+  BASELIBRARY_PUBLIC_FUNC int index_of(const std::vector<std::string> &list, const std::string &s);
+  
+  // You cannot export a template function, only specializations. But since the code is in the
+  // header we don't need to export.
+  template<class C> std::string join(const C &list, const std::string &sep)
+  {
+    std::string s;
+    for (typename C::const_iterator i = list.begin(); i != list.end(); ++i)
+    {
+      if (i != list.begin())
+        s.append(sep);
+      s.append(*i);
+    }
+    return s;
+  }
 
   // These functions will perform newline conversion depending on OS. In case of error an exception is thrown.
   BASELIBRARY_PUBLIC_FUNC void set_text_file_contents(const std::string &filename, const std::string &data);
@@ -100,6 +117,9 @@ namespace base
   BASELIBRARY_PUBLIC_FUNC std::string unquote_identifier(const std::string& identifier);
   BASELIBRARY_PUBLIC_FUNC std::string quote_identifier_if_needed(const std::string &ident, const char quote_char);
 
+  BASELIBRARY_PUBLIC_FUNC bool stl_string_compare(const std::string &first, const std::string &second, bool case_sensitive = true);
+  BASELIBRARY_PUBLIC_FUNC int string_compare(const std::string &first, const std::string &second, bool case_sensitive = true);
+
 #ifdef __APPLE__
 #undef check
 #endif
