@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,8 @@
 
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
+#include "base/file_functions.h"
+
 #include <stdexcept>
 #include <glib.h>
 #include <stdio.h>
@@ -97,7 +99,7 @@ namespace base {
 
     std::string pure_pattern = pattern.substr(strlen(path) + 1);
 
-    GPatternSpec *pat = g_pattern_spec_new(g_basename(pattern.c_str()));
+    GPatternSpec *pat = g_pattern_spec_new(g_path_get_basename(pattern.c_str()));
     GDir *dir;
     {
       GError *err = NULL;
@@ -308,7 +310,7 @@ namespace base {
                               path.c_str()), GetLastError());
     }
 #else
-    if (g_mkdir(path_from_utf8(path).c_str(), mode) < 0)
+    if (g_mkdir_with_parents(path_from_utf8(path).c_str(), mode) < 0)
     {
       if (errno == EEXIST)
         return false;
@@ -329,6 +331,33 @@ namespace base {
       throw file_error(strfmt("Could not rename file %s to %s", from.c_str(), to.c_str()), errno);
 #endif
   }
+
+  bool remove_recursive(const std::string &path)
+  {
+    GError *error= NULL;
+    GDir* dir;
+    const char *dir_entry;
+    gchar *entry_path;
+  
+    dir= g_dir_open(path.c_str(), 0, &error);
+    if (!dir && error)
+      return false;
+  
+    while ((dir_entry= g_dir_read_name(dir)))
+    {
+      entry_path= g_build_filename(path.c_str(), dir_entry, NULL);
+      if (g_file_test(entry_path, G_FILE_TEST_IS_DIR))
+        (void) remove_recursive(entry_path);
+      else
+        (void) ::g_remove(entry_path);
+      g_free(entry_path);
+    }
+  
+    (void) g_rmdir(path.c_str());
+  
+    g_dir_close(dir);
+    return true;
+  }
   
   bool remove(const std::string &path)
   {
@@ -371,17 +400,19 @@ namespace base {
       g_free(f);
       return true;
     }
+    g_free(f);
     return false;
   }
 
   bool is_directory(const std::string &path)
   {
     char *f = g_filename_from_utf8(path.c_str(), -1, NULL, NULL, NULL);
-    if (g_file_test(f, (GFileTest)(G_FILE_TEST_EXISTS|G_FILE_TEST_IS_DIR)))
+    if (g_file_test(f, G_FILE_TEST_IS_DIR))
     {
       g_free(f);
       return true;
     }
+    g_free(f);
     return false;
   }
   /*
@@ -433,5 +464,120 @@ namespace base {
     }
     return path;
   }
+
+
+  //---
+  /** @brief Wrapper around fopen that expects a filename in UTF-8 encoding
+   @param filename name of file to open
+   @param mode second argument of fopen
+   @return If successful, base_fopen returns opened FILE*.
+   *//////////////////////////////////////////////////////////////////////////////
+  FILE *fopen(const std::string &filename, const char *mode)
+  {
+#ifdef _WIN32
+    
+    // Convert filename from UTF-8 to UTF-16.
+    int required;
+    WCHAR* converted;
+    WCHAR* converted_mode;
+    WCHAR* in;
+    char* out;
+    FILE* result;
+    
+    required= MultiByteToWideChar(CP_UTF8, 0, filename.c_str(), -1, NULL, 0);
+    if (required == 0)
+      return NULL;
+    
+    // Required contains the length for the result string including the terminating 0.
+    converted= new WCHAR[required];
+    memset(converted, 0, sizeof(WCHAR) * required);
+    MultiByteToWideChar(CP_UTF8, 0, filename.c_str(), -1, converted, required);
+    
+    converted_mode= new WCHAR[strlen(mode) + 1];
+    memset(converted_mode, 0, sizeof(WCHAR) * (strlen(mode) + 1));
+    in= converted_mode;
+    out= (char*) mode;
+    while (*out)
+      *in++ = (WCHAR) (*out++);
+    
+    result= _wfopen(converted, converted_mode);
+    delete[] converted;
+    delete[] converted_mode;
+    
+    return result;
+    
+#else
+    
+    FILE *file;
+    char * local_filename;
+    
+    if (! (local_filename= g_filename_from_utf8(filename.c_str(),-1,NULL,NULL,NULL)))
+      return NULL;
+    
+    file= ::fopen(local_filename, mode);
+    
+    g_free(local_filename);
+    
+    return file;
+#endif
+  }
+  
+
+  FILE * FileHandle::fopen(const char *filename, const char *mode, bool throw_on_fail)
+  {
+    FILE *file= base::fopen(filename, mode);
+    if (!file && throw_on_fail)
+      throw file_error(std::string("Failed to open file \"").append(filename), errno);
+    return file;
+  }
+ 
+  void FileHandle::fclose(FILE *f)
+  {
+    ::fclose(f);
+  }
+ 
+  FileHandle & FileHandle::operator =(FileHandle &fh)
+  {
+    dispose();
+    swap(fh);
+    return *this;
+  }
+ 
+  void FileHandle::swap(FileHandle &fh)
+  {
+    std::swap(_file, fh._file);
+  }
+ 
+  void FileHandle::dispose()
+  {
+    if (_file)
+    {
+      fclose(_file);
+      _file= NULL;
+    }
+  }
 };
 
+/**
+ * Returns the last modification time of the given file.
+ */
+bool base::file_mtime(const std::string &path, time_t &mtime)
+{
+#ifdef _WIN32
+  struct _stat stbuf;
+#else
+  struct stat stbuf;
+#endif
+
+  if (base_stat(path.c_str(), &stbuf) == 0)
+  {
+#ifdef __APPLE__
+    mtime = (time_t)stbuf.st_mtimespec.tv_sec;
+#else
+    mtime = stbuf.st_mtime;
+#endif
+    return true;
+  }
+  return false;
+}
+
