@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -218,6 +218,17 @@ string toupper(const string& s)
   return result;
 }
 
+//--------------------------------------------------------------------------------------------------
+int utf8_case_collate(const std::string &s1, const std::string &s2)
+{
+  char *t1 = g_utf8_casefold(s1.data(), s1.length());
+  char *t2 = g_utf8_casefold(s2.data(), s2.length());
+  int r = g_utf8_collate(t1, t2);
+  g_free(t1);
+  g_free(t2);
+  return r;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 std::string truncate_text(const std::string& s, int max_length)
@@ -349,7 +360,7 @@ std::vector<std::string> split_token_list(const std::string &s, int sep)
           if (p < end)
           {
             if (s[p] != sep)
-              log_debug("Error splitting string list");
+              log_debug("Error splitting string list\n");
             else
               p++;
           }
@@ -382,7 +393,7 @@ std::vector<std::string> split_token_list(const std::string &s, int sep)
           if (p < end)
           {
             if (s[p] != sep)
-              log_debug("Error splitting string list");
+              log_debug("Error splitting string list\n");
             else
               p++;
           }
@@ -434,20 +445,22 @@ bool partition(const std::string &s, const std::string &sep, std::string &left,
   right = "";
   return false;
 }
-  
+
 //--------------------------------------------------------------------------------------------------
 
 /**
  * Returns the index of the given string in the given vector or -1 if not found.
  */
-int index_of(std::vector<std::string> list, const std::string &s)
+int index_of(const std::vector<std::string> &list, const std::string &s)
 {
-  std::vector<string>::iterator location = std::find(list.begin(), list.end(), s);
+  std::vector<string>::const_iterator location = std::find(list.begin(), list.end(), s);
   if (location == list.end())
     return -1;
   return location - list.begin();
 }
 
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Returns a string containing all characters beginning at "start" from the given string "id", which form
  * a valid, unqualified identifier. The returned identifier does not contain any quoting anymore.
@@ -940,34 +953,39 @@ std::string escape_sql_string(const std::string &s, bool wildcards)
   return result;
 }
 
-
-std::string unescape_sql_string(const std::string &s)
+/**
+ * Removes repeated quote chars and supported escape sequences from the given string.
+ * Invalid escape sequences are handled like in the server, by dropping the backslash and
+ * using the wrong char as normal char.
+ */
+std::string unescape_sql_string(const std::string &s, char quote_char)
 {
   std::string result;
   result.reserve(s.size());
   
-  for (std::string::const_iterator ch= s.begin(); ch != s.end(); ++ch)
+  for (std::string::const_iterator ch = s.begin(); ch != s.end(); ++ch)
   {
     int out = *ch;
-    if (out == '\'' && (ch+1) != s.end() && *(ch+1) == '\'')
-      continue; // skip ' in ''
+    if (out == quote_char)
+    {
+      if ((ch + 1) != s.end() && *(ch + 1) == quote_char)
+        ++ch; // Skip the first of the quote char pair.
+    }
     else if (out == '\\')
     {
-      if ((ch+1) != s.end())
+      ++ch;
+      if (ch == s.end())
+        break;
+
+      switch (*ch)
       {
-        ++ch;
-        switch (*ch)
-        {
         case 'n': out = '\n'; break;
         case 't': out = '\t'; break;
         case 'r': out = '\r'; break;
         case 'b': out = '\b'; break;
         case '0': out = 0; break;         // Ascii null
-        case 'Z': out = '\032'; break;         // Win32 end of file
-     //   case 'N': found_null=1;
-        /* fall through */
+        case 'Z': out = '\032'; break;    // Win32 end of file
         default: out = *ch; break;
-        }
       }
     }
     result.push_back(out);
@@ -979,7 +997,8 @@ std::string unescape_sql_string(const std::string &s)
 
 //--------------------------------------------------------------------------------------------------
 
-// NOTE: This is not the same as escape_sql_string, as embedded ` are escaped as ``, not \`
+// NOTE: This is not the same as escape_sql_string, as embedded ` must be escaped as ``, not \`
+// and \ ' and " must not be escaped
 std::string escape_backticks(const std::string &s)
 {
   std::string result;
@@ -1000,15 +1019,6 @@ std::string escape_backticks(const std::string &s)
       case '\r':
         escape= 'r';
         break;
-      case '\\':
-        escape= '\\';
-        break;
-      case '\'':
-        escape= '\'';
-        break;
-      case '"':                           /* Better safe than sorry */
-        escape= '"';
-        break;
       case '\032':                        /* This gives problems on Win32 */
         escape= 'Z';
         break;
@@ -1082,10 +1092,10 @@ std::string extract_option_from_command_line(const std::string& option, const st
  *
  * @return True if successful, otherwise false.
  */
-bool parse_font_description(const std::string &fontspec, std::string &font, int &size, bool &bold,
+bool parse_font_description(const std::string &fontspec, std::string &font, float &size, bool &bold,
                             bool &italic)
 {
-  std::vector<std::string> parts= split(fontspec, " ");
+  std::vector<std::string> parts = split(fontspec, " ");
   font = fontspec;
   size = 12;
   bold = false;
@@ -1094,7 +1104,7 @@ bool parse_font_description(const std::string &fontspec, std::string &font, int
   if (parts.empty())
     return false;
   
-  if (!parts.empty() && sscanf(parts.back().c_str(), "%i", &size) == 1)
+  if (!parts.empty() && sscanf(parts.back().c_str(), "%f", &size) == 1)
     parts.pop_back();
   
   for (int i= 0; i < 2 && !parts.empty(); i++)
@@ -1114,9 +1124,9 @@ bool parse_font_description(const std::string &fontspec, std::string &font, int
   
   if (!parts.empty())
   {
-    font= parts[0];
-    for (unsigned int i= 1; i < parts.size(); i++)
-      font+= " " + parts[i];
+    font = parts[0];
+    for (unsigned int i = 1; i < parts.size(); i++)
+      font += " " + parts[i];
   }
   return true;  
 }
@@ -1128,13 +1138,16 @@ std::string unquote_identifier(const std::string& identifier)
   int start = 0;
   int size = identifier.size();
 
+  if (size == 0)
+    return "";
+
   if (identifier[0] == '"' || identifier[0] == '`')
     start++;
 
-  if (identifier[size-1] == '"' || identifier[size-1] == '`')
+  if (identifier[size - 1] == '"' || identifier[size - 1] == '`')
     size--;
 
-  size-=start;
+  size -= start;
 
   return identifier.substr(start, size);
 }
@@ -1156,20 +1169,65 @@ std::string quote_identifier(const std::string& identifier, const char quote_cha
 std::string quote_identifier_if_needed(const std::string &ident, const char quote_char)
 {
   bool needs_quotation= false;
+  int digits = 0;
   for (std::string::const_iterator i= ident.begin(); i != ident.end(); ++i)
   {
     if ((*i >= 'a' && *i <= 'z') || (*i >= 'A' && *i <= 'Z') || (*i >= '0' && *i <= '9')
       || (*i == '_') || (*i == '$') || ((unsigned char)(*i) > 0x7F))
+    {
+      if (*i >= '0' && *i <= '9')
+        digits++;
+      
       continue;
+    }
     needs_quotation = true;
     break;
   }
-  if (needs_quotation)
+  if (needs_quotation || digits == ident.length())
     return quote_char + ident + quote_char;
   else
     return ident;
 }
 
+/**
+ * Function : stl_string_compare
+ * Description : comparison function to be used on the sorting process
+ * Return Value : following the STL requirements should return true if the
+ *                first string is lower than the second
+ */
+bool stl_string_compare(const std::string &first, const std::string &second, bool case_sensitive)
+{
+  return string_compare(first, second, case_sensitive) < 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Function : string_compare
+ * Description : compares two strings either case sensitive or no
+ * Parameters : the strings to be compared
+ * Return Value:
+ *              0 - If the strings are equal
+ *              Negative value if the first string is lower than the second
+ *              Positive value if the first string is bigger than the second
+ */
+ int string_compare(const std::string &first, const std::string &second, bool case_sensitive)
+{
+  int ret_val = 0;
+  
+  if (case_sensitive)
+  {
+    if (first < second)
+      ret_val = -1;
+    else if (first > second)
+      ret_val = 1;
+  }
+  else
+    ret_val = g_strcasecmp(first.c_str(), second.c_str());
+    
+  return ret_val;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 EolHelpers::Eol_format EolHelpers::detect(const std::string &text)
