@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,9 +20,11 @@
 #include "stdafx.h"
 
 #include "base/string_utilities.h"
-
+#include "base/log.h"
 #include "mforms/mforms.h"
 
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
+
 using namespace mforms;
 
 //--------------------------------------------------------------------------------------------------
@@ -38,18 +40,23 @@ AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
     _app_view_impl->create(this, horiz);
   
   _identifier = base::strfmt("avid%i", ++_serial);
+
+  _dpoint = 0;
 }
 
 #else
 
-AppView::AppView(bool horiz, const std::string &context_name, bool is_main) 
+AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
 : Box(horiz), _context_name(context_name), _menubar(0), _toolbar(0), _is_main(is_main)
 {
 #ifdef __APPLE__
   // default, empty toolbar for mac, to show the 3px bar under the top tabs
   _toolbar = new mforms::ToolBar(mforms::MainToolBar);
+  set_back_color("#e8e8e8");
 #endif
   _identifier = base::strfmt("avid%i", ++_serial);
+
+  _dpoint = 0;
 }
 
 #endif
@@ -66,6 +73,13 @@ AppView::~AppView()
 
 //--------------------------------------------------------------------------------------------------
 
+void AppView::set_containing_docking_point(mforms::DockingPoint *dpoint)
+{
+  _dpoint = dpoint;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void AppView::set_menubar(mforms::MenuBar *menu)
 {
   if (_menubar != menu)
@@ -97,8 +111,10 @@ void AppView::set_toolbar(mforms::ToolBar *toolbar)
 void AppView::set_title(const std::string &title)
 { 
   _title= title;
-  //XXX remove this callback and make it send a notification
-  App::get()->set_view_title(this, title); 
+  if (_dpoint)
+    _dpoint->set_view_title(this, title);
+  else
+    log_error("Calling AppView::set_title() on an undocked view\n");
 }
 
 //--------------------------------------------------------------------------------------------------
