@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,15 +24,6 @@
 
 #import "NSColor_extras.h"
 
-enum
-{
-  ExpandFlag = (1<<0),
-  FillFlag =   (1<<1),
-  PackEndFlag= (1<<2),
-  RemovingFlag=(1<<3), // the subview is being removed and should be ignored
-  
-  BoxFlagMask= 0xf
-};
 
 @implementation MFBoxImpl
 
@@ -298,7 +289,7 @@ enum
 
 - (void)subviewMinimumSizeChanged
 {
-  if (!mOwner->is_destroying())
+  if (!mDestroying && !mOwner->is_destroying())
     [super subviewMinimumSizeChanged];
 }
 
@@ -372,10 +363,22 @@ enum
 {
   // mark the subview so that it's not accounted when relayouting
   [(id)subview setTag: ([subview tag]&~BoxFlagMask) | RemovingFlag];
-  
-  [self subviewMinimumSizeChanged];
+  if (!mDestroying)
+    [self subviewMinimumSizeChanged];
 }
 
+- (void)setFrame: (NSRect)frame
+{
+  [super setFrame: frame];
+  if (!mOwner->is_destroying())
+      (*mOwner->signal_resized())();
+}
+
+- (void)destroy
+{
+  mDestroying = YES;
+  [self removeFromSuperview];
+}
 
 #if 0
 - (void)drawRect:(NSRect)rect
@@ -439,7 +442,7 @@ static void box_set_homogeneous(::mforms::Box *self, bool flag)
 static void box_add(::mforms::Box *self, ::mforms::View *child, bool expand, bool fill)
 {
   if ([child->get_data() tag] < 0)
-    NSLog(@"NEGATIVE TAG IN NEW OBJECT %i", [child->get_data() tag]);
+    NSLog(@"NEGATIVE TAG IN NEW OBJECT %li", (long)[child->get_data() tag]);
 
   NSUInteger flags= [child->get_data() tag] & ~BoxFlagMask;
   id last= nil;
@@ -470,14 +473,17 @@ static void box_add(::mforms::Box *self, ::mforms::View *child, bool expand, boo
     [self->get_data() addSubview:child->get_data() positioned:NSWindowBelow relativeTo:last];
   assert([child->get_data() superview] == self->get_data());
   if ([self->get_data() setFreezeRelayout: NO])
+  {
     [self->get_data() subviewMinimumSizeChanged];
+    [self->get_data() resizeSubviewsWithOldSize: [self->get_data() frame].size];
+  }
 }
 
 
 static void box_add_end(::mforms::Box *self, ::mforms::View *child, bool expand, bool fill)
 {
   if ([child->get_data() tag] < 0)
-    NSLog(@"NEGATIVE TAG IN NEW OBJECT %i", [child->get_data() tag]);
+    NSLog(@"NEGATIVE TAG IN NEW OBJECT %li", (long)[child->get_data() tag]);
 
   NSUInteger flags= ([child->get_data() tag] & ~BoxFlagMask) | PackEndFlag;
   id last= nil;
