@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,8 +32,8 @@
 - (id)initWithFrame:(NSRect)frameRect
 {
   self = [super initWithFrame: frameRect];
-  if (self != nil)
-    mDefaultBackColor = [[NSColor colorWithDeviceWhite:232/255.0 alpha:1.0] retain];
+  //  if (self != nil)
+  //  mDefaultBackColor = [[NSColor colorWithDeviceWhite:232/255.0 alpha:1.0] retain];
   return self;
 }
 
@@ -111,7 +111,7 @@
 
 //--------------------------------------------------------------------------------------------------
 
-- (void)setFreezeRelayout:(BOOL)flag
+- (BOOL)setFreezeRelayout:(BOOL)flag
 {
   if (flag)
     mFreezeRelayout++;
@@ -120,10 +120,16 @@
     mFreezeRelayout--;
     if (mFreezeRelayout == 0)
     {
-      [self subviewMinimumSizeChanged];
+      // exec when idle to avoid crashes caused by unsettled structure changes
+      // (like when removing subviews from a container destructor)
+      [self performSelector: @selector(subviewMinimumSizeChanged)
+                 withObject: nil
+                 afterDelay: 0];
       [[self window] recalculateKeyViewLoop];
+      return YES;
     }
   }
+  return NO;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -153,6 +159,7 @@
     // check if we're inside a NSBox or NSTabView, which draw their own background
     while (parent)
     {
+      // TODO: move MFTabViewImpl to mforms dylib and use normal class access instead objc_getClass.
       if ([parent isKindOfClass: [NSBox class]] || ([parent isKindOfClass: objc_getClass("MFTabViewImpl")] && 
                                                     [[parent tabView] tabViewType] != NSNoTabsNoBorder))
         break;
@@ -170,52 +177,68 @@
     float x, y;
     NSSize isize = [mBackImage size];
     NSSize fsize = [self frame].size;
-    
+
+    if (fsize.height < isize.height)
+      isize.height = fsize.height;
+    if (fsize.width < isize.width)
+      isize.width = fsize.width;
+
     switch (mBackImageAlignment)
     {
       case mforms::BottomLeft:
-        x = 0;
-        y = 0;
-        break;
-      case mforms::BottomCenter:
-        x = (isize.width+fsize.width) / 2;
-        y = 0;
-        break;
-      case mforms::BottomRight:
-        x = fsize.width - isize.width;
-        y = 0;
-        break;
       case mforms::MiddleLeft:
-        x = 0;
-        y = (isize.height+fsize.height) / 2;
-        break;
-      case mforms::MiddleCenter:
-        x = (isize.width+fsize.width) / 2;
-        y = (isize.height+fsize.height) / 2;
-        break;
-      case mforms::MiddleRight:
-        x = fsize.width - isize.width;
-        y = (isize.height+fsize.height) / 2;
-        break;
       case mforms::TopLeft:
         x = 0;
-        y = fsize.height - isize.height;
         break;
+      case mforms::BottomCenter:
+      case mforms::MiddleCenter:
       case mforms::TopCenter:
-        x = (isize.width+fsize.width) / 2;
-        y = fsize.height - isize.height;
+        x = (fsize.width - isize.width) / 2;
         break;
+      case mforms::BottomRight:
+      case mforms::MiddleRight:
       case mforms::TopRight:
         x = fsize.width - isize.width;
-        y = fsize.height - isize.height;
         break;
       default:
+        x = 0;
         break;
     }
-    [mBackImage drawAtPoint: NSMakePoint(x, y)
-                   fromRect: NSZeroRect
-                  operation: NSCompositeSourceOver
-                   fraction: 1.0];
+
+    switch (mBackImageAlignment)
+    {
+      case mforms::BottomLeft:
+      case mforms::BottomCenter:
+      case mforms::BottomRight:
+        if (![self isFlipped])
+          y = 0;
+        else
+          y = fsize.height - isize.height;
+        break;
+      case mforms::MiddleLeft:
+      case mforms::MiddleCenter:
+      case mforms::MiddleRight:
+          y = (fsize.height - isize.height) / 2;
+        break;
+      case mforms::TopLeft:
+      case mforms::TopCenter:
+      case mforms::TopRight:
+        if ([self isFlipped])
+          y = 0;
+        else
+          y = fsize.height - isize.height;
+        break;
+      default:
+        y = 0;
+        break;
+    }
+
+    [mBackImage drawInRect: NSMakeRect(x, y, isize.width, isize.height)
+                  fromRect: NSZeroRect
+                 operation: NSCompositeSourceOver
+                  fraction: 1.0
+            respectFlipped: YES
+                     hints: nil];
   }
 }
 
