@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,17 +22,8 @@
 #import "MFView.h"
 #import "MFMForms.h"
 
-// TODO: remove when we stop supporting 10.5 Starting with 10.6 this is part of NSSavePanel anyway.
-@interface NSSavePanel(TemporaryPredefinition)
-- (void)setShowsHiddenFiles:(BOOL)flag;
-@end
-
 @implementation MFFileChooserImpl
 
-#ifndef NSAppKitVersionNumber10_5
-#define NSAppKitVersionNumber10_5 949
-#endif
-
 - (id)initWithObject: (mforms::FileChooser*) chooser type: (mforms::FileChooserType) type showHidden: (BOOL) showHidden
 {
   self= [super init];
@@ -54,12 +45,7 @@
         [panel setExtensionHidden: NO];
         
         if (showHidden)
-        {
-          // Only available when running on 10.6 or later.
-          if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
-            [panel setShowsHiddenFiles: YES];
-          }
-        }
+          [panel setShowsHiddenFiles: YES];
         break;
       }
       
@@ -79,11 +65,7 @@
         [panel setExtensionHidden: NO];
 
         if (showHidden)
-        {
-          if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
-            [panel setShowsHiddenFiles: YES];
-          }
-        }
+          [panel setShowsHiddenFiles: YES];
         break;
       }
     }
@@ -142,12 +124,6 @@
 }
 
 
-- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
-{
-  [NSApp stopModalWithCode: returnCode];
-}
-
-
 - (NSString *)panel:(id)sender userEnteredFilename:(NSString *)filename confirmed:(BOOL)okFlag
 {
   NSPopUpButton *fileTypeMenu = [mOptionControls objectForKey: @"format"];
@@ -192,32 +168,34 @@ static bool filechooser_run_modal(mforms::FileChooser *self)
     if (chooser->mParent)
     {
       [chooser->mPanel setMessage: [chooser->mPanel title]];
-      [chooser->mPanel beginSheetForDirectory: nil
-                                         file: filename
-                               modalForWindow: chooser->mParent->get_data()
-                                modalDelegate: chooser 
-                               didEndSelector: @selector(savePanelDidEnd:returnCode:contextInfo:)
-                                  contextInfo: 0];
-      if ([NSApp runModalForWindow: chooser->mPanel] == NSFileHandlingPanelOKButton)
-        return true;
+      [chooser->mPanel setDelegate: chooser];
+      if (filename != nil)
+        [chooser->mPanel setNameFieldStringValue: filename];
+      __block bool retval = false;
+      [chooser->mPanel beginSheetModalForWindow: chooser->mParent->get_data()
+                              completionHandler: ^(NSInteger result) {
+                                retval = result == NSFileHandlingPanelOKButton;
+                              }];
+      return retval;
     }
     else
     {
       [chooser->mPanel setMessage: @""];
-      if ([chooser->mPanel runModalForDirectory:nil file: filename] == NSFileHandlingPanelOKButton)
+      if (filename != nil)
+        [chooser->mPanel setNameFieldStringValue: filename];
+      if ([chooser->mPanel runModal] == NSFileHandlingPanelOKButton)
         return true;
     }
   }
   return false;
 }
 
-
 static void filechooser_set_directory(mforms::FileChooser *self, const std::string &path)
 {
   MFFileChooserImpl *chooser= self->get_data();
   if (chooser)
   {
-    [chooser->mPanel setDirectory:wrap_nsstring(path)];
+    [chooser->mPanel setDirectoryURL: [NSURL fileURLWithPath: wrap_nsstring(path) isDirectory: YES]];
   }
 }
 
@@ -227,7 +205,7 @@ static std::string filechooser_get_directory(mforms::FileChooser *self)
   MFFileChooserImpl *chooser= self->get_data();
   if (chooser)
   {
-    return [[chooser->mPanel directory] UTF8String];
+    return [chooser->mPanel.URL.path UTF8String];
   }
   return "";
 }
@@ -237,7 +215,7 @@ static std::string filechooser_get_path(mforms::FileChooser *self)
 {
   MFFileChooserImpl *chooser= self->get_data();
   if (chooser)
-    return [[chooser->mPanel filename] UTF8String];
+    return [chooser->mPanel.URL.path UTF8String];
   return "";
 }
 
@@ -250,7 +228,9 @@ static void filechooser_set_path(mforms::FileChooser *self, const std::string &p
     NSString *fpath = wrap_nsstring(path);
 
     NSString *extension = [fpath pathExtension];
-    [chooser->mPanel setDirectory: [fpath stringByDeletingLastPathComponent]];
+    NSString *directory = [fpath stringByDeletingLastPathComponent];
+    if (directory)
+      [chooser->mPanel setDirectoryURL: [NSURL fileURLWithPath: directory isDirectory: YES]];
     chooser->mDefaultFileName = [[fpath lastPathComponent] retain];
     
     NSPopUpButton *popup = [chooser->mOptionControls objectForKey: @"format"];
