@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,8 +32,8 @@ static struct
   const float spacing;
 } layout_info[] = 
 {
-  {36, 12, 5, 36 - 2 * 7, 7}, // Main
-  {26, 12, 2, 25 - 2 * 7, 7}, // Secondary
+  {32, 12, 5, 36 - 2 * 7, 10}, // Main
+  {26, 12, 2, 25 - 2 * 7, 8}, // Secondary
   {36, 12, 7, 36 - 2 * 7, 7}, // ToolPicker
   {20, 8, 1, 20-2*1, 4}, // Options
   {26, 4, 3, 26-2*3, 2} // Palette
@@ -298,6 +298,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
     [self setBordered: NO];
     [self setEditable: NO];
     [self setDrawsBackground: NO];
+    [self setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
   }
   return self;
 }
@@ -307,6 +308,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return mOwner;
 }
 
+/*
 - (void) viewDidMoveToSuperview
 {
   switch ([(MFToolBarImpl*)[self superview] type])
@@ -317,7 +319,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
     default:
       break;
   }
-}
+}*/
 
 - (void) setStringValue:(NSString *)aString
 {
@@ -384,7 +386,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
     mOwner = item;
     mOwner->set_data(self);
     [[self cell] setSendsSearchStringImmediately: NO];
-    [[self cell] setSendsActionOnEndEditing: YES];
+//    [[self cell] setSendsActionOnEndEditing: YES];
     [self setTarget: self];
     [self setAction: @selector(perform:)];
   }
