@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -374,11 +374,11 @@ public:
     if (parent)
       parent->expand();
 
+    [[[_self treeNodeView] outlineView] expandItem: _self];
     [[[_self treeNodeView] outlineView] performSelector: @selector(expandItem:)
                                              withObject: _self
                                              afterDelay: 0.0];
 
-//    [[[_self treeNodeView] outlineView] expandItem: _self];
   }
   
   virtual void collapse()
@@ -446,16 +446,15 @@ public:
     return image;
   }
   
- 
   virtual void set_icon_path(int column, const std::string &icon)
   {
     NSImage *image = get_icon(icon);
 
     if (image)
-    {
       [_self setObject: image
                 forKey: [[[_self treeNodeView] keyForColumn: column] stringByAppendingString: @"icon"]];
-    }
+    else
+      [_self removeObjectForKey: [[[_self treeNodeView] keyForColumn: column] stringByAppendingString: @"icon"]];
   }
   
   virtual void set_attributes(int column, const mforms::TreeNodeTextAttributes& attrs)
@@ -500,8 +499,9 @@ public:
     NSString *s = [_self objectForKey: [[_self treeNodeView] keyForColumn: column]];
     if (s)
       return [s UTF8String];
+//    NSLog(@"Invalid column %i for TreeNode::grt_string()", column);
     return "";
-	}
+  }
 
   virtual int get_int(int column) const
   {
@@ -606,6 +606,9 @@ inline TreeNodeImpl *from_ref(mforms::TreeNodeRef node)
   NSMutableArray *children = [mParent children];
   if (children)
   {
+    // if the node being removed is selected, unselect it, so that the selection isn't passed to a different node
+
+
     [[self retain] autorelease];
     [children removeObject: self];
     mParent = nil;
@@ -654,12 +657,9 @@ static NSImage *descendingSortIndicator=nil;
 
 - (NSMenu*)menuForEvent:(NSEvent *)event
 {
-  mforms::Menu *menu = mOwner->get_context_menu();
+  mforms::ContextMenu *menu = mOwner->get_context_menu();
   if (menu)
-  {
-    (*menu->signal_will_show())();
     return menu->get_data();
-  }
   return nil;
 }
 
@@ -699,12 +699,16 @@ static NSImage *descendingSortIndicator=nil;
     rect.origin= NSMakePoint(0, 0);
     rect.size= [NSScrollView contentSizeForFrameSize:[self frame].size hasHorizontalScroller:YES hasVerticalScroller:YES
                                           borderType:NSBezelBorder];
+
+    // TODO: might be worth making this not auto released but release it manually so we can control when it goes.
     mOutline= [[[TreeNodeViewOutlineView alloc] initWithFrame: rect] autorelease];
     ((TreeNodeViewOutlineView*)mOutline)->mOwner = mOwner;
 
     [self setDocumentView: mOutline];
     [mOutline setColumnAutoresizingStyle: NSTableViewLastColumnOnlyAutoresizingStyle];
 
+    [mOutline setAllowsEmptySelection: YES];
+
     mRootNode= [[MFTreeNodeImpl alloc] initWithOwner: self];
     [mOutline setDataSource: self];
     [mOutline setDelegate: self];
@@ -725,6 +729,10 @@ static NSImage *descendingSortIndicator=nil;
 
 - (void) dealloc
 {
+  // Setting nil is necessary since there are still datasouce and delegate messages comming in after dealloc.
+  // Might become unnecessary if we don't auto release mOutline.
+  [mOutline setDataSource: nil];
+  [mOutline setDelegate: nil];
   mTagMap.clear();
   [mAttributedFonts release];
   [mColumnKeys release];
@@ -806,8 +814,13 @@ static NSImage *descendingSortIndicator=nil;
     }
     case mforms::IntegerColumnType:
     case mforms::LongIntegerColumnType:
+    {
+      NSNumberFormatter *nf = [[[NSNumberFormatter alloc] init] autorelease];
+      [nf setNumberStyle: NSNumberFormatterNoStyle];
       [[column dataCell] setAlignment: NSRightTextAlignment];
+      [[column dataCell] setFormatter: nf];
       break;
+    }
   }
   
   [[column dataCell] setEditable: editable];
@@ -873,8 +886,23 @@ didClickTableColumn: (NSTableColumn *) tableColumn
       [outlineView setIndicatorImage: nil inTableColumn: [[outlineView tableColumns] objectAtIndex: mSortColumn]];
     mSortColumn = column;
     
-    NSSortDescriptor *sd = [[[NSSortDescriptor alloc] initWithKey: [tableColumn identifier]
-                                                        ascending: ascending] autorelease];
+    NSSortDescriptor *sd;
+
+    if ([[tableColumn dataCell] alignment] == NSRightTextAlignment)
+      sd = [NSSortDescriptor sortDescriptorWithKey: [tableColumn identifier]
+                                         ascending: ascending
+                                        comparator: ^(id o1, id o2) {
+                                          int d = [o1 intValue] - [o2 intValue];
+                                          if (d > 0)
+                                            return (NSComparisonResult)NSOrderedAscending;
+                                          else if (d < 0)
+                                            return (NSComparisonResult)NSOrderedDescending;
+                                          else
+                                            return (NSComparisonResult)NSOrderedSame;
+                                        }];
+    else
+      sd = [NSSortDescriptor sortDescriptorWithKey: [tableColumn identifier]
+                                         ascending: ascending];
     [outlineView setSortDescriptors: [NSArray arrayWithObject: sd]];
   }
 }
@@ -1534,7 +1562,7 @@ static mforms::TreeNodeRef treeview_node_at_row(mforms::TreeNodeView *self, int
         [tree reloadTreeData];
         
       id n = [tree->mOutline itemAtRow: row];
-      if (n)      
+      if (n)
         return [n nodeRef];
     }
   }
@@ -1542,6 +1570,23 @@ static mforms::TreeNodeRef treeview_node_at_row(mforms::TreeNodeView *self, int
 }
 
 
+static void treeview_set_column_visible(mforms::TreeNodeView *self, int column, bool flag)
+{
+  MFTreeNodeViewImpl *tree= self->get_data();
+  if (tree)
+    [[tree->mOutline tableColumnWithIdentifier: [NSString stringWithFormat:@"%i", column]] setHidden: !flag];
+}
+
+
+static bool treeview_get_column_visible(mforms::TreeNodeView *self, int column)
+{
+  MFTreeNodeViewImpl *tree= self->get_data();
+  if (tree)
+    return ![[tree->mOutline tableColumnWithIdentifier: [NSString stringWithFormat:@"%i", column]] isHidden];
+  return true;
+}
+
+
 void cf_treenodeview_init()
 {
   ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
@@ -1570,6 +1615,9 @@ void cf_treenodeview_init()
   f->_treenodeview_impl.row_for_node = &treeview_row_for_node;
   f->_treenodeview_impl.node_at_row = &treeview_node_at_row;
   f->_treenodeview_impl.node_with_tag = &treeview_node_with_tag;
+
+  f->_treenodeview_impl.set_column_visible = &treeview_set_column_visible;
+  f->_treenodeview_impl.get_column_visible = &treeview_get_column_visible;
 }
 
 
