@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
 
 #import <Cocoa/Cocoa.h>
 
-#include "MFBase.h"
+#import "MFBase.h"
 
 
 /** MForms control implementations must subclass their own NS counterpart and implement the methods
@@ -35,11 +35,17 @@
 - (NSSize)preferredSize;
 - (NSSize)preferredSizeForWidth:(float)width;
 
+- (void)setFixedFrameSize:(NSSize)size;
+- (NSSize)fixedFrameSize;
 - (BOOL)widthIsFixed;
 - (BOOL)heightIsFixed;
 
 // Tag is used for internal purposes (special flags), do not use it for anything else
 - (void)setTag:(NSInteger)tag;
+
+- (void)destroy;
+
+- (void)drawBounds:(NSRect)rect;
 @end
 
 namespace mforms { class View; }
