@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -199,13 +199,32 @@ void CodeEditorConfig::parse_settings()
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Recursively collects text of this and all its child entries.
+ */
+std::string collect_text(TiXmlNode *entry)
+{
+  std::string result;
+  for (TiXmlNode* child = entry->FirstChild(); child!= NULL; child = child->NextSibling())
+  {
+    const TiXmlText* childText = child->ToText();
+    if (childText)
+      result += childText->ValueStr() + collect_text(child);
+  }
+
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void CodeEditorConfig::parse_keywords()
 {
   for (TiXmlElement* entry = _language_element->FirstChildElement("keywords"); entry != NULL;
     entry = entry->NextSiblingElement("keywords"))
   {
     std::string property_name = *entry->Attribute(std::string("name"));
-    _keywords[property_name] = entry->GetText();
+    std::string text = collect_text(entry);
+    _keywords[property_name] = text;
   }
 }
 
@@ -234,7 +253,8 @@ void CodeEditorConfig::parse_styles()
 
 //----------------- CodeEditor ---------------------------------------------------------------------
 
-CodeEditor::CodeEditor()
+CodeEditor::CodeEditor(void *host)
+  : _host(host)
 {
   _code_editor_impl= &ControlFactory::get_instance()->_code_editor_impl;
 
@@ -297,7 +317,12 @@ void CodeEditor::append_text(const char* text, int length)
 
 void CodeEditor::replace_selected_text(const std::string& text)
 {
+  int start, length;
+  get_selection(start, length);
   _code_editor_impl->send_editor(this, SCI_REPLACESEL, 0, (sptr_t)text.c_str());
+
+  _code_editor_impl->send_editor(this, SCI_SETSELECTIONSTART, start + text.length(), 0);
+  _code_editor_impl->send_editor(this, SCI_SETSELECTIONEND, start + text.length(), 0);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -419,7 +444,7 @@ void CodeEditor::load_configuration(SyntaxHighlighterLanguage language)
   CodeEditorConfig config(language);
 
   // Keywords.
-  std::map<std::string, const char*> keywords = config.get_keywords();
+  std::map<std::string, std::string> keywords = config.get_keywords();
 
   // Key word list sets are from currently active lexer, so that must be set before calling here.
   int length = _code_editor_impl->send_editor(this, SCI_DESCRIBEKEYWORDSETS, 0, 0);
@@ -429,13 +454,13 @@ void CodeEditor::load_configuration(SyntaxHighlighterLanguage language)
   std::vector<std::string> keyword_list_names = base::split(keyword_sets, "\n");
   free(keyword_sets);
 
-  for (std::map<std::string, const char*>::const_iterator iterator = keywords.begin();
+  for (std::map<std::string, std::string>::const_iterator iterator = keywords.begin();
     iterator != keywords.end(); ++iterator)
   {
     std::string list_name = iterator->first;
     int list_index = base::index_of(keyword_list_names, list_name);
     if (list_index > -1)
-      _code_editor_impl->send_editor(this, SCI_SETKEYWORDS, list_index, (long)iterator->second);
+      _code_editor_impl->send_editor(this, SCI_SETKEYWORDS, list_index, (long)iterator->second.c_str());
   }
 
   // Properties.
@@ -556,7 +581,7 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
 #endif
   int lineNumberStyleWidth = _code_editor_impl->send_editor(this, SCI_TEXTWIDTH, STYLE_LINENUMBER, (long)"_999999");
   _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 0, lineNumberStyleWidth);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 0, 1); // Margin is clickable.
+  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 0, 0);
 
   // Margin: Markers.
   _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 1, 16);
@@ -566,7 +591,7 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
   _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", (sptr_t)"1");
   _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 2, 16);
   _code_editor_impl->send_editor(this, SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 2, 1);
+  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 2, 1); // Margin is clickable.
   _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
   _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
   _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
@@ -782,7 +807,7 @@ void CodeEditor::set_font(const std::string& fontDescription)
 {
   // Set this font for all styles.
   std::string font;
-  int size;
+  float size;
   bool bold;
   bool italic;
   if (base::parse_font_description(fontDescription, font, size, bold, italic))
@@ -791,7 +816,7 @@ void CodeEditor::set_font(const std::string& fontDescription)
     // scintilla requires the ! in front of the font name to interpret it as a pango/fontconfig font
     // the non-pango version is totally unusable
     if (!font.empty() && font[0] != '!')
-       font = "!"+font;
+       font = "!"+fontDescription;
 #endif
     for (int i = 0; i < 128; i++)
     {
@@ -877,7 +902,7 @@ void CodeEditor::on_notify(SCNotification* notification)
     {
     case SC_UPDATE_CONTENT:   // Contents, styling or markers have been changed.
       break;
-    case SC_UPDATE_SELECTION: // Selection has been changed.
+    case SC_UPDATE_SELECTION: // Selection has been changed or the caret moved.
       NotificationCenter::get()->send("GNTextSelectionChanged", this);
       break;
     case SC_UPDATE_V_SCROLL:  // Scrolled vertically.
@@ -891,6 +916,16 @@ void CodeEditor::on_notify(SCNotification* notification)
     break;
   }
 }
+//--------------------------------------------------------------------------------------------------
+
+void CodeEditor::on_command(int command) {
+  switch(command)
+  {
+  case SCEN_KILLFOCUS:
+    _signal_lost_focus();
+  break;
+  }
+}
 
 //--------------------------------------------------------------------------------------------------
 
@@ -995,7 +1030,7 @@ void CodeEditor::toggle_features(CodeEditorFeature features)
 
 void CodeEditor::reset_dirty()
 {
-  _code_editor_impl->send_editor(this, SCI_SETSAVEPOINT, 0, 0);
+  _code_editor_impl->send_editor(this, SCI_EMPTYUNDOBUFFER, 0, 0);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1535,7 +1570,7 @@ sptr_t CodeEditor::send_editor(unsigned int message, uptr_t wParam, sptr_t lPara
 //--------------------------------------------------------------------------------------------------
 
 
-void CodeEditor::edit_end()
+void CodeEditor::lost_focus()
 {
-  _signal_edit_end();
+  _signal_lost_focus();
 }
