@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -59,6 +59,7 @@ protected:
   static void relayout(::mforms::View *view);
   static void set_needs_repaint(::mforms::View *view);
   void size_changed();
+  void on_focus_grab();
   void setup();
   virtual void move_child(ViewImpl *child, int x, int y);
   static void suspend_layout(::mforms::View *view, bool flag);
@@ -88,6 +89,7 @@ protected:
 public:
   static void init();
   static Gtk::Widget *get_widget_for_view(mforms::View *view);
+  static mforms::View *get_view_for_widget(Gtk::Widget *w);
 };
 
 bool expose_event_slot(GdkEventExpose* event, Gtk::Widget* w);
@@ -99,6 +101,13 @@ inline Gtk::Widget *widget_for_view(mforms::View *view)
 {
   return gtk::ViewImpl::get_widget_for_view(view);
 }
+
+
+inline mforms::View *view_for_widget(Gtk::Widget *w)
+{
+  return gtk::ViewImpl::get_view_for_widget(w);
+}
+
 };
 
 #endif
