@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,11 @@
 
 #include "mforms/mforms.h"
 
+#include "base/log.h"
+#include "base/notifications.h"
+
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
+
 using namespace mforms;
 
 //--------------------------------------------------------------------------------------------------
@@ -87,6 +92,9 @@ void View::set_managed()
 
 void View::add_subview(View *sv)
 {
+  if (!sv)
+    throw std::logic_error("mforms: attempt to add NULL subview");
+
   if (sv->get_parent() != NULL)
     throw std::logic_error("mforms: attempt to add a subview already contained somewhere");
 
@@ -445,9 +453,20 @@ void View::flush_events()
 
 void View::focus()
 {
-  if (_view_impl->focus)//TODO Windows
+  if (_view_impl->focus)
     _view_impl->focus(this);
   else
-    g_warning("mforms::View::focus not implemented");
+    log_warning("mforms::View::focus not implemented");
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * To be called by platform code when the active control changes (either by code or user interaction).
+ */
+void View::focus_changed()
+{
+  base::NotificationCenter::get()->send("GNFocusChanged", this);
+}
+
+//--------------------------------------------------------------------------------------------------
