@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,10 +19,19 @@
 
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_toolbar.h"
+
+#include "base/log.h"
+#include "base/string_utilities.h"
+#include "ConvUtils.h"
+
 using namespace System::Windows::Forms;
 using namespace System::IO;
 using namespace System::Drawing;
 
+using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Controls;
 using namespace MySQL::Utilities;
@@ -48,21 +57,16 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
   int maxHeight = 0;
   int expandCount= 0;
 
-  // Consider extra pixels for the height if we have a main tool strip (it renders an inner border).
-  int borderOffset = 0;
-  if (toolstrip->Renderer->GetType() == FlatMainToolStripRenderer::typeid)
-    borderOffset = 2;
-
   if (!preferredSizeOnly)
   {
     proposedSize.Width -= toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
-    proposedSize.Height = toolstrip->Size.Height - toolstrip->Padding.Vertical - borderOffset;
+    if (proposedSize.Width < 0)
+      proposedSize.Width = 0;
   }
 
-  // The Margin property of a ToolStripItem has a special meaning. The horizontal values determine
-  // the distances to neighbor items. The vertical values reduce the available vertical space into
-  // which an item can be placed. An item is always vertically centered in its remaining space.
-  // Hence the horizontal margin extends the overall width, while the vertical distance does not.
+  proposedSize.Height = toolstrip->Size.Height - toolstrip->Padding.Vertical;
+  if (proposedSize.Height < 0)
+    proposedSize.Height = 0;
 
   // Determine total width of all items so we can distribute any remaining space over
   // all expander items.
@@ -72,12 +76,14 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
       continue;
 
     if (item->GetType() != ToolStripSegmentedButton::typeid)
-      item->Size = item->GetPreferredSize(proposedSize);
-    itemsTotalWidth += item->Width+ item->Margin.Horizontal;
+      item->Size = item->GetPreferredSize(proposedSize); // The size includes the item's margin.
+    
     if (item->Height > maxHeight)
       maxHeight = item->Height;
     if (item->GetType() == ToolStripExpander::typeid)
-      expandCount++;
+      expandCount++; // Expanders have no width per se, but get all that's unused.
+    else
+      itemsTotalWidth += item->Width;
   }
   
   // Adjust width of the container if it is too small.
@@ -105,6 +111,9 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
 
     for each (ToolStripItem^ item in toolstrip->Items)
     {
+      if (!item->Visible)
+        continue;
+
       if (item->GetType() == ToolStripExpander::typeid)
       {
         offset += expanderFraction;
@@ -117,29 +126,18 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
       else
       {
         System::Drawing::Point itemLocation;
-        int remainingHeight = proposedSize.Height - item->Margin.Vertical;
 
-        // Vertically center the item within the bounds that remain when we take off its margin.
-        itemLocation.Y = toolstrip->Padding.Top + item->Margin.Top + borderOffset / 2 +
-          (int) Math::Ceiling((remainingHeight - item->Height) / 2.0);
+        // Vertically center the item within the bounds of the toolstrip (less its padding).
+        itemLocation.Y = toolstrip->Padding.Top + (int) Math::Ceiling((proposedSize.Height - item->Height) / 2.0);
         itemLocation.X = offset;
         toolstrip->ApplyLocation(item, itemLocation);
         offset += item->Width;
-
-        // No spacing between segmented buttons, but still spacing around the first and last button.
-        if (item->GetType() == ToolStripSegmentedButton::typeid)
-        {
-          // Check if this item is surrounded by other segmented buttons.
-          offset += 0;
-        }
-        else
-          offset += item->Margin.Horizontal;
       }
     }
   }
 
   proposedSize.Width += toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
-  proposedSize.Height += toolstrip->Padding.Vertical + borderOffset;
+  proposedSize.Height += toolstrip->Padding.Vertical;
 
   return proposedSize;
 }
@@ -357,9 +355,9 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
       native_toolbar = ViewImpl::create<MformsToolStrip>(item, toolbar);
       native_toolbar->Name = "MainToolBar";
       native_toolbar->Renderer = gcnew FlatMainToolStripRenderer();
-      native_toolbar->Padding = Padding(0);
+      native_toolbar->Padding = Padding(2);
       native_toolbar->GripMargin = Padding(4, 6, 4, 6);
-      native_toolbar->Margin = Padding(4, 8, 2, 8);
+      native_toolbar->Margin = Padding(4, 5, 2, 5); // Used only for separator items. Does not influence the height.
       native_toolbar->AutoSize = true;
       break;
 
@@ -372,7 +370,7 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
       native_toolbar->GripMargin = Padding(2, 4, 2, 5);
       native_toolbar->Padding = Padding(5, 0, 0, 0);
       native_toolbar->AutoSize = false;
-      native_toolbar->Size = Size(100, 28);
+      native_toolbar->Size = Size(100, 23);
       try
       {
         native_toolbar->Font = ControlUtilities::GetFont("Microsoft Sans Serif", 8.25f);
@@ -383,7 +381,7 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
         log_error("ToolBarImpl::create_tool_bar failed. %s\n", e->Message);
       }
 
-      native_toolbar->BackColor = ColorTranslator::FromHtml("#bcc7d8");
+      native_toolbar->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
       break;
 
     case mforms::OptionsToolBar:
@@ -398,11 +396,10 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
     case mforms::ToolPickerToolBar:
       native_toolbar = ViewImpl::create<ToolStrip>(item, toolbar);
       native_toolbar->Name = "ToolPickerToolBar";
-      native_toolbar->BackColor = SystemColors::ButtonFace;
+      native_toolbar->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
       native_toolbar->GripStyle = ToolStripGripStyle::Hidden;
-      //native_toolbar->ImageScalingSize = System::Drawing::Size(24, 22);
       native_toolbar->LayoutStyle = ToolStripLayoutStyle::VerticalStackWithOverflow;
-      native_toolbar->Padding = Padding(0, 0, 0, 0);
+      native_toolbar->Padding = Padding(0);
       native_toolbar->RenderMode = ToolStripRenderMode::ManagerRenderMode;
       native_toolbar->AutoSize = true;
       break;
@@ -425,6 +422,11 @@ void ToolBarImpl::insert_item(mforms::ToolBar* toolbar, int index, mforms::ToolB
   {
     ToolStrip^ native_toolbar = toolbar_impl->get_control<ToolStrip>();
     ToolStripItem^ native_item = toolbar_item->get_control<ToolStripItem>();
+
+    // update the name of the control, so the accessibility support has something to work with
+    native_item->Name = CppStringToNative(item->get_name());
+    native_item->AccessibleName = native_item->Name;
+
     if (index < 0 || index >= native_toolbar->Items->Count)
     {
       index = native_toolbar->Items->Count;
@@ -470,12 +472,12 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
   case mforms::TextActionItem:
     native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
     native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
-    native_item->ForeColor = ColorTranslator::FromHtml("#4a6184");
+    native_item->ForeColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, true);
     break;
 
   case mforms::LabelItem:
     native_item = ViewImpl::create<ToolStripLabel>(item, toolbar_item, native_item);
-    native_item->ForeColor = Color::Black;
+    native_item->ForeColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, true);
     break;
 
   case mforms::ToggleItem:
@@ -548,6 +550,7 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
   }
 
   native_item->ImageScaling = ToolStripItemImageScaling::None;
+  native_item->Name = CppStringToNative(item->get_name());
 
   return true;
 }
@@ -586,25 +589,45 @@ void ToolBarImpl::set_item_text(mforms::ToolBarItem* item, const std::string& te
   if (toolbar_item != nullptr)
   {
     ToolStripItem^ native_item = toolbar_item->get_control<ToolStripItem>();
-    native_item->Text = CppStringToNative(text);
-    if (item->get_type() == mforms::ColorSelectorItem)
+    String ^native_text = CppStringToNative(text);
+
+    switch (item->get_type())
     {
-      // If this item is a color selector then find the entry with the corresponding color
-      // and use its image for the selector.
-      ToolStripDropDownButton^ selector = dynamic_cast<ToolStripDropDownButton^>(native_item);
-      bool found = false;
-      for each (ToolStripButton^ button in selector->DropDownItems)
+    case mforms::SelectorItem:
       {
-        if (button->Text == native_item->Text)
+        ToolStripComboBox ^selector = dynamic_cast<ToolStripComboBox^>(native_item);
+        int index = selector->FindStringExact(native_text);
+        if (index < 0)
+          index = 0;
+        selector->SelectedIndex = index;
+        
+        break;
+      }
+      
+    case mforms::ColorSelectorItem:
+      {
+        // If this item is a color selector then find the entry with the corresponding color
+        // and use its image for the selector.
+        ToolStripDropDownButton ^selector = dynamic_cast<ToolStripDropDownButton^>(native_item);
+        bool found = false;
+        for each (ToolStripButton^ button in selector->DropDownItems)
         {
-          native_item->Image = button->Image;
-          found = true;
-          break;
+          if (button->Text == native_text)
+          {
+            native_item->Image = button->Image;
+            found = true;
+            break;
+          }
         }
+
+        if (!found)
+          native_item->Image = create_color_image(native_item->Text);
+
+        break;
       }
 
-      if (!found)
-        native_item->Image = create_color_image(native_item->Text);
+    default:
+      native_item->Text = native_text;
     }
   }
 }
