@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 
 #include "mforms/mforms.h"
 #include "wf_view.h"
+#include "wf_treenodeview.h"
 #include "wf_treenode.h"
 
 #include "ConvUtils.h"
@@ -36,7 +37,6 @@ using namespace MySQL::Utilities;
 TreeNodeImpl::TreeNodeImpl(TreeViewAdv ^tree, Node^ node)
   : _tree(tree), _node(node), _refcount(0)
 {
-
 }
 
 void TreeNodeImpl::release()
@@ -46,26 +46,43 @@ void TreeNodeImpl::release()
     delete this;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void TreeNodeImpl::retain()
 {
   _refcount++;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 MySQL::Forms::TreeViewNode ^TreeNodeImpl::nodeptr() const
 {
   Node ^tmp = _node;
   return dynamic_cast<MySQL::Forms::TreeViewNode^>(tmp);
 }
 
+//--------------------------------------------------------------------------------------------------
+
+TreeNodeViewImpl^ TreeNodeImpl::get_tree_wrapper()
+{
+  mforms::TreeNodeView *native_tree= ObjectImpl::get_backend_control<mforms::TreeNodeView>(_tree);
+  return (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(native_tree);
+}
+  
+//--------------------------------------------------------------------------------------------------
+
 bool TreeNodeImpl::is_root() const
 {
   return _node->Parent == nullptr;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 static TreeNodeAdv ^find_node_adv_for(Node ^node, TreeNodeAdv ^root)
 {
-  if (!node)
+  if (node == nullptr)
     return nullptr;
+
   if (node->Parent == nullptr)
     return root;
   TreeNodeAdv ^parent = find_node_adv_for(node->Parent, root);
@@ -78,6 +95,8 @@ static TreeNodeAdv ^find_node_adv_for(Node ^node, TreeNodeAdv ^root)
   return nullptr;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 TreeNodeAdv^ TreeNodeImpl::find_node_adv()
 {
   // apparently no mapping between Node to TreeNodeAdv, so we need
@@ -86,12 +105,15 @@ TreeNodeAdv^ TreeNodeImpl::find_node_adv()
   return find_node_adv_for(_node, _tree->Root);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 Aga::Controls::Tree::TreeModel ^TreeNodeImpl::model()
 {
   return dynamic_cast<Aga::Controls::Tree::TreeModel^>(_tree->Model);
 }
 
+//--------------------------------------------------------------------------------------------------
+
 bool TreeNodeImpl::equals(const mforms::TreeNode &other)
 {
   const TreeNodeImpl *oth = dynamic_cast<const TreeNodeImpl*>(&other);
@@ -109,7 +131,7 @@ bool TreeNodeImpl::is_valid() const
 
 void TreeNodeImpl::set_icon_path(int column, const std::string &icon)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr && !icon.empty())
   {
     bool invalidate = true;
@@ -134,40 +156,52 @@ void TreeNodeImpl::set_icon_path(int column, const std::string &icon)
 void TreeNodeImpl::set_attributes(int column, 
   const mforms::TreeNodeTextAttributes& attrs)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
   {
     node->Attributes[column] = attrs;
-    _tree->Invalidate(); // Might lead to flickering, investigate if necessary.
+    _tree->Invalidate();
   }
 }
 
 void TreeNodeImpl::set_string(int column, const std::string &value)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
   {
     node->Text[column] = CppStringToNative(value);
-    _tree->Invalidate(); // Might lead to flickering, investigate if necessary.
+
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!wrapper->is_frozen() && is<SortableTreeModel^>(model()) && column == wrapper->current_sort_column())
+      dynamic_cast<SortableTreeModel^>(model())->Resort();
+    _tree->Invalidate();
   }
 }
 
 void TreeNodeImpl::set_int(int column, int value)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
   {
     node->Text[column] = Convert::ToString(value);
+
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!wrapper->is_frozen() && is<SortableTreeModel^>(model()) && column == wrapper->current_sort_column())
+      dynamic_cast<SortableTreeModel^>(model())->Resort();
     _tree->Invalidate();
   }
 }
 
 void TreeNodeImpl::set_long(int column, boost::int64_t value)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
   {
     node->Text[column] = Convert::ToString(value);
+
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!wrapper->is_frozen() && is<SortableTreeModel^>(model()) && column == wrapper->current_sort_column())
+      dynamic_cast<SortableTreeModel^>(model())->Resort();
     _tree->Invalidate();
   }
 }
@@ -175,17 +209,21 @@ void TreeNodeImpl::set_long(int column, boost::int64_t value)
 
 void TreeNodeImpl::set_bool(int column, bool value)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
   {
     node->Text[column] = value ? "1" : "0";
+
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!wrapper->is_frozen() && is<SortableTreeModel^>(model()) && column == wrapper->current_sort_column())
+      dynamic_cast<SortableTreeModel^>(model())->Resort();
     _tree->Invalidate();
   }
 }
 
 std::string TreeNodeImpl::get_string(int column) const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return NativeToCppString(node->Text[column]);
   return "";
@@ -193,7 +231,7 @@ std::string TreeNodeImpl::get_string(int column) const
 
 int TreeNodeImpl::get_int(int column) const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return Convert::ToInt32(node->Text[column]);
   return 0;
@@ -201,7 +239,7 @@ int TreeNodeImpl::get_int(int column) const
 
 bool TreeNodeImpl::get_bool(int column) const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return (node->Text[column] == "Checked") || (node->Text[column] == "1") ? true : false;
   return false;
@@ -210,7 +248,7 @@ bool TreeNodeImpl::get_bool(int column) const
 
 boost::int64_t TreeNodeImpl::get_long(int column) const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return Convert::ToInt64(node->Text[column]);
   return 0;
@@ -315,13 +353,14 @@ void TreeNodeImpl::add_children_from_skeletons(array<MySQL::Forms::TreeViewNode^
     for(int parent_index = 0; parent_index < parents->Length; parent_index++)
       parents[parent_index]->Nodes->Add(added_nodes[parent_index]);
 
-    // If the new nodes will have childrens as well, inserts them
+    // If the new nodes will have children as well insert them.
     if (children[child_index].children.size())
       add_children_from_skeletons(added_nodes, children[child_index].children);
 
   }
 }  
 
+//--------------------------------------------------------------------------------------------------
 
 mforms::TreeNodeRef TreeNodeImpl::insert_child(int index)
 {
@@ -341,19 +380,36 @@ mforms::TreeNodeRef TreeNodeImpl::insert_child(int index)
   return mforms::TreeNodeRef();
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void TreeNodeImpl::remove_from_parent()
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr && node->Parent != nullptr)
+  {
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!node->MyTag.empty())
+      wrapper->process_mapping(nullptr, node->MyTag);
+
     node->Parent->Nodes->Remove(node);
+  }
 }
 
 
-void TreeNodeImpl::clear_children()
+void TreeNodeImpl::remove_children()
 {
   Node^ node= _node;
   if (node != nullptr)
+  {
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    for each (Node^ child in node->Nodes)
+    {
+      TreeViewNode ^node = dynamic_cast<TreeViewNode^>(child);
+      if (node != nullptr && !node->MyTag.empty())
+        wrapper->process_mapping(nullptr, node->MyTag);
+    }
     node->Nodes->Clear();
+  }
 }
 
 
@@ -371,7 +427,7 @@ mforms::TreeNodeRef TreeNodeImpl::get_child(int index) const
 
 mforms::TreeNodeRef TreeNodeImpl::get_parent() const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr && node->Parent != nullptr)
   {
     return mforms::TreeNodeRef(new TreeNodeImpl(_tree, node->Parent));
@@ -405,14 +461,20 @@ bool TreeNodeImpl::is_expanded()
 
 void TreeNodeImpl::set_tag(const std::string &tag)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode ^node = nodeptr();
   if (node != nullptr)
+  {
+    TreeNodeViewImpl ^wrapper = get_tree_wrapper();
+    if (!node->MyTag.empty())
+      wrapper->process_mapping(nullptr, node->MyTag);
     node->MyTag = tag;
+    wrapper->process_mapping(node, node->MyTag);
+  }
 }
 
 std::string TreeNodeImpl::get_tag() const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return node->MyTag;
   return "";
@@ -421,14 +483,14 @@ std::string TreeNodeImpl::get_tag() const
 
 void TreeNodeImpl::set_data(mforms::TreeNodeData *data)
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     node->Data = data;
 }
 
 mforms::TreeNodeData *TreeNodeImpl::get_data() const
 {
-  MySQL::Forms::TreeViewNode^ node= nodeptr();
+  MySQL::Forms::TreeViewNode^ node = nodeptr();
   if (node != nullptr)
     return node->Data;
   return NULL;
