@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,19 +24,24 @@
 
 #include "stdafx.h"
 
+#include "base/log.h"
+
 #include "mforms/mforms.h"
 #include "wf_view.h"
 #include "wf_treenodeview.h"
 #include "wf_treenode.h"
-#include "wf_grttreeview.h"
-#include "wf_menu.h"
+#include "wf_menubar.h"
 
 #include "ConvUtils.h"
 
+using namespace System::Collections;
+
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
 
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
+
 //----------------- ColumnComparer -----------------------------------------------------------------
 
 /**
@@ -123,6 +128,17 @@ Collections::IEnumerable^ SortableTreeModel::GetChildren(TreePath ^treePath)
   return nullptr;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Workaround for tree manipulations not going via the model (which would trigger the
+ * structure change event automatically).
+ */
+void SortableTreeModel::Resort()
+{
+  OnStructureChanged(gcnew TreePathEventArgs());
+}
+
 //----------------- AttributedNodeText -------------------------------------------------------------
 
 void AttributedNodeText::OnDrawText(DrawEventArgs^ args)
@@ -145,9 +161,9 @@ void AttributedNodeText::OnDrawText(DrawEventArgs^ args)
 
 //----------------- TreeNodeViewImpl static interface functions ----------------------------------------
 
-bool TreeNodeViewImpl::create(::mforms::TreeNodeView *self, ::mforms::TreeOptions options)
+bool TreeNodeViewImpl::create(mforms::TreeNodeView *self, mforms::TreeOptions options)
 {
-  TreeNodeViewImpl ^treeview= gcnew TreeNodeViewImpl(self);
+  TreeNodeViewImpl ^treeview = gcnew TreeNodeViewImpl(self, (options & mforms::TreeIndexOnTag) != 0);
 
   if (treeview != nullptr)
   {
@@ -157,18 +173,23 @@ bool TreeNodeViewImpl::create(::mforms::TreeNodeView *self, ::mforms::TreeOption
     t->Model= treeview->_model;
     t->SelectionChanged += gcnew System::EventHandler(&TreeNodeViewImpl::SelectionChanged);
     t->NodeMouseDoubleClick += gcnew System::EventHandler<TreeNodeAdvMouseEventArgs^>(&TreeNodeViewImpl::NodeActivated);
-    t->UseColumns= options & ::mforms::TreeNoColumns ? false : true;
+    t->UseColumns= options & mforms::TreeNoColumns ? false : true;
     t->MouseDown += gcnew MouseEventHandler(&TreeNodeViewImpl::OnMouseDown);
     t->ColumnClicked += gcnew System::EventHandler<TreeColumnEventArgs^>(treeview, &TreeNodeViewImpl::OnColumnClicked);
     t->Collapsed += gcnew System::EventHandler<TreeViewAdvEventArgs^>(treeview, &TreeNodeViewImpl::Collapsed);
     t->Expanded += gcnew System::EventHandler<TreeViewAdvEventArgs^>(treeview, &TreeNodeViewImpl::Expanded);
-    if (options & ::mforms::TreeNoBorder) // Default is 3d border.
+    if (options & mforms::TreeNoBorder) // Default is 3d border.
+    {
       t->BorderStyle = BorderStyle::None;
 
+      // Add some padding or the content will directly start at the boundaries.
+      t->Padding = Padding(2);
+    }
+
     t->ShowLines = false;
     t->ShowPlusMinus = false;
 
-    treeview->_flat_list = (options & ::mforms::TreeFlatList) != 0;
+    treeview->_flat_list = (options & mforms::TreeFlatList) != 0;
     if ((options & mforms::TreeAltRowColors) != 0)
       t->BeforeDrawNode += gcnew System::EventHandler<TreeViewAdvDrawRowEventArgs^>(treeview, &TreeNodeViewImpl::OnBeforeNodeDrawing);
 
@@ -179,7 +200,7 @@ bool TreeNodeViewImpl::create(::mforms::TreeNodeView *self, ::mforms::TreeOption
 
 //--------------------------------------------------------------------------------------------------
 
-int TreeNodeViewImpl::add_column(::mforms::TreeNodeView *self, ::mforms::TreeColumnType type, const std::string &name, 
+int TreeNodeViewImpl::add_column(mforms::TreeNodeView *self, mforms::TreeColumnType type, const std::string &name, 
                              int initial_width, bool editable)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
@@ -190,7 +211,7 @@ int TreeNodeViewImpl::add_column(::mforms::TreeNodeView *self, ::mforms::TreeCol
 
 //--------------------------------------------------------------------------------------------------
 
-void TreeNodeViewImpl::end_columns(::mforms::TreeNodeView *self)
+void TreeNodeViewImpl::end_columns(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -199,7 +220,7 @@ void TreeNodeViewImpl::end_columns(::mforms::TreeNodeView *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void TreeNodeViewImpl::clear(::mforms::TreeNodeView *self)
+void TreeNodeViewImpl::clear(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -217,7 +238,7 @@ void TreeNodeViewImpl::set_row_height(mforms::TreeNodeView *self, int h)
 
 //--------------------------------------------------------------------------------------------------
 
-std::list<mforms::TreeNodeRef> TreeNodeViewImpl::get_selection(::mforms::TreeNodeView *self)
+std::list<mforms::TreeNodeRef> TreeNodeViewImpl::get_selection(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -227,7 +248,7 @@ std::list<mforms::TreeNodeRef> TreeNodeViewImpl::get_selection(::mforms::TreeNod
 
 //--------------------------------------------------------------------------------------------------
 
-mforms::TreeNodeRef TreeNodeViewImpl::get_selected_node(::mforms::TreeNodeView *self)
+mforms::TreeNodeRef TreeNodeViewImpl::get_selected_node(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -237,7 +258,7 @@ mforms::TreeNodeRef TreeNodeViewImpl::get_selected_node(::mforms::TreeNodeView *
 
 //--------------------------------------------------------------------------------------------------
 
-mforms::TreeNodeRef TreeNodeViewImpl::root_node(::mforms::TreeNodeView *self)
+mforms::TreeNodeRef TreeNodeViewImpl::root_node(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -248,7 +269,7 @@ mforms::TreeNodeRef TreeNodeViewImpl::root_node(::mforms::TreeNodeView *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void TreeNodeViewImpl::set_selected(::mforms::TreeNodeView *self, mforms::TreeNodeRef node, bool flag)
+void TreeNodeViewImpl::set_selected(mforms::TreeNodeView *self, mforms::TreeNodeRef node, bool flag)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -267,7 +288,7 @@ void TreeNodeViewImpl::clear_selection(mforms::TreeNodeView *self)
 
 //--------------------------------------------------------------------------------------------------
 
-mforms::TreeSelectionMode TreeNodeViewImpl::get_selection_mode(::mforms::TreeNodeView *self)
+mforms::TreeSelectionMode TreeNodeViewImpl::get_selection_mode(mforms::TreeNodeView *self)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -277,7 +298,7 @@ mforms::TreeSelectionMode TreeNodeViewImpl::get_selection_mode(::mforms::TreeNod
 
 //--------------------------------------------------------------------------------------------------
 
-void TreeNodeViewImpl::set_selection_mode(::mforms::TreeNodeView *self, mforms::TreeSelectionMode mode)
+void TreeNodeViewImpl::set_selection_mode(mforms::TreeNodeView *self, mforms::TreeSelectionMode mode)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
@@ -322,15 +343,46 @@ int TreeNodeViewImpl::row_for_node(mforms::TreeNodeView *self, mforms::TreeNodeR
   return -1;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+mforms::TreeNodeRef TreeNodeViewImpl::node_with_tag(mforms::TreeNodeView *self, const std::string &tag)
+{
+  TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+    return treeview->node_with_tag(tag);
+  return mforms::TreeNodeRef();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void TreeNodeViewImpl::set_column_visible(mforms::TreeNodeView *self, int column, bool flag)
+{
+  TreeNodeViewImpl^ treeview = (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+    treeview->set_column_visible(column, flag);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool TreeNodeViewImpl::get_column_visible(mforms::TreeNodeView *self, int column)
+{
+  TreeNodeViewImpl^ treeview = (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+    return treeview->is_column_visible(column);
+  return false;
+}
+
 //----------------- Actual implementation ----------------------------------------------------------
 
-TreeNodeViewImpl::TreeNodeViewImpl(::mforms::TreeNodeView *self)
+TreeNodeViewImpl::TreeNodeViewImpl(mforms::TreeNodeView *self, bool use_tag_index)
   : ViewImpl(self)
 {
   _model = gcnew SortableTreeModel();
   _current_sort_column = -1;
   _can_sort_column = false;
+  _freeze_count = 0;
   _current_sort_order = SortOrder::None;
+  _tag_map = use_tag_index ? gcnew Dictionary<String^, Node^>() : nullptr;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -347,7 +399,7 @@ TreeNodeViewImpl::~TreeNodeViewImpl()
 
 //--------------------------------------------------------------------------------------------------
 
-int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const std::string &name, int initial_width, bool editable)
+int TreeNodeViewImpl::add_column(mforms::TreeColumnType type, const std::string &name, int initial_width, bool editable)
 {
   TreeViewAdv^ treeview= get_control<TreeViewAdv>();
   
@@ -355,11 +407,11 @@ int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const std::strin
   NodeIcon^ icon = nullptr;
   switch (type)
   {
-  case ::mforms::CheckColumnType:
+  case mforms::CheckColumnType:
     nodeControl= gcnew NodeCheckBox();
     break;
-  case ::mforms::IntegerColumnType:
-  case ::mforms::LongIntegerColumnType:
+  case mforms::IntegerColumnType:
+  case mforms::LongIntegerColumnType:
     {
       NodeTextBox^ box= gcnew NodeTextBox(); // Does an integer column work differently than a normal text column (editor-wise)?
       box->EditEnabled= editable;
@@ -367,7 +419,7 @@ int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const std::strin
       nodeControl= box;
       break;
     }
-  case ::mforms::IconColumnType:
+  case mforms::IconColumnType:
     {
       icon = gcnew NodeIcon();
 
@@ -377,7 +429,7 @@ int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const std::strin
       nodeControl = box;
       break;
     }
-  default: // ::mforms::StringColumnType
+  default: // mforms::StringColumnType
     {
       NodeTextBox^ box= gcnew AttributedNodeText();
       box->EditEnabled= editable;
@@ -457,7 +509,7 @@ void TreeNodeViewImpl::TreeValuePushed(System::Object^ sender, NodeControlValueE
   TreeViewNode^ ourNode= dynamic_cast<TreeViewNode^>(treeNode->Tag);
 
   TreeViewAdv^ tree= dynamic_cast<TreeViewAdv^>(ourNode->Tag);
-  ::mforms::TreeNodeView* native_tree= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+  mforms::TreeNodeView* native_tree= ObjectImpl::get_backend_control<mforms::TreeNodeView>(tree);
 
   // First check if the backend allows editing that value.
   std::string new_value= NativeToCppString(e->Value->ToString());
@@ -481,7 +533,7 @@ void TreeNodeViewImpl::SelectionChanged(System::Object^ sender, EventArgs^ e)
 
   if (tree->Tag != nullptr)
   {
-    ::mforms::TreeNodeView* t= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+    mforms::TreeNodeView* t= ViewImpl::get_backend_control<mforms::TreeNodeView>(tree);
     if (t != 0)
       t->changed();
   }
@@ -498,7 +550,7 @@ void TreeNodeViewImpl::Expanded(System::Object^ sender, TreeViewAdvEventArgs^ e)
 
   if (tree->Tag != nullptr)
   {
-    ::mforms::TreeNodeView* t= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+    mforms::TreeNodeView* t= ViewImpl::get_backend_control<mforms::TreeNodeView>(tree);
     if (t != 0)
     {
       t->expand_toggle(mforms::TreeNodeRef(new TreeNodeImpl(tree, ourNode)), true);
@@ -517,7 +569,7 @@ void TreeNodeViewImpl::Collapsed(System::Object^ sender, TreeViewAdvEventArgs^ e
 
   if (tree->Tag != nullptr)
   {
-    ::mforms::TreeNodeView* t= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+    mforms::TreeNodeView* t= ViewImpl::get_backend_control<mforms::TreeNodeView>(tree);
     if (t != 0)
     {
       t->expand_toggle(mforms::TreeNodeRef(new TreeNodeImpl(tree, ourNode)), false);
@@ -536,7 +588,7 @@ void TreeNodeViewImpl::NodeActivated(System::Object^ sender, TreeNodeAdvMouseEve
 
   if (tree->Tag != nullptr)
   {
-    ::mforms::TreeNodeView* t= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+    mforms::TreeNodeView* t= ViewImpl::get_backend_control<mforms::TreeNodeView>(tree);
     if (t != 0)
     {
       int row = t->row_for_node(mforms::TreeNodeRef(new TreeNodeImpl(tree, ourNode)));
@@ -555,16 +607,16 @@ void TreeNodeViewImpl::OnMouseDown(System::Object^ sender, MouseEventArgs^ e)
 
   if (tree->Tag != nullptr && e->Button == MouseButtons::Right)
   {
-    ::mforms::TreeNodeView* t= ViewImpl::get_backend_control<::mforms::TreeNodeView>(tree);
+    mforms::TreeNodeView* t = ViewImpl::get_backend_control<mforms::TreeNodeView>(tree);
 
     // update the associated context menu
     if (t && t->get_context_menu())
     {
-      ContextMenuStrip^ menu= MenuImpl::GetNativeControl(t->get_context_menu());
+      ToolStrip^ menu = MenuBarImpl::GetNativeControl(t->get_context_menu());
       if (menu != tree->ContextMenuStrip)
       {
-        tree->ContextMenuStrip = menu;
-        (*t->get_context_menu()->signal_will_show())();
+        tree->ContextMenuStrip = (ContextMenuStrip^)menu;
+        t->get_context_menu()->will_show();
       }
     }
     else
@@ -645,7 +697,8 @@ void TreeNodeViewImpl::clear()
 
 void TreeNodeViewImpl::set_row_height(int h)
 {
-  //XXX
+  TreeViewAdv^ treeview= get_control<TreeViewAdv>();
+  treeview->RowHeight = h;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -803,6 +856,55 @@ int TreeNodeViewImpl::row_for_node(mforms::TreeNodeRef node)
 
 //--------------------------------------------------------------------------------------------------
 
+mforms::TreeNodeRef TreeNodeViewImpl::node_with_tag(const std::string &tag)
+{
+  if (_tag_map != nullptr)
+  {
+    String^ native_tag = CppStringToNativeRaw(tag);
+    Node^ node;
+    if (_tag_map->TryGetValue(native_tag, node))
+      return mforms::TreeNodeRef(new TreeNodeImpl(get_control<TreeViewAdv>(), node));
+    return mforms::TreeNodeRef();
+  }
+  throw std::logic_error("Tree node <-> tag mapping requires tree creation option TreeIndexOnTag");
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void TreeNodeViewImpl::set_column_visible(int column, bool flag)
+{
+  TreeViewAdv^ tree = get_control<TreeViewAdv>();
+  tree->Columns[column]->IsVisible = flag;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool TreeNodeViewImpl::is_column_visible(int column)
+{
+  TreeViewAdv^ tree = get_control<TreeViewAdv>();
+  return tree->Columns[column]->IsVisible;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Adds, removes or changes a node <-> tag mapping (if mapping is enabled).
+ */
+void TreeNodeViewImpl::process_mapping(Node^ node, const std::string &tag)
+{
+  if (_tag_map != nullptr)
+  {
+    String^ native_tag = CppStringToNativeRaw(tag);
+
+    if (node == nullptr)
+      _tag_map->Remove(native_tag);
+    else
+      _tag_map[native_tag] = node;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void TreeNodeViewImpl::set_selected(mforms::TreeNodeRef node, bool flag)
 {
   TreeNodeImpl *impl = dynamic_cast<TreeNodeImpl*>(node.ptr());
@@ -821,24 +923,100 @@ void TreeNodeViewImpl::allow_column_sorting(bool flag)
   _can_sort_column = flag;
   if (_current_sort_column > -1)
   {
-    TreeViewAdv^ tree= get_control<TreeViewAdv>();
+    TreeViewAdv^ tree = get_control<TreeViewAdv>();
     tree->Columns[_current_sort_column]->SortOrder = _current_sort_order;
   }
 }
 
 //--------------------------------------------------------------------------------------------------
 
+int TreeNodeViewImpl::current_sort_column()
+{
+  return _current_sort_column;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void TreeNodeViewImpl::freeze_refresh(bool flag)
 {
   TreeViewAdv^ tree= get_control<TreeViewAdv>();
   if (flag)
+    ++_freeze_count;
+  else
+    if (_freeze_count == 0)
+      log_error("TreeNodeView: attempt to thaw an unfrozen tree");
+    else
+      --_freeze_count;
+
+  if (_freeze_count == 1)
     ControlUtilities::SuspendDrawing(tree);
   else
+    if (_freeze_count == 0)
+    {
+      ControlUtilities::ResumeDrawing(tree);
+      if (is<SortableTreeModel^>(_model))
+        _model->Resort();
+      else
+        tree->Refresh();
+    }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool TreeNodeViewImpl::is_frozen()
+{
+  return _freeze_count > 0;
+}
+
+//----------------- TriangleNodeControl ------------------------------------------------------------
+
+TriangleNodeControl::TriangleNodeControl()
+{
+  _expanded_icon = gcnew Bitmap("images/ui/tree_expanded.png", true);
+  _collapsed_icon = gcnew Bitmap("images/ui/tree_collapsed.png", true);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+Size TriangleNodeControl::MeasureSize(TreeNodeAdv^ node, DrawContext context)
+{
+  return _expanded_icon->Size;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void TriangleNodeControl::Draw(TreeNodeAdv^ node, DrawContext context)
+{
+  if (node->CanExpand)
   {
-    ControlUtilities::ResumeDrawing(tree);
-    tree->Refresh();
+    System::Drawing::Rectangle r = context.Bounds;
+    Image^ img;
+    if (node->IsExpanded)
+      img = _expanded_icon;
+    else
+      img = _collapsed_icon;
+    int dy = (int) Math::Round((float) (r.Height - img->Height) / 2);
+    context.Graphics->DrawImageUnscaled(img, System::Drawing::Point(r.X, r.Y + dy));
   }
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void TriangleNodeControl::MouseDown(TreeNodeAdvMouseEventArgs^ args)
+{
+  if (args->Button == MouseButtons::Left)
+  {
+    args->Handled = true;
+    if (args->Node->CanExpand)
+      args->Node->IsExpanded = !args->Node->IsExpanded;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void TriangleNodeControl::MouseDoubleClick(TreeNodeAdvMouseEventArgs^ args)
+{
+  args->Handled = true; // Suppress expand/collapse when double clicking.
+}
+
+//--------------------------------------------------------------------------------------------------
\ No newline at end of file
