@@ -23,7 +23,6 @@
 #include "wf_view.h"
 #include "wf_code_editor.h"
 
-#include "workbench/wb_context.h" // Needed for AppImpl.
 #include "wf_box.h"
 #include "wf_appview.h"
 #include "wf_app.h"
@@ -53,6 +52,7 @@ ViewImpl::ViewImpl(mforms::View *view)
 {
   tooltip = nullptr;
   backgroundImage = nullptr;
+  layoutSuspended = false;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -79,10 +79,11 @@ bool ViewImpl::use_min_width_for_layout(Control^ control)
   if (!needsMinWidth)
   {
     System::Type^ type = control->GetType();
-    needsMinWidth= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
+    needsMinWidth = (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
       || (type == TreeViewAdv::typeid) || (type == ListBox::typeid) || (type == TextBox::typeid)
       || (type == SplitContainer::typeid) || (type == ScintillaControl::typeid)
-      || (type == Windows::Forms::WebBrowser::typeid)
+      || (type == Windows::Forms::WebBrowser::typeid) || (type == HtmlLabel::typeid) || (type == HtmlPanel::typeid)
+      || (type == Windows::Forms::ProgressBar::typeid)
       
       // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
       || (type == FlatTabControl::typeid);
@@ -230,15 +231,27 @@ void ViewImpl::show(mforms::View *self, bool show)
   if (view != nullptr)
   {
     Control^ control= view->get_control<Control>();
-    control->Visible= show;
 
-    // Unfortunately, we cannot optimize handling here. Control::Visible not only returns the
-    // visibility of itself but its entire view hierarchy. So we cannot distinct between the two
-    // cases if the control itself is hidden or one of its parents. Setting Control::Visible
-    // does the right job, though.
-    //if (control->Visible != show)
-    if (control->IsHandleCreated && self->get_parent() != NULL)
-      self->get_parent()->set_layout_dirty(true);
+    // Create the control's window handle if not yet done. This is necessary because otherwise
+    // when setting a control to hidden and its window handle is not created, it will be moved to the end
+    // of it's parent Controls list, messing up our layout.
+    if (!control->IsHandleCreated)
+      control->Handle;
+
+    // Use reflection to get the true visible state. The Visible property returns the value for the
+    // control itself as well as all its parents.
+    bool visible = (bool)Control::typeid
+      ->GetMethod("GetState", System::Reflection::BindingFlags::Instance | System::Reflection::BindingFlags::NonPublic)
+      ->Invoke(control, gcnew array<Object^> { 2 });
+    if (show != visible && self->get_parent() != NULL)
+    {
+      if (view->layoutSuspended)
+        self->get_parent()->set_layout_dirty(true);
+      else
+        control->PerformLayout();
+    }
+
+    control->Visible = show;
   };
 }
 
@@ -412,7 +425,10 @@ void ViewImpl::set_name(mforms::View *self, const std::string& text)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
+  {
     view->get_control<Control>()->Name = CppStringToNative(text);
+    view->get_control<Control>()->AccessibleName = CppStringToNative(text);
+  }
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -437,9 +453,10 @@ void ViewImpl::set_needs_repaint(mforms::View *self)
 
 void ViewImpl::suspend_layout(mforms::View *self, bool flag)
 {
-  ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
+  ViewImpl^ view = (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
   {
+    view->layoutSuspended = flag;
     if (flag)
       view->get_control<Control>()->SuspendLayout();
     else
@@ -548,7 +565,10 @@ void ViewImpl::initialize()
 
   // Can be null, e.g. for non-control objects like dialogs.
   if (control != nullptr)
+  {
     control->Resize += gcnew EventHandler(this, &ViewImpl::Resize);
+    control->Enter += gcnew EventHandler(this, &ViewImpl::Enter);
+  }
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -602,7 +622,7 @@ void ViewImpl::set_tooltip(const std::string& text)
 void ViewImpl::set_font(const std::string& fontDescription)
 {
   std::string font;
-  int size;
+  float size;
   bool bold;
   bool italic;
   base::parse_font_description(fontDescription, font, size, bold, italic);
@@ -713,7 +733,7 @@ void ViewImpl::adjust_auto_resize_from_docking(Control^ control)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::flush_events(::mforms::View *)
+void ViewImpl::flush_events(mforms::View *)
 {
   Application::DoEvents();
 }
@@ -734,6 +754,20 @@ void ViewImpl::Resize(System::Object ^sender, EventArgs ^e)
 
 //-------------------------------------------------------------------------------------------------
 
+void ViewImpl::Enter(System::Object ^sender, EventArgs ^e)
+{
+  Control^ control = (Control^)sender;
+
+  if (control->Tag != nullptr)
+  {
+    mforms::View* view = ViewImpl::get_backend_control<mforms::View>(control);
+    if (!view->is_destroying())
+      view->focus_changed();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
 /**
  * Draws the background image with the earlier set layout, if there's one.
  */
