@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -63,7 +63,7 @@ bool WizardImpl::create(mforms::Wizard* self, mforms::Form* owner)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_title(::mforms::Wizard* self, const std::string& title)
+void WizardImpl::set_title(mforms::Wizard* self, const std::string& title)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -72,7 +72,7 @@ void WizardImpl::set_title(::mforms::Wizard* self, const std::string& title)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::run_modal(::mforms::Wizard* self)
+void WizardImpl::run_modal(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -81,7 +81,7 @@ void WizardImpl::run_modal(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::close(::mforms::Wizard* self)
+void WizardImpl::close(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -90,7 +90,7 @@ void WizardImpl::close(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::flush_events(::mforms::Wizard* self)
+void WizardImpl::flush_events(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -99,7 +99,7 @@ void WizardImpl::flush_events(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_content(::mforms::Wizard* self, ::mforms::View* view)
+void WizardImpl::set_content(mforms::Wizard* self, mforms::View* view)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -108,7 +108,7 @@ void WizardImpl::set_content(::mforms::Wizard* self, ::mforms::View* view)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_heading(::mforms::Wizard* self, const std::string& heading)
+void WizardImpl::set_heading(mforms::Wizard* self, const std::string& heading)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -117,7 +117,7 @@ void WizardImpl::set_heading(::mforms::Wizard* self, const std::string& heading)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_step_list(::mforms::Wizard* self, const vector<string>& steps)
+void WizardImpl::set_step_list(mforms::Wizard* self, const vector<string>& steps)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -126,7 +126,7 @@ void WizardImpl::set_step_list(::mforms::Wizard* self, const vector<string>& ste
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_cancel(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_cancel(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -135,7 +135,7 @@ void WizardImpl::set_allow_cancel(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_back(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_back(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -144,7 +144,7 @@ void WizardImpl::set_allow_back(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_next(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_next(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -153,7 +153,7 @@ void WizardImpl::set_allow_next(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_show_extra(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_show_extra(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -162,7 +162,7 @@ void WizardImpl::set_show_extra(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_extra_caption(::mforms::Wizard* self, const std::string& caption)
+void WizardImpl::set_extra_caption(mforms::Wizard* self, const std::string& caption)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -171,7 +171,7 @@ void WizardImpl::set_extra_caption(::mforms::Wizard* self, const std::string& ca
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_next_caption(::mforms::Wizard* self, const std::string& caption)
+void WizardImpl::set_next_caption(mforms::Wizard* self, const std::string& caption)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -323,7 +323,7 @@ void WizardImpl::nextClick(Object^ sender, EventArgs^ arguments)
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->next_clicked();
   }
@@ -337,7 +337,7 @@ void WizardImpl::backClick(Object^ sender, EventArgs^ arguments)
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->back_clicked();
   }
@@ -351,7 +351,7 @@ void WizardImpl::cancelClick(Object^ sender, EventArgs^ arguments)
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL && wizard->_cancel_slot())
       ((Form^) btn->TopLevelControl)->Close();
   }
@@ -365,7 +365,7 @@ void WizardImpl::extraClick(Object^ sender, EventArgs^ arguments)
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(btn);
+    mforms::Wizard* wizard= ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->extra_clicked();
   }
@@ -376,7 +376,7 @@ void WizardImpl::extraClick(Object^ sender, EventArgs^ arguments)
 void WizardImpl::resize(Object^ sender, EventArgs^ arguments)
 {
   Form^ form = dynamic_cast<Form^>(sender);
-  ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(form);
+  mforms::Wizard* wizard= ViewImpl::get_backend_control<mforms::Wizard>(form);
   WizardImpl^ wizard_impl= (WizardImpl^)ObjectImpl::FromUnmanaged(wizard);
 
   System::Drawing::Rectangle area = form->DisplayRectangle;
@@ -421,7 +421,7 @@ void WizardImpl::flush_events()
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_content(::mforms::View *view)
+void WizardImpl::set_content(mforms::View *view)
 {
   // Remove old stuff if there is some.
   if (_content->Controls->Count > 0)
@@ -433,7 +433,7 @@ void WizardImpl::set_content(::mforms::View *view)
   if (view != NULL)
   {
     ViewImpl^ new_content_view= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
-    view->set_resize_mode(::mforms::ResizeNone);
+    view->set_resize_mode(mforms::ResizeNone);
     Control^ new_content= new_content_view->get_control<Control>();
 
     _content->Controls->Add(new_content);
