@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -398,7 +398,7 @@ namespace Forms {
     }
 
   protected:
-    ObjectImpl(::mforms::Object *object);
+    ObjectImpl(mforms::Object *object);
     virtual ~ObjectImpl();
 
     virtual void initialize() {};
@@ -422,7 +422,7 @@ namespace Forms {
       * For the same reason the wrapper gets a reference of the .NET control.
       */
     template<class T>
-    static T^ create(::mforms::Object* backend, ObjectImpl^ wrapper)
+    static T^ create(mforms::Object* backend, ObjectImpl^ wrapper)
     {
       assert(wrapper->dialog == nullptr);
       assert(wrapper->toolStripItem == nullptr);
@@ -442,7 +442,7 @@ namespace Forms {
     }
 
     template<class T>
-    static T^ create(::mforms::Object* backend, ObjectImpl^ wrapper, ToolStripItem^)
+    static T^ create(mforms::Object* backend, ObjectImpl^ wrapper, ToolStripItem^)
     {
       assert(wrapper->dialog == nullptr);
       assert(wrapper->control == nullptr);
@@ -460,7 +460,7 @@ namespace Forms {
       * Otherwise it is the same as the create<T> function.
       */
     template<class T>
-    static T^ create_dialog(::mforms::Object* backend, ObjectImpl^ wrapper)
+    static T^ create_dialog(mforms::Object* backend, ObjectImpl^ wrapper)
     {
       assert(wrapper->control == nullptr);
       assert(wrapper->toolStripItem == nullptr);
