@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,10 +21,6 @@
 #ifndef _WF_DRAWBOX_H_
 #define _WF_DRAWBOX_H_
 
-
-#include "mforms/drawbox.h"
-#include "wf_view.h"
-
 using namespace System;
 using namespace Windows::Forms;
 
@@ -40,7 +36,7 @@ namespace MySQL {
     public:
       CanvasControl();
 
-      virtual Drawing::Size GetPreferredSize(Drawing::Size proposedSize) override;
+      virtual System::Drawing::Size GetPreferredSize(System::Drawing::Size proposedSize) override;
       void set_backend(mforms::DrawBox* backend);
       void DoRepaint();
     };
@@ -48,11 +44,11 @@ namespace MySQL {
     public ref class DrawBoxImpl : public ViewImpl
     {
     protected:
-      DrawBoxImpl(::mforms::DrawBox *self);
+      DrawBoxImpl(mforms::DrawBox *self);
       ~DrawBoxImpl();
 
-      static bool create(::mforms::DrawBox *self);
-      static void set_needs_repaint(::mforms::DrawBox *self);
+      static bool create(mforms::DrawBox *self);
+      static void set_needs_repaint(mforms::DrawBox *self);
 
       void drawbox_mouse_move(System::Object ^sender, MouseEventArgs ^e);
       void drawbox_mouse_down(System::Object ^sender, MouseEventArgs ^e);
@@ -62,14 +58,15 @@ namespace MySQL {
       void drawbox_mouse_enter(System::Object ^sender, EventArgs ^e);
       void drawbox_mouse_leave(System::Object ^sender, EventArgs ^e);
       void drawbox_repaint(System::Object ^sender, PaintEventArgs ^e);
+      void drawbox_key_down(System::Object ^sender, KeyEventArgs ^e);
 
     public:
       static void init(Manager ^mgr)
       {
-        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
+        mforms::ControlFactory *f= mforms::ControlFactory::get_instance();
 
-        DEF_CALLBACK1(bool, ::mforms::DrawBox*, mgr, f->_drawbox_impl, DrawBoxImpl, create);
-        DEF_CALLBACK1(void, ::mforms::DrawBox*, mgr, f->_drawbox_impl, DrawBoxImpl, set_needs_repaint);
+        DEF_CALLBACK1(bool, mforms::DrawBox*, mgr, f->_drawbox_impl, DrawBoxImpl, create);
+        DEF_CALLBACK1(void, mforms::DrawBox*, mgr, f->_drawbox_impl, DrawBoxImpl, set_needs_repaint);
       }
     };
 
