@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,16 +17,9 @@
  * 02110-1301  USA
  */
 
-
-
 #ifndef _WF_PANEL_H_
 #define _WF_PANEL_H_
 
-
-#include "mforms/panel.h"
-
-#include "wf_view.h"
-
 using namespace System::Windows::Forms;
 using namespace Layout;
 
@@ -152,43 +145,43 @@ namespace MySQL {
     public ref class PanelImpl : public ViewImpl
     {
     private:
-      ::mforms::View *_child;
-      ::mforms::PanelType type;
+      mforms::View *_child;
+      mforms::PanelType type;
 
     protected:
-      PanelImpl(::mforms::View *self)
+      PanelImpl(mforms::View *self)
         : ViewImpl(self)
       {
       }
 
-      static bool create(::mforms::Panel *self, ::mforms::PanelType panelType);
-      static void set_title(::mforms::Panel *self, const std::string &title);
-      static void set_back_color(::mforms::Panel *self, const std::string &color);
-      static void add(::mforms::Panel *self, ::mforms::View *view);
-      static void set_active(::mforms::Panel* self, bool value);
-      static bool get_active(::mforms::Panel* self);
-      static void remove(::mforms::Panel *self, ::mforms::View* view);
+      static bool create(mforms::Panel *self, mforms::PanelType panelType);
+      static void set_title(mforms::Panel *self, const std::string &title);
+      static void set_back_color(mforms::Panel *self, const std::string &color);
+      static void add(mforms::Panel *self, mforms::View *view);
+      static void set_active(mforms::Panel* self, bool value);
+      static bool get_active(mforms::Panel* self);
+      static void remove(mforms::Panel *self, mforms::View* view);
 
     public:
       virtual void set_title(const std::string &title);
       virtual void set_back_color(const std::string &color);
-      virtual void add(::mforms::View *view);
+      virtual void add(mforms::View *view);
       virtual void set_active(bool value);
       virtual bool get_active();
-      virtual void remove(::mforms::View* view);
+      virtual void remove(mforms::View* view);
       virtual void remove();
 
       static void init(Manager ^mgr)
       {
-        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
+        mforms::ControlFactory *f= mforms::ControlFactory::get_instance();
 
-        DEF_CALLBACK2(bool, ::mforms::Panel*, ::mforms::PanelType, mgr, f->_panel_impl, PanelImpl, create);
-        DEF_CALLBACK2(void, ::mforms::Panel*, const std::string&, mgr, f->_panel_impl, PanelImpl, set_title);
-        DEF_CALLBACK2(void, ::mforms::Panel*, ::mforms::View*, mgr, f->_panel_impl, PanelImpl, add);
-        DEF_CALLBACK2(void, ::mforms::Panel*, const std::string&, mgr, f->_panel_impl, PanelImpl, set_back_color);
-        DEF_CALLBACK2(void, ::mforms::Panel*, bool, mgr, f->_panel_impl, PanelImpl, set_active);
-        DEF_CALLBACK1(bool, ::mforms::Panel*, mgr, f->_panel_impl, PanelImpl, get_active);
-        DEF_CALLBACK2(void, ::mforms::Panel*, ::mforms::View*, mgr, f->_panel_impl, PanelImpl, remove);
+        DEF_CALLBACK2(bool, mforms::Panel*, mforms::PanelType, mgr, f->_panel_impl, PanelImpl, create);
+        DEF_CALLBACK2(void, mforms::Panel*, const std::string&, mgr, f->_panel_impl, PanelImpl, set_title);
+        DEF_CALLBACK2(void, mforms::Panel*, mforms::View*, mgr, f->_panel_impl, PanelImpl, add);
+        DEF_CALLBACK2(void, mforms::Panel*, const std::string&, mgr, f->_panel_impl, PanelImpl, set_back_color);
+        DEF_CALLBACK2(void, mforms::Panel*, bool, mgr, f->_panel_impl, PanelImpl, set_active);
+        DEF_CALLBACK1(bool, mforms::Panel*, mgr, f->_panel_impl, PanelImpl, get_active);
+        DEF_CALLBACK2(void, mforms::Panel*, mforms::View*, mgr, f->_panel_impl, PanelImpl, remove);
       }
     };
 
