@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@
 
 using namespace System;
 using namespace Windows::Forms;
-using namespace System::Collections;
 using namespace System::Collections::ObjectModel;
 
 using namespace Aga::Controls::Tree;
@@ -29,10 +28,11 @@ using namespace Aga::Controls::Tree::NodeControls;
 
 namespace MySQL {
   namespace Forms {
+
     private ref class TreeViewNode : Node
     {
     private:
-      std::string* _mytag;
+      std::string *_mytag;
       mforms::TreeNodeData *_data;
       List<String^> _text;
       std::vector<mforms::TreeNodeTextAttributes>* _attributes;
@@ -79,13 +79,15 @@ namespace MySQL {
   };
 };
 
-
 private class TreeNodeImpl : public mforms::TreeNode
 {
   gcroot<TreeViewAdv^> _tree;
   gcroot<Node^> _node;
   int _refcount;
 
+private:
+  MySQL::Forms::TreeNodeViewImpl^ get_tree_wrapper();
+
 protected:
   bool is_root() const;
   Aga::Controls::Tree::TreeModel ^model();
@@ -121,7 +123,7 @@ public:
   virtual void remove_from_parent();
   virtual mforms::TreeNodeRef get_child(int index) const;
   virtual mforms::TreeNodeRef get_parent() const;
-  virtual void clear_children();
+  virtual void remove_children();
 
   Bitmap^ get_cached_icon(const std::string &icon_id);
   virtual std::vector<mforms::TreeNodeRef> add_node_collection(const mforms::TreeNodeCollectionSkeleton &nodes, int position = -1);
