@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 #include "python_context.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
+#include "grtpp_notifications.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
 #include "base/wb_memory.h"
@@ -119,7 +120,11 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   main= PyImport_AddModule("__main__");
   PyDict_SetItemString(PyModule_GetDict(main),
                        "grt", PyImport_ImportModule("grt"));
-  
+
+  PySys_SetObject((char*)"real_stdout", PySys_GetObject((char*)"stdout"));
+  PySys_SetObject((char*)"real_stderr", PySys_GetObject((char*)"stderr"));
+  PySys_SetObject((char*)"real_stdin", PySys_GetObject((char*)"stdin"));
+
   // make sys.stdout and sys.stderr send stuff to GRT
   PySys_SetObject((char*)"stdout", get_grt_module());
   PySys_SetObject((char*)"stderr", get_grt_module());
@@ -136,15 +141,26 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   }
 
   PyEval_SaveThread();
+  
+  // listen to all notifications so it can be forwarded to python
+  GRTNotificationCenter::get()->add_grt_observer(this);
+  NotificationCenter::get()->add_observer(this);
 }
 
 
 PythonContext::~PythonContext()
 {
+  GRTNotificationCenter::get()->remove_grt_observer(this);
+  NotificationCenter::get()->remove_observer(this);
+
   PyEval_RestoreThread(_main_thread_state);
   _main_thread_state = NULL;
 
+  // This causes deadlocks in Windows during shutdown, probably because some thread is not
+  // closed correctly.. remove the ifdefs once it's fixed
+#if !defined(_WIN32) || defined(_DEBUG)
   Py_Finalize();
+#endif
 }
 
 
@@ -193,6 +209,18 @@ void PythonContext::set_db_login_error(const grt::db_login_error &exc)
   PyErr_SetString(PythonContext::get()->_grt_db_login_error, exc.what());
 }
 
+void PythonContext::set_db_error(const grt::db_error &exc)
+{
+  PyObject *arg = Py_BuildValue("(si)", exc.what(), exc.error());
+  PyErr_SetObject(PythonContext::get()->_grt_db_error, arg);
+  Py_DECREF(arg);
+}
+
+void PythonContext::set_db_not_conected(const grt::db_not_connected &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_not_connected, exc.what());
+}
+
 void PythonContext::set_python_error(const grt::type_error &exc, const std::string &location)
 {
   PyErr_SetString(PyExc_TypeError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
@@ -253,6 +281,87 @@ PythonContext *PythonContext::get_and_check()
 }
 
 
+void PythonContext::set_grt_observer_callable(PyObject *obj)
+{
+  _grt_notification_observer = obj;
+}
+
+
+void PythonContext::handle_grt_notification(const std::string &name, ObjectRef sender, DictRef info)
+{
+  if (_grt_notification_observer)
+  {
+    WillEnterPython lock;
+    
+    PyObject *psender = from_grt(sender);
+    PyObject *pinfo = from_grt(info);
+    PyObject *args = Py_BuildValue("sOO", name.c_str(), psender, pinfo);
+    PyObject *res;
+  
+    if (!(res = PyObject_CallObject(_grt_notification_observer, args)))
+    {
+      log_python_error("Error forwarding GRT notification to Python");
+    }
+    if (res)
+    {
+      Py_DECREF(res);
+    }
+    Py_DECREF(psender);
+    Py_DECREF(pinfo);
+    Py_DECREF(args);
+  }
+}
+
+
+void PythonContext::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (_grt_notification_observer)
+  {
+    WillEnterPython lock;
+
+    PyObject *infodict = PyDict_New();
+    PyObject *res;
+    
+    for (base::NotificationInfo::iterator i = info.begin(); i != info.end(); ++i)
+    {
+      PyObject *str = PyString_FromString(i->second.c_str());
+      PyDict_SetItemString(infodict, i->first.c_str(), str);
+      Py_DECREF(str);
+    }
+    PyObject *args = Py_BuildValue("sOO", name.c_str(), Py_None, infodict);
+    
+    if (!(res = PyObject_CallObject(_grt_notification_observer, args)))
+    {
+      log_python_error("Error forwarding notification to Python");
+    }
+    if (res)
+    {
+      Py_DECREF(res);
+    }
+    Py_DECREF(args);
+    
+    // apply back changes made to the info dict from python code
+    {
+      PyObject *key, *value;
+      Py_ssize_t pos = 0;
+      
+      while (PyDict_Next(infodict, &pos, &key, &value)) 
+      {
+        std::string s;
+        grt::ValueRef v;
+        v = from_pyobject(value);
+        if (!pystring_to_string(key, s) || !v.is_valid())
+        {
+          log_error("Error in Python notification handler: info dictionary contains an invalid item");
+          continue;
+        }
+        info[s]= v.repr();
+      }
+    }
+  }  
+}
+
+
 static PyObject *grt_print(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
@@ -595,8 +704,13 @@ static PyObject *grt_readline(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, ""))
     return NULL;
   
-  std::string line = ctx->stdin_readline_slot();
-  return Py_BuildValue("s", line.c_str());
+  if (ctx->stdin_readline_slot)
+  {
+    std::string line = ctx->stdin_readline_slot();
+    return Py_BuildValue("s", line.c_str());
+  }
+  PyErr_SetString(PyExc_NotImplementedError, "input()/stdin reading not available");
+  return NULL;
 }
 
 void decrement_int(int *i)
@@ -827,6 +941,109 @@ static PyObject *grt_unserialize(PyObject *self, PyObject *args)
   return Py_None;
 }
 
+
+static PyObject *grt_set_notification_observer(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+
+  PyObject *object;
+  
+  if (!PyArg_ParseTuple(args, "O", &object))
+    return NULL;
+  
+  if (!PyCallable_Check(object))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification observer argument must be a callable");
+    return NULL;
+  }
+  
+  ctx->set_grt_observer_callable(object);
+  
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+
+static PyObject *grt_send_notification(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  char *name;
+  PyObject *object;
+  PyObject *info;
+  
+  if (!PyArg_ParseTuple(args, "sOO", &name, &object, &info))
+    return NULL;
+  
+  ValueRef infodict = ctx->from_pyobject(info);
+  if (infodict.is_valid() && !DictRef::can_wrap(infodict))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification info must be a dict or None");
+    return NULL;
+  }
+  ValueRef sender = ctx->from_pyobject(object);
+  if (sender.is_valid() && !ObjectRef::can_wrap(sender))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification sender info must be a GRT object");
+    return NULL;
+  }
+
+  try
+  {
+    GRTNotificationCenter::get()->send_grt(name, ObjectRef::cast_from(sender), DictRef::cast_from(infodict));
+  }
+  catch (std::exception &exc)
+  {
+    ctx->set_python_error(exc, "Error sending notification");
+    return NULL;
+  }
+  
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+
+static PyObject *(*wrap_pyobject_func)(PyObject *, PyObject*) = NULL;
+static PyObject *(*unwrap_pyobject_func)(PyObject *, PyObject*) = NULL;
+
+static PyObject *grt_wrap_pyobject(PyObject *self, PyObject *args)
+{
+  // since the GRT objects are defined in an outer level, we forward this
+  // to a callback which must be registered beforehand with PythonContext::set_wrap_callback
+   if (wrap_pyobject_func)
+    return wrap_pyobject_func(self, args);
+    
+  PyErr_SetString(PyExc_SystemError, "GRT python support not initialized correctly");
+  return NULL;
+}
+
+
+static PyObject *grt_unwrap_pyobject(PyObject *self, PyObject *args)
+{
+  if (unwrap_pyobject_func)
+    return unwrap_pyobject_func(self, args);
+    
+  PyErr_SetString(PyExc_SystemError, "GRT python support not initialized correctly");
+  return NULL;
+}
+
+
+void PythonContext::set_wrap_pyobject_func(PyObject *(*func)(PyObject *, PyObject*))
+{
+  wrap_pyobject_func = func;
+}
+
+
+void PythonContext::set_unwrap_pyobject_func(PyObject *(*func)(PyObject *, PyObject*))
+{
+  unwrap_pyobject_func = func;
+}
+
+
 /** Register grt related functionality as a module
 
  Stuff made available in the module include:
@@ -891,7 +1108,19 @@ static PyMethodDef GrtModuleMethods[] = {
 {"unserialize", grt_unserialize, METH_VARARGS,
   "Unserializes a GRT object from a XML file created by serialize. unserialize(path) -> object"},
 
-  
+{"_set_grt_notification_observer", grt_set_notification_observer, METH_VARARGS,
+  "Sets the GRT notification observer. Internal use."},
+
+{"send_grt_notification", grt_send_notification, METH_VARARGS,
+  "Sends a GRT notification to the application. ex: send_grt_notification('GNMyPluginStarted', sender, infodict)"},
+
+{"fromgrt", grt_unwrap_pyobject, METH_VARARGS,
+  "Extracts the Python object wrapped in a grt_PyObject GRT object (suitable for use with GRT object values)"},
+
+{"togrt", grt_wrap_pyobject, METH_VARARGS,
+  "Wraps a Python object in a grt_PyObject GRT object, which you can then use to reference from GRT objects (such as GRT dicts and lists)"},
+
+
 {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
@@ -925,6 +1154,10 @@ void PythonContext::register_grt_module()
     _grt_user_interrupt_error = PyErr_NewException((char*)"grt.UserInterrupt", NULL, NULL);
     PyModule_AddObject(_grt_module, "UserInterrupt", _grt_user_interrupt_error);
   }
+  {
+    _grt_db_error = PyErr_NewException((char*)"grt.DBError", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBError", _grt_db_error);
+  }
   {
     _grt_db_access_denied_error = PyErr_NewException((char*)"grt.DBAccessDenied", NULL, NULL);
     PyModule_AddObject(_grt_module, "DBAccessDenied", _grt_db_access_denied_error);
@@ -933,7 +1166,11 @@ void PythonContext::register_grt_module()
     _grt_db_login_error = PyErr_NewException((char*)"grt.DBLoginError", NULL, NULL);
     PyModule_AddObject(_grt_module, "DBLoginError", _grt_db_login_error);
   }
-  
+  {
+    _grt_db_not_connected = PyErr_NewException((char*)"grt.DBNotConnected", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBNotConnected", _grt_db_not_connected);
+  }
+
   _grt_modules_module = Py_InitModule("grt.modules", NULL);
   if (!_grt_modules_module)
     throw std::runtime_error("Error initializing grt.modules module in Python support");
@@ -1466,7 +1703,9 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
       Py_DECREF(excep);
       Py_DECREF(value);
       if (trace)
+      {
         Py_DECREF(trace);
+      }
       PyErr_Clear();
       return 0; // continued line
     }
