@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,10 +20,14 @@
 #include "stdafx.h"
 
 #include "base/log.h"
+#include "base/string_utilities.h"
+
 #include "mysql_sql_editor_be.h"
 
 #include "mforms/code_editor.h"
 
+#include "grts/structs.db.mysql.h"
+
 using namespace bec;
 using namespace grt;
 
@@ -59,8 +63,11 @@ Mysql_sql_editor::~Mysql_sql_editor()
 
 //--------------------------------------------------------------------------------------------------
 
+#define IS_PART_INCLUDED(part) \
+  ((parts & part) == part)
+
 bool Mysql_sql_editor::fill_auto_completion_keywords(std::vector<std::pair<int, std::string> > &entries,
-  bool major_keywords, bool keywords, bool functions)
+  AutoCompletionWantedParts parts)
 {
   log_debug2("Filling keywords auto completion list for MySQL.\n");
 
@@ -68,35 +75,79 @@ bool Mysql_sql_editor::fill_auto_completion_keywords(std::vector<std::pair<int,
   {
     log_debug2("Adding keywords + function names\n");
 
-    std::map<std::string, const char*> keyword_map = _editor_config->get_keywords();
+    std::map<std::string, std::string> keyword_map = _editor_config->get_keywords();
 
     // MySQL keywords are split into two sets. Major keywords are those that can start a statement.
     // All other keywords appear within a statement.
-    if (major_keywords)
+    if (IS_PART_INCLUDED(Sql_editor::CompletionWantMajorKeywords))
     {
       std::vector<std::string> words = base::split_by_set(keyword_map["Major Keywords"], " \t\n");
       for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
-        entries.push_back(std::pair<int, std::string>(AC_KEYWORD_IMAGE, *iterator));
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, *iterator));
+    }
+    else
+    {
+      if (IS_PART_INCLUDED(Sql_editor::CompletionWantSelect))
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, "select"));
+      if (IS_PART_INCLUDED(Sql_editor::CompletionWantBy))
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, "by"));
     }
 
-    if (keywords)
+    if (IS_PART_INCLUDED(Sql_editor::CompletionWantKeywords))
     {
-      // TODO: optionally keywords can be made uppercase (create a preference and convert here).
       std::vector<std::string> words = base::split_by_set(keyword_map["Keywords"], " \t\n");
       for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
-        entries.push_back(std::pair<int, std::string>(AC_KEYWORD_IMAGE, *iterator));
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, *iterator));
       words = base::split_by_set(keyword_map["Procedure keywords"], " \t\n");
       for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
-        entries.push_back(std::pair<int, std::string>(AC_KEYWORD_IMAGE, *iterator));
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, *iterator));
       words = base::split_by_set(keyword_map["User Keywords 1"], " \t\n");
       for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
-        entries.push_back(std::pair<int, std::string>(AC_KEYWORD_IMAGE, *iterator));
+        entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, *iterator));
     }
-    if (functions)
+    else
+    {
+      // Expression keywords are a subset of the non-major keywords, so we only need to add them
+      // if non-major keywords are not wanted.
+      if (IS_PART_INCLUDED(Sql_editor::CompletionWantExprStartKeywords) ||
+        IS_PART_INCLUDED(Sql_editor::CompletionWantExprInnerKeywords))
+      {
+        std::vector<std::string> words = base::split_by_set(keyword_map["User Keywords 2"], " \t\n");
+        for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
+          entries.push_back(std::pair<int, std::string>(AC_KEYWORD_IMAGE, *iterator));
+
+        if (IS_PART_INCLUDED(Sql_editor::CompletionWantExprInnerKeywords))
+        {
+          std::vector<std::string> words = base::split_by_set(keyword_map["User Keywords 3"], " \t\n");
+          for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
+            entries.push_back(std::make_pair(AC_KEYWORD_IMAGE, *iterator));
+        }
+      }
+    }
+
+    if (IS_PART_INCLUDED(Sql_editor::CompletionWantRuntimeFunctions))
     {
       std::vector<std::string> words = base::split_by_set(keyword_map["Functions"], " \t\n");
       for (std::vector<std::string>::const_iterator iterator = words.begin(); iterator != words.end(); ++iterator)
-        entries.push_back(std::pair<int, std::string>(AC_FUNCTION_IMAGE, *iterator + "()"));
+        entries.push_back(std::make_pair(AC_FUNCTION_IMAGE, *iterator + "()"));
+    }
+
+    if (IS_PART_INCLUDED(Sql_editor::CompletionWantEngines))
+    {
+      grt::GRT *grt = grtm()->get_grt();
+      grt::Module *module = grt->get_module("DbMySQL");
+      if (module != NULL)
+      {
+        grt::BaseListRef args(grt);
+        grt::ListRef<db_mysql_StorageEngine> engines =
+          grt::ListRef<db_mysql_StorageEngine>::cast_from(module->call_function("getKnownEngines", args));
+
+        if (engines.is_valid())
+        {
+          for (size_t c = engines.count(), i= 0; i < c; i++)
+            entries.push_back(std::make_pair(AC_ENGINE_IMAGE, engines[i]->name()));
+          }
+      }
     }
   }
 
