@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -313,7 +313,8 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(table->checksum() != 0)
       callback->create_table_checksum(table->checksum());
 
-    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str()))
+    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str())&&
+        (get_collation_cs(table->defaultCollationName()) == (table->defaultCharacterSetName().c_str())) )
       callback->create_table_collate(table->defaultCollationName());
 
     if(strlen(table->comment().c_str()))
@@ -350,6 +351,9 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(strlen(table->rowFormat().c_str()))
       callback->create_table_row_format(table->rowFormat());
 
+    if(strlen(table->keyBlockSize().c_str()))
+      callback->create_table_key_block_size(table->keyBlockSize());
+
     if(strlen(table->mergeUnion().c_str()))
       callback->create_table_merge_union(table->mergeUnion());
 
@@ -803,6 +807,9 @@ void DiffSQLGeneratorBE::generate_alter(
     case grt::ListItemOrderChanged: // DROP/ADD FK
       {
         const grt::ListItemOrderChange *order_change= static_cast<const grt::ListItemOrderChange *>(fk_change.get());
+        //Only position changed nothing to do
+        if (!order_change->get_subchange())
+          break;
         fk1= db_mysql_ForeignKeyRef::cast_from(order_change->get_old_value());
         fk2= db_mysql_ForeignKeyRef::cast_from(order_change->get_new_value());
       }
@@ -839,6 +846,9 @@ void DiffSQLGeneratorBE::generate_alter(
     case grt::ListItemOrderChanged: // DROP/ADD FK
       {
         const grt::ListItemOrderChange *order_change= static_cast<const grt::ListItemOrderChange *>(fk_change.get());
+        //Only position changed nothing to do
+        if (!order_change->get_subchange())
+          break;
         callback->alter_table_drop_fk(db_mysql_ForeignKeyRef::cast_from(order_change->get_old_value()));
         callback->alter_table_add_fk(db_mysql_ForeignKeyRef::cast_from(order_change->get_new_value()));
       }
@@ -902,13 +912,6 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
   if (table->isStub())
     return;
 
-  bool process_table= true;
-
-  std::string table_name_for_filter(get_old_object_name_for_key(table, _case_sensitive));
-  if(_use_filtered_lists)
-    if(_filtered_tables.find(table_name_for_filter) == _filtered_tables.end())
-      process_table= false;
-
   // process table triggers
   const grt::ChangeSet *cs= diffchange->subchanges();
 
@@ -964,8 +967,9 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     }
   }
 
-  if(!process_table)
-    return;
+  const std::string table_name_for_filter(get_old_object_name_for_key(table, _case_sensitive));
+  if(_use_filtered_lists && (_filtered_tables.find(table_name_for_filter) == _filtered_tables.end()))
+      return;
 
   bool partitions_processed= false;
   cs= diffchange->subchanges();
@@ -1012,7 +1016,8 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       const grt::SimpleValueChange *change= 
         static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
-      callback->alter_table_engine(table, str);
+      if(!str.empty())
+        callback->alter_table_engine(table, str);
     }
     else if(attr_change->get_attr_name().compare("nextAutoInc") == 0)
     {
@@ -1105,6 +1110,12 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_row_format(table, str);
     }
+    else if(attr_change->get_attr_name().compare("keyBlockSize") == 0)
+    {
+      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
+      grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
+      callback->alter_table_key_block_size(table, str);
+    }
     else if(attr_change->get_attr_name().compare("avgRowLength") == 0)
     {
       const grt::SimpleValueChange *change= 
@@ -1139,25 +1150,15 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_connection_string(table, str);
     }
-    else if((attr_change->get_attr_name().compare("partitionType") == 0) 
+    else if(!partitions_processed &&
+       ( (attr_change->get_attr_name().compare("partitionType") == 0) 
        || (attr_change->get_attr_name().compare("partitionExpression") == 0)
        || (attr_change->get_attr_name().compare("subpartitionType") == 0)
        || (attr_change->get_attr_name().compare("subpartitionExpression") == 0)
-       || (attr_change->get_attr_name().compare("subpartitionCount") == 0))
+       || (attr_change->get_attr_name().compare("subpartitionCount") == 0)))
     {
-      if(partitions_processed)
-        continue;
-
-      // TODO: is this get call still needed (no side effects) or can we remove it?
-/*unused      const grt::SimpleValueChange *change= */
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       generate_set_partitioning(table, diffchange);
-
-      partitions_processed= true;
-    }
-    else
-    {
-      continue; // avoid setting comma, see below
+      partitions_processed = true;
     }
   }
 
@@ -1182,14 +1183,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       //// generate_change_partition_count() will return empty string otherwise
       //// for RANGE/LIST we ignore change of this attribute and rely solely on
       //// partition definitions change
-      //if(!part_count_sql.empty())
-      //{
-      //  alter_sql.append(comma).append(part_count_sql);
-      //  partitions_processed= true;
-      //}
-
       std::string part_type(table->partitionType().c_str());
-
       if((part_type.find("HASH") != std::string::npos)
           || (part_type.find("KEY") != std::string::npos))
       {
@@ -1325,7 +1319,8 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
           generate_alter_stmt_drops(
             db_mysql_TableRef::cast_from(static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()),
             static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get());
-        }else if (table_change->get_change_type() == grt::ListItemOrderChanged)
+        }
+        else if (table_change->get_change_type() == grt::ListItemOrderChanged)
         {
             const grt::ListItemOrderChange *oc= 
               static_cast<const grt::ListItemOrderChange *>(table_change);
@@ -1405,7 +1400,19 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
             generate_alter_stmt(old_view, new_view, view_change);
           }
           break;
-        // list item position change is not relevant        
+          case grt::ListItemOrderChanged:
+            // list item position change is not relevant but may hold diff inside
+          {
+            if (!static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange())
+              break;
+            db_mysql_ViewRef old_view= db_mysql_ViewRef::cast_from(
+                             grt::ValueRef(static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange()->get_old_value()));
+            db_mysql_ViewRef new_view= db_mysql_ViewRef::cast_from(
+                             grt::ValueRef(static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange()->get_new_value()));
+
+            generate_alter_stmt(old_view, new_view, view_change);
+            break;
+          }
           default: break;
         }
       }
@@ -1524,102 +1531,28 @@ static void fill_set_from_list(grt::StringListRef string_list, std::set<std::str
     string_set.insert(std::string(string_list.get(i).c_str()));
 }
 
-DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEActionInterface *cb)
-  : callback(cb), _gen_create_index(false), _use_filtered_lists(true), 
-   _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false)
+DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, grt::DictRef dbtraits, DiffSQLGeneratorBEActionInterface *cb)
+    : callback(cb), _gen_create_index(false), _use_filtered_lists(true),
+    _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false)
 {
-  if (options.is_valid())
-  {
-    if (options.has_key("UseOIDAsResultDictKey"))
-      _use_oid_as_dict_key = options.get_int("UseOIDAsResultDictKey", 0) != 0;
+    if (!options.is_valid())
+        return;
+    _case_sensitive = (dbtraits.get_int("CaseSensitive", _case_sensitive) != 0);
 
-    if(options.has_key("SkipForeignKeys"))
-    {
-      grt::IntegerRef SkipForeignKeys= grt::IntegerRef::cast_from(options.get("SkipForeignKeys"));
-      if(SkipForeignKeys.is_valid())
-        _skip_foreign_keys = (SkipForeignKeys.operator long() != 0);
-    }if(options.has_key("SkipFKIndexes"))
-    {
-      grt::IntegerRef skip_fk_indexes= grt::IntegerRef::cast_from(options.get("SkipFKIndexes"));
-      if(skip_fk_indexes.is_valid())
-        _skip_fk_indexes = (skip_fk_indexes.operator long() != 0);
-    }
-    if(options.has_key("UseFilteredLists"))
-    {
-      grt::IntegerRef use_filtered_lists= grt::IntegerRef::cast_from(options.get("UseFilteredLists"));
-      if(use_filtered_lists.is_valid())
-        _use_filtered_lists= (use_filtered_lists.operator long() != 0);
-    }
-    cb->set_short_names(false);
-    if(options.has_key("UseShortNames"))
-    {
-      grt::IntegerRef use_short_names = grt::IntegerRef::cast_from(options.get("UseShortNames"));
-      if(use_short_names.is_valid())
-        cb->set_short_names(use_short_names.operator long() != 0);
-    }
-    if(options.has_key("GenerateUse"))
-    {
-      grt::IntegerRef gen_use = grt::IntegerRef::cast_from(options.get("GenerateUse"));
-      if(gen_use.is_valid())
-        cb->set_gen_use(gen_use.operator long() != 0);
-    }
-    if(options.has_key("GenerateCreateIndex"))
-    {
-      grt::IntegerRef gen_index_value= grt::IntegerRef::cast_from(options.get("GenerateCreateIndex"));
-      if(gen_index_value.is_valid())
-        _gen_create_index= (gen_index_value.operator long() != 0);
-    }
-    if(options.has_key("UserFilterList"))
-    {
-      grt::StringListRef user_list= grt::StringListRef::cast_from(options.get("UserFilterList"));
-      if(user_list.is_valid())
-        fill_set_from_list(user_list, _filtered_users);
-    }
-
-    if(options.has_key("SchemaFilterList"))
-    {
-      grt::StringListRef schemata_list= grt::StringListRef::cast_from(options.get("SchemaFilterList"));
-      if(schemata_list.is_valid())
-        fill_set_from_list(schemata_list, _filtered_schemata);
-    }
-
-    if(options.has_key("TableFilterList"))
-    {
-      grt::StringListRef tables_list= grt::StringListRef::cast_from(options.get("TableFilterList"));
-      if(tables_list.is_valid())
-        fill_set_from_list(tables_list, _filtered_tables);
-    }
-
-    if(options.has_key("ViewFilterList"))
-    {
-      grt::StringListRef views_list= grt::StringListRef::cast_from(options.get("ViewFilterList"));
-      if(views_list.is_valid())
-        fill_set_from_list(views_list, _filtered_views);
-    }
-
-    if(options.has_key("RoutineFilterList"))
-    {
-      grt::StringListRef routines_list= grt::StringListRef::cast_from(options.get("RoutineFilterList"));
-      if(routines_list.is_valid())
-        fill_set_from_list(routines_list, _filtered_routines);
-    }
-
-    if(options.has_key("TriggerFilterList"))
-    {
-      grt::StringListRef triggers_list= grt::StringListRef::cast_from(options.get("TriggerFilterList"));
-      if(triggers_list.is_valid())
-        fill_set_from_list(triggers_list, _filtered_triggers);
-    }
-    
-    grt::DictRef db_settings = grt::DictRef::cast_from(options.get("DBSettings"));
-    if(db_settings.is_valid() && db_settings.has_key("CaseSensitive"))
-    {
-      grt::IntegerRef case_sensitive = grt::IntegerRef::cast_from(db_settings.get("CaseSensitive"));
-      if(case_sensitive.is_valid())
-        _case_sensitive = (case_sensitive.operator long() != 0);
-    }
-
-  }
+    grt::StringListRef empty_list(options.get_grt());
+    _use_oid_as_dict_key = options.get_int("UseOIDAsResultDictKey", _use_oid_as_dict_key) != 0;
+    _skip_foreign_keys = options.get_int("SkipForeignKeys", _skip_foreign_keys) != 0;
+    _skip_fk_indexes = options.get_int("SkipFKIndexes", _skip_fk_indexes) != 0;
+    _gen_create_index = (options.get_int("GenerateCreateIndex", _gen_create_index) != 0);
+    _use_filtered_lists = options.get_int("UseFilteredLists", _use_filtered_lists)!= 0;
+    cb->set_short_names(options.get_int("UseShortNames", 0) != 0);
+    cb->set_gen_use(options.get_int("GenerateUse", 0) != 0);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("UserFilterList", empty_list)), _filtered_users);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("SchemaFilterList", empty_list)), _filtered_schemata);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("TableFilterList", empty_list)), _filtered_tables);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("ViewFilterList", empty_list)), _filtered_views);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("RoutineFilterList", empty_list)), _filtered_routines);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("TriggerFilterList", empty_list)), _filtered_triggers);
 }
 
 void DiffSQLGeneratorBE::process_diff_change(grt::ValueRef org_object, grt::DiffChange *diff, grt::DictRef map)
