@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -90,29 +90,49 @@ def get_linux_terminal_program():
     return None
 
 
+@ModuleInfo.plugin("wb.tools.copyConnectionString", caption="Copy Connection String to Clipboard", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
+@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
+def copyConnectionString(conn):
+    #<user>[:<password>]@<host>[:<port>][:<socket>]
+    connstr = ""
+    if conn.driver.name == "MysqlNative":
+        connstr = "%s:@%s:%s:" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
+    elif conn.driver.name == "MysqlNativeSocket":
+        connstr = "%s:@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["socket"])
+    elif conn.driver.name == "MysqlNativeSSH":
+        connstr = "%s:@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
+    mforms.Utilities.set_clipboard_text(connstr)
+
+
 @ModuleInfo.plugin("wb.tools.cmdlineClient", caption="Start Command Line Client", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
 def startCommandLineClientForConnection(conn):
     import platform
     import os
     if "ssh" in conn.driver.name.lower():
-        raise Exception("Tunneled connections not supported")
-
-    if "socket" in conn.driver.name.lower():
+        host = "127.0.0.1"
+        tun = grt.modules.DbMySQLQuery.openTunnel(conn)
+        if tun < 0:
+            mforms.Utilities.show_error("Start Command Line Client", "Could not open SSH tunnel to host.", "OK", "", "")
+            return
+        port = grt.modules.DbMySQLQuery.getTunnelPort(tun)
+        socket = ""
+    elif "socket" in conn.driver.name.lower():
         if platform.system() == "Windows":
             host = "."
         else:
             host = "localhost"
+        port = None
         socketName = conn.parameterValues["socket"]
         if socketName is None:
             socketName = "MySQL"
         socket = "--socket=" + socketName
     else:
         host = conn.parameterValues["hostName"].replace("\\", "\\\\").replace('"', '\\"')
+        port = conn.parameterValues["port"]
         socket = ""
 
     user = conn.parameterValues["userName"].replace("\\", "\\\\").replace('"', '\\"')
-    port = conn.parameterValues["port"]
     if port is None:
         port = 3306
     schema = conn.parameterValues["schema"]
@@ -121,11 +141,16 @@ def startCommandLineClientForConnection(conn):
     else:
         schema = ""
 
-    if platform.system() == "Darwin":
-        command = """mysql \\"-u%s\\" \\"-h%s\\" -P%i %s -p %s""" % (user, host, port, socket, schema)
+    bundled_client_path = grt.root.wb.options.options.get("mysqlclient", None)
+    if platform.system().lower() == "darwin":
+        if not bundled_client_path:
+            bundled_client_path = mforms.App.get().get_executable_path("mysql")
+        command = """%s \\"-u%s\\" \\"-h%s\\" -P%i %s -p %s""" % (bundled_client_path.replace(" ", "\\ "), user, host, port, socket, schema)
         os.system("""osascript -e 'tell application "Terminal" to do script "%s"'""" % command)
-    elif platform.system() == "Windows":
-        command = """start cmd /C mysql -u%s -h%s -P%i %s -p %s""" % (user, host, port, socket, schema)
+    elif platform.system().lower() == "windows":
+        if not bundled_client_path:
+            bundled_client_path = mforms.App.get().get_executable_path("mysql.exe")
+        command = """start cmd /C %s -u%s -h%s -P%i %s -p %s""" % (bundled_client_path.replace(" ", "\\ "), user, host, port, socket, schema)
         subprocess.Popen(command, shell = True)
     else:
         command = """mysql \\"-u%s\\" \\"-h%s\\" -P%i %s -p %s""" % (user, host, port, socket, schema)
@@ -133,7 +158,7 @@ def startCommandLineClientForConnection(conn):
 
 
 if sys.platform == "linux2":
-    odbcadmin = os.getenv("MWB_MODULE_DIR")+"/../iodbcadm-gtk"
+    odbcadmin = os.getenv("MWB_BINARIES_DIR")+"/iodbcadm-gtk"
     @ModuleInfo.export(grt.INT)
     def startODBCAdmin():
         if os.path.exists(odbcadmin):
@@ -172,39 +197,49 @@ if not disable_utilities:
         import time
 
         if platform.system() == "Windows":
-            command = 'start cmd /K \"cd utilities && echo The following utilities are available: && echo. && dir *.exe /B /W\"'
-            subprocess.Popen(command, shell = True)
+            path = os.path.join(os.getenv("ProgramFiles(x86)"), "MySQL", "MySQL Utilities", "mysqluc.exe")
+            if any(os.path.exists(os.path.join(f, "mysqluc.exe")) for f in os.getenv("PATH").split(";")):
+                # Utils path is in PATH already
+                command = r'start cmd /K "mysqluc"'
+                subprocess.Popen(command, shell = True)
+            elif os.path.exists(path):
+                command = r'start cmd /K "%s"' % path
+                subprocess.Popen(command, shell = True)
+            else:
+                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be located.\n\nTo use them, you must download and install the utilities package for your system from http://dev.mysql.com/downloads/tools/utilities/",
+                                           "OK", "Download...", "") != mforms.ResultOk:
+                    mforms.Utilities.open_url("http://dev.mysql.com/downloads/tools/utilities/")
         elif platform.system() == "Darwin":
-            import tempfile
-            fd, setup_script = tempfile.mkstemp(prefix="delme.", dir=mforms.App.get().get_user_data_folder())
-            f = os.fdopen(fd, "w+")
-            f.write('PATH="$PATH:%s/scripts"\n' % mforms.App.get().get_resource_path(""))
-            f.write('MYSQL_UTILITIES_FOLDER="%s/scripts"\n' % mforms.App.get().get_resource_path(""))
-            f.write('export PYTHONPATH="%s"\n' % os.getenv("PYTHONPATH", ""))
-            f.write('clear\n')
-            f.write('echo "The following MySQL Utilities are available:"\n')
-            f.write('ls "%s/scripts"\n' %  mforms.App.get().get_resource_path(""))
-            f.write('rm -f "%s"\n' % setup_script)
-            f.close()
-            os.chmod(setup_script, 0700)
-            os.system("""osascript -e 'tell application "Terminal" to do script "source \\"%s\\""' -e 'tell front window of application "Terminal" to set custom title to "MySQL Utilities"'""" % setup_script)
+            if any(os.path.exists(f+"/mysqluc") for f in os.getenv("PATH").split(":")):
+                os.system(r"""osascript -e 'tell application "Terminal" to do script "mysqluc -e \"help utilities\""' -e 'tell front window of application "Terminal" to set custom title to "MySQL Utilities"'""")
+            else:
+                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be located.\n\nTo use them, you must download and install the utilities package for your system from http://dev.mysql.com/downloads/tools/utilities/",
+                                                 "OK", "Download...", "") != mforms.ResultOk:
+                    mforms.Utilities.open_url("http://dev.mysql.com/downloads/tools/utilities/")
         else:
-            term = get_linux_terminal_program()
-            if term:
-                import tempfile
-                fd, setup_script = tempfile.mkstemp(prefix="delme.", dir=mforms.App.get().get_user_data_folder())
-                f = os.fdopen(fd, "w+")
-                f.write('echo "The following MySQL Utilities are available:"\n')
-                f.write('echo $MYSQL_UTILITIES_COMMANDS\n')
-                f.write('rm -f "%s"\n' % setup_script)
-                f.write('bash -i\n')
-                f.close()
-                os.chmod(setup_script, 0700)
+            if not any(os.path.exists(f+"/mysqluc") for f in os.getenv("PATH").split(":")):
+                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be located.\n\nTo use them, you must download and install the utilities package for your system from http://dev.mysql.com/downloads/tools/utilities/",
+                                               "OK", "Download...", "") != mforms.ResultOk:
+                    mforms.Utilities.open_url("http://dev.mysql.com/downloads/tools/utilities/")
+            else:
+                term = get_linux_terminal_program()
+                if term:
+                    import tempfile
+                    fd, setup_script = tempfile.mkstemp(prefix="delme.", dir=mforms.App.get().get_user_data_folder())
+                    f = os.fdopen(fd, "w+")
+                    f.write('echo "The following MySQL Utilities are available:"\n')
+                    f.write('echo $MYSQL_UTILITIES_COMMANDS\n')
+                    f.write('rm -f "%s"\n' % setup_script)
+                    f.write('bash -i\n')
+                    f.close()
+                    os.chmod(setup_script, 0700)
 
-                if 'konsole' in term:
-                    subprocess.call([term, "-e", "/bin/bash", setup_script])
+                    if 'konsole' in term:
+                        subprocess.call([term, "-e", "/bin/bash", setup_script])
+                    else:
+                        subprocess.call(["/bin/bash", "-c", "%s -e %s &" % (term, setup_script)])
                 else:
-                    subprocess.call(["/bin/bash", "-c", "%s -e %s &" % (term, setup_script)])
+                    raise RuntimeError("Terminal program could not be found")
 
 
 class CheckForUpdateThread(threading.Thread):
