@@ -1,5 +1,5 @@
-/* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,7 +48,7 @@ bool MySQLTableColumnsListBE::set_field(const NodeId &node, int column, const st
       return false;
 
     db_SimpleDatatypeRef columnType;
-    switch ((MySQLColumnListColumns)column)
+    switch ((ColumnListColumns)column)
     {
     case Default:
       // If a default value is set then auto increment for a column doesn't make sense.
@@ -229,30 +229,44 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
     if (col.is_valid() && col->simpleType().is_valid())
     {
       std::string type = col->simpleType()->name();
+      bool improved_timestamp_support = false;
+      GrtVersionRef target_version = _owner->get_rdbms_target_version();
+      // in MySQL 5.6, CURRENT_TIMESTAMP works for TIMESTAMP and DATETIME
+      // and for any number of colums
+      if (target_version.is_valid() &&
+         (*target_version->majorNumber() > 5 ||
+            (*target_version->majorNumber() == 5 &&
+             *target_version->minorNumber() >= 6)))
+          improved_timestamp_support = true;
       
-      if (type == "TIMESTAMP")
+      if (type == "TIMESTAMP" || (improved_timestamp_support && type == "DATETIME"))
       {
         bool seen_current_ts = false;
-        bool is_first_ts = false; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP, unless all 
+        bool current_timestamp_allowed = false; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP, unless all
         // previous ones are set to 0 or a constant
         bool flag = false;
         
-        GRTLIST_FOREACH(db_Column, columns, c)
+        if (improved_timestamp_support)
+          current_timestamp_allowed = true;
+        else
         {
-          if ((*c)->simpleType().is_valid() && (*c)->simpleType()->name() == "TIMESTAMP")
+          GRTLIST_FOREACH(db_Column, columns, c)
           {
-            if (*c == col)
+            if ((*c)->simpleType().is_valid() && (*c)->simpleType()->name() == "TIMESTAMP")
             {
-              is_first_ts= !seen_current_ts;
-              flag = true;
-            }
-            
-            if (!((*c)->defaultValue() == "0" || can_be_timestamp((*c)->defaultValue().c_str())))
-              seen_current_ts = true;
+              if (*c == col)
+              {
+                current_timestamp_allowed= !seen_current_ts;
+                flag = true;
+              }
+              
+              if (!((*c)->defaultValue() == "0" || can_be_timestamp((*c)->defaultValue().c_str())))
+                seen_current_ts = true;
 
-            // a column before some other TS column that is already marked as CURRENT_TIMESTAMP cannot become CURRENT_TS
-            if (*c != col && flag && strstr((*c)->defaultValue().c_str(), "TIMESTAMP"))
-              is_first_ts = false;
+              // a column before some other TS column that is already marked as CURRENT_TIMESTAMP cannot become CURRENT_TS
+              if (*c != col && flag && strstr((*c)->defaultValue().c_str(), "TIMESTAMP"))
+                current_timestamp_allowed = false;
+            }
           }
         }
 
@@ -263,13 +277,18 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
 
         item.caption = "Default CURRENT_TIMESTAMP";
         item.name    = "currentTSToolStripMenuItem";
-        item.enabled = is_first_ts;
+        item.enabled = current_timestamp_allowed;
         items.push_back(item);
-        
+
+        item.caption = "Default NULL ON UPDATE CURRENT_TIMESTAMP";
+        item.name    = "currentTSNullOnUpdateToolStripMenuItem";
+        item.enabled = current_timestamp_allowed;
+        items.push_back(item);
+
         item.caption = "Default CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP";
         item.name    = "currentTSOnUpdateToolStripMenuItem";
-        item.enabled = is_first_ts;
-        items.push_back(item);        
+        item.enabled = current_timestamp_allowed;
+        items.push_back(item);
       }
       else if (col->simpleType()->group()->name() == "numeric" ||
                col->simpleType()->group()->name() == "datetime")
@@ -305,6 +324,8 @@ bool MySQLTableColumnsListBE::activate_popup_item_for_nodes(const std::string &n
     value = "''";
   else if (name == "currentTSToolStripMenuItem")
     value = "CURRENT_TIMESTAMP";
+  else if (name == "currentTSNullOnUpdateToolStripMenuItem")
+    value = "NULL ON UPDATE CURRENT_TIMESTAMP";
   else if (name == "currentTSOnUpdateToolStripMenuItem")
     value = "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP";
 
@@ -336,11 +357,327 @@ bool MySQLTableColumnsListBE::activate_popup_item_for_nodes(const std::string &n
   return TableColumnsListBE::activate_popup_item_for_nodes(name, orig_nodes);
 }  
 
+
+//--------------------------------------------------------------------------------------------------
+
+#include <mforms/table.h>
+#include <mforms/treenodeview.h>
+#include <mforms/textentry.h>
+#include <mforms/button.h>
+#include <mforms/box.h>
+#include <mforms/label.h>
+#include <mforms/toolbar.h>
+
+class MySQLTriggerPanel : public mforms::Box
+{
+public:
+  MySQLTriggerPanel(MySQLTableEditorBE *editor)
+  : mforms::Box(true), _editor(editor), _list(mforms::TreeFlatList|mforms::TreeSizeSmall), _button(mforms::SmallButton), _refreshing(false)
+  {
+    set_spacing(8);
+    set_padding(4);
+    
+    _list.set_size(230, -1);
+    _list.set_name("triggers list");
+    add(&_list, false, true);
+    _rtable.set_name("trigger right pane");
+    add(&_rtable, true, true);
+    
+    _rtable.set_row_count(2);
+    _rtable.set_column_count(5);
+    
+    _button.set_text("Delete Trigger");
+    _rtable.set_row_spacing(8);
+    _rtable.set_column_spacing(6);
+    
+    _name.set_name("trigger name");
+    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
+    _definer.set_name("trigger definer");
+    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
+
+    _namel.set_style(mforms::SmallStyle);
+    _definerl.set_style(mforms::SmallStyle);
+    _namel.set_text("Trigger Name:");
+    _definerl.set_text("Definer:");
+    _rtable.add(&_namel, 0, 1, 0, 1, mforms::HFillFlag);
+    _rtable.add(&_name, 1, 2, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    _rtable.add(&_definerl, 2, 3, 0, 1, mforms::HFillFlag);
+    _rtable.add(&_definer, 3, 4, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    _rtable.add(&_button, 4, 5, 0, 1, mforms::HFillFlag);
+
+    _list.add_column(mforms::StringColumnType, _("Name"), 110, false);
+    _list.add_column(mforms::StringColumnType, _("Timing/Event"), 100, false);
+    _list.end_columns();
+    _list.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::selection_changed, this));
+
+    _rtable.add(editor->get_sql_editor()->get_container(), 0, 5, 1, 2,
+                mforms::HExpandFlag|mforms::VExpandFlag|mforms::HFillFlag|mforms::VFillFlag);
+    editor->get_sql_editor()->set_sql_check_enabled(true);
+    editor->get_sql_editor()->get_editor_control()->set_language(mforms::LanguageMySQL);
+    
+    _button.signal_clicked()->connect(boost::bind(&MySQLTriggerPanel::clicked, this));
+    _code_editor = _editor->get_sql_editor()->get_editor_control();
+    _code_editor->signal_lost_focus()->connect(boost::bind(&MySQLTriggerPanel::code_edited, this));
+    refresh();
+    selection_changed();
+  }
+
+  ~MySQLTriggerPanel()
+  {
+    _rtable.remove(_editor->get_sql_editor()->get_container());
+  }
+
+  static bool compare_order(db_TriggerRef a, db_TriggerRef b)
+  {
+    return a->order() > b->order();
+  }
+  
+  void refresh()
+  {
+    _refreshing = true;
+
+    std::set<std::string> leftover;
+    leftover.insert("BEFORE INSERT");
+    leftover.insert("AFTER INSERT");
+    leftover.insert("BEFORE UPDATE");
+    leftover.insert("AFTER UPDATE");
+    leftover.insert("BEFORE DELETE");
+    leftover.insert("AFTER DELETE");
+
+    mforms::TreeNodeRef selected = _list.get_selected_node();
+    int old_selected = 0;
+    if (selected)
+      old_selected = _list.row_for_node(selected);
+    
+    _list.clear();
+    mforms::TreeNodeRef node = _list.add_node();
+    node->set_string(0, "ALL");
+
+    grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+    std::vector<db_TriggerRef> trigvec;
+    GRTLIST_FOREACH(db_Trigger, triggers, trig)
+    {
+      trigvec.push_back(*trig);
+    }
+//    std::sort(trigvec.begin(), trigvec.end(), &MySQLTriggerPanel::compare_order);
+    
+    for (std::vector<db_TriggerRef>::const_iterator trig = trigvec.begin(); trig != trigvec.end(); ++trig)
+    {
+      std::string t = (*trig)->timing();
+      t.append(" ").append((*trig)->event());
+      leftover.erase(t);
+      node = _list.add_node();
+      node->set_string(0, (*trig)->name());
+      node->set_string(1, t);
+      
+      if (_selected_trigger == *trig)
+        _list.select_node(node);
+    }
+    
+    for (std::set<std::string>::const_iterator t = leftover.begin(); t != leftover.end(); ++t)
+    {
+      node = _list.add_node();
+      node->set_string(0, "-");
+      node->set_string(1, *t);
+    }
+
+    _list.select_node(_list.node_at_row(old_selected));
+    //    selection_changed();
+    _refreshing = false;
+  }
+  
+  void code_edited()
+  {
+    if (_selected_trigger.is_valid())
+    {
+      if (_code_editor->is_dirty())
+      {
+        AutoUndoEdit undo(_editor);
+        grt::IntegerRef res= _editor->_sql_parser->parse_trigger(_selected_trigger, _code_editor->get_string_value().c_str());
+        _name.set_value(_selected_trigger->name());
+        _definer.set_value(_selected_trigger->definer());
+        undo.end(strfmt(_("Edit trigger `%s` of `%s`.`%s`"), _selected_trigger->name().c_str(), _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
+      }
+    }
+    else
+    {
+      _editor->commit_changes();
+      refresh();
+    }
+  }
+  
+  void name_changed()
+  {
+    if (_selected_trigger.is_valid())
+    {
+      AutoUndoEdit undo(_editor, _selected_trigger, "name");
+      _selected_trigger->name(_name.get_string_value());
+      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
+      mforms::TreeNodeRef node = _list.get_selected_node();
+      if (node)
+        node->set_string(0, _selected_trigger->name().c_str());
+      selection_changed();
+    }
+  }
+
+  void definer_changed()
+  {
+    if (_selected_trigger.is_valid())
+    {
+      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
+      _selected_trigger->definer(_definer.get_string_value());
+      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
+      selection_changed();
+    }
+  }
+
+  void clicked()
+  {
+    std::string timing, event;
+    mforms::TreeNodeRef node = _list.get_selected_node();
+    
+    if (base::partition(node->get_string(1), " ", timing, event))
+    {
+      grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+      db_TriggerRef trigger;
+      GRTLIST_FOREACH(db_Trigger, triggers, trig)
+      {
+        if ((*trig)->timing() == timing && (*trig)->event() == event)
+        {
+          trigger = *trig;
+          break;
+        }
+      }
+      
+      if (trigger.is_valid())
+      {
+        _editor->freeze_refresh_on_object_change();
+        AutoUndoEdit undo(_editor);
+        triggers.remove_value(trigger);
+        undo.end(base::strfmt("Delete trigger %s", trigger->name().c_str()));
+        
+        node->set_string(0, "-");
+        _editor->thaw_refresh_on_object_change(true);
+      }
+      else
+      {
+        _editor->freeze_refresh_on_object_change();
+        AutoUndoEdit undo(_editor);
+        trigger = db_mysql_TriggerRef(_editor->get_grt());
+        trigger->owner(_editor->get_table());
+        trigger->name(base::strfmt("%s_%c%s", _editor->get_name().c_str(),
+                                   timing[0], event.substr(0, 3).c_str()));
+        trigger->event(event);
+        trigger->timing(timing);
+        triggers.insert(trigger);
+        undo.end(base::strfmt("Added trigger to %s.%s", _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
+        node->set_string(0, trigger->name());
+        _editor->thaw_refresh_on_object_change(true);
+      }
+      update_editor();
+    }
+  }
+  
+  void selection_changed()
+  {
+    if (_refreshing)
+      return;
+    if (_code_editor->is_dirty())
+      code_edited();
+
+    update_editor();
+  }
+
+
+  void update_editor()
+  {
+    mforms::TreeNodeRef node = _list.get_selected_node();
+    if (!node)
+    {
+      _list.select_node(_list.node_at_row(0));
+      return;
+    }
+    std::string timing, event;
+    std::string sql;
+    bool editor_enabled = true;
+    if (base::partition(node->get_string(1), " ", timing, event))
+    {
+      grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+      db_TriggerRef trigger;
+      GRTLIST_FOREACH(db_Trigger, triggers, trig)
+      {
+        if ((*trig)->timing() == timing && (*trig)->event() == event)
+        {
+          trigger = *trig;
+          break;
+        }
+      }
+      
+      _selected_trigger = trigger;
+      
+      if (trigger.is_valid())
+      {
+        _button.set_text("Delete Trigger");
+        _name.set_value(trigger->name());
+        _definer.set_value(trigger->definer());
+        _name.set_enabled(true);
+        _definer.set_enabled(true);
+        
+        sql.append(base::strfmt("USE `%s`;\nDELIMITER $$\n\n", _editor->get_schema_name().c_str()));
+        sql.append(base::strfmt("CREATE TRIGGER `%s` %s %s ON `%s` FOR EACH ROW\n", trigger->name().c_str(),
+                                timing.c_str(), event.c_str(), _editor->get_name().c_str()));
+        sql.append("-- Edit trigger body code below this line. Do not edit lines above this one\n").append(trigger->sqlBody()).append("\n");
+      }
+      else
+      {
+        _name.set_value("");
+        _definer.set_value("");
+        _name.set_enabled(false);
+        _definer.set_enabled(false);
+        _button.set_text("Add Trigger");
+        editor_enabled = false;
+        
+        sql = "-- Trigger not defined. Click Add Trigger to create it.\n";
+      }
+      _button.set_enabled(true);
+    }
+    else
+    {
+      _selected_trigger = db_TriggerRef();
+      _button.set_text("Add Trigger");
+      _name.set_value("");
+      _definer.set_value("");
+      _button.set_enabled(false);
+      _name.set_enabled(false);
+      _definer.set_enabled(false);
+      
+      sql = _editor->get_all_triggers_sql();
+    }
+    _code_editor->set_text_keeping_state(sql.c_str());
+    _code_editor->reset_dirty();
+    _code_editor->set_enabled(editor_enabled);
+  }
+  
+private:
+  MySQLTableEditorBE *_editor;
+  mforms::Table _rtable;
+  mforms::TreeNodeView _list;
+  mforms::TextEntry _name;
+  mforms::TextEntry _definer;
+  mforms::Label _namel;
+  mforms::Label _definerl;
+  mforms::Button _button;
+  mforms::CodeEditor *_code_editor;
+  db_TriggerRef _selected_trigger;
+  bool _refreshing;
+};
+
 //--------------------------------------------------------------------------------------------------
 
 MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table, const db_mgmt_RdbmsRef &rdbms)
-  : TableEditorBE(grtm, table, rdbms), _table(table), _columns(this), _partitions(this), _indexes(this)
+  : TableEditorBE(grtm, table, rdbms), _table(table), _columns(this), _partitions(this), _indexes(this), _trigger_panel(0)
 {
+  _updating_triggers = false;
   if (_table->isStub() == 1)
   {
     if (mforms::Utilities::show_warning(
@@ -354,6 +691,18 @@ MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRe
   }
 }
 
+MySQLTableEditorBE::~MySQLTableEditorBE()
+{
+  delete _trigger_panel;
+}
+
+mforms::View *MySQLTableEditorBE::get_trigger_panel()
+{
+  if (!_trigger_panel)
+    _trigger_panel = new MySQLTriggerPanel(this);
+  return _trigger_panel;
+}
+
 
 std::vector<std::string> MySQLTableEditorBE::get_index_types()
 {
@@ -435,6 +784,7 @@ static struct TableOption
   {"AUTO_INCREMENT",  "nextAutoInc", true},
   {"DELAY_KEY_WRITE", "delayKeyWrite", false},
   {"ROW_FORMAT",      "rowFormat", true},
+  {"KEY_BLOCK_SIZE",  "keyBlockSize", true},
   {"AVG_ROW_LENGTH",  "avgRowLength", true},
   {"MAX_ROWS",        "maxRows", true},
   {"MIN_ROWS",        "minRows", true},
@@ -536,6 +886,8 @@ std::string MySQLTableEditorBE::get_table_option_by_name(const std::string& name
     return _table->delayKeyWrite().repr();
   else if(name.compare("ROW_FORMAT") == 0)
     return _table->rowFormat();
+  else if(name.compare("KEY_BLOCK_SIZE") == 0)
+    return _table->keyBlockSize();
   else if(name.compare("AVG_ROW_LENGTH") == 0)
     return _table->avgRowLength();
   else if(name.compare("MAX_ROWS") == 0)
@@ -570,7 +922,10 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
 {
   std::string retval;
 
-  retval.append("-- Trigger DDL Statements\n").append(
+  retval.append("-- Full Trigger DDL Statements\n");
+  retval.append("-- Note: Only CREATE TRIGGER statements are allowed\n");
+
+  retval.append(
     strfmt("DELIMITER %s\n\n", _non_std_sql_delimiter.c_str())).append(
     "USE `").append(_table->owner()->name()).append("`").
     append(_non_std_sql_delimiter.c_str()).append("\n\n");
@@ -609,10 +964,22 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
  */
 void MySQLTableEditorBE::load_trigger_sql()
 {
-  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
-  std::string sql = get_all_triggers_sql();
-  editor->set_text_keeping_state(sql.c_str());
-  editor->reset_dirty();
+  if (_trigger_panel && !_updating_triggers)
+  {
+    _updating_triggers = true;
+    _trigger_panel->refresh();
+    _trigger_panel->update_editor();
+    _updating_triggers = false;
+  }
+}
+
+
+//--------------------------------------------------------------------------------------------------
+
+bool MySQLTableEditorBE::can_close()
+{
+  _trigger_panel->code_edited(); // Same handling as for focus-lost. Might not be necessary but better safe than sorry.
+  return TableEditorBE::can_close();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -624,7 +991,7 @@ void MySQLTableEditorBE::commit_changes()
   {
     const std::string sql = editor->get_text(false);
     set_triggers_sql(sql, true);
-    editor->reset_dirty();
+    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
   }
 
   // TODO: anything else to store?
@@ -714,6 +1081,11 @@ int MySQLTableEditorBE::get_partition_count()
 }
 
 
+bool MySQLTableEditorBE::subpartition_count_allowed()
+{
+  return (*table()->partitionType() == "RANGE" || *table()->partitionType() == "LIST");
+}
+
 bool MySQLTableEditorBE::set_subpartition_type(const std::string &type)
 {
   if (*table()->partitionType() == "RANGE" || *table()->partitionType() == "LIST")
