@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,7 +48,7 @@ static void call_refresh(DbMysqlRoutineEditor *self)
     // Therefore the nib should be designed as small as possible
     // note: the honouring of the min size is not yet implemented
     [self setMinimumSize: [[tabView superview] frame].size];
-    
+
     [self reinitWithArguments: args];
   }
   return self;
@@ -62,15 +62,22 @@ static void call_refresh(DbMysqlRoutineEditor *self)
   
     // setup the editor backend with the schema object (args[0])
   mBackEnd= new MySQLRoutineEditorBE(_grtm, db_mysql_RoutineRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]));
-    
+  
   // register a callback that will make [self refresh] get called
   // whenever the backend thinks its needed to refresh the UI from the backend data (ie, the
   // edited object was changed from somewhere else in the application)
   mBackEnd->set_refresh_ui_slot(boost::bind(call_refresh, self));
   
-  [self setupEditorOnHost: editorHost withSyntaxSupport: YES];
+  [self setupEditorOnHost: editorHost];
   mBackEnd->load_routine_sql();
 
+  if (mBackEnd->is_editing_live_object())
+  {
+    int i = [tabView indexOfTabViewItemWithIdentifier: @"comments"];
+    if (i > 0)
+      [tabView removeTabViewItem: [tabView tabViewItemAtIndex: i]];
+    commentText = nil;
+  }
   // update the UI
   [self refresh];
 }
@@ -94,6 +101,11 @@ static void call_refresh(DbMysqlRoutineEditor *self)
   {
     [nameText setStringValue: [NSString stringWithCPPString: mBackEnd->get_name()]];
     [self updateTitle: [self title]];
+
+    if (!mBackEnd->is_editing_live_object())
+      [commentText setString: [NSString stringWithCPPString: mBackEnd->get_comment()]];
+
+    //mBackEnd->load_routine_sql(); SQL text change actually causes this refresh.
   }
 }
 
@@ -107,9 +119,11 @@ static void call_refresh(DbMysqlRoutineEditor *self)
 
 - (void) textDidEndEditing: (NSNotification *) aNotification
 {
-  if ([[aNotification object] isKindOfClass: [ScintillaView class]])
-  {    
-    mBackEnd->commit_changes();
+  if ([aNotification object] == commentText)
+  {
+    [[aNotification object] breakUndoCoalescing];
+    // Set comment for the view.
+    mBackEnd->set_comment([[commentText string] UTF8String]);
   }
 }
 
@@ -120,9 +134,4 @@ static void call_refresh(DbMysqlRoutineEditor *self)
   return mBackEnd;
 }
 
-
-- (NSImage*)titleIcon
-{
-  return [NSImage imageNamed: @"tab_icon_db.Routine.png"];
-}
 @end
