@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,6 +42,8 @@ static void call_refresh(DbMysqlSchemaEditor *self)
     // load GUI. Top level view in the nib is the NSTabView that will be docked to the main window
     [self loadView];
 
+    [self enablePluginDocking: tabView];
+
     // take the minimum size of the view from the initial size in the nib.
     // Therefore the nib should be designed as small as possible
     // note: the honouring of the min size is not yet implemented
@@ -65,8 +67,11 @@ static void call_refresh(DbMysqlSchemaEditor *self)
         [nameText setEnabled: NO];
         [[[commentText superview] viewWithTag: 101] setEnabled: NO];
       }
-      [commentText setEditable: NO];
-      [[[commentText superview] viewWithTag: 103] setEnabled: NO];
+    }
+    else
+    {
+      [[commentText enclosingScrollView] setHidden: NO];
+      [[[[commentText enclosingScrollView] superview] viewWithTag: 103] setHidden: NO];
     }
     
     // update the UI
@@ -95,11 +100,16 @@ static void call_refresh(DbMysqlSchemaEditor *self)
       [nameText setEnabled: NO];
       [[[commentText superview] viewWithTag: 101] setEnabled: NO];
     }
-    [commentText setEditable: NO];
-    [[[commentText superview] viewWithTag: 103] setEnabled: NO];
+  }
+  else
+  {
+    [[commentText enclosingScrollView] setHidden: NO];
+    [[[[commentText enclosingScrollView] superview] viewWithTag: 103] setHidden: NO];
   }
   // update the UI
   [self refresh];
+
+  [self notifyObjectSwitched];
 }
 
 
@@ -144,6 +154,8 @@ static void call_refresh(DbMysqlSchemaEditor *self)
 - (void)pluginDidShow: (id)sender
 {
   [[tabView window] makeFirstResponder: nameText];
+
+  [super pluginDidShow: sender];
 }
 
 
@@ -169,7 +181,7 @@ static void call_refresh(DbMysqlSchemaEditor *self)
 }
 
 
-- (void)textDidChange:(NSNotification *)aNotification
+- (void)textDidEndEditing:(NSNotification *)aNotification
 {
   if ([aNotification object] == commentText)
   {
@@ -183,8 +195,4 @@ static void call_refresh(DbMysqlSchemaEditor *self)
   return mBackEnd;
 }
 
-- (NSImage*)titleIcon
-{
-  return [NSImage imageNamed: @"tab_icon_db.Schema.png"];
-}
 @end
