@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -54,6 +54,10 @@
 
 static NSString* shouldRaiseException = @"should raise exception";
 
+
+@implementation DbMysqlTableEditor
+
+
 // Callback to update the editor GUI to reflect changes in the backend.
 static void call_refresh(DbMysqlTableEditor* theEditor)
 {
@@ -63,7 +67,34 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
                            waitUntilDone: YES];
 }
 
-@implementation DbMysqlTableEditor
+
+static void call_partial_refresh(int what, DbMysqlTableEditor* theEditor)
+{
+  switch (what)
+  {
+    case bec::TableEditorBE::RefreshColumnMoveUp:
+    {
+      NSTableView *columns = theEditor->mColumnsTable;
+      NSInteger i = [columns selectedRow];
+      [columns reloadData];
+      [columns selectRowIndexes: [NSIndexSet indexSetWithIndex: i-1]
+           byExtendingSelection: NO];
+      break;
+    }
+    case bec::TableEditorBE::RefreshColumnMoveDown:
+    {
+      NSTableView *columns = theEditor->mColumnsTable;
+      NSInteger i = [columns selectedRow];
+      [columns reloadData];
+      [columns selectRowIndexes: [NSIndexSet indexSetWithIndex: i+1]
+           byExtendingSelection: NO];
+      break;
+    }
+    default:
+      call_refresh(theEditor);
+      break;
+  }
+}
 
 #pragma mark Refreshing of views
 
@@ -74,7 +105,6 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     NSString* name = [NSString stringWithCPPString: mBackEnd->get_name()];
     [mTableName setStringValue: name];
     
-    [mSchemaName setEnabled: !mBackEnd->is_editing_live_object()];
     [mSchemaName setStringValue: [NSString stringWithCPPString: mBackEnd->get_schema_name()]];
 
     NSString* collation = [NSString stringWithCPPString: mBackEnd->get_table_option_by_name("CHARACTER SET - COLLATE")];
@@ -169,12 +199,6 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     
     [mIndicesDetailsBox setTitle: [NSString stringWithFormat: @"Index details '%@'", indexName]];
     
-//    obj = [mIndicesDataSource objectValueForValueIndex: valueIndex
-//                                                   row: rowIndex];
-
-//    NSString* columnName = [mIndicesDataSource valueForIndexColumn: bec::IndexColumnsListBE::Name
-//                                                        atIndexRow: rowIndex
-//                                                  withTableBackend: mBackEnd];
     {
       mBackEnd->get_indexes()->select_index(rowIndex);
       
@@ -211,14 +235,9 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     
     col = [mFKColumnsTable tableColumnWithIdentifier: @"referenced column"];
     cell = [col dataCell];
-    NSInteger rowIndex = [mFKTable selectedRow];
-    if (rowIndex >= 0) {
-      NSString* tableName = [mFKDataSource objectValueForValueIndex: bec::FKConstraintListBE::RefTable
-                                                                row: rowIndex];
-      if ([tableName length] > 0) {
-        MFillPopupButtonWithStrings((NSPopUpButton*)cell, mBackEnd->get_table_column_names([tableName UTF8String]));
-      }
-    }
+    NSInteger rowIndex = [mFKColumnsTable selectedRow];
+    if (rowIndex >= 0)
+      MFillPopupButtonWithStrings((NSPopUpButton*)cell, mBackEnd->get_fks()->get_columns()->get_ref_columns_list(rowIndex, false));
   }
   
   int rowIndex = [mFKTable selectedRow];
@@ -346,8 +365,15 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
   option = [NSString stringWithUTF8String: mBackEnd->get_table_option_by_name("ROW_FORMAT").c_str()];
   if ([option length] == 0)
     option = @"Default";
-  [mOptionsRowFormat selectItemWithTitle: option];
-  
+  [mOptionsRowFormat selectItemWithTitle: [option capitalizedString]];
+
+  option = [NSString stringWithUTF8String: mBackEnd->get_table_option_by_name("KEY_BLOCK_SIZE").c_str()];
+  if ([option length] == 0)
+    option = @"Default";
+  else
+    option = [NSString stringWithFormat: @"%@ KB", option];
+  [mOptionsBlockSize selectItemWithTitle: option];
+
   option = [NSString stringWithUTF8String: mBackEnd->get_table_option_by_name("AVG_ROW_LENGTH").c_str()];
   [mOptionsAvgRowLength setStringValue: option];
   
@@ -390,6 +416,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
 
 - (void)refreshTableEditorGUITriggersTab
 {
+  mBackEnd->load_trigger_sql();
 }
 
 
@@ -661,6 +688,8 @@ objectValueForTableColumn: (NSTableColumn*) aTableColumn
       [aCell setState: ( yn ? NSOnState : NSOffState )];
     }
     else if ([identifier isEqual: @"referenced column"]) {
+      MFillPopupButtonWithStrings((NSPopUpButton*)aCell, mBackEnd->get_fks()->get_columns()->get_ref_columns_list(rowIndex, false));
+
       NSString* title = [mFKColumnsDataSource objectValueForValueIndex: bec::FKConstraintColumnsListBE::RefColumn
                                                                    row: rowIndex];
       [aCell selectItemWithTitle: title];
@@ -1171,6 +1200,13 @@ objectValueForItemAtIndex: (NSInteger) index
     [[sender window] makeFirstResponder: sender];
     mBackEnd->set_table_option_by_name("ROW_FORMAT", [popItemTitle UTF8String]);
   }
+  else if (sender == mOptionsBlockSize) {
+    std::string bsize;
+    [[sender window] makeFirstResponder: sender];
+    if ([sender indexOfSelectedItem] > 0)
+      bsize = [[[popItemTitle componentsSeparatedByString: @" "] objectAtIndex: 0] UTF8String];
+    mBackEnd->set_table_option_by_name("KEY_BLOCK_SIZE", bsize);
+  }
   else if (sender == mOptionsMergeMethod) {
     [[sender window] makeFirstResponder: sender];
     if ([sender indexOfSelectedItem] == 0)
@@ -1262,6 +1298,7 @@ objectValueForItemAtIndex: (NSInteger) index
   else {
     NSAssert1(NO, @"DEBUG - User clicked unmatched button: '%@'", [sender title]);
   }
+  [mColumnsTable setNeedsDisplay];
 }
 
 /**
@@ -1307,6 +1344,8 @@ objectValueForItemAtIndex: (NSInteger) index
 
     return;
   }
+  else if ([sender isKindOfClass: [NSTableView class]])
+    return;
 
   if (sender == mTableName) {
     mBackEnd->set_name([[mTableName stringValue] UTF8String]);
@@ -1374,7 +1413,26 @@ objectValueForItemAtIndex: (NSInteger) index
   else if (sender == mOptionsUnionTables) {
     mBackEnd->set_table_option_by_name("UNION", [[sender stringValue] UTF8String]);
   }
-  
+  else if (sender == mColumnsName) {
+    if ([mColumnsTable selectedRow] >= 0)
+      mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Name, [[sender stringValue] UTF8String]);
+    [mColumnsTable setNeedsDisplay];
+  }
+  else if (sender == mColumnsCollation || sender == mColumnsCollation2) {
+    if ([mColumnsTable selectedRow] >= 0)
+      mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Collation, [[sender stringValue] UTF8String]);
+    [mColumnsTable setNeedsDisplay];
+  }
+  else if (sender == mColumnsType) {
+    if ([mColumnsTable selectedRow] >= 0)
+      mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Type, [[sender stringValue] UTF8String]);
+    [mColumnsTable setNeedsDisplay];
+  }
+  else if (sender == mColumnsDefault) {
+    if ([mColumnsTable selectedRow] >= 0)
+      mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Default, [[sender stringValue] UTF8String]);
+    [mColumnsTable setNeedsDisplay];
+  }
   else {
     NSAssert1(NO, @"DEBUG - Unknown text field: %@", sender);
   }
@@ -1389,25 +1447,25 @@ objectValueForItemAtIndex: (NSInteger) index
   id sender = [aNotification object];
   
   if (sender == mTableComment) {
+    [[aNotification object] breakUndoCoalescing];
     mBackEnd->set_comment([[mTableComment string] UTF8String]);
   }
-  else if (sender == mColumnsComment) {
+  else if (sender == mColumnsComment || sender == mColumnsComment2) {
+    [[aNotification object] breakUndoCoalescing];
     int rowIndex = [mColumnsTable selectedRow];
-    [mColumnsDataSource setStringValue: [mColumnsComment string]
+    [mColumnsDataSource setStringValue: [sender string]
                          forValueIndex: bec::TableColumnsListBE::Comment
                               row: rowIndex];
   }
-  else if ([sender isKindOfClass: [ScintillaView class]]) {
-    // Works because we only have one code editor.
-    mBackEnd->commit_changes();
-  }
   else if (sender == mIndicesComment) {
+    [[aNotification object] breakUndoCoalescing];
     int rowIndex = [mIndicesTable selectedRow];
     [mIndicesDataSource setStringValue: [mIndicesComment string]
                          forValueIndex: bec::IndexListBE::Comment
-                              row: rowIndex];
+                                   row: rowIndex];
   }
   else if (sender == mFKComment) {
+    [[aNotification object] breakUndoCoalescing];
     int rowIndex = [mFKTable selectedRow];
     [mFKDataSource setStringValue: [mFKComment string]
                     forValueIndex: bec::FKConstraintListBE::Comment
@@ -1444,6 +1502,7 @@ objectValueForItemAtIndex: (NSInteger) index
 - (void)pluginDidShow: (id)sender
 {
   [[[self view] window] makeFirstResponder: mTableName];
+  [super pluginDidShow: sender];
 }
 
 
@@ -1482,7 +1541,9 @@ objectValueForItemAtIndex: (NSInteger) index
     _grtm = grtm;
     
     [self loadView];
-    
+
+    [self enablePluginDocking: mEditorsTabView];
+
     [self reinitWithArguments: args];
   }
   
@@ -1502,7 +1563,7 @@ objectValueForItemAtIndex: (NSInteger) index
 
   db_mysql_TableRef table = db_mysql_TableRef::cast_from(args[0]);
   mBackEnd = new MySQLTableEditorBE(_grtm, table, get_rdbms_for_db_object(table));
-  
+
   if (!isReinit)
   {
     if (!mBackEnd->is_editing_live_object())
@@ -1517,13 +1578,12 @@ objectValueForItemAtIndex: (NSInteger) index
     {
       mUnusedColumnsDetailsBox = mColumnsDetailsBox2;
       [mColumnsSplitter setVertical: NO];
+
+      [mIndicesComment setEditable: mBackEnd->is_server_version_at_least(5, 5)];
     }
     [mColumnsSplitter addSubview: mColumnsDetailsBox];
   }
-  
-  [mSchemaName removeAllItems];
-  [mSchemaName addItemsWithTitles: MArrayFromStringVector(mBackEnd->get_all_schema_names())];
-  
+
   [mColumnsDataSource release];
   mColumnsDataSource = [[MacTableEditorColumnsInformationSource alloc] initWithListModel: mBackEnd->get_columns()
                                                                             tableBackEnd: mBackEnd];
@@ -1609,7 +1669,8 @@ objectValueForItemAtIndex: (NSInteger) index
   }
   // Register a callback that will call [self refresh] when the edited object is
   // changed from somewhere else in the application.
-  mBackEnd->set_refresh_ui_slot(boost::bind(call_refresh, self));    
+  mBackEnd->set_refresh_ui_slot(boost::bind(call_refresh, self));
+  mBackEnd->set_partial_refresh_ui_slot(boost::bind(call_partial_refresh, _1, self));
   
   [self updateFKPlaceholder];
   {
@@ -1620,9 +1681,11 @@ objectValueForItemAtIndex: (NSInteger) index
     [view setAutoresizesSubviews: YES];
     [view setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable|NSMinXEdge|NSMinYEdge|NSMaxXEdge|NSMaxYEdge];
   }
-  
+
   // Update the GUI.
   [self refreshTableEditorGUI];
+
+  [self notifyObjectSwitched];
 }
 
 
@@ -1649,7 +1712,7 @@ objectValueForItemAtIndex: (NSInteger) index
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"indices"] autorelease];
   [item setView: mEditorIndices];
-  [item setLabel: @"Indices"];
+  [item setLabel: @"Indexes"];
   [mEditorsTabView addTabViewItem: item];
   [mEditorIndices setBackgroundColor: [NSColor whiteColor]];
   
@@ -1714,31 +1777,11 @@ objectValueForItemAtIndex: (NSInteger) index
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Called when text was added or removed in the editor.
- */
-- (void) textDidChange: (NSNotification*) aNotification
-{
-  if ([aNotification object] == mTableComment)
-  {
-    // Set comment for the table.
-    mBackEnd->set_comment([[mTableComment string] UTF8String]);
-  }
-};
-
-//--------------------------------------------------------------------------------------------------
-
 - (bec::BaseEditor*)editorBE
 {
   return mBackEnd;
 }
 
-
-- (NSImage*)titleIcon
-{
-  return [NSImage imageNamed: @"tab_icon_db.Table.png"];
-}
-
 @end
 
 
