@@ -41,7 +41,10 @@ namespace MySQL.GUI.Workbench.Plugins
       InitializeComponent();
       ReinitWithArguments(value);
       if (IsEditingLiveObject)
+      {
         mainTabControl.TabPages.Remove(commentTabPage);
+        AdjustEditModeControls(mainTabControl);
+      }
     }
 
     #endregion
@@ -97,24 +100,19 @@ namespace MySQL.GUI.Workbench.Plugins
 
     protected override void RefreshFormData()
     {
+      base.RefreshFormData();
+
       nameTextBox.Text = routineEditorBE.get_name();
       TabText = routineEditorBE.get_title();
       if (!IsEditingLiveObject)
         commentTextBox.Text = routineEditorBE.get_comment();
-
-      CallRefreshFormData();
-    }
-
-    public override void EditorLostFocus()
-    {
-      routineEditorBE.commit_changes();
     }
 
     #endregion
     
     #region Event Handling
 
-    private void commentTextBox_TextChanged(object sender, EventArgs e)
+    private void commentTextBox_Leave(object sender, EventArgs e)
     {
       if (!IsEditingLiveObject)
         routineEditorBE.set_comment(commentTextBox.Text);
