@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@ namespace MySQL.GUI.Workbench.Plugins
   {
     #region Member Variables
 
-    private MySQLTableEditorBE tableEditorBE { get { return Backend as MySQLTableEditorBE; } } 
+    private MySQLTableEditorWrapper tableEditorBE { get { return Backend as MySQLTableEditorWrapper; } } 
     private bool dragInProgress = false;
 
     private DbMysqlTableColumnsListModel columnsListModel;
@@ -50,7 +50,7 @@ namespace MySQL.GUI.Workbench.Plugins
     private DbObjectEditorPages dbObjectEditorPages = null;
 
     private SimpleGrtTreeModel partitionTreeModel;
-    private MySQLTablePartitionTreeBE partitionTreeBE;
+    private MySQLTablePartitionTreeWrapper partitionTreeBE;
 
     private bool headingCollapsed = false;
     private bool privPageWasActive = false; // Set when doing re-init.
@@ -70,22 +70,18 @@ namespace MySQL.GUI.Workbench.Plugins
       InitializeComponent();
       ReinitWithArguments(value);
 
-      if (!IsEditingLiveObject)
+      if (IsEditingLiveObject)
+        AdjustEditModeControls(mainTabControl);
+      else
         AdjustToSmallerLayout();
+
+      topPanel.Parent = mainTabControl.Parent;
     }
 
     #endregion
 
     #region ObjectEditorPlugin Overrides
 
-    protected override void CallRefreshFormData()
-    {
-        base.CallRefreshFormData();
-
-        // XXX: Kind of a hack since the base class doesn't care about anything other than mainTabControl.
-        topPanel.Parent = mainTabControl.Parent;
-    }
-
     public override bool ReinitWithArguments(GrtValue value)
     {
       if (insertsRecordsetView != null && insertsRecordsetView.Model.has_pending_changes())
@@ -106,7 +102,7 @@ namespace MySQL.GUI.Workbench.Plugins
           mainTabControl.TabPages.Remove(dbObjectEditorPages.PrivilegesTabPage);
         }
 
-        Backend = new MySQLTableEditorBE(GrtManager, value);
+        Backend = new MySQLTableEditorWrapper(GrtManager, value);
 
         Control panel = tableEditorBE.get_trigger_panel();
         triggersTabPage.Controls.Add(panel);
@@ -115,7 +111,7 @@ namespace MySQL.GUI.Workbench.Plugins
         dbObjectEditorPages = new DbObjectEditorPages(GrtManager, tableEditorBE);
 
         InitFormData();
-        CallRefreshFormData();
+        RefreshFormData();
       }
       finally
       {
@@ -212,7 +208,6 @@ namespace MySQL.GUI.Workbench.Plugins
       // engines
       optEngine.Enabled = true;
       optEngine.Items.Clear();
-      optEngine.Items.Add("Server Default");
       System.Collections.Generic.List<String> engines_list = tableEditorBE.get_engines_list();
       foreach (String engine in engines_list)
       {
@@ -264,13 +259,13 @@ namespace MySQL.GUI.Workbench.Plugins
 
       partitionTreeBE = tableEditorBE.get_partitions();
       partitionTreeModel = new SimpleGrtTreeModel(partitionTreeView, partitionTreeBE, partNodeStateIcon, false);
-      partitionTreeModel.AddColumn(partNameNodeControl, (int)MySQLTablePartitionTreeBE.Columns.Name, true);
-      partitionTreeModel.AddColumn(partValuesNodeControl, (int)MySQLTablePartitionTreeBE.Columns.Value, true);
-      partitionTreeModel.AddColumn(partDataDirNodeControl, (int)MySQLTablePartitionTreeBE.Columns.DataDirectory, true);
-      partitionTreeModel.AddColumn(partIndexDirNodeControl, (int)MySQLTablePartitionTreeBE.Columns.IndexDirectory, true);
-      partitionTreeModel.AddColumn(partMaxRowsNodeControl, (int)MySQLTablePartitionTreeBE.Columns.MaxRows, true);
-      partitionTreeModel.AddColumn(partMinRowsNodeControl, (int)MySQLTablePartitionTreeBE.Columns.MinRows, true);
-      partitionTreeModel.AddColumn(partCommentNodeControl, (int)MySQLTablePartitionTreeBE.Columns.Comment, true);
+      partitionTreeModel.AddColumn(partNameNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.Name, true);
+      partitionTreeModel.AddColumn(partValuesNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.Value, true);
+      partitionTreeModel.AddColumn(partDataDirNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.DataDirectory, true);
+      partitionTreeModel.AddColumn(partIndexDirNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.IndexDirectory, true);
+      partitionTreeModel.AddColumn(partMaxRowsNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.MaxRows, true);
+      partitionTreeModel.AddColumn(partMinRowsNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.MinRows, true);
+      partitionTreeModel.AddColumn(partCommentNodeControl, (int)MySQLTablePartitionTreeWrapper.Columns.Comment, true);
       partitionTreeView.Model = partitionTreeModel;
       partitionTreeView.Refresh();
     }
@@ -308,20 +303,42 @@ namespace MySQL.GUI.Workbench.Plugins
         e.Value = false;
     }
 
-    protected override void RefreshPartialFormData(int where)
+    protected override void RefreshPartialFormData(int what)
     {
-      if (where == 0) // Refresh Column List
+      base.RefreshPartialFormData(what);
+
+      switch ((TableEditorWrapper.PartialRefreshes)what)
       {
-        columnsListModel.RefreshModel();
-      }
-      else if (where == 1) // Refresh Column Collation
-      {
-        columnsTreeView_SelectionChanged(columnsTreeView, null);
+        case TableEditorWrapper.PartialRefreshes.RefreshColumnList:
+          columnsListModel.RefreshModel();
+          break;
+
+        case TableEditorWrapper.PartialRefreshes.RefreshColumnCollation:
+          columnsTreeView_SelectionChanged(columnsTreeView, null);
+          break;
+
+        case TableEditorWrapper.PartialRefreshes.RefreshColumnMoveUp:
+          {
+            int index = columnsListModel.TreeView.SelectedNode.Index;
+            columnsListModel.RefreshModel();
+            columnsListModel.TreeView.SelectedNode = columnsListModel.TreeView.Root.Children[index - 1];
+            break;
+          }
+
+        case TableEditorWrapper.PartialRefreshes.RefreshColumnMoveDown:
+          {
+            int index = columnsListModel.TreeView.SelectedNode.Index;
+            columnsListModel.RefreshModel();
+            columnsListModel.TreeView.SelectedNode = columnsListModel.TreeView.Root.Children[index + 1];
+            break;
+          }
       }
     }
 
     protected override void RefreshFormData()
     {
+      base.RefreshFormData();
+
       // Sets all the table options.
       refreshTableOptUi();
 
@@ -573,10 +590,10 @@ namespace MySQL.GUI.Workbench.Plugins
           int isPk;
 
           columnsListModel.GrtList.get_field(node.NodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out isPk);
+            (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK, out isPk);
 
           columnsListModel.GrtList.set_field(node.NodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, (isPk + 1) % 2);
+            (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK, (isPk + 1) % 2);
         }
         else if (e.Control is AdvNodeTextBox)
         {
@@ -593,7 +610,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void columnsTreeView_SelectionChanged(object sender, EventArgs e)
     {
-      if (InsideInitializationOrRefresh())
+      if (InitializingControls)
         return;
 
       if (columnsTreeView.SelectedNode != null)
@@ -602,7 +619,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
         string stringValue;
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, out stringValue))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.Comment, out stringValue))
         {
           columnCommentTextBox.Text = stringValue;
           columnCommentTextBox.Enabled = true;
@@ -616,12 +633,12 @@ namespace MySQL.GUI.Workbench.Plugins
         // charset/collation
         String hasCharset = null;
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.HasCharset, out hasCharset) &&
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.HasCharset, out hasCharset) &&
           (hasCharset == "1"))
         {
           String column_cscoll;
           columnsListModel.GrtList.get_field(nodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation, out column_cscoll);
+            (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.CharsetCollation, out column_cscoll);
 
           columnCollationComboBox.Enabled = true;
 
@@ -645,7 +662,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out stringValue))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.Name, out stringValue))
         {
           columnNameTextBox.Text = stringValue;
           columnNameTextBox.Enabled = true;
@@ -657,7 +674,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out stringValue))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.Type, out stringValue))
         {
           columnDataTypeTextBox.Text = stringValue;
           columnDataTypeTextBox.Enabled = true;
@@ -669,7 +686,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out stringValue))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.Default, out stringValue))
         {
           columnDefaultTextBox.Text = stringValue;
           columnDefaultTextBox.Enabled = true;
@@ -682,7 +699,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
         int flag;
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK, out flag))
         {
           pkCheckBox.Checked = flag != 0;
           pkCheckBox.Enabled = true;
@@ -694,7 +711,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsNotNull, out flag))
         {
           nnCheckBox.Checked = flag != 0;
           nnCheckBox.Enabled = true;
@@ -706,7 +723,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsUnique, out flag))
         {
           uniqueCheckBox.Checked = flag != 0;
           uniqueCheckBox.Enabled = true;
@@ -718,7 +735,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsBinary, out flag))
         {
           binaryCheckBox.Checked = flag != 0;
           binaryCheckBox.Enabled = true;
@@ -730,7 +747,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsUnsigned, out flag))
         {
           unsignedCheckBox.Checked = flag != 0;
           unsignedCheckBox.Enabled = true;
@@ -742,7 +759,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsZerofill, out flag))
         {
           zeroFillCheckBox.Checked = flag != 0;
           zeroFillCheckBox.Enabled = true;
@@ -754,7 +771,7 @@ namespace MySQL.GUI.Workbench.Plugins
         }
 
         if (columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out flag))
+          (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsAutoIncrement, out flag))
         {
           aiCheckBox.Checked = flag != 0;
           aiCheckBox.Enabled = true;
@@ -802,18 +819,18 @@ namespace MySQL.GUI.Workbench.Plugins
         if (columnCollationComboBox.SelectedIndex == 0)
         {
           columnsListModel.GrtList.set_field(nodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation, "");
+            (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.CharsetCollation, "");
         }
         else
         {
           columnsListModel.GrtList.set_field(nodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation,
+            (int)MySQLTableColumnsListWrapper.MySQLColumnListColumns.CharsetCollation,
               columnCollationComboBox.Text);
         }
       }
     }
 
-    private void updateColumnTextField(MySQLTableColumnsListBE.MySQLColumnListColumns column, TextBox box)
+    private void updateColumnTextField(MySQLTableColumnsListWrapper.MySQLColumnListColumns column, TextBox box)
     {
       if (columnsTreeView.SelectedNode != null)
       {
@@ -838,23 +855,23 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void applyColumnText(TextBox box)
     {
-      MySQLTableColumnsListBE.MySQLColumnListColumns column = MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK;
+      MySQLTableColumnsListWrapper.MySQLColumnListColumns column = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK;
       switch (box.Tag.ToString())
       {
         case "0":
-          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Name;
+          column = MySQLTableColumnsListWrapper.MySQLColumnListColumns.Name;
           break;
         case "1":
-          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Comment;
+          column = MySQLTableColumnsListWrapper.MySQLColumnListColumns.Comment;
           break;
         case "2":
-          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Type;
+          column = MySQLTableColumnsListWrapper.MySQLColumnListColumns.Type;
           break;
         case "3":
-          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Default;
+          column = MySQLTableColumnsListWrapper.MySQLColumnListColumns.Default;
           break;
       }
-      if (column != MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK)
+      if (column != MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK)
       {
         updateColumnTextField(column, box);
         box.SelectAll();
@@ -875,33 +892,33 @@ namespace MySQL.GUI.Workbench.Plugins
       NodeId nodeId = new NodeId(columnsTreeView.SelectedNode.Index);
       CheckBox box = sender as CheckBox;
       int value = box.Checked ? 1 : 0;
-      MySQLTableColumnsListBE.MySQLColumnListColumns columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.Name;
+      MySQLTableColumnsListWrapper.MySQLColumnListColumns columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.Name;
       switch (box.Tag.ToString())
       {
         case "0":
-          columnValue =  MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK;
+          columnValue =  MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsPK;
           break;
         case "1":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsNotNull;
           break;
         case "2":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsUnique;
           break;
         case "3":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsBinary;
           break;
         case "4":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsUnsigned;
           break;
         case "5":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsZerofill;
           break;
         case "6":
-          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement;
+          columnValue = MySQLTableColumnsListWrapper.MySQLColumnListColumns.IsAutoIncrement;
           break;
       }
 
-      if (columnValue != MySQLTableColumnsListBE.MySQLColumnListColumns.Name)
+      if (columnValue != MySQLTableColumnsListWrapper.MySQLColumnListColumns.Name)
       {
         if (!columnsListModel.GrtList.set_field(nodeId, (int)columnValue, value))
         {
@@ -952,17 +969,17 @@ namespace MySQL.GUI.Workbench.Plugins
           // assign model to treeview
           indexColumnsTreeView.Model = indexColumnsListModel;
 
-          MySQL.Grt.Db.IndexListBE ilist = tableEditorBE.get_indexes();
+          MySQL.Grt.Db.IndexListWrapper ilist = tableEditorBE.get_indexes();
 
           String value = null;
 
-          ilist.get_field(node.NodeId, (int)MySQL.Grt.Db.IndexListBE.IndexListColumns.Comment, out value);
+          ilist.get_field(node.NodeId, (int)MySQL.Grt.Db.IndexListWrapper.IndexListColumns.Comment, out value);
           indexCommentText.Text = value;
-          ilist.get_field(node.NodeId, (int)MySQLIndexListBE.Columns.StorageType, out value);
+          ilist.get_field(node.NodeId, (int)MySQLIndexListWrapper.Columns.StorageType, out value);
           indexStorageTypeComboBox.SelectedItem = value != "" ? value : null;
-          ilist.get_field(node.NodeId, (int)MySQLIndexListBE.Columns.RowBlockSize, out value);
+          ilist.get_field(node.NodeId, (int)MySQLIndexListWrapper.Columns.RowBlockSize, out value);
           indexRowBlockSizeText.Text = value;
-          ilist.get_field(node.NodeId, (int)MySQLIndexListBE.Columns.Parser, out value);
+          ilist.get_field(node.NodeId, (int)MySQLIndexListWrapper.Columns.Parser, out value);
           indexParserText.Text = value;
         }
         else
@@ -1005,11 +1022,11 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId, 
-            (int)MySQL.Grt.Db.IndexListBE.IndexListColumns.Comment, out text);
+            (int)MySQL.Grt.Db.IndexListWrapper.IndexListColumns.Comment, out text);
           if (indexCommentText.Text != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId, 
-              (int)MySQL.Grt.Db.IndexListBE.IndexListColumns.Comment, indexCommentText.Text);
+              (int)MySQL.Grt.Db.IndexListWrapper.IndexListColumns.Comment, indexCommentText.Text);
           }
         }
       }
@@ -1025,11 +1042,11 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId, 
-            (int)MySQLIndexListBE.Columns.Parser, out text);
+            (int)MySQLIndexListWrapper.Columns.Parser, out text);
           if (indexParserText.Text != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId, 
-              (int)MySQLIndexListBE.Columns.Parser, indexParserText.Text);
+              (int)MySQLIndexListWrapper.Columns.Parser, indexParserText.Text);
           }
         }
       }
@@ -1045,11 +1062,11 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId, 
-            (int)MySQLIndexListBE.Columns.RowBlockSize, out text);
+            (int)MySQLIndexListWrapper.Columns.RowBlockSize, out text);
           if (indexRowBlockSizeText.Text != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId, 
-              (int)MySQLIndexListBE.Columns.RowBlockSize, indexRowBlockSizeText.Text);
+              (int)MySQLIndexListWrapper.Columns.RowBlockSize, indexRowBlockSizeText.Text);
           }
         }
       }
@@ -1065,17 +1082,17 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId,
-            (int)MySQLIndexListBE.Columns.StorageType, out text);
+            (int)MySQLIndexListWrapper.Columns.StorageType, out text);
 
           if (indexStorageTypeComboBox.SelectedItem == null)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId,
-              (int)MySQLIndexListBE.Columns.StorageType, "");
+              (int)MySQLIndexListWrapper.Columns.StorageType, "");
           }
           else if (indexStorageTypeComboBox.SelectedItem.ToString() != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId,
-              (int)MySQLIndexListBE.Columns.StorageType, indexStorageTypeComboBox.SelectedItem.ToString());
+              (int)MySQLIndexListWrapper.Columns.StorageType, indexStorageTypeComboBox.SelectedItem.ToString());
           }
         }
       }
@@ -1214,18 +1231,8 @@ namespace MySQL.GUI.Workbench.Plugins
     private void tableOptChanged(object sender, EventArgs e)
     {
       // When a UI value changes, update the table options
-      if (!InitializingControls && !insideRefreshFormData)
-      {
-        insideRefreshFormData = true;
-        try
-        {
-          setTableOpt();
-        }
-        finally
-        {
-          insideRefreshFormData = false;
-        }
-      }
+      if (!InitializingControls)
+        setTableOpt();
     }
 
     private void nameTextBox_Leave(object sender, EventArgs e)
@@ -1357,14 +1364,12 @@ namespace MySQL.GUI.Workbench.Plugins
 
     void partParams_TextChanged(object sender, System.EventArgs e)
     {
-      if (!insideRefreshFormData)
-        tableEditorBE.set_partition_expression(partParams.Text);
+      tableEditorBE.set_partition_expression(partParams.Text);
     }
 
     void subpartParams_TextChanged(object sender, System.EventArgs e)
     {
-      if (!insideRefreshFormData)
-        tableEditorBE.set_subpartition_expression(subpartParams.Text);
+      tableEditorBE.set_subpartition_expression(subpartParams.Text);
     }
 
     private void partCount_Changed(object sender, EventArgs e)
@@ -1449,25 +1454,25 @@ namespace MySQL.GUI.Workbench.Plugins
             columnEnabledFkNodeControl, columnFkNodeControl,
             targetColumnFkNodeControl, tableEditorBE);
 
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.OnDelete, out text);
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnDelete, out text);
           if (String.IsNullOrEmpty(text))
             onDeleteActionComboBox.SelectedIndex = -1;
           else
             onDeleteActionComboBox.SelectedItem = text;
 
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.OnUpdate, out text);
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnUpdate, out text);
           if (String.IsNullOrEmpty(text))
             onUpdateActionComboBox.SelectedIndex = -1;
           else
             onUpdateActionComboBox.SelectedItem = text;
 
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.Comment, out text);
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Comment, out text);
           fkCommentText.Text = text;
 
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.Index, out text);
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Index, out text);
 
           int value= 0;
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.ModelOnly, out value);
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.ModelOnly, out value);
           fkModelOnlyCheck.Checked = value != 0;
           fkModelOnlyCheck.Enabled = true;
 
@@ -1512,7 +1517,7 @@ namespace MySQL.GUI.Workbench.Plugins
           GrtListNode listNode = node.Tag as GrtListNode;
 
           tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.ModelOnly,
+            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.ModelOnly,
             fkModelOnlyCheck.Checked ? 1 : 0);
         }
       }
@@ -1553,7 +1558,7 @@ namespace MySQL.GUI.Workbench.Plugins
           GrtListNode listNode = node.Tag as GrtListNode;
 
           tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.OnUpdate,
+            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnUpdate,
             onUpdateActionComboBox.SelectedItem.ToString());
         }
       }
@@ -1568,7 +1573,7 @@ namespace MySQL.GUI.Workbench.Plugins
           GrtListNode listNode = node.Tag as GrtListNode;
 
           tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.OnDelete,
+            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnDelete,
             onDeleteActionComboBox.SelectedItem.ToString());
         }
       }
@@ -1581,7 +1586,7 @@ namespace MySQL.GUI.Workbench.Plugins
         GrtListNode listNode = node.Tag as GrtListNode;
 
         tableEditorBE.get_fks().set_field(listNode.NodeId,
-          (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.Comment,
+          (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Comment,
           fkCommentText.Text);
       }
     }
@@ -1800,10 +1805,5 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #endregion
 
-    public override void EditorLostFocus()
-    {
-      tableEditorBE.commit_changes();
-    }
-
   }
 }
