@@ -62,7 +62,7 @@ db_mysql_CatalogRef DbMySQLDiffAlter::get_cat_from_file_or_tree(std::string file
 
   GError *file_error= NULL;
   char *sql_input_script= NULL;
-  size_t sql_input_script_length= 0;
+  gsize sql_input_script_length= 0;
   
   if(!g_file_get_contents(filename.c_str(), &sql_input_script, &sql_input_script_length, &file_error))
   {
@@ -143,6 +143,7 @@ std::string DbMySQLDiffAlter::generate_alter()
   options.set("RoutineFilterList", convert_string_vector_to_grt_list(manager_->get_grt(), routines));
   options.set("TriggerFilterList", convert_string_vector_to_grt_list(manager_->get_grt(), triggers));
   options.set("KeepOrder", grt::IntegerRef(1));
+  options.set("DBSettings", get_db_options());
 
   grt::StringListRef alter_list(manager_->get_grt());
   grt::ListRef<GrtNamedObject> alter_object_list(manager_->get_grt());
@@ -163,48 +164,46 @@ std::string DbMySQLDiffAlter::generate_alter()
   return *script;
 };
 
-boost::shared_ptr<DiffTreeBE> DbMySQLDiffAlter::init_diff_tree(const std::vector<std::string>& schemata, const grt::ValueRef &ext_cat, 
-                                              const grt::ValueRef &cat2,grt::StringListRef SchemaSkipList)
+boost::shared_ptr<DiffTreeBE> DbMySQLDiffAlter::init_diff_tree(const std::vector<std::string>& schemata,
+                                                               const grt::ValueRef &left, const grt::ValueRef &right,
+                                                               grt::StringListRef SchemaSkipList)
 {
-  grt::ValueRef left = ext_cat;
-  grt::ValueRef right = cat2;
-  std::string err;
-  db_mysql_CatalogRef left_cat, right_cat, right_cat_copy;
-
-  left_cat= db_mysql_CatalogRef::cast_from(left);
+  db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(manager_->get_grt()->get("/wb/rdbmsMgmt/rdbms/0"));
   std::string default_engine_name;
   grt::ValueRef default_engine = manager_->get_app_option("db.mysql.Table:tableEngine");
   if(grt::StringRef::can_wrap(default_engine))
     default_engine_name = grt::StringRef::cast_from(default_engine);
+  std::string err;
 
-  _left_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(manager_->get_grt(), left_cat));
+  db_mysql_CatalogRef right_cat_copy;
+
+  _left_catalog= db_mysql_CatalogRef::cast_from(left);
+  _left_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(_left_catalog));
+  bec::apply_user_datatypes(_left_cat_copy, rdbms);
   bec::CatalogHelper::apply_defaults(_left_cat_copy, default_engine_name);
 
-  right_cat= db_mysql_CatalogRef::cast_from(right);
-  right_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(manager_->get_grt(), right_cat));
-  bec::CatalogHelper::apply_defaults(right_cat_copy, default_engine_name);
-
   CatalogMap left_catalog_map;
   build_catalog_map(_left_cat_copy, left_catalog_map);
   update_all_old_names(_left_cat_copy, true, left_catalog_map);
 
+  _right_catalog= db_mysql_CatalogRef::cast_from(right);
+  right_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(_right_catalog));
+  bec::apply_user_datatypes(right_cat_copy, rdbms);
+  bec::CatalogHelper::apply_defaults(right_cat_copy, default_engine_name);
+
   CatalogMap right_catalog_map;
   build_catalog_map(right_cat_copy, right_catalog_map);
   update_all_old_names(right_cat_copy, true, right_catalog_map);
 
-  db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(manager_->get_grt()->get("/wb/rdbmsMgmt/rdbms/0"));
-
-  bec::apply_user_datatypes(right_cat_copy, rdbms);
-  bec::apply_user_datatypes(_left_cat_copy, rdbms);
-
+  // 2. diff with model
 
   grt::DbObjectMatchAlterOmf omf;
   omf.dontdiff_mask = 3;
   grt::NormalizedComparer comparer(manager_->get_grt(),get_db_options());
   comparer.init_omf(&omf);
-  _alter_change= diff_make(_left_cat_copy, right_cat_copy, &omf);
-  SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(manager_->get_grt()->get_module("DbMySQL"));
+  _alter_change= diff_make(right_cat_copy, _left_cat_copy, &omf);
 
+  SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(manager_->get_grt()->get_module("DbMySQL"));
   if (diffsql_module == NULL)
     throw DbMySQLDiffAlterException("error loading module DbMySQL");
 
@@ -214,22 +213,16 @@ boost::shared_ptr<DiffTreeBE> DbMySQLDiffAlter::init_diff_tree(const std::vector
   options.set("OutputObjectContainer", _alter_object_list);
   options.set("UseFilteredLists", grt::IntegerRef(0));
  
-  if(_alter_change && diffsql_module)
+  if (_alter_change && diffsql_module)
   {
-    diffsql_module->generateSQL(_left_cat_copy, options, _alter_change);
+    diffsql_module->generateSQL(_right_catalog, options, _alter_change);
   }
-/*
-  std::vector<std::string> schemata;
-  db_mysql_CatalogRef catalog= db_mysql_CatalogRef::cast_from(right_cat_copy);
-  grt::ListRef<db_mysql_Schema> sch(catalog->schemata());
-  for (size_t i= 0; i < sch.count(); i++)
-    schemata.push_back(sch.get(i)->name().c_str());
-*/
+
   // 3. build the tree
   std::map<DiffNode::ApplicationDirection,DiffNode::ApplicationDirection> apply_directions_map;
   apply_directions_map[DiffNode::ApplyToDb] = DiffNode::DontApply;
   apply_directions_map[DiffNode::DontApply] = DiffNode::ApplyToDb;
-  return _diff_tree = boost::shared_ptr<DiffTreeBE>(new ::DiffTreeBE(schemata, right_cat_copy, _left_cat_copy, _alter_change,apply_directions_map));
+  return _diff_tree = boost::shared_ptr<DiffTreeBE>(new ::DiffTreeBE(schemata, _left_cat_copy, right_cat_copy, _alter_change, apply_directions_map));
 
 }
 
