@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@ using namespace grt;
 #include "diff/changeobjects.h"
 #include "diff/changelistobjects.h"
 #include "grtdb/diff_dbobjectmatch.h"
-
+#include "grtdb/sync_profile.h"
 #include "grtsqlparser/sql_facade.h"
 #include "db.mysql/src/module_db_mysql.h"
 #include "interfaces/sqlgenerator.h"
@@ -301,6 +301,13 @@ db_mysql_CatalogRef DbMySQLScriptSync::get_model_catalog()
   return db_mysql_CatalogRef::cast_from(_manager->get_grt()->get("/wb/doc/physicalModels/0/catalog"));
 }
 
+
+void DbMySQLScriptSync::get_compared_catalogs(db_CatalogRef &left, db_CatalogRef &right)
+{
+  left = _org_cat;
+  right = _mod_cat_copy;
+}
+
 void DbMySQLScriptSync::set_option(const std::string& name, const std::string& value)
 {
   if(name.compare("InputFileName1") == 0)
@@ -387,7 +394,7 @@ db_mysql_CatalogRef DbMySQLScriptSync::get_cat_from_file_or_tree(std::string fil
 
   GError *file_error= NULL;
   char *sql_input_script= NULL;
-  size_t sql_input_script_length= 0;
+  gsize sql_input_script_length= 0;
   
   if (!g_file_get_contents(filename.c_str(), &sql_input_script, &sql_input_script_length, &file_error))
   {
@@ -406,8 +413,6 @@ db_mysql_CatalogRef DbMySQLScriptSync::get_cat_from_file_or_tree(std::string fil
 
 ValueRef DbMySQLScriptSync::sync_task(grt::GRT* grt, grt::StringRef)
 {
-  DbMySQLImpl *diffsql_module= _manager->get_grt()->find_native_module<DbMySQLImpl>("DbMySQL");
-
   std::string err;
 
   db_mysql_CatalogRef mod_cat= get_cat_from_file_or_tree(std::string(), err);
@@ -428,6 +433,21 @@ ValueRef DbMySQLScriptSync::sync_task(grt::GRT* grt, grt::StringRef)
   apply_user_datatypes(org_cat_copy, rdbms);
   apply_user_datatypes(mod_cat_copy, rdbms);
 
+  try
+  {
+    return generate_alter(org_cat, org_cat_copy, mod_cat_copy);
+  }
+  catch (std::exception &e)
+  {
+    return grt::StringRef(e.what());
+  }
+}
+
+
+grt::StringRef DbMySQLScriptSync::generate_alter(db_mysql_CatalogRef org_cat, db_mysql_CatalogRef org_cat_copy, db_mysql_CatalogRef mod_cat_copy)
+{
+  DbMySQLImpl *diffsql_module= _manager->get_grt()->find_native_module<DbMySQLImpl>("DbMySQL");
+
   grt::DbObjectMatchAlterOmf omf;
   omf.dontdiff_mask = 3;
   grt::NormalizedComparer normalizer(_manager->get_grt());
@@ -435,8 +455,8 @@ ValueRef DbMySQLScriptSync::sync_task(grt::GRT* grt, grt::StringRef)
   boost::shared_ptr<DiffChange> alter_change= diff_make(org_cat_copy, mod_cat_copy, &omf);
 
   // nothing changed
-  if(!alter_change)
-    return grt::StringRef("");  
+  if (!alter_change)
+    return grt::StringRef("");
 
   grt::DictRef options(_manager->get_grt());
   grt::StringListRef alter_list(_manager->get_grt());
@@ -451,101 +471,115 @@ ValueRef DbMySQLScriptSync::sync_task(grt::GRT* grt, grt::StringRef)
 
   int res= diffsql_module->makeSQLSyncScript(options, alter_list, alter_object_list);
   if (res)
-    return StringRef("SQL Script Export Module Returned Error");
+    throw std::runtime_error("SQL Script Export Module Returned Error");
 
-  grt::StringRef script= grt::StringRef::cast_from(options.get("OutputScript"));
-
-  g_file_set_contents(_output_filename.c_str(), script.c_str(), (gssize)strlen(script.c_str()), NULL);
-
-  return grt::StringRef("");
+  return grt::StringRef::cast_from(options.get("OutputScript"));
 }
 
-void DbMySQLScriptSync::update_model_old_names()
-{
-  CatalogMap cm;
-  //update_all_old_names(get_model_catalog(), false, cm);
 
+// Called once sync is finished, if there were no errors
+// This saves oldNames and sqlDefinition for views in syncProfile
+void DbMySQLScriptSync::save_sync_profile()
+{
   db_mysql_CatalogRef mod_cat = get_model_catalog();
   GrtObjectRef model_obj = mod_cat->owner();
-  if(_sync_profile_name.is_valid() && model_obj.is_valid() && workbench_physical_ModelRef::can_wrap(model_obj))
+  if (_sync_profile_name.is_valid() && model_obj.is_valid() && workbench_physical_ModelRef::can_wrap(model_obj))
   {
-      db_mgmt_SyncProfileRef profile(_manager->get_grt());
-      load_old_names(GrtObjectRef::cast_from(mod_cat), profile->lastKnownDBNames());
-      workbench_physical_ModelRef::cast_from(model_obj)->syncProfiles().set(_sync_profile_name,profile);
-      //TODO:update time
+    for (size_t i = 0; i < mod_cat->schemata().count(); i++)
+    {
+      db_SchemaRef schema(mod_cat->schemata()[i]);
+      log_info("Saving oldNames and other sync state info for %s::%s (catalog %s)\n", _sync_profile_name.c_str(), schema->name().c_str(), mod_cat.id().c_str());
+      db_mgmt_SyncProfileRef profile = bec::get_sync_profile(workbench_physical_ModelRef::cast_from(model_obj), _sync_profile_name, schema->name());
+      if (!profile.is_valid())
+        profile = bec::create_sync_profile(workbench_physical_ModelRef::cast_from(model_obj), _sync_profile_name, schema->name());
+      bec::update_sync_profile_from_schema(profile, schema);
+    }
   }
 }
 
-boost::shared_ptr<DiffTreeBE> DbMySQLScriptSync::init_diff_tree(const std::vector<std::string>& schemata, const ValueRef &ext_cat, 
-                                              const ValueRef &cat2,StringListRef SchemaSkipList)
+
+// Called when starting sync, to load previously saved oldNames and other info
+// for this specific connection
+void DbMySQLScriptSync::restore_sync_profile(db_CatalogRef catalog)
 {
-  std::string err;
+  GrtObjectRef model_obj = catalog->owner();
+  if (_sync_profile_name.is_valid() && model_obj.is_valid() && workbench_physical_ModelRef::can_wrap(model_obj))
+  {
+    for (size_t i = 0; i < catalog->schemata().count(); i++)
+    {
+      db_SchemaRef schema(catalog->schemata()[i]);
+      db_mgmt_SyncProfileRef profile = bec::get_sync_profile(workbench_physical_ModelRef::cast_from(model_obj), _sync_profile_name, schema->name());
+      if (profile.is_valid())
+      {
+        log_info("Restoring oldNames and other sync state info for %s::%s (catalog %s)\n", _sync_profile_name.c_str(), schema->name().c_str(), catalog.id().c_str());
+        bec::update_schema_from_sync_profile(schema, profile);
+      }
+      else
+        log_info("No sync profile found for %s::%s\n", _sync_profile_name.c_str(), schema->name().c_str());
+    }
+  }
+}
 
-  schemata_list.assign(schemata.begin(), schemata.end());
+
+void DbMySQLScriptSync::restore_overriden_names()
+{
+  db_mysql_CatalogRef mod_cat = get_model_catalog();
+  for (size_t i = 0; i < mod_cat->schemata().count(); i++)
+  {
+    db_SchemaRef schema(mod_cat->schemata()[i]);
+    std::string original_name = schema->customData().get_string("db.mysql.synchronize:originalName", schema->name());
+    std::string original_old_name = schema->customData().get_string("db.mysql.synchronize:originalOldName", schema->oldName());
+    schema->customData().remove("db.mysql.synchronize:originalName");
+    schema->customData().remove("db.mysql.synchronize:originalOldName");
+    schema->name(original_name);
+    schema->oldName(original_old_name);
+  }
+}
+
+boost::shared_ptr<DiffTreeBE> DbMySQLScriptSync::init_diff_tree(const std::vector<std::string>& schemata,
+                                                                const ValueRef &left, const ValueRef &right,
+                                                                StringListRef SchemaSkipList)
+{
   db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(_manager->get_grt()->get("/wb/rdbmsMgmt/rdbms/0"));
-
-  // 1. load db_Catalog-s
-  db_mysql_CatalogRef mod_cat;
-
-  if (!cat2.is_valid())
-  {
-    mod_cat= get_cat_from_file_or_tree(_input_filename2, err);
-    if (!err.empty())
-      throw DbMySQLScriptSyncException(err);
-  }
-  else
-  {
-    mod_cat= db_mysql_CatalogRef::cast_from(cat2);
-  }
-
-  GrtObjectRef model_obj = mod_cat->owner();
-  if(_sync_profile_name.is_valid() && model_obj.is_valid() && workbench_physical_ModelRef::can_wrap(model_obj))
-  {
-      grt::DictRef profiles = workbench_physical_ModelRef::cast_from(model_obj)->syncProfiles();
-      grt::ValueRef profile = profiles[_sync_profile_name];
-      if(profile.is_valid() && db_mgmt_SyncProfileRef::can_wrap(profile))
-          apply_old_names(mod_cat,db_mgmt_SyncProfileRef::cast_from(profile)->lastKnownDBNames());
-   }
-  _mod_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(mod_cat));
-  apply_user_datatypes(_mod_cat_copy, rdbms);
   std::string default_engine_name;
   grt::ValueRef default_engine = _manager->get_app_option("db.mysql.Table:tableEngine");
   if(grt::StringRef::can_wrap(default_engine))
     default_engine_name = grt::StringRef::cast_from(default_engine);
-  bec::CatalogHelper::apply_defaults(_mod_cat_copy,default_engine_name);
+  std::string err;
 
-  CatalogMap catalog_map;
-  build_catalog_map(_mod_cat_copy, catalog_map);
+  schemata_list.assign(schemata.begin(), schemata.end());
 
-  update_all_old_names(_mod_cat_copy, true, catalog_map);
+  // 1. load db_Catalog-s
+  db_mysql_CatalogRef left_catalog(db_mysql_CatalogRef::cast_from(left));
+  restore_sync_profile(left_catalog);
 
-  if (!ext_cat.is_valid())
+  _mod_cat_copy= db_mysql_CatalogRef::cast_from(grt::copy_object(left_catalog));
+  apply_user_datatypes(_mod_cat_copy, rdbms);
+  bec::CatalogHelper::apply_defaults(_mod_cat_copy, default_engine_name);
+
+  CatalogMap left_catalog_map;
+  build_catalog_map(_mod_cat_copy, left_catalog_map);
+  update_all_old_names(_mod_cat_copy, true, left_catalog_map);
+
+  _org_cat= db_mysql_CatalogRef::cast_from(right);
+  if(_org_cat.is_valid())
   {
-    _org_cat= get_cat_from_file_or_tree(_input_filename1, err);
-    if (!err.empty())
-      throw DbMySQLScriptSyncException(err);
+      apply_user_datatypes(_org_cat, rdbms);
+      bec::CatalogHelper::apply_defaults(_org_cat, default_engine_name);
+
+      CatalogMap right_catalog_map;
+      build_catalog_map(_org_cat, right_catalog_map);
+      update_all_old_names(_org_cat, true, right_catalog_map);
   }
-  else
-  {
-    _org_cat= db_mysql_CatalogRef::cast_from(ext_cat);
-  }
-
-  
-  apply_user_datatypes(_org_cat, rdbms);
-  bec::CatalogHelper::apply_defaults(_org_cat,default_engine_name);
-
-  CatalogMap catalog_map2;
-  build_catalog_map(_org_cat, catalog_map2);
-  update_all_old_names(_org_cat, true, catalog_map2);
-
   // 2. diff with model
 
   for(size_t i= 0; i < SchemaSkipList.count(); i++)
   {
     StringRef schema_name = SchemaSkipList.get(i);
-    for(size_t schema_count = 0; schema_count < _org_cat->schemata().count(); schema_count++)
-      while((schema_count < _org_cat->schemata().count())&& (_org_cat->schemata().get(schema_count)->name() == schema_name))
-        _org_cat->schemata().remove(schema_count);
+    if(_org_cat.is_valid())
+        for(size_t schema_count = 0; schema_count < _org_cat->schemata().count(); schema_count++)
+            while((schema_count < _org_cat->schemata().count())&& (_org_cat->schemata().get(schema_count)->name() == schema_name))
+                _org_cat->schemata().remove(schema_count);
 
     for(size_t schema_count = 0; schema_count < _mod_cat_copy->schemata().count(); schema_count++)
       while((schema_count < _mod_cat_copy->schemata().count())&& (_mod_cat_copy->schemata().get(schema_count)->name() == schema_name))
@@ -554,11 +588,12 @@ boost::shared_ptr<DiffTreeBE> DbMySQLScriptSync::init_diff_tree(const std::vecto
 
   grt::DbObjectMatchAlterOmf omf;
   omf.dontdiff_mask = 3;
-  grt::NormalizedComparer comparer (_manager->get_grt(),get_db_options());
+  grt::NormalizedComparer comparer(_manager->get_grt(),get_db_options());
   comparer.init_omf(&omf);
   _alter_change= diff_make(_org_cat, _mod_cat_copy, &omf);
 
   DbMySQLImpl *diffsql_module= _manager->get_grt()->find_native_module<DbMySQLImpl>("DbMySQL");
+
   grt::DictRef options(_manager->get_grt());
   options.set("DBSettings", get_db_options());
   options.set("OutputContainer", _alter_list);
@@ -566,7 +601,7 @@ boost::shared_ptr<DiffTreeBE> DbMySQLScriptSync::init_diff_tree(const std::vecto
   options.set("UseFilteredLists", grt::IntegerRef(0));
 //  options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
 
-  if(_alter_change && diffsql_module)
+  if (_alter_change && diffsql_module)
   {
     //_alter_change->dump_log(0);
     diffsql_module->generateSQL(_org_cat, options, _alter_change);
@@ -574,8 +609,7 @@ boost::shared_ptr<DiffTreeBE> DbMySQLScriptSync::init_diff_tree(const std::vecto
   }
 
   // 3. build the tree
-  return _diff_tree= boost::shared_ptr<DiffTreeBE>(new ::DiffTreeBE(
-    schemata, _mod_cat_copy, _org_cat, _alter_change));
+  return _diff_tree= boost::shared_ptr<DiffTreeBE>(new ::DiffTreeBE(schemata, _mod_cat_copy, _org_cat, _alter_change));
 }
 
 std::string DbMySQLScriptSync::get_sql_for_object(GrtNamedObjectRef obj)
@@ -619,27 +653,27 @@ std::string DbMySQLScriptSync::generate_diff_tree_script()
 
     if(db_mysql_SchemaRef::can_wrap(v))
     {
-      db_mysql_SchemaRef schema= db_mysql_SchemaRef::cast_from(v);
+      //db_mysql_SchemaRef schema= db_mysql_SchemaRef::cast_from(v);
       schemata.push_back(name);
     }
     else if(db_mysql_TableRef::can_wrap(v))
     {
-      db_mysql_TableRef table= db_mysql_TableRef::cast_from(v);
+      //db_mysql_TableRef table= db_mysql_TableRef::cast_from(v);
       tables.push_back(name);
     }
     else if(db_mysql_ViewRef::can_wrap(v))
     {
-      db_mysql_ViewRef view= db_mysql_ViewRef::cast_from(v);
+      //db_mysql_ViewRef view= db_mysql_ViewRef::cast_from(v);
       views.push_back(name);
     }
     else if(db_mysql_RoutineRef::can_wrap(v))
     {
-      db_mysql_RoutineRef routine= db_mysql_RoutineRef::cast_from(v);
+      //db_mysql_RoutineRef routine= db_mysql_RoutineRef::cast_from(v);
       routines.push_back(name);
     }
     else if(db_mysql_TriggerRef::can_wrap(v))
     {
-      db_mysql_TriggerRef trigger= db_mysql_TriggerRef::cast_from(v);
+      //db_mysql_TriggerRef trigger= db_mysql_TriggerRef::cast_from(v);
       triggers.push_back(name);
     }
   }
@@ -1222,7 +1256,8 @@ void DbMySQLScriptSync::apply_changes_to_model()
   applier.set_case_sensitive(get_db_options().get_int("CaseSensitive", 1));
 
   applier.build_obj_mapping(diff_mod_cat, mod_cat);
-  applier.build_obj_mapping(diff_db_cat, mod_cat);
+  if(diff_db_cat.is_valid())
+    applier.build_obj_mapping(diff_db_cat, mod_cat);
   applier.consolidate_mapping();
   // Update changes to the model
   applier.apply_node_to_model(rootnode);
@@ -1251,38 +1286,3 @@ std::string DbMySQLScriptSync::get_col_name(const size_t col_id)
   };
   return "No Column Name Defined";
 };
-
-void load_old_name(grt::DictRef dict, const GrtObjectRef obj)
-{
-    if (!GrtNamedObjectRef::can_wrap(obj) )
-        return;
-    GrtNamedObjectRef named_obj = GrtNamedObjectRef::cast_from(obj);
-    if(!named_obj.is_valid())
-        return;
-  // log_debug2("Saved Name %s for object with id %s\n", named_obj->name().c_str(), named_obj.id().c_str());
-    dict.set(named_obj.id(),named_obj->name());
-};
-
-void load_old_names(GrtObjectRef obj, DictRef result)
-{
-    iterate_object(obj,boost::bind(load_old_name, result, _1));
-};
-
-void apply_old_name(const grt::DictRef& dict, grt::ObjectRef obj)
-{
-    if (!GrtNamedObjectRef::can_wrap(obj) )
-        return;
-    GrtNamedObjectRef named_obj = GrtNamedObjectRef::cast_from(obj);
-    if(!named_obj.is_valid())
-        return;
-    grt::ValueRef new_name = dict[named_obj.id()];
-    if (!new_name.is_valid() || !grt::StringRef::can_wrap(new_name))
-        return;
-    log_debug2("Reset OldName for object named %s with id %s from %s to %s\n", named_obj->name().c_str(), named_obj.id().c_str(), named_obj->oldName().c_str(), grt::StringRef::cast_from(new_name).c_str());
-    named_obj->oldName(grt::StringRef::cast_from(new_name));
-}
-
-void apply_old_names(GrtObjectRef obj, DictRef profile)
-{
-    iterate_object(obj,boost::bind(apply_old_name,profile,_1));
-};
