@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -183,19 +183,19 @@ DiffNodeController::DiffNodeController(const std::map<DiffNode::ApplicationDirec
 
 };
 
-void DiffNodeController::set_next_apply_direction(DiffNode *node)const
+void DiffNodeController::set_next_apply_direction(DiffNode *node) const
 {
   std::map<DiffNode::ApplicationDirection,DiffNode::ApplicationDirection>::const_iterator newdir = _directions_map.find(node->apply_direction());
   node->apply_direction(newdir == _directions_map.end()?DiffNode::DontApply:newdir->second);
 };
 
-void DiffNodeController::set_apply_direction(DiffNode *node, DiffNode::ApplicationDirection dir, bool recursive)const
+void DiffNodeController::set_apply_direction(DiffNode *node, DiffNode::ApplicationDirection dir, bool recursive) const
 {
   node->apply_direction(dir);
 
   if (recursive)
     for(DiffNode::DiffNodeVector::const_iterator It = node->get_children_begin(); It != node->get_children_end(); ++It)
-      set_apply_direction(*It,dir,recursive);
+      set_apply_direction(*It, dir, recursive);
 };
 
 
@@ -441,9 +441,9 @@ void DiffTreeBE::fill_tree(DiffNode *root, db_mysql_CatalogRef model_catalog, co
 
 bool is_node_object(const grt::ValueRef v)
 {
-    return  (db_SchemaRef::can_wrap(v) || db_TableRef::can_wrap(v) 
-        || db_ViewRef::can_wrap(v) || db_RoutineRef::can_wrap(v)         
-        || db_TriggerRef::can_wrap(v));
+  return (db_SchemaRef::can_wrap(v) || db_TableRef::can_wrap(v)
+          || db_ViewRef::can_wrap(v) || db_RoutineRef::can_wrap(v)
+          || db_TriggerRef::can_wrap(v));
 }
 
 void DiffTreeBE::update_tree_with_changes(const boost::shared_ptr<DiffChange> diffchange)
@@ -510,11 +510,12 @@ void DiffTreeBE::update_tree_with_changes(const boost::shared_ptr<DiffChange> di
             const grt::ListItemOrderChange *oc= 
                 static_cast<const grt::ListItemOrderChange *>(diffchange.get());
             grt::ValueRef v(oc->get_old_value());
-            if(db_ColumnRef::can_wrap(v) || db_IndexColumnRef::can_wrap(v))
+            // order changes matter for colums and indexcolumn... in anything else, it doesn't
+            if (db_ColumnRef::can_wrap(v) || db_IndexColumnRef::can_wrap(v))
                 obj_change_detected = true;
             // the list item order change action can have a nested
             // object-modified action
-            else if(oc->get_subchange())
+            else if (oc->get_subchange())
                 update_tree_with_changes(oc->get_subchange());
         }
         break;
@@ -527,19 +528,26 @@ void DiffTreeBE::update_tree_with_changes(const boost::shared_ptr<DiffChange> di
 //TODO check how new DiffNode being deleted
 void DiffTreeBE::apply_change(GrtObjectRef obj, boost::shared_ptr<DiffChange> change)
 {
-    DiffNode *obj_node = _root->find_node_for_object(obj);
-    if(obj_node)
-        obj_node->set_modified_and_update_dir(true, change);
-    else
-    {
-        DiffNode *owner_node = _root->find_node_for_object(obj->owner() );
-        if(!owner_node)
-            owner_node = _root;
+  DiffNode *obj_node = _root->find_node_for_object(obj);
 
-        DiffNode * newnode = new DiffNode(GrtNamedObjectRef(), GrtNamedObjectRef::cast_from(obj), false, change);
-        owner_node->append(newnode);
-    }
-};
+#if 1
+  // Uncomment to see what are the differences detected for a given object
+  if (obj->name() == "individual_contribution")
+    change->dump_log(0);
+#endif
+
+  if(obj_node)
+    obj_node->set_modified_and_update_dir(true, change);
+  else
+  {
+    DiffNode *owner_node = _root->find_node_for_object(obj->owner() );
+    if(!owner_node)
+      owner_node = _root;
+
+    DiffNode * newnode = new DiffNode(GrtNamedObjectRef(), GrtNamedObjectRef::cast_from(obj), false, change);
+    owner_node->append(newnode);
+  }
+}
 
 DiffTreeBE::DiffTreeBE(const std::vector<std::string>& schemata,
                        db_mysql_CatalogRef model_catalogRef, 
@@ -552,7 +560,8 @@ _node_controller(controller)
   _schemata.assign(schemata.begin(), schemata.end());
   
   CatalogMap map;
-  build_catalog_map(external_catalog, map);
+  if(external_catalog.is_valid())
+    build_catalog_map(external_catalog, map);
   fill_tree(_root, model_catalogRef, map, false);
   update_tree_with_changes(diffchange);
 
@@ -561,6 +570,7 @@ _node_controller(controller)
   change_forward_icon= bec::IconManager::get_instance()->get_icon_id("change_forward.png");
   change_ignore_icon= bec::IconManager::get_instance()->get_icon_id("change_ignore.png");
   alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_thin.png");
+  create_alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_create.png");
   drop_alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_drop.png");
 }
 
@@ -622,7 +632,16 @@ bool DiffTreeBE::get_field(const bec::NodeId &node_id, int column, std::string &
   {
   case ModelObjectName:
     if(node->get_model_part().is_valid_object())
+    {
       value= node->get_model_part().get_name();
+      if (db_SchemaRef::can_wrap(node->get_model_part().get_object()))
+      {
+        // put the actual name of the schema if it's being synchronized with a differently named schema
+        std::string real_name = db_SchemaRef::cast_from(node->get_model_part().get_object())->customData().get_string("db.mysql.synchronize:originalName");
+        if (!real_name.empty())
+          value.append(" ("+real_name+")");
+      }
+    }
     else
       value= "N/A";
     break;
@@ -663,27 +682,86 @@ bec::IconId DiffTreeBE::get_field_icon(const bec::NodeId &node_id, int column, I
 
   switch(column)
   {
-  case ModelChanged:
-    return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
-  case ModelObjectName:
-    return object_icon;
-  case ApplyDirection:
-    switch(node->get_application_direction())
-    {
-    case DiffNode::ApplyToModel:
-      return change_backward_icon;
-    case DiffNode::ApplyToDb:
-      return change_forward_icon;
-    case DiffNode::DontApply:
-      return change_ignore_icon;
-    case DiffNode::CantApply:
-      return change_nothing_icon;
-    }
-    return -1;
-  case DbObjectName:
-    return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
-  default:
-    return -1;
+    case ModelChanged:
+      if (node->is_modified_recursive())
+      {
+        bool model_valid = node->get_model_part().is_valid_object();
+        bool db_valid = node->get_db_part().is_valid_object();
+        DiffNode::ApplicationDirection dir = node->get_application_direction();
+
+        if (model_valid && !db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToModel)
+            return drop_alert_icon;
+          else
+            return alert_icon;
+        }
+        else if (!model_valid && db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return create_alert_icon;
+          else
+            return alert_icon;
+        }
+        else
+          return alert_icon;
+      }
+      else
+        return 0;
+      //return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
+    case ModelObjectName:
+      return object_icon;
+    case ApplyDirection:
+      if (node->is_modified())
+      {
+        switch(node->get_application_direction())
+        {
+          case DiffNode::ApplyToModel:
+            return change_backward_icon;
+          case DiffNode::ApplyToDb:
+            return change_forward_icon;
+          case DiffNode::DontApply:
+            return change_ignore_icon;
+          case DiffNode::CantApply:
+            return change_nothing_icon;
+        }
+      }
+      else
+        return change_nothing_icon;
+      return -1;
+    case DbObjectName:
+      if (node->is_modified())
+      {
+        bool model_valid = node->get_model_part().is_valid_object();
+        bool db_valid = node->get_db_part().is_valid_object();
+        DiffNode::ApplicationDirection dir = node->get_application_direction();
+
+        if (model_valid && !db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return create_alert_icon;
+          else
+            return alert_icon;
+        }
+        else if (!model_valid && db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return drop_alert_icon;
+          else
+            return alert_icon;
+        }
+        else
+          return alert_icon;
+        //return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
+      }
+      else
+        return 0;
+    default:
+      return -1;
   }
 }
 
@@ -701,6 +779,13 @@ void DiffTreeBE::set_apply_direction(const bec::NodeId &node_id, DiffNode::Appli
     _node_controller.set_apply_direction(node, dir, recursive);
 }
 
+DiffNode::ApplicationDirection DiffTreeBE::get_apply_direction(const bec::NodeId &node_id)
+{
+  DiffNode *node= dynamic_cast<DiffNode *>(get_node_with_id(node_id));
+  if (node)
+    return node->get_application_direction();
+  return DiffNode::CantApply;
+}
 
 void DiffTreeBE::get_object_list_for_script(std::vector<grt::ValueRef>& vec) const
 {
