@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@ from workbench.utils import server_version_str2tuple
 
 pysource = {}
 pysource['engine-list'] = "grt.root.wb.options.options[\"@db.mysql.Table:tableEngine/Items\"]"
-multi_separator = ';'
+multi_separator = r'\n'
 
 #-------------------------------------------------------------------------------
 def ver_cmp(v1, v2):
@@ -305,7 +305,8 @@ class ApplyWizard(Form):
                     self.accept_action(text_from_box) # call action passed from caller(client)
                     App.get().set_status_text("Configuration File Saved.")
                 except InvalidPasswordError, exc:
-                    r = Utilities.show_error("Could not Save Configuration File", "There was a permission error while saving the configuration file: %s\nClick Retry to enter the password again.",
+                    r = Utilities.show_error("Could not Save Configuration File",
+                                             "Invalid admin password while saving the configuration file.\nClick Retry to enter the password again.",
                                   "Retry", "Cancel", "")
                     if r == mforms.ResultOk:
                         continue
@@ -539,7 +540,7 @@ class WbAdminConfigFileBE(object):
         return (option_type, option_default)
 
     #---------------------------------------------------------------------------
-    def open_configuration_file(self, file_name):
+    def open_configuration_file(self, file_name, warn_missing = True):
         self.file_name = file_name
 
         self.needs_root_for_file_read = False
@@ -555,13 +556,16 @@ class WbAdminConfigFileBE(object):
         exception = None
         try:
             content = self.read_mysql_cfg_file(file_name)
-        except IOError,e:
+        except (OSError, IOError), e:
             if e.errno != errno.ENOENT:
                 exception = e
             else:
-                Utilities.show_warning("Read Configuration File",
-                                       "Configuration file '%s' can not be found. New file will be created on apply of changes."%file_name,
-              "OK", "", "")
+                if warn_missing:
+                    Utilities.show_warning("Read Configuration File",
+                                           "Configuration file '%s' can not be found. New file will be created when changes are applied."%file_name,
+                                          "OK", "", "")
+                else:
+                    return
 
         self.parse_file_contents(content)
 
@@ -622,15 +626,15 @@ class WbAdminConfigFileBE(object):
         # split the path for / and \\
         directory, filename = splitpath(self.file_name)
 
-        try:
-            helper.check_dir_writable(directory)
-            
+        if helper.check_dir_writable(directory):
             password = None
-        except Exception:
+            as_user = Users.CURRENT
+        else:
             password = self.ctrl_be.password_handler.get_password_for("file")
+            as_user = Users.ADMIN
 
         try:
-            helper.set_file_content_and_backup(self.file_name, user_modified_file_content, ".wba.bak", password is not None, password)
+            helper.set_file_content_and_backup(self.file_name, user_modified_file_content, ".wba.bak", as_user, password)
         except InvalidPasswordError, err:
             self.ctrl_be.password_handler.reset_password_for("file")
             raise err
@@ -732,6 +736,11 @@ class WbAdminConfigFileBE(object):
             self.sections = sorted(self.sections, lambda x,y: cmp(x[0], y[0]))
 
         section = self.server_profile.config_file_section
+
+        # Sets a default section in case it is empty
+        if section == "":
+            section = "mysqld"
+
         # Check if the wanted section is in the file, if not, add it
         if self.server_profile.admin_enabled and not any(_section == section for _line, _section in self.sections):
             Utilities.show_warning("Read Configuration File",
