@@ -26,12 +26,12 @@ import StringIO
 import wb_admin_utils
 from workbench.utils import server_version_str2tuple
 from wb_admin_ssh import SSHDownException
-from workbench.db_utils import MySQLConnection, MySQLError, QueryError
+from workbench.db_utils import MySQLConnection, MySQLError, QueryError, strip_password
 
 from wb_common import OperationCancelledError, Users, PermissionDeniedError, InvalidPasswordError
 
 from wb_server_control import PasswordHandler, ServerControlShell, ServerControlWMI
-from wb_server_management import ServerManagementHelper, SSH
+from wb_server_management import ServerManagementHelper, SSH, wbaOS
 
 import mforms
 
@@ -130,7 +130,7 @@ class SQLQueryExecutor(object):
         self.mtx.acquire()
         try:
             if self.is_connected():
-                log_debug(_this_file, "Executing query %s\n" % query)
+                log_debug(_this_file, "Executing query %s\n" % strip_password(query))
                 result = self.dbconn.executeQuery(query)
         finally:
             self.mtx.release()
@@ -141,7 +141,7 @@ class SQLQueryExecutor(object):
         self.mtx.acquire()
         try:
             if self.is_connected():
-                log_debug(_this_file, "Executing query %s\n" % query)
+                log_debug(_this_file, "Executing statement %s\n" % strip_password(query))
                 result = self.dbconn.execute(query)
         finally:
             self.mtx.release()
@@ -329,9 +329,6 @@ def apply_filters(raw_text, filters, dbg = None):
     dbg("Filters execution code = " + str(code) + "\n===================\n")
     return (out, code) # return last code from filters
 
-def run_filter_debugger(loginInfo, serverInfo):
-    wb_admin_utils.run_filter_debugger(loginInfo, serverInfo, build_filters, apply_filters)
-
 
 #===============================================================================
 
@@ -345,8 +342,7 @@ class WbAdminControl(object):
     raw_version = "unknown"
     server_version = "unknown"
 
-    def __init__(self, server_profile,
-                            connect_sql=True):
+    def __init__(self, server_profile, connect_sql=True):
 
         self.server_control = None
         self.events   = EventManager()
@@ -372,57 +368,64 @@ class WbAdminControl(object):
         self.server_status = {}
         self.variables_pool = wb_variables.WBAVariables(self)
 
+        self.server_variables = {}
+
+
     def get_variables_pool(self):
         return self.variables_pool
 
-    def init(self):
+    @property
+    def admin_access_available(self):
+        return self.server_control is not None
+
+    def acquire_admin_access(self):
+        """Make sure we have access to the instance for admin (for config file, start/stop etc)"""
+        if self.server_control or not self.server_profile.admin_enabled:
+            return
         if self.server_profile.uses_ssh:
             try:
                 self.ssh = SSH(self.server_profile, self.password_handler)
             except OperationCancelledError, e:
                 self.ssh = None
-                if not self.sql_enabled:
-                    raise OperationCancelledError("SSH connection cancelled")
-                else:
-                    if mforms.Utilities.show_warning('SSH connection cancelled',
-                            "Would you like to continue with a database connection only?\nSome features will be disabled.",
-                            "Continue", "Cancel", "") != mforms.ResultOk:
-                        raise OperationCancelledError("SSH connection cancelled")
+                raise OperationCancelledError("SSH connection cancelled")
 
             except SSHDownException, e:
                 self.ssh = None
                 if self.sql_enabled:
                     if mforms.Utilities.show_warning('SSH connection failed',
-                            "Check if the SSH server is up on the remote side.\nYou may continue anyway, but some functionality will be unavailable",
-                            "Continue", "Cancel", "") != mforms.ResultOk:
+                                                     "Check if the SSH server is up on the remote side.\nYou may continue anyway, but some functionality will be unavailable",
+                                                     "Continue", "Cancel", "") != mforms.ResultOk:
                         raise OperationCancelledError("Could not connect to SSH server")
                 else:
                     mforms.Utilities.show_warning('SSH connection failed',
-                            "Check if the SSH server is up on the remote side.", "OK", "", "")
+                                                  "Check if the SSH server is up on the remote side.", "OK", "", "")
         else:
             self.ssh = None
-
         # init server management helper (for doing remote file operations, process management etc)
         self.server_helper = ServerManagementHelper(self.server_profile, self.ssh)
-
         if self.server_helper:
             self.server_profile.expanded_config_file_path = self.server_helper.shell.expand_path_variables(self.server_profile.config_file_path)
-
-        # init server control instance appropriate for what we're connecting to
-        if not self.server_profile.admin_enabled:
-            self.server_control = None
+        # detect the exact type of OS we're managing
+        os_info = self.detect_operating_system_version()
+        if os_info:
+            os_type, os_name, os_variant, os_version = os_info
+            log_info(_this_file, "Target OS detection returned: os_type=%s, os_name=%s, os_variant=%s, os_version=%s\n" % (os_type, os_name, os_variant, os_version))
+            self.server_profile.set_os_version_info(os_type, os_name, os_variant, os_version)
         else:
-            if self.server_profile.uses_wmi:
-                self.server_control = ServerControlWMI(self.server_profile, self.server_helper, self.password_handler)
-                self.server_control.set_filter_functions(build_filters, apply_filters)
-            elif self.server_profile.uses_ssh or self.server_profile.is_local:
-                self.server_control = ServerControlShell(self.server_profile, self.server_helper, self.password_handler)
-                self.server_control.set_filter_functions(build_filters, apply_filters)
-            else:
-                log_error(_this_file, """Unknown management method selected. Server Profile is possibly inconsistent
+            log_warning(_this_file, "Could not detect target OS details\n")
+        # init server control instance appropriate for what we're connecting to
+        if self.server_profile.uses_wmi:
+            self.server_control = ServerControlWMI(self.server_profile, self.server_helper, self.password_handler)
+            self.server_control.set_filter_functions(build_filters, apply_filters)
+        elif self.server_profile.uses_ssh or self.server_profile.is_local:
+            self.server_control = ServerControlShell(self.server_profile, self.server_helper, self.password_handler)
+            self.server_control.set_filter_functions(build_filters, apply_filters)
+        else:
+            log_error(_this_file, """Unknown management method selected. Server Profile is possibly inconsistent
 uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_profile.uses_wmi))
-                raise Exception("Unknown management method selected. Server Profile is possibly inconsistent")
+            raise Exception("Unknown management method selected. Server Profile is possibly inconsistent")
 
+    def init(self):
         # connect SQL
         while self.sql_enabled:
             try:
@@ -556,16 +559,14 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
 
 
     #---------------------------------------------------------------------------
-    def get_new_sql_connection(self, interactive):
+    def get_new_sql_connection(self):
         connection = None
         try:
             connection = MySQLConnection(self.server_profile.db_connection_params, self.sql_status_callback)
             connection.connect()
         except MySQLError, err:
-            print "Error creating SQL connection for monitoring: %r" % err
+            log_error(_this_file, "Error creating SQL connection for monitoring: %r\n" % err)
             connection = None
-            if interactive:
-                mforms.Utilities.show_error("Connection Error", "Error connecting to MySQL: %s\nMySQL stat graphs may be disabled." % err, "OK", "", "")
             return None
 
         return connection
@@ -635,114 +636,126 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
 
             self.sql = SQLQueryExecutor(connection)
 
-        if self.is_sql_connected():
-            # perform some server capabilities checking
+            if self.is_sql_connected():
+                # perform some server capabilities checking
 
-            # check version
-            result = self.exec_query("select version() as version")
-            if result and result.nextRow():
-                self.server_version= result.stringByName("version")
-                self.raw_version = self.server_version
-                if self.server_profile.server_version != self.server_version: # Update profile version with live data from server
-                    log_info(_this_file, '%s.connect_sql(): The server version stored in the server instance profile was "%s". '
-                                'Changed to the version reported by the server: "%s"\n' % (self.__class__.__name__,
-                                self.server_profile.server_version, self.server_version) )
-                    self.server_profile.server_version = self.server_version
-                self.get_server_version()  # This will convert self.server_version into a version tuple
-
-            if self.server_version >= (5, 1, 5):
-                # The command to retrieve plugins was 'SHOW PLUGIN' for v. [5.1.5, 5.1.9)
-                # and was changed to 'SHOW PLUGINS' from MySQL Server v. 5.1.9 onwards:
-                plugin_var = 'PLUGIN' if self.server_version < (5, 1, 9) else 'PLUGINS'
-                result = self.exec_query('SHOW %s' % plugin_var)
-                # check whether Windows authentication plugin is available
+                self.server_variables = {}
+                result = self.exec_query("SHOW VARIABLES")
                 while result and result.nextRow():
-                    name = result.stringByName("Name")
-                    status = result.stringByName("Status")
-                    plugin_type = result.stringByName("Type")
-                    if status == "ACTIVE":
-                        self.server_active_plugins.add((name, plugin_type))
+                    self.server_variables[result.stringByName("Variable_name")] = result.stringByName("Value")
 
-            opts = get_config_options()
-            config_section = self.server_profile.config_file_section or 'mysqld'
+                # check version
+                result = self.exec_query("select version() as version")
+                if result and result.nextRow():
+                    self.server_version= result.stringByName("version")
+                    self.raw_version = self.server_version
+                    if self.server_profile.server_version != self.server_version: # Update profile version with live data from server
+                        log_info(_this_file, '%s.connect_sql(): The server version stored in the server instance profile was "%s". '
+                                    'Changed to the version reported by the server: "%s"\n' % (self.__class__.__name__,
+                                    self.server_profile.server_version, self.server_version) )
+                        self.server_profile.server_version = self.server_version
+                    self.get_server_version()  # This will convert self.server_version into a version tuple
 
-            request_save_profile = False
+                if self.server_version >= (5, 1, 5):
+                    # The command to retrieve plugins was 'SHOW PLUGIN' for v. [5.1.5, 5.1.9)
+                    # and was changed to 'SHOW PLUGINS' from MySQL Server v. 5.1.9 onwards:
+                    plugin_var = 'PLUGIN' if self.server_version < (5, 1, 9) else 'PLUGINS'
+                    result = self.exec_query('SHOW %s' % plugin_var)
+                    # check whether Windows authentication plugin is available
+                    while result and result.nextRow():
+                        name = result.stringByName("Name")
+                        status = result.stringByName("Status")
+                        plugin_type = result.stringByName("Type")
+                        if status == "ACTIVE":
+                            self.server_active_plugins.add((name, plugin_type))
 
-            hostname = self.get_server_variable('hostname', '')
-            if not hostname and self.server_profile.is_local:
-                hostname = socket.gethostname()
+                opts = get_config_options()
+                config_section = self.server_profile.config_file_section or 'mysqld'
 
-            datadir = self.get_server_variable('datadir', '')
-            if datadir and self.server_profile.datadir != datadir:
-                self.server_profile.datadir = datadir
-                request_save_profile = True
+                request_save_profile = False
 
-            basedir = self.get_server_variable('basedir', '')
-            if basedir and self.server_profile.basedir != basedir:
-                self.server_profile.basedir = basedir
-                request_save_profile = True
+                hostname = self.server_variables.get('hostname', '')
+                if not hostname and self.server_profile.is_local:
+                    hostname = socket.gethostname()
 
-            try:
-                general_log_enabled = self.get_server_variable('general_log') in ('ON', '1')
-                if self.server_profile.general_log_enabled != general_log_enabled:
-                    self.server_profile.general_log_enabled = general_log_enabled
-                    request_save_profile = True
-            except ValueError:
-                pass
-
-            try:
-                slow_log_enabled = self.get_server_variable('log_slow_queries') in ('ON', '1')
-                if self.server_profile.slow_log_enabled != slow_log_enabled:
-                    self.server_profile.slow_log_enabled = slow_log_enabled
-                    request_save_profile = True
-            except ValueError:
-                pass
-
-            if self.server_version < (5, 1, 29):
-                general_log_file_path = opts[config_section].get('log', '').strip(' "') if opts.has_key(config_section) else ''
-                if self.server_profile.general_log_file_path != general_log_file_path:
-                    self.server_profile.general_log_file_path = general_log_file_path or os.path.join(self.server_profile.datadir, hostname + '.log')
+                datadir = self.server_variables.get('datadir', '')
+                if datadir and self.server_profile.datadir != datadir:
+                    self.server_profile.datadir = datadir
                     request_save_profile = True
 
-                slow_query_log_file = opts[config_section].get('log-slow-queries', '').strip(' "') if opts.has_key(config_section) else ''
-                if self.server_profile.slow_log_file_path != slow_query_log_file:
-                    self.server_profile.slow_log_file_path = slow_query_log_file or os.path.join(self.server_profile.datadir, hostname + '.slow')
+                basedir = self.server_variables.get('basedir', '')
+                if basedir and self.server_profile.basedir != basedir:
+                    self.server_profile.basedir = basedir
                     request_save_profile = True
 
-                error_log_file_path = opts[config_section].get('log-error', '').strip(' "') if opts.has_key(config_section) else ''
-                if self.server_profile.error_log_file_path != error_log_file_path:
-                    self.server_profile.error_log_file_path = error_log_file_path or os.path.join(self.server_profile.datadir, hostname + '.err')
-                    request_save_profile = True
+                try:
+                    general_log_enabled = self.server_variables.get('general_log') in ('ON', '1')
+                    if self.server_profile.general_log_enabled != general_log_enabled:
+                        self.server_profile.general_log_enabled = general_log_enabled
+                        request_save_profile = True
+                except ValueError:
+                    pass
 
+                try:
+                    if self.server_version < (5, 1, 29):
+                        slow_query_var = 'log_slow_queries'
+                    else:
+                        slow_query_var = 'slow_query_log'
+                        
+                    slow_log_enabled = self.server_variables.get(slow_query_var) in ('ON', '1')
+                    if self.server_profile.slow_log_enabled != slow_log_enabled:
+                        self.server_profile.slow_log_enabled = slow_log_enabled
+                        request_save_profile = True
+                except ValueError:
+                    pass
+
+                if self.server_version < (5, 1, 29):
+                    general_log_file_path = opts[config_section].get('log', '').strip(' "') if opts.has_key(config_section) else ''
+                    if self.server_profile.general_log_file_path != general_log_file_path:
+                        self.server_profile.general_log_file_path = general_log_file_path or os.path.join(self.server_profile.datadir, hostname + '.log')
+                        request_save_profile = True
+
+                    slow_query_log_file = opts[config_section].get('log-slow-queries', '').strip(' "') if opts.has_key(config_section) else ''
+                    if self.server_profile.slow_log_file_path != slow_query_log_file:
+                        self.server_profile.slow_log_file_path = slow_query_log_file or os.path.join(self.server_profile.datadir, hostname + '.slow')
+                        request_save_profile = True
+
+                    error_log_file_path = opts[config_section].get('log-error', '').strip(' "') if opts.has_key(config_section) else ''
+                    if self.server_profile.error_log_file_path != error_log_file_path:
+                        self.server_profile.error_log_file_path = error_log_file_path or os.path.join(self.server_profile.datadir, hostname + '.err')
+                        request_save_profile = True
+
+                else:
+                    path = self.server_variables.get('general_log_file')
+                    general_log_file_path = path if path and path != '0' else ''
+                    if self.server_profile.general_log_file_path != general_log_file_path:
+                        self.server_profile.general_log_file_path = general_log_file_path
+                        request_save_profile = True
+
+                    path = self.server_variables.get('slow_query_log_file')
+                    slow_query_log_file_path = path if path and path != '0' else ''
+                    if self.server_profile.slow_log_file_path != slow_query_log_file_path:
+                        self.server_profile.slow_log_file_path = slow_query_log_file_path
+                        request_save_profile = True
+
+                    path = self.server_variables.get('log_error')
+                    log_error_path = path if path and path != '0' else ''
+                    if self.server_profile.error_log_file_path != log_error_path:
+                        self.server_profile.error_log_file_path = log_error_path
+                        request_save_profile = True
+
+                log_info(_this_file, "Created SQL connection to MySQL server. Server version is " + repr(self.server_version) +
+                         ", conn status = " + repr(self.is_sql_connected()) +
+                         ", active plugins = " + str([x[0] for x in self.server_active_plugins]) + '\n')
+
+                # Save the server profile if at least one of its values has changed:
+                if request_save_profile:
+                    from grt.modules import Workbench
+                    Workbench.saveInstances()
             else:
-                path = self.get_server_variable('general_log_file')
-                general_log_file_path = path if path and path != '0' else ''
-                if self.server_profile.general_log_file_path != general_log_file_path:
-                    self.server_profile.general_log_file_path = general_log_file_path
-                    request_save_profile = True
-
-                path = self.get_server_variable('slow_query_log_file')
-                slow_query_log_file_path = path if path and path != '0' else ''
-                if self.server_profile.slow_log_file_path != slow_query_log_file_path:
-                    self.server_profile.slow_log_file_path = slow_query_log_file_path
-                    request_save_profile = True
-
-                path = self.get_server_variable('log_error')
-                log_error_path = path if path and path != '0' else ''
-                if self.server_profile.error_log_file_path != log_error_path:
-                    self.server_profile.error_log_file_path = log_error_path
-                    request_save_profile = True
-
-            log_info(_this_file, "Created SQL connection to MySQL server. Server version is " + repr(self.server_version) +
-                     ", conn status = " + repr(self.is_sql_connected()) +
-                     ", active plugins = " + str(self.server_active_plugins) + '\n')
-
-            # Save the server profile if at least one of its values has changed:
-            if request_save_profile:
-                from grt.modules import Workbench
-                Workbench.saveInstances()
+                print "Failed to connect to MySQL server"
         else:
-            print "Failed to connect to MySQL server"
+            print "Already connected to MySQL server"
 
     #---------------------------------------------------------------------------
     def disconnect_sql(self):
@@ -782,7 +795,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             try:
                 ret = self.sql.exec_query(q)
             except QueryError, e:
-                log_warning(_this_file, "Error executing query %s: %s\n"%(q, e))
+                log_warning(_this_file, "Error executing query %s: %s\n"%(q, strip_password(str(e))))
                 if auto_reconnect and e.is_connection_error():
                     log_warning(_this_file, "exec_query: Loss of connection to mysql server was detected.\n")
                     self.handle_sql_disconnection(e)
@@ -799,7 +812,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             try:
                 ret = self.sql.execute(q)
             except QueryError, e:
-                log_warning(_this_file, "Error executing SQL %s: %s\n"%(q, e))
+                log_warning(_this_file, "Error executing SQL %s: %s\n"%(strip_password(q), strip_password(str(e))))
                 if auto_reconnect and e.is_connection_error():
                     log_warning(_this_file, "exec_sql: Loss of connection to mysql server was detected.\n")
                     self.handle_sql_disconnection(e)
@@ -825,16 +838,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
 
     #---------------------------------------------------------------------------
     def get_server_variable(self, variable, default = None):
-        result = self.exec_query("SHOW VARIABLES LIKE '%s'" % variable)
-        if result and result.nextRow():
-            var = result.stringByName('Value')
-        else:  # No result available
-            if default is not None:
-                var = default
-            else:
-                raise ValueError('Could not successfully retrieve the value for the '
-                                 'server variable "%s"' % variable)
-        return var
+        return self.server_variables.get(variable, default)
 
     #---------------------------------------------------------------------------
     def get_server_version(self):
@@ -858,86 +862,79 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
         return self.ssh is not None
 
     #---------------------------------------------------------------------------
-    def execute_filtered_command(self, script, as_user = Users.CURRENT, user_password = False, verbose = None):
-        (script, filters) = build_filters(script)
+    def detect_operating_system_version(self):
+        """Try to detect OS information in the remote server, via SSH connection.
+            
+        The information returned is (os_type, os_name, os_variant, os_version)
+            
+            
+            os_type: one of the main types of OS supported (one of wbaOS.windows, wbaOS.linux, wbaOS.darwin)
+            os_name: the exact name of the OS (eg Windows, Linux, Mac OS X, Solaris)
+            os_variant: the variant of the OS, esp for Linux distributions (eg Ubuntu, Fedora etc)
+            os_version: the version of the OS (eg 12.04, XP etc)
+        """
+        if self.ssh or self.server_profile.is_local:
+            o = StringIO.StringIO()
+            # check if windows
+            rc = self.server_helper.execute_command('ver', output_handler=o.write)
+            if rc == 0 and o.getvalue().strip().startswith("Microsoft Windows"):
+                os_type = wbaOS.windows
+                os_name = "Windows"
+                os_variant = "Windows"
+                os_version = o.getvalue().strip()
 
-        output_text = []
-        def gather_output(line, l=output_text):
-            l.append(line)
-        retcode = self.server_helper.execute_command(script, as_user=as_user, user_password=user_password, output_handler=gather_output)
-        output_text = "".join(output_text)
-
-        if verbose:
-            self.output("Raw output from '%s': status - %s, text - %s" % (script, retcode, output_text))
-
-        (filtered_text, filters_code) = apply_filters(output_text, filters)
-
-        log_debug3(_this_file, "execute_filtered_command: " + script + " => " + str(filtered_text) + "(%s)\n" % str(filters_code))
-
-        if filters_code is not None:
-            if retcode is not None:
-                retcode = int(retcode) or filters_code
+                o = StringIO.StringIO()
+                rc = self.server_helper.execute_command('reg query "HKLM\Software\Microsoft\Windows NT\CurrentVersion" /v "ProductName"', output_handler=o.write)
+                if rc == 0:
+                    os_name = " ".join(o.getvalue().strip().split("\n")[-1].split()[2:])
+                return os_type, os_name, os_variant, os_version
             else:
-                retcode = filters_code
+                os_type, os_name, os_variant, os_version = (None, None, None, None)
 
-        log_debug3(_this_file, "script='%s', cret,fret='%i,%s', output='%s'\n" % (script, retcode, str(filters_code), str(filtered_text)[:32].replace('\n', '')))
-        return (filtered_text, retcode)
+                o = StringIO.StringIO()
+                if self.server_helper.execute_command("uname", output_handler=o.write) == 0:
+                    ostype = o.getvalue().strip()
+                    log_debug(_this_file, "uname in remote system returned %s\n"%ostype)
+                    if ostype == "Darwin":
+                        os_type = wbaOS.darwin
+                        o = StringIO.StringIO()
+                        if self.server_helper.execute_command("sw_vers", output_handler=o.write) == 0:
+                            for line in o.getvalue().strip().split("\n"):
+                                line = line.strip()
+                                if line:
+                                    k, v = line.split(":", 1)
+                                    if k == "ProductName":
+                                        os_name = v.strip()
+                                        os_variant = v.strip()
+                                    elif k == "ProductVersion":
+                                        os_version = v.strip()
+                    else:
+                        os_type = wbaOS.linux
+
+                        o = StringIO.StringIO()
+                        if self.server_helper.execute_command("lsb_release -a", output_handler=o.write) == 0:
+                            os_name = "Linux"
+                            for line in o.getvalue().strip().split("\n"):
+                                line = line.strip()
+                                if line:
+                                    k, sep, v = line.partition(":")
+                                    if k == "Distributor ID":
+                                        os_variant = v.strip()
+                                    elif k == "Release":
+                                        os_version = v.strip()
+                        else:
+                            # some distros don't install lsb things by default
+                            try:
+                                info = self.server_helper.get_file_content("/etc/fedora-release")
+                                if info:
+                                    os_name = "Linux"
+                                    os_variant = "Fedora"
+                                    os_version = info[info.find("release")+1:].split()[0].strip()
+                            except (IOError, OSError):
+                                pass
+                    return os_type, os_name, os_variant, os_version
+                else:
+                    log_error(_this_file, "Could not execute uname command on remote server, system type is unknown\n")
+        return None
 
 
-
-
-
-# === Unit tests ===
-def unit_tests(settings, ctrl_be):
-    print "is running", ctrl_be.is_running()
-    return ()
-
-
-if __name__ == "__main__":
-    test_nr = 0
-
-    #-------------
-    if run_cmd("echo 'Test'", 0, "")[1] is not None:
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
-
-    #-------------
-    if run_cmd("", 1, "")[1] is None:
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
-
-    #-------------
-    import getpass
-    password = getpass.getpass("Enter password: ")
-
-    if run_cmd("", 1, password)[1] is None:
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
-
-    #-------------
-    if run_cmd("echo Test", 1, password)[1] == 0:
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
-
-    #-------------
-    if run_cmd("/bin/false", 1, password)[1] == 1:
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
-
-    #-------------
-    result = run_cmd("echo 'Test\ message'", 0, None)
-    if result[1] == 0 and result[0].strip("\r\n ") == "Test message":
-        print "Test", test_nr, " - Ok"
-    else:
-        print "Test", test_nr, " - Fail"
-    test_nr += 1
