@@ -1,4 +1,4 @@
-# Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -95,10 +95,14 @@ import re
 import os
 import StringIO
 
-from wb_admin_ssh import WbAdminSSH, ConnectionError
+from grt import log_info, log_error
+_this_file = os.path.basename(__file__)
+
 from wb_server_control import PasswordHandler
 
-from wb_common import LogFileAccessError, ServerIOError
+from wb_server_management import SudoTailInputFile, LocalInputFile, SFTPInputFile
+from wb_common import LogFileAccessError, ServerIOError, InvalidPasswordError, Users
+from workbench.utils import server_os_path
 
 #========================= Query Based Readers ================================
 
@@ -121,6 +125,7 @@ class BaseQueryLogReader(object):
         :type ordering_column: int
         """
         self.log_table = log_table
+        self.log_file = None
         self.ctrl_be = ctrl_be
         self.column_specs = column_specs
         self.ordering_column = ordering_column
@@ -158,6 +163,9 @@ class BaseQueryLogReader(object):
         self.show_start = max(self.total_count - self.show_count, 0)
         return self._query_records()
 
+    def size_text(self):
+        return '%d records' % self.total_count
+
     def range_text(self):
         return 'Records %d..%d of %d' % (self.show_start,
                                          min(self.show_start + self.show_count, self.total_count),
@@ -228,239 +236,150 @@ class SlowQueryLogReader(BaseQueryLogReader):
 
 #========================= File Based Readers =================================
 
+
 class BaseLogFileReader(object):
     '''
-    The base class for logs stored in files.
+        The base class for logs stored in files unreadable to the current user.
 
-    **This is not intended for direct instantiation.**
-    '''
+        **This is not intended for direct instantiation.**
+        '''
     def __init__(self, ctrl_be, log_file, pat, chunk_size, truncate_long_lines, append_gaps=True):
         """Constructor
 
-        :param ctrl_be:  Control backend instance to retrieve root password if needed
-        :param log_file: The path to the log file to read from or a file like instance to
-                         read log entries from
-        :type log_file: str/file
-        :param pat: A regular expression pattern that matches a log entry
-        :type pat: regular expression object
-        :param chunk_size: The size in bytes of the chunks that are read from the log file
-        :type chunk_size: int
-        :param truncate_long_lines: Whether the log entries that are long should be abbreviated
-        :type truncate_long_lines: bool
-        :param append_gaps: Whether the data between the end of the record regex and the start
-                            of the next record regex should be added to the previous entry
-        :type append_gaps: bool
-        """
+            :param ctrl_be:  Control backend instance to retrieve root password if needed
+            :param log_file: The path to the log file to read from or a file like instance to
+            read log entries from
+            :type log_file: str/file
+            :param pat: A regular expression pattern that matches a log entry
+            :type pat: regular expression object
+            :param chunk_size: The size in bytes of the chunks that are read from the log file
+            :type chunk_size: int
+            :param truncate_long_lines: Whether the log entries that are long should be abbreviated
+            :type truncate_long_lines: bool
+            :param append_gaps: Whether the data between the end of the record regex and the start
+            of the next record regex should be added to the previous entry
+            :type append_gaps: (not used)
+            """
 
         self.pat = pat  # the regular expression that identifies a record
-        self.append_gaps = append_gaps  # if characters between the occurrences of the
-                                        # regex are considered to belong to the last field of the record
+        self.append_gaps = append_gaps
+        # regex are considered to belong to the last field of the record
         self.truncate_long_lines = truncate_long_lines
         self.ctrl_be = ctrl_be
 
-        self.is_local = self.ctrl_be.server_profile.is_local
-        self.not_readable = False
         self.partial_support = False
-        self.mtime = None
 
-        if isinstance(log_file, (file, StringIO.StringIO)):
-            self.log_file = log_file
-            if hasattr(log_file, 'name'):
-                self.log_file_name = log_file.name
-                self.mtime = os.stat(self.log_file_name).st_mtime
-            else:
-                self.log_file_name = None
-        else:
-            # If there isn't a directory component in the path, use @@datadir as the directory for the log file:
-            log_file = log_file.strip(' "')
-            self.log_file_name = log_file if os.path.dirname(log_file) else os.path.join(self.ctrl_be.get_server_variable('datadir'), log_file)
-            if self.is_local:
-                try:
-                    self.mtime = os.stat(self.log_file_name).st_mtime
-                    self.log_file = open(self.log_file_name, 'rb')
-                except IOError, e:
-                    if self.ctrl_be.server_profile.target_is_windows:
-                        raise LogFileAccessError('Cannot read from %s. Check permissions and retry.' % log_file)
-                    if os.path.isfile(log_file):
-                        self.not_readable = True
-                        self.last_known_size = os.path.getsize(log_file)
-                        self.password = ctrl_be.server_control.get_password()
-                        self.tail_data = ctrl_be.execute_filtered_command('tail -c %d %s' % (chunk_size, log_file),
-                                                                          as_admin = True,
-                                                                          admin_password = self.password)[0]
-                        self.log_file = StringIO.StringIO(self.tail_data)
-                        self.partial_support = 'The current user has no permission to read %s. Reading it with superuser privileges. Workbench has only limited support for this.' % log_file
-                    else:
-                        raise IOError(e)
-                except OSError, e:
-                    raise LogFileAccessError('''An error appeared when looking for the log file "%s". Please verify that you can access the directory
-where the log file is placed ("%s") and that the log file is actually there. Try again when done.''' % (log_file, os.path.dirname(log_file)))
+        # If there isn't a directory component in the path, use @@datadir as the directory for the log file:
+        log_file = log_file.strip(' "')
+        ospath = server_os_path(self.ctrl_be.server_profile)
+        datadir = self.ctrl_be.server_profile.datadir
+        if not datadir and ctrl_be.is_sql_connected():
+            datadir = self.ctrl_be.get_server_variable('datadir')
+        self.log_file_name = log_file if ospath.isabs(log_file) else ospath.join(datadir, log_file)
 
-            else:  # Remote server
+        use_sftp = False
+        if self.ctrl_be.server_profile.target_is_windows:
+            use_sudo = False
+            # In Windows we can either access the file locally as a plain file or remotely with sftp
+            # Ther is no upport for reading log files that are only readable by the admin
+            if not self.ctrl_be.server_profile.is_local:
                 if not self.ctrl_be.server_profile.remote_admin_enabled:
                     raise LogFileAccessError('''You have not enabled remote administration for this server. Without it this log file cannot be shown.
-Please enable remote administration in this server instance and try again.''')
+                        Please enable remote administration in this server instance and try again.''')
                 if not self.ctrl_be.server_profile.uses_ssh:
                     raise LogFileAccessError('''Remote log files are only supported for SSH connection.
-Please configure an SSH connection and try again.''')
+                        Please configure an SSH connection and try again.''')
+                use_sftp = True
+        else:
+            use_sudo = self.ctrl_be.server_profile.use_sudo
+            # In Mac/Linux, we can access local files with sudo and remote files with sudo too
+            if not self.ctrl_be.server_profile.is_local:
+                if not self.ctrl_be.server_profile.remote_admin_enabled:
+                    raise LogFileAccessError('''You have not enabled remote administration for this server. Without it this log file cannot be shown.
+                        Please enable remote administration in this server instance and try again.''')
 
-                if self.ctrl_be.ssh:
-                    self.ssh = self.ctrl_be.ssh
-                else:
-                    self.ssh = WbAdminSSH()
-                    while True:
-                        try:  # Repeat get password while password is misspelled. Re-raise other exceptions
-                            self.ssh.wrapped_connect(self.ctrl_be.server_profile,
-                                                     PasswordHandler(self.ctrl_be.server_profile))
-                        except ConnectionError, error:
-                            if not str(error).startswith('Could not establish SSH connection: Authentication failed'):
-                                raise
-                        else:
-                            break
-                if not self.ssh.is_connected():
-                    raise LogFileAccessError('Could not connect to remote server')
-                self.sftp = self.ssh.client.open_sftp()
-                try:
-                    self.log_file = self.sftp.open(self.log_file_name)
-                    self.mtime = self.sftp.stat(self.log_file_name).st_mtime
-                except IOError, e:
-                    raise LogFileAccessError('Could not read file %s in remote server. Please verify path and permissions' % self.log_file_name)
+                if not self.ctrl_be.server_profile.uses_ssh:
+                    raise LogFileAccessError('''Remote log files are only supported for SSH connection.
+                        Please configure an SSH connection and try again.''')
+
+        if use_sudo:
+            log_info(_this_file, "Will use sudo and dd to get contents of log file %s\n" % self.log_file_name)
+            password = ctrl_be.password_handler.get_password_for('file')
+            try:
+                self.log_file = SudoTailInputFile(self.ctrl_be.server_helper, self.log_file_name, password)
+                self.file_size = self.log_file.size
+            except InvalidPasswordError, error:
+                log_error(_this_file, "Invalid password to sudo %s\n" % error)
+                ctrl_be.password_handler.reset_password_for('file')
+                raise
+        elif use_sftp:
+            log_info(_this_file, "Will use sftp to get contents of log file %s\n" % self.log_file_name)
+            self.log_file = SFTPInputFile(self.ctrl_be, self.log_file_name)
+            self.file_size = self.log_file.size
+        else:
+            log_info(_this_file, "Will use plain file access to get contents of log file %s\n" % self.log_file_name)
+            self.log_file = LocalInputFile(self.log_file_name)
+            self.file_size = self.log_file.size
 
-        self.log_file.seek(0, 2)  # Move to EOF
-        self.file_size = self.log_file.tell()
         self.chunk_size = chunk_size
-        self.chunk_start = self.file_size
-
-        # reuse self._adjust_chunk_end to find first record in the file:
-        self.chunk_end = 0
-        self._adjust_chunk_end()
-        self.first_record_pos = self.chunk_end
-
+        # chunk_start is the start of the chunk to be read (ie, we start reading from the end of the file, so we get the last page starting from there)
+        self.chunk_start = max(0, self.file_size - chunk_size)
         self.chunk_end = self.file_size
 
+        self.record_count = 0
+
 
 
     def has_previous(self):
         '''
-        If there is a previous chunk that can be read.
-        '''
-        return self.chunk_start > self.first_record_pos
+            If there is a previous chunk that can be read.
+            '''
+        return self.chunk_start > 0
 
     def has_next(self):
         '''
-        If there is a next chunk that can be read.
-        '''
+            If there is a next chunk that can be read.
+            '''
         return self.chunk_end != self.file_size
 
     def range_text(self):
-        if not self.not_readable:
-            return 'Log file size: %s' % self._format_size(self.file_size)
-        else:
-            return 'Log file size: %s. (Reading last %s)' % (self._format_size(self.last_known_size),
-                                                             self._format_size(self.file_size))
+        return '%s records starting at byte offset %s' % (self.record_count, self.chunk_start)
+
+    def size_text(self):
+        return self._format_size(self.file_size)
 
     def _format_size(self, bytes):
         '''
-        Returns a string with a human friendly representation of a file size
-        '''
+            Returns a string with a human friendly representation of a file size
+            '''
         if bytes <= 0:
             return '0 B'
         units = (
-            (1., 'B'),
-            (1024., 'kB'),
-            (1024*1024., 'MB'),
-            (1024*1024*1024., 'GB'),
-                )
+                 (1., 'B'),
+                 (1024., 'kB'),
+                 (1024*1024., 'MB'),
+                 (1024*1024*1024., 'GB'),
+                 )
         for idx, unit in enumerate(units):
             if bytes < unit[0]:
                 return '%.1f %s' % (bytes/units[idx-1][0], units[idx-1][1])
         return '%.1f %s' % (bytes/units[-1][0], units[-1][1])
 
-    def _read_previous_chunk(self):
-        '''
-        Reads the previous chunk.
-        '''
-        if self.chunk_start <= self.first_record_pos:
-            raise IndexError('No more data to read from log file')
-        self.chunk_end = self.chunk_start
-        self._adjust_chunk_end()
-        self.chunk_start = max(self.chunk_start - self.chunk_size, self.first_record_pos)
-        self._adjust_chunk_start()
-        self.log_file.seek(self.chunk_start)
-        return self._read_chunk()
-
-    def _read_next_chunk(self):
-        '''
-        Reads the next chunk.
-        '''
-        if self.chunk_end == self.file_size:
-            raise IndexError('No more data to read from log file')
-        self.chunk_start = self.chunk_end
-        self.chunk_end = min(self.chunk_end + self.chunk_size - 1, self.file_size)
-        self._adjust_chunk_end()
-        self._adjust_chunk_start()
-        return self._read_chunk()
-
-    def _read_chunk(self):
-        '''
-        Reads the current chunk.
-        '''
-        bytes = self.chunk_end - self.chunk_start
-        self.log_file.seek(self.chunk_start)
-        data = self.log_file.read(bytes)
-        return data
-
-    def _adjust_chunk_start(self):
-        '''
-        Puts the chunk start marker at the beginning of a record. If a record starts within
-        the range [self.chunk_start, self.chunk_end] the chunk start marker will point to it.
-        If there's no record in the range, move backwards until there's at least one record
-        in the chunk.
-        Precondition: the chunk end marker must be set to delimit the end of the chunk.
-        '''
-        if self.chunk_end > self.chunk_start:
-            if self.chunk_start <= self.first_record_pos:
-                self.chunk_start = self.first_record_pos
-                return
-            data = ''
-            bytes = self.chunk_end - self.chunk_start
-            while True:
-                self.log_file.seek(self.chunk_start)
-                data = self.log_file.read(bytes) + data
-                pat_occurrence = self.pat.search(data)
-                if pat_occurrence:
-#                    self.chunk_start += pat_occurrence.start()
-                    break
-                if self.chunk_start <= self.first_record_pos:
-                    raise IndexError('No more data to read from log file')
-                bytes = min(self.chunk_size/4, self.chunk_start - self.first_record_pos)
-                self.chunk_start -= bytes
-
-    def _adjust_chunk_end(self):
-        '''
-        Moves the chunk termination marker forward to just before the start of the next record
-        or to the EOF if not next record.
-        '''
-        if self.chunk_end != self.file_size:
-            trailing_data = ''
-            self.log_file.seek(self.chunk_end)
-            bytes = self.chunk_size/4  # TODO check this value
-            while True:
-                trailing_data += self.log_file.read(bytes)
-                pat_occurrence = self.pat.search(trailing_data)
-                if pat_occurrence or self.log_file.tell()==self.file_size:
-                    break
-            self.chunk_end = self.chunk_end + pat_occurrence.start() if pat_occurrence else self.file_size
+    def _get_offset_to_first_record(self, data):
+        match = self.pat.search(data)
+        if match:
+            return match.start()
+        return 0
 
     def _parse_chunk(self, data):
         '''
-        Extracts the records from a chunk of data.
-        '''
+            Extracts the records from a chunk of data.
+            '''
         records = []
         found = self.pat.search(data)
         if found:
             end = found.start()
-            if self.chunk_start > self.first_record_pos:  # optional?
+            if self.chunk_start > 0:  # optional?
                 self.chunk_start += end
         while found:
             start = found.start()
@@ -477,13 +396,14 @@ Please configure an SSH connection and try again.''')
             if self.truncate_long_lines:
                 records[-1][-1] = self._shorten_query_field(records[-1][-1])  # now shorten the last record
 
+        self.record_count = len(records)
         return records
 
     def _shorten_query_field(self, data):
         '''
-        Receives a query stored in a log file and prepares it for the output in
-        the log viewer shortening to 256 characters and taking care of encoding issues
-        '''
+            Receives a query stored in a log file and prepares it for the output in
+            the log viewer shortening to 256 characters and taking care of encoding issues
+            '''
         l = len(data)
         try:
             abbr = data[:256].encode('utf-8')
@@ -491,113 +411,74 @@ Please configure an SSH connection and try again.''')
             abbr = data[:256].decode('latin1').encode('utf-8')
         size = '%d bytes' % l if l < 1024 else '%.1f KB' % (l / 1024.0)
         return abbr if l <= 256 else abbr + ' [truncated, %s total]' % size
-
+    
     def current(self):
         '''
-        Returns a list with the records in the current chunk.
-        Each record is a list with the values for each column of
-        the corresponding log entry.
-        '''
-        try:
-            data = self._read_chunk()
-        except IndexError:
-            return []
-        return self._parse_chunk(data)
+            Returns a list with the records in the current chunk.
+            Each record is a list with the values for each column of
+            the corresponding log entry.
+            '''
+        data = self.log_file.get_range(self.chunk_start, self.chunk_end)
+        if self.chunk_start > self.chunk_size / 10:
+            # adjust the start of the current chunk to the start of the 1st record (if we're not too close to the top)
+            offset = self._get_offset_to_first_record(data)
+            self.chunk_start += offset
+        else:
+            offset = 0
+        return self._parse_chunk(data[offset:])
 
     def previous(self):
         '''
-        Returns a list with the records in the previous chunk.
-        Each record is a list with the values for each column of
-        the corresponding log entry.
-        '''
-        try:
-            data = self._read_previous_chunk()
-        except IndexError:
+            Returns a list with the records in the previous chunk.
+            Each record is a list with the values for each column of
+            the corresponding log entry.
+            '''
+        if self.chunk_start == 0:
             return []
-        return self._parse_chunk(data)
-
+        self.chunk_end = self.chunk_start
+        self.chunk_start = max(0, self.chunk_start - self.chunk_size)
+        return self.current()
+    
     def next(self):
         '''
-        Returns a list with the records in the next chunk.
-        Each record is a list with the values for each column of
-        the corresponding log entry.
-        '''
-        try:
-            data = self._read_next_chunk()
-        except IndexError:
+            Returns a list with the records in the next chunk.
+            Each record is a list with the values for each column of
+            the corresponding log entry.
+            '''
+        if self.chunk_end == self.file_size:
             return []
-        return self._parse_chunk(data)
-
+        self.chunk_start = self.chunk_end
+        self.chunk_end = min(self.chunk_start + self.chunk_size, self.file_size)
+        return self.current()
+    
     def first(self):
         '''
-        Returns a list with the records in the first chunk
-        '''
-        self.chunk_end = self.first_record_pos
-        return self.next()
-
+            Returns a list with the records in the first chunk
+            '''
+        self.chunk_start = 0
+        self.chunk_end = self.chunk_size
+        return self.current()
+    
     def last(self):
         '''
-        Returns a list with the records in the first chunk
-        '''
-        self.chunk_start = self.file_size
+            Returns a list with the records in the first chunk
+            '''
+        self.chunk_start = max(0, self.file_size - self.chunk_size)
         self.chunk_end = self.file_size
-        return self.previous()
-
-    def log_file_changed(self):
-        '''
-        If the log file has grown since the time self.file_size was
-        computed.
-        '''
-        self.log_file.seek(0, 2)  # Move to EOF
-        return self.log_file.tell() != self.file_size
+        return self.current()
 
     def refresh(self):
         '''
-        Checks if the log file has been updated since it was opened and if so
-        reopen the file again to keep going with the changes.
-        Warning: this function only supports appending to the log file.
-        '''
-        if self.log_file_name:
-            if self.is_local:
-                mtime = os.stat(self.log_file_name).st_mtime
-                if mtime != self.mtime:
-                    if self.not_readable:
-                        if self.ctrl_be.server_profile.target_is_windows:
-                            return  #TODO: Add support for not readable files in Windows
-                        size = os.path.getsize(self.log_file_name)
-                        bytes = size - self.last_known_size
-                        # Read new data (capped at chunk_size bytes) from the end of the log file:
-                        new_data = self.ctrl_be.execute_filtered_command('tail -c %d %s' % (min(bytes, self.chunk_size), self.log_file_name),
-                                                      as_admin = True,
-                                                      admin_password = self.password)[0]
-                        if len(new_data) < self.chunk_size:  # Proceed to complete using what was stored in tail_data
-                            self.tail_data = self.tail_data[-min( len(self.tail_data), self.chunk_size - len(new_data) ) : ] + new_data
-                        else:
-                            self.tail_data = new_data
+            Checks if the log file has been updated since it was opened and if so
+            reopen the file again to keep going with the changes.
+            Warning: this function only supports appending to the log file.
+            '''
+        new_size = self.log_file.size
+        if new_size != self.file_size:
+            self.file_size = new_size
+            self.chunk_start = max(0, self.file_size - self.chunk_size)
+            self.chunk_end = self.file_size
 
-                        self.log_file = StringIO.StringIO(self.tail_data)
-
-                        # Reuse _adjust_chunk_end to set self.first_record_pos
-                        chunk_end = self.chunk_end
-                        self.chunk_end = 0
-                        self._adjust_chunk_end()
-                        self.first_record_pos = self.chunk_end
-                        self.chunk_end = chunk_end
-
-                        self.last_known_size = size
-                    else:
-                        self.log_file.close()
-                        self.log_file = open(self.log_file_name, 'rb')
-
-            else:  # Remote file
-                mtime = self.sftp.stat(self.log_file_name).st_mtime
-                if mtime != self.mtime:
-                    self.log_file.close()
-                    self.log_file = self.sftp.open(self.log_file_name, 'rb')
-
-            self.log_file.seek(0, 2)  # Move to EOF
-            self.file_size = self.log_file.tell()
-            self.mtime = mtime
 
 #==============================================================================
 class ErrorLogFileReader(BaseLogFileReader):
@@ -605,26 +486,85 @@ class ErrorLogFileReader(BaseLogFileReader):
     This class enables the retrieval of log entries in a MySQL error
     log file.
     '''
-    def __init__(self, ctrl_be, file_name, chunk_size=4 * 1024, truncate_long_lines=False):
-        pat = re.compile(r'^(\d{6} {1,2}\d{1,2}:\d{2}:\d{2}) (.*?)$', re.M)
-        super(ErrorLogFileReader, self).__init__(ctrl_be, file_name, pat, chunk_size, truncate_long_lines)
+    def __init__(self, ctrl_be, file_name, chunk_size=64 * 1024, truncate_long_lines=False):
+        # The error log is a mess, there are several different formats for each entry and a new one comes up every version
+        partial_re = r'^(?P<v56>(\d{2,4}-\d{1,2}-\d{2} {1,2}\d{1,2}:\d{2}:\d{2}) (\d+) \[(.*)\] (.*?))$|^(?P<v55>(\d{6} {1,2}\d{1,2}:\d{2}:\d{2}) {1,2}([^ ]*) (.*?))$|^(?P<old>(\d{2})(\d{2})(\d{2}) {1,2}(\d{1,2}:\d{2}:\d{2}) ([a-zA-Z0-9_]*?) (.*?))$'
+
+        pat = re.compile(partial_re, re.M)
+        super(ErrorLogFileReader, self).__init__(ctrl_be, file_name, pat, chunk_size, truncate_long_lines, append_gaps=False)
         self.column_specs = (
-                ('Time', 150),
+                ('Timestamp', 150),
+                ('Thread', 100),
+                ('Type', 100),
                 ('Details', 500),
                             )
+        # regex that matches all known entry formats + anything else
+        self.pat2 = re.compile(partial_re + r'|^(?P<any>.+)$', re.M)
+
         self.detail_column = 1
 
+    
+    def _parse_chunk(self, data):
+        '''
+            Extracts the records from a chunk of data.
+            '''
+        records = []
+        found = self.pat2.search(data)
+        if found:
+            end = found.start()
+            if self.chunk_start > 0:  # optional?
+                self.chunk_start += end
+        while found:
+            start = found.start()
+            #if start-end > 1 and self.append_gaps:  # there's a gap between occurrences of the pattern
+            #    records[-1][-1] += data[end:start]  # append the gap to previous record
+            gdict = found.groupdict()
+            g = found.groups()
+            if gdict['v56']:
+                records.append(g[1:5])
+            elif gdict['v55']:
+                ts = g[6]
+                records.append(["20%s-%s-%s %s" % (ts[:2], ts[2:4], ts[4:6], ts[6:].lstrip()), "", g[7], g[8]])
+            elif gdict['old']:
+                records.append(["20%s-%s-%s %s" % (g[10], g[11], g[12], g[13]), "", g[14], g[15]])
+            else:
+                records.append(["", "", "", g[-1]])
+            end = found.end()
+            found = self.pat2.search(data, end)
+            if found and self.truncate_long_lines:  # shorten all but the last record
+                records[-1][-1] = self._shorten_query_field(records[-1][-1])
+        if records:
+            #if self.append_gaps:
+            #    records[-1][-1] += data[end:]  # add what remains in data
+            if self.truncate_long_lines:
+                records[-1][-1] = self._shorten_query_field(records[-1][-1])  # now shorten the last record
+        self.record_count = len(records)
+        return records
+
+
+    def current(self):
+        records = super(ErrorLogFileReader, self).current()
+        if self.chunk_end < self.file_size:
+            # check if the last record is truncated
+            rec = records[-1]
+            if not any(f != "" for f in rec[:-1]):
+                # the last record wasn't recognized, it's probably truncated so leave it out of this chunk
+                self.chunk_end -= len(rec[-1])
+                del records[-1]
+        return records
+
+
 #==============================================================================
 class GeneralLogFileReader(BaseLogFileReader):
     '''
     This class enables the retrieval of log entries in a MySQL general query
     log file.
     '''
-    def __init__(self, ctrl_be, file_name, chunk_size=4 * 1024, truncate_long_lines=True):
+    def __init__(self, ctrl_be, file_name, chunk_size=64 * 1024, truncate_long_lines=True):
         pat = re.compile(r'^(\d{6} {1,2}\d{1,2}:\d{2}:\d{2}[\t ]+|[\t ]+)(\s*\d+)(\s*.*?)(?:\t+| {2,})(.*?)$', re.M)
         super(GeneralLogFileReader, self).__init__(ctrl_be, file_name, pat, chunk_size, truncate_long_lines)
         self.column_specs = (
-                ('Time', 150),
+                ('Timestamp', 150),
                 ('Thread', 80),
                 ('Command Type', 80),
                 ('Detail', 500),
@@ -638,8 +578,8 @@ class SlowLogFileReader(BaseLogFileReader):
     This class enables the retrieval of log entries in a MySQL slow query
     log file.
     '''
-    def __init__(self, ctrl_be, file_name, chunk_size=4 * 1024, truncate_long_lines=False, append_gaps=False):
-        pat = re.compile(r'(?:^|\n)# Time: (\d{6} {1,2}\d{1,2}:\d{2}:\d{2}).*?\n# User@Host: (.*?)\n# Query_time: +([0-9.]+) +Lock_time: +([\d.]+) +Rows_sent: +(\d+) +Rows_examined: +(\d+)\n(.*?)(?=\n# |\n[^\n]+, Version: |$)', re.S)
+    def __init__(self, ctrl_be, file_name, chunk_size=64 * 1024, truncate_long_lines=False, append_gaps=False):
+        pat = re.compile(r'(?:^|\n)# Time: (\d{6} {1,2}\d{1,2}:\d{2}:\d{2}).*?\n# User@Host: (.*?)\n# Query_time: +([0-9.]+) +Lock_time: +([\d.]+) +Rows_sent: +(\d+) +Rows_examined: +(\d+)\s*\n(.*?)(?=\n# |\n[^\n]+, Version: |$)', re.S)
         super(SlowLogFileReader, self).__init__(ctrl_be, file_name, pat, chunk_size, truncate_long_lines, append_gaps)
         self.column_specs = (
                 ('Start Time', 150),
