@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -19,6 +19,8 @@ import threading
 from threading import currentThread
 import sys
 import os
+import ntpath
+import posixpath
 
 import grt
 
@@ -75,11 +77,22 @@ def get_local_ip_list():
 
 local_ip_list = get_local_ip_list()
 
+
+#===============================================================================
+
+UnixVariant = {
+    "" : {
+        'sudo_command' : "/usr/bin/sudo -S -p EnterPasswordHere ",
+    }
+}
+
 #===============================================================================
 class ServerProfile(object):
-    def __init__(self, instance_profile, force_remote_admin_off=False):
+    def __init__(self, connection, instance_profile, force_remote_admin_off=False):
         self.disable_remote_admin = force_remote_admin_off
+        self.__connection = connection
         self.__settings = instance_profile
+        self.__os_specifics = None # one of UnixVariant
         self.expanded_config_file_path = None
 
     def get_settings_object(self):
@@ -87,6 +100,8 @@ class ServerProfile(object):
 
     def _int_server_info(self, name, default_value=0):
         # always returns an int, a None value will be replaced with default_value
+        if not self.__settings:
+            return default_value
         try:
             value = self.__settings.serverInfo[name]
         except KeyError, exc:
@@ -117,6 +132,8 @@ class ServerProfile(object):
 
     def _str_server_info(self, name, default_value=""):
         # always returns an str or unicode, a None value will be replaced with default_value
+        if not self.__settings:
+            return default_value
         try:
             value= self.__settings.serverInfo[name]
         except KeyError, exc:
@@ -141,11 +158,11 @@ class ServerProfile(object):
 
     @property
     def name(self):
-        return self.__settings.name
+        return self.__connection.name
 
     @property
     def db_connection_params(self):
-        return self.__settings.connection
+        return self.__connection
 
     @property
     def host_os(self):
@@ -202,11 +219,12 @@ class ServerProfile(object):
     @property
     def is_local(self):
         local_addrs = ["localhost", "", "0", "127.0.0.1"] + local_ip_list
-        if self.uses_wmi:
-            return self.wmi_hostname in local_addrs
-        elif self.uses_ssh:
-            return self.ssh_hostname in local_addrs
-        else:
+        #if self.uses_wmi:
+        #    return self.wmi_hostname in local_addrs
+        #elif self.uses_ssh:
+        #    return self.ssh_hostname in local_addrs
+        #else:
+        if True:
             if self.db_connection_params:
                 params = self.db_connection_params.parameterValues
                 if params:
@@ -319,10 +337,6 @@ Please edit your connection settings and try again."""
     def use_sudo(self):
         return self._int_server_info("sys.usesudo")
 
-    @property
-    def sudo_prefix(self):
-        return self._str_server_info("sys.sudo", "sudo") or "sudo"
-
     @property
     def use_sudo_for_status(self):
         return self._int_server_info("sys.usesudostatus")
@@ -342,10 +356,6 @@ Please edit your connection settings and try again."""
     def config_file_section(self):
         return self._str_server_info('sys.config.section', '').strip(" \r\t\n\b")
 
-    @property
-    def sys_script(self):
-        return self._str_server_info('sys.script', '')
-
     #-------------------------------------------------------
     def server_version(self):
         return self._str_server_info("serverVersion", None)
@@ -440,21 +450,47 @@ Please edit your connection settings and try again."""
 
     basedir = property(basedir, set_basedir)
 
+    #-------------------------------------------------------
+    def set_os_version_info(self, os_type, os_name, os_variant, os_version):
+        self.__settings.serverInfo['detected_os_type'] = os_type
+        self.__settings.serverInfo['detected_os_name'] = os_name
+        self.__settings.serverInfo['detected_os_variant'] = os_variant
+        self.__settings.serverInfo['detected_os_version'] = os_version
+
+    @property
+    def detected_os_type(self):
+        return self.__settings.serverInfo['detected_os_type']
+
+    @property
+    def detected_os_name(self):
+        return self.__settings.serverInfo['detected_os_name']
+
+    @property
+    def detected_os_variant(self):
+        return self.__settings.serverInfo['detected_os_variant']
+
+    @property
+    def detected_os_version(self):
+        return self.__settings.serverInfo['detected_os_version']
+
+
+    def join_target_path(self, path, *comps):
+        pathmodule = ntpath if self.target_os == wbaOS.windows else posixpath
+        return pathmodule.join(path, *comps)
+
+    # OS specific things
+    @property
+    def sudo_prefix(self):
+        return UnixVariant['']['sudo_command']
+
 
 class PasswordHandler(object):
     def __init__(self, server_profile):
         self.server_profile = server_profile
         self.pwd_store = {}
 
-    def get_password_parameters(self, service_type):
-        # known service types: remoteshell, ssh, sshkey, file, service.startstop, service.status
-        # these are mapped to
-        # known password types: ssh, sshkey, sudo, UAC, wmi
-        # which are then mapped to (title, service, account) or "UAC" or None
-
+    def get_password_type(self, service_type):
         profile = self.server_profile
-
-        password_type = None
         if service_type == "file":
             if profile.target_is_windows:
                 # in windows, auth is handled by UAC which is done externally
@@ -489,7 +525,17 @@ class PasswordHandler(object):
             password_type = "sshkey"
         else:
             raise Exception("Unknown password type: %s" % service_type)
+        return password_type
 
+    def get_password_parameters(self, service_type):
+        # known service types: remoteshell, ssh, sshkey, file, service.startstop, service.status
+        # these are mapped to
+        # known password types: ssh, sshkey, sudo, UAC, wmi
+        # which are then mapped to (title, service, account) or "UAC" or None
+
+        profile = self.server_profile
+
+        password_type = self.get_password_type(service_type)
 
         # sudo pwd may be either user's password, or root password. It depends on sudo setup
         if password_type == "sudo":
@@ -540,17 +586,20 @@ class PasswordHandler(object):
 
     # call this if a InvalidPasswordError is caught
     def reset_password_for(self, service_type):
-        if self.pwd_store.has_key(service_type):
+        password_type = self.get_password_type(service_type)
+        if self.pwd_store.has_key(password_type):
             # None means the stored password was bad
-            self.pwd_store[service_type] = None
+            self.pwd_store[password_type] = None
 
     def get_password_for(self, service_type): # Raises OperationCancelledError if user cancels pwd dlg.
         force_reset = False
-        if self.pwd_store.has_key(service_type):
-            if self.pwd_store[service_type] is None:
+        password_type = self.get_password_type(service_type)
+
+        if self.pwd_store.has_key(password_type):
+            if self.pwd_store[password_type] is None:
                 force_reset = True
             else:
-                return self.pwd_store[service_type]
+                return self.pwd_store[password_type]
 
         details = self.get_password_parameters(service_type)
         if not details: # No password needed for this service_type
@@ -568,7 +617,7 @@ class PasswordHandler(object):
         if accepted:
             if not password:
                 password = ""
-            self.pwd_store[service_type] = password.encode("utf8")
+            self.pwd_store[password_type] = password.encode("utf8")
             return password
         else:
             raise OperationCancelledError("Password input cancelled")
@@ -597,19 +646,11 @@ class ServerControlBase(object):
         # do cleanup, free up resources etc
         pass
 
-    def start(self, password): # overriden by concrete subclasses
-        "Starts the server using the method specified in the instance profile"
-        assert 0
-
     def start_async(self, finish_callback): # callback(status) where status can be success, bad_password or an error message
-        try:
-            # ask subclass to provide password
-            password = self.get_password()
-        except OperationCancelledError, exc:
-            return False
-        thread = threading.Thread(target=self.worker_thread, args=(self.start, password, finish_callback))
-        thread.run()
-        return True
+        raise NotImplementedError()
+
+    def stop_async(self, finish_callback):
+        raise NotImplementedError()
 
     def worker_thread(self, action, password, finish_callback):
         try:
@@ -622,19 +663,6 @@ class ServerControlBase(object):
             traceback.print_exc()
             finish_callback(err)
 
-
-    def stop(self, password): # overriden by concrete subclasses, must be non-interactive
-        assert 0
-
-    def stop_async(self, finish_callback):
-        try:
-            password = self.get_password()
-        except OperationCancelledError, exc:
-            return False
-        thread = threading.Thread(target=self.worker_thread, args=(self.stop, password, finish_callback))
-        thread.run()
-        return True
-
     def get_status(self, verbose=0): # overriden by concrete subclasses
         return 0
 
@@ -666,16 +694,46 @@ class ServerControlShell(ServerControlBase):
     def get_password(self):
         return self.password_delegate.get_password_for("service.startstop")
 
+    def start_async(self, finish_callback): # callback(status) where status can be success, bad_password or an error message
+        if not self.profile.start_server_cmd:
+            self.info("Command to start the server is not configured. Please set the command that must be used to start the server in the remote management section of this connections settings.")
+            return False
+        if self.profile.use_sudo:
+            try:
+                password = self.get_password()
+            except OperationCancelledError, exc:
+                return False
+        else:
+            password = None
+        thread = threading.Thread(target=self.worker_thread, args=(self.start, password, finish_callback))
+        thread.run()
+        return True
 
+    def stop_async(self, finish_callback):
+        if not self.profile.stop_server_cmd:
+            self.info("Command to stop the server is not configured. Please set the command that must be used to stop the server in the remote management section of this connections settings.")
+            return False
+        if self.profile.use_sudo:
+            try:
+                password = self.get_password()
+            except OperationCancelledError, exc:
+                return False
+        else:
+            password = None
+        thread = threading.Thread(target=self.worker_thread, args=(self.stop, password, finish_callback))
+        thread.run()
+        return True
+    
     def start(self, password):
         """ Can also throw InvalidPasswordError"""
         self.info("Starting server...") # TODO: This does not come to info output
         try:
+            self.info("Executting '%s'" % self.profile.start_server_cmd)
             as_user = Users.ADMIN if self.profile.use_sudo else Users.CURRENT
             r = self.helper.execute_command(self.profile.start_server_cmd,
                                     as_user=as_user,
                                     user_password=password,
-                                    output_handler=lambda s:self.info("Start server: %s"%(s if ((type(s) is str) or (type(s) is unicode)) else "").replace(password, "")))
+                                    output_handler=lambda s:self.info("Start server: %s"%(s if ((type(s) is str) or (type(s) is unicode)) else "").replace(password or "", "")))
         except InvalidPasswordError, exc:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
@@ -699,6 +757,7 @@ class ServerControlShell(ServerControlBase):
                 self.info("Stop server: %s"%str(output))
 
             as_user = Users.ADMIN if self.profile.use_sudo else Users.CURRENT
+            self.info("Executing '%s'" % self.profile.stop_server_cmd)
             r = self.helper.execute_command(self.profile.stop_server_cmd,
                                     as_user=as_user,
                                     user_password=password,
@@ -873,6 +932,25 @@ class ServerControlWMI(ServerControlBase):
         self.wmi_session_ids = {}
         ServerControlBase.close() # not certain if the call is needed.
 
+    def start_async(self, finish_callback): # callback(status) where status can be success, bad_password or an error message
+        try:
+            # ask subclass to provide password
+            password = self.get_password()
+        except OperationCancelledError, exc:
+            return False
+        thread = threading.Thread(target=self.worker_thread, args=(self.start, password, finish_callback))
+        thread.run()
+        return True
+
+    def stop_async(self, finish_callback):
+        try:
+            password = self.get_password()
+        except OperationCancelledError, exc:
+            return False
+        thread = threading.Thread(target=self.worker_thread, args=(self.stop, password, finish_callback))
+        thread.run()
+        return True
+    
     def start(self, password):
         self.info("Starting server...");
         if self.shell:
