@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,28 +15,35 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
+import os
+import sys
+import copy
+
 from mforms import newTabView, newBox, newTable, newPanel, TitledBoxPanel, newScrollPanel, newCheckBox, newTextEntry, newTextBox
 from mforms import newLabel, HFillFlag, VFillFlag, HExpandFlag, Utilities, newSelector
-from mforms import newButton, SmallHelpTextStyle, FileChooser, OpenDirectory
+from mforms import newButton, SmallHelpTextStyle, FileChooser, OpenDirectory, OpenFile, SaveFile
 from mforms import SelectorPopup, SelectorCombobox, Form, newImageBox
 import mforms
+
 import wb_admin_config_file_be
-import os
-from wb_common import dprint_ex, debug_level
+from wb_common import dprint_ex, debug_level, PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users
 from wb_admin_config_file_be import multi_separator
-from wb_admin_utils import no_remote_admin_warning_label
-import copy
+from wb_admin_utils import no_remote_admin_warning_label, make_panel_header
+
+from workbench.utils import server_os_path
+
+#from grt.modules.WBAdmin import openRemoteFileSelector
 
 CATOPTS = os.getenv("WB_CATOPTS")
 if CATOPTS is not None:
-    cat_sec = ('General', 'Advanced', 'MyISAM Parameters', 'Performance', 'Log Files', 'Security', 'InnoDB Parameters', 'Networking', 'Replication')
+    cat_sec = ('General', 'Advanced', 'MyISAM Parameters', 'Performance', 'Logging', 'Security', 'InnoDB Parameters', 'Networking', 'Replication')
     cat_grp = ('Networking', 'Advanced log options', 'Slave replication objects', 'Slave default connection values', 'Activate Logging', 'Memory', 'Fulltext search', 'Data / Memory size', 'Datafiles', 'Localization', 'Thread specific settings', 'Advanced', 'Advanced Settings', 'Various', 'Binlog Options', 'Memory usage', 'Directories', 'Logfiles', 'Relay Log', 'Master', 'General slave', 'Security', 'Activate InnoDB', 'Slave Identification', 'Query cache', 'General', 'Insert delayed settings', 'Slow query log options', 'Naming', 'Timeout Settings')
 
     def handle_cat_opt(cat, grp, enabled):
         print "CATOPT", "'" + enabled + "', '" + cat.get_string_value() + "', '" + grp.get_string_value()
 
 # The list of versions to show to user when detected/given version is not supported
-supported_versions = [5.0, 5.1, 5.3, 5.4, 5.5, 6.0]
+supported_versions = [5.0, 5.1, 5.3, 5.4, 5.5, 5.6, 6.0]
 
 #===============================================================================
 def verify_selected_version(version_selector, set_back):
@@ -153,9 +160,16 @@ class Page(object):
 
 #===============================================================================
 class WbAdminConfigFileUI(mforms.Box):
+    @classmethod
+    def wba_register(cls, admin_context):
+        admin_context.register_page(cls, "wba_instance", "Options File", True)
+
+    @classmethod
+    def identifier(cls):
+        return "admin_option_file"
+
         #---------------------------------------------------------------------------
     def page_activated(self):
-        self.main_view.set_content_label(" Options File")
         if not self.ui_created:
             self.ui_created = True
             self.suspend_layout()
@@ -170,30 +184,38 @@ class WbAdminConfigFileUI(mforms.Box):
             self.search_panel.set_enabled(on)
 
     #---------------------------------------------------------------------------
-    def __init__(self, server_profile, ctrl_be, main_view, version="5.1"):
+    def __init__(self, ctrl_be, server_profile, main_view, version="5.1"):
         mforms.Box.__init__(self, False)
+        self.set_managed()
+        self.set_release_on_add()
+
+        self.set_padding(12)
+        self.set_spacing(8)
+
+        self.search_panel = self.create_search_panel()
+
+        self.heading = make_panel_header("title_options.png", server_profile.name, "Options File", self.search_panel)
+        self.add(self.heading, False, True)
+
         self.tab_view = newTabView()
         self.main_view = main_view
 
         self.myopts = None
         self.opt2ctrl_map = {}
+        self.not_multiline_options = set()
         self.loading = True
         self.section = None
         self.ui_created = False
 
-        self.set_managed()
-        self.set_release_on_add()
         self.version = version
         self.ctrl_be = ctrl_be
         self.server_profile = server_profile
         self.cfg_be = None # Will be created later in self.create_ui
         self.version = version
+        self.dir_dict = {}  # Will be filled later when loading default values
 
         self.pack_to_top()
-        container = self.main_view.add_content_page(self, "CONFIGURATION", "Options File", "admin_option_file_win", True)
 
-        self.search_panel = self.create_search_panel()
-        container.add_end(self.search_panel, False, False);
 
     #---------------------------------------------------------------------------
     def create_search_panel(self):
@@ -213,7 +235,6 @@ class WbAdminConfigFileUI(mforms.Box):
         search_box.add(search_btn, False, False)
         search_panel = newPanel(mforms.FilledPanel)
         search_panel.add(search_box)
-        self.main_view.ui_profile.apply_style(search_panel, 'option-search-panel')
 
         def lookup_option_wrapper():
             self.locate_option(self.option_lookup_entry.get_string_value())
@@ -250,6 +271,12 @@ class WbAdminConfigFileUI(mforms.Box):
         options = self.cfg_be.get_options(self.section_ctrl.get_string_value())
         opts_map = dict(options)
 
+        # Fill the dict with dirs to be used in defaults values:
+        self.dir_dict = {'datadir'   : self._get_datadir(), 
+                         'basedir'   : self._get_basedir(),
+                         'configdir' : os.path.dirname(self.server_profile.config_file_path),
+                        }
+
         for group in page_content['groups']:
             controls = group['controls']
 
@@ -432,7 +459,7 @@ class WbAdminConfigFileUI(mforms.Box):
         te = newTextEntry()
         te.set_enabled(False)
         te.add_changed_callback(lambda: self.control_action(name))
-        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be the first char")
+        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be at the start or end of the value")
         return te
 
     #---------------------------------------------------------------------------
@@ -441,7 +468,7 @@ class WbAdminConfigFileUI(mforms.Box):
         dir_box.set_spacing(4)
 
         te = newTextEntry()
-        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be the first char")
+        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be at the start or end of the value")
         btn = newButton()
         btn.set_text("...")
         btn.enable_internal_padding(False)
@@ -454,6 +481,226 @@ class WbAdminConfigFileUI(mforms.Box):
 
         return (dir_box, te, btn)
 
+    #---------------------------------------------------------------------------
+    def _get_datadir(self):
+        if self.server_profile.datadir:
+            return self.server_profile.datadir
+        elif self.cfg_be.original_opts.get('datadir'):
+            return self.cfg_be.original_opts['datadir'].val(0)
+        return ''
+
+    #---------------------------------------------------------------------------
+    def _get_basedir(self):
+        if self.server_profile.basedir:
+            return self.server_profile.basedir
+        elif self.cfg_be.original_opts.get('basedir'):
+            return self.cfg_be.original_opts['basedir'].val(0)
+        return ''
+
+    #---------------------------------------------------------------------------
+    def create_fileedit(self, name, ctrl_def):
+        def download_file_from_server_cb(te):
+            ospath = server_os_path(self.server_profile)
+            remote_path = te.get_string_value().strip('" ')
+            if not remote_path:
+                res = mforms.Utilities.show_error('Specify remote location',
+                    'No remote location available. Please specify the path to the file in the server using '
+                    'the line edit control at the left of the download button and try again. Or you can click '
+                    'on the "Use Suggested" button to let Workbench use an appropriate value for the remote '
+                    'location',
+                    'OK', 'Use Suggested', '')
+                if res == mforms.ResultCancel:
+                    # Use ctrl_def default if available before attempting to build a default value:
+                    if ctrl_def.get('default'):
+                        remote_path = ctrl_def['default'] % self.dir_dict 
+                    else:
+                        remote_path = self._get_datadir()
+                        if remote_path:
+                            remote_path = ospath.abspath(ospath.join(remote_path, '%s.txt' % name))
+                        else:
+                            mforms.Utilities.show_warning('Could not find server data directory',
+                                'Please type the full path to the file in the server before attempting to upload it',
+                                'OK', '', '')
+                            return
+                    te.set_value(remote_path)
+                else:
+                    return
+            elif not ospath.isabs(remote_path):  # file is relative to datadir
+                dirname = self._get_datadir()
+                if dirname:
+                    remote_path = ospath.abspath(ospath.join(dirname, remote_path))
+            file_exists = False
+
+            # Check if remote_path exists in the server before attempting to download it:
+            as_user, user_password = Users.CURRENT, None
+            try:
+                file_exists = self.ctrl_be.server_helper.file_exists(remote_path)
+            except Exception:  # Try again as admin
+                user_password = None
+                while True:
+                    try:
+                        user_password = self.ctrl_be.password_handler.get_password_for("file")
+                        if user_password is not None:
+                            as_user = Users.ADMIN
+                        file_exists = self.ctrl_be.server_helper.file_exists(remote_path, as_user, user_password)
+                    except InvalidPasswordError, e:  # Wrong pwd, retry:
+                        print str(e)
+                        self.ctrl_be.password_handler.reset_password_for("file")
+                        pass
+                    except OperationCancelledError:  # The user chose Cancel in pwd input dialog
+                        return
+                    else:
+                        break
+            if not file_exists:
+                mforms.Utilities.show_error('File does not exist',
+                    'The file "%s" does not exist in %s' % (remote_path, self.server_profile.host_name),
+                    'OK', '', '')
+                return
+            filechooser = FileChooser(SaveFile)
+            filechooser.set_directory(os.path.expanduser('~'))
+            if filechooser.run_modal():
+                content = None
+                try:
+                    content = self.ctrl_be.server_helper.get_file_content(remote_path, as_user, user_password)
+                except PermissionDeniedError, e:
+                    try:
+                        user_password = self.ctrl_be.password_handler.get_password_for("file")
+                    except Exception, e:
+                        return
+                    if user_password is not None:
+                        as_user = Users.ADMIN
+                    try:
+                        content = self.ctrl_be.server_helper.get_file_content(remote_path, as_user, user_password)
+                    except InvalidPasswordError, err:
+                        self.ctrl_be.password_handler.reset_password_for("file")
+                        raise err
+                except Exception, e:
+                    mforms.Utilities.show_error('Error while reading from "%s"' % remote_path,
+                        str(e),
+                        'OK', '', '')
+                    return
+
+                if content is not None:
+                    local_path = filechooser.get_path()
+                    try:
+                        local_file = open(local_path, 'w')
+                    except IOError, e:
+                        mforms.Utilities.show_error('Error while opening "%s" for writing' % local_path,
+                            str(e),
+                            'OK', '', '')
+                        return
+
+                    local_file.write(content)
+                    mforms.App.get().set_status_text('File %s in %s successfully downloaded to %s' % (remote_path,
+                                                                                                      self.server_profile.host_name,
+                                                                                                      local_path)
+                                                    )
+        def upload_file_to_server_cb(te):
+            ospath = server_os_path(self.server_profile)
+            remote_path = te.get_string_value().strip('" ')
+            if not remote_path:
+                res = mforms.Utilities.show_error('Specify remote location',
+                    'No remote location available. Please specify the path to the file in the server using '
+                    'the line edit control at the left of the upload button and try again. Or you can click '
+                    'on the "Use Suggested" button to let Workbench use an appropriate value for the remote '
+                    'location',
+                    'OK', 'Use Suggested', '')
+                if res == mforms.ResultCancel:
+                    # Use ctrl_def default if available before attempting to build a default value:
+                    if ctrl_def.get('default'):
+                        remote_path = ctrl_def['default'] % self.dir_dict
+                    else:
+                        remote_path = self._get_datadir()
+                        if remote_path:
+                            remote_path = ospath.abspath(ospath.join(remote_path, '%s.txt' % name))
+                            te.set_value(remote_path)
+                        else:
+                            mforms.Utilities.show_warning('Could not find server data directory',
+                                'Please type the full path to the file in the server before attempting to upload it',
+                                'OK', '', '')
+                            return
+                    te.set_value(remote_path)
+                else:
+                    return
+            elif not ospath.isabs(remote_path):  # file is relative to datadir
+                dirname = self._get_datadir()
+                if dirname:
+                    remote_path = ospath.abspath(ospath.join(dirname, remote_path))
+            filechooser = FileChooser(OpenFile)
+            filechooser.set_directory(os.path.expanduser('~'))
+            if filechooser.run_modal():
+                local_path = filechooser.get_path()
+                try:
+                    local_file = open(local_path)
+                except IOError, e:
+                    mforms.Utilities.show_error('Error while opening "%s"' % local_path,
+                        str(e),
+                        'OK', '', '')
+                    return
+
+                try:
+                    content = local_file.read()
+                    self.ctrl_be.server_helper.set_file_content(remote_path, content, as_user=Users.CURRENT, user_password=None)
+                except PermissionDeniedError, e:
+                    try:
+                        user_password = self.ctrl_be.password_handler.get_password_for("file")
+                    except Exception, e:
+                        return
+                    if user_password is not None:
+                        as_user = Users.ADMIN
+                    try:
+                        self.ctrl_be.server_helper.set_file_content(remote_path, content, as_user, user_password)
+                    except InvalidPasswordError, err:
+                        self.ctrl_be.password_handler.reset_password_for("file")
+                        raise err
+                except Exception, e:
+                    mforms.Utilities.show_error('Error while uploading the file to "%s"' % remote_path,
+                        str(e),
+                        'OK', '', '')
+                    return
+                mforms.App.get().set_status_text('File %s successfully uploaded to %s as %s' % (local_path,
+                                                                                                self.server_profile.host_name,
+                                                                                                remote_path)
+                                                )
+
+        dir_box = newBox(True)
+        dir_box.set_spacing(4)
+
+        te = newTextEntry()
+        te.set_tooltip('The path to the file in your server')
+        dir_box.add(te, True, True)
+        te.set_enabled(False)
+        
+        btn = newButton()
+        btn.set_text("...")
+        btn.set_tooltip('Browse a file in your server...')
+        btn.enable_internal_padding(False)
+        btn.add_clicked_callback(lambda: self.open_file_chooser(OpenFile, te, name))
+        dir_box.add(btn, False, False)
+
+        btn_dwn = None
+        if ctrl_def['type'] in ('filedownload', 'fileedit'):
+            btn_dwn = newButton(mforms.ToolButton)
+            btn_dwn.set_icon(mforms.App.get().get_resource_path('record_export.png'))
+            btn_dwn.set_tooltip('Download this file to your local computer')
+            btn_dwn.enable_internal_padding(False)
+            btn_dwn.add_clicked_callback(lambda: download_file_from_server_cb(te))
+            dir_box.add(btn_dwn, False, False)
+            btn_dwn.set_enabled(False)
+
+        btn_upl = None
+        if ctrl_def['type'] == 'fileedit':
+            btn_upl = newButton(mforms.ToolButton)
+            btn_upl.set_icon(mforms.App.get().get_resource_path('record_import.png'))
+            btn_upl.set_tooltip('Upload a file from your local computer to the server')
+            btn_upl.enable_internal_padding(False)
+            btn_upl.add_clicked_callback(lambda: upload_file_to_server_cb(te))
+            dir_box.add(btn_upl, False, False)
+            btn_upl.set_enabled(False)
+
+
+        return (dir_box, te, btn, btn_dwn, btn_upl)
+
     #---------------------------------------------------------------------------
     def create_numeric(self, name, ctrl_def):
         #spin_box = newBox(True)
@@ -545,11 +792,21 @@ class WbAdminConfigFileUI(mforms.Box):
             table.add(te, 1, 2, row, row+1, HExpandFlag | HFillFlag)
             ctrl = ('txt', (enabled, te), ctrl_def)
             self.opt2ctrl_map[name] = ctrl
-        elif ctype == "directory" or ctype == "filename" or ctype == "dirname":
-            (dir_box, te, btn) = self.create_dir_file_edit(name, ctrl_def)
+        elif ctype in ['filename', 'fileedit', 'filebrowse', 'filedownload']:
+            if name.endswith('dir'):
+                (dir_box, te, btn) = self.create_dir_file_edit(name, ctrl_def)
+                ctrl = ('dir', (enabled, te, btn), ctrl_def)
+            else:
+                (dir_box, te, btn, btn_dwn, btn_upl) = self.create_fileedit(name, ctrl_def)
+                ctrl = ('fed', (enabled, te, btn, btn_dwn, btn_upl), ctrl_def)
             table.add(dir_box, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
             te.add_changed_callback(lambda: self.control_action(name))
+            self.opt2ctrl_map[name] = ctrl
+        elif ctype in ['directory', 'dirname']:
+            (dir_box, te, btn) = self.create_dir_file_edit(name, ctrl_def)
             ctrl = ('dir', (enabled, te, btn), ctrl_def)
+            table.add(dir_box, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
+            te.add_changed_callback(lambda: self.control_action(name))
             self.opt2ctrl_map[name] = ctrl
         elif ctype == "numeric" or ctype == "spinedit":
             #(spin_box, te, unitcontrol, unit_items) = self.create_numeric(name, ctrl_def)
@@ -560,7 +817,7 @@ class WbAdminConfigFileUI(mforms.Box):
             self.opt2ctrl_map[name] = ctrl
             #table.add(spin_box, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
             table.add(te, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
-        elif ctype == "dropdownbox" or ctype == 'dropdownboxentry':
+        elif ctype == "dropdownbox" or ctype == 'dropdownboxentry' or ctype == "enum":
             if 'choice' not in ctrl_def:
                 te = newTextEntry()
                 te.set_enabled(False)
@@ -573,6 +830,8 @@ class WbAdminConfigFileUI(mforms.Box):
                 table.add(dropbox, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
                 ctrl = ('drp', (enabled, dropbox, items), ctrl_def)
                 self.opt2ctrl_map[name] = ctrl
+        else:
+            raise NotImplementedError("Control type %s not implemented for config file editor"%ctype)
 
         if CATOPTS is None:
             table.add(enabled, 0, 1, row, row + 1, HFillFlag)
@@ -596,14 +855,20 @@ class WbAdminConfigFileUI(mforms.Box):
 
     #---------------------------------------------------------------------------
     def open_file_chooser(self, file_chooser_type, textfield, name):
-        filechooser = FileChooser(file_chooser_type)
-        filechooser.set_directory(textfield.get_string_value())
-        if filechooser.run_modal():
-            filename = filechooser.get_directory()
-            if filename and (type(filename) is str or type(filename) is unicode):
-                filename = filename.replace("\\", "/") # TODO: Check for backslashed spaces and so on
-                textfield.set_value("\"" + filename + "\"")
-                self.control_action(name)
+        filename = None
+        if self.server_profile.is_local:
+            filechooser = FileChooser(file_chooser_type)
+            filechooser.set_directory(textfield.get_string_value())
+            if filechooser.run_modal():
+                filename = filechooser.get_directory() if file_chooser_type is OpenDirectory else filechooser.get_path()
+        else:
+            from wba_ssh_ui import remote_file_selector
+            filename = remote_file_selector(self.server_profile, self.ctrl_be.password_handler, self.ctrl_be.ssh)
+
+        if filename and (type(filename) is str or type(filename) is unicode):
+            filename = filename.replace("\\", "/") # TODO: Check for backslashed spaces and so on
+            textfield.set_value('"' + filename + '"')
+            self.control_action(name)
 
     #---------------------------------------------------------------------------
     def enabled_checkbox_click(self, name, force_enabled = None):
@@ -633,11 +898,19 @@ class WbAdminConfigFileUI(mforms.Box):
             elif tag == "dir":
                 control(1).set_enabled(enabled)
                 control(2).set_enabled(enabled)
+            elif tag == "fed":
+                control(1).set_enabled(enabled)
+                control(2).set_enabled(enabled)
+                try:
+                    control(3).set_enabled(enabled)
+                    control(4).set_enabled(enabled)
+                except Exception:
+                    pass
 
             if not self.loading:
                 # Notify config BE about change
                 if enabled:
-                    self.cfg_be.option_added(name, self.get_string_value_from_control(ctrl), self.section)
+                    self.cfg_be.option_added(name, self.get_string_value_from_control(name, ctrl), self.section)
                 else:
                     self.cfg_be.option_removed(name, self.section)
 
@@ -650,10 +923,10 @@ class WbAdminConfigFileUI(mforms.Box):
             ctrl = self.opt2ctrl_map[name]
 
             if not self.loading:
-                self.cfg_be.option_changed(name, self.get_string_value_from_control(ctrl), self.section)
+                self.cfg_be.option_changed(name, self.get_string_value_from_control(name, ctrl), self.section)
 
     #---------------------------------------------------------------------------
-    def get_string_value_from_control(self, ctrl):
+    def get_string_value_from_control(self, option_name, ctrl):
         #ctrl is a tupple from map
         value = ""
 
@@ -679,7 +952,11 @@ class WbAdminConfigFileUI(mforms.Box):
         elif tag == "dir":
             value = control(1).get_string_value()
             if is_multiple:
-                value = value.split(';')
+                value = value.split(multi_separator)
+        elif tag == "fed":
+            value = control(1).get_string_value()
+            if is_multiple:
+                value = value.split(multi_separator)
         elif tag == "chk":
             value = (control(0).get_active(),)
 
@@ -698,16 +975,18 @@ class WbAdminConfigFileUI(mforms.Box):
             is_string = type(value) is str or type(value) is unicode
             has_separator = value.find(multi_separator) > 0
 
-        if is_multiple == False and is_string and has_separator and not self.loading:
-            answer = Utilities.show_message("Confirm"
-                ,"Multi-line option format entered. Would you like to convert option to multi-line?"
-                , "Convert", "Skip", "")
+        if is_multiple == False and is_string and has_separator and not self.loading and option_name not in self.not_multiline_options:
+            answer = Utilities.show_message("Multi Line Option"
+                ,"The %s sequence was detected in the string, which is used for separating multi-line options. Would you like to split the value to multiple options?" % multi_separator
+                , "Convert", "Keep as Is", "")
             if answer == mforms.ResultOk:
                 control(1).set_name("Multiple")
+            else:
+                self.not_multiline_options.add(option_name)
 
         # some controls return values in form of one-item tuples
         # so we need to extract that item for processing below
-        if has_separator and is_string:
+        if has_separator and is_string and not option_name in self.not_multiline_options:
             if type(value) is tuple:
                 if len(value) == 1:     # Only extract values from one-item tuples
                     value = value[0]      # If tuple has more items it already has been converted to multi-line
@@ -770,11 +1049,15 @@ class WbAdminConfigFileUI(mforms.Box):
         else:
             control(1).set_name("Single")
 
-        if tag == "txt" or tag == "dir":
-            if value is None or value.__class__ != "".__class__:
+        if tag in ['txt', 'dir', 'fed']:
+            if value is None or not isinstance(value, (str, unicode)):
                 # TODO: Add config file error message
                 value = ""
             value = value.strip(" \r\n\t")
+            try:
+                value = value.format(**self.dir_dict)
+            except Exception:
+                pass
             control(1).set_value(value)
         elif tag == "spn":
             if value is None:
@@ -826,6 +1109,7 @@ class WbAdminConfigFileUI(mforms.Box):
 
     #---------------------------------------------------------------------------
     def clear_and_load(self):
+        self.not_multiline_options = set()
         if self.loading == False:
             self.load_defaults()
             self.load_options_from_cfg(self.section_ctrl.get_string_value())
@@ -914,7 +1198,6 @@ class WbAdminConfigFileUI(mforms.Box):
     #-------------------------------------------------------------------------------
     def pack_to_top(self):
         self.suspend_layout()
-        self.main_view.ui_profile.apply_style(self, 'page')
 
         #if self.server_profile.admin_enabled:
         self.file_name_ctrl = newLabel("")
@@ -934,7 +1217,7 @@ class WbAdminConfigFileUI(mforms.Box):
         self.bottom_box = newBox(True)
 
         accept_btn = newButton()
-        accept_btn.set_text("Apply ...")
+        accept_btn.set_text("Apply...")
         accept_btn.add_clicked_callback(self.config_apply_changes_clicked)
 
         discard_btn = newButton()
@@ -959,7 +1242,7 @@ class WbAdminConfigFileUI(mforms.Box):
         
     def shutdown(self):
         if hasattr(self.cfg_be, 'changeset') and self.cfg_be.changeset:
-            res = Utilities.show_warning('Configuration file', '''Configuration file from instance '%s' may contain unsaved changed.
+            res = Utilities.show_warning('Configuration file', '''Configuration file from instance '%s' may contain unsaved changes.
 Would you like to review them?''' % self.server_profile.name, 'Review', 'Cancel', 'Discard')
             if res == mforms.ResultOk:
                 self.config_apply_changes_clicked()
