@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,12 +15,16 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
-from mforms import newBox, newLabel, newButton, newTextEntry, newTreeNodeView, newTable, newRadioButton, newListBox, newSelector, newPanel, newTabView, Utilities, newCheckBox, newImageBox, App
-import mforms
 import re
+import time
 import getpass
+import threading
+import functools
 
-from wb_admin_utils import not_running_warning_label
+import mforms
+
+from mforms import newBox, newLabel, newButton, newTextEntry, newTreeNodeView, newTable, newRadioButton, newListBox, newSelector, newPanel, newTabView, Utilities, newCheckBox, newImageBox, App
+from wb_admin_utils import not_running_warning_label, make_panel_header
 from wb_admin_security_be import AdminSecurity, PrivilegeInfo, PrivilegeReverseDict, SecurityAdminRoles, WBSecurityValidationError
 from wb_common import PermissionDeniedError
 
@@ -53,12 +57,24 @@ SCHEMA_OTHER_RIGHTS = [
 ]
 
 
+DEFAULT_AUTH_PLUGIN = "mysql_native_password"
 AUTHENTICATION_PLUGIN_TYPES = {
+"mysql_native_password" : {"name" : "Standard",
+        "auth_string_label": "",
+        "enable_host": True,
+        "enable_password" : True,
+        },
 None : {"name" : "Standard",
         "auth_string_label": "",
         "enable_host": True,
         "enable_password" : True,
         },
+"mysql_old_password" : {"name" : "Standard (old)",
+        "auth_string_label": "",
+        "enable_host": True,
+        "enable_password" : True,
+        },
+
 "authentication_windows" : { "name": "Windows Native",
         "auth_string_label": "Supply the Windows username and/or group names that are allowed to use this account, separated by a comma (,)\nThis account will be usable by these users whenever they are logged into Windows, without additional passwords.\nExample: Administrator, %s" % (getpass.getuser() or "joe"),
         "enable_host": True,
@@ -69,6 +85,11 @@ None : {"name" : "Standard",
         "enable_host": True,
         "enable_password" : False
         },
+"sha256_password" : {"name": "SHA256 Password",
+        "auth_string_label": "",
+        "enable_host": True,
+        "enable_password" : True
+},
 "*" : {"name": None,
         "auth_string_label": "See the plugin documentation for valid values and details.",
         "enable_host": True,
@@ -87,6 +108,97 @@ def dLabel(text):
     return l
 
 
+
+class ThreadedInputValidator(object):
+    """This class validates the changes in the associated text entry widget displaying the result in a validation label
+
+    Usage:
+        Instanciate this class and set is callback method to be the callback of the associated text entry.
+    """
+    def __init__(self, owner, text_entry, validation_label, ctrl_be, delay=1, colors=('#33aa33', '#aa3333')):
+        self.owner = owner
+        self.text_entry = text_entry
+        self.validation_label = validation_label
+        self.ctrl_be = ctrl_be
+        self.delay = delay
+        self.colors = colors
+
+        self.last_keypress_time = None
+        self.text_to_validate = None
+        self.lock = threading.Lock()
+        self.timer = None
+        self.strength = ['Blank','Weak', 'Medium strength', 'Strong']
+        self.message = '%s password.'
+
+    def update_validation_label(self, estimate):  # Call this from the main thread
+        color = self.colors[0 if estimate > 1 else 1]
+        message = self.strength[0] if estimate == 0 else (self.message % self.strength[estimate])
+        self.validation_label.set_color(color)
+        self.validation_label.set_text(message)
+        self.owner.set_dirty()
+
+    def callback(self):
+        with self.lock:
+            self.last_keypress_time = time.time()
+            self.text_to_validate = self.text_entry.get_string_value()
+
+        if not self.timer or not self.timer.is_alive():
+            self.timer = threading.Timer(self.delay, self.validate)
+            self.timer.start()
+
+    def validate(self):
+        def compute_password_strength(passwd):
+            if passwd == '':
+                return 0
+            if len(passwd) < 8:
+                return 1
+
+            estimate = 1
+            if len(passwd) >= 12:
+                estimate += 1
+            if re.search('\d+', passwd):
+                estimate += 1
+            if re.search('[a-z]', passwd) and re.search('[A-Z]', passwd):
+                estimate += 1
+            if re.search('(\W|_)+', passwd):
+                estimate += 1
+
+            return  1 if estimate < 3 else (
+                    2 if estimate < 5 else
+                    3                      ) 
+
+        def query_password_strength(passwd):
+            if passwd == '':
+                return 0
+
+            result = self.ctrl_be.exec_query("SELECT VALIDATE_PASSWORD_STRENGTH('%s')" % passwd)
+            if result and result.nextRow():
+                estimate = result.intByIndex(1)
+            else:
+                return compute_password_strength(passwd)
+
+            return 1 if estimate < 50 else (
+                   2 if estimate < 75 else
+                   3                       ) 
+
+        query_strength = self.ctrl_be.is_sql_connected() and ('validate_password', 'VALIDATE PASSWORD') in self.ctrl_be.server_active_plugins
+        strength_function = query_password_strength if query_strength else compute_password_strength
+        while True:  # If after processing the current text a new text to validate is present, then process it
+            with self.lock:
+                last_keypress_time = self.last_keypress_time
+                text_to_validate = self.text_to_validate
+
+            if time.time() - last_keypress_time >= self.delay:
+                if text_to_validate is None:  # delay have passed and there is no text to validate
+                    break  # Leave this function thus stopping this thread
+                with self.lock:
+                    self.text_to_validate = None
+                estimate = strength_function(text_to_validate)
+                mforms.Utilities.perform_from_main_thread(functools.partial(self.update_validation_label, estimate=estimate),
+                                                          False)
+            time.sleep(0.2)
+
+
 class AddSchemaPrivilegeForm(mforms.Form):
     def __init__(self, secman, user=""):
         mforms.Form.__init__(self, None, mforms.FormResizable | mforms.FormMinimizable)
@@ -239,6 +351,7 @@ class SecurityAccount(mforms.Box):
         self.set_managed()
         self.set_release_on_add()
         self.owner = owner
+        self.dirty = False
 
         self._selected_user = None
         self._selected_user_original = None
@@ -265,7 +378,7 @@ class SecurityAccount(mforms.Box):
         self.dup_button.add_clicked_callback(self.dup_account)
 
         self.del_button = newButton()
-        self.del_button.set_text("Remove")
+        self.del_button.set_text("Drop")
         bottom_box.add(self.del_button, False, True)
         self.del_button.add_clicked_callback(self.del_account)
 
@@ -284,6 +397,15 @@ class SecurityAccount(mforms.Box):
         bottom_box.add_end(self.revert_button, False, True)
         self.revert_button.add_clicked_callback(self.revert)
 
+        if self.owner.ctrl_be.server_version >= (5, 6, 7):
+            self.expire_button = newButton()
+            self.expire_button.set_text("Expire Password")
+            self.expire_button.set_tooltip("Force user to change password after next login. The user will be unable to issue any command other than SET PASSWORD.")
+            bottom_box.add_end(self.expire_button, False, True)
+            self.expire_button.add_clicked_callback(self.expire)
+        else:
+            self.expire_button = None
+
         self.revoke_all_button = newButton()
         self.revoke_all_button.set_text("Revoke All Privileges")
         bottom_box.add_end(self.revoke_all_button, False, True)
@@ -324,6 +446,7 @@ class SecurityAccount(mforms.Box):
 
         tabView = newTabView(False)
         abox.add(tabView, True, True)
+        self.inner_tabview = tabView
 
         # Login Tab
         vbox = newBox(False)
@@ -345,7 +468,10 @@ class SecurityAccount(mforms.Box):
         self.username.set_max_length(16) # max username length for mysql
         self.password = newTextEntry(mforms.PasswordEntry)
         self.password.set_size(150, -1)
-        self.password.add_changed_callback(self.password_callback)
+        self.password_advice = 'Consider using a password with 8 or more characters with mixed case letters, numbers and punctuation marks.'
+        self.password_label = dLabel(self.password_advice)
+        self.password_validator = ThreadedInputValidator(self, self.password, self.password_label, self.owner.ctrl_be)
+        self.password.add_changed_callback(self.password_validator.callback)
         self.confirm = newTextEntry(mforms.PasswordEntry)
         self.confirm.set_size(150, -1)
         self.confirm.add_changed_callback(self.set_dirty)
@@ -357,7 +483,7 @@ class SecurityAccount(mforms.Box):
         table.add(self.username, 1, 2, 0, 1, mforms.HFillFlag)
         table.add(dLabel("You may create multiple accounts with the same name\nto connect from different hosts."), 2, 3, 0, 1, mforms.HFillFlag|mforms.HExpandFlag)
 
-        self.has_extra_plugins = len([p for p in self.active_plugins if p not in ("mysql_native_password", "mysql_old_password")]) > 0
+        self.has_extra_plugins = len([p for p in self.active_plugins]) > 0
 
         if self.has_extra_plugins:
             self.auth_type_label = rLabel("Authentication Type:")
@@ -367,9 +493,7 @@ class SecurityAccount(mforms.Box):
             table.add(dLabel("\nFor the standard password and/or host based authentication,\nselect 'Standard'.\n"), 2, 3, 1, 2, mforms.HFillFlag)
             self.auth_type_sel.add_changed_callback(self.auth_type_changed)
             self.auth_type_list = []
-            for plugin in [None] + self.active_plugins:
-                if plugin in ("mysql_native_password", "mysql_old_password"):
-                    continue
+            for plugin in self.active_plugins:
                 if AUTHENTICATION_PLUGIN_TYPES.has_key(plugin):
                     self.auth_type_sel.add_item(AUTHENTICATION_PLUGIN_TYPES[plugin]["name"])
                 else:
@@ -379,28 +503,27 @@ class SecurityAccount(mforms.Box):
             self.auth_type_list = None
             self.auth_string_param = None
 
-        self.auth_type_password_labels = []
-        def add(label, l = self.auth_type_password_labels):
-            l.append(label)
-            return label
-
-        table.add(add(rLabel("Limit Connectivity to Hosts Matching:")), 0, 1, 2, 3, mforms.HFillFlag)
+        self.hostlimithost_caption = rLabel("Limit Connectivity to Hosts Matching:")
+        table.add(self.hostlimithost_caption, 0, 1, 2, 3, mforms.HFillFlag)
         table.add(self.hostlimithost, 1, 2, 2, 3, mforms.HFillFlag)
         self.hostlimit_box = newBox(True)
         self.hostlimithost_valid_icon = newImageBox()
         self.hostlimit_box.add(self.hostlimithost_valid_icon, False, False)
-        self.hostlimit_box.add(add(dLabel("% and _ wildcards may be used")), True, True)
+        self.password_caption2 = dLabel("% and _ wildcards may be used")
+        self.hostlimit_box.add(self.password_caption2, True, True)
         table.add(self.hostlimit_box, 2, 3, 2, 3, mforms.HFillFlag|mforms.HExpandFlag)
 
-        table.add(add(rLabel("Password:")), 0, 1, 3, 4, mforms.HFillFlag)
+        self.password_caption = rLabel("Password:")
+        table.add(self.password_caption, 0, 1, 3, 4, mforms.HFillFlag)
         table.add(self.password, 1, 2, 3, 4, mforms.HFillFlag)
-        table.add(add(dLabel("Type a password to reset it.")), 2, 3, 3, 4, mforms.HFillFlag|mforms.HExpandFlag)
-        self.password_advice = 'Consider using a password with 8 or more characters combining upper and lower-case letters, numbers and punctuation marks.'
-        self.password_label = add(dLabel(self.password_advice))
+        self.password_caption2 = dLabel("Type a password to reset it.")
+        table.add(self.password_caption2, 2, 3, 3, 4, mforms.HFillFlag|mforms.HExpandFlag)
         table.add(self.password_label, 1, 3, 4, 5, mforms.HFillFlag|mforms.HExpandFlag)
-        table.add(add(rLabel("Confirm Password:")), 0, 1, 5, 6, mforms.HFillFlag)
+        self.confirm_caption = rLabel("Confirm Password:")
+        table.add(self.confirm_caption, 0, 1, 5, 6, mforms.HFillFlag)
         table.add(self.confirm, 1, 2, 5, 6, mforms.HFillFlag)
-        table.add(add(dLabel("Enter password again to confirm.")), 2, 3, 5, 6, mforms.HFillFlag|mforms.HExpandFlag)
+        self.confirm_caption2 = dLabel("Enter password again to confirm.")
+        table.add(self.confirm_caption2, 2, 3, 5, 6, mforms.HFillFlag|mforms.HExpandFlag)
 
         table.add(newLabel(""), 1, 2, 6, 7, mforms.HFillFlag)
 
@@ -408,9 +531,10 @@ class SecurityAccount(mforms.Box):
             self.auth_string_param = newTextEntry()
             self.auth_string_param.add_changed_callback(self.set_dirty)
             self.auth_string_label = rLabel("Authentication String:")
+            self.auth_string_desc = dLabel("Authentication plugin specific parameters.")
             table.add(self.auth_string_label, 0, 1, 7, 8, mforms.HFillFlag)
             table.add(self.auth_string_param, 1, 2, 7, 8, mforms.HFillFlag|mforms.HExpandFlag)
-            table.add(dLabel("Authentication plugin specific parameters."), 2, 3, 7, 8, mforms.HFillFlag)
+            table.add(self.auth_string_desc, 2, 3, 7, 8, mforms.HFillFlag)
             self.auth_string_help = newLabel("")
             vbox.add(self.auth_string_help, False, True)
 
@@ -493,30 +617,40 @@ class SecurityAccount(mforms.Box):
             return self.auth_type_list[max(self.auth_type_sel.get_selected_index(), 0)]
         return None
 
-    def update_enable_state_for_auth_type(self, auth_type):
+    def update_enable_state_for_auth_type(self, auth_type, is_new_user):
         info = AUTHENTICATION_PLUGIN_TYPES.get(auth_type)
         if not info:
             info = AUTHENTICATION_PLUGIN_TYPES["*"]
 
         if info:
+            self.password_caption.set_enabled(info["enable_password"])
+            self.password_caption2.set_enabled(info["enable_password"])
+            self.password_label.set_enabled(info["enable_password"])
             self.password.set_enabled(info["enable_password"])
             self.confirm.set_enabled(info["enable_password"])
+            self.confirm_caption.set_enabled(info["enable_password"])
+            self.confirm_caption2.set_enabled(info["enable_password"])
             self.hostlimithost.set_enabled(info["enable_host"])
+            self.hostlimithost_caption.set_enabled(info["enable_host"])
 
             if self.auth_string_param:
-                self.auth_string_help.set_text(info["auth_string_label"])
-
-                if auth_type is None:
-                    self.auth_string_label.set_enabled(False)
-                    self.auth_string_param.set_enabled(False)
-                    for label in self.auth_type_password_labels:
-                        label.set_enabled(True)
+                if info["auth_string_label"]:
+                    self.auth_string_help.show(True)
+                    self.auth_string_help.set_text(info["auth_string_label"])
+                    self.auth_string_label.show(True)
+                    self.auth_string_param.show(True)
+                    self.auth_string_desc.show(True)
+                    if is_new_user:
+                        self.auth_string_label.set_enabled(True)
+                        self.auth_string_param.set_enabled(True)
+                    else:
+                        self.auth_string_label.set_enabled(False)
+                        self.auth_string_param.set_enabled(False)
                 else:
-                    self.auth_string_label.set_enabled(True)
-                    self.auth_string_param.set_enabled(True)
-                    for label in self.auth_type_password_labels:
-                        label.set_enabled(False)
-
+                    self.auth_string_help.show(False)
+                    self.auth_string_label.show(False)
+                    self.auth_string_param.show(False)
+                    self.auth_string_desc.show(False)
 
     def set_hostlimithost(self, value):
         self.hostlimithost.set_value(value)
@@ -532,7 +666,8 @@ class SecurityAccount(mforms.Box):
             return
 
         auth_type = self.selected_plugin_type()
-        self.update_enable_state_for_auth_type(auth_type)
+        # this method is only called for new users (because auth-type can't be changed for existing users)
+        self.update_enable_state_for_auth_type(auth_type, True)
 
         info = AUTHENTICATION_PLUGIN_TYPES.get(auth_type)
         if not info:
@@ -546,9 +681,6 @@ class SecurityAccount(mforms.Box):
             if not info["enable_host"]:
                 self.set_hostlimithost("%")
 
-            if auth_type is None and self.auth_string_param:
-                self.auth_string_param.set_value("")
-
         self.set_dirty()
 
     def refresh_role_list(self):
@@ -626,21 +758,21 @@ class SecurityAccount(mforms.Box):
         self._selected_user = user
         self._selected_user_original = user and user.snapshot_for_revert()
 
-        self.password_label.set_color('#000000')
-
         if user:
             self.dup_button.set_enabled(True)
             self.del_button.set_enabled(True)
+            if self.expire_button:
+                self.expire_button.set_enabled(not user.password_expired)
 
             self.username.set_value(user.username)
-            self.password.set_value("")
-            self.confirm.set_value("")
+            self.password.set_value(user.password or "")
+            self.confirm.set_value(user.password or "")
 
             self.set_hostlimithost(user.host)
 
             if self.auth_type_list:
                 try:
-                    index = self.auth_type_list.index(user.auth_plugin or None)
+                    index = self.auth_type_list.index(user.auth_plugin or DEFAULT_AUTH_PLUGIN)
                 except ValueError:
                     mforms.Utilities.show_warning("Invalid Authentication Plugin",
                               "User %s has plugin type %s, which is not listed as a known authentication plugin by the server." % (user.formatted_name(), user.auth_plugin),
@@ -651,7 +783,7 @@ class SecurityAccount(mforms.Box):
                 # changing auth type not supported atm
                 self.auth_type_sel.set_enabled(not user.is_commited)
                 self.auth_string_param.set_value(user.auth_string or "")
-                self.update_enable_state_for_auth_type(user.auth_plugin or None)
+                self.update_enable_state_for_auth_type(user.auth_plugin or None, not user.is_commited)
 
             self.max_questions.set_value(str(user.max_questions))
             self.max_updates.set_value(str(user.max_updates))
@@ -659,11 +791,13 @@ class SecurityAccount(mforms.Box):
             self.max_uconnections.set_value(str(user.max_user_connections))
 
             self.refresh_priv_list()
-            self.refresh_role_list()
+            self.refresh_role_list()              
         else:
             self.username.set_value("")
             self.password.set_value("")
             self.confirm.set_value("")
+            if self.expire_button:
+                self.expire_button.set_enabled(False)
 
             self.set_hostlimithost("")
 
@@ -680,6 +814,13 @@ class SecurityAccount(mforms.Box):
                 if role_list_node:
                     self.role_list.node_at_row(i).set_bool(0, False)
 
+        if user and user.password_expired:
+            self.password_label.set_text("Password is expired. User must change password to use the account.")
+            self.password_label.set_color('#aa3333')
+        else:
+            self.password_label.set_text(self.password_advice)
+            self.password_label.set_color('#000000')
+
 
     def _find_user_position(self, user, host):
         users_count = len(self.owner.secman.account_names)
@@ -692,6 +833,7 @@ class SecurityAccount(mforms.Box):
     def add_account(self):
         account = self.owner.secman.create_account()
         self.refresh()
+        self.inner_tabview.set_active_tab(0)
         pos = self._find_user_position(account.username, account.host)
         if pos is not None and pos >= 0:
             self.user_list.select_node(self.user_list.node_at_row(pos))
@@ -711,11 +853,15 @@ class SecurityAccount(mforms.Box):
 
     def del_account(self):
         if self._selected_user:
-            if not self._selected_user.is_commited or Utilities.show_message("Remove Account",
+            if not self._selected_user.is_commited or Utilities.show_message("Drop Account",
                   "The account '%s' will be permanently removed. Please confirm."%(self._selected_user.formatted_name()),
-                  "Remove", "Cancel", "") == mforms.ResultOk:
+                  "Drop", "Cancel", "") == mforms.ResultOk:
                 the_name = self._selected_user.formatted_name()
-                self.owner.secman.delete_account(self._selected_user)
+                try:
+                    self.owner.secman.delete_account(self._selected_user)
+                except Exception, e:
+                    title, message = e.args[:2] if len(e.args) > 1 else ('Error:', str(e))
+                    Utilities.show_error(title, message, 'OK', '', '')
                 self._selected_user = None
                 self._selected_user_original = None
                 self.owner.do_refresh()
@@ -747,7 +893,12 @@ class SecurityAccount(mforms.Box):
             self.user_list.select_node(selected_row)
             self.user_selected()
 
-        self.password_label.set_text(self.password_advice)
+        if su and su.password_expired:
+            self.password_label.set_text("Password is expired. User must change password to use the account.")
+            self.password_label.set_color('#aa3333')
+        else:
+            self.password_label.set_text(self.password_advice)
+            self.password_label.set_color('#000000')
 
 
     def refresh_priv_list(self):
@@ -762,13 +913,28 @@ class SecurityAccount(mforms.Box):
 
     def validate_hostlimithost(self):
         host = self.hostlimithost.get_string_value()
-        self.valid_name = False
-        if len(host) <= 255:
-            if host[-1:] == ".":
-                host = host[:-1]
-            allowed = re.compile("(?!-)[_%/A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
+        self.valid_name = True
+        if len(host) > 255:
+            self.valid_name = False
+
+        subnet_mask = ''
+        if host[-1:] == ".":
+            host = host[:-1]
+        elif '/' in host:
+            host, _, subnet_mask = host.rpartition('/')
+            if not subnet_mask:
+                self.valid_name = False
+
+        if self.valid_name:
+            allowed = re.compile(r"(?!-)[A-Z%_\d-]{1,63}(?<!-)$", re.IGNORECASE)
             self.valid_name = all(allowed.match(x) for x in host.split("."))
 
+        if self.valid_name and subnet_mask:
+            self.valid_name = ( (subnet_mask.isdigit() and 0 <= int(subnet_mask) <= 32) or
+                                (re.match(r'\d{1,3}(\.\d{1,3}){3}', subnet_mask) and            # 255.255.254.0 is a valid netmask
+                                 all( int(item) <= 255 for item in subnet_mask.split('.') ) )
+                              )
+
         if self.valid_name:
             #self.set_dirty()
             self.hostlimithost_valid_icon.show(False)
@@ -782,31 +948,41 @@ class SecurityAccount(mforms.Box):
         self.save_button.set_enabled(True)
         self.revert_button.set_enabled(True)
         self.user_list.set_enabled(False)
+        self.dirty = True
 
 
     def unset_dirty(self):
         self.save_button.set_enabled(False)
         self.revert_button.set_enabled(False)
         self.user_list.set_enabled(True)
+        self.dirty = False
 
+  
+    def expire(self):
+        if self._selected_user:
+            self._selected_user.expire_password()
+        self.refresh()
 
+  
     def revoke_all(self):
         if self._selected_user:
             if Utilities.show_message("Revoke All Privileges",
                   "Please confirm revokation of all privileges for the account '%s'@'%s'.\nNote: the account itself will be maintained."%(self._selected_user.username, self._selected_user.host),
                   "Revoke", "Cancel", "") == mforms.ResultOk:
-                self._selected_user.revoke_all()
-                self._selected_user.load(self._selected_user.username, self._selected_user.host)
-                self.show_user(self._selected_user)
+                try:
+                    self._selected_user.revoke_all()
+                    self._selected_user.load(self._selected_user.username, self._selected_user.host)
+                    self.show_user(self._selected_user)
+                except Exception, e:
+                    title, message = e.args[:2] if len(e.args) > 1 else ('Error:', str(e))
+                    Utilities.show_error(title, message, 'OK', '', '')
 
 
     def revert(self):
-        self.password_label.set_color('#000000')
         if self._selected_user_original:
             self.show_user(self.owner.secman.revert_account(self._selected_user, self._selected_user_original))
         if not self._selected_user.is_commited:
             self.owner.secman.delete_account(self._selected_user)
-        self.password_label.set_text(self.password_advice)
         self.refresh()
 
 
@@ -814,12 +990,17 @@ class SecurityAccount(mforms.Box):
         if self._selected_user:
             username = self.username.get_string_value()
             host = self.hostlimithost.get_string_value()
+            if not self.valid_name:
+                Utilities.show_error('Invalid host specification',
+                    'The host specification "%s" is not valid. Please correct it and try again.' % host,
+                                    'OK', '', '' )
 
             is_new_user = not self._selected_user.is_commited
 
             password_unneeded = False
-            self.password_label.set_text(self.password_advice)
-            if self.has_extra_plugins and not AUTHENTICATION_PLUGIN_TYPES[self.selected_plugin_type()]["enable_password"]:
+            self.password_label.set_text("Password is expired. User must change password to use the account." if self._selected_user.password_expired else self.password_advice)
+            plugin_info = AUTHENTICATION_PLUGIN_TYPES.get(self.selected_plugin_type(), {})
+            if self.has_extra_plugins and not plugin_info.get("enable_password", True):
                 password_unneeded = True
             if is_new_user and not self.password.get_string_value() and not password_unneeded:
                 if Utilities.show_warning("Save Account Changes",
@@ -873,7 +1054,8 @@ class SecurityAccount(mforms.Box):
             if is_new_user and self.has_extra_plugins:
                 self._selected_user.auth_plugin = self.selected_plugin_type()
 
-            if self._selected_user.auth_plugin:
+            self._selected_user.auth_string = None
+            if self._selected_user.auth_plugin and AUTHENTICATION_PLUGIN_TYPES[self._selected_user.auth_plugin]["auth_string_label"]:
                 self._selected_user.auth_string = self.auth_string_param.get_string_value()
 
             try:
@@ -894,6 +1076,7 @@ class SecurityAccount(mforms.Box):
             the_name = self._selected_user.formatted_name()
 
             self._selected_user.load(self._selected_user.username, self._selected_user.host)
+            self.unset_dirty()
             self.owner.refresh()
 
             if is_new_user:
@@ -901,35 +1084,6 @@ class SecurityAccount(mforms.Box):
             else:
                 mforms.App.get().set_status_text("Updated account '%s'" % the_name)
 
-    def password_callback(self):
-        self.password_label.set_color('#aa3333')
-        self.password_label.set_text(self.estimate_password_strength())
-        self.set_dirty()
-
-    def estimate_password_strength(self, password=None):
-        passwd = password if password is not None else self.password.get_string_value()
-        strength = ['Blank','Weak', 'Medium strength', 'Strong']
-        message = '%s password.'
-
-        if passwd == '':
-            return strength[0]
-        if len(passwd) < 8:
-            return message % strength[1]
-
-        estimate = 1
-        if len(passwd) >= 12:
-            estimate += 1
-        if re.search('\d+', passwd):
-            estimate += 1
-        if re.search('[a-z]', passwd) and re.search('[A-Z]', passwd):
-            estimate += 1
-        if re.search('(\W|_)+', passwd):
-            estimate += 1
-
-        return message % strength[ 1 if estimate < 3 else (
-                                   2 if estimate < 5 else
-                                   3                      ) ]
-
 
 #############################
 
@@ -940,6 +1094,7 @@ class SecuritySchemaPrivileges(mforms.Box):
         self.set_managed()
         self.set_release_on_add()
         self.owner = owner
+        self.dirty = False
 
         self._selected_user = None
         self._selected_user_original = None
@@ -948,20 +1103,13 @@ class SecuritySchemaPrivileges(mforms.Box):
         self.set_spacing(8)
         self.set_padding(8)
 
-        schema_list_box = newBox(False)
-        schema_list_box.set_spacing(8)
-        schema_list_box.set_size(150, -1)
-        self.add(schema_list_box, False, True)
-
-        #searchbox = TextEntry(SearchEntry)
-        #schema_list_box.add(searchbox, False, True)
-
         self.user_list = newTreeNodeView(mforms.TreeFlatList)
         self.user_list.add_column(mforms.StringColumnType, "Users", 140, False)
         self.user_list.end_columns()
         self.user_list.add_changed_callback(self.user_selected)
         self.user_list.set_allow_sorting(True)
-        schema_list_box.add(self.user_list, True, True)
+        self.user_list.set_size(150, -1)
+        self.add(self.user_list, False, True)
 
         self.schema_rights_checks = {}
 
@@ -1100,12 +1248,14 @@ class SecuritySchemaPrivileges(mforms.Box):
         self.save_button.set_enabled(True)
         self.revert_button.set_enabled(True)
         self.user_list.set_enabled(False)
+        self.dirty = True
 
 
     def unset_dirty(self):
         self.save_button.set_enabled(False)
         self.revert_button.set_enabled(False)
         self.user_list.set_enabled(True)
+        self.dirty = False
 
 
     def schema_priv_checked(self):
@@ -1297,6 +1447,9 @@ class SecuritySchemaPrivileges(mforms.Box):
                 Utilities.show_error("Save Account Changes",
                       str(exc), "OK", "", "")
                 return
+            except Exception, e:
+                title, message = e.args[:2] if len(e.args) > 1 else ('Error:', str(e))
+                Utilities.show_error(title, message, 'OK', '', '')
             self.unset_dirty()
 
 
@@ -1306,20 +1459,33 @@ class WbAdminSecurity(mforms.Box):
     _schema_priv_entries = []
     ui_created = False
 
+    @classmethod
+    def wba_register(cls, admin_context):
+        admin_context.register_page(cls, "wba_management", "Users and Privileges")
+
+    @classmethod
+    def identifier(cls):
+        return "admin_manage_privs"
+
     def __init__(self, ctrl_be, server_profile, main_view):
         mforms.Box.__init__(self, False)
-        main_view.ui_profile.apply_style(self, 'page')
         self.set_managed()
         self.set_release_on_add()
         self.ctrl_be = ctrl_be
         self.secman  = None
+        self.server_profile = server_profile
         self.main_view = main_view
-        self.main_view.add_content_page(self, "SECURITY", "Users and Privileges", "admin_manage_privs_win")
 
 
     def create_ui(self):
         self.suspend_layout()
 
+        self.set_padding(12)
+        self.set_spacing(8)
+
+        self.heading = make_panel_header("title_users.png", self.server_profile.name, "Users and Privileges")
+        self.add(self.heading, False, True)
+
         self.warning = not_running_warning_label()
         self.add(self.warning, False, True)
 
@@ -1335,6 +1501,12 @@ class WbAdminSecurity(mforms.Box):
         self.resume_layout()
 
 
+    def show_no_permission(self):
+        self.warning.set_text("\n\n\n\nThe account you are currently using does not have sufficient privileges to make changes to MySQL users and privileges.")
+        self.warning.show(True)
+        self.tabview.show(False)
+
+
     def update_ui(self):
         if self.ctrl_be.is_sql_connected():
             self.secman = AdminSecurity(self.ctrl_be)
@@ -1350,7 +1522,6 @@ class WbAdminSecurity(mforms.Box):
 
 
     def page_activated(self):
-        self.main_view.set_content_label(" Users and Privileges")
         if not self.ui_created:
             self.create_ui()
             self.ui_created = True
@@ -1391,11 +1562,13 @@ class WbAdminSecurity(mforms.Box):
             try:
                 self.secman.async_refresh(self.do_refresh)
             except PermissionDeniedError, e:
-                Utilities.show_error("Access Denied", str(e), "OK", "", "")
+                self.show_no_permission()
 
 
 
     def do_refresh(self):
-        self.account_tab.refresh()
+        if not self.account_tab.dirty:
+            self.account_tab.refresh()
 
-        self.schema_privs_tab.refresh()
+        if not self.schema_privs_tab.dirty:
+            self.schema_privs_tab.refresh()
