@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,8 +19,6 @@
 
 #include "stdafx.h"
 
-#include "wb_config.h"
-
 #include "grtpp.h"
 #include "grt/common.h"
 
@@ -33,7 +31,7 @@
 #include "wb_module_printing.h"
 
 #include "mdc_canvas_view_printing.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 
 #ifdef _WIN32
