@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -22,6 +22,7 @@ import cairo_utils
 from cairo_utils import Context, ImageSurface, Pattern
 from cairo_utils import VBoxNode, TextRectangle, intersect_lines, draw_arrow_head
 
+import grt
 
 
 class TreeNode(VBoxNode):
@@ -63,6 +64,29 @@ class TreeNode(VBoxNode):
         return 0
 
 
+class PlainTextNode(VBoxNode):
+    def __init__(self, parent, name, title, data):
+        VBoxNode.__init__(self)
+        self.padding = (0, 0, 0, 0)
+        self.expand_to_fill = False
+        self.name = name
+        self.parent = parent
+        self.data = data
+        self.children = []
+        self.content = TextRectangle(title)
+        self.content.set_border_color(1, 1, 1, 0)
+        self.content.set_fill_color(1, 1, 1, 0)
+        self.items.append(self.content)
+        self.extra_bottom_space = 0
+
+    def __repr__(self):
+        return "<node %s>"%self.content.text
+
+    def process(self):
+        for ch in self.children:
+            ch.process()
+
+
 class OperationNode(TreeNode):
     def __init__(self, parent, name, title, data):
         TreeNode.__init__(self, parent, name, title, data)
@@ -144,8 +168,9 @@ class SubqueryNode(TreeNode):
 
 
 class TableTreeNode(TreeNode):
-    def __init__(self, parent, name, title, data):
+    def __init__(self, context, parent, name, title, data):
         TreeNode.__init__(self, parent, name, title, data)
+        self.context = context
         self.border_color = 125.0/255, 125.0/255, 125.0/255, 1
         self.heading = self.content
         self.heading.border_color = None
@@ -154,6 +179,7 @@ class TableTreeNode(TreeNode):
         self.heading.set_color(1, 1, 1, 1)
         self.body = None
         self.flags_box = None
+        self.attached_text = None
         flags = []
         if data.has_key("using_join_buffer"):
             flags.append("join buf (%s)" % data["using_join_buffer"])
@@ -201,7 +227,18 @@ class TableTreeNode(TreeNode):
             x, y = (p1[0]+p2[0])/2, (p1[1]+p2[1])/2
             cairo_utils.show_centered_text_with_background(c, x, y, total, (1,1,1))
 
- 
+
+    def process(self):
+        TreeNode.process(self)
+
+        attached_condition = self.data.get("attached_condition", "")
+        if attached_condition:
+            self.context["attached_conditions"].append((self.name, attached_condition))
+            i = len(self.context["attached_conditions"])
+            condition_length = self.context.get("opt_condition_length", 32)
+            self.attached_text = "attached_condition[%i]: %s%s" % (i, attached_condition[:condition_length], "..." if len(attached_condition) > condition_length else "")
+
+
     def do_render_attachments(self, c):
         c.set_line_width(self.line_width)
         c.save()
@@ -210,28 +247,20 @@ class TableTreeNode(TreeNode):
         w, h = self.size
         if self.data.has_key("rows"):
             cairo_utils.show_centered_text_with_background(c, self.center()[0], y-12, str(self.data["rows"]), (1,1,1))
-        cond = self.data.get("attached_condition", None)
-        if cond:
+
+        if self.attached_text:
             c.set_font_size(9)
-            suffix = ""
-            while len(cond) > 10:
-                ext = c.text_extents(cond)
-                if ext.width > self.gsize[0]*1.2:
-                    cond = cond[:-1]
-                    suffix = "..."
-                else:
-                    break
-            ext = c.text_extents(cond+suffix)
+            ext = c.text_extents(self.attached_text)
             padding = 4
             c.set_source_rgba(0xcc/255.0, 0xcc/255.0, 0xcc/255.0, 1)
-            c.rounded_rect(x + 10, y + self.gsize[1], ext.width + 2*padding, ext.height + 2*padding, 7)
+            c.rounded_rect(x + 10, y + self.size[1], ext.width + 2*padding, ext.height + 2*padding, 7)
             c.stroke_preserve()
             c.set_source_rgba(0xeb/255.0, 0xeb/255.0, 0xeb/255.0, 1)
             c.fill()
 
             c.set_source_rgba(0x30/255.0, 0x30/255.0, 0x30/255.0, 1)
-            c.move_to(int(x + 10 + padding) + 0.5, int(y+self.gsize[1] + ext.height - (ext.height + ext.y_bearing) + padding) + 0.5)
-            c.show_text(cond+suffix)
+            c.move_to(int(x + 10 + padding) + 0.5, int(y+self.size[1] + ext.height - (ext.height + ext.y_bearing) + padding) + 0.5)
+            c.show_text(self.attached_text)
 
         c.restore()
 
@@ -266,6 +295,7 @@ class TableTreeNode(TreeNode):
         if self.references:
             self.extra_bottom_space = 50
 
+
     def stroke_ref_to(self, c, ch):
         p1 = ch.center()
         p2 = self.center()
@@ -315,8 +345,8 @@ class TableTreeNode(TreeNode):
 
 
 class IndexedTableTreeNode(TableTreeNode):
-    def __init__(self, parent, name, title, data):
-        TableTreeNode.__init__(self, parent, name, title, data)
+    def __init__(self, context, parent, name, title, data):
+        TableTreeNode.__init__(self, context, parent, name, title, data)
 
 
     def process(self):
@@ -346,6 +376,13 @@ class IndexedTableTreeNode(TableTreeNode):
         else:
             self.set_is_key_ref(False)
 
+        index_condition = self.data.get("index_condition", "")
+        if index_condition:
+            self.context["index_conditions"].append((self.name, index_condition))
+            i = len(self.context["index_conditions"])
+            condition_length = self.context.get("opt_condition_length", 32)
+            self.attached_text = "index_condition[%i]: %s%s" % (i, index_condition[:condition_length], "..." if len(index_condition) > condition_length else "")
+
 
     def set_is_key_ref(self, flag):
         self.is_key_ref = flag
@@ -353,23 +390,30 @@ class IndexedTableTreeNode(TableTreeNode):
         keys = self.data.get("possible_keys", [])
         key_length = self.data.get("key_length", "")
         if key_length:
-            key_length = "[%s]"%key_length
+            key_length = "key length: %s" % key_length
+        used_key_parts = self.data.get("used_key_parts", "")
+        if used_key_parts:
+            used_key_parts = "used parts: %s" % ", ".join(used_key_parts)
+        used_key_info = "\n    %s" % "\n    ".join(filter(None, [key_length, used_key_parts]))
+
         # if ref is not an external column name, then we add the ref value next to the key
         if flag or not self.data.has_key("ref"):
             if key in keys:
                 keys.remove(key)
-            keys.append("* %s%s"%(key, key_length))
+            text = "key: %s%s"%(key, used_key_info)
         else:
             if key in keys:
                 keys.remove(key)
-            keys.append("* %s%s -> %s"%(key, key_length, ", ".join(self.data["ref"])))
-        self.set_body_text("\n".join(keys))
-
+            text = "key: %s -> %s%s"%(key, ", ".join(self.data["ref"]), used_key_info)
+        if keys:
+            self.set_body_text(text + "\npossible keys:\n    " + "\n    ".join(keys))
+        else:
+            self.set_body_text(text)
 
 
 class MaterializedTableTreeNode(TableTreeNode):
-    def __init__(self, parent, name, title, data):
-        TableTreeNode.__init__(self, parent, name, title, data)
+    def __init__(self, context, parent, name, title, data):
+        TableTreeNode.__init__(self, context, parent, name, title, data)
         self.expand_to_fill = True
         self.padding = 0, 10, 10, 10
         self.set_body_text("SUBQUERY, materialized from")
@@ -433,11 +477,18 @@ class MaterializedTableTreeNode(TableTreeNode):
 class TreeLayouter:
     def __init__(self, root, options):
         self.root = root
+        self.bottom_items = []
         self.yspacing = options.get("yspacing", 60)
         self.xspacing = options.get("xspacing", 30)
+        self.bottom_spacing = 30
+        self.bottom_node_spacing = 10
         self.vertical = options.get("vertical", True)
 
-    
+
+    def add_bottom_node(self, node):
+        self.bottom_items.append(node)
+
+
     def layout(self, ctx, node, x = 0, y = 0):
         t, l, b, r = node.padding
         
@@ -553,9 +604,21 @@ class TreeLayouter:
             for ch in node.children:
                 calc_all(c, ch)
         calc_all(ctx, self.root)
- 
-        return self.layout(ctx, self.root, 0, 0)
 
+        # calculate extra space needed by the items added below the main tree
+        max_width = 0
+        extra_height = self.bottom_spacing if self.bottom_items else 0
+        for node in self.bottom_items:
+            calc_all(ctx, node)
+            max_width = max(max_width, node.size[0])
+            extra_height += node.size[1] + self.bottom_node_spacing
+
+        main_width, main_height = self.layout(ctx, self.root, 0, 0)
+
+        main_width = max(main_width, max_width)
+        main_height += extra_height
+
+        return main_width, main_height
 
     def render(self, c, x, y):
         def calc_all(c, node):
@@ -563,10 +626,23 @@ class TreeLayouter:
             for ch in node.children:
                 calc_all(c, ch)
         calc_all(c, self.root)
-        self.layout(c, self.root, x, y)
+
+        for node in self.bottom_items:
+            calc_all(c, node)
+
+        # draw the main part
+        width, height = self.layout(c, self.root, x, y)
         self.do_render(c, self.root)
         self.do_render_lines(c, self.root)
 
+        # draw the extras added below
+        ey = height + self.yspacing + self.bottom_spacing
+        for node in self.bottom_items:
+            node.pos = x, ey
+            self.do_render(c, node)
+            ey += node.size[1] + self.bottom_node_spacing
+
+
     def do_render_lines(self, c, node):
         for ch in node.children:
             ch.stroke_line_to_parent(c, node, self.vertical)
@@ -593,7 +669,7 @@ def decode_json(text):
 
 
 
-def process_table(parent, table):
+def process_table(context, parent, table):
     # Possible col_join_types (access_type) according to mysql 5.6.5 sources
     col_join_types = [
     ("UNKNOWN",         (1, 0, 0, 1),           TableTreeNode),
@@ -603,7 +679,7 @@ def process_table(parent, table):
     ("index",           (1, 0.5, 0, 1),         IndexedTableTreeNode),
     ("ALL",             (0.75, 0.25, 0.25, 1),  TableTreeNode),
     ("range",           (0.0, 0.5, 0.25, 1),    IndexedTableTreeNode),
-    ("ref",             (0.0, 0.5, 0.25, 1),      IndexedTableTreeNode),
+    ("ref",             (0.0, 0.5, 0.25, 1),    IndexedTableTreeNode),
     ("fulltext",        (1, 0.5, 0, 1),         TableTreeNode),
     ("ref_or_null",     None,                   TableTreeNode),
     ("unique_subquery", None,                   TableTreeNode),
@@ -615,7 +691,7 @@ def process_table(parent, table):
     table_name = table.get("table_name", "")
 
     if not access_type:
-        node = TableTreeNode(parent, table_name, table.get("message", "???"), table)
+        node = TableTreeNode(context, parent, table_name, table.get("message", "???"), table)
         node.set_fill_color(0.25, 0.5, 0.75, 1)
         return node
 
@@ -630,7 +706,7 @@ def process_table(parent, table):
     if table.has_key("materialized_from_subquery"):
       tableClass = MaterializedTableTreeNode
 
-    node = tableClass(parent, table_name, table_name+"  (%s)"%access_type, table)
+    node = tableClass(context, parent, table_name, table_name+"  (%s)"%access_type, table)
     node.set_fill_color(*tableColor)
     return node
 
@@ -659,9 +735,9 @@ def process_node(parent, name, data):
     return node
 
 
-def tree_from_json(parent, name, json):
+def tree_from_json(context, parent, name, json):
     if name == "table":
-        node = process_table(parent, json)
+        node = process_table(context, parent, json)
     elif name.endswith("_subqueries item"):
         node = process_subquery(parent, json)
     else:
@@ -669,10 +745,10 @@ def tree_from_json(parent, name, json):
     if not node:
         return None
     for key, value in json.items():
-        if key in ["possible_keys", "ref"]:
+        if key in ["possible_keys", "ref", "used_key_parts"]:
             continue
         if type(value) is dict:
-            ch = tree_from_json(node, key, value)
+            ch = tree_from_json(context, node, key, value)
             if ch:
                 node.children.append(ch)
         elif type(value) is list:
@@ -681,14 +757,14 @@ def tree_from_json(parent, name, json):
                 interm.set_fill_color(0.83137254901960789, 0.92941176470588238, 0.99215686274509807, 1)
                 node.children.append(interm)
                 for item in value:
-                    ch = tree_from_json(interm, "table", item["table"])
+                    ch = tree_from_json(context, interm, "table", item["table"])
                     if ch:
                         interm.children.append(ch)
             else:
                 interm = TreeNode(node, key, key, {})
                 node.children.append(interm)
                 for item in value:
-                    ch = tree_from_json(interm, key+" item", item)
+                    ch = tree_from_json(context, interm, key+" item", item)
                     if ch:
                         interm.children.append(ch)
     return node
@@ -714,14 +790,46 @@ def strip_useless_nodes(tree):
     return tree
 
 
+def prepare_layout(json, options):
+    context = {
+        "index_conditions" : [],
+        "attached_conditions" : []
+    }
+    context.update(options)
+
+    tree = tree_from_json(context, None, "query_block", json["query_block"])
+    tree.process()
+    tree = strip_useless_nodes(tree)
+    layout = TreeLayouter(tree, options)
+
+    def reformat(text):
+        # add a select in front of the expression to fake a valid statement
+        prefix = "SELECT "
+        text = prefix+text
+        try:
+            output = grt.modules.SQLIDEUtils.reformatSQLStatement(text)
+        except Exception, exc:
+            grt.log_error("ExplainRenderer", "Exception reformatting snippet '%s': %s\n" % (text, exc))
+            output = text
+        # strip the SELECT prefix
+        return output[len(prefix):].strip()
+
+    for i, (name, cond) in enumerate(context["index_conditions"]):
+        node = PlainTextNode(None, "index_condition[%i]" % (i+1), "index_condition[%i] (%s):\n%s" % (i+1, name, reformat(cond)), {})
+        layout.add_bottom_node(node)
+
+    for i, (name, cond) in enumerate(context["attached_conditions"]):
+        node = PlainTextNode(None, "attached_condition[%i]" % (i+1), "attached_condition[%i] (%s):\n%s" % (i+1, name, reformat(cond)), {})
+        layout.add_bottom_node(node)
+
+    return layout
+
+
 def render_json_data(ctx, json_text, background_image, options):
     padding = 50
 
     json = decode_json(json_text)
-    tree = tree_from_json(None, "query_block", json["query_block"])
-    tree.process()
-    tree = strip_useless_nodes(tree)
-    layout = TreeLayouter(tree, options)
+    layout = prepare_layout(json, options)
     w, h = layout.get_total_size(ctx)
     ctx.set_font_size(12)
     if background_image:
@@ -746,16 +854,16 @@ def render_json_data(ctx, json_text, background_image, options):
 def render_json_data_to_file(json_text, background_image, png_file, options):
     padding = 50
 
+    scale = options.get("scale", 1)
+
     json = decode_json(json_text)
-    tree = tree_from_json(None, "query_block", json["query_block"])
-    tree.process()
-    tree = strip_useless_nodes(tree)
-    layout = TreeLayouter(tree, options)
+    layout = prepare_layout(json, options)
     img = ImageSurface(cairo.CAIRO_FORMAT_ARGB32, 10, 100)
     ctx = Context(img)
     w, h = layout.get_total_size(ctx)
-    img = ImageSurface(cairo.CAIRO_FORMAT_ARGB32, w+padding*2, h+padding*2)
+    img = ImageSurface(cairo.CAIRO_FORMAT_ARGB32, (w+padding*2)*scale, (h+padding*2)*scale)
     ctx = Context(img)
+    ctx.scale(scale, scale)
     ctx.set_font_size(12)
     if background_image:
         bgimage = ImageSurface.from_png(background_image)
